#ident  "@(#)nvram.c	35.2"

/*
 *	Copyright (c) 1988 Motorola Inc
 *	All Rights Reserved	
 *	@(#)nvram.c	35.2
 *
 *
 *	nvram.c: "generic" nvram driver interface
 */


#include "sys/types.h"
#include "sys/param.h"
#include "sys/buf.h"
#include "sys/errno.h"
#include "sys/file.h"
#include "sys/fs/s5dir.h"
#include "sys/signal.h"
#include "sys/user.h"
#include "sys/sysmacros.h"
#include "sys/nvram.h"


/*************
**  nvropen  *
**************
**
**  FUNCTION:  Open the nvram.  Check existance and permissions
**		for the user / operation.
**
**  CALLED:  By any application that wants to access the nvram.
**
**  RETURNS: ...none... [nvropen is declared as int function because all
**			 other examples of device_open are int.]
**************/

nvropen(dev, mode)
    register int dev, mode;
{
	register int nvrseg;
	extern int  nvr_addr[];	/* base addr of chip from master/dfile */

	ENTRY("nvropen");
	nvrseg = minor(dev);

	/*
	 *  If the device is nonexistent, then error.
	 *  Minor devices are 0 - 7 (incl.), though some are of 0 length.
	 */
	if (nvr_addr[0] == 0) {
		u.u_error = ENODEV;
		EXIT("nvropen");
		return;
	}
	if (nvrseg >= MAXNSLICE) {
		u.u_error = ENXIO;
		EXIT("nvropen");
		return;
	}

	/*
	 *  If not the appropriate user for this device, error.
	 *  Only super user is permitted to write the "system"
	 *  portion of the nvram.  Permissions for the user 
 	 *  portion are as set by the file permissions of minor
	 *  device 0 of the nvram.
	 */	

	if ((nvrseg > 0) && (mode&FWRITE) && !suser()) {
		u.u_error = EPERM;
		EXIT("nvropen");
		return;
	}

	EXIT("nvropen");

}  /* end nvropen */


/*************
**  nvrread  *
**************
**
**  FUNCTION:  Read nvram as a character device.
**
**  CALLED:  Any application that wishes to read info from the nvram.
**
**  RETURNS: ...none...
**
**************/

nvrread(dev)
    register int dev;
{
	register int nvrseg, n;
	register struct buf *bp;
	int chrs;
	caddr_t hbuf;

	extern struct buf *geteblk();
	

	/* 
	 *  Check boundry conditions before attempting read.  Call
	 *  the nvram routine to determine nvram type and do the
	 *  actual read; nvram expects buffer space in kernel area.
	 */

	ENTRY("nvrread");
	nvrseg = minor(dev);

	if (nvram(CK_BNDS,nvrseg,0,u.u_count,u.u_offset) != OK)
		u.u_error = EINVAL;
	else {
		bp = geteblk();
		hbuf = (caddr_t) paddr(bp);

		while (u.u_count > 0)  {
			n = min(u.u_count, SBUFSIZE);
			chrs = nvram(READ,nvrseg,hbuf,n,u.u_offset);

			if (chrs < 0) {			/* ERROR on read */
				u.u_error = EINVAL;
				break;
			} 
			else if (chrs == 0)		/* 0 length read */
				break;
 			else {				/* bytes read */

				if (copyout(hbuf, u.u_base, chrs)) {
					u.u_error = EFAULT;
					break;
				}
				u.u_count -= chrs;
				if (chrs != n)		/* incomplete read */
					break;
	
				u.u_offset += chrs;
				u.u_base += chrs;
			}
		}  /* endwhile */

		brelse(bp);
	} /* endif-else */

	EXIT("nvrread");
	return;
}  /* end nvrread */


/**************
**  nvrwrite  *
***************
**
**  FUNCTION:  Write nvram character device.
**
**  CALLED:  Any application that wishes to write to the nvram.
**
**  RETURNS:
**
***************/

nvrwrite(dev)
    register int dev;
{
	register int nvrseg, n;
	register struct buf *bp;
	int chrs;
	caddr_t hbuf;
	
	extern struct buf *geteblk();


	/* 
	 *  Check boundry conditions before attempting write.  Call
	 *  the nvram routine to determine nvram type and do the
	 *  actual write; nvram expects buffer space in kernel area.
	 */

	ENTRY("nvrwrite");
	nvrseg = minor(dev);

	if (nvram(CK_BNDS,nvrseg,0,u.u_count,u.u_offset) != OK)
		u.u_error = EINVAL;
	else {
		bp = geteblk();
		hbuf = (caddr_t) paddr(bp);

		while (u.u_count > 0)  {
			n = min(u.u_count, SBUFSIZE);
			if (copyin(u.u_base, hbuf, n)) {
				u.u_error = EFAULT;
				break;
			}
			chrs = nvram(WRITE,nvrseg,hbuf,n,u.u_offset);
	
			if (chrs <= 0) {		/* ERROR on write */
				u.u_error = EINVAL;
				break;
			} else {			/* bytes written */
				u.u_count -= chrs;
				if (chrs != n)		/* incomplete write */
					break;

				u.u_offset += chrs;
				u.u_base += chrs;
			}
		}  /* endwhile */

		brelse(bp);
	} /* endif-else */

	EXIT("nvrwrite");
	return;

}  /* end nvrwrite */

