/* Copyright (c) 1984 AT&T	*/
/*	All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF AT&T	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ident	"@(#)kern-port:io/physdsk.c	35.2"
/* 
 *
 */

#include "sys/types.h"
#include "sys/param.h"
#include "sys/immu.h"
#include "sys/fs/s5dir.h"
#include "sys/psw.h"
#include "sys/region.h"
#include "sys/sysmacros.h"
#include "sys/conf.h"
#include "sys/signal.h"
#include "sys/user.h"
#include "sys/proc.h"
#include "sys/errno.h"
#include "sys/buf.h"
#include "sys/elog.h"
#include "sys/iobuf.h"
#include "sys/systm.h"
#include "sys/debug.h"

#define DEBUG
#undef DEBUG

#ifdef DEBUG

#define PHYSBSIZE (64 * 1024)		/* 64K physio buffer */
#define CNT  4

char physbuf[CNT*PHYSBSIZE];

char physlock[CNT];		/* controls use of "physbuf" buffer */

int physbsize = PHYSBSIZE;		/*  physio buffer size */
int phys_cnt = CNT;

#else


extern char physlock[];			/* controls use of "physbuf" buffer */
extern physbsize;			/*  physio buffer size */
extern char physbuf[];
extern int phys_cnt;
#endif

static char inited = 0;			/* first time only */
static short physwanted;		/* set while sleeping on physbuf */

caddr_t
getphysbuf(size)
register unsigned size;
{
	register unsigned short i;
	register char *buff = NULL;
	register s;

	if (size > physbsize)
		return(0);
	s = spl6();
	while(buff == NULL) {
		for(i = 0; i < phys_cnt; i++) {
			if(physlock[i]) 
				continue;
			physlock[i] = 1;
			buff = &physbuf[i*physbsize];
			break;
		}
		if(buff == NULL) {
			physwanted=1;	/* wait for availability of "physbuf" */
			splx(s);
#ifdef DEBUGP
			printf("getphysbuf: sleeping \n");
#endif
			sleep((caddr_t)physbuf, PRIBIO);
			s = spl6();
		}
	}
	splx(s);
	return(buff);
}

freephysbuf(addr)
register caddr_t addr;
{
	register i;
	register s;

	s = spl6();
	for(i = 0; i < phys_cnt; i++) {
		if(addr != &physbuf[i*physbsize])	/* find buffer */
			continue;
		physlock[i] = 0;
	}
	splx(s);
	if (physwanted){
		physwanted = 0;
#ifdef DEBUGP
			printf("freephysbuf: waking \n");
#endif
		wakeup((caddr_t)physbuf);
	}
}

/*
 * Calls the designated strategy routine to 
 * perform the I/O transfer described in the supplied bp.
 * A large static buffer is used to store and forward consecutive disk
 * data to/from physically discontiguous pages of user memory rather than
 * perform a separate disk request for each disjoint section of user memory.
 */

dma_breakup(strat, bp, sectsize)
register  (*strat)(); 		/* address of device driver strategy routine */
register struct buf *bp;	/* buffer header from physical buffer list */
register  sectsize; 			/* sector size */
{
	register caddr_t bufaddr;	/* kernel buffer address */
	register int count;		/* user requested byte count */
	register int rw;

	if( inited == 0 ) {
		inited = 1;
		for( rw=0; rw<phys_cnt; rw++ )
			physlock[rw] = 0; /* initialize lock */
	}
	count = bp->b_bcount;
	rw = bp->b_flags & B_READ;

	if ((count & (sectsize - 1)) || ((bufaddr = getphysbuf(count)) == 0)) {
		bp->b_flags |= B_ERROR | B_DONE;
  		bp->b_error = ENXIO;
		return;
	}
	bp->b_un.b_addr = bufaddr;

	if (rw == B_READ) {
		doio(strat, bp);
		iomove(bufaddr, count, rw);
	}
	else {
		iomove(bufaddr, count, rw);
		doio(strat, bp); 
	}

	freephysbuf(bufaddr);
}

doio(strat, bp)
register  (*strat)();
register struct buf *bp;
{
	bp->b_flags &= ~(B_DONE | B_ERROR);

	(*strat)(bp);
	spl6();
	while ((bp->b_flags & B_DONE) == 0) {
		sleep(bp, PRIBIO);
	}
	spl0();
}
