#include "sys/param.h"
#include "sys/types.h"
#include "sys/sysmacros.h"
#include "sys/dir.h"
#include "sys/signal.h"
#include "sys/user.h"
#include "sys/tty.h"
#include "sys/termio.h"
#include "sys/errno.h"
#include "sys/sysinfo.h"
#include "sys/conf.h"
#include "sys/gentty.h"
#include "sys/MC68681.h"

#define	 NDUARTS	1

#define	UC(x)		((unsigned char)((x)))

#define port_minor(x) (int)((x) & 0x7f)

extern struct mc68681 *port_addr[];
extern struct tty port_tty[];
extern uint		port_cnt;

static struct pp {
	struct port *p_p;
	struct tty *p_tty;
	char	p_rxi;
	char	p_txi;
	struct imr *p_imr;
	unsigned char	p_fill[2];/* this allows for more efficient indexing */
} ports[NDUARTS*NPORTS];

static struct imr {
	char	*i_pimr;
	char	i_imr;
} dart_imr[NDUARTS];

static char	port_spd[16] = {	
	S9600, S9600,   S75,  S110,
	S134,  S150, S9600,  S300,
	S600, S1200, S9600, S2400,
	S4800, S9600, S19200, S9600 };


portinit()
{
	register struct port *p;
	register struct mc68681 *d;
	register struct tty *t;
	register uint i;

	for (i = 65536; --i; )
		;	/* wait a while until the port is empty */
	for (i = 0, t = &port_tty[0]; i < port_cnt; i++, t++) {
		d = port_addr[i/NPORTS];
		p = i & 01 ? &d->B : &d->A;
		ports[i].p_p = p;
		ports[i].p_tty = t;
		ports[i].p_imr = &dart_imr[i/NPORTS];
		ports[i].p_imr->i_pimr = &d->d_isim;
		ports[i].p_imr->i_imr  = 0;
		ports[i].p_rxi = i & 01 ? IRRXB : IRRXA;
		ports[i].p_txi = i & 01 ? IRTXB : IRTXA;
		if (!(i & 01)) {
			d->d_isim = 0;
			d->d_ipac = ACR;
			d->d_iv   = 95;
		}
		p->d___cm = RXDIS | TXDIS;
		p->d___cm = R_RCV;
		p->d___cm = R_TXM;
		p->d___cm = R_MRP;
		p->d_mr12 = MR1;
		/* p->d_mr12 = MR2; */
		p->d_mr12 = MR2 | 0x10;
		p->d_srcs = CS;
		p->d___cm = R_ERR | RXEN | TXEN;

	}

}


port_putchar(mdev, c)
register uint mdev;
register char	c;
{
	register struct mc68681 *d;
	register struct port *p;
	register int	ps;

	d = port_addr[0];
		/* Putchar can now use both DUART ports */
		/* because V.3 Rel 4 has dual consoles. M. Woods */
	if ( mdev )	/* Minor device 1 */
		p = &d->B;
	else
		p = &d->A;	/* Minor device 2 */
	ps = splhi();
	for (; ; ) {
		while (!(p->d_srcs & CANSEND))
			;
		p->d_data = c;
		if (c != '\n') 
			break;
		c = '\r';
	}
	while (!(p->d_srcs & CANSEND))
		;
	splx(ps);
}



static portpara(t)
register struct tty *t;
{
	register struct port *p;
	register struct pp *pp;
	register struct imr *im;
	register ushort cflag;
	register char	m1;

	pp = &ports[t - &port_tty[0]];
	if (pp->p_tty != t) 
		return;
	p = pp->p_p;
	im = pp->p_imr;
	cflag = t->t_cflag;
	if (!(cflag & CBAUD)) {
		t->t_state &= ~BUSY;
		im->i_imr &= ~(pp->p_txi);
		im->i_imr |= pp->p_rxi;
		*(im->i_pimr) = im->i_imr;
		return;
	}
	*(im->i_pimr) = 0;
	m1 = (cflag & CSIZE) >> 4;
	if (!(cflag & PARENB)) 
		m1 |= NOPARITY;
	else 
		m1 |= (cflag & PARODD) ? PODD : PEVEN;
	for (p->d___cm = TXEN; !(p->d_srcs & TXEMT); )
		;
	p->d___cm = RXDIS | TXDIS;
	p->d___cm = R_MRP;
	p->d_mr12 = m1;
	/* p->d_mr12 = (cflag & CSTOPB) ? 0x0f : 0x07; */
	p->d_mr12 = (cflag & CSTOPB) ? 0x1f : 0x07;
	p->d_srcs = port_spd[cflag & CBAUD];
	p->d___cm = R_ERR | RXEN | TXEN;
	*(im->i_pimr) = im->i_imr;
}


static portproc(t, cmd)
register struct tty *t;
int	cmd;
{
	register struct pp *pp;
	register struct ccblock *cc;
	register struct imr *im;

	pp = &ports[t - &port_tty[0]];
	if (pp->p_tty != t) 
		return;
	im = pp->p_imr;
	switch (cmd) {
	case T_TIME:	
		t->t_state &= ~(TIMEOUT | BUSY);
	case T_OUTPUT:
OUTPUT:
		if (t->t_state & (TTSTOP | BUSY)) 
			break;
		cc = &t->t_tbuf;
		if (!cc->c_ptr || !cc->c_count) {
			if (cc->c_ptr) 
				cc->c_ptr -= cc->c_size - cc->c_count;
			if (!((*linesw[UC(t->t_line)].l_output)(t) & CPRES))
				break;
		}
		t->t_state |= BUSY;
ENINT:
		im->i_imr |= pp->p_txi;
		*(im->i_pimr) = im->i_imr;
		break;
	case T_SUSPEND:	
		t->t_state |= TTSTOP;
		break;
	case T_WFLUSH:	
		t->t_tbuf.c_size -= t->t_tbuf.c_count;
		t->t_tbuf.c_count = 0;
	case T_RESUME:	
		t->t_state &= ~TTSTOP;
		goto OUTPUT;
	case T_BLOCK:	
		t->t_state &= ~TTXON;
		t->t_state |= TTXOFF | TBLOCK;
		goto ENINT;
	case T_RFLUSH:	
		if (!(t->t_state & TBLOCK)) 
			break;
	case T_UNBLOCK:	
		t->t_state &= ~(TTXOFF | TBLOCK);
		t->t_state |= TTXON;
		goto ENINT;
	case T_BREAK:	
		break;		/* not yet implemented */
	case T_INPUT:	
		im->i_imr |= pp->p_rxi;
		*(im->i_pimr) = im->i_imr;
		break;
	}
}


portintr(mdev)
dev_t mdev;
{
	register struct pp *pp;
	register struct port *p;
	register struct tty *t;
	register unsigned char	c, sr;
	register ushort iflag;
#define  cc	((unsigned char)iflag)
	register uint i, k;
	register char	isr;
	char	buf[3];


	for (i = 0, pp = &ports[mdev*NPORTS]; i < NPORTS; i++, pp++) {
		p = pp->p_p;
		t = pp->p_tty;
		if (!(i & 01)) 
			isr = *(pp->p_imr->i_pimr) & pp->p_imr->i_imr;
		if ((isr & pp->p_rxi) && ((sr = p->d_srcs) & GOTCHAR)) {
			c = p->d_data;
			if (!(t->t_state & CARR_ON)) {
				wakeup(&t->t_canq);
				t->t_state |= CARR_ON;
				goto DOOUT;
			}
			if (t->t_iflag & IXON) {
				iflag = c & 0177;
				if (t->t_state & TTSTOP) {
					if (cc == CSTART || t->t_iflag &
					    IXANY)
						portproc(t, T_RESUME);
				} else if (cc == CSTOP) 
					portproc(t, T_SUSPEND);
				if (cc == CSTART || cc == CSTOP)
					goto DOOUT;
			}
			iflag = t->t_iflag;
			k = 1;
			if (sr & PARITY && !(iflag & INPCK)) 
				sr &= ~ PARITY;
			if (sr & ERRBITS) {
				if (sr & BRK) {
					if (iflag & IGNBRK) 
						goto DOOUT;
					if (iflag & BRKINT) {
						signal(t->t_pgrp, SIGINT);
						ttyflush(t, 3);
						goto DOOUT;
					}
				} else {
					if ( sr & OVERUN)
						p->d___cm = R_ERR;
					if (iflag & IGNPAR) 
						goto DOOUT;
				}
				if (iflag & PARMRK) {
					buf[2] = -1; 
					buf[1] = 0; 
					k = 3;
					sysinfo.rawch += 2;
				} else 
					k = 0;
			} else
			 {
				if (iflag & ISTRIP) 
					c &= 0177;
				else if (c == 0xff && iflag & PARMRK) {
					buf[1] = -1; 
					k = 2;
				}
			}
			buf[0] = c;
			if (t->t_rbuf.c_ptr) {
				for (; k; ) {
					for (; k; ) {
						*(t->t_rbuf.c_ptr++) = buf[--k];
						if (--t->t_rbuf.c_count ==
						    0) 
							break;
					}
					t->t_rbuf.c_ptr -= t->t_rbuf.c_size -
					    t->t_rbuf.c_count;
					(*linesw[UC(t->t_line)].l_input)(t,L_BUF);
				}
			}
		}
DOOUT:
		if (!(isr & pp->p_txi)) 
			continue;
		if (t->t_state & TTXON) {
			p->d_data = CSTART; 
			t->t_state &= ~TTXON; 
			continue;
		}
		if (t->t_state & TTXOFF) {
			p->d_data = CSTOP;  
			t->t_state &= ~TTXOFF; 
			continue;
		}
		if (t->t_state & BUSY) {
			if (t->t_tbuf.c_count) {
				p->d_data = *(t->t_tbuf.c_ptr++);
				--t->t_tbuf.c_count;
			}
			t->t_state &= ~BUSY;
			portproc(t, T_OUTPUT);
			continue;
		}
		t->t_state &= ~BUSY;
		pp->p_imr->i_imr &= ~(pp->p_txi);
		*(pp->p_imr->i_pimr) = pp->p_imr->i_imr;
	}
}

portopen(dev, flag)
dev_t dev; 
int	flag;
{
	register struct tty *t;
	register uint mdev;

	if ((mdev = port_minor(dev)) >= port_cnt) {
		u.u_error = ENXIO; 
		return;
	}
	t = ports[mdev].p_tty;
	if (!(t->t_state & (WOPEN | ISOPEN))) {
		ttinit(t);
		t->t_proc = portproc;
		portpara(t);
		t->t_state |= CARR_ON;	/* no DCD on ports */
	}
	(*linesw[UC(t->t_line)].l_open)(t);
}


portclose(dev)
dev_t dev;
{
	register struct tty *t;
	register uint mdev;

	t = ports[mdev = port_minor(dev)].p_tty;
	(*linesw[UC(t->t_line)].l_close)(t);
}


portread(dev)
dev_t dev;
{
	register struct tty *t;

	t = ports[port_minor(dev)].p_tty;
	return((*linesw[UC(t->t_line)].l_read)(t));
}


portwrite(dev)
dev_t dev;
{
	register struct tty *t;

	t = ports[port_minor(dev)].p_tty;
	return((*linesw[UC(t->t_line)].l_write)(t));
}


portioctl(dev, arg0, arg1, arg2)
dev_t dev; 
int	arg0, arg1, arg2;
{
	register struct tty *t;

		/* Console driver cons calls CPU board dependant putchar
		   using ioctl calls
		*/ 
	if ( arg0 == CONC_IPUTC){
		port_putchar( port_minor(dev), '~');
		return;
	}
	if ( arg0 == CONC_PUTC ){
		port_putchar( port_minor(dev), arg1);
		return;
	}
	if ( arg0 == CONC_BOOT){
		/* Initialise second DUART port for dual consoles */
		/* Already initialised by portinit. M. Woods */
		return;
	}
	t = ports[port_minor(dev)].p_tty;
	if (ttiocom(t, arg0, arg1, arg2))
		portpara(t);
}


