
/*
 *	Copyright (c) 1988 Motorola Inc
 *	All Rights Reserved
 *
 *	@(#)prfclk.c	35.1	
 * 
 *	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF AT&T
 *	The copyright notice above does not evidence any
 *	actual or intended publication of such source code.
*/

#ident	"@(#)prfclk.c	35.1"
/*
******************************************************
	@(#)prfclk.c	1.0	
	This file contains the routines required to 
	manipulate a profiling clock.  At present, this
	only supports the Motorola M146818 hardware clock
	resident on the MVME050 board.
******************************************************
*/

#include "sys/types.h"
#include "sys/mc68153.h"

/* Value to use in a dfile Force statement to pull in this profiling
** clock and not some other.  This allows static choice of drivers;
** a dynamic method would be preferable (necessitating a change of
** symbol names in here to prfc050_*).
*/

int	PRF050;

	/* defines used in conjunction with the clock registers */


#define  CLK_ADDR 	0x100-1		/* clock offset */
	/* -1 because m050_addr will be an odd address 	*/

#define  C_BSET		0x80		/* enable set flag in register B     */
#define  C_VRT		0x80		/* valid time flag in register D     */
#define  C_INTRATE	0x0a		/* set the periodic interrupt rate   */
#define  C_INIT		0x40		/* initial control values in reg. B;
					   periodic interrupt enable	     */
#define C_TODVEC	114		/* BIM vecnum for TOD xrupt
					   114 = 0x72 = vecaddr 0x1c8        */
#define C_TODCHL	2		/* BIM interrupt chl for TOD xrupt   */ 



	/* memory map of the MC146818 Clock Registers			     */

struct clk_map
   {
   char fill0,  c_secs;			/* seconds elapsed in current minute */
   char fill2,  c_sec_alarm;		/* when to generate alarm interrupt  */
   char fill4,  c_mins;			/* minutes elapsed in current hour   */
   char fill6,  c_min_alarm;		/* when to generate alarm interrupt  */
   char fill8,  c_hrs;			/* hours elapsed in current day	     */
   char fill10, c_hrs_alarm;		/* when to generate alarm interrupt  */
   char fill12, c_day_wk;		/* day of week (1-7); sunday=1       */
   char fill14, c_day_mon;		/* day of month (1-31);              */
   char fill16, c_mon;			/* month of year (1-12)	             */
   char fill18, c_yr;			/* year of century (0-99)            */
   char fill20, c_a;			/* register A		             */
   char fill22, c_b;			/* register B		             */
   char fill24, c_c;			/* register C		             */
   char fill26, c_d;			/* register D		             */
   };


/* Start the timer.  Return 0 if no hardware.  Rate of 0 is off, so
** any positive return implies external clock is operating.
*/

prfc_start (rate)
int rate;

   {
   register struct clk_map *clk;
   register char	scratch;

   extern int m050_addr[];
   register int base_addr = m050_addr[0];

       /* Rate 0 is implicit 'off'.  */

       if(m050_addr[0]){
	    clk = (struct clk_map *)(CLK_ADDR+base_addr);
	    /* if bad rate, no startee */
	    if (rate < 0 || rate > 0xF)
		return (0);

	    clk->c_b |= C_BSET;		/*inhibit update so can set clock    */
	    clk->c_a  = rate;		/*set periodic interrupt rate	     */
	    scratch = clk->c_c;		/*reset interrupt flag		     */
	    clk->c_b  = C_INIT;		/*initialize and start timer	     */

	    /* init BIM1:  chl , vecnum, lvl 7 */

	    m050prime (C_TODCHL, C_TODVEC, (IRE | 7));
	    return (rate);
       }
       return(0);
   }


   /*for the M146818 timer there is no need to reload the timer after every
      interrupt.  prfc_reld merely clears the interrupt flags in register C   */

prfc_reld ()
   {
   register struct clk_map *clk;
   register short	scratch;

   extern int m050_addr[];
   register int base_addr = m050_addr[0];

   if (m050_addr[0])  {
       clk = (struct clk_map *)(CLK_ADDR+base_addr);
       scratch = clk->c_c;		/*clear interrupt flag	     */
       return (1);
   }
   return(0);
   }

