
/*
 *
 * Copyright (c) 1987 Motorola Computer Division
 *	  All Rights Reserved  
 */
#ident	"@(#)pty.c	35.3"

/*
 *	Pseudo-teletype driver. This is really two drivers, requiring
 *	two entries in cdevsw.
 *
 *	New implementation for Sys V, includes 4.2BSD PKT mode.
 */

#include "sys/types.h"
#include "sys/param.h"
#include "sys/sysmacros.h"
#include "sys/immu.h"
#include "sys/region.h"
#include "sys/proc.h"
#include "sys/fs/s5dir.h"
#include "sys/signal.h"
#include "sys/user.h"
#include "sys/errno.h"
#include "sys/file.h"
#include "sys/termio.h"
#include "sys/conf.h"
#include "sys/tty.h"
#include "sys/pty.h"
#include "sys/cmn_err.h"

/*
 * the MAXPTY stuff should be split out into a (YUK) space/pty.h,
 * along with the ptychars and info structure.
 */
#define MAXPTY	64

/* Berzerkely uses t_proc to flag open/closed  status of controller side.
** We need something similar, (that stays around through pts close/reopens),
** but we can't just go around zeroing t_proc while the slave side may be
** asleep.  Thus, use of unused t_dstat field and PTC_OPEN flag.  We choose
** to completely zero t_state at pts or ptc close, (mainly so ptc can see
** transitions as pts users come and go) so we can't use that...
*/
#define PTC_OPEN	1

extern struct tty pts_tty[];	/* TTY structs for us to use		*/
extern pts_cnt;			/* # of PTY devices configured		*/

static char ptychars[2*MAXPTY];
unchar ptswclose[MAXPTY];

struct info {
	struct proc *rsel;	/* ^ to processing selecting for read	*/
	struct proc *wsel;	/* ^ to processing selecting for writ	*/
	unsigned char flags;	/* controller select flags		*/
} pt_info[MAXPTY];

/*
 *	"Slave" side.  Shows up as /dev/ttyp?, and can be used as
 *	a normal tty device.
 */
ptsopen (dev, flag) {
	register struct tty *tp;

	if (minor (dev) >= pts_cnt ) {
		u.u_error = ENXIO;
		return;
	}

	tp = &pts_tty[(short) minor(dev)];

	while(ptswclose[minor(dev)]) {
		if(sleep((caddr_t)&tp->t_canq, TTIPRI|PCATCH))
		{
			u.u_error = ENXIO;
			return;
		}
	}	

	if ( (tp->t_state&ISOPEN) == 0 ) {
		ttinit (tp);
		tp->t_cflag &= ~CBAUD;
		tp->t_cflag |= EXTB;
	}
	
	if (tp->t_dstat & PTC_OPEN)		/* controller still active */
		tp->t_state |= CARR_ON;
	while ( (tp->t_state & CARR_ON) == 0 ) {
		tp->t_state |= WOPEN;
		sleep ((caddr_t)&tp->t_rawq, TTIPRI);
	}

	(*linesw[tp->t_line].l_open)(tp);

	if ( tp->t_tbuf.c_ptr == NULL ) {
		tp->t_tbuf.c_ptr = &ptychars[(short)minor(dev)*2];
		tp->pt_send = tp->pt_flags = 0;
		wakeup (&tp->t_tbuf.c_ptr);
	}

	ptcwakeup (FREAD|FWRITE);
}

ptsclose (dev)
{
	register struct tty *tp;

	tp = &pts_tty[(short) minor(dev)];

	ptswclose[minor(dev)] = 1;

	tp->pt_send = tp->pt_flags = 0;
	tp->t_tbuf.c_ptr = NULL;
	(*linesw[tp->t_line].l_close)(tp);

	/* This is a little extreme, but should be safe.  
	** Berkeley code clears t_state on pts close, which is
	** the only way (short of ptc closing) that CARR_ON
	** can go away.  Ptc uses CARR_ON to see if pts has
	** opened and closed.
	*/
	tp->t_state = 0;

	ptswclose[minor(dev)] = 0;
	ptcwakeup(tp, FREAD|FWRITE);
}

ptsread (dev)			/* This is the SysV tty interface */
{
	register struct tty *tp;

	tp = &pts_tty[(short) minor(dev)];

	/*  why not let l_read tell us if it makes sense to read
	if ((tp->t_state&ISOPEN) == 0 ) {
		u.u_error = ENODEV;
		return;
	}
	*/

/*
  This is a point of contention between BSD and SysV
  concerning what happens when NODELAY is set.
        SysV  -  if no data available on tty device,
	         return 0 ( and no error ).  This
		 determination is performed in the line discipline.
	BSD   -  if no data available on any
	         type device, return EWOULDBLOCK
		 as the error.
  This modification is only done to the slave side of the pty
  interface since this is the point of conflict with SysV.
  Old code follows:

	if ((u.u_fmode&FNDELAY) && ttnread(tp) == 0) {
		u.u_error = EWOULDBLOCK;
		return;
	}
*/
	(*linesw[tp->t_line].l_read)(tp);
	/*
	 * If the controller is blocked (or selecting),
	 * goose it.
	 */
	ptcwakeup(tp, FWRITE);
}

ptswrite (dev)			/* This is the SysV tty interface */
{
	register struct tty *tp;

	tp = &pts_tty[(short) minor(dev)];

	if ( (tp->t_state&CARR_ON) == 0) {
		u.u_error = ENODEV;
		return;
	}

/*
  see ptsread() comment on NODELAY

	if ((u.u_fmode&FNDELAY) && (tp->t_state&OASLP)) {
		u.u_error = EWOULDBLOCK;
		return;
	}
*/
	(*linesw[tp->t_line].l_write)(tp);
}

ptsioctl (dev, cmd, arg, mode)
{
	register struct tty *tp;

	tp = &pts_tty[(short) minor(dev)];

	if ( (tp->t_state&CARR_ON) == 0) {
		u.u_error = ENODEV;
		return;
	}

	ttiocom (tp, cmd, arg, mode);

	if (tp->pt_flags&PF_NOSTOP) {
		if (tp->t_iflag&IXON) {		/* ^S/^Q enabled */
			tp->pt_send &= ~TIOCPKT_NOSTOP;
			tp->pt_send |= TIOCPKT_DOSTOP;
			tp->pt_flags &= ~PF_NOSTOP;
			ptcwakeup(tp, FREAD);
		}
	} else {
		if ( (tp->t_iflag&IXON) == 0 ) {
			tp->pt_send &= ~TIOCPKT_DOSTOP;
			tp->pt_send |= TIOCPKT_NOSTOP;
			tp->pt_flags |= PF_NOSTOP;
			ptcwakeup(tp, FREAD);
		}
	}
}

#if 0
/*
 * support for select()
 */
ptsselect(dev, rw)
dev_t dev; {
	register struct tty *tp;

	tp = &pts_tty[(short) minor(dev)];

	if ( (tp->t_state&CARR_ON) == 0) {
		u.u_error = ENODEV;
		return 0;
	}

	return ttselect(dev, rw);
}
#endif

ptyproc (tp, cmd)
register struct tty *tp;
{
	if ( tp->t_state & ISOPEN )
	switch (cmd) {
	case T_OUTPUT:
	start:
		if (tp->t_state&TTSTOP)
			break;
		if (tp->pt_flags&PF_STOPPED) {
			tp->pt_flags &= ~PF_STOPPED;
			tp->pt_send = TIOCPKT_START;
		}
		if (tp->t_outq.c_cc) {
			ptcwakeup(tp, FREAD);
			tp->t_state |= BUSY;
		}
		break;

	case T_SUSPEND:
		tp->t_state |= TTSTOP;
		tp->pt_flags |= PF_STOPPED;
		tp->pt_send |= TIOCPKT_STOP;
		break;

	case T_WFLUSH:
		tp->pt_send |= TIOCPKT_FLUSHWRITE;
	case T_RESUME:
		tp->t_state &= ~TTSTOP;
		goto start;

	case T_RFLUSH:
		tp->pt_send |= TIOCPKT_FLUSHREAD;
		break;
	}
}



/*
 *	"Controller" side.  This is considerably more complicated
 *	than the slave side, mostly because it has to spend time
 *	defeating the line discipline operation.
 *
 *	Shows up as /dev/ptyp?
 *
 *	NOTE: the cdevsw entry for the controller MUST NOT have
 *	a tty struct array associated with it.
 *
 */

ptcopen (dev, flag)
{
	register struct tty *tp;
	register struct info *ip;

	if (minor(dev) >= pts_cnt) {
		u.u_error = ENXIO;
		return;
	}

	tp = &pts_tty [(short) minor(dev)];
	ip = &pt_info[(short) minor(dev)];

	if ( tp->t_dstat & PTC_OPEN ) {		/* already opened */
		u.u_error = EIO;
		return;
	}

	tp->t_proc = ptyproc;
	tp->t_dstat |= PTC_OPEN;
	if (tp->t_state&WOPEN)
		wakeup ((caddr_t)&tp->t_rawq);
	tp->t_state |= CARR_ON;

	ip->flags = 0;
	ip->rsel  = 0;
	ip->wsel  = 0;
}

ptcclose (dev)
{
	register struct tty *tp;

	tp = &pts_tty[(short) minor(dev)];

	if (tp->t_state & ISOPEN)
		signal (tp->t_pgrp, SIGHUP);
	ttyflush (tp, FREAD|FWRITE);
	tp->t_state &= ~(CARR_ON | ISOPEN);
	tp->t_state = 0;
	tp->t_dstat &= ~(PTC_OPEN);
}

ptcwrite (dev)
{
	register struct tty *tp;
	register char *cp;
	register struct cblock *cb;
	register int n, i, max, c;

	tp = &pts_tty[(short) minor(dev)];
	if ((tp->t_state&ISOPEN) == 0) {
		u.u_error = EIO;
		return;
	}

	while (tp->t_rbuf.c_ptr == NULL) {
		cb = getcf();
		if ( cb == NULL ) {
			cfreelist.c_flag = 1;
			sleep ((caddr_t)&cfreelist, TTOPRI);
		}
		else {
			tp->t_rbuf.c_ptr = cb->c_data;
			tp->t_rbuf.c_size = cfreelist.c_size;
			tp->t_rbuf.c_count = tp->t_rbuf.c_size;
		}
	}

	while (u.u_count) {

		if (!(tp->t_state & CARR_ON)) {
			u.u_error = EIO;
			return;
		}

	       /* If there's any immediate data available in the
	       ** canq for canonical readers, or reader is in raw
	       ** mode, wake up reader to try to consume what's 
	       ** pending; then sleep.  Otherwise, fall through to the
	       ** normal call to l_input (which will flush the queue if
	       ** it's greater than the hog value)  -wjc
	       */

		while (tp->t_rawq.c_cc + tp->t_canq.c_cc >= 
		    (TTYHOG - CLSIZE - 1) &&
			(tp->t_canq.c_cc > 0 || !(tp->t_lflag & (ICANON))) ) {


			/* must check this again */ 

			if (!(tp->t_state & CARR_ON)) {
				u.u_error = EIO;
				return;
			}

			ttwakeup(tp, FREAD);

			if (u.u_fmode & FNDELAY) {
				u.u_error = EWOULDBLOCK;
				return;
			}

			sleep((caddr_t)&tp->t_rawq.c_cf, TTOPRI);
		}

		n = min(tp->t_rbuf.c_count, u.u_count);
		cp = tp->t_rbuf.c_ptr + (tp->t_rbuf.c_size -
						tp->t_rbuf.c_count);
		copyin (u.u_base, cp, n);

		/* check for flow control chars */
		max = n;
		if (tp->t_iflag & IXON) {
			for (n = i = 0; i < max; ++i) {
				c = cp [i];
				if (tp->t_state & TTSTOP) {
					if ((c == CSTART) || (tp->t_iflag&IXANY))
						(*tp->t_proc) (tp, T_RESUME);
				} else {
					if (c == CSTOP)
						(*tp->t_proc) (tp, T_SUSPEND);
				}
				if (c != CSTOP && c != CSTART)
					cp [n++] = c;
			}
		}

		tp->t_rbuf.c_count -= n;
		u.u_count -= max;
		u.u_base += max;

		(*linesw[tp->t_line].l_input) (tp, 0);
	}
}

ptcread (dev)
{
	register struct tty *tp;
	register struct cblock *cb;
	register int n;
	register int c;

	tp = &pts_tty[(short) minor(dev)];

again:
	/* if pts is really closed (ie, not just waiting for us to drain off
	** some data in ptsclose/ttyclosewait), then shout about it.
	*/
	if ((tp->t_state&ISOPEN) == 0 && (!ptswclose[minor(dev)])) {
		u.u_error = EIO;
		goto out;
	}

	if ( (tp->t_state&TTSTOP) || (tp->t_outq.c_cc == 0) ) {
		if ( u.u_fmode&FNDELAY) {
			u.u_error = EWOULDBLOCK;
			return;
		}
		sleep ((caddr_t)&tp->t_outq.c_cf, TTIPRI);
		goto again;
	}

	if (tp->pt_flags&PF_PKT) {
		if (tp->pt_send) {
			passc (tp->pt_send);
			tp->pt_send = 0;
			return;
		}
		else	passc (0);
	}

	while ( tp->t_outq.c_cc && u.u_count ) {
		cb = tp->t_outq.c_cf;
		n = cb->c_last - cb->c_first;
		if ( u.u_count >= n ) {
			copyout (&cb->c_data[cb->c_first], u.u_base, n);
			u.u_base += n;
			u.u_count -= n;
			putcf (getcb (&tp->t_outq));
		} else {
			if ( (c = getc (&tp->t_outq)) < 0 )
				break;
			passc (c);
		}
	}

out:
	tp->t_state &= ~BUSY;

	if (tp->t_outq.c_cc <= ttlowat[tp->t_cflag&CBAUD]) {
		if (tp->t_state&OASLP) {
			tp->t_state &= ~OASLP;
			wakeup ((caddr_t) &tp->t_outq);
		}
		/*
		 * If we have a write selector, wake him up
		 */
		if (tp->t_wsel) {
			selwakeup(tp->t_wsel, tp->t_state & WCOLL);
			tp->t_wsel = 0;
			tp->t_state &= ~WCOLL;
		}
	}
	if (tp->t_state&TTIOW) {
		if (tp->t_outq.c_cc == 0) {
			tp->t_state &= ~TTIOW;
			wakeup ((caddr_t)&tp->t_oflag);
		}
	}
}

ptcioctl (dev, cmd, arg, mode)
{
	register struct tty *tp;
	register v;

	tp = &pts_tty[(short) minor(dev)];

#if 0
	-- deleted this check because if the slave goes away, the
	-- controller just hangs.  -dap

	while ( tp->t_tbuf.c_ptr == NULL )
		sleep ( &tp->t_tbuf.c_ptr, TTOPRI );
#endif

	switch (cmd) {
	case TIOCPKT:
		v = fuword( arg );
		if ( v == -1 ) {
			u.u_error = EFAULT;
			return;
		}
		if ( v )
			tp->pt_flags |= PF_PKT;
		else	tp->pt_flags &= ~PF_PKT;
		return;

	case TIOCREMOTE:
		v = fuword( arg );
		if ( v == -1 ) {
			u.u_error = EFAULT;
			return;
		}
		if ( v )
			tp->pt_flags |= PF_REMOTE;
		else
			tp->pt_flags &= ~PF_REMOTE;

		ttyflush (tp, FREAD|FWRITE);
		return;

/*
	case TIOCSETP:
		while (getc (&tp->t_outq) >= 0)
			;
		break;
*/
	}

	ttiocom (tp, cmd, arg, mode);

	if (tp->pt_flags&PF_NOSTOP) {
		if (tp->t_iflag&IXON) {		/* ^S/^Q enabled */
			tp->pt_send &= ~TIOCPKT_NOSTOP;
			tp->pt_send |= TIOCPKT_DOSTOP;
			tp->pt_flags &= ~PF_NOSTOP;
			wakeup ((caddr_t)&tp->t_outq.c_cf);
		}
	} else {
		if ( (tp->t_iflag&IXON) == 0 ) {
			tp->pt_send &= ~TIOCPKT_DOSTOP;
			tp->pt_send |= TIOCPKT_NOSTOP;
			tp->pt_flags |= PF_NOSTOP;
			wakeup ((caddr_t)&tp->t_outq.c_cf);
		}
	}
}

ptcselect(dev, rw)
dev_t dev; {
	register struct info *ip;
	register struct tty *tp;
	register x;
	extern selwait;

	tp = &pts_tty [(short) minor(dev)];
	ip = &pt_info[(short) minor(dev)];

	/* We want to:
	 **	return false if slave hasn't yet opened (FREAD/FWRITE),
	 **	    or there are no chars yet in slave's outq (FREAD),
	 **	    or there is no space for chars in slave's rawq (FWRITE),
	 **	    or there are unread chars in the slave's canq (FWRITE). 
	 **	return true if slave was here, but went away
	 **	    (ie. if CARR_ON transition),
	 **	    or there are chars in slave's outq (FREAD),
	 **	    or there is room for chars in slave's rawq (FWRITE),
	 **	    or there is room for chars in slave's canq AND the
	 **			slave is in canonical mode (FWRITE).
	 */

	if ((tp->t_state&CARR_ON) == 0)	   /* Lost slave */
		return 1;

	x = spltty();

	switch (rw) {
	    case FREAD:
		/* If pts is open *or* was open and is now trying to
		** close (probably in ttyclosewait()) *and* there's
		** data on the outq....
		*/
		if ((tp->t_state & ISOPEN || ptswclose[minor(dev)])
		    && ((tp->t_state & TTSTOP) == 0 && tp->t_outq.c_cc)) {
			goto win;
		}
		if (ip->rsel && ip->rsel->p_wchan == (caddr_t)&selwait)
			ip->flags |= RCOLL;
		else
			ip->rsel = u.u_procp;
		break;

	    case FWRITE:
		if (tp->t_state & ISOPEN) {
		    if (((tp->pt_flags & PF_REMOTE) && (tp->t_canq.c_cc == 0))
		    ||  (tp->t_rawq.c_cc + tp->t_canq.c_cc <
						(TTYHOG - CLSIZE - 1))
		    ||  (tp->t_canq.c_cc == 0 && (tp->t_lflag & (ICANON))) ) {
				goto win;
		    }

		}
		if (ip->wsel && ip->wsel->p_wchan == (caddr_t)&selwait)
			ip->flags |= WCOLL;
		else
			ip->wsel = u.u_procp;
		break;
	}

	splx(x);
	return 0;
win:
	splx(x);
	return 1;
}

ptcwakeup(tp, flag)
register struct tty *tp; {
	register struct info *ip;

	ip = &pt_info[tp - pts_tty];

	if (flag & FREAD) {
		if (ip->rsel) {
			selwakeup(ip->rsel, ip->flags & RCOLL);
			ip->rsel = 0;
			ip->flags &= ~RCOLL;
		}
		wakeup((caddr_t)&tp->t_outq.c_cf);
	}
	if (flag & FWRITE) {
		if (ip->wsel) {
			selwakeup(ip->wsel, ip->flags & WCOLL);
			ip->wsel = 0;
			ip->flags &= ~WCOLL;
		}
		wakeup((caddr_t)&tp->t_rawq.c_cf);
	}
}
