/*
 *	Copyright (c) 1984 AT&T
 *	Copyright (c) 1987 Motorola Inc
 *	All Rights Reserved
 *	@(#)kern-port:io/ram.c	35.2
 */

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF AT&T	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ident	"@(#)kern-port:io/ram.c	35.2"

#include "sys/param.h"
#include "sys/types.h"
#include "sys/sysmacros.h"
#include "sys/var.h"
#include "sys/map.h"
#include "sys/ram.h"
#include "sys/immu.h"
#include "sys/debug.h"
#include "sys/cmn_err.h"
#include "sys/systm.h"

extern struct mem memmap[];	/* map with flag comprising memory areas */
extern unsigned short mem_nbr;	/* number of entries in memmap[] */
extern struct ram ram_tbl[];	/* ram configuration table */
extern unsigned short ram_nbr;	/* number of entries in ram_tbl[] */
extern caddr_t bsmear();	/* zeros a block of ram, catches buserr */
extern int bprobe();		/* probes a byte of ram, catches buserr */
extern struct probe probe_tbl[];/* probe initialization table */
/*
 * RAM configuration assumptions.
 * 	Noncontiguous blocks of memory expected.
 *	Minimum granuality of memory assumed to be 1 click.
 *	It is found in the address range 0-16Mb.
 *	Locate each ram board comprising memory and call mfree() to add
 *	it to the memmap.  Include the private areas that must be
 *      mapped in as well.  Care must be taken so that an area of memory
 *	isn't freed twice.
 */
#define MAXPHYSCL	SYSSEGS

/*
 * raminit(physclick) - initializes memory.
 *	Zeros memory beginning at physclick and then higher according
 *	    to ram_tbl[].
 *	Initializes troublesome memory by probing it.
 *	Sets physmem to the total number of clicks of physical memory.
 *	Returns click number of last physical click found.
 */

int
raminit(physclick)
register int physclick;	/* first click to test and zero */
{
	void m_free();
	register struct ram *ram = ram_tbl;
	register unsigned short nbr = ram_nbr;
	register unsigned low, nxt, ram_end, end, top, siz, lastclick, physize;

	lastclick = physclick - 1;
	physize = 0;
	ram_end = MAXPHYSCL;
    	/*
	 * locate and initialize ram,
	 * all addresses and sizes from ram_tbl in clicks
	 */
	mapinit(memmap, mem_nbr);
	for ( ; nbr ; nbr--, ram++ ) {
		register unsigned int b_low, b_siz;

		low = ram->ram_lo;
		b_low = low;
		siz = ram->ram_sz;
		nxt = low + siz;
		b_siz = siz;

		if ( ram->ram_hi < ram_end )
			end = ram->ram_hi + 1;
		else
			end = ram_end;

		if ( low < physclick ) {/* start with physclick */
			b_siz -= physclick - low;
			b_low = physclick;
		}
		if ( siz > end - b_low )
			siz = end - b_low;

		if ( siz <= 0 ) {
			siz = end - low;
			ram->ram_flg |= RAM_FOUND;
			m_free( siz, low, ram->ram_flg );
			physize += siz;
			top = end - 1;
			continue;
		}

		for ( ; b_low < end; b_siz = siz, low = nxt, b_low = low,
		    nxt += siz ) {
			register unsigned int last;
	
			/* zero chunk of memory */
			if ( ram->ram_flg & RAM_INIT ) {
				last = btoct( bsmear(ctob( b_low ),
				    ctob( b_siz )));
				b_siz = last - low;
				if ( last == b_low )
					continue;
				else if ( !(ram->ram_flg & RAM_PRIV ) )
					goto exists;
			}

			if ( ram->ram_flg & RAM_PRIV ) {
				m_free( siz, low, ram->ram_flg );
				continue;
			}
		    
			/* probe chunk of memory */
			if ( ( ram->ram_flg & (RAM_INIT | RAM_HUNT ) )
			    == RAM_HUNT ) {
				register unsigned int b_end;

				for ( last = b_low, b_end = b_low + b_siz ;
				    last < b_end ; last++ ) {
					if ( bprobe ( ctob ( last ), -1 ) )
						break;
				}
				b_siz = last - low;
				if ( last == b_low )
					continue;
			}
exists:	
			ram->ram_flg |= RAM_FOUND;
			m_free( b_siz, low, ram->ram_flg );
			physize += b_siz;
			top = last - 1;
		}

		if ( ram->ram_flg & RAM_FOUND ) {
			if ( lastclick < top ) {
				lastclick = top;
			}
		}
	}

	physmem = physize;

	{ /* probe any required bytes */
	register struct probe *probe = probe_tbl;

	for( ; probe->probe_addr ; probe++ )
		(void) bprobe( probe->probe_addr, probe->probe_byte );
	}

	return( lastclick );
}
/*
 * Free the previously allocated space.
 * Sort a into map and combine on
 * one or both ends if possible.
 */
void
m_free(size, a, flag)
register int size;
register unsigned int a;
register int flag;
{
	void m_insert();
	void m_delete();
	register struct mem *bp, *ms = mapstart(memmap);
	register int tmp0, tmp1 = a + size;
static char msg[] = "m_free() overlapping ram items with different flags.\n\
current item	addr = %x, size = %x, flag = %x\n\
new item	addr = %x, size = %x, flag = %x\n";

	ASSERT(size > 0);
	if (size == 0)
		return;
	for (bp = ms; a > bp->m_addr && bp->m_size != 0; bp++);
	if ( bp > ms ) {
		/* coalesce overlapping entry */
		if (a >= bp->m_addr &&
		    (tmp0 = bp->m_addr + bp->m_size) >= a ) {
			if (tmp0 >= tmp1) {
				if (flag != bp->m_flag) {
					cmn_err(CE_WARN, msg,
					  bp->m_addr, bp->m_size,
					  bp->m_flag, a, size, flag);
				}
				return;
			}
			else {
				bp->m_size += tmp1 - tmp0;
				if ((bp+1)->m_size) {
					if (tmp1 > (bp+1)->m_addr) {
					    if (flag != (bp+1)->m_flag) {
						cmn_err(CE_WARN, msg,
						    (bp+1)->m_addr,
						    (bp+1)->m_size,
						    (bp+1)->m_flag,
						    a, size, flag);
					    }
					    m_delete(bp);
					}
				}
			}
		}
		/* coalesce with preceding entry */
		if (flag == (bp-1)->m_flag &&
		   (a >= (bp-1)->m_addr &&
		   (tmp0 = (bp-1)->m_addr + (bp-1)->m_size) >= a )) {
			if (tmp0 >= tmp1) {
				return;
			}
			else { /* check for new overlapping entry */
				(bp-1)->m_size += tmp1 - tmp0;
				if (bp->m_size) {
				    if ((tmp0 = (bp-1)->m_addr +
				      (bp-1)->m_size) >= bp->m_addr) {
					if (flag == bp->m_flag) {
					    m_delete(bp);
					}
					else if (tmp0 > bp->m_addr) {
						cmn_err(CE_WARN, msg,
						    (bp-1)->m_addr,
						    (bp-1)->m_size,
						    (bp-1)->m_flag,
						    bp->m_addr,
						    bp->m_size,
						    bp->m_flag);
					}
				    }
				}
			}
		}
		else
			m_insert(bp, size, a, flag);
	}
	else { /* coalesce with current entry */
		if (flag == bp->m_flag &&
		    (tmp1 == bp->m_addr && bp->m_size )) {
			bp->m_addr = a;
			bp->m_size += size;
			ASSERT(bp == ms ||
			    ((bp-1)->m_addr + (bp-1)->m_size) <
			    bp->m_addr);
		}
		else {
			ASSERT(size);
			if (mapsize(memmap) == 0) {
				cmn_err(CE_WARN,"m_free mem overflow %x.\
  Lost %d items at %d\n",memmap,size,a) ;
				return;
			}
			m_insert(bp, size, a, flag);
		}
	}
}

void
m_insert(bp, size, a, flag)
register struct mem *bp;
register int size;
register unsigned int a;
register int flag;
{
	register unsigned int t0, t1;

	do {
		t0 = bp->m_addr;
		bp->m_addr = a;
		a = t0;
		t0 = bp->m_size;
		bp->m_size = size;
		t1 = bp->m_flag;
		bp->m_flag = flag;
		flag = t1;
		bp++;
	} while (size = t0);
	mapsize(memmap)--;
}

void
m_delete(bp)
register struct mem *bp;
{
	(bp-1)->m_size += bp->m_size;
	while (bp->m_size) {
		bp++;
		(bp-1)->m_addr = bp->m_addr;
		(bp-1)->m_size = bp->m_size;
		(bp-1)->m_flag = bp->m_flag;
	}
	mapsize(memmap)++;
}
