/*	Copyright (c) 1987 Motorola Inc
	All Rights Reserved
	@(#)kern-port:io/ramdisk.c	35.2
*/
#ident	"@(#)kern-port:io/ramdisk.c	35.2"

/*	INCLUDE files 	*/

#include "sys/param.h"
#include "sys/types.h"
#include "sys/signal.h"
#include "sys/sysmacros.h"
#include "sys/systm.h"
#include "sys/buf.h"

#ifdef REL2
#include "sys/dir.h"
#else
#include "sys/fs/s5dir.h"
#include "sys/immu.h"
#include "sys/var.h"
#endif

#include "sys/user.h"
#include "sys/errno.h"
#include "sys/map.h"


#define	DEBUG
#undef	DEBUG

/*
 *	The constant "CLRMEM" may be defined upon compilation
 *	of this module to activate code that will zero the
 *	memory containing the ramdisk
 *	at driver initialization.
 *	However, this action will also clear a ramdisk image that
 *	may have been placed in memory before system startup.
 */


#ifdef	DEBUG		/* debugging code */

#define CLRMEM
#define	STATIC
#define	DEBUGPR(x,d,s)	{ if( x & ramdDebug ) ramdprint(d,s); }

int	ramdDebug = 0;

#define	DBINIT		(1<<0)
#define	DBOPEN		(1<<1)
#define	DBCLOSE		(1<<2)
#define	DBSTRAT		(1<<3)
#define	DBREAD		(1<<4)
#define	DBWRITE		(1<<5)

#else			/* no debugging code */

#define	STATIC	static
#define	DEBUGPR(x,d,s)

#endif

#define	MAXNSLICE	8		/* slices per device */

/*
 *	from io.h
 */

struct size
{
	daddr_t nbytes;		/* size of slice in bytes */
	daddr_t byteoff;	/* first physical byte in slice */
};

extern struct size ramd_sizes[];

/*
 *	conf.c
 */

extern	int	ramd_cnt;

/*
 *	bcopy protection from panic() on bus error
 */

#ifdef REL2 	
extern	int	nofault;
#endif

/*
 *
 *	ramdinit: initialize the ramdisk driver
 */

ramdinit()
{
	register struct size *sp;
	register c = ramd_cnt*MAXNSLICE;
	register n;

	ramd_cnt = c;
	
	DEBUGPR(DBINIT,0,"init");

	for( sp = &ramd_sizes[0], n = 0; n < c; n++, sp++ ) {
		if( sp->byteoff == 0 )
			continue;
		if((sp->nbytes == 0) || bprobe(sp->byteoff,-1) || ramdchk(sp)){
			sp->byteoff = 0;
			continue;
		}
#ifdef CLRMEM
#ifdef REL2
		nofault = 1;	/* don't panic! */
#endif
		if( bzero( sp->byteoff, sp->nbytes ) )
			sp->byteoff = 0;	/* something's missing */
#ifdef REL2
		nofault = 0;
#endif
#endif
	}
}

/*
 *	ramdchk: verify that the given range is not in memmap
 */

STATIC ramdchk(sp)
register struct size *sp;
{	register low;
	register high;
#ifdef REL2	/* Until R3 maps memory */
	register struct map *mp = mapstart(memmap);
	register m;

	low = btoc(sp->byteoff);
	high = btoc(sp->nbytes)+low;

	/*
	 *	check memmap
	 *
	 *	[l...h)	== l,l+1,...,h-1
	 *
	 *	Given address ranges	(1) [low...high)
	 *			and	(2) [m_addr...m_addr+m_size)
	 *
	 *	then the two ranges do NOT overlap if:
	 *
	 *		the top of range (1) is less than or equal to
	 *		the bottom of range (2) (high <= m_addr)
	 *
	 *	or	the bottom of range (1) is greater than or equal
	 *		to the top of range (2) (low >= m_addr+m_size)
	 *				
	 */

	for( m=1; m < mapsize(memmap); m++, mp++ ) {
		if( high <= mp->m_addr )		/* low enough? */
			continue;
		if( low >= (mp->m_addr+mp->m_size) )	/* high enough? */
			continue;
		/* overlap! */
		return(1);
	}

#endif
	return(0);	/* memory range ok for ramdisk */
}

/*
 *	ramdopen: open a ramdisk device
 */

ramdopen(dev)
register dev;
{
	DEBUGPR(DBOPEN,dev,"open");

	/*
	 *	if the device is nonexistent, then error
	 */

#ifdef REL2
	if(( dev > ramd_cnt )
#else
	if(( minor(dev) > ramd_cnt )
#endif
		|| ( ramd_sizes[minor(dev)].byteoff == 0 )) {
		u.u_error = ENXIO;
		return;
	}

	DEBUGPR(DBOPEN,dev,"open successful");
}

/*
 *	ramdclose: close device
 */
ramdclose(dev)
{
	DEBUGPR(DBCLOSE,dev,"close");
}

/*
 *	ramdstrategy: perform an I/O
 */

ramdstrategy(bp)
register struct buf *bp;
{
	register struct size *sp = &ramd_sizes[minor(bp->b_dev)];
	register last;		/* last block in slice */
	register unsigned src;	/* ram address */
	register unsigned dst;
	register unsigned ram;

	DEBUGPR(DBSTRAT,bp->b_dev,"strategy");

	last = sp->nbytes >> BSHIFT;

	if ((bp->b_blkno < 0) || (bp->b_blkno >= last)) {
		if ((bp->b_blkno == last) && (bp->b_flags&B_READ))
			bp->b_resid = bp->b_bcount;
		else {
			bp->b_flags |= B_ERROR;
			bp->b_error = ENXIO;
		}

	} else {
		ram = (unsigned)(sp->byteoff + (bp->b_blkno << BSHIFT));

		if(( last = bp->b_bcount) > sp->nbytes )
			last = sp->nbytes;

		/* last == no. of bytes to copy */

		if( last > 0 )  {
		
			if(bp->b_flags & B_PHYS) {
				iomove(ram,last,
					((bp->b_flags&B_READ)?B_READ:B_WRITE));	
			} else {
				if(bp->b_flags & B_READ) {
					src = ram;
					dst = paddr(bp);
				} else {
					src = paddr(bp);
					dst = ram;
				}
#ifdef REL2
				nofault = 1;/* no panic if bad addr on bcopy */
#endif
				if( bcopy( src, dst, last ) ) {
					/* copy error */
					ramdprint( bp->b_dev, "Error" );
					bp->b_flags |= B_ERROR;
					bp->b_error = ENXIO;
					/* nofault = 0 already done */
				} else {
#ifdef REL2
					nofault = 0;
#endif
				}
			}
			bp->b_resid = bp->b_bcount - last;
		}
	}

	iodone(bp);
}

/*
 *	ramdread: character device read
 */

ramdread(dev)
register dev;
{
	register struct size *sp = &ramd_sizes[minor(dev)];

	DEBUGPR(DBREAD,dev,"read");

	/* guarantee block alignment of both offset and count */
	if ((u.u_offset & BMASK) || (u.u_count & BMASK)) {
		u.u_error = EINVAL;
		return;
	}

	if( physck( (sp->nbytes >> BSHIFT),B_READ))
		physio( ramdstrategy, 0, dev, B_READ );
}

/*
 *	ramdwrite: character device write
 */

ramdwrite(dev)
{
	register struct size *sp = &ramd_sizes[minor(dev)];

	DEBUGPR(DBREAD,dev,"write");

	/* guarantee block alignment of both offset and count */
	if ((u.u_offset & BMASK) || (u.u_count & BMASK)) {
		u.u_error = EINVAL;
		return;
	}

	if( physck( (sp->nbytes >> BSHIFT),B_WRITE))
		physio( ramdstrategy, 0, dev, B_WRITE );
}

/*
 *	ramdprint: driver print procedure
 */

ramdprint(dev,s)
{	register x;
	x = spl6();
	printf("RAMDISK: %s on slice %d\n", s, minor(dev) );
	splx(x);
}

/*
 *	ramddump: dump routine
 */

ramddump()
{
}
