/*
 *                  Copyright (c) 1988 Ferranti Plc.
 *                      All Rights Reserved
 *
 *
 * THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF FERRANTI PLC THE COPYRIGHT 
 *                   OF WHICH VESTS IN FERRANTI PLC.
 *
 *   The copyright notice above does not evidence any actual or intended 
 *                   publication of such source code.
 *
 * This source code and the information contained herein is supplied without
 * liability for errors or ommissions and no part may be copied, re-produced, 
 * used or disclosed except as authorised by the Ferranti organisation by 
 *                contract or by other written permission.
 */

/******************************************************************************

	Rev = 5.1		Date of 'get'   = 14:15:55 2/14/89
				Date of 'delta' = 14:48:29 1/25/89

	File = scc.c

******************************************************************************/

#ident "@(#)scc-driver:driver/scc.c Rev 5.1 14:48:29 1/25/89"

#include <sys/param.h>
#include <sys/types.h>
#include <sys/errno.h>
#include <sys/signal.h>
#include <sys/fs/s5dir.h>
#include <sys/user.h>
#include <sys/immu.h>
#include <sys/region.h>
#include <sys/proc.h>
#include <sys/sysinfo.h>
#include <sys/conf.h>
#include <sys/termio.h>
#include <sys/tty.h>
#include <sys/file.h>
#include <sys/cmn_err.h>
#include <sys/ttold.h>

#include <local/scc/fifo.h>
#include <local/scc/lowmem.h>
#include <local/scc/duartchan.h>
#include <local/scc/duartdefs.h>
#include <local/scc/termctrl.h>
#include <local/scc/scctermio.h>
#include <local/scc/scccmd.h>
#include <local/scc/sccdctrl.h>
#include <local/scc/sccioctl.h>

/*
 * Define standard booleans
 */
#define	TRUE	1
#define	FALSE	0

/*
 * Define water mark
 */
#define	WATER 	25
#define MIN(a,b)	( a > b ? b : a )

#define	READ_BYTE	-1

/*
 * Card offset address
 */
#define	COA	(unsigned char *)scc_addr[cindex]
#define	COAUL	((unsigned long)scc_addr[cindex])
#define	UL(x)	((unsigned long)x)
#define UC(x)	((unsigned char)(x))
#define UCP(x)	((unsigned char *)(x))

/*
 * Processor priority level
 */
#define	SPLX(x)	splx(scc_ilev[x]<<8)

/*
 * Convert a device number into just the minor device number
 */
#define	minor(x)	x&0xFF

/* 
 *
 * External declarations generated by config from dfile and master.
 * They appear in the generated source file conf.c
 *
 */

extern struct tty scc_tty[];
extern int scc_cnt;
extern int scc_addr[];
extern int scc_ilev[];

/*
 * Extern declarations for structures defined in scc_conf.c
 */
extern struct duart_channel *scc_duart[];
extern struct scc_ctrl scc_ctrl[];

extern int scc_msg, scc_tty_max, scc_card_max;

#define	NO_STTIN	0x4e4f4e45 	/* scc_line value if not in linesw */
#define	NO_SXTIN	0x4e4f4e45 	/* sxt_line value if not in linesw */

static int sxt_line;		/* index into linesw table for sxt driver */
static int scc_line;		/* index into linesw table for scc driver */

/*
 * Forward function definitions
 */
void get_dc();
void rel_dc();
dev_t valid_tty();

/*
 *
 * dev_t valid_tty(tp)
 *    struct tty *tp;
 *
 * Validates the tty pointer. Is it the address of a scc driver tty
 * structure ?
 * Returns -1 if invalid, otherwise returns minor device number of type dev_t.
 */
dev_t valid_tty(tp)
register struct tty *tp;
{
	dev_t mdev;
	
	mdev = ((int)tp - (int)scc_tty) / sizeof(struct tty);

	if (valid_mdev(mdev) == -1) {
		u.u_error = EINVAL;
		return((dev_t)-1);
	}
	else {
		return(mdev);
	}

}	/* end valid_tty */

/*
 *
 * int valid_mdev(mdev)
 *    dev_t mdev;
 *
 * Validates the the mdev parameters as being a valid minor device number.
 * Returns -1 if invalid otherwise the index into the scc_addr[] array
 * for this mdev number.
 */
int valid_mdev(mdev)
dev_t mdev;
{
	register cindex; 	/* index into scc_addr[] array */

	if ( (mdev < 0) || (mdev >= scc_cnt) ) {
		/*
	 	 * mdev is out of range.
		 */
		u.u_error = EINVAL;
		return(-1);
	}

	/* 
	 * Should never fail on next test, but need to know cindex anyway 
	 */
	cindex = mdev / PORTS_PER_SCC;

	if ( COA == (unsigned char *)-1 ) {
		/*
		 * If address is -1, no card is present
		 */
		u.u_error = EINVAL;
		return(-1);
	}

	if ( scc_ctrl[mdev].scc_chan_present != TRUE ) {
		/*
		 * This port not present on this card
		 */
		u.u_error = EINVAL;
		return(-1);
	}

	return(cindex);

}	/* end valid_mdev */

/*
 *
 * sccproc
 */
static sccproc(tp, cmd)
register struct tty *tp;
unsigned cmd;
{
	register struct duart_channel *dc; /* duart channel control block */
	register struct fifo * ffoutp;	/* pointer to output fifo */
	dev_t mdev;			/* minor device number   */
	register nchar;
	register n, s;
	int	cindex;

	if ( (mdev=valid_tty(tp)) == -1 ) return;
	dc = scc_duart[mdev];
	cindex = mdev / PORTS_PER_SCC;

	switch(cmd) {
	case T_WFLUSH:
		s = SPLX(cindex);
		get_dc(dc,mdev);
		dc->dc_modifier = OFLUSH;
		wait_cmd_done(dc,tp,SIODUMMY,mdev);
		splx(s);
		break;
	case T_RFLUSH:
		s = SPLX(cindex);
		get_dc(dc,mdev);
		dc->dc_modifier = IFLUSH;
		wait_cmd_done(dc,tp,SIODUMMY,mdev);
		splx(s);
		break;
	case T_OUTPUT:
OUTPUT:
		if ( tp->t_state & TTSTOP ) break;
		/* A little worried about this bit */
		n = (*linesw[UC(tp->t_line)].l_output)(tp);
		/* 
	 	 * If still NULL pointer after calling sxtout do nothing
		 */
		if ( tp->t_tbuf.c_ptr )
		{
			ffoutp = (struct fifo *)(UL(dc->dc_hostoutf) + COAUL);
							
			nchar = tp->t_tbuf.c_size; /* - tp->t_tbuf.c_count;*/
			/* 
			 * Position pointer
			 */
			tp->t_tbuf.c_ptr -= tp->t_tbuf.c_size - tp->t_tbuf.c_count;
			for ( ; nchar; )
			{
				s = SPLX(cindex);
				while ( fifo_entries(ffoutp) > FFSIZE(ffoutp) - WATER )
				{
					dc->dc_flags[HOSTOPWAIT] = TRUE;
					tp->t_state |= OASLP;
					sleep( (caddr_t)&tp->t_outq, TTOPRI );
				}
				n  = MIN( nchar, fifo_room(ffoutp) );
				while ( --n >= 0 )
				{
					bfifoput(ffoutp, *(tp->t_tbuf.c_ptr++));
					--nchar;
				}
				splx(s);
			}
		}
		break;
	case T_RESUME:
		tp->t_state &= ~TTSTOP;
		goto OUTPUT;
	case T_SUSPEND:
		tp->t_state |= TTSTOP;
		break;
	case T_INPUT:
		/*
		 * No action required
		 */
		break;
	case T_TIME:
		printf("sccproc: T_TIME\n");
		break;
	case T_BLOCK:
		printf("sccproc: T_BLOCK\n");
		break;
	case T_UNBLOCK:
		printf("sccproc: T_UNBLOCK\n");
		break;
	case T_BREAK:
		printf("sccproc: T_BREAK\n");
		break;
	case T_PARM:
		printf("sccproc: T_PARM\n");
		break;
	case T_SWTCH:
		printf("sccproc: T_SWTCH\n");
		break;
	default:
	    	u.u_error = EINVAL;
		break;
	}
}

/*
 *
 * int wait_cmd_done(dc, tp, cmd, mdev)
 *    struct duart_channel dc;
 *    struct tty *tp;
 *    unsigned short cmd;
 *    dev_t mdev;
 *
 * wait_cmd_done() issues the specified command and then waits for the 
 * command to be executed by the firmware
 * Returns TRUE if have an error, else FALSE.
 */
int wait_cmd_done(dc, tp, cmd, mdev)
register struct duart_channel *dc;
register struct tty *tp;
unsigned short cmd;
register dev_t mdev;
{
	register s;
	ushort cmd_err;
	register int sync_cnt;

	/*
	 * Check dc_cmd_lock is set
	 */
	if (!dc->dc_cmd_lock) {
		cmn_err(CE_WARN,
		   "scc(wait_cmd_done): Channel %d dc_cmd_lock not set",mdev);
		/*
		 * The command structure has not been locked. Driver fault.
		 */
		 return(FALSE);
	}

	cmd_err = 0;

	switch (cmd) {
	case SIOOPEN:
	case SIOCLOSE:
	case SIODUMMY:
	case SIOSET:
	case SIOBRK:
		s = SPLX(mdev/PORTS_PER_SCC);
		dc->dc_flags[HOSTCMDWAIT] = TRUE;
		dc->dc_cmd = cmd;
		tp->t_state |= TTIOW;

		/*
		 * Watch out. sleep will return 0 if a signal is sent and
		 * the handler is set to SIG_IGN. Thus must check reason
		 * for return even if get 0 back. TTIOW will be clear
		 * if had a wakeup from the interrupt handler.
		 *
		 * SIOCLOSE must not be allowed to abort
		 */

sleep_again:		
		while (tp->t_state & TTIOW) {

			if (sleep((caddr_t)&dc->dc_cmd, TTOPRI|PCATCH)) {

				/* 
				 * Sleep interrupted.
				 * If an SIOCLOSE decrease timeout to 1 (can't
				 * use zero as would be wait forever) as an
				 * SIOCLOSE is not interruptable. 
				 * Other commands try and sync with firmware.
				 * Allow upto 10 seconds
				 */
				if (dc->dc_cmd == SIOCLOSE) {
					dc->dc_draintout = 1;
					goto sleep_again;
				}

				dc->dc_flags[SYNCHAND] = FALSE;

sync_again:
				dc->dc_cmd = SIOSYNC;
				dc->dc_arg1 = cmd;

				sync_cnt = 0;
				while (dc->dc_cmd && (++sync_cnt <= 150) )
					delay(4);   /* 1/15th sec */
			
				/*
				 *  If dc->dc_cmd, check for handshake.
				 */
				if (dc->dc_cmd == 0) {
					if (dc->dc_flags[SYNCHAND] == FALSE) {
						goto sync_again;
					}
				}

				/*
				 * Did we sync or timeout
				 */
				if (dc->dc_cmd)
					cmn_err(CE_WARN,
	"scc(wait_cmd_done): Channel %d, failed to sync with firmware",mdev);
				/*
				 * Final tidyup then longjmp
				 */
				dc->dc_cmd_lock = FALSE;

				if (scc_ctrl[mdev].scc_cmd_wait) {
					scc_ctrl[mdev].scc_cmd_wait = FALSE;
					wakeup(&scc_ctrl[mdev].scc_cmd_wait);
				}
				splx(s);
				longjmp(u.u_qsav);

			}	/* end of if sleep() */

		} 	/* end of while (tp->t_state & TTIOW) */
		
		/*
		 * Sleep terminated and TTIOW has been cleared
		 */
		
		if (dc->dc_cmd)
			cmn_err(CE_WARN,
	"scc(wait_cmd_done): Channel %d woken, but dc->dc_cmd not zero",mdev);

		if (cmd_err = dc->dc_cmd_error)
			dc->dc_cmd_error = 0;

		dc->dc_cmd_lock = FALSE;
		if (scc_ctrl[mdev].scc_cmd_wait) {
			scc_ctrl[mdev].scc_cmd_wait = FALSE;
			wakeup(&scc_ctrl[mdev].scc_cmd_wait);
		}

		if (cmd_err) {
			u.u_error = EIO;
			cmn_err(CE_CONT,
			   "!scc(wait_cmd_done): Channel %d (0x%x) error %d\n",
							mdev,dc,cmd_err);
		}
		splx(s);
		break;
	default:
		cmn_err(CE_WARN,
		   "scc(wait_cmd_done): Called with invalid command (0x%x)",
									cmd);
		break;

	}	/* end of switch(cmd) */

	if (cmd_err)
		return(TRUE);
	else
		return(FALSE);

}	/* end of wait_cmd_done */


/*
 *
 * int get_dc(dc, mdev)
 *    register struct duart_channel *dc;
 *    register dev_t mdev;
 *
 * Lock the duart_channel structure specified by dc. If the structure is
 * already locked then when the command that has it locked terminates it
 * will wakeup all tasks that are sleeping.
 * Only ONE task may access the duart_channel structure at a time. Thus this
 * code will implement a 'critical section scheme'.
 */
void get_dc(dc,mdev)
register struct duart_channel *dc;
register dev_t mdev;
{
	register s;
	
	s = SPLX(mdev/PORTS_PER_SCC);
	while (tas_mem(&dc->dc_cmd_lock) == -1) {
		scc_ctrl[mdev].scc_cmd_wait = TRUE;
		sleep(&scc_ctrl[mdev].scc_cmd_wait,TTOPRI);
	}
	splx(s);

} 	/* end of get_dc */

/*
 * int rel_dc(dc, mdev)
 *    register struct duart_channel *dc;
 *    register dev_t mdev;
 *
 * Release a duart_channel structure specified by dc. Wake anyone who is
 * sleeping.
 */
void rel_dc(dc,mdev)
register struct duart_channel *dc;
register dev_t mdev;
{
	register s;
	
	s = SPLX(mdev/PORTS_PER_SCC);
	dc->dc_cmd_lock = FALSE;
	if (scc_ctrl[mdev].scc_cmd_wait) {
		scc_ctrl[mdev].scc_cmd_wait = FALSE;
		wakeup(&scc_ctrl[mdev].scc_cmd_wait);
	}
	splx(s);
}


/*
 *
 * sccopen(dev, flag)
 *    unsigned int dev, flag;
 *
 * Open the specified device. The dev argument holds both the major and minor
 * device numbers.
 *
 * The flag argument (file open mode bits) must be none zero. If FNDELAY is 
 * set, i.e. O_NDELAY set in open system call, must wait for DCD.
 */
sccopen(dev, flag)
unsigned int dev;
unsigned int flag;
{
	register struct duart_channel *dc;
	register struct tty *tp;
	register dev_t mdev;		/* minor device number of channel */
	register struct term_control *tcp; 
	unsigned short *sptr;		/* Used to load tc_?flag values   */
	int cindex;		        /* Must be set for use by COA()   */

	mdev = (dev_t)minor(dev);

	if ( (cindex = valid_mdev(mdev)) == -1) {
		/*
		 * Specified minor device not present. u.u_error will have 
		 * been set by valid_mdev().
		 */
		if (scc_msg >= 2)
			cmn_err(CE_WARN,
"scc(sccopen): Channel %d not present or card not initialised",
							mdev);
		else
			cmn_err(CE_CONT,
"!scc(sccopen): Channel %d not present or card not initialised\n",
							mdev);
		return;
	}
	
	/*
	 * Check scc_line is NOT NO_STTIN. If it is error
	 */

	if (scc_line == NO_STTIN) {
		cmn_err(CE_WARN,
"scc(sccopen): Channel %d open rejected. No SCC support via linesw",
							mdev);
		u.u_error = EINVAL;
		return;
	}

	if (flag == 0x0) {
		/*
		 * Should never happen. Must be opened for read or write !
		 */
		cmn_err(CE_WARN,
		   "scc(sccopen): Channel %d open request, but flag == 0x0",
							mdev);
		u.u_error = EINVAL;
		return;
	}

	/*
	 * Setup pointers to duart_channel structure in the firmware and
	 * the drivers tty structure in scc_tty.
	 */
	dc = scc_duart[mdev];
	tp = &scc_tty[mdev];

	/*
	 * Is this the first call ?
	 * If it is then must do a 'full' initialisation of the driver
	 * and the firmware.
	 * ISOPEN is set by ttopen().
	 */
	if (!(tp->t_state & ISOPEN)) {

		/*
		 * Lock the command structure for subsequent use and thus
		 * make this code into a critical section.
		 */
		get_dc(dc,mdev);

		/*
		 * Now for a little check. Did someone else go through here
		 * before I got woken. If so don't call firmware again !
		 * There are no 'sleeps' between wait_cmd_done being woken
		 * and ISOPEN being set. Thus if ISOPEN is now set just get an
		 * rbuf and exit.
		 */
		if (tp->t_state & ISOPEN) {
			rel_dc(dc,mdev);
			(*linesw[UC(tp->t_line)].l_open)(tp);
			return;
		}

		/*
		 * Now have duart_channel structure and thus can guarantee 
		 * no other command is active.
	 	 */
		ttinit(tp);		/* set default ?flag values */
		tp->t_line = scc_line;
		tp->t_proc = sccproc;
		tp->t_term = 0;

		/*
		 * Ensure HOSTCLIST flag is clear. Thus always start without
		 * support for generation of clists.
		 */
		dc->dc_flags[HOSTCLIST] = FALSE;

		/*
		 * Setup command arguments 
		 *
		 * dc_arg1 Currently must be 0
		 * dc_arg2 Currently must be 0
		 * dc_arg3 is the firmware discipline to use. 0 for the 
		 *         normal UNIX line discipline.
		 * dc_arg4 is used to determine which input signals are
		 *         required by the SCC card before the SIOOPEN
		 *         command returns. If CLOCAL is not set then DSR
		 *         is required. If it is not present an error is
		 *         returned; however WAIT_DSR causes a wait until have
		 *         DSR input. If WAIT_DCD is set then a DCD input is
		 *         also required.
		 *
		 */
		dc->dc_arg1 = 0;
		dc->dc_arg2 = 0;
		dc->dc_arg3 = UNIXLD0;

		if (flag & FNDELAY)
			dc->dc_arg4 = WAIT_NEVER; 
		else
			dc->dc_arg4 = (WAIT_DCD | WAIT_DSR); 

		/*
		 * Setup the terminal configuration block in the firmware
		 */
	
		tcp = dc->dc_tcp;	/* dc->dc_tcp is card relative */
		tcp = (struct term_control *)( UL(tcp) + COAUL);
		
		/*
		 * tcp is now an absolute address pointing to this channels
		 * terminal control block.
		 * Load bottom 16 bits of each tc_?flag with the corresponding
		 * UNIX flag.
		 */				
		sptr = (unsigned short *)(UL(&tcp->tc_iflag)); 
		*sptr = 0;	/* Clear top 16 bits */
		sptr = (unsigned short *)( UL(&tcp->tc_iflag)+2); 
		*sptr = tp->t_iflag;

		sptr = (unsigned short *)(UL(&tcp->tc_oflag)); 
		*sptr = 0;	/* Clear top 16 bits */
		sptr = (unsigned short *)( UL(&tcp->tc_oflag)+2); 
		*sptr = tp->t_oflag;

		/*
		 * Any controls are left set (nortscts, ignnodcd, ignnodsr)
		 * Split speed is cleared by the close call to the f/w dropping
		 * speed to B300.
		 */
		sptr = (unsigned short *)( UL(&tcp->tc_cflag)+2); 
		*sptr = tp->t_cflag;

		sptr = (unsigned short *)(UL(&tcp->tc_lflag)); 
		*sptr = 0;	/* Clear top 16 bits */
		sptr = (unsigned short *)( UL(&tcp->tc_lflag)+2); 
		*sptr = tp->t_lflag;
	
		/*
		 * Load the control characters array. Firmware will set
		 * default values for any not initialised by driver.
		 */
		tcp->tc_control_chars[SIO_VINTR]   = tp->t_cc[VINTR];
		tcp->tc_control_chars[SIO_VQUIT]   = tp->t_cc[VQUIT];
		tcp->tc_control_chars[SIO_VERASE]  = tp->t_cc[VERASE];
		tcp->tc_control_chars[SIO_VKILL]   = tp->t_cc[VKILL];
		tcp->tc_control_chars[SIO_VEOF]    = tp->t_cc[VEOF]; /* VMIN  */
		tcp->tc_control_chars[SIO_VEOL]    = tp->t_cc[VEOL]; /* VTIME */
		tcp->tc_control_chars[SIO_VSWTCH]  = tp->t_cc[VSWTCH];

		/*
		 * Set modifier bits to flush both input and output fifos
		 */
		dc->dc_modifier = (OFLUSH | IFLUSH);
		wait_cmd_done(dc,tp,SIOOPEN,mdev);
		
		if (u.u_error)  {
			cmn_err(CE_CONT,
				"scc(sccopen): Channel %d open error\n",mdev);
			return;
		}
	
		/*
		 * If we get here must have correct DCD state. Set CARR_ON
		 */	
		tp->t_state |= CARR_ON;

	}

	/*
	 * This calls the 'high level' open routine, usually ttopen, via 
	 * the linesw table. ISOPEN set in ttopen().
	 * 
	 * ttopen will cause, via ttioctl with paramter LDOPEN, a call of
	 * sccproc(T_INPUT). Sequence will allocate an rbuf.
	 */
	(*linesw[UC(tp->t_line)].l_open)(tp);
	
}	/* end of sccopen */

/*
 *
 * sccclose(dev, flag)
 *    unsigned int dev, flag;
 *
 * Close the specified device. The dev argument holds both the major and minor
 * device numbers.
 * The argument flag is not used.
 */
sccclose(dev, flag)
unsigned int dev;
unsigned int flag;
{
	register struct duart_channel *dc;
	register struct tty *tp;
	register dev_t mdev;
	int cindex;


	mdev = (dev_t)minor(dev);

	if ( (cindex = valid_mdev(mdev)) == -1) {
		/*
		 * Specified minor device not present. u.u_error will have 
		 * been set by valid_mdev().
		 */
		return;
	}

	/*
	 * Setup pointers to duart_channel structure in the firmware and
	 * the drivers tty structure in scc_tty.
	 */
	dc = scc_duart[mdev];
	tp =  &scc_tty[mdev];
	
	if ((tp->t_state & ISOPEN) == 0) {
		/*
		 * UNIX doesn't think the channel is open, check with firmware
		 */
		if ((dc->dc_state & CHANNEL_OPEN) == 0)
			return;
		else {
			/*
			 * Consistency fault. Driver says closed, firmware 
			 * say open. Should never happen. Log it
			 */
			cmn_err(CE_WARN,
		"scc(sccclose): Channel %d has !ISOPEN, but CHANNEL_OPEN",
							mdev);
			return;
		}
	}

	/*
	 * Get the duart channel before trying to issue command. Makes
	 * this code into a critical section.
	 */
	get_dc(dc,mdev);

	/*
	 * Clear down state flags an process group
	 */
	tp->t_state &= ~ISOPEN;
	tp->t_state &= ~IASLP;
	tp->t_state &= ~OASLP;
	tp->t_pgrp = 0;

	/*
	 * Set arg1 so that firmware will drop line speed to 300, as part of
	 * the close operation, and thus not poll any faster that is required
	 */
	dc->dc_arg1 = TRUE;

	/*
	 * Set modifier so that will wait for output to drain before close
	 */

	dc->dc_modifier = ODRAIN;

	/*
	 * Specify a command drain timeout
	 */
	dc->dc_draintout = 3000;
	wait_cmd_done(dc,tp,SIOCLOSE,mdev);
}

/*
 *
 * sccioctl(dev, cmd, arg, mode)
 *    unsigned int dev;
 *    int cmd, arg, mode;
 */
sccioctl(dev, cmd, arg, mode)
unsigned int dev;
int cmd, arg, mode;
{
	register struct duart_channel *dc;
	register struct tty * tp;
	struct termio tio;
	register struct term_control *tcp; /* terminal control block pointer*/
	struct sgttyb tb;
	unsigned short *sptr;
	int line;
	register s;
	register mdev;
	int flag, cindex;;



	/*
	 * Convert dev to just the minor device number and then validate it.
	 * The call to valid_mdev will set u.u_error.
	 */
	mdev = minor(dev);
	if ((cindex = valid_mdev(mdev)) == -1)
		return;

	/*
	 * Setup pointers to structures scc_duart, scc_tty and the channels
	 * terminal control block via dc->dc_tcp.
	 */
	dc = scc_duart[mdev];
	tcp = dc->dc_tcp;
	tp =  &scc_tty[mdev];

	switch(cmd) {
	case TCSETAW:
	case TCSETAF:
	case TCSETA:
		if (copyin(arg,(caddr_t)&tio,sizeof(struct termio))) {
			u.u_error = EFAULT;
			break;
		}

		line = (int)tio.c_line;
		if ( (line < 0) || (line >= linecnt) ) {
			u.u_error = EINVAL;
			break;
		}

		/*
		 * Lock the dc_cmd sub-structure before we go any further
		 * as another command could be waiting to complete.
		 * This also stops two or more tasks going through this
		 * code sequence 'at once' (i.e 2nd one starting while 1st
		 * one is sleeping waiting for the firmware)
		 */
		get_dc(dc,mdev);

		/*
		 * In effect we are now in a critical section as only one
		 * user thread may be active beyond here at once.
		 * The tp tty structure is owned by a terminal line and may
		 * therefore be accessed by several processes. Changes to it
		 * must be 'atomic'. In days of old this was accomplised simply
		 * by not sleeping in the code sequence. However the driver
		 * sleeps waiting for the command to complete.
		 *
		 * Change line discipline ?
	 	 *
	 	 * If we are using the scc_line line discipline then clists are
		 * not in use so HOSTCLIST must be false.
		 */
		tp->t_line = (char)line;

		if (tp->t_line == (char)scc_line)
			dc->dc_flags[HOSTCLIST] = FALSE;
		else
			dc->dc_flags[HOSTCLIST] = TRUE;

		/*
		 * Load the local tty structure
		 */
		tp->t_iflag = tio.c_iflag;
		tp->t_oflag = tio.c_oflag;
		tp->t_cflag = tio.c_cflag;
		tp->t_lflag = tio.c_lflag;

		/*
		 * Compute the address of the firmwares terminal control block
		 */
		tcp = (struct term_control *)(UL(tcp) + COAUL);
	
		/*
		 * Ensure tcp area same as dc area
		 */	
		tcp->tc_iflag = dc->dc_iflag;
		tcp->tc_oflag = dc->dc_oflag;
		tcp->tc_cflag = dc->dc_cflag;
		tcp->tc_lflag = dc->dc_lflag;

		/*
		 * Load the terminal control block. UNIX, normally, only
		 * accesses the bottom 16 bits of each flag, hence all this
		 * pointer work.
		 */
		sptr = (unsigned short *)(UL(&tcp->tc_iflag)+2); 
		*sptr = tio.c_iflag;
		sptr = (unsigned short *)(UL(&tcp->tc_oflag)+2); 
		*sptr = tio.c_oflag;
		sptr = (unsigned short *)(UL(&tcp->tc_cflag)+2); 
		*sptr = tio.c_cflag;
		sptr = (unsigned short *)(UL(&tcp->tc_lflag)+2); 
		*sptr = tio.c_lflag;
	
		/*
		 * Load the control characters
		 */
		bcopy(tio.c_cc, tp->t_cc, NCC);
		tcp->tc_control_chars[SIO_VINTR]  = tio.c_cc[VINTR];
		tcp->tc_control_chars[SIO_VQUIT]  = tio.c_cc[VQUIT];
		tcp->tc_control_chars[SIO_VERASE] = tio.c_cc[VERASE];
		tcp->tc_control_chars[SIO_VKILL]  = tio.c_cc[VKILL];
		tcp->tc_control_chars[SIO_VEOF]   = tio.c_cc[VEOF]; /* VMIN  */
		tcp->tc_control_chars[SIO_VEOL]   = tio.c_cc[VEOL]; /* VTIME */
		tcp->tc_control_chars[SIO_VSWTCH] = tio.c_cc[VSWTCH];

		/*
		 * Set the command modifier bits
		 */
		if (cmd == TCSETAW)
			dc->dc_modifier = ODRAIN;
		else if (cmd == TCSETAF)
			dc->dc_modifier = (ODRAIN | IFLUSH);
		else
			dc->dc_modifier = 0x0000;

		/*
		 * Issue the command
		 */
		wait_cmd_done(dc,tp,SIOSET,mdev);
		break;
		
	case TCGETA:
		/*
		 * Compute the address of the firmwares terminal control block
		 */
		tcp = (struct term_control *)( UL(tcp) + COAUL);

		/*
		 * Read the ?flags from the firmware duart channel structure
		 */
		tio.c_iflag = tp->t_iflag = (unsigned short)dc->dc_iflag;
		tio.c_oflag = tp->t_oflag = (unsigned short)dc->dc_oflag;
		tio.c_lflag = tp->t_lflag = (unsigned short)dc->dc_lflag;
		tio.c_cflag = tp->t_cflag = (unsigned short)dc->dc_cflag;
		tio.c_line = tp->t_line; 

		/*
		 * Read the control characters
		 */
		tio.c_cc[VINTR]  = tcp->tc_control_chars[SIO_VINTR];
		tio.c_cc[VQUIT]  = tcp->tc_control_chars[SIO_VQUIT];
		tio.c_cc[VERASE] = tcp->tc_control_chars[SIO_VERASE];
		tio.c_cc[VKILL]  = tcp->tc_control_chars[SIO_VKILL];
		tio.c_cc[VEOF]   = tcp->tc_control_chars[SIO_VEOF]; /* VMIN  */
		tio.c_cc[VEOL]   = tcp->tc_control_chars[SIO_VEOL]; /* VTIME */
		tio.c_cc[VSWTCH] = tcp->tc_control_chars[SIO_VSWTCH];
		
		/*
		 * Copy the settings to the local tty structure and the 
		 * user process
		 */
		bcopy( tio.c_cc, tp->t_cc, NCC );
		if ( copyout((caddr_t)&tio, arg, sizeof(struct termio)) ) {
			u.u_error = EFAULT;
		}
		break;

	case TCSBRK:
		s = SPLX(mdev/PORTS_PER_SCC);
		get_dc(dc,mdev);
		dc->dc_arg1 = arg;
		dc->dc_arg2 = 25;	/* BREAK for 25 1/100ths */
		dc->dc_modifier = ODRAIN;
		wait_cmd_done(dc, tp, SIOBRK,mdev);
		splx(s);
		break;
	case TCXONC:
		if ( arg != 0 && arg != 1 ) {
			u.u_error = EINVAL;
			break;
		}
		if (arg == 0) 
			dc->dc_halt = -1;
		else
			dc->dc_halt = 0;
		break;
	case TCFLSH:
		s = SPLX(mdev/PORTS_PER_SCC);
		if ( arg < 0 || arg > 2 ) {
			u.u_error = EINVAL;
			splx(s);
			break;
		}
		get_dc(dc,mdev);
		if (arg == 0)
			dc->dc_modifier = IFLUSH;
		else if (arg == 1)
			dc->dc_modifier = OFLUSH;
		else
			dc->dc_modifier = (OFLUSH | IFLUSH);

		wait_cmd_done(dc, tp,SIODUMMY,mdev);
		splx(s);
		break;
	case TIOCSETP:
		/* 
		 * This system call stty is nolonger supported 
		 */
		cmn_err(CE_WARN,"sccioctl: TIOCSETP call, not supported");
		u.u_error = EINVAL;
		break;
	case TIOCGETP:
		tb.sg_ispeed = (unsigned short)(dc->dc_cflag & CBAUD);
		tb.sg_ospeed = tb.sg_ispeed; 
		tb.sg_erase  = tcp->tc_control_chars[SIO_VERASE];
		tb.sg_kill   = tcp->tc_control_chars[SIO_VKILL];

		flag = 0;

		if (dc->dc_cflag&HUPCL)
			flag |= O_HUPCL;
		if (!(dc->dc_lflag&ICANON))
			flag |= O_RAW;
		if (dc->dc_lflag&XCASE)
			flag |= O_LCASE;
		if (dc->dc_lflag&ECHO)
			flag |= O_ECHO;
		if (!(dc->dc_lflag&ECHOK))
			flag |= O_NOAL;
		if (dc->dc_cflag&PARODD)
			flag |= O_ODDP;
		else if (dc->dc_iflag&INPCK)
			flag |= O_EVENP;
		else
			flag |= O_ODDP|O_EVENP;
		if (dc->dc_oflag&ONLCR) {
			flag |= O_CRMOD;
			if (dc->dc_oflag&CR1)
				flag |= O_CR1;
			if (dc->dc_oflag&CR2)
				flag |= O_CR2;
		} else {
			if (dc->dc_oflag&CR1)
				flag |= O_NL1;
			if (dc->dc_oflag&CR2)
				flag |= O_NL2;
		}
		if ((dc->dc_oflag&TABDLY)==TAB3)
			flag |= O_XTABS;
		else if (dc->dc_oflag&TAB1)
			flag |= O_TBDELAY;
		if (dc->dc_oflag&FFDLY)
			flag |= O_VTDELAY;
		if (dc->dc_oflag&BSDLY)
			flag |= O_BSDELAY;
		tb.sg_flags = flag;
		if (copyout(&tb, arg, sizeof(tb)))
			u.u_error = EFAULT;
		break;

	case SCCSETAW:
	case SCCSETAF:
	case SCCSETA:
		if (copyin(arg,(caddr_t)&tio,sizeof(struct termio))) {
			u.u_error = EFAULT;
			break;
		}

		/*
		 * Lock the dc_cmd sub-structure before we go any further
		 * as another command could be waiting to complete.
		 * This also stops two or more tasks going through this
		 * code sequence 'at once' (i.e 2nd one starting while 1st
		 * one is sleeping waiting for the firmware)
		 */
		get_dc(dc,mdev);

		/*
		 * In effect we are now in a critical section as only one
		 * user thread may be active beyond here at once.
		 * The tp tty structure is owned by a terminal line and may
		 * therefore be accessed by several processes. Changes to it
		 * must be 'atomic'. In days of old this was accomplised simply
		 * by not sleeping in the code sequence. However the driver
		 * sleeps waiting for the command to complete.
		 *
		 * Compute the address of the firmwares terminal control block
		 */
		tcp = (struct term_control *)(UL(tcp) + COAUL);

		/*
		 * Ensure tcp area same as dc area
		 */	
		tcp->tc_iflag = dc->dc_iflag;
		tcp->tc_oflag = dc->dc_oflag;
		tcp->tc_cflag = dc->dc_cflag;
		tcp->tc_lflag = dc->dc_lflag;

		/*
		 * Load the terminal control block. UNIX, normally, only
		 * accesses the bottom 16 bits of each flag, hence all this
		 * pointer work as we are putting in the top 16 bits this time.
		 */
		sptr = (unsigned short *)(UL(&tcp->tc_iflag)); 
		*sptr = tio.c_iflag;
		sptr = (unsigned short *)(UL(&tcp->tc_oflag)); 
		*sptr = tio.c_oflag;
		sptr = (unsigned short *)(UL(&tcp->tc_cflag)); 
		*sptr = tio.c_cflag;
		sptr = (unsigned short *)(UL(&tcp->tc_lflag)); 
		*sptr = tio.c_lflag;
	
		/*
		 * Set the command modifier bits
		 */
		if (cmd == SCCSETAW)
			dc->dc_modifier = ODRAIN;
		else if (cmd == SCCSETAF)
			dc->dc_modifier = (ODRAIN | IFLUSH);
		else
			dc->dc_modifier = 0x0000;

		/*
		 * Issue the command
		 */
		wait_cmd_done(dc,tp,SIOSET,mdev);
		break;
		
	case SCCGETA:
		/*
		 * Compute the address of the firmwares terminal control block
		 */
		tcp = (struct term_control *)( UL(tcp) + COAUL);

		/*
		 * Read the ?flags from the firmware duart channel structure
		 */
		tio.c_iflag = (unsigned short)(dc->dc_iflag >> 16);
		tio.c_oflag = (unsigned short)(dc->dc_oflag >> 16);
		tio.c_lflag = (unsigned short)(dc->dc_lflag >> 16);
		tio.c_cflag = (unsigned short)(dc->dc_cflag >> 16);

		/*
		 * Copy the settings of the local tty structure to the 
		 * user process
		 */
		if ( copyout((caddr_t)&tio, arg, sizeof(struct termio)) ) {
			u.u_error = EFAULT;
		}
		break;
	
	case SCCSWRAP:
		if (copyin(arg,(caddr_t)&line,sizeof(int))) {
			u.u_error = EFAULT;
			break;
		}
		dc->dc_width = (unsigned short)line;
		break;

	case SCCGWRAP:
		line = (int)dc->dc_width;
		if (copyout((caddr_t)&line,arg,sizeof(int))) {
			u.u_error = EFAULT;
			break;
		}
		break;

	case SCCBRGT:
		line = (int)dc->dc_brgss;
		if (copyout((caddr_t)&line,arg,sizeof(int))) {
			u.u_error = EFAULT;
			break;
		}
		break;

	case SCCIETR:
		line = (int)(UL(dc->dc_echotbl) + COAUL);
		if ( (line < COAUL) || (line > (COAUL+0xFFE00)) ) {
			u.u_error = EFAULT;
			break;
		}
		

		if (copyout((caddr_t)line,arg,512)) {
			u.u_error = EFAULT;
			break;
		}
		break;

	case SCCIETW:
		line = (int)(UL(dc->dc_echotbl) + COAUL);

		if ( (line < COAUL) || (line > (COAUL+0xFFE00)) ) {
			u.u_error = EFAULT;
			break;
		}
		
		if (copyin(arg,(caddr_t)line,512)) {
			u.u_error = EFAULT;
			break;
		}
		break;

	case SCCPERR:
		line = (int)dc->dc_proc_error;
		if (copyout((caddr_t)&line,arg,sizeof(int))) {
			u.u_error = EFAULT;
			break;
		}
		break;

	case SCCPCLR:
		dc->dc_proc_error = (unsigned short)0;
		break;

	default:
		if ( (cmd & IOCTYPE) == LDIOC ) 
			ldioctl(dc, tp, cmd, arg, mode,mdev);
		else {
			cmn_err(CE_CONT,
				"!sccioctl: Bad cmd 0x%x, channel %d\n",
								cmd,mdev);
			u.u_error = EINVAL;
		}
		break;
	
	} /* end of switch */

}	/* end of sccioctl */

/*
 *
 * sccread(dev)
 *    unsigned int dev;
 */
sccread(dev)
unsigned int dev;
{
	register struct tty *tp;
	register int mdev;

	/*
	 * Convert to a minor device, get the devices tty structure and
	 * call the input routine.
	 */

	mdev = minor(dev);
	tp =  &scc_tty[mdev];
	(*linesw[UC(tp->t_line)].l_read)(tp);
}

/* STTREAD : linesw table read routine for scc driver */
/* 	     This function is called via the linesw table from elsewhere
 *	     in the kernel. The tty address passed to it may belong to 
 * 	     either a real or a virtual terminal.
 */
sttread(tp)
register struct tty * tp;
{
	dev_t dev;
	register struct duart_channel * dc;
	register struct fifo * ffinp;
	register struct fifo **ptr_to_fifo_ptr;
	register short c, n;
	register long ch, prev_ch;
	register s;
	int cindex, call_fw;


	/* If tp not within real terminal address range, then assume
         * we have a virtual terminal and take appropriate action. 
	 * If we have a virtual terminal the index into the linesw table
         * should be the same as for real terminals when shl (shell layers)
	 * is not active.
         */
	if ( ((tp < scc_tty) || (tp > &scc_tty[scc_cnt - 1])) && 
	     					(tp->t_line == scc_line) )
	{
		/* virtual terminal or invalid tty address(?) */
		vttread(tp);
		return;
	}
	/* else ordinary real terminal : at this point tp->t_line could be
         * equal to either sxt_line or scc_line depending on whether or not
	 * shell layers is active.
	 */

	dev = ((int)tp - (int)scc_tty) / sizeof(struct tty);
	if ( (cindex = valid_mdev(dev)) == -1 ) return;

	dc = scc_duart[dev];
	ptr_to_fifo_ptr = (struct fifo **)(&dc->dc_hostinf);

	/* if canonical mode, then sleep if input queue empty */
	if ( tp->t_lflag & ICANON )
	{
		while (FFEMPTY((struct fifo *)(UL(*ptr_to_fifo_ptr)+ COAUL))) {
			s = SPLX(cindex);
			dc->dc_flags[HOSTIPWAIT] = TRUE;
			tp->t_state |= IASLP;
			sleep( (caddr_t)&tp->t_rawq, TTIPRI );
			splx(s);
		}
	}
	/* if raw mode, then sleep if input queue empty and not time out */
	else {

		if (u.u_fmode & FNDELAY)
			goto get_the_data;

		/*
		 * Am in raw mode. Will we need to wait on the firmware ?
		 *
		 * if (VTIME == 0) {
		 *	if (VMIN > fifo_enteries) && 
		 *		(u.u_count > fifo_enteries) 'call f/w'
		 * else {
		 *	if (VMIN == 0)
		 *		if (u.u_count > fifo_enteries) 'call f/w'
		 *	else if (VMIN > fifo_enteries) &&
		 *			(u.u_count > fifo_enteries) 'call f/w'
		 * }
		 */
#define	FENT	fifo_entries((struct fifo *)(UL(*ptr_to_fifo_ptr)+COAUL))

		s = FENT;
		call_fw = FALSE;

		if (!tp->t_cc[VTIME]) {
			if ( (tp->t_cc[VMIN] > s) && (u.u_count > s) ) {
				call_fw = TRUE;
			}
		}
		else {
			if (!tp->t_cc[VMIN]) {
				if (u.u_count > s) {
					call_fw = TRUE; 
				}
			}
			else if ( (tp->t_cc[VMIN] > s) && (u.u_count > s) ) {
				call_fw = TRUE;
			}
		}

		if (call_fw) {
			do { 
				s = SPLX(cindex);
				dc->dc_flags[HOSTIPWAIT] = TRUE;
				dc->dc_flags[26] = FALSE;
				tp->t_state |= IASLP;
				while (tp->t_state & IASLP)
					sleep( (caddr_t)&tp->t_rawq, TTIPRI );
				splx(s);
			} while ( (tp->t_cc[VTIME] <= 1)
					&& (tp->t_cc[VMIN] > FENT) );
		}
	}
		
	/* The data in the input queue is processed by the firmware */
	/* Copy the processed data into user data space */

get_the_data:			/* Label used if FNDELAY set */

	ffinp = (struct fifo *)(UL(*ptr_to_fifo_ptr)+COAUL);
	s = splx(scc_ilev[cindex]<<8);
	n = MIN( u.u_count, fifo_entries(ffinp) );
	sysinfo.rawch += n;	/* system accounting info. */

	prev_ch = 0;

	if ( tp->t_lflag & ICANON )
	{
		sysinfo.canch += n;	/* system accounting info. */
		while ( --n >= 0 ) 	/* while something left to process
					 * in this batch */
		{
   			if ( (ch = bfifoget(ffinp)) == (long)tp->t_cc[VEOF] ) {
				/*
				 * Have an EOF character. Was previous 
				 * character an escape chareacter '\' ? If it
				 * wasn't then terminate the input loop.
				 */
				if (prev_ch != (long)'\\') break;
			}

			/*
			 * Check for prev_ch being an escape. If it is
			 * then it can now be passed on to the user.
			 * The case of "\ctrl-D" is caught to stop the escape
			 * character being passed on to the user.
			 */
			if ( (prev_ch == (long)'\\') &&
			     (ch != (long)tp->t_cc[VEOF]) ) {

				if (subyte(u.u_base++,prev_ch)) 
				{
					u.u_error = EFAULT;
					splx(s);
					return;
				}
				else
					u.u_count--;
			}

			if (ch == -1) break;

			if (ch == (long)'\\') {
				prev_ch = ch;
				continue;
			}

			if (subyte(u.u_base++,prev_ch = ch ) ) 
			{
				u.u_error = EFAULT;
				splx(s);
				return;
			}
			else
				u.u_count--;
#ifdef RT
			letswtch(u.u_procp);
#endif
		}
	}
	else /* raw mode */
	{
		while ( --n >= 0 ) 
		{
			if ( subyte( u.u_base++, bfifoget(ffinp) ) )
			{	
				u.u_error = EFAULT;
				splx(s);
				return;
			}
			else
				u.u_count--;
#ifdef RT
			letswtch(u.u_procp);
#endif
		}
	}
	splx(s);

}

/* VTTREAD : read routine for virtual terminal, called when shell 
	     layers is active.
 */
vttread(tp)
register struct tty * tp;
{
	register struct clist * tq;


	tq = &tp->t_canq;

	/* if canonical queue is empty, call vcanon to copy rawq into canq */
	if ( tq->c_cc == 0 )
		vcanon(tp);

	/* while there are bytes remaining for i/o and no error... */
	while ( u.u_count != 0 && u.u_error == 0 )
	{	
		/* if bytes remaining for i/o >= max. bytes held in one 
 		 * cblock, then copy in blocks ( of size n ).... */
		if ( u.u_count >= CLSIZE )
		{
			register n;
			register struct cblock * cp;
	
			/* get a pointer to a cblock containing data */
			if ( ( cp = getcb(tq) ) == (struct cblock * )NULL )
				break;	/* no data to copy ! */
			n = min( u.u_count, cp->c_last - cp->c_first );
			/* copy n bytes from system space to user segment */
			if ( copyout( &cp->c_data[cp->c_first], u.u_base, n ) )
				u.u_error = EFAULT;
			/* put cblock on freelist, finished with this cblock */
			putcf(cp);
			u.u_base += n;
			u.u_count -= n;
		}
		else /* ....copy bytes one by one */
		{
			register c;

			/* get one character */
			if ( ( c = getc(tq) ) < 0 )
				break;	/* no (more) data */
			/* store byte in user data space */
			if ( subyte(u.u_base++, c) )
				u.u_error = EFAULT;
			u.u_count--;
#ifdef RT
			letswtch(u.u_procp);
#endif
		}
	}

	/* if state is blocked and the no. of chars in raw q < TTXOLO,
     	 * call proc routine ( sxtvtproc ) */
	if ( tp->t_state & TBLOCK )
		if ( tp->t_rawq.c_cc < TTXOLO )
			(*tp->t_proc)(tp, T_UNBLOCK);

}

/* VCANON : copy raw q to canon q, called with a virtual tty address */
vcanon(tp)
register struct tty * tp;
{
	register s;


	s = spltty();
	/* sleep if raw q is empty and timeout set OR not enough chars in raw q
         * to satisfy VMIN */
	if ( ( !tp->t_cc[VMIN] && tp->t_cc[VTIME] && (tp->t_rawq.c_cc == 0) ) ||
	     ( tp->t_cc[VMIN] && (tp->t_cc[VMIN] > tp->t_rawq.c_cc) ) )
	{
		do
		{
			tp->t_state |= IASLP;
			/* wakeup via interupt (interupt routine calls sxtin,
  			 * which calls sttin) */
			sleep( (caddr_t)&tp->t_rawq, TTIPRI );
		}
		while  ( tp->t_cc[VMIN] && (tp->t_rawq.c_cc == 0) );
	}
	tp->t_canq = tp->t_rawq;
	tp->t_rawq = ttnulq;
	splx(s);

}
			
/*
 *
 * sccwrite(dev)
 *    unsigned int dev;
 */
sccwrite(dev)
unsigned int dev;
{
	register struct tty *tp;
	

	tp =  &scc_tty[minor(dev)];
	(*linesw[UC(tp->t_line)].l_write)(tp);

}

/* STTWRITE : linesw table write routine for scc driver */
/* 	      This function is called via the linesw table from elsewhere
 *	      in the kernel. The tty address passed to it may belong to 
 * 	      either a real or a virtual terminal.
 */
sttwrite(tp)
register struct tty *tp;
{
	dev_t dev;
	register struct duart_channel * dc;
	register struct fifo * ffoutp;
	register s;
	register short c, n;
	int cindex;


	/* If tp not within real terminal address range, then assume
         * we have a virtual terminal and take appropriate action.
	 * If we have a virtual terminal the index into the linesw table
         * should be the same as for real terminals when shl (shell layers)
	 * is not active.
         */
	if ( ((tp < scc_tty) || (tp > &scc_tty[scc_cnt - 1])) && 
	     					(tp->t_line == scc_line) )
	{
		/* virtual terminal or invalid tty address(?) */
		vttwrite(tp);
		return;
	}
	/* else ordinary real terminal : at this point tp->t_line could be
         * equal to either sxt_line or scc_line depending on whether or not
	 * shell layers is active.
	 */
		dev = ((int)tp - (int)scc_tty) / sizeof(struct tty);
		if ( (cindex = valid_mdev(dev)) == -1 ) return;
	

	dc = scc_duart[dev];
	ffoutp = (struct fifo *)(UL(dc->dc_hostoutf) + COAUL);

	while ( u.u_count ) /* while there are bytes remaining for output... */
	{
		/* sleep while above high water mark, i.e. f/w has all
                 * it can cope with for the moment, so sleep until the
		 * the f/w is ready to process more data */
		while ( fifo_entries(ffoutp) > FFSIZE(ffoutp) - WATER )
		{
			s = splx(scc_ilev[cindex]<<8);
			dc->dc_flags[HOSTOPWAIT] = TRUE;
			tp->t_state |= OASLP;
			sleep( (caddr_t)&tp->t_outq, TTOPRI );
			splx(s);
		}

		/* Fetch data from user data space */
		s = splx(scc_ilev[cindex]<<8);
		/* May not have enough room in fifo for all bytes
   		 * that want to be processed (i.e. u.u_count).
   		 * Maximum bytes that can be dealt with is the 
  		 * number of unused entries in fifo
		 */

		n  = MIN( u.u_count, fifo_room(ffoutp) );
		sysinfo.outch += n;	/* system accounting info. */

		while ( --n >= 0 )
		{
			if ( ( c = fubyte( u.u_base++ ) ) >= 0 ) {
				if (bfifoput(ffoutp, c) == -1)
					printf("sttwrite: bfifoput failure\n");
				u.u_count--;
			}
			else
			{
				u.u_error = EFAULT;
				splx(s);
				return;
			}
		}
		splx(s);
	}
}

/* VTTWRITE : write routine for virtual terminal, called when shell 
	     layers is active.
 */
vttwrite(tp)
register struct tty * tp;
{
	register s;


	/* 
	 * If no physical connection to the device then return
	 */
	if ( !(tp->t_state & CARR_ON) ) {
		return;
	}

	/* 
	 * While there are bytes remaining for I/O ...
	 */

	while ( u.u_count )
	{
		/* if the number of chars in the output q exceeds the high
 		 * water mark, (i.e output q too full to receive more chars
		 * at the moment) then... */
		if ( tp->t_outq.c_cc > tthiwat[tp->t_cflag & CBAUD] )
		{
			s = spltty();
			/* call sxtvtproc with T_OUTPUT and then test the
 			 * condition again, sleep if the condition is still
			 * not true until enough space becomes available
			 * in the output q */
			/* note : sxtvtproc calls sccproc with T_OUTPUT to
			 * process the data currently in the output q and 
			 * therefore free some space! */
			(*tp->t_proc)(tp, T_OUTPUT);
			while ( tp->t_outq.c_cc > tthiwat[tp->t_cflag & CBAUD] )
			{
				tp->t_state |= OASLP;
				if ( sleep((caddr_t)&tp->t_outq, (TTOPRI|PCATCH)))
				{
					splx(s);
					goto out;
				}
			}
			splx(s);
		}
		/* if the number of bytes remaining for io demands it, then
		 * copy the bytes from the user segment to system space in
		 * chunks of n and then put those n bytes on the output q */
		if ( u.u_count >= CLSIZE/2 )
		{
			register n;
			register struct cblock * cp;

			if ( (cp = getcf()) == ( struct cblock * )NULL) break;
			n = min( u.u_count, cp->c_last );
			if ( copyin(u.u_base, cp->c_data, n) )
			{
				u.u_error = EFAULT;
				putcf(cp);
				break;
			}
			u.u_base += n;
			u.u_count -= n;
			cp->c_last = n;
			ttxput(tp, cp, n);
		}
		/* ... else copy the bytes one by one and put then on the output
		 * q one by one also */
		else
		{
			register c;
		
			c = fubyte(u.u_base++);
			if ( c < 0 )
			{
				u.u_error = EFAULT;
				break;
			}
			u.u_count--;
			ttxput(tp, c, 0);
		}
	}
out:
	s = spltty();
	(*tp->t_proc)(tp, T_OUTPUT);
	splx(s);
}

/* SCCINTR : Interupt routine */
/* If the index into the linesw table for tp is the same as 
 * the index for the sxt driver routines then shell layers
 * must be active and so appropriate action must be taken.
 */
sccintr(dev)
dev_t dev;
{
	register struct duart_channel * dc;
	register struct tty * tp;
	register struct fifo * ffinp;
	register struct term_control * tcp;
	register s, sig;
	register char ch;
	register n, count, cindex;
	int index;

	if ( (cindex = valid_mdev(dev)) == -1 ) {
	
		/*
		 * Have a 'bad' interrupt - log the fault
		 */	
		dri_printf("***** sccintr: Bad dev number 0x%x *****\n",
						dev);
		return;
	}

	*(unsigned long *)(COAUL + 0x2F44) = 0x1;

	index = (int)dev;
	tp =  &scc_tty[index];
	dc = scc_duart[index];

	for ( n = 0; n < PORTS_PER_SCC; n++, tp =  &scc_tty[index],
					 dc = scc_duart[index] )
	{
		if (scc_ctrl[index++].scc_chan_present != TRUE) /* channel not present */
			continue; 
		if (tp <  scc_tty || tp >  &scc_tty[scc_cnt - 1])
			continue;
		if ( tp->t_line == sxt_line )	/* virtual tty */
		{
			if ( sig = dc->dc_signal )
			{
				/* could be trouble here if f/w control chars
				 * are not the same as the s/w control chars */
				tcp = dc->dc_tcp;
				tcp = (struct term_control *)( UL(tcp) + COAUL);
				dc->dc_signal = 0;

				if (sig == 0x7f)	sig = SIGUSR1;

				switch(sig)
				{
				case SIGINT:
					ch = tcp->tc_control_chars[SIO_VINTR];
					break;
				case SIGQUIT:
					ch = tcp->tc_control_chars[SIO_VQUIT];
					break;
				case SIGUSR1:
					ch = tcp->tc_control_chars[SIO_VSWTCH];
					break;
				default:
					return;
				}
				/* put signal into rbuf */
				if ( tp->t_rbuf.c_ptr )
				{
					*tp->t_rbuf.c_ptr++ = ch;
					--tp->t_rbuf.c_count;
					tp->t_rbuf.c_ptr -= tp->t_rbuf.c_size -
						tp->t_rbuf.c_count;
					/* call sxt driver proc routine */
					(*linesw[UC(tp->t_line)].l_input)(tp, L_BUF);
				}
			}
			/* else copy chars from input q into rbuf */
			else
			{
				if ( tp->t_rbuf.c_ptr )
				{
					ffinp = (struct fifo *)(UL(dc->dc_hostinf) + COAUL);
					count = fifo_entries(ffinp);
					for ( ; count; ) 
					{
						for ( ; count; )
						{
							*(tp->t_rbuf.c_ptr++) = bfifoget(ffinp);
							count--;
							if (--tp->t_rbuf.c_count == 0 )
								break;
						}
						tp->t_rbuf.c_ptr -= tp->t_rbuf.c_size - tp->t_rbuf.c_count;
						(*linesw[UC(tp->t_line)].l_input)(tp, L_BUF);
					}
				}
			}
			if ( !tp->t_state & (OASLP|TTIOW) ) continue;
                }
		else	/* real tty */
		{ 

			if (dc->dc_flags[FWREADINTR]) {
				dc->dc_flags[HOSTIPWAIT] = 0;
				dc->dc_flags[FWREADINTR] = 0;
				if (tp->t_state & IASLP) {
					tp->t_state &= ~IASLP;
					wakeup( (caddr_t)&tp->t_rawq);
				}
				else {
				}
			}

			if (dc->dc_flags[FWSIGNAL]) {
			 	sig = dc->dc_signal;
				/*
				 * sig is either 2 or 3 ?
				 */
				if ( (sig != 2) && (sig != 3) )
					printf("sccintr: Bad signal\n");
				else
					signal(tp->t_pgrp, sig);
				dc->dc_signal = 0;
				dc->dc_flags[FWSIGNAL] = 0;
			}

		}

		/* common action for both real and virtual tty's */

		if (dc->dc_flags[HOSTLOSTDCD]) {
			dc->dc_flags[HOSTLOSTDCD] = 0;
			signal(tp->t_pgrp, SIGHUP);
		}
		
		if (dc->dc_flags[HOSTINZAP]) {
			zapfifo( (struct fifo *)(UL(dc->dc_hostinf)+COAUL) );
			dc->dc_flags[HOSTINZAP] = 0;
		}
	
		if (dc->dc_flags[HOSTOFLUSH]) {
			zapfifo( (struct fifo *)(UL(dc->dc_hostoutf)+COAUL) );
			dc->dc_flags[HOSTOFLUSH] = 0;
		}

		
		if (dc->dc_flags[FWCMDDONE]) {

			if (dc->dc_cmd)
				printf("sccintr: dc_cmd NOT 0\n");

			dc->dc_flags[FWCMDDONE] = 0;
			dc->dc_flags[HOSTCMDWAIT] = 0;
			if (tp->t_state & TTIOW) {
				tp->t_state &= ~TTIOW;
				wakeup((caddr_t)&dc->dc_cmd);
			}
			else {
			}
		}

		if (dc->dc_flags[FWWRITEINTR]) {
			dc->dc_flags[HOSTOPWAIT] = 0;
			dc->dc_flags[FWWRITEINTR] = 0;
			if (tp->t_state & OASLP) {
				tp->t_state &= ~OASLP;
				wakeup( (caddr_t)&tp->t_outq);
			}
			else {
			}
		}
	}
	/* clear interupt */

	*(unsigned long *)(COAUL + 0x2F44) = 0x0;
}

/* STTIN : input routine for virtual terminals, called from sxtin. 
           1. copy data from rbuf to raw q,
  	   2. block tty if conditions require it,
  	   3. flush input queue if no. chars in rawq > HOG value,
           4. check on high water mark and unset flags if necc.,
  	   5. process signals if any to process,
  	   6. wakeup raw q (sent to sleep in vcanon).
 */
sttin(tp, code)
register struct tty * tp;
{
	register c;
	register flg;
	register char * cp;
	ushort nchar, nc;


	nchar = tp->t_rbuf.c_size - tp->t_rbuf.c_count;
	/* reinit tx control block */
	tp->t_rbuf.c_count = tp->t_rbuf.c_size;
	if ( nchar == 0 ) /* no chars to process */
		return;	
	flg = tp->t_iflag;
	nc = nchar;
	cp = tp->t_rbuf.c_ptr;
	/* if no. of chars in rbuf is < the space left in the freelist OR
         * the input mode is set as below, then process the chars one 
   	 * by one.... */
	if ( nc < cfreelist.c_size || (flg & (INLCR|IGNCR|ICRNL|IUCLC) ) )
	{
		for ( ; nc-- ; cp++ )
		{
			c = *cp;
			if ( putc( c, &tp->t_rawq ) ) continue;
			sysinfo.rawch++;
		}
		cp = tp->t_rbuf.c_ptr;
	}
	else /* ...may process chars block by block */
	{
		putcb( CMATCH( (struct cblock *) cp), &tp->t_rawq );
		sysinfo.rawch += nc;
		/* allocate new rx buffer */
		if ( (tp->t_rbuf.c_ptr = getcf()->c_data) == 
			((struct cblock *) NULL)->c_data )
		{
			tp->t_rbuf.c_ptr = NULL;
			return;
		}
		tp->t_rbuf.c_count = cfreelist.c_size;
		tp->t_rbuf.c_size = cfreelist.c_size;
	}

	if ( tp->t_rawq.c_cc > TTXOHI )
	{
		if ( flg & IXOFF && !( tp->t_state & TBLOCK ) )
			(*tp->t_proc)(tp, T_BLOCK);
		if ( tp->t_rawq.c_cc > TTYHOG )
		{
			vttflsh(tp, FREAD);
			return;
		}
	}

	flg = lobyte(tp->t_lflag);
	if ( tp->t_outq.c_cc > ( tthiwat[tp->t_cflag&CBAUD] + TTECHI ) )
		flg &= ~(ECHO|ECHOK|ECHONL|ECHOE);

	/* signal processing */
	if (flg)
	{
		while ( nchar-- )
		{
			c = *cp++;
			if ( flg & ISIG )
			{
				if ( c == tp->t_cc[VINTR] )
				{
					signal(tp->t_pgrp, SIGINT);
					if ( !( flg & NOFLSH) )
						vttflsh( tp, (FREAD|FWRITE) );
					continue;
				}
				if ( c == tp->t_cc[VQUIT] )
				{
					signal(tp->t_pgrp, SIGQUIT);
					if ( !( flg & NOFLSH) )
						vttflsh( tp, (FREAD|FWRITE) );
					continue;
				}
				if ( c == tp->t_cc[VSWTCH] )
				{
					if ( !( flg & NOFLSH) )
						vttflsh( tp, FREAD );
					/* sxtvtproc */
					(*tp->t_proc)( tp, T_SWTCH );
					continue;
				}
			}
		}

	}
					
	/* wakeup anything sent to sleep in vcanon */
	if ( tp->t_state & IASLP )
	{
		tp->t_state &= ~IASLP;
		wakeup( (caddr_t) &tp->t_rawq );
	}

}

/* STTOUT : Output routine for virtual terminals, called from sxtout.
	    1) Wakeup anything sleeping on &tp->t_oflag if state set 
	       to waiting on i/o and output q empty.
	    2) Copy data from tbuf to output q.
	    3) If state set to sleeping on output and number of chars in
	       output q is not greater than the low water mark, then
	       wakeup output q.
 */
sttout(tp)
register struct tty * tp;
{
	register struct ccblock * tbuf;
	register c;
	register char * cptr;
	register retval;


	if ( tp->t_state & TTIOW && tp->t_outq.c_cc == 0 ) {
		tp->t_state &= ~TTIOW;
		printf("sttout: wakeup t_oflag\n");
		wakeup((caddr_t)&tp->t_oflag);
	}

	tbuf = &tp->t_tbuf;
	retval = 0;
	if ( ( (tp->t_state&EXTPROC) || (!(tp->t_oflag&OPOST)) ) && 
		tp->t_term == 0)
	{
		if (tbuf->c_ptr)
			putcf(CMATCH((struct cblock *)tbuf->c_ptr));
		if ((tbuf->c_ptr = (char *)getcb(&tp->t_outq)) == NULL)
			return(0);
		tbuf->c_count = ((struct cblock *)tbuf->c_ptr)->c_last -
				((struct cblock *)tbuf->c_ptr)->c_first;
		tbuf->c_size = tbuf->c_count;
		tbuf->c_ptr = &((struct cblock *)tbuf->c_ptr)->c_data
				[((struct cblock *)tbuf->c_ptr)->c_first];
		retval = CPRES;
	} 
	else 
	{	
		if (tbuf->c_ptr == NULL) {
			if ((tbuf->c_ptr = getcf()->c_data)
				== ((struct cblock *)NULL)->c_data) {
				tbuf->c_ptr = NULL;
				return(0);
			}
		}
		tbuf->c_count = 0;
		cptr = tbuf->c_ptr;
		while ((c=getc(&tp->t_outq)) >= 0) {
			retval = CPRES;
			*cptr++ = c;
			tbuf->c_count++;
			if (tbuf->c_count >= cfreelist.c_size)
				break;
		}
		tbuf->c_size = tbuf->c_count;
	}

	if ( tp->t_state & OASLP && 
	     tp->t_outq.c_cc <= ttlowat[tp->t_cflag & CBAUD] ) 
	{
		tp->t_state &= ~OASLP;
		wakeup((caddr_t)&tp->t_outq);
	}

	return(retval);
}

/* VTTFLSH : flush i/o queues for virtual terminal, called from sttin. 
  	     Same as ttyflush except for setting t_delct = 0. 
  	     Both ttyflush and vttflsh result in flushing of the real
             tty input and/or output queues.
 */
vttflsh(tp, cmd)
register struct tty * tp;
{
	register struct cblock * cp ;
	register s;


	if ( cmd & FWRITE )
	{
		/* get all cblocks associated with out q and put
                 * them back on the free list */
		while ( (cp = getcb(&tp->t_outq)) != NULL )
			putcf(cp);
		(*tp->t_proc)(tp, T_WFLUSH);
		if ( tp->t_state & OASLP )
		{
			tp->t_state &= ~OASLP;
			wakeup( (caddr_t)&tp->t_outq );
		}
		if ( tp->t_state & TTIOW ) 
		{
			tp->t_state &= ~TTIOW;
			printf("vttflsh: wakeup t_oflag\n");
			wakeup( (caddr_t)&tp->t_oflag );
		}
	}
	if ( cmd & FREAD )
	{
		/* get all cblocks associated with canonical q and put
                 * them back on the free list */
		while ( (cp = getcb(&tp->t_canq)) != NULL )
			putcf(cp);
		s = spltty();
		/* get all cblocks associated with raw q and put
                 * them back on the free list */
		while ( (cp = getcb(&tp->t_rawq)) != NULL )
			putcf(cp);
		splx(s);
		(*tp->t_proc)( tp, T_RFLUSH );
		if ( tp->t_state & IASLP )
		{
			tp->t_state &= ~IASLP;
			wakeup( (caddr_t)&tp->t_rawq );
		}
	}

}

extern sxtin();

/* 
 *
 * sccinit
 *
 * This routine is called at power-up, or reboot, to determine if a Serial
 * Communication Card (SCC) is present.
 *  
 * scc_cnt MUST be a multiple of PORTS_PER_SCC to be valid. A warning is output
 * if it is not and it is scaled down to the next multiple if possible.
 * scc_cnt CAN'T exceed scc_tty_max.
 */

sccinit()
{
	register unsigned long *id_code;
	register unsigned long *type_code;
	register unsigned char *addr, *end;
	register cnt, i;

	/*
	 * One thing that this code insists on is:
	 *
	 * scc_tty_max == PORTS_PER_SCC * scc_card_max
	 */
	if (scc_tty_max != (PORTS_PER_SCC * scc_card_max)) {

		/*
		 * They are not equal - this can only happen after a 
		 * recompilation of the file scc_conf.c.
		 * Give warning but allow boot to continue, but with NO
		 * serial I/O cards configured !
		 */ 
		cmn_err(CE_WARN,
		   "scc(sccinit): scc_tty_max != PORTS_PER_SCC * scc_card_max");
		cmn_err(CE_WARN,
		   "scc(sccinit): scc_cnt(%d) forced to ZERO", scc_cnt);
		scc_cnt = 0;

	} /* end of if (scc_tty_max != (PORTS_PER_SCC * scc_card_max)) */
	

	if (scc_cnt != 0) {

		/*
		 * Check scc_cnt is a multiple of PORTS_PER_SCC and not 
		 * greater than scc_tty_max.
		 */
		if ((scc_cnt % PORTS_PER_SCC) != 0) {
			/*
			 * scc_cnt NOT a multiple of PORTS_PER_SCC. 
			 * Scale down and output a warning
			 */
			scc_cnt = ((i = scc_cnt) / PORTS_PER_SCC) * PORTS_PER_SCC;
			cmn_err(CE_WARN,
				"scc(sccinit): scc_cnt set to %d from %d",
				scc_cnt,i);
		}

		if (scc_cnt > scc_tty_max) {
			/*
			 * To many SCC's for this driver build. Scale down
			 */
			cmn_err(CE_WARN,
			"scc(sccinit): scc_cnt(%d) > scc_tty_max,(%d) set to %d",
				scc_cnt, scc_tty_max, scc_tty_max);
			scc_cnt = scc_tty_max;
		}

	} /* end of if (scc_cnt != 0) */

	/*
	 * Now the scc_duart[] structure to NULL (e.g. no channel) and no
	 * channels present
	 */
	for ( i = 0; i < scc_tty_max; i++ ) {
		scc_duart[i] = (struct duart_channel *)NULL;
		scc_ctrl[i].scc_chan_present = FALSE;
		scc_ctrl[i].scc_cmd_wait     = FALSE;
	}

	/*
	 * Now go a hunting for SCC's .... (only if scc_cnt != 0)
	 */

	if (scc_cnt == 0) {
		cmn_err(CE_WARN,"scc(sccinit): Aborted, scc_cnt is zero");
		return;
	}

	cnt = 0;

	for (i = 0; i < (scc_cnt/PORTS_PER_SCC); i++) {

		/*
		 * bprobe at address scc_addr[i] to see if any memory at the 
		 * start address boundary
		 */
		if ( bprobe(UCP(scc_addr[i]), READ_BYTE) ) {
			/*
			 * Mark as no card present. Set address to -1
			 */
			scc_addr[i] = -1;
			continue;
		}

		/*
		 * Have some memory at address scc_addr[i], now try for 
		 * a 'magic' word at address offset 0x404 from base 
		 * address scc_addr[i]
		 */
		if ( bprobe(UCP(scc_addr[i]+ID_CODE), READ_BYTE) ) {
			/*
			 * Mark as no card present. Set address to -1
			 */
			scc_addr[i] = -1;
			continue;
		}

		/*
		 * Going to get data from this address to see if it is the
		 * 'magic' value ....
		 */
		id_code = (unsigned long *)(scc_addr[i] + ID_CODE);
		if (*id_code != 0x42456546) {
			/*
			 * Mark as no card present. Set address to -1
			 */
			scc_addr[i] = -1;
			continue;
		}

		/*
		 * Have identified the card. Is it an SCC.
		 */
		
		type_code = (unsigned long *)(scc_addr[i] + TYPE_CODE);
		if (*type_code != 0x53494f50) {
			/*
			 * Mark as no card present. Set address to -1
			 */
			scc_addr[i] = -1;
			continue;
		}

		/*
		 * Have an SCC card
		 */
		cnt++;
		if (scc_msg >= 2)
			dri_printf("scc(sccinit): SCC card at 0x%x\n", scc_addr[i]);

	}	/* end of for loop */

	if (cnt == 0) {
		cmn_err(CE_WARN,"scc(sccinit): No SCC cards found");
		return;
	}
	else {
		if (scc_msg >= 1)
			dri_printf("scc(sccinit): %d SCC card(s) found\n", cnt);
	}

	
	/*
	 * Now do some driver checking ...
	 * 
	 * Firstly find index into line switch table for the
	 * sxt (shell layers) driver
	 * sxtin procedure MUST always exist, even if not in linesw table
	 */
	for (sxt_line = 0; sxt_line < linecnt; sxt_line++) {
		if (linesw[sxt_line].l_input == sxtin)
			break;
	}

	if (sxt_line == linecnt) {
		sxt_line = NO_SXTIN;
		if (scc_msg >= 3)
			cmn_err(CE_WARN,
	"scc(sccinit): No entry for sxt driver in linesw table");
		else
			cmn_err(CE_CONT,
	"!scc(sccinit): No entry for sxt driver in linesw table");
	}

	/*
	 * Now find index for this drivers 'clist' input routine !
	 */
	for (scc_line = 0; scc_line < linecnt; scc_line++) {
		if (linesw[scc_line].l_input == sttin)
			break;
	}
	
	if (scc_line == linecnt) {
		cmn_err(CE_WARN,
			"scc(sccinit): No entry for sttin in linesw table");
		scc_line = NO_STTIN;
	}

}	/* end of sccinit() */



ldioctl(dc, tp, cmd, arg, mode, mdev)
register struct duart_channel * dc;
register struct tty * tp;
{
	register s;

	switch(cmd)
	{
	case LDOPEN:
	case LDCHG:
		s = splx(scc_ilev[mdev/16]<<8);
		get_dc(dc,mdev);
		dc->dc_arg1 = UNIXLD0;
		wait_cmd_done(dc, tp,SIOOPEN,mdev);
		splx(s);
		break;
	case LDCLOSE:
		s = splx(scc_ilev[mdev/16]<<8);
		get_dc(dc,mdev);
		wait_cmd_done(dc, tp, SIOCLOSE,mdev);
		splx(s);
		break;
	case LDGETT:
		break;
	case LDSETT:
		break;
	default:
		break;
	}

}

