 #
 #                  Copyright (c) 1988 Ferranti Plc.
 #                      All Rights Reserved
 #
 #
 # THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF FERRANTI PLC THE COPYRIGHT 
 #                   OF WHICH VESTS IN FERRANTI PLC.
 #
 #   The copyright notice above does not evidence any actual or intended 
 #                   publication of such source code.
 #
 # This source code and the information contained herein is supplied without
 # liability for errors or ommissions and no part may be copied, re-produced, 
 # used or disclosed except as authorised by the Ferranti organisation by 
 #                contract or by other written permission.
 #

 #*****************************************************************************
 #
 #	Rev = 5.1		Date of 'get'   = 11:11:03 1/26/89
 #				Date of 'delta' = 15:28:11 12/14/88
 #
 #	File = sccfifo.as
 #
 #*****************************************************************************


 # WARNING: To keep /lib/cpp happy no comment must have a # in column 1.
 #          Any 'comment' starting with a # in that position will be treated
 #          as a directive to /lib/cpp ! You have been warned.
 #
 # The define TASFRIG should be set if the code is to be used with a PMMU that
 # has the RMW fault - the ATC must be pre-loaded
 

	file "sccfifo.as"
#ifdef	TASFRIG
	ident "@(#)scc-common:fw/common/sccfifo.as Rev 5.1 15:28:11 12/14/88 (TASFRIG)"
#else
	ident "@(#)scc-common:fw/common/sccfifo.as Rev 5.1 15:28:11 12/14/88"
#endif

#define	ASSEMBLY
#include "local/scc/fifo.h"

	text
	even

 # 
 # C interface Synopsis
 # ====================
 #
 # int bfifoget (ff)
 #	struct fifo *ff;
 #
 # ff is the pointer to the header of the fifo structure from which a byte
 # is to be read. If the fifo is empty -1 is returned in %d0, otherwise the 
 # byte is returned in the bottom 8 bits of %d0 with the top 24 bits cleared.
 # The fifo read index is also updated if a byte is read from the fifo.

	global	bfifoget

bfifoget:
	mov.l	4(%sp), %a0		# Get addr of fifo from stack into %a0

	# Must lock fifo read index before reading from fifo.
	# Release only when written back to fifo.

Lgetlk:
#ifdef	TASFRIG
	mov.w	%sr, %d0		# Save the old status register
	or.w	&0x0700, %sr		# Level 7, no interrupts
	clr.b	RMOV(%a0)		# Access location next to tas location
#endif
	tas.b	RLCK(%a0)		# Acquire read index lock flag
	beq.b	Lgetok			# Bit 7 was 0, got flag. Now bit 7 is 1

	# tas has failed. Bit 7 was already set to 1
#ifdef	TASFRIG
	mov.w	%d0, %sr 		# Restore %sr before delay
#endif
	
	# Increment failed to lock counter for read flag
	addq.l	&1, RLCKCNT(%a0) 

	mov.w	&0x10, %d0		# Load 16 into %d0
Lgetdly:
	dbra	%d0, Lgetdly		# Loop 16 times as a short delay
	bra.b	Lgetlk			# Try and lock again

Lgetok:
#ifdef	TASFRIG
	mov.w	%d0, %sr 		# Restore %sr, then continue
#endif
	mov.w	RDINDEX(%a0), %d1	# Get read index into %d1
	cmp.w	%d1, WRINDEX(%a0)	# Compare read/write indices
	beq.b	Lgetret			# Equal ? (If yes return -1, empty fifo)
	clr.l 	%d0			# Clear %d0
	mov.b	DATA(%a0,%d1.W), %d0	# Get byte into %d0
	add.w	&1, %d1			# Increment index in %d1
	cmp.w	%d1, SIZE(%a0)		# Compare new read index and fifo size
	blt.b	Lreadok			# Less ? (If yes new read index OK)
	clr.l 	%d1			# Wrap round. Set read index to 0
Lreadok:
	mov.w	%d1, RDINDEX(%a0)	# Store new read index in fifo
	clr.b	RLCK(%a0)		# Release fifo read index lock
	rts				# Return.

Lgetret:
	clr.b	RLCK(%a0)		# Release fifo read index lock
	mov.l	&0xffffffff, %d0	# Store -1 in %d0
	rts				# Return.


	# Used for 'turn round' tests only.

	global	bfifoqget

bfifoqget:
	mov.l	4(%sp), %a0		# Get addr of fifo from stack into %a0
	mov.w	RDINDEX(%a0), %d1	# Get read index into %d1
	cmp.w	%d1, WRINDEX(%a0)	# Compare read/write indices
	beq.b	Lqgetret		# Equal ? (If yes return -1, empty fifo)
	clr.l 	%d0			# Clear %d0
	mov.b	DATA(%a0,%d1.W), %d0	# Get byte into %d0
	add.w	&1, %d1			# Increment index in %d1
	cmp.w	%d1, SIZE(%a0)		# Compare new read index and fifo size
	blt.b	Lqreadok		# Less ? (If yes new read index OK)
	clr.l 	%d1			# Wrap round. Set read index to 0
Lqreadok:
	mov.w	%d1, RDINDEX(%a0)	# Store new read index in fifo
	rts				# Return.

Lqgetret:
	mov.l	&0xffffffff, %d0	# Store -1 in %d0
	rts				# Return.

 
 # 
 # C interface Synopsis
 # ====================
 #
 # int bfifoput (ff, ch)
 #	struct fifo *ff;
 #	char ch;
 #
 # ff is the pointer to the header of the fifo structure to which the character
 # is to be written. If the fifo is full -1 is returned in %d0, otherwise the 
 # character is written to the fifo and the write index updated and 0 returned
 # in %d0.

	global	bfifoput

bfifoput:
	mov.l	4(%sp), %a0		# Get addr of fifo from stack into %a0

	# Must lock fifo write index before writing to fifo.
	# Release only when written back to fifo.

Lputlk:
#ifdef	TASFRIG
	mov.w	%sr, %d0		# Save the old status register
	or.w	&0x0700, %sr		# Level 7, no interrupts
	clr.b	WMOV(%a0)		# Access location next to tas location
#endif
	tas.b	WLCK(%a0)		# Acquire write index lock flag
	beq.b	Lputok			# Bit 7 was 0, got flag. Now bit 7 is 1

	# tas has failed. Bit 7 was already set to 1
#ifdef	TASFRIG
	mov.w	%d0, %sr 		# Restore %sr before delay
#endif

	# Increment failed to lock counter for write flag
	addq.l	&1, WLCKCNT(%a0) 

	mov.w	&0x10, %d0		# Load 16 into %d0
Lputdly:
	dbra	%d0, Lputdly		# Loop 16 times as a short delay
	bra.b	Lputlk			# Try and lock again

Lputok:
#ifdef	TASFRIG
	mov.w	%d0, %sr 		# Restore %sr, then continue
#endif
	mov.w	WRINDEX(%a0), %d1	# Get write index into %d1
					# Write index always points to any
					# location in the fifo, next one to use
	mov.b	11(%sp),DATA(%a0,%d1.W) # Get byte from stack into fifo 
	add.w	&1, %d1			# Increment write index in %d1
	cmp.w	%d1, SIZE(%a0)		# Compare new write index with size
	blt.b	Lwriteok		# Less ? (If yes new write index OK)
	clr.l 	%d1			# Wrap round. Set write index to 0
Lwriteok:
	cmp.w	%d1, RDINDEX(%a0)	# Compare write index with read index
	beq.b	Lputret			# Equal ? (If yes fifo full)
					# Don't store new write index in fifo
	mov.w	%d1, WRINDEX(%a0)	# Store new write index in fifo
	clr.b	WLCK(%a0)		# Release fifo write index lock
	clr.l 	%d0			# Return 0 in %d0
	rts				# Return.

Lputret:
	clr.b	WLCK(%a0)		# Release fifo write index lock
	mov.l	&0xffffffff, %d0	# Store -1 in %d0
	rts				# Return.
 

	# Use for 'turn round' test only.
 
	global	bfifoqput

bfifoqput:
	mov.l	4(%sp), %a0		# Get addr of fifo from stack into %a0
	mov.w	WRINDEX(%a0), %d1	# Get write index into %d1
					# Write index always points to any
					# location in the fifo, next one to use
	mov.b	11(%sp),DATA(%a0,%d1.W) # Get byte from stack into fifo 
	add.w	&1, %d1			# Increment write index in %d1
	cmp.w	%d1, SIZE(%a0)		# Compare new write index with size
	blt.b	Lqwriteok		# Less ? (If yes new write index OK)
	clr.l 	%d1			# Wrap round. Set write index to 0
Lqwriteok:
	cmp.w	%d1, RDINDEX(%a0)	# Compare write index with read index
	beq.b	Lqputret		# Equal ? (If yes fifo full)
					# Don't store new write index in fifo
	mov.w	%d1, WRINDEX(%a0)	# Store new write index in fifo
	clr.l 	%d0			# Return 0 in %d0
	rts				# Return.

Lqputret:
	mov.l	&0xffffffff, %d0	# Store -1 in %d0
	rts				# Return.
	
 # 
 # C interface Synopsis
 # ====================
 #
 # int nopsloop(i)
 #	int i;
 #
 # int bfifoecho(ffin, ffout, cnt)
 #	struct fifo *ffin, *ffout;
 #	int cnt;
 #

	global	nopsloop

nopsloop:
	mov.l	4(%sp), %d0

Lnopsly:
	nop
	dbra	%d0, Lnopsly		# Loop 16 times as a short delay
	rts

	global	bfifoecho

bfifoecho:
	mov.l	4(%sp), %a0		# Get addr of input fifo into %a0
	mov.l	8(%sp), %a1		# Get addr of output fifo into %a1

	# Dec stacked copy of cnt
Lbfifoecho:
	subq.l	&1, 12(%sp)
	blt.b	Lbexit			# Less than zero


	mov.w	RDINDEX(%a0), %d1	# Get read index into %d1
	cmp.w	%d1, WRINDEX(%a0)	# Compare read/write indices
	beq.b	Leret			# Equal ? (If yes return -1, empty fifo)
	clr.l 	%d0			# Clear %d0
	mov.b	DATA(%a0,%d1.W), %d0	# Get byte into %d0
	add.w	&1, %d1			# Increment index in %d1
	cmp.w	%d1, SIZE(%a0)		# Compare new read index and fifo size
	blt.b	Lereadok		# Less ? (If yes new read index OK)
	clr.l 	%d1			# Wrap round. Set read index to 0
Lereadok:
	mov.w	%d1, RDINDEX(%a0)	# Store new read index in fifo

	# Data in %d0

	mov.w	WRINDEX(%a1), %d1	# Get write index into %d1
					# Write index always points to any
					# location in the fifo, next one to use
	mov.b	%d0,DATA(%a1,%d1.W) 	# Get byte from stack into fifo 
	add.w	&1, %d1			# Increment write index in %d1
	cmp.w	%d1, SIZE(%a1)		# Compare new write index with size
	blt.b	Lewriteok		# Less ? (If yes new write index OK)
	clr.l 	%d1			# Wrap round. Set write index to 0
Lewriteok:
	cmp.w	%d1, RDINDEX(%a1)	# Compare write index with read index
	beq.b	Leret			# Equal ? (If yes fifo full)
					# Don't store new write index in fifo
	mov.w	%d1, WRINDEX(%a1)	# Store new write index in fifo
	bra.b	Lbfifoecho		# Next copy

Lbexit:
	clr.l 	%d0			# Return 0 in %d0
	rts				# Return.

Leret:
	mov.l	&0xffffffff, %d0	# Store -1 in %d0
	rts				# Return. Error
 

 # 
 # C interface Synopsis
 # ====================
 #
 # int bfifounput (ff)
 #	struct fifo *ff;
 #
 # ff is the pointer to the header of the fifo structure from which the last
 # byte written is to be removed. If the fifo is empty -1 is returned in %d0,
 # otherwise the byte is returned in the bottom 8 bits of %d0 with the top 
 # 24 bits cleared.
 # The fifo write index is also updated if a byte is removed from the fifo.

	global	bfifounput

bfifounput:
	mov.l	4(%sp), %a0		# Get addr of fifo from stack into %a0

	# Must lock fifo write index before reading from fifo and then read 
	# index must also be locked. All this to stop a 'pointer crossover' !
	# ALWAYS do in order WLCK the RLCK
	# Release only when written back to fifo.

Lunputlk:
#ifdef	TASFRIG
	mov.w	%sr, %d0		# Save the old status register
	or.w	&0x0700, %sr		# Level 7, no interrupts
	clr.b	WMOV(%a0)		# Access location next to tas location
#endif
	tas.b	WLCK(%a0)		# Acquire write index lock flag
	beq.b	Lunwok			# Bit 7 was 0, got flag. Now bit 7 is 1

	# tas has failed. Bit 7 was already set to 1
#ifdef	TASFRIG
	mov.w	%d0, %sr		# Restore %sr before delay
#endif
	
	# Increment failed to lock counter for read flag
	addq.l	&1, WLCKCNT(%a0) 

	mov.w	&0x10, %d0		# Load 16 into %d0
Lunputdly:
	dbra	%d0, Lunputdly		# Loop 16 times as a short delay
	bra.b	Lunputlk		# Try and lock again

	# Have WLCK now for RLCK
Lunwok:					# Lock read index flag
#ifdef	TASFRIG
					# Still at level 7
	clr.b	RMOV(%a0)		# Access location next to tas location
#endif
	tas.b	RLCK(%a0)		# Acquire read index lock flag
	beq.b	Lunputok		# Bit 7 was 0, got flag. Now bit 7 is 1

	# tas has failed. Bit 7 was already set to 1
#ifdef	TASFRIG
	mov.w	%d0, %sr		# Restore %sr before delay
#endif

	# Increment failed to lock counter for read flag
	addq.l	&1, RLCKCNT(%a0) 

	mov.w	&0x10, %d0		# Load 16 into %d0
Lunrdly:
	dbra	%d0, Lunrdly		# Loop 16 times as a short delay
	bra.b	Lunwok			# Try and lock again

Lunputok:				# Have both locked
#ifdef	TASFRIG
	mov.w	%d0, %sr 		# Restore %sr, then continue
#endif
	mov.w	WRINDEX(%a0), %d1	# Get write index into %d1
	cmp.w	%d1, RDINDEX(%a0)	# Compare read/write indices
	beq.b	Lunputret		# Equal ? (If yes return -1, empty fifo)
	clr.l 	%d0			# Clear %d0
	sub.w	&1, %d1			# Decrement index in %d1
	bge.b	Lunreadok		# Greater than equal 0, still OK
	mov.w 	SIZE(%a0), %d1		# Wrap round. Set write index to SIZE-1
	sub.w	&1,%d1
Lunreadok:
	mov.b	DATA(%a0,%d1.W), %d0	# Get byte into %d0
	mov.w	%d1, WRINDEX(%a0)	# Store new write index in fifo
	clr.b	WLCK(%a0)		# Release fifo write index lock
	clr.b	RLCK(%a0)		# Release fifo read index lock
	rts				# Return.

Lunputret:
	clr.b	WLCK(%a0)		# Release fifo write index lock
	clr.b	RLCK(%a0)		# Release fifo read index lock
	mov.l	&0xffffffff,%d0		# Store -1 in %d0
	rts				# Return.


 # 
 # C interface Synopsis
 # ====================
 #
 # int zapfifo (ff)
 #	struct fifo *ff;
 #
 # ff is the pointer to the header of the fifo structure which is to be
 # emptied, or zapped. To avoid possible pointer problems both index lock
 # flags MUST be acquired prior to the fifo pointers being manipulated.
 # 0 is always returned in %d0.

	global	zapfifo

zapfifo:
	mov.l	4(%sp), %a0		# Get addr of fifo from stack into %a0

	# Must lock fifo read and write indices before writing to fifo.
	# ALWAYS do in order WLCK the RLCK
	# Release only when written back to fifo.

Lwlk:					# Lock write index flag
#ifdef	TASFRIG
	mov.w	%sr, %d0		# Save the old status register
	or.w	&0x0700, %sr		# Level 7, no interrupts
	clr.b	WMOV(%a0)		# Access location next to tas location
#endif
	tas.b	WLCK(%a0)		# Acquire write index lock flag
	beq.b	Lwok			# Bit 7 was 0, got flag. Now bit 7 is 1

	# tas has failed. Bit 7 was already set to 1
#ifdef	TASFRIG
	mov.w	%d0, %sr		# Restore %sr before delay
#endif
	
	# Increment failed to lock counter for write flag
	addq.l	&1, WLCKCNT(%a0) 

	mov.w	&0x10, %d0		# Load 16 into %d0
Lwdly:
	dbra	%d0, Lwdly		# Loop 16 times as a short delay
	bra.b	Lwlk			# Try and lock again

	# Have WLCK now for RLCK
Lwok:					# Lock read index flag
#ifdef	TASFRIG
					# Still at level 7
	clr.b	RMOV(%a0)		# Access location next to tas location
#endif
	tas.b	RLCK(%a0)		# Acquire read index lock flag
	beq.b	Lbothok			# Bit 7 was 0, got flag. Now bit 7 is 1

	# tas has failed. Bit 7 was already set to 1
#ifdef	TASFRIG
	mov.w	%d0, %sr		# Restore %sr before delay
#endif

	# Increment failed to lock counter for read flag
	addq.l	&1, RLCKCNT(%a0) 

	mov.w	&0x10, %d0		# Load 16 into %d0
Lrdly:
	dbra	%d0, Lrdly		# Loop 16 times as a short delay
	bra.b	Lwok			# Try and lock again

Lbothok:				# Have both index flags locked
#ifdef	TASFRIG
	mov.w	%d0, %sr		# Restore %sr, then continue
#endif

	# Set read index to write index to zero
	
	clr.w	WRINDEX(%a0)		# write index to zero
	clr.w	RDINDEX(%a0)		# read index to zero

	clr.b	WLCK(%a0)		# Release fifo read/write index lock
	clr.b	RLCK(%a0)
	
	mov.l 	&0, %d0			# Return 0 in %d0
	rts				# Return.


 # 
 # C interface Synopsis
 # ====================
 #
 # int wlock_fifo (ff)
 #	struct fifo *ff;
 #
 # ff is the pointer to the header of the fifo structure which is to be
 # write locked. Once a fifo is write locked no data may be put into the fifo
 # via the fifo handler routines.
 # 0 is always returned in %d0.

	global	wlock_fifo

wlock_fifo:
	mov.l	4(%sp), %a0		# Get addr of fifo from stack into %a0

	# Lock fifo write index

Lw1lk:					# Lock write index flag
#ifdef	TASFRIG
	mov.w	%sr, %d0		# Save the old status register
	or.w	&0x0700, %sr		# Level 7, no interrupts
	clr.b	WMOV(%a0)		# Access location next to tas location
#endif
	tas.b	WLCK(%a0)		# Acquire write index lock flag
	beq.b	Lw1ok			# Bit 7 was 0, got flag. Now bit 7 is 1

	# tas has failed. Bit 7 was already set to 1
#ifdef	TASFRIG
	mov.w	%d0, %sr		# Restore %sr before delay
#endif
	
	# Increment failed to lock counter for write flag
	addq.l	&1, WLCKCNT(%a0) 

	mov.w	&0x10, %d0		# Load 16 into %d0
Lw1dly:
	dbra	%d0, Lw1dly		# Loop 16 times as a short delay
	bra.b	Lw1lk			# Try and lock again

Lw1ok: 					# Locked write index flag
#ifdef	TASFRIG
	mov.w	%d0, %sr		# Restore %sr, then continue
#endif
	clr.l	 %d0			# Returen zero
	rts


 # 
 # C interface Synopsis
 # ====================
 #
 # int wunlock_fifo (ff)
 #	struct fifo *ff;
 #
 # Unlock a write locked fifo. The fifo tp be unlocked is indexed by ff.
 # 0 is always returned in %d0.

	global	wunlock_fifo

wunlock_fifo:
	mov.l	4(%sp), %a0		# Get addr of fifo from stack into %a0
	clr.b	WLCK(%a0) 		# Clear fifo write index
	clr.l	%d0
	rts


 # 
 # C interface Synopsis
 # ====================
 #
 # int fifo_room(ff)
 #	struct fifo *ff;
 #
 # ff is the pointer to the header of a fifo structure. The routine returns
 # the number of empty positions (unused entries) that are left in the 
 # specified fifo.

	global	fifo_room

fifo_room:
	mov.l	4(%sp), %a0		# Get addr of fifo from stack into %a0
	clr.l	%d0			# Clear %d0, prior to movm
	movm.w	RDINDEX(%a0), &0x0003	# pick up read and write index to d0/d1
	sub.w	%d1, %d0
	bgt.b	Lfifo_room
	add.w	SIZE(%a0), %d0
Lfifo_room:
	sub.w 	&1, %d0
	rts				# Free slots in %d0

 
 # 
 # C interface Synopsis
 # ====================
 #
 # int fifo_entries(ff)
 #	struct fifo *ff;
 #
 # ff is the pointer to the header of a fifo structure. The routine returns
 # the number of valid entries in the specified fifo.
 
	global	fifo_entries

fifo_entries:
	mov.l	4(%sp), %a0		# Get addr of fifo from stack into %a0
	movm.w	RDINDEX(%a0), &0x0003	# Load read/write index to d0/d1
	sub.w	%d0, %d1
	bge.b	Lfifo_entries
	add.w	SIZE(%a0), %d1
Lfifo_entries:
	clr.l	%d0
	mov.w	%d1, %d0
	rts				# Entries in %d0
	

 #
 # C interface Synopsis
 # ====================
 #
 # int wfifoget (ff)
 #	struct fifo *ff;
 #
 # ff is the pointer to the header of the fifo structure from which a word
 # is to be read. If the fifo is empty -1 is returned in %d0, otherwise the 
 # word is returned in the bottom 16 bits of %d0 with the top 16 bits cleared.
 # The fifo read index is also updated if a word is read from the fifo.

	global	wfifoget

wfifoget:
	mov.l	4(%sp), %a0		# Get addr of fifo from stack into %a0

	# Must lock fifo read index before reading from fifo.
	# Release only when written back to fifo.

Lwgetlk:
#ifdef	TASFRIG
	mov.w	%sr, %d0		# Save the old status register
	or.w	&0x0700, %sr		# Level 7, no interrupts
	clr.b	RMOV(%a0)		# Access location next to tas location
#endif
	tas.b	RLCK(%a0)		# Acquire read index lock flag
	beq.b	Lwgetok			# Bit 7 was 0, got flag. Now bit 7 is 1

	# tas has failed. Bit 7 was already set to 1
#ifdef	TASFRIG
	mov.w	%d0, %sr		# Restore %sr before delay
#endif
	
	# Increment failed to lock counter for read flag
	addq.l	&1, RLCKCNT(%a0) 

	mov.w	&0x10, %d0		# Load 16 into %d0
Lwgetdly:
	dbra	%d0, Lwgetdly		# Loop 16 times as a short delay
	bra.b	Lwgetlk			# Try and lock again

Lwgetok:
#ifdef	TASFRIG
	mov.w	%d0, %sr		# Restore %sr, then continue
#endif
	clr.l	%d1			# Clear %d1 ready for times two, could 
					# extend into top 16 bits
	mov.w	RDINDEX(%a0), %d1	# Get read index into %d1
	cmp.w	%d1, WRINDEX(%a0)	# Compare read/write indices
	beq.b	Lwgetret		# Equal ? (If yes return -1, empty fifo)
	clr.l 	%d0			# Clear %d0, ensures no problem with
					# index extending into top 16 bits 
	lsl.l	&1, %d1			# times index by 2 for a word access
	mov.w	DATA(%a0,%d1.L), %d0	# Get byte into %d0 (%d1 is LONG)
	lsr.l	&1, %d1			# restore index again
	add.w	&1, %d1			# Increment index in %d1
	cmp.w	%d1, SIZE(%a0)		# Compare new read index and fifo size
	blt.b	Lwreadok		# Less ? (If yes new read index OK)
	clr.l 	%d1			# Wrap round. Set read index to 0
Lwreadok:
	mov.w	%d1, RDINDEX(%a0)	# Store new read index in fifo
	clr.b	RLCK(%a0)		# Release fifo read index lock
	rts				# Return.

Lwgetret:
	clr.b	RLCK(%a0)		# Release fifo read index lock
	mov.l	&0xffffffff, %d0	# Store -1 in %d0
	rts				# Return.


 # 
 # C interface Synopsis
 # ====================
 #
 # int wfifounget (ff, wd)
 #	struct fifo *ff;
 #	unsigned short wd;
 #
 # ff is the pointer to the header of the fifo structure to which a word
 # is to be written. If the fifo is full -1 is returned in %d0, otherwise 0. 
 # The fifo read index is also updated if a word is written to the fifo.
 # This operation is used to 'stuff' data back into a fifo so that it will
 # be the next data read by a wfifoget()

	global	wfifounget

wfifounget:
	mov.l	4(%sp),%a0		# Get addr of fifo from stack into %a0

	# Must lock fifo read and write indices before writing to fifo.
	# ALWAYS do in order WLCK the RLCK
	# Release only when written back to fifo.

Lwunwlk:				# Lock write index flag
#ifdef	TASFRIG
	mov.w	%sr, %d0		# Save the old status register
	or.w	&0x0700, %sr		# Level 7, no interrupts
	clr.b	WMOV(%a0)		# Access location next to tas location
#endif
	tas.b	WLCK(%a0)		# Acquire write index lock flag
	beq.b	Lwunwok			# Bit 7 was 0, got flag. Now bit 7 is 1

	# tas has failed. Bit 7 was already set to 1
#ifdef	TASFRIG
	mov.w	%d0, %sr		# Restore %sr before delay
#endif
	
	# Increment failed to lock counter for write flag
	addq.l	&1, WLCKCNT(%a0) 

	mov.w	&0x10, %d0		# Load 16 into %d0
Lwunwdly:
	dbra	%d0, Lwunwdly		# Loop 16 times as a short delay
	bra.b	Lwunwlk			# Try and lock again

Lwunwok:				# Lock read index flag
#ifdef	TASFRIG
					# Still at level 7
	clr.b	RMOV(%a0)		# Access location next to tas location
#endif
	tas.b	RLCK(%a0)		# Acquire read index lock flag
	beq.b	Lwunbothok		# Bit 7 was 0, got flag. Now bit 7 is 1

	# tas has failed. Bit 7 was already set to 1
#ifdef	TASFRIG
	mov.w	%d0, %sr		# Restore %sr before delay
#endif

	# Increment failed to lock counter for read flag
	addq.l	&1, RLCKCNT(%a0) 

	mov.w	&0x10, %d0		# Load 16 into %d0
Lwunrdly:
	dbra	%d0, Lwunrdly		# Loop 16 times as a short delay
	bra.b	Lwunwok			# Try and lock again

Lwunbothok:				# Have both locked 
#ifdef	TASFRIG
	mov.w	%d0, %sr		# Restore %sr, then continue 
#endif
	clr.l	%d1			# Clear to 0
	mov.w	RDINDEX(%a0), %d1	# Get read index
	sub.w	&1, %d1			# Decrement read index
	bge.b	Lwungetok		# Still +ve or 0

	# -ve, wrap round index to size-1

	clr.l	%d1			# Clear to 0
	mov.w	SIZE(%a0), %d1
	sub.w	&1, %d1
Lwungetok:				# Have correct read index
	cmp.w	%d1, WRINDEX(%a0)	# Check for full. If equal full
	beq.b	Lwungetret		# Exit

	lsl.l	&1, %d1  		# Write data to fifo at %d1
	mov.w	10(%sp), DATA(%a0,%d1.L)
	lsr.l	&1, %d1

	# Update read index

	mov.w	%d1, RDINDEX(%a0)
	clr.b	WLCK(%a0)		# Release fifo write index lock
	clr.b	RLCK(%a0)		# Release fifo read index lock
	clr.l	%d0			# Store 0 in %d0
	rts				# Return.

	# Failed to put data at read index, fifo full
Lwungetret:
	clr.b	WLCK(%a0)		# Release fifo write index lock
	clr.b	RLCK(%a0)		# Release fifo read index lock
	mov.l	&0xffffffff, %d0	# Store -1 in %d0
	rts				# Return.


 # 
 # C interface Synopsis
 # ====================
 #
 # int wfifotget (ff, &ptr)
 #	struct fifo *ff;     (%a0)
 #	short *ptr;          (%a1)
 #
 # ff is the pointer to the header of the fifo structure from which a word
 # is to be read. The normal fifo read index is NOT used or altered, but the
 # short indexed by 'ptr' is used as a tempory read index. It is updated and
 # used as if it were the normal f_rdindex.

	global	wfifotget

wfifotget:
	mov.l	4(%sp), %a0		# Get addr of fifo from stack into %a0
	mov.l	8(%sp), %a1		# Get addr of pointer word

	#  No need to lock temporary read index

	clr.l	%d1			# Clear %d1 ready for times two, could 
					# extend into top 16 bits
	mov.w	(%a1), %d1		# Get read index into %d1
	cmp.w	%d1, WRINDEX(%a0)	# Compare read/write indices
	beq.b	Lwtgetret		# Equal ? (If yes return -1, empty fifo)
	clr.l 	%d0			# Clear %d0, ensures no problem with
					# index extending into top 16 bits 
	lsl.l	&1, %d1			# times index by 2 for a word access
	mov.w	DATA(%a0,%d1.L), %d0	# Get byte into %d0 (%d1 is LONG)
	lsr.l	&1, %d1			# restore index again
	add.w	&1, %d1			# Increment index in %d1
	cmp.w	%d1, SIZE(%a0)		# Compare new read index and fifo size
	blt.b	Lwtreadok		# Less ? (If yes new read index OK)
	clr.l 	%d1			# Wrap round. Set read index to 0
Lwtreadok:
	mov.w	%d1, (%a1)		# Store new read index in tmp rd index
	rts				# Return.

Lwtgetret:
	mov.l	&0xffffffff, %d0	# Store -1 in %d0
	rts				# Return.


 # 
 # C interface Synopsis
 # ====================
 #
 # int wfifoclw (ff, &ptr)
 #	struct fifo *ff;     (%a0)
 #	short *ptr;          (%a1)
 #
 # ff is the pointer to the header of the fifo structure to which a NULL word
 # is to be written. The normal fifo read index is NOT used or altered, but the
 # short indexed by 'ptr' is used as a pointer to the word + 1 

	global	wfifoclw

wfifoclw:
	mov.l	4(%sp), %a0		# Get addr of fifo from stack into %a0
	mov.l	8(%sp), %a1		# Get addr of pointer word

	#  No need to lock temporary read index

	clr.l	%d1			# Clear to 0
	clr.l	%d0			# Data for fifo. Must be >=0
	mov.w	(%a1), %d1		# Get tmp read index
	sub.w	&1, %d1			# Decrement read index
	bge.b	Lwclwok  		# Still +ve or 0

	# -ve, wrap round index to size-1
	mov.w	SIZE(%a0), %d1
	sub.w	&1, %d1
Lwclwok:				# Have correct write index in %d1

	# Write data (NULL) to fifo at %d1

	lsl.l	&1, %d1
	mov.w	%d0, DATA(%a0,%d1.L)
	
	rts				# Return.

