/*
 *                  Copyright (c) 1988 Ferranti Plc.
 *                      All Rights Reserved
 *
 *
 * THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF FERRANTI PLC THE COPYRIGHT 
 *                   OF WHICH VESTS IN FERRANTI PLC.
 *
 *   The copyright notice above does not evidence any actual or intended 
 *                   publication of such source code.
 *
 * This source code and the information contained herein is supplied without
 * liability for errors or ommissions and no part may be copied, re-produced, 
 * used or disclosed except as authorised by the Ferranti organisation by 
 *                contract or by other written permission.
 */

/******************************************************************************

	Rev = 5.1		Date of 'get'   = 14:17:56 2/14/89
				Date of 'delta' = 14:48:36 1/25/89

	File = sccl.c

******************************************************************************/
			 
#ident "@(#)scc-driver:driver/sccl.c Rev 5.1 14:48:36 1/25/89"

/*
 * Include files
 */

#include <sys/param.h>
#include <sys/types.h>
#include <sys/errno.h>
#include <sys/signal.h>
#include <sys/fs/s5dir.h>
#include <sys/user.h>
#include <sys/immu.h>
#include <sys/region.h>
#include <sys/proc.h>
#include <sys/sysinfo.h>
#include <sys/conf.h>
#include <sys/termio.h>
#include <sys/tty.h>
#include <sys/file.h>
#include <sys/cmn_err.h>

#include <local/scc/fifo.h>
#include <local/scc/lowmem.h>
#include <local/scc/status.h>
#include <local/scc/duartchan.h>
#include <local/scc/duartdefs.h>
#include <local/scc/termctrl.h>
#include <local/scc/scccmd.h>
#include <local/scc/scclioctl.h>
#include <local/scc/scclerr.h>
#include <local/scc/sccdctrl.h>

/*
 * cmn_err print statements can be controlled by the use of the first 
 * character of the print statement to determine where the message is output
 * to the kernel's internal buffer only, the console only, or both.
 *
 * 1st char '!' print to buffer only
 * 1st char '^' print to console only
 * Any other 1st character print to both
 */

/*
 * Standard #defines for booleans
 */
#define	TRUE	1
#define	FALSE	0

/*
 * extern structure declarations from scc_conf.c
 */
extern struct duart_channel *scc_duart[];
extern struct scc_ctrl scc_ctrl[];
extern int scc_card_max;

#define	UL(x)	((unsigned long)x)
#define UC(x)	((unsigned char)(x))
#define UCP(x)	((unsigned char *)(x))

/*
 * extern structures generated by config from dfile and master.
 * Actual source declarations are in the config generated file conf.c
 */
extern int scc_addr[];
extern int scc_ivec[];
extern int scc_ilev[];
extern int scc_cnt;
extern struct tty scc_tty[];

/*
 * Local structures and declarations
 */
int sccl_open, digits[132];
char ddata[132];
struct load_info record;
struct card_data card_data;

unsigned long sccl_fw_swait();
unsigned char *sccl_get_addr();

/*
 * SCCL_LOOP fifo in/out pointer array
 */
static struct fifo *hin [PORTS_PER_SCC];
static struct fifo *hout[PORTS_PER_SCC];
static struct fifo *hinptr, *houtptr;

/*
 * SCCL driver error word
 */
unsigned long sccl_error;

/**/
scclinit()
{
	sccl_open  = FALSE;
	sccl_error = 0;
}

/* 
 * scclopen 
 *
 * Mark device /dev/loaders as open. Only one task is allowed to have this
 * device open at any one time. 
 */
scclopen(dev, flag)
dev_t dev;
int flag;
{
	if (sccl_open)
		u.u_error = EBUSY;
	else {
		sccl_open = TRUE;
		sccl_error = 0;
	}

}

/*
 * scclclose
 *
 * Close the device and wake any task that is sleeping
 */
scclclose(dev, flag)
dev_t dev;
int flag;
{
	sccl_open = FALSE;
}

/*
 *
 * scclioctl
 *
 * Main entry point. Is used to allow control of an SCC card within the system
 */
scclioctl(dev, cmd, arg, mode)
int dev, cmd, mode;
char *arg;
{
	register s, i, j;
	char * data;
	unsigned long *ptr;
	register unsigned char *addr;
	int index;
	unsigned long sdata;
	int n, m;

	switch(cmd) {
		/*  */
	case SCCL_FIFO:
		/*
		 * SCCL_FIFO: Define the SCC fifo buffer sizes and complete
		 *            firmware initialisation sequence. The firmware
		 *            may have been loaded from file or EPROM.
		 */

		/*
		 * root only access
		 */
		if (!suser()) {
			u.u_error = EPERM;
			break;
		}

		/*
		 * The card index is the last long word in each 256 byte
		 * fifo definition block. arg is a pointer to the start of
		 * the first block for channel 0.
		 * Get get the card index word into index
		 */

		if ( (addr = sccl_get_addr(UL(arg)+OFF_CARD_INDEX,&index)) == 
							UCP(-1) )
			break;

		/*
	 	 * Now need to read the mem_grab field of the fifo definition
		 * block. The mem_grab field is an unsigned long at 
		 * offset MG_OFFSET from the start of each definition block.
		 */
		if (copyin(UL(arg)+OFF_MEM_GRAB,(caddr_t)&sdata,
					sizeof(unsigned long))) {
			u.u_error = EFAULT;
			break; 
		}


		/*
		 * Wait for a correct status from the firmware (HOST_MDEF)
		 * Either timeout are have correct status. Which is it ?
		 */
		if ( sccl_fw_swait(addr,HOST_MDEF) != (CHK_32|HOST_MDEF) ) {
			/*
		         * Not HOST_MDEF status !
			 */
			u.u_error = EIO;
			sccl_error = SE_HOSTMDEFW;
			cmn_err(CE_CONT,
			   "!scclioctl: SCCL_FIFO status not HOST_MDEF\n");
			break;
		}

		/*
	 	 * Setup fifos. Use the firmware CNFG command.
		 * Firmware is polling for BF_CMD to become 0. The config 
		 * command must be written to BF_ARG1 ('C'N'F'G)
		 * BF_ARG1 command is F_CONFIG which is the ASCII of the 
		 * characters D (0x44), F (0x46), L (0x4C), T (0x54).
		 */
		*(unsigned long *)(addr + BF_ARG1) = F_CONFIG;	/* ARG1 */
		*(unsigned long *)(addr + BF_ARG2) = sdata;	/* ARG2 */
		*(unsigned long *)(addr + BF_CMD)  = 0;		/* CMD  */
	
		/*
		 * Wait for a correct status from the firmware. 
		 * Need HOST_FDEF to continue. Give firmware a small delay
		 * to let it set the status
		 */
		delay(4);
		
		if ( sccl_fw_swait(addr,HOST_FDEF) != (CHK_32|HOST_FDEF) ) {
			/*
		         * Not HOST_FDEF status !
			 */
			u.u_error = EIO;
			sccl_error = SE_HOSTFDEFW;
			cmn_err(CE_CONT,
			   "!scclioctl: SCCL_FIFO status not HOST_FDEF\n");
			break;
		}

		/*
		 * Have HOST_FDEF status prior to first data block load
		 * ARG1 will contain (~chan & 0xFF). Load each configuration
		 * block in sequence.
		 */
		for (i = 0; i < MAXNDUARTS; i++) {
			/*
			 * Wait for ARG1 to be (~cnt & 0xFF)
			 */
			s = 0;
			while (*(unsigned long *)(addr + BF_ARG1) 
						!= (~i & 0xFF) ) {
				if (++s > 60) {
					u.u_error = EIO;
					sccl_error = SE_HOSTFARG1W;
					cmn_err(CE_CONT,
		"!scclioctl: SCCL_FIFO failed to get ARG1 complement\n");

					/*
					 * A break here will only terminate
					 * the while loop.
					 */
					break;
				}
				delay(4);
			}

			if (s > 60)
				break;		/* Now out of for loop */

			/*
			 * Firmware ready for descriptor
			 */
			if (copyin((UL(arg)+(i*256)),(caddr_t)(addr+0x2E00),256)) {
				u.u_error = EFAULT;
				break;
			}
			*(unsigned long *)(addr + BF_ARG2) = F_CONFIG;
			*(unsigned long *)(addr + BF_ARG1) = 0x00; 
		}

		if (s > 60)
			break;		/* Out at last */

		/*
		 * Wait for a correct status from the firmware. 
		 * Need HOST_IDEF to continue. Sequence same as before
		 */
		
		if ( sccl_fw_swait(addr,HOST_IDEF) != (CHK_32|HOST_IDEF) ) {
			/*
		         * Not HOST_IDEF status !
			 */
			u.u_error = EIO;
			sccl_error = SE_HOSTIDEFW;
			cmn_err(CE_CONT,
			   "!scclioctl: SCCL_FIFO status not HOST_IDEF\n");
			break;
		}

		/*
		 * F/W waiting for host to clear BF_CMD to 0.
		 *
		 * Things to be set in response to HOST_IDEF:
		 *
		 * BF_ARG1 - interrupt type (SINGLE_INTR)
		 * BF_ARG2 - interrupt level
		 * BF_ARG3 - interrupt vector
		 * BF_ARG4 - card access code (code, data, user, supervisor)
		 */
		*(unsigned long *)(addr + BF_ARG1) = SINGLE_INTR;
		*(unsigned long *)(addr + BF_ARG2) = scc_ilev[index];
		*(unsigned long *)(addr + BF_ARG3) = scc_ivec[index]/4;
		*(unsigned long *)(addr + BF_ARG4) = 0x3;

		/*
		 * Clear command and release f/w
		 */
		*(unsigned long *)(addr + BF_CMD) = 0;

		/*
		 *        CARD MUST NOT BE ACCESSED FOR 12.5 usecs AS 
		 *                   ACCESS CODES MAY CHANGE
		 * (F/w is in a 10 instruction loop. 5 cycles per instruction,
		 * 3 clock cycles per instruction cycle. Thus 150 clocks.
		 * Max time for 150 clocks at 12 MHz = 12.5 * 10(-6), 
		 * 12.5 microseconds ).
		 * A call of delay(1) will greatly exceed the required delay.
		 */
		delay(1);

		/*
		 * Now wait for a status of LM_TEST and then do location 
		 * monitor test.
		 */
		if ( sccl_fw_swait(addr,LM_TEST) != (CHK_32|LM_TEST) ) {
			/*
		         * Not LM_TEST status !
			 */
			u.u_error = EIO;
			sccl_error = SE_LMTESTW;
			cmn_err(CE_CONT,
			   "!scclioctl: SCCL_FIFO status not LM_TEST\n");
			break;
		}

		/*
		 * Access cards location monitor
		 */
		*(unsigned long *)(addr + LM) = 0;

		/*
		 * Complete location monitor test and then do handshake
		 * Do a short delay to allow firmware to setup response.
		 */
		delay(1); 

		if ( sccl_fw_swait(addr,LM_TEST_DONE) != 
						(CHK_32|LM_TEST_DONE) ) {
			/*
		         * Not LM_TEST_DONE status !
			 */
			u.u_error = EIO;
			sccl_error = SE_LMTSTDW;
			cmn_err(CE_CONT,
			   "!scclioctl: SCCL_FIFO status not LM_TEST_DONE\n");
			break;
		}

		/*
		 * Acknowledge and complete handshake
		 */
		*(unsigned long *)(addr + BF_CMD) = 0;

		/*
		 * Now to get all addresses for this 'index' into scc_duart[]
		 */

		ptr = (unsigned long *)(addr+FSBP);
		j  = *ptr;
#ifdef	DBUG
		cmn_err(CE_CONT,"scclioctl: FSBP@ 0x%x\n",j);
#endif
		for (s = 0; s < PORTS_PER_SCC; s++) {
			ptr = (unsigned long *)(UL(j) + UL(addr) + UL(s*4));
#ifdef	DBUG
			cmn_err(CE_CONT,"scclioctl: %d ptr 0x%x\n",s,ptr);
#endif
			i = *ptr;
			scc_duart[(index*PORTS_PER_SCC)+s] = 
				(struct duart_channel *)(UL(addr) + UL(i));
			scc_ctrl[(index*PORTS_PER_SCC)+s].scc_chan_present =
						TRUE;
#ifdef	DBUG
			cmn_err(CE_CONT,
"scclioctl: %d 0x%x\n",s,scc_duart[(index*PORTS_PER_SCC)+s]);
#endif
			
		}

		/*
		 * Now wait for ready status from firmware server task. Do a
		 * short delay first so that firmware should be ready
		 */
		delay(4);

		if ( sccl_fw_swait(addr,SIO_READY) != (CHK_32|SIO_READY) ) {
			/*
		         * Not SIO_READY status !
			 */
			u.u_error = EIO;
			sccl_error = SE_SIOREADYW;
			cmn_err(CE_CONT,
			   "!scclioctl: SCCL_FIFO status not SIO_READY\n");
			break;
		}

		break;

	case SCCL_FWINIT:
		/*
	 	 * SCCL_FWINIT: Initialise the on-board firmware once it has
 		 *              been loaded either from a file or EPROM.
		 */

		/*
		 * root only access
		 */
		if (!suser()) {
			u.u_error = EPERM;
			break;
		}

		/*
		 * First get the card index word into index
		 */

		if ( (addr = sccl_get_addr(arg,&index)) == UCP(-1) )
			break;

		/*
		 * Wait for a correct status from the firmware (HOST_MDEF)
		 * Either timeout are have correct status. Which is it ?
		 */
		if ( sccl_fw_swait(addr,HOST_MDEF) != (CHK_32|HOST_MDEF) ) {
			/*
		         * Not HOST_MDEF status !
			 */
			u.u_error = EIO;
			sccl_error = SE_HOSTMDEFW;
			cmn_err(CE_CONT,
			   "!scclioctl: SCCL_FWINIT status not HOST_MDEF\n");
			break;
		}

		/*
	 	 * Setup fifos. Use the firmware defaults.
		 * Firmware is polling for BF_CMD to become 0. The 'use default
		 * fifos' command must be written to BF_ARG1 ('D'F'L'T)
		 * BF_ARG1 command is F_DEFAULT which is the ASCII of the 
		 * characters D (0x44), F (0x46), L (0x4C), T (0x54).
		 */
		*(unsigned long *)(addr + BF_ARG1) = F_DEFAULT;	/* ARG1 */
		*(unsigned long *)(addr + BF_CMD)  = 0;		/* CMD  */
		
		/*
		 * Wait for a correct status from the firmware. 
		 * Need HOST_IDEF to continue. Sequence same as before
		 */
		
		if ( sccl_fw_swait(addr,HOST_IDEF) != (CHK_32|HOST_IDEF) ) {
			/*
		         * Not HOST_IDEF status !
			 */
			u.u_error = EIO;
			sccl_error = SE_HOSTIDEFW;
			cmn_err(CE_CONT,
			   "!scclioctl: SCCL_FWINIT status not HOST_IDEF\n");
			break;
		}

		/*
		 * F/W waiting for host to clear BF_CMD to 0.
		 *
		 * Things to be set in response to HOST_IDEF:
		 *
		 * BF_ARG1 - interrupt type (SINGLE_INTR)
		 * BF_ARG2 - interrupt level
		 * BF_ARG3 - interrupt vector
		 * BF_ARG4 - card access code (code, data, user, supervisor)
		 */
		*(unsigned long *)(addr + BF_ARG1) = SINGLE_INTR;
		*(unsigned long *)(addr + BF_ARG2) = scc_ilev[index];
		*(unsigned long *)(addr + BF_ARG3) = scc_ivec[index]/4;
		*(unsigned long *)(addr + BF_ARG4) = 0x3;

		/*
		 * Clear command and release f/w
		 */
		*(unsigned long *)(addr + BF_CMD) = 0;

		/*
		 *        CARD MUST NOT BE ACCESSED FOR 12.5 usecs AS 
		 *                   ACCESS CODES MAY CHANGE
		 * (F/w is in a 10 instruction loop. 5 cycles per instruction,
		 * 3 clock cycles per instruction cycle. Thus 150 clocks.
		 * Max time for 150 clocks at 12 MHz = 12.5 * 10(-6), 
		 * 12.5 microseconds ).
		 * A call of delay(1) will greatly exceed the required delay.
		 */
		delay(1);

		/*
		 * Now wait for a status of LM_TEST and then do location 
		 * monitor test.
		 */
		if ( sccl_fw_swait(addr,LM_TEST) != (CHK_32|LM_TEST) ) {
			/*
		         * Not LM_TEST status !
			 */
			u.u_error = EIO;
			sccl_error = SE_LMTESTW;
			cmn_err(CE_CONT,
			   "!scclioctl: SCCL_FWINIT status not LM_TEST\n");
			break;
		}

		/*
		 * Access cards location monitor
		 */
		*(unsigned long *)(addr + LM) = 0;

		/*
		 * Complete location monitor test and then do handshake
		 * Do a short delay to allow firmware to setup response.
		 */
		delay(1); 

		if ( sccl_fw_swait(addr,LM_TEST_DONE) != 
						(CHK_32|LM_TEST_DONE) ) {
			/*
		         * Not LM_TEST_DONE status !
			 */
			u.u_error = EIO;
			sccl_error = SE_LMTSTDW;
			cmn_err(CE_CONT,
			   "!scclioctl: SCCL_FWINIT status not LM_TEST_DONE\n");
			break;
		}

		/*
		 * Acknowledge and complete handshake
		 */
		*(unsigned long *)(addr + BF_CMD) = 0;

		/*
		 * Now to get all addresses for this 'index' into scc_duart[]
		 */

		ptr = (unsigned long *)(addr+FSBP);
		j  = *ptr;
#ifdef	DBUG
		cmn_err(CE_CONT,"scclioctl: FSBP@ 0x%x\n",j);
#endif
		for (s = 0; s < PORTS_PER_SCC; s++) {
			ptr = (unsigned long *)(UL(j) + UL(addr) + UL(s*4));
#ifdef	DBUG
			cmn_err(CE_CONT,"scclioctl: %d ptr 0x%x\n",s,ptr);
#endif
			i = *ptr;
			scc_duart[(index*PORTS_PER_SCC)+s] = 
				(struct duart_channel *)(UL(addr) + UL(i));
			scc_ctrl[(index*PORTS_PER_SCC)+s].scc_chan_present =
						TRUE;
#ifdef	DBUG
			cmn_err(CE_CONT,
"scclioctl: %d 0x%x\n",s,scc_duart[(index*PORTS_PER_SCC)+s]);
#endif
			
		}

		/*
		 * Now wait for ready status from firmware server task. Do a
		 * short delay first so that firmware should be ready
		 */
		delay(4);

		if ( sccl_fw_swait(addr,SIO_READY) != (CHK_32|SIO_READY) ) {
			/*
		         * Not SIO_READY status !
			 */
			u.u_error = EIO;
			sccl_error = SE_SIOREADYW;
			cmn_err(CE_CONT,
			   "!scclioctl: SCCL_FWINIT status not SIO_READY\n");
			break;
		}

		break;

		/*  */
	case SCCL_LDINIT:
		/*
	 	 * SCCL_LDINIT: Prepare for a memory load over the VMEbus.
		 *
		 * root only access
		 */
		if (!suser()) {
			u.u_error = EPERM;
			break;
		}

		/*
		 * First get the card index word into index
		 */
		if ( (addr = sccl_get_addr(arg,&index)) == UCP(-1) )
			break;

		/*
		 * Wait for loader status from card (0x62AE0001)
		 */
		if (sccl_load_wait(index,addr)) {
			/*
			 * load_wait failed, errno and index set
			 */
			if (copyout((caddr_t)&index,arg,sizeof(index))) 
				u.u_error = EFAULT;
			break;
		}

		/*
		 * Have status of 0x62AE0001, do hand shake
		 */
		*(unsigned long *)(addr + 0x480) = 0x62AE0003;

		/*
		 * Short pause for firmware ...
		 */
		delay(1);
		
		/*
		 * Get status MUST be 62AE0002
		 */
		sdata = *(unsigned long *)(addr + STATUS);

		if (sdata != 0x62AE0002) {
			u.u_error = EIO;
			sccl_error = SE_LDINIT;
			break;
		}

		/*
		 * All done return to user. Now ready for SCCL_LOADFW call
		 * Firmware looking for none zero value in 0x484, addr 0x488
		 */
		break;

		/*  */	
	case SCCL_LOADFW:
		/*
	 	 * SCCL_LOADFW: Load S-record file to card over the VMEbus.
		 *
		 * root only access
		 */
		if (!suser()) {
			u.u_error = EPERM;
			break;
		}

		if (copyin(arg, (caddr_t)&record, sizeof(struct load_info))) {
			/*
			 * Failed to copy data from user to kernel space
			 */
			u.u_error = EFAULT;
			break;
		}

		/*
		 * Check that record.index is a valid SCC address
		 */
		if ((record.index >= (scc_cnt/PORTS_PER_SCC)) || 
				(record.index < 0)) {
			/*
			 * Card index out of range for this kernel build
			 */
			u.u_error = EIO;
			sccl_error = SE_INVCARD;
			break;
		}

		/*
		 * Get the address of the card and convert it into an unsigned
		 * character pointer. Check that the address if not -1. If
		 * it is then no card is present in at the index specified
		 */	
		if ( (addr = UCP(scc_addr[record.index])) == UCP(-1)) {
			u.u_error = EIO;
			sccl_error = SE_NOCARD;
			break;
		}
		
		/*
		 * Get status MUST be 62AE0002, else abort
		 */
		if (*(unsigned long *)(addr + STATUS) != 0x62AE0002) {
			u.u_error = EIO;
			sccl_error = SE_BADLDSTAT;
			break;
		}

		switch (record.type) {

		case 7:
		case 8:
		case 9:
			*(unsigned long *)(addr+GO_ADDR) = 
							record.offset;
			*(unsigned short*)(addr+GO_QUALIFIER) = 1;
			break;
		case 1:
		case 2:
		case 3:
			if (copyin(record.data,ddata,record.data_sz)) {
				u.u_error = EFAULT;
				break;
			}
			data = (char *)(addr + record.offset);

			for ( j = 0; j < record.data_sz; j++) {
				switch (ddata[j]) {
				case '0': digits[j] = 0; break;
				case '1': digits[j] = 1; break;
				case '2': digits[j] = 2; break;
				case '3': digits[j] = 3; break;
				case '4': digits[j] = 4; break;
				case '5': digits[j] = 5; break;
				case '6': digits[j] = 6; break;
				case '7': digits[j] = 7; break;
				case '8': digits[j] = 8; break;
				case '9': digits[j] = 9; break;
				case 'a': 
				case 'A': digits[j] = 10; break;
				case 'b':
				case 'B': digits[j] = 11; break;
				case 'c':
				case 'C': digits[j] = 12; break;
				case 'd':
				case 'D': digits[j] = 13; break;
				case 'e':
				case 'E': digits[j] = 14; break;
				case 'f':
				case 'F': digits[j] = 15; break;
				default :
					  u.u_error = EINVAL;
				       	  break;
				}	/* end switch (ddata[j]) */
			}
			for (j = 0; j < record.data_sz; j++)
				*data++ = (char) ((digits[j++] * 16)
				   		+ (digits[j]));
			break;
		default:
			u.u_error = EINVAL;
			break;
		}	/* end switch (record.type) */
		break;
	
	case SCCL_ADRS:
		if (copyin(arg,(caddr_t)&card_data,sizeof(struct card_data))) {
			u.u_error = EFAULT;
			break;
		}
		
		if ( (card_data.index >= (scc_cnt/PORTS_PER_SCC)) ||
				(card_data.index < 0) ) {
			u.u_error = ENXIO;
			card_data.adrs   = UCP(scc_card_max);
			card_data.data = (unsigned long)scc_cnt;
		}
		else if ( (card_data.adrs = UCP(scc_addr[card_data.index])) !=
						UCP(-1) ) {
			/*
			 * Load data as well
			 */
			card_data.data =
				*(unsigned long *)(card_data.adrs + STATUS);
		}
		else
			card_data.data = -1;

		if (copyout((caddr_t)&card_data,arg,sizeof(struct card_data))) {
			u.u_error = EFAULT;
			break;
		}

		break;
/**/
	case SCCL_SCCS:
		/*
		 * Use the status long word of the card_data structure to 
		 * return the Issue number. Checks that index and adrs are
		 * valid.
		 */
		if (copyin(arg,(caddr_t)&card_data,sizeof(struct card_data))) {
			u.u_error = EFAULT;
			break;
		}
		
		if ( (card_data.index >= (scc_cnt/PORTS_PER_SCC)) ||
				(card_data.index < 0) ) {
			u.u_error = ENXIO;
		}
		else if ( card_data.adrs != UCP(scc_addr[card_data.index]) ) {
			/*
			 * Index and adrs don't match
			 */
			u.u_error = ENXIO;
			break;
		}
		else if (card_data.adrs == UCP(-1)) {
			/*
			 * Asked for data on a missing card !
			 */
			u.u_error = ENXIO;
			break;
		}
		else 
			card_data.data =
				*(unsigned long *)(card_data.adrs + SCCS);

		if (copyout((caddr_t)&card_data,arg,sizeof(struct card_data))) {
			u.u_error = EFAULT;
			break;
		}

		break;
/**/
	case SCCL_LONG:
		/*
		 * Use the status long word of the card_data structure to 
		 * return the long data word. Checks that index and adrs are
		 * valid.
		 */
		if (copyin(arg,(caddr_t)&card_data,sizeof(struct card_data))) {
			u.u_error = EFAULT;
			break;
		}
		
		if ( (card_data.index >= (scc_cnt/PORTS_PER_SCC)) ||
				(card_data.index < 0) ) {
			u.u_error = ENXIO;
			break;
		}
		else {
			addr = UCP(scc_addr[card_data.index]);
		
			if ( (card_data.adrs < addr)
		       		|| (card_data.adrs > addr + 0xFFFFF) ) {
				/*
			 	 * Index and adrs don't match (not in range)
				 */
				u.u_error = ENXIO;
				break;
			}
			else 
				card_data.data =
					*(unsigned long *)(card_data.adrs);

			if (copyout((caddr_t)&card_data,arg,
						sizeof(struct card_data))) {
				u.u_error = EFAULT;
				break;
			}
		}
		break;
/**/
	case SCCL_PROMLOAD:
		/*
		 * SCCL_PROMLOAD: Load firmware from EPROM.
		 *
		 * root only access
		 */
		if (!suser()) {
			u.u_error = EPERM;
			break;
		}

		/*
		 * Get card address
		 */
		if ( (addr = sccl_get_addr(arg,&index)) == UCP(-1) )
			break;
		/*
		 * Wait for loader status from card (0x62AE0001)
		 */
		if (sccl_load_wait(index,addr)) {
			/*
			 * load_wait failed, errno and sccl_error 
			 */
			break;
		}

		/*
		 * Have loader waiting status, issue prom load and go
		 * command (0x62AE0005)
		 */
		*(unsigned long *)(addr + 0x480) = 0x62AE0005;

		break;
	
	case SCCL_SWRESET:
		/*
		 * root only access
		 */
		if (!suser()) {
			u.u_error = EPERM;
			break;
		}

		/*
		 * First get the card index word into index
		 */

		if ( (addr = sccl_get_addr(arg,&index)) == UCP(-1) )
			break;

		/*
		 * Issue a software reset command. (ASCII 'K'I'L'L)
		 */
		*(unsigned long *)(addr + SRESET) = 0x4B494C4C;

		/*
		 * Delay 1 second and check status
		 */
		delay(60);
		sdata = *(unsigned long *)(addr + STATUS);
		if ((sdata & 0xFFFF0FFF) == 0x62AE0001) {
			/*
			 * Now clear out the scc_ctrl structure
			 */
			j = index * PORTS_PER_SCC;
			for (i = 0; i < PORTS_PER_SCC; i++) {
				scc_ctrl[j+i].scc_chan_present = FALSE;
				scc_ctrl[j+i].scc_cmd_wait     = FALSE;
			}
			cmn_err(CE_CONT,
			"!scclioctl: SCCL_SWRESET(1) complete card %d\n",
									index);
			break;
		}

		/*
		 * Delay 4 second and check status
		 */
		delay(240);
		sdata = *(unsigned long *)(addr + STATUS);
		if ((sdata & 0xFFFF0FFF) == 0x62AE0001) {
			/*
			 * Now clear out the scc_ctrl structure
			 */
			j = index * PORTS_PER_SCC;
			for (i = 0; i < PORTS_PER_SCC; i++) {
				scc_ctrl[j+i].scc_chan_present = FALSE;
				scc_ctrl[j+i].scc_cmd_wait     = FALSE;
			}
			cmn_err(CE_CONT,
			"!scclioctl: SCCL_SWRESET(4) complete card %d\n",
									index);
			break;
		}
		
		/*
		 * Reset failed
		 */
		sccl_error = SE_FWNOTLD;
		u.u_error = EIO;
		break;

/**/	
	case SCCL_GERROR:
		/*
		 * Use the status long word of the card_data structure to 
		 * return the error number. Checks that index and adrs are
		 * valid.
		 */
		if (copyin(arg,(caddr_t)&card_data,sizeof(struct card_data))) {
			u.u_error = EFAULT;
			break;
		}
		
		if ( (card_data.index >= (scc_cnt/PORTS_PER_SCC)) ||
				(card_data.index < 0) ) {
			u.u_error = ENXIO;
		}
		else if ( card_data.adrs != UCP(scc_addr[card_data.index]) ) {
			/*
			 * Index and adrs don't match
			 */
			u.u_error = ENXIO;
			break;
		}
		else if (card_data.adrs == UCP(-1)) {
			/*
			 * Asked for data on a missing card !
			 */
			u.u_error = ENXIO;
			break;
		}
		else 
			card_data.data =
				*(unsigned long *)(card_data.adrs + GERROR);

		if (copyout((caddr_t)&card_data,arg,sizeof(struct card_data))) {
			u.u_error = EFAULT;
			break;
		}

		break;

/**/	
	case SCCL_ERROR:
		if (copyout((caddr_t)&sccl_error,arg,sizeof(long))) {
			u.u_error = EFAULT;
			break;
		}

		break;

	case SCCL_LOOP:
		/*
		 * root only access
		 */
		if (!suser()) {
			u.u_error = EPERM;
			break;
		}

		cmn_err(CE_CONT,"scclioctl: SCCL_LOOP entered\n");
		
		/*
		 * First get the card index word into index
		 */
		if (copyin(arg,(caddr_t)&index,sizeof(int))) {
			u.u_error = EFAULT;
			break;
		}

		sdata = index;

		/*
		 * index = <4><4><4><16><4> [delay,min,max,nops,index]
		 */
		index = index & 0x0000000F;

		if ((index >= (scc_cnt/PORTS_PER_SCC)) || (index < 0)) {
			u.u_error = EIO;
			sccl_error = SE_INVCARD;
			break;
		}
	
		if ( (addr = UCP(scc_addr[index])) == UCP(-1) ) {
			/* 
		 	* No card at this index !
		 	*/
			u.u_error = EIO;
			sccl_error = SE_NOCARD;
			break;
		}

		cmn_err(CE_CONT,"scclioctl: SCCL_LOOP index %d\n",index);
		for (s = 0; s < PORTS_PER_SCC; s++) {
			hin [s] = (struct fifo *)( UL(addr) +
				UL(scc_duart[(index*PORTS_PER_SCC)+s]->dc_hostinf));
			hout[s] = (struct fifo *)( UL(addr) +
				UL(scc_duart[(index*PORTS_PER_SCC)+s]->dc_hostoutf));
			cmn_err(CE_CONT,
				"scclioctl: hin[%d] 0x%x, hout[%d] 0x%x\n",
						s, hin[s], s, hout[s]);
		}
		cmn_err(CE_CONT,"scclioctl: SCCL_LOOP array loaded\n");
		cmn_err(CE_CONT,"scclioctl: SCCL_LOOP while address 0x%x\n",
							UL(addr)+0x2FF0);

		cmn_err(CE_CONT,"scclioctl: delay(%d)\n",
					((sdata & 0xF0000000)>>28));
		cmn_err(CE_CONT,"scclioctl: min %d\n",
					16*((sdata & 0x0F000000)>>24));
		cmn_err(CE_CONT,"scclioctl: max %d\n",
					16*((sdata & 0x00F00000)>>20));
		cmn_err(CE_CONT,"scclioctl: nop %d\n",
					2*((sdata & 0x000FFFF0)>>4));
		
		cmn_err(CE_CONT,"scclioctl: SCCL_LOOP delay(360)\n");
		delay(360);
		i = 0;
		cmn_err(CE_CONT,"scclioctl: SCCL_LOOP ** GO ** \n");

		while (*(unsigned long *)(addr + 0x2FF0)) {
			delay((sdata & 0xF0000000)>>28);
			for (s = 0; s < (PORTS_PER_SCC-1); s += 2) {

				/*
				 * Block input 
				 */
				hinptr = hin[s];
				houtptr = hout[s+1];
				if ( (n = fifo_entries(hinptr)) > 
						16*((sdata & 0x00F00000)>>20) )
					n = 16*((sdata & 0x00F00000)>>20);

				if ( (m = fifo_room(houtptr)) < n)
					n = m;
				
				if (n < 16*((sdata & 0x0F000000)>>24) )
					continue;

				nopsloop(2*((sdata & 0x000FFFF0)>>4));

				bfifoecho(hinptr,houtptr,n);
				i += (n*2);
			}
		}
		cmn_err(CE_CONT,"scclioctl: SCCL_LOOP chars %d\n",i);

		break;

	default:
		u.u_error = EINVAL;
		break;
	}
}

sccl_load_wait(card, addr)
register int card;
unsigned char *addr;
{
	int pass, card;
	unsigned long sdata, masked_sdata;

	pass = 0;
	while (++pass <= 20) {
		
		/*
		 * If not first pass do a delay of 1 seconds
		 */
		if (pass > 1)
			delay(60);
		
		/*
		 * Read the status word ... 
		 */
		sdata = *(unsigned long *)(addr + STATUS);
		masked_sdata = sdata & 0xFFFFF000;

		if (masked_sdata == 0x62AEF000) {
			/*
			 * Self test still running ....
			 */
			if (pass >= 10) {
				/*
				 * 10 seconds and still not ready. Error
				 */
				sccl_error = SE_FWSELFTEST;
				u.u_error = EIO;
				return(-1);
			}
		}
		else if (masked_sdata == 0x62AEE000) {
			/*
			 * Fatal error for now ...
			 */
			sccl_error = SE_FWFATAL;
			u.u_error = EIO;
			return(-1);
		}
		else if (masked_sdata == 0x62AE0000) {
			/*
			 * Say its OK, it is waiting in loaded ?
			 * If 0 loop, 1 OK, >1 error
			 */
			if ((sdata & 0xFFF) == 0x000) continue;
			if ((sdata & 0xFFF) == 0x001) return(0);
		
			sccl_error = SE_FWNOTLD;	
			u.u_error = EIO;
			return(-1);
		}
		else {
			/*
			 * Bad status word 
			 */
			u.u_error = EIO;
			sccl_error = SE_FWBADSTAT;
			cmn_err(CE_WARN,
			   "sccl(sccl_load_wait): Card %d bad SCC status 0x%x",
							card,sdata);
			return(-1);
		}
	}	/* end of while */
}

unsigned long sccl_fw_swait(addr,status)
register char *addr;
register int status;
{
	register int pass_cnt;
	register unsigned long sdata;

	/*
	 * Wait for a correct status from the firmware (status). Timeout
	 * wait after 10 seconds.
	 */
		
	pass_cnt = 0;

	/*
	 * Will wait a maximum of 10 seconds before aborting ....
	 */
	while (++pass_cnt <= 10) {
	
		/*
		 * If not first pass do a delay of 1 seconds
		 */
		if (pass_cnt > 1)
			delay(60);
	
		/*
		 * Read the status word ... 
		 */
		sdata = *(unsigned long *)(addr + STATUS);
	
		/*
		 * If we have status continue, else retry
		 */	
		if ( sdata == (CHK_32 | (status & 0xFFF)) ) 
			break;
	}

	return(sdata);
}

/**/
unsigned char *sccl_get_addr(arg,index)
int arg;
register int *index;
{
	register unsigned char *addr;

	if (copyin(arg,(caddr_t)index,sizeof(int))) {
		u.u_error = EFAULT;
		return(UCP(-1));
	}

	if ((*index >= (scc_cnt/PORTS_PER_SCC)) || (*index < 0)) {
		u.u_error = EIO;
		sccl_error = SE_INVCARD;
		return(UCP(-1));
	}
	
	if ( (addr = UCP(scc_addr[*index])) == UCP(-1) ) {
		/* 
		 * No card at this index !
		 */
		u.u_error = EIO;
		sccl_error = SE_NOCARD;
	}
	
	return(addr);
}

