#
#	@(#)scsi.m4	35.1
#
#	MVME147 assembly language interface macro file
#
#
#	pseudo-code:
#
#
#		call m147fwentry with address of packet
#
#		tos (top of stack): 
#		 -0	return address
#		 -4	address of packet
#
#		m147fwentry:
#				get address of packet from stack
#				set A2 to the address of the packet
#				jump to the firmware FUNNEL entry point
#			tos: 
#			 -0	saved register A2
#			 -4	return address of calling function
#			 -8	address of packet
#
#				>> firmware acts on packet
#				>> firmware jumps back to 
#				   m147fwreturn via vector table
#
#		m147fwreturn:
#				get status byte 0 from packet
#				if the status is RTS
#				begin
#				     tos: 
#					 -0	saved register A2
#					 -4	return address
#					 -8	address of packet
#
#					restore reg d0-d2/a0-a2
#					restore space to stack
#					restore register A2
#					do RTS
#				end
#				save old status
#				call driver for more work, pass address
#				     of packet as it will be put back 
#				     on the free queue if this is an RTE
#				get old status
#				if the status is RTE
#					do RTE processing
#			
#
define(`ASM',
`
	set	STAT0,0x02
	set	STAT2,0x18
	set	INTHAND,0x28
	set	SAVEREGA2,0x0400
	set	GETREGA2,0x0400
	set	SAVEFOUR, 0x0f00
	set	GETFOUR, 0x0f00
	set	PACKET, 0x40
	set	RTEBIT, 0x02
	set	FINAL, 0x00

	text

	global	m147fwentry

m147fwentry:

	# tos: return address
	#  4: address of packet
	# save all the registers
	movem.l &0xfffe, -(%sp)

	# tos: saved register A2
	#  4: saved register A6
	#  8: return address
	#  c: address of packet
	# get the address of the packet from the stack and put it in A2
	move.l	(PACKET,%sp), %a2

	# goto correct entry point
	jmp	SCSI_FUN
	

#
# This is the assembly code for the return of the scsi command.
# This is entered via an entry in the vector table and is jumped to by the
# scsi firmware. Register A2 is the address of the packet passed.
# Register A3 is the address of the registers to be restored.
#
	global	d147intr
	global	t147intr
d147intr:
t147intr:

	# save the status from the packet, as the packet might not be saved
	# if this is final status for a command
	# save status byte 0 and 1
	move.w	(STAT0,%a2), -(%sp)

	# save status byte 2 and 3
	move.w	(STAT2,%a2), -(%sp)

	# call the trace code in mvme147dk.c
	# pass the packet address
	move.l	%a2, -(%sp)
	jsr	intr_trc
	add.l	&4, %sp

	# do not do anything on intermediate
	bftst	(STAT0,%a2){&FINAL:&1}
	bne	nohand
	
	# save the address of the packet to be freed
	move.l	%a2, -(%sp)

	# call the driver interrupt processor based on the address in the packet
	jsr	([INTHAND,%a2])
	# remove address of packet from stack
	add.l	&4, %sp

nohand:
	# test status register 0 in the packet for RTS or RTE
	bftst	(STAT0,%sp){&RTEBIT:&1}
	bne	dorts

	add.l	&4, %sp		 # remove saved scsi status
	add.l	&4, %sp		# remove return address
	add.l	&4, %sp		# remove passed d0

	# restore d0-d7/a0-a6
	movem.l	(%a3), &0x7fff

	# get rid of old d0-d1/a0-a1
	add.l	&0x10, %a7

	# save d0,d1,a0,a1 for intret processing
	movem.l	&0xc0c0, -(%sp)
	
	
	# proceed with normal unix interrupt completion
	jmp	intret
	
		
	
dorts:
	# tos:
	#	0: return address in interrupt handler
	#	4: return address of caller
	#	8: register A2
	#	c: register A6
	# do not go back to the original interrupt handler d147int
	# remove scsi status
	add.l	&4, %sp
	# remove return address
	add.l	&4, %sp
	# remove d0 
	add.l	&4, %sp	
	# restore d0-d1/a0-a1 
	movem.l	(%sp)+, &0x0303 

	# restore m147fwentry saved registers
	movem.l (%sp)+, &0x7fff
	rts
') dnl 

ASM()
