/*		Copyright (c) 1986,1987 Motorola		
 *		All Rights Reserved		
 *
 *		sobppmgr.c - son of the buffered pipe protocol "bpp"
 *		@(#)sobppmgr.c	35.1
 */

#include "sys/types.h"
#include "sys/sobpp.h"

#define TOUT	2000 	/* length for timeout */
#define NULL	0

/*
 *	sobppcrqueue - create envelope and command packet queue
 */

sobppcrque(que,cnt,pp,ep,am,busw)
register struct sb_envelope **que;
register cnt;
register struct sb_packet *pp;
register struct sb_envelope *ep;
register unsigned char am;
register unsigned char busw;
{
	register unsigned short i;

	/*
	 *	place packets in envelopes
	 */

	for(i = 0; i < cnt; i++) {

		ep->e_link = *que;
		*que = ep;

		/* initialize packets */
		pp->p_am = am;
		pp->p_busw = busw;

		(ep++)->e_pkt = pp++;
	}
}

/*
 *	sobppcrchan - create channel
 */

sobppcrchan(cp,rp,que,ilev,ivec,am)
register struct sb_device *rp;
register struct sb_channel *cp;
register struct sb_envelope **que;
register unsigned char ilev;
register unsigned char ivec;
register unsigned char am;
{
	register tout;		/* timeout */
	register struct sb_envelope *ep;
	struct sb_envelope *sobppget();

	/* set up channel */

	/* put NULL envelopes on channel pipes */

	if((ep = sobppget(que)) == NULL)
		return(-1);
	ep->e_valid = INVALID;
	cp->c_cmdhead = cp->c_cmdtail = ep;

	if((ep = sobppget(que)) == NULL)
		return(-1);
	ep->e_valid = INVALID;
	cp->c_stathead = cp->c_stattail = ep;

	/* fill in rest of fields in channel */

	cp->c_level = ilev;
	cp->c_vector = ivec;
	cp->c_am = am;
	cp->c_valid = INVALID;

	/*
	 *	gain exclusive access to device
	 */

	for( tout = TOUT; tout && (rp->m_cr&SB_BSY); tout-- );
	if( tout <= 0 ) {
		return(-1);	/* error */
	}

	for( tout = TOUT; tout && (rp->m_tas&TASBIT); tout-- );
	if( tout <= 0 ) {
		return(-1);	/* error */
	}

	rp->m_tas |= TASBIT;	/* not safe like a real TAS */

	/*
	 *	set up register file
	 */

	rp->m_am = am;
	rp->m_addr = (unsigned)cp;
	rp->m_tas = (rp->m_tas&TASMASK) | VCMD | CREATE;

	/*
	 *	tell controller that command is ready
	 */

	rp->m_cr |= SB_ATTN;

	/*
	 *	wait for VSTAT bit
	 */

	for( tout=TOUT; tout && ((rp->m_tas&VSTAT) == 0); tout-- );

	if( tout <= 0 ) {
		return(-1);	/* error */
	}

	if( rp->m_sr ) {
		return(-1);	/* error */
	}

	/*
	 *	indicate command complete
	 */

	rp->m_tas |= CCOMP;

	/*
	 *	now tell controller
	 */

	rp->m_cr |= SB_ATTN;

	return(cp->c_chan);		/* return channel number */
}

/*
 *	sobppget - get free env from que
 */

struct sb_envelope *sobppget(que)
register struct sb_envelope **que;
{
	register struct sb_envelope *ep;

	if(*que == NULL)
		return(NULL);
	ep = *que;
	*que = ep->e_link;
	ep->e_link = NULL;
	return(ep);
}

/*
 *	sobppfree - free envelope and put on queue
 */

sobppfree(que,ep)
register struct sb_envelope **que;
register struct sb_envelope *ep;
{
	ep->e_link = *que;
	*que = ep;
}

/*
 *	sobppsget -   safe get of free env from que
 */

struct sb_envelope *sobppsget(que)
register struct sb_envelope **que;
{
	register struct sb_envelope *ep;
	register s;

	s = splhi();
	if(*que == NULL)
		return(NULL);
	ep = *que;
	*que = ep->e_link;
	ep->e_link = NULL;
	splx(s);
	return(ep);
}

/*
 *	sobppsfree - safe free envelope and put on queue
 */

sobppsfree(que,ep)
register struct sb_envelope **que;
register struct sb_envelope *ep;
{
	register s;

	s = splhi();
	ep->e_link = *que;
	*que = ep;
	splx(s);
}

/*
 *	sobppsend - send a packet to controller
 */

sobppsend(cp,dp,ep)
register struct sb_channel *cp;
register struct sb_envelope *ep;
register struct sb_device *dp;
{
	register struct sb_packet *pp;
	register struct sb_envelope *tp = cp->c_cmdtail;

	/* switch packet from envelope and on null envelope */

	pp = ep->e_pkt;
	ep->e_pkt = tp->e_pkt;
	tp->e_pkt = pp;

	/* put envelope on tail as null envelope */

	tp->e_link = ep;
	cp->c_cmdtail = ep;
	tp->e_valid = VALID;

	/* tell controller */

	dp->m_cr |= SB_ATTN;
}

/*
 *	sobpprcv - receive an envelope from a channel
 */

struct sb_envelope *sobpprcv(cp)
register struct sb_channel *cp;
{
	register struct sb_envelope *ep = cp->c_stathead;

	if( ep->e_valid == INVALID ) { /* spurious interrupt */
		return(NULL);
	}

	ep->e_valid = INVALID;

	/* set status head to next envelope */

	cp->c_stathead = ep->e_link;
	return(ep);
}
