
/*		Copyright (c) 1986, 1987, 1988  Motorola		*/
/*		All Rights Reserved				*/
/*		@(#)sxt332.c	35.1	*/
/*
 *	sxt332.c: Motorola Shell layer Driver for MVME332xt
 *
 *	The MVME332xt VMEmodule Communication Controller is an intelligent
 *	peripheral controller (IPC) VMEmodule designed to interface
 *	with the extended address line VMEmodules of the VMEbus family.
 *	The MVME332xt supports eight serial communication lines and one
 *	parallel printer port.
 *
 *
 */


#include "sys/types.h"
#include "sys/param.h"
#include "sys/sysmacros.h"

#ifndef REL2
#include "sys/immu.h"
#include "sys/region.h"
#define	 ISP
#define	 KERNEL		1
#define	 defined_io	1
#else
#include "sys/text.h"	
#endif

#include "sys/proc.h"
#include "sys/dir.h"
#include "sys/signal.h"
#include "sys/user.h"
#include "sys/inode.h"
#include "sys/elog.h"
#include "sys/file.h"
#include "sys/errno.h"
#include "sys/termio.h"
#include "sys/ttold.h"
#include "sys/tty.h"
#include "sys/sxt.h"
#include "sys/mvme332xt.h"

#define	TIOCEXCL	(('t'<<8)|13)		/* Should be in 'ttyold.h' */
#define	TIOCNXCL	(('t'<<8)|14)		/* Should be in 'ttyold.h' */

#define	S332TRACE				/* turn on debug mode */
#undef	S332TRACE				/* turn off debug mode */

#ifdef	S332TRACE
char	s332Debug = 0xff;		/* print osm msgs for s332Debug */
#define	DBUGPR(string, dev)\
	if (s332Debug)  printf("%s: link=%x, chan=%x, sxtcon=%x\n",\
			string, LINK(dev), CHAN(dev),\
			linkTable[LINK(dev)]->cb->sxtcon);

#define	DPRINT(string, value)\
	if (s332Debug)  printf(string, value);
#else
#define	DBUGPR(string, dev)
#define	DPRINT(string, value)
#endif
 
struct channel
{
	struct termio	tio;		/* Virtual termio structure */
};

/* modified structure of struct Link */
typedef	struct sxtlink
{
	struct cb	*cb;		/* pointer to mvme332xt port cb */
	struct m332_dev *dp;		/* pointer to mvme332xt base address */
	char		nchans;		/* Number of channels allowed */
	char		open;		/* Channel open bits */
	char		xopen;		/* Exclusive open bits */
	char		ioblock;	/* block channels on output */
	char		iblocked;	/* channels blocked for input */
	char		oblocked;	/* channels blocked for output*/
	dev_t		dev;		/* major and minor device # */
	struct channel	chans[1];	/* Array of channels for this link */
} SXTLINK;

extern	SXTLINK	*sxtalloc();
extern	SXTLINK	*linkTable[];
extern	int	m332major;		/* 332xt major device number */
extern	struct	m332_dev *m332_addr[];	/* 332xt base address pointer */
extern	struct	termio	 m332deftio;	/* 332xt default open termio */

/*
 * SXT332 open:
 */
s332open(dev, flag)
{
	register SXTLINK *lp;
	register int	chan;
	register int	bit;
	int	oldpri;

	/*  linkTable:  0 - unused, !0 - master operational */
	chan = CHAN(dev);
	if ((lp = linkTable[LINK(dev)]) == NULL) {
		if (major(u.u_ttyd) != m332major) { /* check for m332xt port */
			sxtopen(dev, flag);	    /* call normal sxt driver */
			return;
		}
		if (chan != 0) { /* Master on chan 0 must be opened first */
			u.u_error = EINVAL;
			return;
		}
		if ((lp = sxtalloc(MAXPCHAN)) == NULL) {
			u.u_error = ENOMEM;	/* No memory, try again */
			return;
		}
		lp->dev = u.u_ttyd;		/* save major/minor dev #s */
		lp->nchans = MAXPCHAN;
		lp->dp = m332_addr[CTL(lp->dev)]; /* save board address */
		lp->cb = &lp->dp->cb[UNIT(lp->dev)];/* save cb address */
		lp->cb->sxtcon = 0;		/* controlling channel is 0 */
		linkTable[LINK(dev)] = lp;	/* Now visible		*/
	}
	else {
		if (major(lp->dev) != m332major) { /* check for m332xt port */
			sxtopen(dev, flag);	    /* call normal sxt driver */
			return;
		}
		if (chan == 0) {
			u.u_error = EBUSY;
			return;
		}
	}

	DBUGPR("s332open", dev);
	if (chan >= lp->nchans) {	/* channel number out of range */
		u.u_error = ENXIO;
		return;
	}
	bit = (1 << chan);
	if (lp->open & bit) {
		if ((flag & FEXCL) || (lp->xopen & bit)) {
			u.u_error = ENXIO;
			return;
		}
	}
	else {				/* open new channel */
		lp->open |= bit;
		if (flag & FEXCL)
			lp->xopen |= bit;
		bcopy(&m332deftio, &lp->chans[chan].tio, sizeof(struct termio));
	}
	/* do the pertinent part of ttopen() here */ 
	oldpri = spltty();
	if ((u.u_procp->p_pid == u.u_procp->p_pgrp) &&
	    (u.u_ttyp == NULL) && (lp->cb->pgrp[chan] == 0)) {
		u.u_ttyp = &lp->cb->pgrp[chan];
		lp->cb->pgrp[chan] = u.u_procp->p_pgrp;
	}
	splx(oldpri);
}


/*
 * SXT332 close:
 */
s332close(dev, flag)
{
	register SXTLINK *lp;
	register int	bit;
	int	oldpri;
	int	i;

	if ((lp = linkTable[LINK(dev)]) == NULL)
		return;
	if (major(lp->dev) != m332major) {	/* check for m332xt port */
		sxtclose(dev, flag);		/* call normal sxt driver */
		return;
	}

	DBUGPR("s332close", dev);
	bit = (1 << CHAN(dev));
	lp->xopen &= ~bit;
	lp->open &= ~bit;

	oldpri = spltty();
	if (bit == 1) {					/* e.g. channel 0 */
		lp->cb->sxtcon = 0;
		/* don't clear the process group for channel 0 since
		 * the real driver needs it.
		 */
		for (i = 1; i < lp->nchans; i++)
			lp->cb->pgrp[i] = 0;
	}
	else
		lp->cb->pgrp[CHAN(dev)] = 0;

	if (lp->open == 0) {	/* no other opens on this set of virt ttys */
		linkTable[LINK(dev)] = NULL;
		sxtfree(lp);
	}
	splx(oldpri);
}


/*
 * SXT332 read:
 */
s332read(dev)
{
	register SXTLINK *lp;
	register int chan;
	int	oldpri;

	lp = linkTable[LINK(dev)];
	if (major(lp->dev) != m332major) {	/* check for m332xt port */
		sxtread(dev);			/* call normal sxt driver */
		return;
	}

	DBUGPR("s332read", dev);
	chan = CHAN(dev);
	oldpri = spltty();
	while (chan != lp->cb->sxtcon) {
		lp->iblocked |= (1 << chan);
		sleep(&lp->cb->pgrp[chan], TTOPRI);
	}
	lp->iblocked &= ~(1 << chan);
	splx(oldpri);
	m332read(lp->dev);		/* perform a real read */
}


/*
 * SXT332 write:
 */
s332write(dev)
{
	register SXTLINK *lp;
	register int chan;
	int	oldpri;

	lp = linkTable[LINK(dev)];
	if (major(lp->dev) != m332major) {	/* check for m332xt port */
		sxtwrite(dev);			/* call normal sxt driver */
		return;
	}
	
	DBUGPR("s332write", dev);
	chan = CHAN(dev);
	oldpri = spltty();
	while (lp->ioblock & (1 << chan) && chan != lp->cb->sxtcon) {
		lp->oblocked |= (1 << chan);
		sleep(&lp->cb->pgrp[chan], TTOPRI);
	}
	lp->oblocked &= ~(1 << chan);
	splx(oldpri);
	m332write(lp->dev);			/* perform a real write */
}


/*
 * SXT332 ioctl:
 */
s332ioctl(dev, cmd, arg, mode)
{
	register SXTLINK *lp;
	register struct cb *cb;
	register int chan;
	int	oldpri;

	lp = linkTable[LINK(dev)];
	if (major(lp->dev) != m332major) {	/* check for m332xt port */
		sxtioctl(dev, cmd, arg, mode);	/* call normal sxt driver */
		return;
	}

	DBUGPR("s332ioctl", dev);	DPRINT("cmd = %x\n", cmd);
	chan = CHAN(dev);
	cb = lp->cb;				/* pointer to control block */

	switch (cmd) {

	case SXTIOCLINK:
		if (arg > MAXPCHAN || arg < 1) {
			u.u_error = EINVAL;
			break;
		}
		/*
		 * Nothing else to do here since we've done everything
		 * in open().
		 */
		break;

	case SXTIOCSWTCH:
		/* 
		 * switch controlling to new device specified in "arg".
		 */
		if (!(1<<arg & lp->open)) {	/* it should be opened */
			u.u_error = EINVAL;
			break;
		}
		if (chan != 0) {		/* only controller can switch */
			u.u_error = EPERM;	/* windows */
			break;
		}

		DPRINT("switch arg=%x\n", arg);
		oldpri = spltty();
		if (cb->sxtcon != arg) {
			cb->sxtcon = arg;
			/* download new termio to the real device */
			m332ioctl(lp->dev, TCSETAS, &lp->chans[arg].tio, 0);
			/*
			 * Send a read packet just in case a process is 
			 * sleeping on read.
			 */
			if (!(cb->stat & R_PENDING)) {
				cb->stat |= R_PENDING;
				bppsend(&lp->dp->channel, &lp->dp->ipc,
					 	cb->read_env);
			}
		}
		splx(oldpri);
		wakeup(&cb->pgrp[arg]);
		break;

	case SXTIOCWF:
		/*
		 * wait til chan arg is in foreground and
		 * then return
		 */

		DPRINT("wait_for_fg arg=%x\n", arg);
		if (!(1<<arg & lp->open)) {
			u.u_error = EINVAL;
			break;
		}
		if (cb->sxtcon == arg)		/* nothing to be done */
			break;

		oldpri = spltty();
		while (cb->sxtcon != arg) {
			sleep(&cb->pgrp[arg], TTOPRI);
		}
		splx(oldpri);
		break;

	case SXTIOCBLK:
		/*
		 *  set LOBLK in indicated window
		 */
		if (!(1<<arg & lp->open)) {
			u.u_error = EINVAL;
			break;
		}
		lp->ioblock |= (1 << arg);
		break;

	case SXTIOCUBLK:
		/*
		 *  unset LOBLK in indicated window
		 */
		if (!(1<<arg & lp->open) || (arg == 0)) {
			u.u_error = EINVAL;
			break;
		}
		lp->ioblock &= ~(1 << arg);
		wakeup(&cb->pgrp[arg]);
		break;

	case SXTIOCSTAT:
		/*
		 *  return bit map of blocked channels to user
		 */
		{
			struct sxtblock sb;

			sb.input = lp->iblocked;
			sb.output = lp->oblocked;
			if (copyout(&sb, arg, sizeof sb)) {
				u.u_error = EFAULT;
				break;
			}
		}
		break;

#ifdef	S332TRACE
	case SXTIOCTRACE:
		s332Debug = 0xff;
		break;

	case SXTIOCNOTRACE:
		s332Debug = 0;
		break;
#endif
	case TIOCEXCL:
		lp->xopen |= (1 << chan);
		break;

	case TIOCNXCL:
		lp->xopen &= ~(1 << chan);
		break;

	case TCGETA:		/* get current termio */
		if (copyout(&lp->chans[chan].tio, arg, sizeof(struct termio)))
			u.u_error = EFAULT;
		break;

	case TCSETA:
	case TCSETAW:
	case TCSETAF:		/* set to new termio */
		if (copyin(arg, &lp->chans[chan].tio, sizeof(struct termio))) {
			u.u_error = EFAULT;
			break;
		}
		/* perform a real ioctl on the controlling channel */
		if (chan == cb->sxtcon)
			m332ioctl(lp->dev, TCSETAS, &lp->chans[chan].tio, mode);
		break;

	case TIOCGETP:		/* get current sgttyb */
		{
			struct sgttyb sg;

			/* convert termio to sgttyb */
			tio2sgt(&lp->chans[chan].tio, &sg);
			if (copyout(&sg, arg, sizeof(sg)))
				u.u_error = EFAULT;
		}
		break;

	case TIOCSETP:		/* set to new sgttyb */
		{
			struct sgttyb sg;

			if (copyin(arg, &sg, sizeof(sg))) {
				u.u_error = EFAULT;
				break;
			}
			/* convert sgttyb to termio */
			sgt2tio(&sg, &lp->chans[chan].tio);
		}
		/* perform a real ioctl on the controlling channel */
		if (chan == cb->sxtcon)
			m332ioctl(lp->dev, TCSETAS, &lp->chans[chan].tio, mode);
		break;

	case TCSBRK:
	case TCXONC:
	case TCFLSH:
		/* ignore all these commands if not a controlling channel */
		if (chan == lp->cb->sxtcon)
			m332ioctl(lp->dev, cmd, arg, mode);
		break;

	default:
		oldpri = spltty();
		while (lp->ioblock & (1 << chan) &&
		    chan != cb->sxtcon) {
			lp->oblocked |= (1 << chan);
			sleep(&cb->pgrp[chan], TTOPRI);
		}
		lp->oblocked &= ~(1 << chan);
		splx(oldpri);

		/* perform a real write */
		m332ioctl(lp->dev, cmd, arg, mode);
		break;
	}
}


/*
 * sgt2tio (&sgttyb, &termio):
 *	converts sgttyb structure to termio structure.
 */
sgt2tio(sgt, tio)
register struct sgttyb *sgt;
register struct termio *tio;
{
	register flag;

	tio->c_iflag = 0;
	tio->c_oflag = 0;
	tio->c_lflag = 0;
	tio->c_cflag = (sgt->sg_ispeed & CBAUD) | CREAD;
	if ((sgt->sg_ispeed & CBAUD) == B110)
		tio->c_cflag |= CSTOPB;
	tio->c_cc[VERASE] = sgt->sg_erase;
	tio->c_cc[VKILL] = sgt->sg_kill;
	flag = sgt->sg_flags;
	if (flag & O_HUPCL)
		tio->c_cflag |= HUPCL;
	if (flag & O_XTABS)
		tio->c_oflag |= TAB3;
	else if (flag & O_TBDELAY)
		tio->c_oflag |= TAB1;

	if (flag & O_LCASE) {
		tio->c_iflag |= IUCLC;
		tio->c_oflag |= OLCUC;
		tio->c_lflag |= XCASE;
	}
	if (flag & O_ECHO)
		tio->c_lflag |= ECHO;
	if (!(flag & O_NOAL))
		tio->c_lflag |= ECHOK;

	if (flag & O_CRMOD) {
		tio->c_iflag |= ICRNL;
		tio->c_oflag |= ONLCR;
		if (flag & O_CR1)
			tio->c_oflag |= CR1;
		if (flag & O_CR2)
			tio->c_oflag |= ONOCR|CR2;
	}
	else {
		tio->c_oflag |= ONLRET;
		if (flag & O_NL1)
			tio->c_oflag |= CR1;
		if (flag & O_NL2)
			tio->c_oflag |= CR2;
	}

	if (flag & O_RAW) {
		tio->c_cc[VTIME] = 1;
		tio->c_cc[VMIN] = 6;
		tio->c_iflag &= ~(ICRNL|IUCLC);
		tio->c_cflag |= CS8;
	}
	else {
		tio->c_cc[VEOF] = CEOF;
		tio->c_cc[VEOL] = 0;
		tio->c_cc[VEOL2] = 0;
		tio->c_iflag |= BRKINT|IGNPAR|ISTRIP|IXON|IXANY;
		tio->c_oflag |= OPOST;
		tio->c_cflag |= CS7|PARENB;
		tio->c_lflag |= ICANON|ISIG;
	}
	tio->c_iflag |= INPCK;
	if (flag & O_ODDP)
		if (flag & O_EVENP)
			tio->c_iflag &= ~INPCK;
		else
			tio->c_cflag |= PARODD;
	if (flag & O_VTDELAY)
		tio->c_oflag |= FFDLY;
	if (flag & O_BSDELAY)
		tio->c_oflag |= BSDLY;
}

/*
 * tio2sgt (&termio, &sgttyb):
 *	converts termio structure to sgttyb structure.
 */
tio2sgt(tio, sgt)
register struct termio *tio;
register struct sgttyb *sgt;
{
	register flag;

	sgt->sg_ispeed = tio->c_cflag & CBAUD;
	sgt->sg_ospeed = sgt->sg_ispeed;
	sgt->sg_erase = tio->c_cc[VERASE];
	sgt->sg_kill = tio->c_cc[VKILL];
	flag = 0;
	if (tio->c_cflag & HUPCL)
		flag |= O_HUPCL;
	if (!(tio->c_lflag & ICANON))
		flag |= O_RAW;
	if (tio->c_lflag & XCASE)
		flag |= O_LCASE;
	if (tio->c_lflag & ECHO)
		flag |= O_ECHO;
	if (!(tio->c_lflag & ECHOK))
		flag |= O_NOAL;

	if (tio->c_cflag & PARODD)
		flag |= O_ODDP;
	else if (tio->c_iflag & INPCK)
		flag |= O_EVENP;
	else
		flag |= O_ODDP|O_EVENP;

	if (tio->c_oflag & ONLCR) {
		flag |= O_CRMOD;
		if (tio->c_oflag & CR1)
			flag |= O_CR1;
		if (tio->c_oflag & CR2)
			flag |= O_CR2;
	}
	else {
		if (tio->c_oflag & CR1)
			flag |= O_NL1;
		if (tio->c_oflag & CR2)
			flag |= O_NL2;
	}
	if ((tio->c_oflag & TABDLY) == TAB3)
		flag |= O_XTABS;
	else if (tio->c_oflag & TAB1)
		flag |= O_TBDELAY;

	if (tio->c_oflag & FFDLY)
		flag |= O_VTDELAY;
	if (tio->c_oflag & BSDLY)
		flag |= O_BSDELAY;
	sgt->sg_flags = flag;
}

