#		Copyright (c) 1985 AT&T		#
#		All Rights Reserved		#
#		@(#)cswitch.s	35.6

# MC68020 version
# This file contains the routines used for context switching on UNIX V/68.
# The 1.1 level was derived from the 68000 cswitch.s level 1.3.

#	The routines contained here are:

	global	setjmp		# set up environment for external goto
	global	longjmp		# non-local goto
	global	save		# saves register sets
	global	resume		# restores register sets & u, then longjmps

	set	USIZE,4096	# number of bytes in the static u area
	set	HIGHPRI,0x3700	# supervisor state priority 7
	set	REGMASK,0xfefc	# save/restore a7-a1,d7-d2
	set	RESMASK,0x7efc	# restore a6-a1,d7-d2
	set 	MSB, 0x1000	# Master Stack bit of SR
	set	SP,48		# offset into savearea of SP


# setjmp (save_area)
# save (save_area)
#	saves registers and return location in save_area


	text
setjmp:
save:
	mov.l	(%sp)+,%a1	# pop return address of calling routine into %a1
	mov.l	(%sp),%a0	# address of save area
	movm.l	&REGMASK,(%a0)	# save a7-a1,d7-d2
	mov.l	&0,%d0		# return(0)
	jmp	(%a1)


# longjmp (save_area)
#	resets registers from save_area and returns to the location
#	from WHERE setjmp() WAS CALLED.
#	** Never returns 0 - this lets the returned-to environment know that the
#	return was the result of a long jump.

longjmp:
	mov.l	4(%sp),%a0	# get address of save_area
	movm.l	(%a0),&REGMASK	# restore a7-a1,d7-d2
	mov.l	&1,%d0		# return(1)
	jmp	(%a1)



# resume (&u, save_area)
#	switches to another process's "u" area. Returns non-zero.

resume:

#
	tst.w	do_fpu		# Check for fpu on system.
	beq.b	no_fpu1

	mov.l	fpu_ptr,%a0
	btst	&0,3(%a0)	# CHECK FOR UFPRWRT
	bne.b	no_fpu1

	fsave   fp_fsave(%a0)   #  Save internal state.
	tst.b   fp_fsave(%a0)   #  Check for null internal state.
	beq.b	no_fpu1

	fmovm.x &allfpi,fp_regs(%a0) # Save floating point registers.
	fmovm.l %control,%status,%iaddr,fp_contr(%a0)   # and system regs.

no_fpu1:
	mov.l	4(%sp),%a1	# address of new "u" areas PTEs
	mov.l	mmuupde,%a2	# address of global "u" areas PTEs
	mov.l	8(%sp),%a0	# address of save_area
	mov.w	%sr,%d0		# save current PS
	mov.w	&HIGHPRI-MSB,%sr	# inhibit interrupts
	# using interrupt stack to prevent a level seven from profiling
	# from putting a throwaway stack frame on the Master Stack
	set	RP,0		# Root Pointer register
	set	MMUADDR,0x10000	# in CPU space

	mov.l	(%a1)+,(%a2)+	# get the users valid PTE
	mov.l	(%a1)+,(%a2)+	# ditto (4 descrs for 4K Uvec)
	mov.l	(%a1)+,(%a2)+	# ditto (4 descrs for 4K Uvec)
	mov.l	(%a1)+,(%a2)+	# ditto (4 descrs for 4K Uvec)

	tst.w	do_pmmu
	bne.b	pmmuflsh
# this is MMB code!
	mov.l	&RP+MMUADDR,%a2	# flush the ATC by re-writing root pointer
	mov.l	%dfc,%a1
	mov.l	&7,%d1
	mov.l	%d1,%dfc
	mov.l	fcltbl,%d1
	movs.l	%d1,(%a2)
	mov.l	%a1,%dfc
	bra.b	pastpmmu
# end MMB
pmmuflsh:
	mov.l	fcltbl,rpstore+4
	nop
	short	0xf039
	short	0x4c00
	long	rpstore

	nop
	short	0xf000		# PMMU flush all instruction.
	short	0x2400
				# at this point, the new process
				# u-vector will magically appear at
				# location 0x1000000 to 0x1000000+USIZE-1

pastpmmu:
	mov.l	cacr,%a1	# Clear old cache entries.
	mov.l	%a1,%cacr

	movm.l	(%a0),&RESMASK	# restore previous a6-a1,d7-d2
	mov.l	12*4(%a0),%d1 	# restore previous Master Stack Pointer
	mov.l	%d1,%msp 	#
# The master stack pointer is not the current stack because profile interrupt
# runs at level 7 and so we switch to the interrupt stack before entering 
# critical region where we don't want a throwaway frame to be put on the
# master stack when A7 is pointing to the old Ublock. So we load the MSP
# instead of A7 and then select the Master stack when we drop the mask.
	mov.w   %d0,%sr         # restore priority & switch to master stack.

	tst.w	do_fpu		# Check for fpu chip.
	beq.b	no_fpu2

	mov.l	fpu_ptr,%a0
	tst.b   fp_fsave(%a0)        # Was state null?
	beq.b	L%47

	fmovm.x fp_regs(%a0),&allfpi # Restore floating point regs.
	fmovm.l fp_contr(%a0),%control,%status,%iaddr        # and system regs.
	frestore        fp_fsave(%a0)
	and.l	&-2,(%a0)
	bra.b	L%49
L%47:
	frestore        fp_fsave(%a0)        #Restore
	btst	&0,3(%a0)	# Check UFPRWRT flag.
	beq.b	L%49

	fmovm.x fp_regs(%a0),&allfpi # Restore floating point regs.
	fmovm.l fp_contr(%a0),%control,%status,%iaddr        # and system regs.

	and.l	&-2,(%a0)
L%49:
no_fpu2:

	mov.l	&1,%d0		# return (1)
	jmp	(%a1)
