/*
 * Copyright (c) 1987, Motorola Microcomputer Division
 * All rights reserved.
 *
 * Definitions pertaining to the network interfaces.
 */
#ident	"@(#)if.h	35.2"

struct ifioctl {
	char name[8];		/* interface name	*/
	long addr;		/* internet address	*/
	unsigned short flags;	/* network flags	*/
	short mtu;		/* max transfer unit	*/
};

#define IIOC		('I' << 8)	/* interface ioctl()		*/
#define IIOG		(IIOC | (1<<7))	/* GET data			*/
#define IIOS		(IIOC | (0<<7))	/* SET data			*/
#define	IIOCSETIFADDR	(IIOS | 1)	/*	set interface address	*/
#define	IIOCGETIFADDR	(IIOG | 1)	/*	get interface address	*/
#define	IIOCSETIFFLAG	(IIOS | 2)	/*	set interface flags	*/
#define	IIOCGETIFFLAG	(IIOG | 2)	/*	get interface flags	*/
#define	IIOCSETIFMTU	(IIOS | 4)	/*	set interface mtu	*/
#define	IIOCGETIFMTU	(IIOG | 4)	/*	get interface mtu	*/
#define	IIOCSETIF	(IIOS | (1|2|4))/*	set interface data	*/
#define	IIOCGETIF	(IIOG | (1|2|4))/*	get interface data	*/
#define IIOCIOCTL	(IIOC | 8)	/* pass ioctl down to interface	*/
#define IIOCIFLIST	(IIOC | 9)	/* get list of interfaces	*/

/*
 * indirect ioctl structure
 */
struct ifindir {
	long	cmd;			/* io control command		*/
	long	bytes;			/* amt of data following	*/
};

/*
 * structure for IFI_ARP_ADD request
 */
struct ifi_arp_add {
	union {
		long	aad_inaddr;	/* internet address of host	*/
	} aad0;
	union {
		e_addr	aad_etaddr;	/* ethernet address of host	*/
	} aad1;
};
#define aa_inaddr	aad0.aad_inaddr
#define aa_etaddr	aad1.aad_etaddr

/*
 * structure for the IFI_ARP_DELETE request
 */
struct ifi_arp_delete {
	union {
		long	adel_inaddr;	/* internet address of host	*/
	} adel;
};
#define ad_inaddr	adel.adel_inaddr

/*
 * structure for the reply to the IFI_GET_STATS request
 *
 * This is for generalised stats reporting.  Interface device specific
 * statistics should be handled at the interface specific I/O control level.
 */
struct ifi_stats {
	long	ipkts;		/* input pkts	*/
	long	ierrs;		/* input errs	*/
	long	opkts;		/* output pkts	*/
	long	oerrs;		/* output errs	*/
};

/*
 * Some common indirect ioctl commands
 */
#define IFI_(x)		(((('*' << 8) | 'I') << 8) | (x))
#define	IFI_ARP_DUMP	IFI_(1)		/* dump entire table		*/
#define	IFI_ARP_CLEAR	IFI_(2)		/* clear entire table		*/
#define	IFI_ARP_ADD	IFI_(0)		/* add an entry			*/
#define IFI_ARP_DELETE	IFI_(3)		/* delete an entry		*/
#define	IFI_GET_HWADDR	IFI_(4)		/* get hardware address		*/
#define	IFI_SET_HWADDR	IFI_(5)		/* set hardware address		*/
#define	IFI_GET_STATS	IFI_(6)		/* get interface statistics	*/
#define	IFI_TRACE_TX	IFI_(7)		/* trace all transmitted	*/
#define	IFI_TRACE_RX	IFI_(8)		/* trace all received   	*/
#define	IFI_TRACE_OFF	IFI_(9)		/* trace off            	*/
#define IFI_TRACE_NOBCT IFI_(10)        /* don't trace broadcasts       */

/*
 * interface flags
 */
#define IF_UP		0x0001		/* interface up			*/
#define IF_ALLOC	0x0800		/* interface has own allocator	*/
#define IF_BCAST	0x1000		/* interface supports broadcast	*/
#define IF_ARP_SERVER	0x2000		/* provide ARP service		*/
#define IF_ARP		0x4000		/* interface uses ARP		*/
#define IF_LOOP		0x8000		/* interface is loop back	*/

/*
 * definition of M_PROTO messages used for inter module communication
 */
struct ip_proto {
	char type;		/* request/response type		*/
	union {
		short tidu;	/* least mtu				*/
	} u;
};

#define IP_ERROR	0	/* error acknowledgement		*/
#define IP_TIDU		1	/* request least MTU			*/

/*
 * The format of the argument prefix that heads the data packet
 * passed down to one of our network interfaces
 *
 * A peculiar thing is that it is at least as large as an ethernet address
 * (and potentially any other network interface header)
 *
 * since the IFARG structure is present at all protocol layers,
 * it provides a mechanism to pass args downwards between protocol layers.
 * Usually, this is done in the ifarg.flags.
 */
typedef struct ifarg {
	struct route *rt;		/* route to nexthop	*/
	short len;			/* pkt length		*/
	short flags;			/* misc flags		*/
	union {
		HOST addr;		/* nexthop for pkt	*/
		e_frame et_addr;	/* room for ether hdr	*/
	} _u;
} IFARG;
#define ifarg_addr	_u.addr
#define ifarg_ether	_u.et_addr

/* definitions for if to ip control */
typedef struct {
	caddr_t         cookie;        /* ip finder for interface */
	int             code;          /* code for control        */
} IFCTLBLK;

#define IF_ERROR_DOWN   1              /* interface is down       */
