#ident "@(#)icmp.h	35.2"

/*
 * Copyright (c) 1987, Motorola Microcomputer Division.  All rights reserved.
 * Definitions for ICMP protocol (RFC 792)
 */

/*
 * ICMP header	- every ICMP pkt has one of these
 */
typedef struct {
	unsigned char	type;
	unsigned char	code;
	unsigned short	chksum;
	union {
		ulong	_unused;		/* mostly unused	*/
		uchar	_pointer;		/* ^ to offending octet	*/
		HOST	_gateway;		/* Gateway Internet addr*/
		struct {
			ushort _id;		/* identifier		*/
			ushort _seq;		/* sequence #		*/
		} _id_seq;
	} lword1;				/* long word 1 union	*/
} ICMPHDR;

#define unused		lword1._unused
#define pointer		lword1._pointer
#define gateway		lword1._gateway
#define id		lword1._id_seq._id
#define seq		lword1._id_seq._seq

/*
 * ICMP data -	some ICMP pkts have these
 */
typedef struct {
	unsigned long	addr;			/* long word 2		*/
	union {
	    INHDR	r_in;
	    unsigned long	time[3];
	} data;
} ICMPDATA;

/*
 * general ICMP pkt structure
 */
typedef struct {
	ICMPHDR		ic_hdr;
	ICMPDATA	ic_data;
} ICMPPKT;

/*
 * Legal type field values
 */

#define EREPLY		0	/* echo reply			*/
#define UNREACH		3	/* destination unreachable	*/
#define QUENCH		4	/* source quench		*/
#define DIRECT		5	/* redirect			*/
#define ECHO		8	/* echo				*/
#define TIME		11	/* time exceeded		*/
#define PARAM		12	/* parameter problem		*/
#define TSTAMP		13	/* time stamp			*/
#define TREPLY		14	/* time stamp reply		*/
#define INFO		15	/* information request		*/
#define IREPLY		16	/* information reply		*/
#define AM1             17      /* subnet address request       */
#define AM2             18      /* subnet address response      */

/*
 * Legal code field values
 */

#define U_NET		0	/* net unreachable			*/
#define U_HOST		1	/* host unreachable			*/
#define U_PROT		2	/* protocol unreachable			*/
#define U_PORT		3	/* port unreachable			*/
#define U_FRAG		4	/* fragmentation needed and DF set	*/
#define U_ROUTE		5	/* source route failed			*/
#define D_NET		0	/* net redirect				*/
#define D_HOST		1	/* host redirect			*/
#define D_SNET		2	/* Type of service and net redirect	*/
#define D_SHOST		3	/* Type of service and host redirect	*/
#define T_LIVE		0	/* Time to live exceeded in transit	*/
#define T_FRAG		1	/* Fragment reassembly time exceeded	*/

/*
 * ICMP statistics
 */
struct icmp_stats {
	unsigned total;		/* total ICMP pkts received		*/
	struct {
		unsigned unreachable;
		unsigned redirect;
	} host, net;		/* host/net redirect/unreachable	*/
	unsigned ignore;	/* ICMP pkts ignored			*/
	unsigned quench;	/* source quench ignored		*/
	unsigned bad;		/* bad ICMP pkts			*/
	unsigned mem;		/* memory denials			*/
	unsigned frag_time;	/* fragmentation time exceeded		*/
	unsigned live_time;	/* ttl time exceeded			*/
#ifdef INKERNEL
	INHDR	lasti;		/* last input pkt's IP hdr		*/
	ICMPPKT	last;		/* last input pkt's ICMP hdr		*/
#endif
};

/*
 * ICMP TPImux options
 */
struct icmpopt {
	short	nothing;	/* no options to speak of		*/
};

/*
 * routing table entry format
 */
struct route_tab {
	HOST index;		/* internet address		*/
	HOST alternate;		/* different way to reach dest	*/
	int timestamp;		/* timeout for old routing info	*/
};
#define ROUTE_TSIZE	32	/* routing table size 			*/
#define DELAY		(30*HZ)	/* 30 second delay for ICMP timer	*/
#define RESET		50	/* RESET x DELAY = timeout for routing table */
#define PERMANENT	RESET+1	/* permanent table entry		*/
#define DELETE		0	/* cause entry to be delete from table	*/


/*
 * ICMP I/O controls;  these are performed on the IP control stream.
 */
#define IOCICMP(n)		(((((('i' << 8) | 'c') << 8) | 'm') << 8) | (n))
#define IOC_ADD_ROUTE		IOCICMP(0)	/* add route		*/
#define IOC_DEL_ROUTE		IOCICMP(1)	/* delete route		*/
#define IOC_GET_NET_TABLE	IOCICMP(2)	/* get net redirect tab	*/
#define IOC_GET_HOST_TABLE	IOCICMP(3)	/* get host redir tab	*/
#define IOC_CLEAR_NET_TABLE	IOCICMP(4)	/* clear net redirects	*/
#define IOC_CLEAR_HOST_TABLE	IOCICMP(5)	/* clear host redirects	*/
#define IOC_SET_GATEWAY		IOCICMP(6)	/* set gateway address	*/
#define IOC_GET_GATEWAY		IOCICMP(7)	/* get gateway address	*/
#define IOC_GET_ICMP_STATS	IOCICMP(8)	/* get ICMP stats	*/

/*
 * structure used for ADD_ROUTE and DELETE_ROUTE
 */
struct mod_route {
	HOST	net;			/* for network redirects	*/
	HOST	host;			/* for host redirects		*/
	HOST	alternate;		/* where to redirect to		*/
};
