/*
 * Copyright (c) 1987, Motorola Microcomputer Division.  All rights reserved.
 *
 * Definitions pertaining to the INTERNET
 */
#ident "@(#)ip.h	35.1"

#include "netinet/types.h"

/*
 * internet address manipulation
 */
#define _clsA(addr)		!((addr) & 0x80000000)
#define _clsB(addr)		!((addr) & 0x40000000)
#define netmask(a)	\
	(_clsA(a) ? 0xff000000			/* class A network	*/\
		  : (_clsB(a) ? 0xffff0000	/* class B network	*/\
			      : 0xffffff00))	/* class C network	*/
#define netfield(a)	((a) &  netmask(a))	/* isolate net  field	*/
#define hostfield(a)	((a) & ~netmask(a))	/* isolate host field	*/

#define is_local(addr)	(hostfield(addr) ? addr == in_whoami(addr) : 1)

/*
 * Internet header format - Standard 
 */
typedef struct inhdr {
/* 00 */	BYTE	i_ihl;		/* Version & header length	*/
/* 01 */	BYTE	i_tos;		/* Type-of-service		*/
/* 02 */	WORD	i_tlen;		/* Total length			*/
/* 04 */	WORD	i_fragid;	/* Fragment Id			*/
/* 06 */	WORD	i_fragoff;	/* Fragment flags & offset	*/
/* 08 */	BYTE	i_ttl;		/* Time to live			*/
/* 09 */	BYTE	i_proto;	/* Next level protocol		*/
/* 10 */	WORD	i_chksum;	/* Header's checksum		*/
/* 12 */	HOST	i_shost;	/* Source host & net		*/
/* 16 */	HOST	i_dhost;	/* Dest.  host & net		*/
} INHDR;

# define	MAXPKT	1088	/* Max. ip packet := 1024 data + 64 hdr   */

/*
 * pseudo header format
 * The pseudo header os conceptually prefixed to the UDPHDR, for the
 * purposes of chksum calculation.
 */
typedef struct {
/*  0 */	HOST	shost;		/* source address		*/
/*  4 */	HOST	dhost;		/* destination address		*/
/*  8 */	BYTE	zero;		/* must be zero			*/
/*  9 */	BYTE	proto;		/* UDP protocol number		*/
/* 10 */	WORD	len;		/* same length as above		*/
} PSEUDO;

/*
 * IP statistics
 */
struct ip_stats {
	long opkts;		/* pkts output				*/
	long ipkts;		/* pkts input				*/
	long upkts;		/* pkts transmitted for user		*/
	long gpkts;		/* packets routed through gateway	*/
	long if_flowctld;	/* # times interface flow ctld		*/
	long ofrag;		/* fragments generated			*/
	long ifrag;		/* fragments received			*/
	HOST unrdest;		/* last unreachable destination		*/
	short badproto;		/* pkts with bad proto field		*/
	short badver;		/* pkts with bad version no		*/
	short badsum;		/* network pkts with bad chksum		*/
	short unreach;		/* gateway pkts with unreachable dest	*/
	short ttl;		/* bad time to live status		*/
	short gbadlen;		/* gateway pkts dropped: too short	*/
};

#define	IPVERSION	4		/* Version of IP we conform to	  */
#define	NINPROTOS	10		/* number of higher level protos  */
#define	INPROTO_ICMP	1		/* internet control message proto */
#define	INPROTO_GGP	3		/* gateway-to-gateway proto	  */
#define	INPROTO_CMCC	4		/* gateway monitoring proto 	  */
#define INPROTO_TCP	6		/* transmission control proto	  */
#define INPROTO_UDP	17		/* user datagram proto	 	  */

/*
 * type of service defines 
 */
#define PRECEDENCE	07		/* precedence field */
#define STRM		010		/* stream mode */
#define RELIABILITY	060		/* kind of reliability requested */
#define SR		0100		/* speed vs reliability */
#define SPEED		0200		/* speed requested */

/*
 * fragmentation flags
 */
#define DONTFRAG	0x4000		/* dont fragment this packet	*/
#define MOREFRAG	0x2000		/* more fragments coming	*/
#define FRAGOFF		0x1fff		/* fragment offset		*/

/*
 * internet options
 */
typedef struct ioptn {
	uchar	io_type;		/* option type */
	uchar	io_len;			/* option length */
	uchar	*io_data;		/* option data */
} IOPTN;

/*
 * internet option types		all not presently handled
 */
#define IEOL		0		/* end of option list */
#define INOP		1		/* nop */
#define ISEC		2		/* security */
#define ISROUTE		3		/* source routing */
#define IRROUTE		7		/* return routing */
#define ISTRMID		010		/* stream id */
#define IGENERR		041		/* general error msg */
#define IINSTMP		0104		/* internet time stamp */
#define ISATSTMP	0105		/* satellite time stamp */

#define IOCDEBUG_IP	(('i' << 8) | 'p') << 8 | 0
#define IOCSETPROTO	(('i' << 8) | 'p') << 8 | 1
#define IOCIPSTATS	(('i' << 8) | 'p') << 8 | 2

/*
 * Internet 'reason' codes for TPI functions supplying reasons for events
 */
#define INET_CLOSED	1	/* normal close request caused disconnect */
#define INET_REFUSED	2	/* connexion refused			  */
#define INET_ABORT	3	/* connexion aborted			  */
#define INET_ERROR	4	/* unspecified error			  */
#define INET_TIMEOUT	5	/* connexion timed out			  */

/*
 * IP :: network interface  intermodule protocol
 */
struct ip_req {
	long ipr_request;	/* request				*/
	caddr_t ipr_cookie;	/* magic passed without interpretation	*/
	long ipr_flags;		/* interface flags			*/
	long ipr_addr;		/* internet address of interface	*/
};

struct ip_resp {
	long ipr_response;	/* response (same as request)		*/
	caddr_t ipr_cookie;	/* magic passed without interpretation	*/
	long ipr_flags;		/* interface flags			*/
	union {
		struct {
			char name[8];	/* interface device name	*/
			long mtu;	/* maximum transfer unit	*/
			struct msgb *(*alloc)();/* allocator		*/
		} info;
	} ipresp_U;
};
#define ipr_name	ipresp_U.info.name
#define ipr_mtu		ipresp_U.info.mtu
#define ipr_alloc	ipresp_U.info.alloc

/*
 * ip_req.request types
 */
#define IP_INFO		(('I' << 8 | 'P') << 8 | 'i')
#define IP_SETADDR	(('I' << 8 | 'P') << 8 | 'a')
#define IP_SETFLAG	(('I' << 8 | 'P') << 8 | 'f')

/*
 * flag definitions for IFARG.flags parameters passed from an upper level
 * protocol to IP.
 */
#define IP_ALLOWBCAST	0x0001		/* allow use of a broadcast addr */
