/*
 * Copyright (c) 1987, Motorola Microcomputer Division.  All rights reserved.
 */
#ident "@(#)tcp.h	35.1"

#include "netinet/tcp_opts.h"
#include "netinet/tcp_stats.h"

/*
 * retransmission buffer header 
 */
typedef struct rtxb {
	struct rtxb 	*rt_link;		/* next on the list	 */
	 short		 rt_time;		/* relative time to rtx	 */
	ushort		 rt_len;		/* flow size of rtx	 */
	uchar		 rt_flags;		/* flags		 */
	uchar		 rt_rtxcnt;		/* curr # of tries	 */
} RTXB;

#define RTXERR		10			/* num of rtx before err */

/*
 * Check sum pseudo header format
 */
typedef struct pseudosum {
	long	ps_shost;
	long	ps_dhost;
	char	ps_zero;
	char	ps_proto;
	short	ps_len;
} PSEUDOSUM;

/*
 * Transmission control block -- one per connection
 */
typedef struct tcp_tcb {
	struct tm_tcb tm;		/* common part of all TCB's	 */
	struct tcp_tcb *link;		/* state info			 */
	struct   route  rt;		/* route			 */
	mblk_t		*rtxq;		/* retransmission queue		 */
	mblk_t		*ordrq;		/* out of order queue 		 */
	mblk_t		*sndq_head;	/* send queue head		 */
	mblk_t		*sndq_tail;	/* send queue tail		 */
	unsigned long	suna;		/* send unacknowledged		 */
	unsigned long	snxt;		/* send next			 */
	unsigned long	sup;		/* send urgent pointer		 */
	unsigned long	swl_seq;	/* last seq of swnd update	 */
	unsigned long	swl_ack;	/* last ack of swnd update	 */
	unsigned long	sisn;		/* send initial seq #		 */
	unsigned long	rnxt;		/* rcv next seq			 */
	unsigned long	rup;		/* rcv urgent pointer		 */
	unsigned long	risn;		/* rcv initial seq #		 */
	unsigned long 	rttseq;		/* seq # of pkt used for rtt calc*/
	unsigned long	rtxtime;	/* tod until ack comes back	 */
	unsigned short	sndq_cc;	/* send queue char count	 */
	unsigned short	sndq_hiwat;	/* send queue high water mark	 */
	unsigned short	sndq_lowat;	/* send queue low  water mark	 */
	unsigned short	swnd;		/* send window			 */
	unsigned short 	rwnd;		/* rcv window			 */
	unsigned short 	last_rwnd;	/* last advertised rcv window	 */
	unsigned short  rurg_cnt;	/* bytes till end of urg	 */
	unsigned short  rtxmin;		/* min value for RTX timer	 */
	unsigned short  rtxmax;		/* max value for RTX timer	 */
		 short	rtxticks;	/* ticks until rtx		 */
	unsigned short	rtxbase;	/* refill for rtxticks		 */
	unsigned short	rtxrtt;		/* current pkt round trip	 */
	unsigned short	mss;		/* maximum segment size		 */
	unsigned short	ackticks;	/* ticks until ack		 */
	unsigned short	twaiticks;	/* ticks until timewait expires	 */
	unsigned short  usrticks;	/* ticks 'till user timeout	 */
	unsigned short  usrtime;	/* time  'till user timeout	 */
	unsigned short	keepaticks;	/* keep alive timer ticks	 */
	unsigned short	zwpticks;	/* ticks 'till next ZWP		 */
	unsigned char	keepa;		/* our keep alive counter	 */
	unsigned char 	rtxstop;	/* stop packetizing		 */
	unsigned char	sndwrk;		/* work to do flags		 */
	unsigned char	zwpcnt;		/* # of probes of this window	 */
	unsigned char	usract;		/* action to take on timeout	 */
	unsigned char	rtxerr;		/* max # of retransmits		 */
	unsigned char	state;		/* state of the connection	 */
	unsigned char	flags;		/* unspecified flags		 */
		 char	ntoack;		/* num pkts to next ack		 */
	unsigned char 	nackbase;	/* refresh for ntoack		 */
} TCB;

/*
 * work flags (TCB.sndwrk)
 */
#define FL_SYN		01			/* send a syn		*/
#define FL_FIN		02			/* send a fin		*/
#define FL_CTL		03			/* syn or fin		*/
#define FL_ACK		04			/* send an ack		*/

/*
 * connection states (TCB.state)
 */
#define CLOSED		00			/* not in use		*/
#define LISTEN		01			/* listening		*/
#define SYNRCVD		02			/* received a syn	*/
#define SYNSENT		03			/* sent a syn		*/
#define ESTAB		04			/* established		*/
#define CLSWAIT		05			/* close wait		*/
#define FINWAIT1	06			/* fin wait 1		*/
#define FINWAIT2	07			/* fin wait 2		*/
#define CLOSING		08			/* closing		*/
#define TIMEWAIT	09			/* time wait		*/
#define LASTACK		10			/* last ACK		*/

/*
 * flag field flags (TCB.flag)
 */
#define FL_AWAIT_ACCEPT	0001			/* accept pending	*/
#define FL_AUTOEOL	0002			/* snd eol with iorb	*/
#define FL_AUTOPSH	0002			/* snd PUSH with iorb	*/
#define FL_UH		0004			/* host unspecified	*/
#define FL_UP		0010			/* port unspecified	*/
#define FL_FIN_ACKED	0020			/* our FIN has been ACKd*/
#define FL_FIN_RCVD	0040			/* received a FIN	*/
#define FL_URG		0100			/* in URGENT mode	*/
#define FL_FLOW_CTRLD	0200			/* write side flow ctrld*/

/*
 * misc
 */
#define RTXHZ		4			/* HZ for RTX clock	*/
#define KEEPALIVE_TIME	(45 * RTXHZ)		/* keep alive timer	*/
#define KEEPALIVE_MAX	5			/* maximum keep alive	*/
#define ZWP_TIME	(1 * 60 * RTXHZ)	/* ZW probe interval	*/
#define RTXDLY		(30 * RTXHZ)		/* default RTX delay	*/
#define RTXUBOUND	(1 * 60 * RTXHZ)	/* RTX upper bound	*/
#define RTXLBOUND	(1 * RTXHZ)		/* RTX lower bound	*/
#define MAXWINDOW	(6*1024)		/* max window size	*/
#define TCP_TTL		10			/* time-to-live for TCP	*/
#define NACKBASE	8			/* packets between ACKs	*/

/*
 * convert from HZ to RTXHZ
 */
#define hz_to_rtxhz(n)	((RTXHZ * (n)) / HZ)

/*
 * tcp header format 
 */
typedef struct tcphdr {
/* 00 */	short	t_sport;	/* source port			*/
/* 02 */	short	t_dport;	/* destination port		*/
/* 04 */	unsigned long	t_seq;	/* packet sequence number	*/
/* 08 */	unsigned long	t_ack;	/* packet acknowledge number	*/
/* 12 */	char	t_offset;	/* data offset			*/
/* 13 */	char	t_ctl;		/* flags			*/
/* 14 */	unsigned short t_window;/* allocated window		*/
/* 16 */	unsigned short t_chksum;/* tcp check sum		*/
/* 18 */	short	t_urgent;	/* urgent pointer		*/
} TCPHDR;				/* TCPHDR length is 20 bytes	*/

/*
 * packet control field flags (TCPHDR.t_ctl)
 */
#define PC_FIN		01		/* finis requested		*/
#define PC_SYN		02		/* synch requested		*/
#define PC_RST		04		/* reset requested		*/
#define PC_EOL		010		/* end of letter		*/
#define PC_PSH		010		/* push to receiver		*/
#define PC_ACK		020		/* ack field valid		*/
#define PC_URG		040		/* urgent field valid		*/

/*
 *  prototype output packet
 */
typedef struct protopkt {
	RTXB	p_rtx;		/* RTX block			*/
	IFARG	p_ifarg;	/* network interface args	*/
	INHDR	p_inh;		/* IP header			*/
	TCPHDR	p_tcph;		/* TCP header			*/
} PROTOPKT; 

/*
 * TCP timer flags (for TPImux based timer)
 */
#define TFLAG_RTX	0x01		/* retransmit timer		*/
#define TFLAG_ACK	0x02		/* ACK timer			*/
#define TFLAG_TWAIT	0x04		/* TIMEWAIT timer		*/
#define TFLAG_USR	0x08		/* user timeout timer		*/
#define TFLAG_FLOW	0x10		/* streams flow control `timer'	*/
#define TFLAG_KEEPALIVE	0x20		/* keep alive timer		*/
#define TFLAG_ZWP	0x40		/* zero window probe timer	*/

/*
 * TCP I/O control requests
 */
#define	IOC_TCP(x)	((((('t' << 8) | 'c') << 8) | 'p') | (x))

#define	IOCTCPSTATS	IOC_TCP(0)	/* dump stats buffer		*/
#define IOCTCPDEBUG	IOC_TCP(1)	/* turn on/off debugging	*/
#define IOCTCPTCBADDR	IOC_TCP(2)	/* get TCB address		*/
#define IOCTCBTRKADDR	IOC_TCP(3)	/* get TRK buffer address	*/
#define IOCTCPGETOPTS	IOC_TCP(4)	/* get default options		*/
#define IOCTCPSETOPTS	IOC_TCP(5)	/* set default options		*/

#ifdef INKERNEL
/*
 * TCP globals
 */
extern short	tcpup;			/* flag signaling tcp alive	*/
extern short	tcp_ncon;		/* # of currently active conns	*/
extern short	timer_started;		/* timer ticking for rtxing	*/
extern PROTOPKT protopkt;
extern char	*cur_data;		/* pointer to curr user data	*/
extern short	 cur_len;		/* number of bytes of user data	*/
extern RTXB	*cur_rtxb;		/* current retrans pointer	*/
extern TCB	*cur_tcb;		/* current tcb			*/
extern INHDR	*cur_ipkt;		/* current internet header	*/
extern INHDR	*cur_inp;		/* initpkt internet header	*/
extern TCPHDR	*cur_tpkt;		/* current tcp header		*/
extern long	 cur_shost;		/* current source host		*/
extern long	 cur_dhost;		/* current destination host	*/
extern PSEUDOSUM pseudosum;		/* pseudo header for checksum	*/
extern struct tcpstats tcpstats;	/* TCP statistics		*/
extern unsigned short tcpcksum();	/* TCP checksum routine		*/
#endif	/* INKERNEL	*/
