/*
 *	Copyright (c) 1982 UCB
 *	Copyright (c) 1987 Motorola Inc
 *	All Rights Reserved
 *	@(#)berk.c	35.4
 */

/*
 * Copyright (c) 1982 Regents of the University of California.
 * All rights reserved.  The Berkeley software License Agreement
 * specifies the terms and conditions for redistribution.
 */

#ident	"@(#)berk.c	35.4"

#include "sys/types.h"
#include "sys/param.h"
#include "sys/fs/s5dir.h"
#include "sys/signal.h"
#include "sys/errno.h"
#include "sys/immu.h"
#include "sys/user.h"
#include "sys/file.h"
#include "sys/inode.h"
#include "sys/fstyp.h"
#include "sys/sysinfo.h"
#include "sys/region.h"
#include "sys/proc.h"
#include "sys/mount.h"
#include "sys/conf.h"
#include "sys/time.h"
#include "sys/uio.h"

#define	getbits(name, x) \
	if (uap->name) { \
		u.u_error = copyin((caddr_t)uap->name, (caddr_t)&ibits[x], \
		    ni * sizeof(fd_mask)); \
		if (u.u_error) \
			goto done; \
	}

#define	putbits(name, x) \
	if (uap->name) { \
		int error = copyout((caddr_t)&obits[x], (caddr_t)uap->name, \
		    ni * sizeof(fd_mask)); \
		if (error) \
			u.u_error = error; \
	}

extern time_t	time,		/* time in seconds since 1970 */
		lbolt;		/* time in HZ since last boot */

#define hzto(tv)	((((tv)->tv_sec - (lbolt / HZ) ) * HZ) + \
			((tv)->tv_usec - ((lbolt % HZ) * (1000000 / HZ))) / \
			(1000000 / HZ))

int	unselect();
int	nselcoll;
int	selwait;

/*
 * Select system call.
 */
select() {
	register struct uap  {
		int	nd;
		fd_set	*in, *ou, *ex;
		struct	timeval *tv;
	} *uap = (struct uap *)u.u_ap;
	fd_set ibits[3], obits[3];
	struct timeval atv, ctv;
	int s, ncoll, ni, tid;

	if (uap->tv) {
		u.u_error = copyin((caddr_t)uap->tv, (caddr_t)&atv,
			sizeof (atv));
		if (u.u_error)
			goto done;

		if (itimerfix(&atv)) {
			u.u_error = EINVAL;
			goto done;
		}

		s = splhi();
		atv.tv_sec  += (lbolt / HZ);
		atv.tv_usec += (lbolt % HZ) * (1000000/HZ);
		splx(s);
	}

	bzero(ibits, sizeof(ibits));
	bzero(obits, sizeof(obits));
	if (uap->nd > NOFILE)
		uap->nd = NOFILE;	/* forgiving, if slightly wrong */
	ni = howmany(uap->nd, NFDBITS);

	if(ni) {
		getbits(in, 0);
		getbits(ou, 1);
		getbits(ex, 2);
	}

retry:
	ncoll = nselcoll;
	u.u_procp->p_flag |= SSEL;
	if(ni) {
	    u.u_rval1 = selscan(ibits, obits, uap->nd);
	    if (u.u_error || u.u_rval1)
		goto done;
	}

	s = splhi();
	if ((u.u_procp->p_flag & SSEL) == 0 || nselcoll != ncoll) {
		u.u_procp->p_flag &= ~SSEL;
		splx(s);
		goto retry;
	}
	u.u_procp->p_flag &= ~SSEL;

	if (uap->tv)
		tid = timeout(unselect, (caddr_t)u.u_procp, hzto(&atv));

	if (sleep((caddr_t)&selwait, PZERO+1|PCATCH)) {
		if (uap->tv)
			untimeout(tid);
		u.u_error = EINTR;
		splx(s);
		goto done;
	}

	if (uap->tv) {
		ctv.tv_sec = (lbolt / HZ);
		ctv.tv_usec = (lbolt % HZ) * (1000000/HZ);
		if (ctv.tv_sec > atv.tv_sec
		    || ctv.tv_sec == atv.tv_sec && ctv.tv_usec >= atv.tv_usec) {
			splx(s);
			goto done;
		}
		untimeout(tid);
	}

	splx(s);
	goto retry;
done:
	u.u_procp->p_flag &= ~SSEL;

	if (u.u_error != EINTR) {
		putbits(in, 0);
		putbits(ou, 1);
		putbits(ex, 2);
	}
}

unselect(p)
	register struct proc *p;
{
	register int s = splhi();

	switch (p->p_stat) {

	case SSLEEP:
		setrun(p);
		break;

	case SSTOP:
		unsleep(p);
		break;
	}
	splx(s);
}

selscan(ibits, obits, nfd)
	fd_set *ibits, *obits;
{
	register int which, i, j;
	register fd_mask bits;
	int flag;
	struct file *fp;
	int n = 0;

	for (which = 0; which < 3; which++) {
		switch (which) {

		case 0:
			flag = FREAD; break;

		case 1:
			flag = FWRITE; break;

		case 2:
			flag = 0; break;
		}
		for (i = 0; i < nfd; i += NFDBITS) {
			bits = ibits[which].fds_bits[i/NFDBITS];
			while ((j = ffs(bits)) && i + --j < nfd) {
				bits &= ~(1 << j);
				fp = u.u_ofile[i + j];
				if (fp == NULL) {
					u.u_error = EBADF;
					break;
				}
				if (FS_SELECT(fp->f_inode, flag)) {
					FD_SET(i + j, &obits[which]);
					n++;
				}
			}
		}
	}
	return (n);
}

/*ARGSUSED*/
seltrue(dev, flag)
	dev_t dev;
	int flag;
{

	return (1);
}

selwakeup(p, coll)
	register struct proc *p;
	int coll;
{

	if (coll) {
		++nselcoll;
		wakeup((caddr_t)&selwait);
	}
	if (p) {
		int s = splhi();
		if (p->p_wchan == (caddr_t)&selwait) {
			if (p->p_stat == SSLEEP)
				setrun(p);
			else
				unsleep(p);
		} else if (p->p_flag & SSEL)
			p->p_flag &= ~SSEL;
		splx(s);
	}
}

/*
 * Check that a proposed value to load into the .it_value or
 * .it_interval part of an interval timer is acceptable, and
 * fix it to have at least minimal value (i.e. if it is less
 * than the resolution of the clock, round it up.)
 */
itimerfix(tv)
	struct timeval *tv;
{

	if (tv->tv_sec < 0 || tv->tv_sec > 100000000 ||
	    tv->tv_usec < 0 || tv->tv_usec >= 1000000)
		return (EINVAL);
	if (tv->tv_sec == 0 && tv->tv_usec != 0 && tv->tv_usec < (1000000/HZ))
		tv->tv_usec = (1000000/HZ);
	return (0);
}

/*
 * readv system call
 */
readv()
{
	sysinfo.sysread++;
	rdwrv(FREAD);
}

/*
 * writev system call
 */
writev()
{
	sysinfo.syswrite++;
	rdwrv(FWRITE);
}

/*
 * common code for read and write vector calls:
 * check permissions, set u_base, u_count, and u_offset,
 * and switch out to readi or writei code in a loop for all vectors.
 */
rdwrv(mode)
register mode;
{
	register struct file	*fp;
	register struct inode	*ip;
	extern file_t		*getf();
	register struct a {
		int	fdes;
		struct	iovec *iovp;
		unsigned iovcnt;
	}			*uap;
	struct	uio auio;
	struct	iovec	aiov[16];		/* XXX */
	struct	iovec	*iov;

	int			type;
	int			bcount;
	int			vcount;
	register int		count;
	register uint		ulimit;

	uap = (struct a *)u.u_ap;
	if (uap->iovcnt > sizeof(aiov)/sizeof(aiov[0])) {
		u.u_error = EINVAL;
		return;
	}
	u.u_error = copyin((caddr_t)uap->iovp, (caddr_t)aiov,
	    uap->iovcnt * sizeof(struct iovec));
	if(u.u_error)
		return;

	if ((fp = getf(uap->fdes)) == NULL)
		return;
	if ((fp->f_flag & mode) == 0) {
		u.u_error = EBADF;
		return;
	}

	ip = fp->f_inode;
	type = ip->i_ftype;
	u.u_segflg = 0;
	u.u_fmode = fp->f_flag;
	if (type == IFREG || type == IFDIR) {
		plock(ip);
		if ((u.u_fmode&FAPPEND) && (mode == FWRITE))
			fp->f_offset = ip->i_size;
	} else if (type == IFIFO) {
		plock(ip);
		fp->f_offset = 0;
	}
	if (mode == FREAD) {
		bcount = u.u_ior;
	} else {
		bcount = u.u_iow;
	}
	count = 0;
	vcount = 0, iov = aiov;
	while (vcount < uap->iovcnt) {
	    register int	iocnt;
	    register int	req_cnt;
	    if(iov->iov_len == 0) {
 		vcount++, iov++;
		continue;
	    }
	    if(iov->iov_len < 0) {
		u.u_error = EINVAL;
		break;
	    }
	    u.u_offset = fp->f_offset;
	    u.u_base = (caddr_t)iov->iov_base;
	    req_cnt = u.u_count = iov->iov_len;
	    if (mode == FREAD) {
		    FS_READI(ip);
	    } else {
		    if(type == IFREG) {
		    /*	Make sure that the user can write all the way up
		    **	to the ulimit value.
		    */
			ulimit = (uint)(u.u_limit << SCTRSHFT) - fp->f_offset;
			if (ulimit < req_cnt) {
			    if(ulimit > 0)
				req_cnt = u.u_count = ulimit;
			    else if(count > 0)
				break;
			}
		    }
		    FS_WRITEI(ip);
	    }
	    iocnt = req_cnt - u.u_count;
	    iov->iov_len -= iocnt;
	    count += iocnt;

	    /*  **************************************************** */
	    /*  Used by 3bnet to prevent f_offset from being changed */

	    if ((fp->f_flag & FNET) == 0)
		    fp->f_offset += iocnt;
	    /*  **************************************************** */

	    if(u.u_error || u.u_count)
		break;
	    if(iov->iov_len == 0) {
 		vcount++, iov++;
	    }
	}
	if (u.u_error) {
	    if(!count) { /* if no data was transfered */
		if (type == IFREG || type == IFDIR || type == IFIFO)
			prele(ip);
		return;
	    }
	    else	/* don't return error number, instead return count */
		u.u_error = 0;
	}
	if (type == IFREG) {
		/* If synchronous write, update inode now. */
		if ((u.u_fmode & FSYNC) && (mode == FWRITE)) {
			ip->i_flag |= ISYN;
			FS_IUPDAT(ip, &time, &time);
		}
		prele(ip);
	} else if (type == IFDIR || type == IFIFO)
		prele(ip);

	u.u_rval1 = count;
	u.u_ioch += (unsigned)u.u_rval1;
	if (mode == FREAD) {
		sysinfo.readch += (unsigned)u.u_rval1;
		fp->f_inode->i_mntdev->m_bcount += u.u_ior - bcount;
	} else {
		sysinfo.writech += (unsigned)u.u_rval1;
		fp->f_inode->i_mntdev->m_bcount += u.u_iow - bcount;
	}
}
