/*
 *	Copyright (c) 1984 AT&T
 *	Copyright (c) 1987 Motorola Inc
 *	All Rights Reserved
 *	@(#)kern-port:os/machdep.c	35.5
 */

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF AT&T	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ident	"@(#)kern-port:os/machdep.c	35.5"

#include "sys/sysmacros.h"
#include "sys/param.h"
#include "sys/types.h"
#include "sys/immu.h"
#include "sys/systm.h"
#include "sys/psw.h"
#include "sys/fs/s5dir.h"
#include "sys/signal.h"
#include "sys/region.h"
#include "sys/proc.h"
#include "sys/file.h"
#include "sys/inode.h"
#include "sys/user.h"
#include "sys/var.h"
#include "sys/reg.h"
#include "sys/todc.h"

extern char *fpu_ext;
extern char *fpu_sig;

/*
 *	Read time of day chip
 */
extern time_t time;

time_t clkread()
{
	time_t	tdctime;

	if (todc(GETTIME, &tdctime) == OK)
		return(tdctime);
	else
		return(time);
}

/*
 * Send an interrupt to process
 */
sendsig(hdlr, signo, arg)
caddr_t hdlr;
{
	register *usp, *regs;

	regs = u.u_ar0;
	usp = (int *)regs[SP];

#ifdef m88k

	if (grow((unsigned)(usp-8)) == -1) {		/* 8 integers */
		return;
	}
	/* Save registers to be restored later on the user's stack */
	suword((caddr_t)--usp, regs[SP]);     /* Stack pointer. */
	suword((caddr_t)--usp, regs[PS]);     /* Status register. */
	suword((caddr_t)--usp, regs[SFIP]);   /* interrupted PC. */
	suword((caddr_t)--usp, regs[SNIP]);   /* interrupted PC. */
	suword((caddr_t)--usp, regs[R1]);     /* parameter regs. */
	suword((caddr_t)--usp, regs[R2]);
	suword((caddr_t)--usp, regs[R3]);
	suword((caddr_t)--usp, regs[R4]);

	regs[SP] = (int)usp;
	regs[R1] = regs[SNIP] + 1;
	regs[R2] = signo;
	regs[R3] = arg;
	regs[R4] = u.u_traptype;
	regs[SFIP] = (int)hdlr | VALIDIP;
	regs[SNIP] &= ~VALIDIP;

#else

	  /* For M68881/M68882 exceptions */
	if(u.u_traptype == TRAPFPE) {
		register cnt;

		if(u.u_fpu.ustate & UFPSAVE) {
			/* cannot nest FPE signal handlers */
			/* get rid of any user-specified handler */
			u.u_signal[SIGFPE - 1] = 0;
			psignal(u.u_procp, SIGFPE);
			return;
		}
		fsave(fpu_ext);
		/*
		 * The following is needed because of an MC68882 mask fault.
		 *
		 * Note that we pull the state frame size from the
		 * header and use that to determine the address of
		 * the BIU flags.
		 */

		cnt = ((fp_state *)(fpu_ext))->fp_siz;
		*(int *)(fpu_ext + cnt) |= EXC_PEND;
		bcopy(fpu_ext, fpu_sig, cnt + 4);
		u.u_fpu.ustate = UFPSAVE;  /* u_fpu.excop has valid data */
		u.u_trapflag = 1;          /* Save stack frame */
	} else {
		if((u.u_ar0[18] & 0xF000) == 0x9000) {
			  /*  This is a co-processor mid-instruction      */
			  /*  frame.  We do not execute the signal        */
			  /*  handler, but save the signal and arg in     */
			  /*  the U-block, then force a trap by setting   */
			  /*  the trace bit.  On return, the instruction  */
			  /*  will complete, then trap.  At that time     */
			  /*  we will retrieve signal and arg and exe-    */
			  /*  cute the handler.                           */
			u.u_ar0[PS] |= PS_T1;           /* Force trap   */
			u.u_trapsig = signo;            /* Save signal  */
			u.u_trapcode = arg;             /* Save arg     */
			  /* Re-arm the signal */
			u.u_signal[signo - 1] = (void (*)())hdlr;
			return;                 /* Don't do handler */
		}
	}

	if (u.u_traptype) {
		regs[PC] |= 1;	/* make odd to prevent sucessful 'rtr' */
	}
	if (grow((unsigned)(usp-4)) == -1) {		/* 5 integers */
		return;
	}
	/* simulate an interrupt on the user's stack */
	suword((caddr_t)--usp, regs[PC]);
	suword((caddr_t)--usp, (regs[PS] & 0xffff) | (u.u_traptype << 16));
	suword((caddr_t)--usp, arg);
	suword((caddr_t)--usp, signo);

	regs[SP] = (int)usp;
	regs[PS] &= ~(PS_T1 | PS_T0);
	regs[PC] = (int)hdlr;
}
#endif

/*
 * Clear registers on exec
 */

setregs()
{
	register int	i;
	void (* *rp)();

#ifdef	m88k
	for (i = 0; i < 31; i++) 
		u.u_ar0[i] = 0;	/* ignore PS and PC */

		/* special register for 88000; initial pc */
		/* it will have 8 added upon return */
	u.u_ar0[SCIP] = u.u_exdata.ux_entloc - 8;
#else
	for (i = 0; i < 15; i++) 
		u.u_ar0[i] = 0;	/* ignore PS and PC */

		/* special registers for 68000; initial pc */
	u.u_ar0[PC] = u.u_exdata.ux_entloc;

		/* Initial stack region */
	u.u_rval1 = u.u_ar0[R8] = USRSTACK - ctob(u.u_ssize);
	u.u_rval2 = 0;
#endif

	u.u_procp->p_chold = 0;	/* clear p_chold */

	/* Any pending signal remain held, so don't clear p_hold and
	p_sig */	

	/* If the action was to catch the signal, then the action
	must be reset to SIG_DFL */

	for (rp = &u.u_signal[0]; rp < &u.u_signal[NSIG]; rp++)
		if (*rp != SIG_IGN)
			*rp = SIG_DFL;

	for (i=0; i<v.v_nofiles; i++)
		if ((u.u_pofile[i] & EXCLOSE) && u.u_ofile[i] != NULL) {
			closef(u.u_ofile[i]);
			u.u_ofile[i] = NULL;
		}
}

addupc(pc, p, incr)
unsigned pc;
register struct
{
	short	*pr_base;
	unsigned pr_size;
	unsigned pr_off;
	unsigned pr_scale;
} *p;
{
	register int   curval;	/* current value of word @ 'slot' */
	register int   tmpval;
	register short *slot;

	slot = &p->pr_base[((((pc - p->pr_off) * p->pr_scale) >> 16))>>1];
	if ((caddr_t)slot >= (caddr_t)(p->pr_base) &&
		(caddr_t)slot < (caddr_t)((unsigned)p->pr_base + p->pr_size))
		{
			tmpval = (int)fubyte((char *)slot);
			tmpval = (int)(tmpval<<8 | fubyte((char *)slot+1));
			if ((curval = tmpval) == -1)
				u.u_prof.pr_scale = 0;	/* turn off */
			else
			{
			    /*
			    ** Inc curval;  Ovfl lets loword (short) wraparound.
			    */
				curval += incr;
				subyte((char *)slot, curval>>8);
				subyte((char *)slot+1, curval&0xff);
			}
		}
}
