/*
 *	Copyright (c) 1984 AT&T
 *	Copyright (c) 1987 Motorola Inc
 *	All Rights Reserved
 *	@(#)kern-port:os/pmmu.c	35.11
 */

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF AT&T	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ident	"@(#)kern-port:os/pmmu.c	35.11"

#include "sys/types.h"
#include "sys/param.h"
#include "sys/immu.h"
#include "sys/systm.h"
#include "sys/mvmecpu.h"
#include "sys/sysmacros.h"
#include "sys/pfdat.h"
#include "sys/signal.h"
#include "sys/fs/s5dir.h"
#include "sys/user.h"
#include "sys/inode.h"
#include "sys/region.h"
#include "sys/proc.h"
#include "sys/map.h"
#include "sys/var.h"
#include "sys/ram.h"
#include "sys/cmn_err.h"
#include "sys/debug.h"

#ifndef CMMU
#define NCPD	128*NCPS	/* Number of clicks per domain */
#define NBPD	ctob(NCPD)	/* Number of bytes per domain */
#endif

/*
 * File: pmmu.c
 *  This file contains primitives to support the MC-68851 PMMU chip and
 *  the MMB emulation board.  Currently there are no dependencies in
 *  this file--all dependencies being located in assembly language routines.
 *
 *  These primitives support the 134 mapping requirements, i.e, all
 *  pages are allocated in continguous blocks of 4.
 */

/*
 * System memory management variables
 */

pde_t *mmuupde;	/* pointer to table of pde's mapping the U-Vector */

extern struct map sptmap[];  /* map for dynamically allocated virtual space */
extern etext;

#ifdef	m88k
extern end;
extern M88Kvec[];
#else
extern struct boardid *boardid; /* Board ID packet */
extern long M68Kvec[];
extern short transparent;	/* transparent translation */
extern short a99a;		/* Bad pmmu mask */
#endif

/*
 * System table structures 
 * 	The following must be quad longword (hex) aligned.
 */

#ifndef CMMU
static char tblarea[FCLTSZ * sizeof(sde_t) + 15];  /* table area */

sde_t *fcltbl;		/* function lookup table */
sde_t *udtbl;		/* user domain table */
sde_t *sdtbl;		/* system domain table */
#else
sde_t *ustbl;		/* user segment table */
sde_t *sstbl;		/* system segment table */
#endif


#ifndef CMMU
/* The following variables are used by the macro newsde() */
int     nexttfn;
int     lasttfn;

/*
 * The following macro is used by mmuinit and mkpgtbl (make page tables)
 * to put more than one sde table into a page of memory.  It is a little
 * like a "pre mmu" small page.  It should only be used during
 * initialization before the mmu is turned on.
 */
/* When the compiler gets fixed replace the two lines with
   nexttfn = ctob(Y++) */
#define newsde(X,Y)                     \
	if(nexttfn == lasttfn) {        \
		nexttfn = ctob(Y);      \
		Y++;                    \
		lasttfn = ctob(Y);      \
	}                               \
	X = (sde_t *)nexttfn;           \
	nexttfn += NBPPT;
#endif

/*
 * mmuinit(physclick)
 *
 * Sets up the system space for the MMU.
 * Enter this routine running in physical space and exit it running
 * mapping turned on.
 *
 * physclick is the first physical click available for tables.
 */
#ifdef CMMU	/* this should be in mmu.h file */
typedef union batc {
	struct {
	unsigned long	lba	:	13,	/* logical block addr	*/
			pba	:	13,	/* physical block addr	*/
			supv	:	1,	/* supervisor LA 	*/
			wt	:	1,	/* write through 	*/
			global	:	1,	/* global block 	*/
			ci	:	1,	/* cache inhibit 	*/
			wp	:	1,	/* write protect 	*/
			val	:	1;	/* valid block discrip 	*/
	} batc_b;
	struct {
	unsigned long	word;
	} batc_w;
} batc_t;
#define	BWP0		0x400	/* base plus this = batc write port */
#define	DATACMMU	0x1000	/* code base plus this = base of data cmmu */
#define BLKSIZE		0x80000	/* each batc maps 512K block */
#define BLKSHFT		19	/* bits to shift to divide by 512K */
#define btoblk(x)	((x) >> BLKSHFT)
#define pg_setwt(P)	((P)->pgm.pg_wt = 1)
#define pg_clrwt(P)	((P)->pgm.pg_wt = 0)
#define pg_setg(P)	((P)->pgm.pg_g = 1)
#define pg_clrg(P)	((P)->pgm.pg_g = 0)
#endif

int mmuinit(physclick)
int physclick;
{
	extern struct mem memmap[];
	extern int textro;
	register sde_t *sde;
	register sde_t *stbl;
	register pde_t *pde;
	register unsigned int addr, endaddr;
	register struct mem *mem;
	register int vindex;
 	register unsigned int size;
#ifdef CMMU
	unsigned int firstpt = ctob(physclick);

	{
	/* load block address translation for kernel text now */
	batc_t batc;	/* working batc */
	int i = 0;
	addr = 0;	/* starting at text at zero */
	batc.batc_w.word = 0;
	/* there are only 8 BATC entries in each CMMU */
	while( addr < (unsigned int)&etext && i < (sizeof(long)*8) ) {	
		batc.batc_b.lba = batc.batc_b.pba = btoblk(addr);
		batc.batc_b.supv = 1;
		batc.batc_b.val = 1;

		cmmu_store(BWP0 + i, batc.batc_w.word);
		i += sizeof(long);
		addr += BLKSIZE;
	}
	i = DATACMMU;
	/* this will only load data BATC for the 512K blocks before end of bss
	the last partial block from end of bss to end of last block is 
	not in BATC because the page tables start on a page boundry
	If the start of free mem was rounded up to the next block then
	all the data and bss could be in BATC */
	while(addr < (unsigned int)&end && (i-DATACMMU) < (sizeof(long)*8)) {	
		/* set the previous 512K block in data BATC */
		batc.batc_b.lba = batc.batc_b.pba = btoblk(addr - BLKSIZE);
		batc.batc_b.supv = 1;
		batc.batc_b.val = 1;
		batc.batc_b.wt = 1;	/* ??write thr. need in kernel space? */

		cmmu_store(BWP0 + i, batc.batc_w.word);
		i += sizeof(long);
		addr += BLKSIZE;
	}
	}
#endif

	/*
	 * Set up MMU translation tree for all kernel virtual space.
	 */

#ifndef CMMU
	chk_mmu();
	if (cputype == PROC020 && a99a)
		wakemmu();

	/* Initialize variables for newsde() macro */
/*	nexttfn = ctob(physclick++);  */
/*
 * When the compiler gets fixed, replace the following two lines
 * with the above commented out line.
 */
	nexttfn = ctob(physclick);
	physclick++;
	lasttfn = ctob(physclick);

	/* Allocate user and system domain tables */
	newsde(udtbl,physclick);
	newsde(sdtbl,physclick);

	/*
	 * Set up function code lookup table.
	 * Note that all pointer and page tables must be 16 BYTE aligned.
	 * Most actually end up page alligned.
	 */
	fcltbl = (sde_t *) (((int) tblarea + 15) & ~15);

	sde = &fcltbl[SUPVDATA];	/* for system function codes */
	setsde(sde, SDTLIM, btopt(sdtbl), PT_VLD );
	fcltbl[SUPVPROG] = *sde;

	sde = &fcltbl[USERDATA];	/* for user function codes */
	setsde(sde, UDTLIM, btopt(udtbl), PT_VLD );
	fcltbl[USERPROG] = *sde;


	/* 
	 * Set up user domain table.
	 */

	setsde(udtbl, USTLIM, btopt(NULLADDR), PT_INV );

#else	/* CMMU */
	sstbl = (sde_t *)ctob(physclick);
	physclick++;
	ustbl = (sde_t *)0;	/* user segment table is set during switch */
#endif

	/* 
	 * Build domain for static and dynamic kernel virtual space
	 */
	for ( vindex = 0 ; vindex < 2 ; vindex++ ) {
 		static unsigned int virt[2] = { 0, SYSSEGS};

		/*
		 * If transparent is set for tt0 and
		 *    cputype is 683 and
		 *    text not read only and
		 *    vindex is zero then
		 * skip building translation tables for the kernel's
		 * static space.  Do transparent translation using tt0.
		 */
		if ((transparent & (short)TT0_ENB) &&
		    cputype == PROC030 &&
		    !textro &&
		    !vindex)
			continue;
		for ( addr = virt[vindex], endaddr = addr + ctob( physmem );
		    addr < endaddr ; addr += NBPD ) {
#ifndef CMMU
			register unsigned int dindex;
	
			dindex = dnum(addr);
#endif
			size = endaddr - addr;
#ifndef CMMU
			/*
			 * Make sure size is less than NBPD
			 */
			if (size > NBPD)
				size = NBPD;
			/* allocate segment table and attach to domain */
			if(pt_tstvalid(sdtbl + dindex, PT_INV)) {
				newsde(stbl, physclick);
				setsde(sdtbl + dindex, SSTLIM, btopt(stbl),
				    PT_VLD);
			}
			else
				stbl = (sde_t*)pttob(pt_gettfn(sdtbl + dindex));
			physclick = mkpgtbl( pnum(doff(addr)), btoc(size),
			    stbl, physclick );
#else
			physclick = mkpgtbl( pnum(addr), btoc(size),
			    sstbl, physclick );
#endif
		}
	}

	for ( mem = mapstart(memmap); mem->m_size ; mem++ ) {
		register unsigned int flag, ci;

		flag = mem->m_flag;
		if ( !( flag & ( RAM_FOUND | RAM_PRIV ) ) )
			continue;

		size = mem->m_size;
		addr = ctob(mem->m_addr);
		ci = flag & RAM_CI;

		for ( ; size ; addr += ctob(1), size-- ) {
#ifndef CMMU
			register unsigned int dindex;

			/*
			 * If cputype is 683 and
			 *    ((transparent is set for tt0 and
			 *    text not read only and
			 *    address in kernel's static space) or
			 *    (transparent is set for tt1 and
			 *    address in kernel's IO area))) then skip
			 * building page tables.
			 * Do transparent translation.
			 */
			if (cputype == PROC030 &&
			    (((transparent & (short)TT0_ENB) &&
			    !textro &&
			    addr < SYSSEGS) ||
			    ((transparent & (short)TT1_ENB) &&
			    addr >= IO_32_MIN)))
				continue;

			dindex = dnum(addr);
			/* allocate segment table and attach to domain */
			if(pt_tstvalid(sdtbl + dindex, PT_INV)) {
				newsde(stbl, physclick);
				setsde(sdtbl + dindex, SSTLIM, btopt(stbl),
				    PT_VLD);
			}
			else
				stbl = (sde_t*)pttob(pt_gettfn(sdtbl + dindex));
			physclick = mkpgtbl(pnum(doff(addr)), 1, stbl,
			    physclick);
#else
			physclick = mkpgtbl(pnum(addr), 1, sstbl,
			    physclick);
#endif
			pde = kvtopde(addr);
			pg_setpfn(pde, pnum(addr));
#ifdef CMMU
			/* because kernel page tables must be cache inhibited */
			if( (addr >= firstpt && addr < ctob(physclick) ) || 
			/* ROM's data in first 64k must be cache inhibited */
			    ( addr >= 0  && addr < 0x00010000L )  ||
			    (ci))
#else
			if (ci)
#endif
				pg_setci(pde);
#ifdef CMMU		/* make all free pages not ci'ed write through */
			else /* if( addr >= firstpt) attemp to fix cache bug */
				pg_setwt(pde);
#endif
			pg_setvalid(pde);
		}
	}

	kptbl = kvtopde(SYSSEGS);	/* set table address */
	mmuupde = kptbl;		/* set U-Vector pde table address */

	/* Initialize the map used to dynamically allocate
	 * kernel virtual space. Skip room for UBLOCK.
	 */

	mapinit(sptmap, v.v_sptmap);
	mfree(sptmap, physmem-USIZE, btoc(SYSSEGS)+USIZE);

	/* 
	 * Enable mapping
	 */

#ifndef CMMU
	mmu_setup();
	/*
	 * if cputype is 683 then
	 * 	if transparent is set for tt0 then
	 * 		enable tt0 for kernel's static area
	 * 	if transparent is set for tt1 then
	 * 		enable tt1 for kernel's IO area
	 */
	if (cputype == PROC030) {
		if (transparent & TT0_ENB)
			tt0_enable(0,SYSSEGS-1,0,textro);
		if (transparent & TT1_ENB)
			tt1_enable(IO_32_MIN,ALLFFFS-IO_32_MIN,1,0);
	}
	mmu_enable();
#else
	cmmu_store(CMMU_SRP, (unsigned long)sstbl);
/*	cmmu_store(CMMU_TCR, P_CMMU_FIXED);	/* for pmmu simulation */
#endif
	return(physclick);
}

/*
 * loadstbl(up, prp, change)
 *
 * Change the content of a process's segment descriptor entries (pointer tbl):
 *    	change > 0 -> load sdt table using the affected entries of prp->r_list.
 *    	change = 0 -> invalidate all sdt table entries for this prp.
 *	change < 0 -> invalidate the affected sdt table entries. 
 *
 * Region data always represents the new state when loadstbl() is called. 
 *  Exception: Special case when (change == 0), then invalidate all entries.
 *   In this case, region has not change to new state or size.
 */

loadstbl(up, prp, change)
user_t		*up;
preg_t		*prp;
register int	change;
{
	register reg_t	*rp;
	register sde_t	*st;
	register proc_t *p;
	int	seg;	/* first segment to be modified (pttbl index) */
	int	rseg;	/* first segment to be modified (rlist index) */
	int	lim;	/* first segment not to be modified */
	int	newend, oldend, lowend, highend;
	int	osize;

	p = up->u_procp;
	rp = prp->p_reg;

	osize = btoct(prp->p_regva) + rp->r_pgoff;

	if (prp->p_type != PT_STACK) {
		newend = osize + rp->r_pgsz;
		if (change)
			oldend = newend - change;
		else
			oldend = osize;
	} else {
		newend = osize;
		if (change)
			oldend = newend + change;
		else
			oldend = osize + rp->r_pgsz;
	}

	lowend  = min(newend, oldend);
	highend = max(newend, oldend);

	lim = ctos(highend);			/* round up */

	if (change <= 0) {	/* shrinking region,  clear sde's */
		seg = ctos(lowend - rp->r_pgoff);	/* round up */
		st = &p->p_pttbl[seg];		/* first seg to change */
		for (; seg < lim; seg++, st++) {	
			pt_clrvalid(st);	
			pt_settfn(st, 0);
		}
	} else {			/* expanding region, set up sde's */
		seg  = ctost(lowend - rp->r_pgoff);	/* round down */
		st = &p->p_pttbl[seg];		/* first seg to change */
		rseg = ctost(lowend - rp->r_pgoff - btoc(prp->p_regva) );
		for (; seg < lim; seg++, rseg++, st++) {    
#ifndef CMMU
			pt_setup(st);
#endif
			pt_settfn(st, kvtotfn(rp->r_list[rseg]));
#ifndef CMMU
			pt_setvalid(st,PT_VSD);
#else
			pt_setvalid(st,PT_VLD);
#endif
			if (prp->p_flags & PF_RDONLY)
				pt_setprot(st);
#ifdef CMMU
			else
	/* this was done by default in pt_setup, not used in CMMU */
				pt_clrprot(st);
#endif
		}
	}
	mmuflush();
}


/*
 * mmucswtch(p)
 *
 * Sets up the MMU to switch to process p.
 * Requires a change of the single descriptor in the 
 * user domain table (udtbl).
 * The MMU must be properly flushed after the change.
 */
mmucswtch(p)
register proc_t	*p;
{
	register int	ipl;

	ASSERT(p->p_pttbl != NULL);
#ifndef CMMU
	pt_settfn(udtbl,kvtotfn(p->p_pttbl));
	pt_setvalid(udtbl,PT_VLD);
#else	/* save for user virtual to phys macros */
	ustbl = (sde_t *)pttob(kvtotfn(p->p_pttbl));
	cmmu_store(CMMU_URP, (long)ustbl);	/* load the USER(CPU) RP */
#endif
	/*
	 * mmuflush not required here as long as resume
	 * must flush using mmuflush.  With PMMU the USERPROG
	 * and USERDATA function code related accesses should
	 * be flushed here.
	 */
}

/*	The one and only MMB flush option is to flush the entire ATC
 *	by rewriting the root pointer.
 *	Note that kvtophys(fcltbl) would be appropriate but we know
 *	the kernel is mapped 1:1 so the kvtophys() is skipped
 *	...we have enough overhead already.
 */

/*
 * mmuflush() flushes entire MMU.
 * Probably wouldn't need to flush supervisor program space.
 */
mmuflush()
{
#ifndef CMMU
	mmbstore(MMB_RP,fcltbl);
 	if (cputype == PROC030)
 		dcachclr();
#else
	cmmuflush();
#endif
}

/*
 * mkpgtbl( firstclick, size, stbl, nextfree )
 *
 * Makes page tables as required to map the specified 
 * virtual address range from firstclick for size clicks 
 * within a domain whose segment table is pointed to by stbl,
 * taking clicks for tables from a zeroed click heap beginning at nextfree. 
 *
 * Returns nextfree, the next free click for tables.
 * The new page table is invalidated by default since the heap is zeroed.
 *
 * Note: We make the assumption here that NPTPP page tables fit exactly
 * in one page and that clicksize is the same as pagesize.
 */
int
mkpgtbl( firstclick, size, stbl, nextfree )
int	firstclick;	/* click offset into domain */
int	size;
sde_t	*stbl;		/* segment table */
int	nextfree;
{
	sde_t *sde, *lastsde, *tempsde;

#ifndef CMMU
	/*
	 * Check to make sure that the last entry does not
	 * exceed the the number of clicks in a domain.
	 */
	ASSERT( firstclick + size <= NCPD);
#endif
	ASSERT( size > 0 );

	/* For each segment descriptor within the range */

	lastsde = stbl + ctost(firstclick + size -1);
	for ( sde = stbl + ctost(firstclick); sde <= lastsde; sde++ ) {

#ifndef CMMU
		if ( !pt_tstvalid(sde,PT_VSD) ) { /* if the sde is not valid */

			/* allocate and attach page table */
			newsde(tempsde, nextfree);
			setsde(sde, PGTLIM, btopt(tempsde), PT_VSD);
#else
		if ( !pt_tstvalid(sde,PT_VLD) ) { /* if the sde is not valid */

			/* allocate and attach page table */
			tempsde = (sde_t *) ctob(nextfree);
			++nextfree;
			setsde(sde, 0, btopt(tempsde), PT_VLD);
#endif
		}
	}

	return( nextfree );
}

/*
 * setsde(sde, lim, tfn, val )
 *
 * Sets default values, limit and tfn (table frame no) for specified sde.
 * If val is set, validates the sde using the value of val.
 */
setsde(sde, lim, tfn, val )
sde_t	*sde;
int	lim;
int     tfn;
int	val;
{
#ifndef CMMU
	pt_setup(sde);
	pt_setlim(sde,lim);
#endif
	pt_settfn(sde,tfn);
	if( val )
		pt_setvalid(sde,val);
}


/*	Eliminate troublesome supervisor instruction mode RMC cycles 
 *	by PMMU, by presetting the U bits in the kernel page tables.
 *	Required on shared bus architecture CPU boards with PMMU
 *	when using VMEbus memory for page tables. 
 *	(currently, only the MVME134)
 */

rmcinit()
{
#ifndef CMMU
	extern short do_pmmu;

	if( do_pmmu ) 				/* only needed with PMMU */
#ifdef	m88k
		mmutouch( M88Kvec, ctob(kpbase) ); /* touch kernel memory */
#else
		mmutouch( M68Kvec, ctob(kpbase) ); /* touch kernel memory */
#endif
#endif
}


/*
 * mmutouch(addr1, addr2 )
 *
 * touches (reads and writes one byte of) the memory covered by each
 * mmu descriptor in the address range specified by addr1 and addr2 
 * (addr1 <= addr2).
 * Used to preset the U and M bits (used and modified ) in the translation
 * tree to avoid problem RMC cycles run by the PMMU when updating those bits.
 */

#ifndef CMMU
#define NBPPDE	1024	/*  bytes of virtual space covered by each pde */
#else /* CMMU */
#define NBPPDE	4096	/*  bytes of virtual space covered by each pde */
#endif

mmutouch( addr1, addr2 )
char *addr1, *addr2;
{
	char c ;

	for (; addr1 <= addr2 ; addr1 += NBPPDE ) {
		c = *addr1;		/* fool the optimizer */
		*addr1 = c;		/* into doing *addr1 = *addr1 */
	}
}

/*
 *  Protect kernel text
 */
protext()
{
	pde_t  *pde, *endpde;

#ifdef	m88k
	pde = kvtopde(&M88Kvec[0]);
#else
	pde = kvtopde(&M68Kvec[0]);
#endif
	++pde;
	endpde = kvtopde(&etext);
	/*
	 * Note: At this point, the pde points 1k into
	 * a 4k page.  The following protects the remaining
	 * 3k of this page.  After that we use pg_setcw()
	 * that protects 4k at a time.
	 */
	pde->pgm.pg_wp = BITSET; ++pde;
	pde->pgm.pg_wp = BITSET; ++pde;
	pde->pgm.pg_wp = BITSET; ++pde;
	while(pde < endpde) {
		pg_setcw(pde);
		pde += NPDEPP;
	}
}
