/*
 *	Copyright (c) 1984 AT&T
 *	Copyright (c) 1987 Motorola Inc
 *	All Rights Reserved
 *	@(#)kern-port:os/sysm68k.c	35.6
 */

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF AT&T	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ident	"@(#)kern-port:os/sysm68k.c	35.6"

#include "sys/param.h"
#include "sys/types.h"
#include "sys/sysmacros.h"
#include "sys/sysm68k.h"
#include "sys/fs/s5dir.h"
#include "sys/signal.h"
#include "sys/systm.h"
#include "sys/utsname.h"
#include "sys/swap.h"
#include "sys/user.h"
#include "sys/errno.h"
#include "sys/mvmecpu.h"
#include "sys/mvme131.h"
#include "sys/mvme141.h"
#include "sys/reg.h"
#include "sys/todc.h"

extern int cpuboard;
extern short do_fpu;
extern char *fpu_ext;
extern char *fpu_sig;

#define S4_2    0x80

char *saveflag = &u.u_trapflag;		/* Flag to save/restore-the-frame */
long *saveaddr = &u.u_trapsave[0];	/* Area to save-the-frame in	*/
unsigned char stkfmtsz[16] = {		/* (# bytes-8) on stack frame	*/
	 0,  0,  4,  0, 0, 0, 0, 0,	/* for the exceptions		*/
	 0, 12, 24, 84, 0, 0, 0, 0};	/* index is frame type format	*/

/*
 * sysm68k - system call
 */


#ifdef	m88k
syslocal()
{
#else
sysm68k ()
{
	register int i, c, kind, *regs, *usp;
	int     cnt;
#endif

	struct a {
		int cmd;
		int arg1;
		int arg2;
		int arg3;
	} *uap = (struct a *) u.u_ap;


	switch (uap->cmd) {

#ifdef m88k
	case S88CONT: /* instruction continuation after signal handlers */
	{
		register i,kind, *regs, *usp;
		register struct fltinfo *fltptr;

		regs = u.u_ar0;
		usp = (int *) regs[SP];

		/* find the return address for the 'rtr' */
		/* in the user's stack area */
		/* stack format: */
		/*   0 - _sysm68k return addr */
		/*   1 - command */
		/*   2 - old %d0 */
		/*   3 - <kind of resume, previous trap process status reg> */
		/*   4 - previous trap PC */

 		/* get resume kind */
		kind = regs[R4];
		u.u_traptype = kind; /* make it appear that we just got here */
				     /* for the first time for psignal */

		regs[R31] = fuword((caddr_t) (usp+7));
		regs[SFIP] = fuword((caddr_t) (usp+5)); 
		regs[SNIP] = fuword((caddr_t) (usp+4)); 
		/* make sure we don't allow transfer to supervisor mode */
		regs[PS] = (regs[PS] & ~PS_C) |
			   (fuword((caddr_t) (usp+6)) & PS_C);
		regs[R4] = fuword((caddr_t) (usp+0));
		regs[R3] = u.u_rval2 = fuword((caddr_t) (usp+1)); 
		regs[R2] = u.u_rval1 = fuword((caddr_t) (usp+2));
		regs[R1] = fuword((caddr_t) (usp+3));

		fltptr = (struct fltinfo *)u.u_trapsave;
		for(i = 0; i < NUMTRAN; i++,fltptr++){
			if(fltptr->tran & TRN_VALID){
				if(!fault(fltptr->addr,u.u_procp,fltptr->tran & TRN_RW))
					dotran(regs,fltptr);
				else{
					psignal(u.u_procp,SIGSEGV);
					psig(fltptr->addr);
					return;
				}
			}
		}
		break;
	}

	case S88FPEX:	/* Floating Point Exception */

		/* TBD */
		break;

	case S88RTODC:	/* read TODC */
	{
		time_t	rtctime;
		extern	time_t clkread();

		if (!suser())
			break;

		rtctime = clkread();
		if (copyout((caddr_t)&rtctime,
		    (caddr_t)uap->arg1, sizeof(time_t)))
			u.u_error = EFAULT;
		break;
	}

	case S88STIME:	/* set internal time, not hardware clock */
		if (suser())
			time = (time_t) uap->arg1;
		break;
 
	case S88SETNAME:	/* rename the system */
	{
		register i, c;

		char	sysnamex[sizeof(utsname.sysname)];

		if (!suser())
			break;

		for (i = 0; (c = fubyte((caddr_t) uap->arg1 + i)) > 0
		    && i < sizeof(sysnamex) - 1; ++i)
			sysnamex[i] = c;

		if (c) {
			u.u_error = c < 0 ? EFAULT : EINVAL;
			break;
		}
		sysnamex[i] = '\0';
		str8cpy(utsname.sysname, sysnamex);
		str8cpy(utsname.nodename, sysnamex);
		break;
	} 

	case S88MEMSIZE:	/*  return the size of memory */
		u.u_rval1 = USRSTACK;
		u.u_rval2 = ctob(physmem);
		break;

	case S88SWAP:
	/*	General interface for adding, deleting, or
	**	finding out about swap files.  See swap.h
	**	for a description of the argument.
	**/
	{
		swpi_t	swpbuf;

		if (copyin(uap->arg1, &swpbuf, sizeof(swpi_t)) < 0) {
			u.u_error = EFAULT;
			return;
		}
		swapfunc(&swpbuf);
		break;
	}

	case S88CACHEON:	/* Enable CMMU caches */
#ifdef CMMU
		cacheon();
#endif
		break;

	case S88CACHEOFF:	/* disable CMMU caches */
#ifdef CMMU
		cacheoff();
#endif
		break;

	case S88CACHEFLUSH:	/*  */

		/* TBA for VME188 */
		break;


	case S88DELMEM:
	/*	Delete memory from the available list.  Forces
	**	tight memory situation for load testing.
	*/
		if (suser())
			delmem(uap->arg1);
		break;

	case S88ADDMEM:		/* Add back previously deleted memory */
		if (suser())
			addmem(uap->arg1);
		break;

	case S88HW:	/* Tell a user whether we have a coprocessor or not.  */

		/* TBD */

	default:
		u.u_error = EINVAL;
	}
}
#else

 	case S68STACK:	/* grow stack on the MC68000 */
		i = uap->arg1;
		if (i>0)
			if (grow(USRSTACK - ctob(u.u_ssize + btoc(i))) != -1)
				u.u_rval1 = USRSTACK - ctob(u.u_ssize);
			else
				u.u_error = ENOMEM;
		break;

	case S68CONT: /* instruction continuation after signal handlers */
		regs = u.u_ar0;
		usp = (int *) regs[SP];

		/* find the return address for the 'rtr' */
		/* in the user's stack area */
		/* stack format: */
		/*   0 - _sysm68k return addr */
		/*   1 - command */
		/*   2 - old %d0 */
		/*   3 - <kind of resume, previous trap process status reg> */
		/*   4 - previous trap PC */

 		/* get resume kind */
		kind = (fuword((caddr_t) (usp+3)) >> 16) & 0xffff;
		u.u_traptype = kind; /* make it appear that we just got here */
				     /* for the first time for psignal */

		regs[PC] = fuword((caddr_t) (usp+4)); 
		/* make sure we don't allow transfer to supervisor mode */
		regs[PS] = (regs[PS] & 0xff00) |
			   (fuword((caddr_t) (usp+3)) & 0xff);
		u.u_rval1 = fuword((caddr_t) (usp+2));	/* user's D0 */
		usp += 5;
		regs[SP] = (int) usp;
		if (kind)
			regs[PC] &= ~1;	/* make even again */
		if (kind == TRAPBUS) {  /* Return from bus error handler */
			if (u.u_trapsave[0] == 0)	{ /* no valid info */
				/* get rid of any user-specified handler */
				u.u_signal[SIGBUS - 1] = 0;
				psignal(u.u_procp, SIGBUS);
			} else
				u.u_trapflag = 1;     /* restore the frame */
		}

		if (kind == TRAPFPE) {  /* Return from MC68881 handler */
			if((u.u_fpu.ustate & UFPSAVE) == 0) { /* no info */
				/* get rid of any user-specified handler */
				u.u_signal[SIGFPE - 1] = 0;
				psignal(u.u_procp, SIGFPE);
			} else {
				cnt = ((fp_state *)(fpu_sig))->fp_siz;
				bcopy(fpu_sig, fpu_ext, cnt + 4);
				frestore(fpu_ext);
				u.u_trapflag = 2;     /* restore the frame */
						      /* do not call fixprr */
			}
			u.u_fpu.ustate &= ~UFPSAVE;
		}

		if (kind == TRAPADDR) { /* addr error */
			/* get rid of any user-specified handler */
			u.u_signal[SIGBUS - 1] = 0;
			psignal(u.u_procp, SIGBUS);
		}

		/* else we can just resume here */
		break;

/* The following code supports the "sysm68k" subcommand that
** returns the offending exception operand after an 881 exception.
*/
	case S68FPEX:	/* Floating Point Exception */
		if (!do_fpu)
			{u.u_error = EINVAL; break;}
		if ((u.u_fpu.ustate & UFPSAVE) == 0)
			{u.u_error = EINVAL; break;} /* Not in FPE handler */

		cnt = ((fp_state *)(fpu_ext))->fp_siz;
		if(copyout(fpu_sig + cnt - EX_OFFS, uap->arg1,
			sizeof(excopr)))
			u.u_error = EFAULT;
		break;

	case S68BIURW:  /* read/write BIU flags of 68881 idle frame */
		if (!do_fpu)
			{u.u_error = EINVAL; break;}
		if ((u.u_fpu.ustate & UFPSAVE) == 0)
			{u.u_error = EINVAL; break;}
		{
			int *biuptr;

			cnt = ((fp_state *)(fpu_ext))->fp_siz;
			biuptr = (int *)(fpu_sig + cnt);

			if(uap->arg1)  /* Read flags */
				u.u_rval1 = *biuptr;
			else {
				if(uap->arg2)
					*biuptr |= EXC_PEND;
				else
					*biuptr &= ~EXC_PEND;
			}
		}
		break;

	case S68RTODC:	/* read TODC */
	{
		time_t	rtctime;
		extern	time_t clkread();

		if (!suser())
			break;

		rtctime = clkread();
		if (copyout((caddr_t)&rtctime,
		    (caddr_t)uap->arg1, sizeof(time_t)))
			u.u_error = EFAULT;
		break;
	}

	case S68TODCSTAT:  /* return status of time of day clock */
	{
		extern  unsigned char tdcinfo;

		u.u_rval1 = tdcinfo;
		break;
	}

	case S68STIME:	/* set internal time and hardware clock */
	{
		extern  int todc();

		if (suser()) {
			(void) todc(SETTIME, &uap->arg1);
			time = (time_t) uap->arg1;
		}
		break;
	}
 
	case S68SETNAME:	/* rename the system */
	{
		char	sysnamex[sizeof(utsname.sysname)];

		if (!suser())
			break;

		for (i = 0; (c = fubyte((caddr_t) uap->arg1 + i)) > 0
		    && i < sizeof(sysnamex) - 1; ++i)
			sysnamex[i] = c;

		if (c) {
			u.u_error = c < 0 ? EFAULT : EINVAL;
			break;
		}
		sysnamex[i] = '\0';
		str8cpy(utsname.sysname, sysnamex);
		str8cpy(utsname.nodename, sysnamex);
		break;
	} 

	case S68MEMSIZE:	/*  return the size of memory */
		u.u_rval1 = USRSTACK;
		u.u_rval2 = ctob(physmem);
		break;

	case S68SWAP:
	/*	General interface for adding, deleting, or
	**	finding out about swap files.  See swap.h
	**	for a description of the argument.
	**/
	{
		swpi_t	swpbuf;

		if (copyin(uap->arg1, &swpbuf, sizeof(swpi_t)) < 0) {
			u.u_error = EFAULT;
			return;
		}
		swapfunc(&swpbuf);
		break;
	}

	case S68CACHESET: {	/* Set chip cache for the MC680x0 */
		cacr_t j;

		j.caci.cacr_cacr = uap->arg1;
		if (cputype == PROC030) {
		/* Do checks for 030 data cache */
			if (j.cacm.cacr_ed
			    && (!j.cacm.cacr_cd || !j.cacm.cacr_wa)
			    || j.cacm.cacr_dbe && !j.cacm.cacr_ed
			    || j.cacm.cacr_fd || j.cacm.cacr_ced) {
				u.u_error = EINVAL;
				return;
			}
		}
		else if (cputype != PROC020) {
			u.u_error = EINVAL;
			return;
		}
		/* Do checks for both 020 and 030 instruction cache */
		if (j.cacm.cacr_ei && !j.cacm.cacr_ci
		    || j.cacm.cacr_ibe && !j.cacm.cacr_ei
		    || j.cacm.cacr_fi || j.cacm.cacr_cei
		    || j.cacm.cacr_z0 || j.cacm.cacr_z1) {
			u.u_error = EINVAL;
			return;
		}

		if (suser()) {
			u.u_rval1 = cacheset(j);
		}
		break;
	}

	case S68CACHERD: 	/* Read chip cache for the MC680x0 */
		u.u_rval1 = cacherd();
		break;

	case S68ICACHEON:	/* Enable internal cache of the MC680x0 */
		if (suser())
			u.u_rval1 = icacheon();
		break;

	case S68ICACHEOFF:	/* Disable internal cache of the MC680x0 */
		if (suser())
			u.u_rval1 = icacheoff();
		break;

	case S68BCACHEON:	/* Enable board cache of the MVME131XT */
				/* and the MVME141 */
		if (!suser())
			break;
		switch (cpuboard) {
		case VME131:
			*XCMR = uap->arg1 & 0x0f;
			*XCCR = 0xf1;
			break;

		case VME141: {
			csr_ab_t ab;

			  /* clear cclr, cwen*, and cren* bits */
			ab.csr_abi = V141_CSR_AB->csr_abi;
			ab.csr_abm.csr_ab.cclr = 0;
			ab.csr_abm.csr_ab.cwen_ = 0;
			ab.csr_abm.csr_ab.cren_ = 0;
			V141_CSR_AB->csr_abi = ab.csr_abi;
			break;
		}
		default:
			u.u_error = EINVAL;
		}
		break;

	case S68BCACHEOFF:	/* Disable board cache of the MVME131XT */
				/* and the MVME141 */
		if (!suser())
			break;
		switch (cpuboard) {
		case VME131:
			*XCCR = 0xff;
			break;

		case VME141: {
			csr_ab_t ab;

			  /* set cclr, cwen*, and cren* bits */
			ab.csr_abi = V141_CSR_AB->csr_abi;
			ab.csr_abm.csr_ab.cclr = 1;
			ab.csr_abm.csr_ab.cwen_ = 1;
			ab.csr_abm.csr_ab.cren_ = 1;
			V141_CSR_AB->csr_abi = ab.csr_abi;
			break;
		}

		default:
			u.u_error = EINVAL;
		}
		break;

	case S68WPOSTON:	/* Enable write posting on the MVME141 */
		if (!suser())
			break;
		switch (cpuboard) {
		case VME141: {
			csr_ab_t ab;

			  /* clear lwpen* bit */
			ab.csr_abi = V141_CSR_AB->csr_abi;
			ab.csr_abm.csr_ab.lwpen_ = 0;
			V141_CSR_AB->csr_abi = ab.csr_abi;
			break;
		}

		default:
			u.u_error = EINVAL;
		}
		break;

	case S68WPOSTOFF:	/* Disable write posting on the MVME141 */
		if (!suser())
			break;
		switch (cpuboard) {
		case VME141: {
			csr_ab_t ab;

			  /* set lwpen* bit */
			ab.csr_abi = V141_CSR_AB->csr_abi;
			ab.csr_abm.csr_ab.lwpen_ = 1;
			V141_CSR_AB->csr_abi = ab.csr_abi;
			break;
		}

		default:
			u.u_error = EINVAL;
		}
		break;

	case S68DELMEM:
	/*	Delete memory from the available list.  Forces
	**	tight memory situation for load testing.
	*/
		if (suser())
			delmem(uap->arg1);
		break;

	case S68ADDMEM:		/* Add back previously deleted memory */
		if (suser())
			addmem(uap->arg1);
		break;

	case S68FPHW:	/* Tell a user whether we have a coprocessor or not.  */
		if (do_fpu) {
			if (suword(uap->arg1, MC68881) == -1)
				u.u_error = EFAULT;
		} else {
			if (suword(uap->arg1, NOFPHW) == -1)
				u.u_error = EFAULT;
		}
		break;

	case S68BRDSTAT:    /* Return structure for board id if available */
		if(boardid) {
		    register unsigned int size;
		    struct boardid bugstat;

		    size = uap->arg2;
		    if (size > sizeof(struct boardid)) {
			u.u_error = EINVAL;
			break;
		    }

		    if (size == 0) { /* If zero simply do nothing */
			break;
		    }
			
		    bcopy((caddr_t)boardid, (caddr_t)&bugstat,
		      sizeof(struct boardid));
		    bugstat.bd_ssid = NULL;
		    if (copyout((caddr_t)&bugstat, (caddr_t)(uap->arg1), size)
		      < 0) {
			u.u_error = EFAULT;
			break;
		    }
		}
		else {
			u.u_error = ENODEV;
		}
		break;

	case S68CPUBRD:	/* Tell a user which cpu board we're currently on.  */
		u.u_rval1 = cpuboard;
		break;

	default:
		u.u_error = EINVAL;
	}
}
#endif
