/*	Copyright (c) 1987 Motorola Inc
	All Rights Reserved
	@(#)kern-port:os/trap.c	35.5
*/
#ident	"@(#)kern-port:os/trap.c	35.5"

/*	Copyright (c) 1984 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF AT&T	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#include "sys/sysmacros.h"
#include "sys/param.h"
#include "sys/types.h"
#include "sys/systm.h"
#include "sys/fs/s5dir.h"
#include "sys/signal.h"
#include "sys/reg.h"
#include "sys/psw.h"
#include "sys/immu.h"
#include "sys/trap.h"
#include "sys/region.h"
#include "sys/user.h"
#include "sys/errno.h"
#include "sys/proc.h"
#include "sys/sysinfo.h"
#include "sys/utsname.h"
#include "sys/cmn_err.h"
#include "sys/mvmecpu.h"

extern unsigned nofulow();
extern unsigned nofuhigh();
extern unsigned errret();

extern int bootstate;	/* rfs state: up or down */

#ifdef	m88k
/*
 * Called from the trap handler when a processor trap occurs.
 */
trap(regptr,type)
	int *regptr;
	register type;
{
	int *old_ar0;		/* old ar0		*/
	register sig = 0;
	register struct proc *pp;
	time_t syst;
	register code;
	int inst, s1, s2;
	
	syst = u.u_stime;
	pp = u.u_procp;
	old_ar0 = u.u_ar0;	/* save old ar0 and restore on return	      */
	u.u_ar0 = regptr;
	code = u.u_ar0[PC];		/* pc is the most common use */
	u.u_traptype = TRAPNORM;
	sysinfo.trap++;

	if (USERMODE(u.u_ar0[PS]))
		type |= USER;
	switch (type) {

	/*
	 * Trap not expected.
	 * Usually a kernel mode bus error.
	 * If user mode, signal trap.
	 */
	default:
		if (USERMODE(u.u_ar0[PS])){
			sig = SIGILL;
			break;
		}
		spl7();
		cmn_err(CE_WARN,"       *** Kernel Trap *** ");
		cmn_err(CE_WARN,"proc = 0x%x, user = 0x%x",u.u_procp,&u);
		cmn_err(CE_WARN,"pid  = 0x%x, code = 0x%x",u.u_procp->p_pid,code);
		cmn_err(CE_WARN,"pc   = 0x%x, ps   = 0x%x",code,u.u_ar0[PS]);
		cmn_err(CE_WARN,"argp = 0x%x, aps  = 0x%x",regptr,&u.u_ar0[PS]);
		cmn_err(CE_PANIC,"   *** Unknown Trap Type = 0x%x ***", type);

	case RESCHED + USER:	/* Allow process switch */
		sysinfo.resched++;
		qswtch();
		goto out;

	case SFU1PR+USER:      /* Precise floating point exception come here */
		sig = SIGFPE;
		if(!fpup()) {  /* Assume that fpup return sig (0=nosig) flag */
			sig = 0;
		}
		break;
	case SFU1IMP+USER:
		sig = SIGFPE;
		if(!fpui()) {  /* Assume that fpui return sig (0=nosig) flag */
			sig = 0;
		}
		break;

	case ZDVDERR + USER:	/* zero divide check */
	case ZDVDERR:	/* zero divide check */
		
	    if (type == ZDVDERR)
		inst = * (long *) u.u_ar0[SCIP] & ~3;
	    else
		inst = fuiword(u.u_ar0[SCIP] & ~3);
	    
		if((inst) > 0){  /* If literal*/
			s2 = inst & 0xFFFF;
			if (s2 & 0x8000)
				s2 |= 0xFFFF0000;
		}
		else
			s2 = u.u_ar0[inst & 0x1F];

		s1 = u.u_ar0[(inst >> 16) & 0x1F];

		if( s2 != 0){		/* Signed divide exception. */
			if(s2 > 0){
				s1 *= -1;
				s1 /= s2;
				u.u_ar0[(inst >> 21) & 0x1f] = s1 * -1;
			}
			else if (s1 > 0) {
				s2 *= -1;
				s1 /= s2;
				u.u_ar0[(inst >> 21) & 0x1f] = s1 * -1;
			}
			else{
				s1 *= -1;
				s2 *= -1;
				u.u_ar0[(inst >> 21) & 0x1f] = s1 / s2;
			}
			if(type == ZDVDERR){	/* If in kernel, return. */
				u.u_ar0 = old_ar0;
				return;
			}

			break;
		}
		else{ 		/* Zero divide error */
			if(type == ZDVDERR){	/* If in kernel, panic. */
				cmn_err(CE_PANIC,"   *** Kernel Divide by zero pc: 0x%x ***", u.u_ar0[SCIP]);
			}
			sig = SIGFPE;
			code = KINTDIV;
		}
		break;

	case CHKTRAP + USER:	/* tbnd trap */
		sig = SIGFPE;
		code = KSUBRNG;
		break;

	case ADDRERR + USER:	/* address error */
		sig = SIGBUS;
		code = u.u_ar0[PC];
		u.u_traptype = TRAPADDR;
		break;

	case BPTFLT + USER:	/* bpt instruction fault */
		/* fall thru */
	case TRCTRAP + USER:	/* trace trap */

		u.u_ar0[SFIP] = u.u_ar0[SNIP];    /* Back up IPs */
		u.u_ar0[SNIP] = u.u_ar0[SCIP];	  

		if(u.u_trace[1].addr != -1){
			settrace(u.u_trace[1].addr, u.u_trace[1].inst);
			u.u_trace[1].addr = -1;
		}
		if(u.u_trace[0].addr != -1){
			settrace(u.u_trace[0].addr, u.u_trace[0].inst);
			u.u_trace[0].addr = -1;
		}

		sig = SIGTRAP;
		break;

	case IOTTRAP + USER:
		sig = SIGIOT;
		break;

	case PRIVFLT + USER:	/* privileged instruction fault */
	case INSTERR + USER:	/* Illegal instruction */
		sig = SIGILL;
		break;

	}

	if (sig)
		psignal(pp, sig);

	if (runrun != 0)
		qswtch();
out:
	curpri = pp->p_pri = calcppri(pp);

	if ((pp->p_flag & SPRSTOP) || (pp->p_sig && issig())) {
		psig(code);
		curpri = pp->p_pri = calcppri(pp);
	}

	if(u.u_prof.pr_scale)
		addupc((caddr_t)u.u_ar0[PC], &u.u_prof, (int)(u.u_stime-syst));
}

/* System call preprocessing */
syscall(arg1,arg2,arg3,arg4,arg5,arg6, regptr, code)
	int  *regptr;
	register code;
{

	register 		i;
	register struct sysent	*callp;
	register struct proc	*pp;
	time_t			syst;
	short			pid;

	syst = u.u_stime;
	pid = u.u_procp->p_pid;
	u.u_traptype = TRAPNORM;
	u.u_ar0 = regptr;
	sysinfo.syscall++;
	u.u_error = 0;

	u.u_syscall = code;
	callp = &sysent[code];

	u.u_arg[0] = arg1;
	u.u_arg[1] = arg2;
	u.u_arg[2] = arg3;
	u.u_arg[3] = arg4;
	u.u_arg[4] = arg5;
	u.u_arg[5] = arg6;

	u.u_dirp = (caddr_t)u.u_arg[0];
	u.u_rval1 = 0;
	u.u_rval2 = u.u_ar0[R3];
	u.u_ap = u.u_arg;

#define DEBUG
#ifdef DEBUG
	startsyscall();
#endif

	if ((bootstate || !callp->sy_setjmp) && setjmp(u.u_qsav)) {
		if (!(u.u_rflags & U_RSYS) && !u.u_error)
			u.u_error = EINTR;
		spl0();
	} else
		(*callp->sy_call)();

	u.u_rflags &= ~(U_RSYS | U_DOTDOT | U_LBIN);

#ifdef DEBUG
	endsyscall();
#endif

	if (u.u_error) {
		u.u_ar0[R2] = u.u_error;
		u.u_error = 0;
		if (++u.u_errcnt > 16) {
			u.u_errcnt = 0;
			runrun++;
		}
	} else {
		if(u.u_arg[0] == 2 && (code == 38 || code == 50)){   /* Don't do this on sig cont*/
			u.u_ar0[R2] = u.u_rval1; /* setup return values */
			u.u_ar0[R3] = u.u_rval2;  /* Old register contents */
		}
		else{
			u.u_ar0[R2] = u.u_rval1; /* setup return values */
			u.u_ar0[R3] = u.u_rval2;
			u.u_ar0[SFIP] = (u.u_ar0[SCIP] + 8) | VALIDIP;
			u.u_ar0[SNIP] = (u.u_ar0[SCIP] + 8) & ~VALIDIP; /* bump pc */
		}
	}

	pp = u.u_procp;
	curpri = pp->p_pri = calcppri(pp);

	if (runrun != 0)
		qswtch();

	if (!u.u_trapflag && ((pp->p_flag & SPRSTOP)||(pp->p_sig && issig()))) {
		psig((fuiword(u.u_ar0[SCIP] & ~3) & 0x1FF) - 256);
		curpri = pp->p_pri = calcppri(pp);
	}

	/*	If pid != pp->p_pid, then we are the child
	**	returning from a fork system call.  In this
	**	case, ignore syst since our time was reset
	**	in fork.
	*/

	if (u.u_prof.pr_scale)
		addupc((caddr_t)u.u_ar0[PC], &u.u_prof, 
			pid == pp->p_pid ? (int)(u.u_stime - syst)
					 : (int)u.u_stime);
}

#ifdef DEBUG

int scdebug;
char scflag[128];

char *
getusrstr(p)
char *p;
{
	static char buf[512];
	char *bufp = buf;

	while (*bufp++ = fubyte(p++))
		;
	*bufp = 0;
	return(buf);
}


startsyscall()
{
	int a1 = u.u_arg[0];
	int a2 = u.u_arg[1];
	int a3 = u.u_arg[2];
	int a4 = u.u_arg[3];
	int i;

	if (!scdebug)
		return;
	if (!scflag[u.u_syscall])
		return;

	printf("begin syscall,\tpid %d,\t", u.u_procp->p_pid);
	switch (u.u_syscall) {
	case 0:
		printf("nosys()\n", a1, a2, a3, a4);
		break;
	case 1:
		printf("exit(%d)\n", a1, a2, a3, a4);
		break;
	case 2:
		printf("fork()\n", a1, a2, a3, a4);
		break;
	case 3:
		printf("read(%d, 0x%x, %d)\n", a1, a2, a3, a4);
		break;
	case 4:
		printf("write(%d, 0x%x, %d)\n", a1, a2, a3, a4);
		break;
	case 5:
		printf("open(\"%s\", 0x%x)\n", getusrstr(a1), a2, a3, a4);
		break;
	case 6:
		printf("close(%d)\n", a1, a2, a3, a4);
		break;
	case 7:
		printf("wait(0x%x)\n", a1, a2, a3, a4);
		break;
	case 8:
		printf("creat(\"%s\", 0%o)\n", getusrstr(a1), a2, a3, a4);
		break;
	case 9:
		printf("link(\"%s\", \"%s\")\n", getusrstr(a1), getusrstr(a2), a3, a4);
		break;
	case 10:
		printf("unlink(\"%s\")\n", getusrstr(a1), a2, a3, a4);
		break;
	case 11:
		printf("exec(\"%s\")\n", getusrstr(a1), a2, a3, a4);
		break;
	case 12:
		printf("chdir(\"%s\")\n", getusrstr(a1), a2, a3, a4);
		break;
	case 13:
		printf("time(0x%x)\n", a1, a2, a3, a4);
		break;
	case 14:
		printf("mknod(\"%s\", 0%o, 0x%x)\n", getusrstr(a1), a2, a3, a4);
		break;
	case 15:
		printf("chmod(\"%s\", 0%o)\n", getusrstr(a1), a2, a3, a4);
		break;
	case 16:
		printf("chown(\"%s\", %d, %d)\n", getusrstr(a1), a2, a3, a4);
		break;
	case 17:
		printf("brk(0x%x)\n", a1, a2, a3, a4);
		break;
	case 18:
		printf("stat(\"%s\", 0x%x)\n", getusrstr(a1), a2, a3, a4);
		break;
	case 19:
		printf("lseek(%d, 0x%x, %d)\n", a1, a2, a3, a4);
		break;
	case 20:
		printf("getpid()\n", a1, a2, a3, a4);
		break;
	case 21:
		printf("mount(\"%s\", \"%s\", 0x%x, 0x%x)\n", getusrstr(a1), getusrstr(a2), a3, a4);
		break;
	case 22:
		printf("umount(\"%s\")\n", getusrstr(a1), a2, a3, a4);
		break;
	case 23:
		printf("setuid(%d)\n", a1, a2, a3, a4);
		break;
	case 24:
		printf("getuid()\n", a1, a2, a3, a4);
		break;
	case 25:
		printf("stime(0x%x)\n", a1, a2, a3, a4);
		break;
	case 26:
		printf("ptrace(%d, %d, 0x%x, 0x%x)\n", a1, a2, a3, a4);
		break;
	case 27:
		printf("alarm(%d)\n", a1, a2, a3, a4);
		break;
	case 28:
		printf("fstat(%d, 0x%x)\n", a1, a2, a3, a4);
		break;
	case 29:
		printf("pause()\n", a1, a2, a3, a4);
		break;
	case 30:
		printf("utime(\"%s\", 0x%x)\n", getusrstr(a1), a2, a3, a4);
		break;
	case 31:
		printf("stty()\n", a1, a2, a3, a4);
		break;
	case 32:
		printf("gtty()\n", a1, a2, a3, a4);
		break;
	case 33:
		printf("access(\"%s\", 0%o)\n", getusrstr(a1), a2, a3, a4);
		break;
	case 34:
		printf("nice(%d)\n", a1, a2, a3, a4);
		break;
	case 35:
		printf("statfs(\"%s\", 0x%x, %d, %d)\n", getusrstr(a1), a2, a3, a4);
		break;
	case 36:
		printf("sync()\n", a1, a2, a3, a4);
		break;
	case 37:
		printf("kill(%d, %d)\n", a1, a2, a3, a4);
		break;
	case 38:
		printf("sysm68k()\n", a1, a2, a3, a4);
		break;
	case 39:
		printf("setpgrp()\n", a1, a2, a3, a4);
		break;
	case 40:
		printf("nosys()\n", a1, a2, a3, a4);
		break;
	case 41:
		printf("dup(%d)\n", a1, a2, a3, a4);
		break;
	case 42:
		printf("pipe()\n", a1, a2, a3, a4);
		break;
	case 43:
		printf("times(0x%x)\n", a1, a2, a3, a4);
		break;
	case 44:
		printf("profil(0x%x, %d, %d, %d)\n", a1, a2, a3, a4);
		break;
	case 45:
		printf("plock(%d)\n", a1, a2, a3, a4);
		break;
	case 46:
		printf("setgid()\n", a1, a2, a3, a4);
		break;
	case 47:
		printf("getgid()\n", a1, a2, a3, a4);
		break;
	case 48:
		printf("signal(%d, 0x%x)\n", a1, a2, a3, a4);
		break;
	case 49:
		printf("sysmsg(0x%x, 0x%x 0x%x)\n", a1, a2, a3, a4);
		break;
	case 50:
		printf("sysm68k(%d, 0x%x, 0x%x)\n", a1, a2, a3, a4);
		break;
	case 51:
		printf("acct(\"%s\")\n", getusrstr(a1), a2, a3, a4);
		break;
	case 52:
		printf("shm(0x%x, 0x%x, 0x%x, 0x%x)\n", a1, a2, a3, a4);
		break;
	case 53:
		printf("sem(0x%x, 0x%x, 0x%x, 0x%x)\n", a1, a2, a3, a4);
		break;
	case 54:
		printf("ioctl(%d, 0x%x, 0x%x)\n", a1, a2, a3, a4);
		break;
	case 55:
		printf("uadmin(%d, 0x%x, 0x%x)\n", a1, a2, a3, a4);
		break;
	case 56:
		printf("locking()\n", a1, a2, a3, a4);
		break;
	case 57:
		printf("utssys()\n", a1, a2, a3, a4);
		break;
	case 58:
		printf("nosys()\n", a1, a2, a3, a4);
		break;
	case 59:
		printf("exece()\n", a1, a2, a3, a4);
		break;
	case 60:
		printf("umask(o%o)\n", a1, a2, a3, a4);
		break;
	case 61:
		printf("chroot(\"%s\")\n", getusrstr(a1), a2, a3, a4);
		break;
	case 62:
		printf("fcntl(%d, 0x%x, 0x%x)\n", a1, a2, a3, a4);
		break;
	case 63:
		printf("ulimit(%d, 0x%x)\n", a1, a2, a3, a4);
		break;
	
	case 64:
		printf("nosys()\n", a1, a2, a3, a4);
		break;
	case 65:
		printf("nosys()\n", a1, a2, a3, a4);
		break;
	case 66:
		printf("nosys()\n", a1, a2, a3, a4);
		break;
	case 67:
		printf("nosys()\n", a1, a2, a3, a4);
		break;
	case 68:
		printf("nosys()\n", a1, a2, a3, a4);
		break;
	case 69:
		printf("nosys()\n", a1, a2, a3, a4);
		break;
	
	case 70:
		printf("advfs()\n", a1, a2, a3, a4);
		break;
	case 71:
		printf("unadvfs()\n", a1, a2, a3, a4);
		break;
	case 72:
		printf("rmount()\n", a1, a2, a3, a4);
		break;
	case 73:
		printf("rumount()\n", a1, a2, a3, a4);
		break;
	case 74:
		printf("rfstart()\n", a1, a2, a3, a4);
		break;
	case 75:
		printf("nosys()\n", a1, a2, a3, a4);
		break;
	case 76:
		printf("rdebug()\n", a1, a2, a3, a4);
		break;
	case 77:
		printf("rfstop()\n", a1, a2, a3, a4);
		break;
	case 78:
		printf("rfsys()\n", a1, a2, a3, a4);
		break;
	case 79:
		printf("rmdir(\"%s\")\n", getusrstr(a1), a2, a3, a4);
		break;
	case 80:
		printf("mkdir(\"%s\", 0%o)\n", getusrstr(a1), a2, a3, a4);
		break;
	case 81:
		printf("getdents(0x%x, 0x%x, %d)\n", a1, a2, a3, a4);
		break;
	case 82:
		printf("fstatfs(%d, 0x%x, %d, %d)\n", a1, a2, a3, a4);
		break;
	case 83:
		printf("nosys()\n", a1, a2, a3, a4);
		break;
	case 84:
		printf("sysfs(%d, 0x%x, 0x%x)\n", a1, a2, a3, a4);
		break;
	case 85:
		printf("getmsg(%d, 0x%x, 0x%x, 0x%x)\n", a1, a2, a3, a4);
		break;
	case 86:
		printf("putmsg(%d, 0x%x, 0x%x, 0x%x)\n", a1, a2, a3, a4);
		break;
	case 87:
		printf("poll(0x%x, %d, %d)\n", a1, a2, a3, a4);
		break;
	case 88:
		printf("socket()\n", a1, a2, a3, a4);
		break;
	case 89:
		printf("bind()\n", a1, a2, a3, a4);
		break;
	case 90:
		printf("connect()\n", a1, a2, a3, a4);
		break;
	case 91:
		printf("listen()\n", a1, a2, a3, a4);
		break;
	case 92:
		printf("accept()\n", a1, a2, a3, a4);
		break;
	case 93:
		printf("getsockname()\n", a1, a2, a3, a4);
		break;
	case 94:
		printf("getpeername()\n", a1, a2, a3, a4);
		break;
	case 95:
		printf("writev()\n", a1, a2, a3, a4);
		break;
	case 96:
		printf("sendto	()\n", a1, a2, a3, a4);
		break;
	case 97:
		printf("sendmsg()\n", a1, a2, a3, a4);
		break;
	case 98:
		printf("recv()\n", a1, a2, a3, a4);
		break;
	case 99:
		printf("recvfrom()\n", a1, a2, a3, a4);
		break;
	case 100:
		printf("recvmsg()\n", a1, a2, a3, a4);
		break;
	case 101:
		printf("select()\n", a1, a2, a3, a4);
		break;
	case 102:
		printf("shutdown()\n", a1, a2, a3, a4);
		break;
	case 103:
		printf("setsockopt()\n", a1, a2, a3, a4);
		break;
	case 104:
		printf("getsockopt()\n", a1, a2, a3, a4);
		break;
	case 105:
		printf("readv()\n", a1, a2, a3, a4);
		break;
	case 106:
		printf("writev()\n", a1, a2, a3, a4);
		break;
	default:
		printf("nosys()\n", a1, a2, a3, a4);
		break;
	}

	for (i=0; i<scdebug; i++)
		;
}

endsyscall()
{
	char *f = "failed";
	char *s = "succeeded";
	int e = u.u_error;
	int i;

	if (!scdebug)
		return;
	
	if (!scflag[u.u_syscall])
		return;

	printf("end syscall,\tpid %d,\t", u.u_procp->p_pid);

	switch (u.u_syscall) {
	case 0:
		printf("nosys %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 1:
		printf("exit %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 2:
		printf("fork %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 3:
		printf("read %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 4:
		printf("write %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 5:
		printf("open %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 6:
		printf("close %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 7:
		printf("wait %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 8:
		printf("creat %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 9:
		printf("link %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 10:
		printf("unlink %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 11:
		printf("exec %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 12:
		printf("chdir %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 13:
		printf("time %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 14:
		printf("mknod %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 15:
		printf("chmod %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 16:
		printf("chown %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 17:
		printf("brk %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 18:
		printf("stat %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 19:
		printf("lseek %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 20:
		printf("getpid %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 21:
		printf("mount %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 22:
		printf("umount %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 23:
		printf("setuid %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 24:
		printf("getuid %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 25:
		printf("stime %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 26:
		printf("ptrace %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 27:
		printf("alarm %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 28:
		printf("fstat %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 29:
		printf("pause %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 30:
		printf("utime %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 31:
		printf("stty %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 32:
		printf("gtty %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 33:
		printf("access %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 34:
		printf("nice %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 35:
		printf("statfs %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 36:
		printf("sync %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 37:
		printf("kill %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 38:
		printf("sysm68k %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 39:
		printf("setpgrp %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 40:
		printf("nosys %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 41:
		printf("dup %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 42:
		printf("pipe %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 43:
		printf("times %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 44:
		printf("profil %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 45:
		printf("plock %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 46:
		printf("setgid %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 47:
		printf("getgid %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 48:
		printf("signal %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 49:
		printf("sysmsg %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 50:
		printf("sysm68k %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 51:
		printf("acct %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 52:
		printf("shm %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 53:
		printf("sem %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 54:
		printf("ioctl %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 55:
		printf("uadmin %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 56:
		printf("locking %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 57:
		printf("utssys %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 58:
		printf("nosys %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 59:
		printf("exece %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 60:
		printf("umask %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 61:
		printf("chroot %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 62:
		printf("fcntl %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 63:
		printf("ulimit %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	
	case 64:
		printf("nosys %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 65:
		printf("nosys %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 66:
		printf("nosys %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 67:
		printf("nosys %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 68:
		printf("nosys %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 69:
		printf("nosys %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	
	case 70:
		printf("advfs %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 71:
		printf("unadvfs %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 72:
		printf("rmount %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 73:
		printf("rumount %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 74:
		printf("rfstart %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 75:
		printf("nosys %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 76:
		printf("rdebug %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 77:
		printf("rfstop %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 78:
		printf("rfsys %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 79:
		printf("rmdir %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 80:
		printf("mkdir %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 81:
		printf("getdents %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 82:
		printf("fstatfs %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 83:
		printf("nosys %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 84:
		printf("sysfs %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 85:
		printf("getmsg %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 86:
		printf("putmsg %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 87:
		printf("poll %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 88:
		printf("socket %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 89:
		printf("bind %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 90:
		printf("connect %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 91:
		printf("listen %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 92:
		printf("accept %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 93:
		printf("getsockname %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 94:
		printf("getpeername %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 95:
		printf("writev %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 96:
		printf("sendto	 %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 97:
		printf("sendmsg %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 98:
		printf("recv %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 99:
		printf("recvfrom %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 100:
		printf("recvmsg %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 101:
		printf("select %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 102:
		printf("shutdown %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 103:
		printf("setsockopt %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 104:
		printf("getsockopt %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 105:
		printf("readv %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	case 106:
		printf("writev %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	default:
		printf("sysfs %s, returned %d\n", e?f:s, u.u_rval1);
		break;
	}

	for (i=0; i<scdebug; i++)
		;
}
#endif




/*
 * Called from the trap handler when a processor bus error occurs.
 *	    return value		Meaning
 *	    	 0	   Faulting page has been fixed, continue inst.
 *	    	value	   Abort cycle.  Clean up stack frame.
 *				New pc has been assigned.
 */


buserr(regptr)
	int *regptr;
{
	register typesig = 0;	/* signal type		*/
	register proc_t	*p;	/* faulting process	*/
	register int	ssw;	/* special status work	*/
	register int	vaddr;	/* faulting virt addr	*/
	register int	berrsrc;/* STAT2 bus error reg	*/
	register int	write;	/* write flag		*/
	register int	*ppc;	/* pointer to pc	*/
	int 	i;
	time_t syst;		/* time of fault	*/
	int *old_ar0;		/* old ar0		*/
	int old_hold;		/* old held signals	*/
	struct fltinfo *fltptr;

	minfo.buserr++;
	p = u.u_procp;
	u.u_traptype = TRAPNORM;
	old_ar0 = u.u_ar0;	/* save old ar0 and restore on return	      */
	u.u_ar0 = regptr;	/* set up new ar0 for current regs	      */
	ppc = &u.u_ar0[SCIP];	/* address of pc - referenced very often      */

	/*
	 * Calculate the address of the bus error 
	 *	This is the logical address to the MMU
	 */

#ifdef	ASPEN
	berrsrc = *SSR;	/* Get the contents  of the system status reg*/
#endif
	syst = u.u_stime;

/*  Check for code faults. */
	if(*ppc & EXCEPT_IP){
		if(USERMODE(u.u_ar0[PS])){   /*  User code fault */
			regptr[SFIP] = regptr[SNIP] & ~EXCEPT_IP;
			regptr[SNIP] = *ppc & ~EXCEPT_IP;  /* Rerun fault*/
			if(typesig = fault(*ppc,p,0)){
				u.u_trapflag = 1;	/* copy fault info. to u area */
				psignal(p, typesig);
				old_hold = p->p_hold;
				p->p_hold = ~( (1L<<(SIGSEGV-1)) | (1L<<(SIGKILL-1)) |
				       (1L<<(SIGBUS-1)) );
					/* except SIGBUS, SIGSEGV & SIGKILL */
				if ((p->p_flag & SPRSTOP) || (p->p_sig && issig()))
					psig(*ppc);
				p->p_hold = old_hold;	/* reset held signals */
				goto out;
			}
		}
		else {  /* Kernel mode code fault. */
			cmn_err(CE_WARN,"pc   = 0x%x, ps   = 0x%x",*ppc,u.u_ar0[PS]);
			cmn_err(CE_PANIC,"       *** Kernel Code fault *** ");
			
		}
	}

/* Check for data access faults. */
	if(u.u_ar0[PS] & PS_DEXC){  /* Fault? */

		/* Only TRAN1 is known to be faulted.  If its kernel, die */
		if(regptr[TRAN1] & TRN_MD) {   /* kernel data address fault? */
			if(*ppc >= nofulow && *ppc <= nofuhigh){
				/* Must be read of nomem from /dev/mem */
				regptr[SFIP] = (int)errret | VALIDIP;
				regptr[SNIP] = ~VALIDIP;
				if (u.u_caddrflt){
					regptr[SFIP] = u.u_caddrflt | VALIDIP;
					regptr[SNIP] = ~VALIDIP;
				}
				regptr[TRAN1] &= ~TRN_VALID;  /* Don't rerun. */
			}
			else{
				cmn_err(CE_WARN,"pc= 0x%x, ps= 0x%x, vaddr= %x",
				*ppc, u.u_ar0[PS],regptr[ADDR1]);
				cmn_err(CE_PANIC,"       *** Kernel data fault *** ");
			}
		}

		u.u_ar0[PS] &= ~PS_DEXC;    /* clear it */
		for(i = 0, fltptr = &regptr[TRAN1]; i < NUMTRAN; i++, fltptr++){
			if(fltptr->tran & TRN_VALID) {    /* If not NOP */
				if(!(fltptr->tran & TRN_MD) && (typesig = fault(fltptr->addr, p,(fltptr ->tran & TRN_RW)))) {  
					if(u.u_ar0[PS] & PS_SUP && *ppc >= nofulow && *ppc <= nofuhigh){
						/* fault during userio */
						regptr[SFIP] = (int)errret | VALIDIP;
						regptr[SNIP] = ~VALIDIP;
						if (u.u_caddrflt){
							regptr[SFIP] = u.u_caddrflt | VALIDIP;
							regptr[SNIP] = ~VALIDIP;
						}
						
					}
					else{
						/* User fault, send signal */
						
						u.u_traptype = TRAPBUS;	/* bus error trap type */
						u.u_trapflag = 1;	/* copy fault info. to u area */

						psignal(p, typesig);	/* post bus error signal */
						old_hold = p->p_hold;	/* Hold off all other signals */
						p->p_hold = ~( (1L<<(SIGSEGV-1)) | (1L<<(SIGKILL-1)) |
				       			(1L<<(SIGBUS-1)) );
						/* except SIGBUS, SIGSEGV & SIGKILL */
						if ((p->p_flag & SPRSTOP) || (p->p_sig && issig()))
							psig(fltptr->addr);

						p->p_hold = old_hold;	/* reset held signals */
						goto out;
					}
				}
				else {  /* paged in or supervisor, complete the access */
					if(fltptr->tran & TRN_LK /* If xmem */
					   && i!=2){	/* on stage 0 or 1, */
						/* combine into 1 transaction */
						fltptr->data=(fltptr+1)->data;
						(fltptr+1)->tran &= ~TRN_VALID;
						fltptr->tran &=
							 ~(TRN_RW|TRN_SIGN);
					}
					/* complete access in software */
					if(i != 2 || !(fltptr->tran & TRN_RW)){
						dotran(regptr,fltptr);
					}
				}
			}
		}
	}

out:
	if(USERMODE(u.u_ar0[PS])){
		if (runrun != 0)
			qswtch();

		if (u.u_prof.pr_scale)
		  	addupc((caddr_t)*ppc ,&u.u_prof, (int)(u.u_stime-syst));
	}

	

	u.u_ar0 = old_ar0;
	return(typesig);
}

dotran(regptr,fltptr)
/* Complete a memory transaction */
/*  This subroutine will complete a pending memory transaction based
   on the information present in the shadow transaction registers. 
   The register image on the stack is corrected if neccessary */

/*  This is a heavily used subroutine.  Candidate for ml. */

	struct fltinfo *fltptr;
	int *regptr;
{
	int size;
	int write;

	size = fltptr->tran & STROBES;
	write = (fltptr->tran & TRN_RW);   
	fltptr->tran &= ~TRN_VALID;   /* Mark transaction complete. */

	if(write){  /* Complete a st */
		switch (size){

		case TRN_WORD:
		if(fltptr->tran & TRN_MD)
			*(int *)(fltptr->addr) = fltptr->data;
		else
			suword(fltptr->addr,fltptr->data);
			
		break;

		case UPHALF:
		if(fltptr->tran & TRN_MD)
			*(short *)(fltptr->addr) = fltptr->data >>16;
		else
			suhalf(fltptr->addr,fltptr->data >>16);
		break;

		case LOHALF:
		if(fltptr->tran & TRN_MD)
			*(short *)(fltptr->addr+2) = fltptr->data;
		else
			suhalf(fltptr->addr+2,fltptr->data);
		break;

		case HIBYTE:
		if(fltptr->tran & TRN_MD)
			*(fltptr->addr) = fltptr->data >>24;
		else
			subyte(fltptr->addr,fltptr->data >>24);
		break;

		case MHBYTE:
		if(fltptr->tran & TRN_MD)
			*(fltptr->addr+1) = fltptr->data >>16;
		else
			subyte(fltptr->addr+1,fltptr->data >>16);
		break;

		case MLBYTE:
		if(fltptr->tran & TRN_MD)
			*(fltptr->addr+2) = fltptr->data >>8;
		else
			subyte(fltptr->addr+2,fltptr->data >>8);
		break;

		case LOBYTE:
		if(fltptr->tran & TRN_MD)
			*(fltptr->addr+3) = fltptr->data;
		else
			subyte(fltptr->addr+3,fltptr->data);
		break;

		}
		
	}
	else {  /* Complete ld */
		switch (size){

		case TRN_WORD:
		if(fltptr->tran & TRN_MD)
			fltptr->data = *(int *)(fltptr->addr);
		else{
			if(fltptr->tran & TRN_LK)    /* If xmem word */
				fltptr->data = xmem(fltptr->addr, fltptr->data, 1);
			else{
				fltptr->data = fuword(fltptr->addr);
					if(fltptr->tran & TRN_DB)
	regptr[R0+1+((fltptr->tran & TRN_REG)>>7)] = fuword(fltptr->addr+4);
			}
		}
		break;

		case UPHALF:
		if(fltptr->tran & TRN_MD)
			fltptr->data = *(short *)(fltptr->addr) & 0xFFFF;
		else
			fltptr->data=fuhalf(fltptr->addr);
		break;

		case LOHALF:
		if(fltptr->tran & TRN_MD)
			fltptr->data = *(short *)(fltptr->addr+2) & 0xFFFF;
		else
			fltptr->data=fuhalf(fltptr->addr+2);
		break;

		case HIBYTE:
		if(fltptr->tran & TRN_MD)
			fltptr->data = *(fltptr->addr) & 0xFF;
		else{
			if(fltptr->tran & TRN_LK)    /* If xmem word */
				fltptr->data = xmem(fltptr->addr, fltptr->data, 0);
			else
				fltptr->data=fubyte(fltptr->addr);
		}
		break;

		case MHBYTE:
		if(fltptr->tran & TRN_MD)
			fltptr->data = *(fltptr->addr+1) & 0xFF;
		else{
			if(fltptr->tran & TRN_LK)    /* If xmem word */
				fltptr->data = xmem(fltptr->addr+1, fltptr->data, 0);
			else
				fltptr->data=fubyte(fltptr->addr+1);
		}
		break;

		case MLBYTE:
		if(fltptr->tran & TRN_MD)
			fltptr->data = *(fltptr->addr+2) & 0xFF;
		else
		{
			if(fltptr->tran & TRN_LK)    /* If xmem word */
				fltptr->data = xmem(fltptr->addr+2, fltptr->data, 0);
			else
				fltptr->data=fubyte(fltptr->addr+2);
		}
		break;

		case LOBYTE:
		if(fltptr->tran & TRN_MD)
			fltptr->data = *(fltptr->addr+3) & 0xFF;
		else
		{
			if(fltptr->tran & TRN_LK)    /* If xmem word */
				fltptr->data = xmem(fltptr->addr+3, fltptr->data, 0);
			else
				fltptr->data=fubyte(fltptr->addr+3);
		}
		break;

		}
		if(fltptr -> tran & TRN_SIGN){
			switch (size){
			case UPHALF:
			case LOHALF:
				if(fltptr->data & 0x8000)
					fltptr->data |= 0xFFFF0000;
			break;
			case HIBYTE:
			case MHBYTE:
			case MLBYTE:
			case LOBYTE:
				if(fltptr->data & 0x80)
					fltptr->data |= 0xFFFFFF00;
			break;
			}
		}
		/* Load register with result and clear scoreboard bit */
		regptr[R0+((fltptr->tran & TRN_REG)>>7)] = fltptr->data;
	}
}
/*
 * nonexistent system call-- signal bad system call.
 */
nosys()
{
	psignal(u.u_procp, SIGSYS);
}

parbuserr(berrsrc, vaddr, pc, Status)
{
	cmn_err(CE_WARN,"Kernel Parity Error (PC=0x%x, PS=0x%x)",pc,Status);
/*	switch (berrsrc & (STAT2_VMEBERR | STAT2_VMXBERR | STAT2_LBTO)) {
	case STAT2_VMEBERR:
		cmn_err(CE_PANIC,"VME131: VME bus error at 0x%x",vaddr);
	case STAT2_VMXBERR:
		cmn_err(CE_PANIC,"VME131: VMX bus error at 0x%x",vaddr);
	case STAT2_LBTO:
		cmn_err(CE_PANIC,"VME131: Local bus time out at 0x%x",vaddr);
	default:
	cmn_err(CE_PANIC,"VME131: Unknown bus error (0x%x) at 0x%x",berrsrc,vaddr);
	}
*/
	cmn_err(CE_PANIC,"VME131: Unknown bus error (0x%x) at 0x%x",berrsrc,vaddr);
}

fpui()
/* Process a floating point imprecise exception */
{
	return(0);
}

fpup()
/* Process a floating point precise exception */
{
	return(0);
}

stray(vector)
int vector;
{
	cmn_err(CE_WARN, " ***** Stray interrupt,  SSR = 0x%x *****\n", vector);
}

#else	/*m68k*/

extern     benign_berr; /* check for benign bus errors */

/*
 * Called from the trap handler when a processor trap occurs.
 */
trap(sp,r0,r1,d2,d3,d4,d5,d6,d7,a0,a1,a2,a3,a4,a5,a6,Status,pc)
struct ex_status Status;
{
	register type;
	register sig = 0;
	register struct proc *pp;
	time_t syst;
	register code;
	
	type = (int) Status.vec_num & 0xfff;
	syst = u.u_stime;
	pp = u.u_procp;
	u.u_ar0 = &r0;
	code = u.u_ar0[PC];		/* pc is the most common use */
	u.u_traptype = TRAPNORM;
	sysinfo.trap++;

	if (USERMODE(Status.psw))
		type |= USER;
	switch (type) {

	/*
	 * Trap not expected.
	 * If user mode, send illegal instruction signal.
	 */
	default:
		if (USERMODE(Status.psw)) {
			sig = SIGILL;
			break;
		}
		spl7();
		cmn_err(CE_WARN,"       *** Kernel Trap *** ");
		cmn_err(CE_WARN,"proc = 0x%x, user = 0x%x",u.u_procp,&u);
		cmn_err(CE_WARN,"pid  = 0x%x, code = 0x%x",u.u_procp->p_pid,code);
		cmn_err(CE_WARN,"pc   = 0x%x, ps   = 0x%x",pc,Status.psw);
		cmn_err(CE_WARN,"argp = 0x%x, aps  = 0x%x",&r0,&Status);
		cmn_err(CE_PANIC,"   *** Unknown Trap Type = 0x%x ***", type);

	case RESCHED + USER:	/* Allow process switch */
		sysinfo.resched++;
		qswtch();
		goto out;

	case FPETRAP + USER:	/* floating point exception */
		sig = SIGFPE;
		code = r0;
		break;

	case ZDVDERR + USER:	/* zero divide check */
		sig = SIGFPE;
		code = KINTDIV;
		break;

	case CHKTRAP + USER:	/* CHK trap */
		sig = SIGFPE;
		code = KSUBRNG;
		break;

	case ADDRERR + USER:	/* address error */
		sig = SIGBUS;
		code = u.u_ar0[DCFA];
		u.u_traptype = TRAPADDR;
		break;

	case TRAPVFT + USER:	/* TRAPV instruction */
		sig = SIGFPE;
		code = KINTOVF;
		break;

	case IOTTRAP + USER:	/* IOT trap */
		sig = SIGIOT;
		code = r0;
		break;

	case EMTTRAP + USER:	/* EMT trap */
		sig = SIGEMT;
		code = r0;
		break;

	case FMTERR + USER:	/* bad exception format */
		sig = SIGILL;
		break;

	case FPTBSUN + USER:	/* FPU exc. - Branch or Set on UNordered cond.*/
	case FPTINEX + USER:	/* FPU exc. - INEXact result */
	case FPTDZ   + USER:	/* FPU exc. - Divide by Zero */
	case FPTUNFL + USER:	/* FPU exc. - UNderFLow */
	case FPTOPER + USER:	/* FPU exc. - OPerand ERror */
	case FPTOVFL + USER:	/* FPU exc. - OVerFLow */
	case FPTSNAN + USER:	/* FPU exc. - Signalling NAN */
	     {
		extern short do_fpu;
		if (do_fpu) {
			sig = SIGFPE;
			code = fpu_trap(type-USER);
			   /* force handler to call sys68 */
			u.u_traptype = TRAPFPE;
			break;
		}
	     }
	   /* Otherwise, fall thru to spurious interrupt processing */

	case UNINTR:		/* uninitialized interrupt;fall thru to STRAY */
	case UNINTR + USER:	/* ####### NOTE ##### may move */
	case STRAYFT:		/* spurious interrupt */
	case STRAYFT + USER:
		stray(sp,r0,r1,d2,d3,d4,d5,d6,d7,
				a0,a1,a2,a3,a4,a5,a6,Status,pc);
		return;

	case COPROTV + USER:	/* coprocessor protocol violation trap */
		sig = SIGILL;
		break;

	case BPTFLT + USER:	/* bpt instruction fault */
		u.u_ar0[PC] -= 2; /* back up pc for simulated breakpoint */
		/* fall thru */

	case TRCTRAP + USER:	/* trace trap */
		u.u_ar0[PS] &= ~(PS_T1 | PS_T0); /* turn off trace bits */
		if(u.u_trapsig) {
			  /* Not ordinary trace trap.  Earlier, we      */
			  /* forced a co-processor instruction to       */
			  /* complete, then trap.  Now we will finish   */
			  /* processing the signal.                     */
			++sysinfo.fpemidi;
			sig = u.u_trapsig;
			code = u.u_trapcode;
			u.u_trapsig = 0;
			u.u_trapcode = 0;
		}
		else
			sig = SIGTRAP;
		break;

	case PRIVFLT + USER:	/* privileged instruction fault */
	case EMT11FT + USER:	/* Line 1111 emulator trap */
	case EMT10FT + USER:	/* Line 1010 emulator trap */
	case INSTERR + USER:	/* Illegal instruction */
		sig = SIGILL;
		break;

	case ACFAIL:		/* VME AC failure interrupt signal */
	case ACFAIL + USER:
		splhi();
		cmn_err(CE_WARN,"System AC failure - No sync!");
		while(1);			/* No sync */
		break;

	case SYSFAIL:		/* VME SYSFAIL signal */
	case SYSFAIL + USER:
		cmn_err(CE_PANIC,"VME SYSFAIL signaled");
		break;
	}

	if (sig)
		psignal(pp, sig);

	if (runrun != 0)
		qswtch();
out:
	curpri = pp->p_pri = calcppri(pp);

	if ((pp->p_flag & SPRSTOP) || (pp->p_sig && issig())) {
		psig(code);
		curpri = pp->p_pri = calcppri(pp);
	}

	if(u.u_prof.pr_scale)
		addupc((caddr_t)u.u_ar0[PC], &u.u_prof, (int)(u.u_stime-syst));
}

/*
 * Called from the trap handler when a processor syscall trap occurs.
 */
syscall(sp,r0,r1,d2,d3,d4,d5,d6,d7,a0,a1,a2,a3,a4,a5,a6,Status,pc)
struct ex_status Status;
{
	register		code;
	register 		i;
	register int		*ap;
	register struct sysent	*callp;
	register struct proc	*pp;
	time_t			syst;
	short			pid;

	syst = u.u_stime;
	pid = u.u_procp->p_pid;
	u.u_traptype = TRAPNORM;
	u.u_ar0 = &r0;
	sysinfo.syscall++;
	u.u_error = 0;
	u.u_ar0[PS] &= ~PS_C;
	ap = (int *)u.u_ar0[SP];
	ap++;			/* skip word with param count */
	code = u.u_ar0[R0];
	i = code&0377;
	if (i >= NSYSENT)
		i = 0;
	else if (i==0) {	/* indirect */
		i = fuword(ap++)&0377;
		if (i >=NSYSENT)
			i = 0;
	}
	u.u_syscall = i;
	callp = &sysent[i];
	for (i = 0; i < callp->sy_narg; i++) {
		u.u_arg[i] = fuword(ap++);
	}
	u.u_dirp = (caddr_t)u.u_arg[0];
	u.u_rval1 = 0;
	u.u_rval2 = u.u_ar0[R1];
	u.u_ap = u.u_arg;

	if ((bootstate || !callp->sy_setjmp) && setjmp(u.u_qsav)) {
		if (!(u.u_rflags & U_RSYS) && !u.u_error)
			u.u_error = EINTR;
		spl0();
	} else
		(*callp->sy_call)();

	u.u_rflags &= ~(U_RSYS | U_DOTDOT | U_LBIN);

	if (u.u_error) {
		u.u_ar0[R0] = u.u_error;
		u.u_error = 0;
		u.u_ar0[PS] |= PS_C;
		if (++u.u_errcnt > 16) {
			u.u_errcnt = 0;
			runrun++;
		}
	} else {
		u.u_ar0[R0] = u.u_rval1;
		u.u_ar0[R1] = u.u_rval2;
	}

	pp = u.u_procp;
	curpri = pp->p_pri = calcppri(pp);

/*	if the system call trap was executed with the T bit set,
 *	the trace trap will be ignored.  So, the bit is reset
 *	and the SIGTRAP signal is sent to the process.
 */

	if (u.u_ar0[PS] & (PS_T1 | PS_T0)) {	/* were trace bits set ? */
		u.u_ar0[PS] &= ~(PS_T1 | PS_T0); /* reset the trace bits */
		psignal(pp, SIGTRAP);	/* send signal to process */
	}

	if (runrun != 0)
		qswtch();

	if (!u.u_trapflag && ((pp->p_flag & SPRSTOP)||(pp->p_sig && issig()))) {
		psig(code);
		curpri = pp->p_pri = calcppri(pp);
	}

	/*	If pid != pp->p_pid, then we are the child
	**	returning from a fork system call.  In this
	**	case, ignore syst since our time was reset
	**	in fork.
	*/

	if (u.u_prof.pr_scale)
		addupc((caddr_t)u.u_ar0[PC], &u.u_prof, 
			pid == pp->p_pid ? (int)(u.u_stime - syst)
					 : (int)u.u_stime);
}


/*
 * Called from the trap handler when a processor bus error occurs.
 *	    return value		Meaning
 *	    	 0	   Faulting page has been fixed, continue inst.
 *	    	value	   Abort cycle.  Clean up stack frame.
 *				New pc has been assigned.
 */
buserr(sp,r0,r1,d2,d3,d4,d5,d6,d7,a0,a1,a2,a3,a4,a5,a6,Status,pc)
struct ex_status Status;
unsigned pc;
{
	extern int clr_regA();

	register typesig = 0;	/* signal type		*/
	register proc_t	*p;	/* faulting process	*/
	register int	ssw;	/* special status work	*/
	register int	vaddr;	/* faulting virt addr	*/
	register int	write;	/* write flag		*/
	register int	*ppc;	/* pointer to pc	*/
	time_t syst;		/* time of fault	*/
	int *old_ar0;		/* old ar0		*/
	int old_hold;		/* old held signals	*/
	ushort fc; 		/* cpu function code	*/
	int regA;		/* generic cpu status register */
	int atc_psr;		/* pmmu status register value for ATC look */
	int tree_psr;		/* pmmu status register value tree walk */
	int spl_sav;

	spl_sav = spl7();
	minfo.buserr++;
	p = u.u_procp;
	u.u_traptype = TRAPNORM;
	old_ar0 = u.u_ar0;	/* save old ar0 and restore on return	      */
	u.u_ar0 = &r0;		/* set up new ar0 for current regs	      */
	ppc = &u.u_ar0[PC];	/* address of pc - referenced very often      */
	ssw = u.u_ar0[SSW];	/* write only if data fault and write bit clr */
	write = ((ssw & (SSW_DF|SSW_RW)) == SSW_DF);	/* otherwise a read   */

	/*
	 * Calculate the address of the bus error 
	 *	This is the logical address to the MMU
	 */

	if (ssw & SSW_DF) {			/* If data cycle fault,       */
		vaddr = u.u_ar0[DCFA];		/* use data cycle fault addr  */
		fc = ssw & SSW_FCD;		/* save for later use */
	} else {				/* else check frame type.     */
		if (Status.vec_num == SHRTBUSSTKFRM)	/* If short frame     */
			vaddr = *ppc + 4;	/* use pc+4 for B stage addr, */
		else				/* otherwise pick up stage B  */
			vaddr = u.u_ar0[SBA];	/* fault address directly.    */
		if (ssw & SSW_FC)		/* If fault on stage C then   */
			vaddr -= 2;		/* use B address - 2 as addr. */
		fc = FCD_PROG;			/* build pgm. function code   */
		if (!(USERMODE(Status.psw))) 	/* if supervisor state        */
			fc |= FCD_KERNEL;	/* build supv. function code  */
	}

	/*
	 * Get status from MMU Address Translation Cache (if any) 
	 *  for descriptor mapping the fault address (fc, vaddr).
	 * The atc entry for the fault address could already be flushed
	 * if an interrupt slipped in before the spl7 above.
	 */
	if(benign_berr)
		atc_psr = chk_atc(fc, vaddr);
	splx(spl_sav);

	/*
	 * Determine the source of the bus error
	 *
	 *	The STAT2 bus error status register holds the source of the
	 * bus error.  However, this register has some problems.  On the vme131
	 * card stray bus errors are seen.  This is due to the 68020 
	 * architecture.  When a prefetch fault and an operand (data) fault
	 * occur at the same time, it is possible to get here and the bus error
	 * status register will be zero.  In this case, we will default to
	 * processing the bus fault normally.  Parity errors are checked
	 * only if the VME, VMX, or LBTO bits are set in the status register.
	 */

	regA = get_regA();			/* get cpu status */
	/*
	 * The MVME143 has some problems with the status register for
	 * bus errors.  The status flags stay latched until they are
	 * cleared.  The status register may contain more than one source.
	 * This is ambiguous.  Clear the status and try again.
	 */
	if (regA) {
		if(chk_srcs(regA)) {
			regA = 0;
			clr_regA();
			cmn_err(CE_WARN, "      *** MVME143 %s Bus Error ***",
			    (USERMODE(Status.psw)) ? "User" : "Kernel");
			cmn_err(CE_WARN,
			    "      %s virtual address 0x%x",
			    (write) ? "Writing" : "Reading",
			    vaddr);
			cmn_err(CE_WARN,
			    "      at location PC = 0x%x",*ppc);
		}
	}
	if (regA & (A_VMEBERR | A_VMXBERR | A_LBTO | A_PE)) {
		extern void clr_bcache();

		clr_bcache();
		if (chkparity(vaddr, USERMODE(Status.psw))) {
			clr_regA();
			if (USERMODE(Status.psw) || 
			  (*ppc>(unsigned)nofulow && *ppc<(unsigned)nofuhigh)) {
				psignal(p, SIGKILL);
				psig(vaddr);
				return(SIGKILL);
			} else {
					parbuserr(regA, vaddr, *ppc, Status);
			}
		}
	}
	/*
	 * If not parity error then maybe user gave copyout
	 * a non-existent location.
	 */

	/*
	 *	Stray bus errors and MMU generated faults are treated 
	 * the same.  Since there is no status register on the MMB, we must
	 * first determine what caused the fault and call the appropriate fault
	 * handler.  If the program can not continue, determine the appropriate
	 * error handling.
	 */

	if (USERMODE(Status.psw)) {	/* user mode bus error */

		syst = u.u_stime;
		if (typesig = fault(vaddr, p, write)) {
			u.u_traptype = TRAPBUS;	/* bus error trap type */
			u.u_trapflag = 1;	/* copy stack frame to u area */

			psignal(p, typesig);	/* post bus error signal */
			old_hold = p->p_hold;	/* Hold off all other signals */
			p->p_hold = ~( (1L<<(SIGSEGV-1)) | (1L<<(SIGKILL-1)) |
				       (1L<<(SIGBUS-1)) );
					/* except SIGBUS, SIGSEGV & SIGKILL */
			if ((p->p_flag & SPRSTOP) || (p->p_sig && issig()))
				psig(vaddr);

			p->p_hold = old_hold;	/* reset held signals */
		}

		curpri = p->p_pri = calcppri(p);

		if (runrun != 0)
			qswtch();

		if (u.u_prof.pr_scale)
		   addupc((caddr_t)*ppc ,&u.u_prof, (int)(u.u_stime-syst));

	} else {			/* kernel mode bus error */
		if (*ppc > (unsigned)nofulow && *ppc < (unsigned)nofuhigh) {
			if (ssw & FCD_KERNEL) {

				/* Check mmu tree to determine if fault
			         * address is invalid. If so then, 
				 * 	return error status.
				 * If there is an ATC BE entry, 
				 *        flush it
				 * If we have VME/VSB or LTO bits
				 *    return error status.
				 * else 
				 *    return rerun status.
				 */

				if(benign_berr) {
					tree_psr = chk_tree(fc, vaddr);
					if (tree_psr & P_INV) {
						typesig = -1;
					}
					if (atc_psr & P_BE)
						mmuflush();
					if (regA & (A_VMEBERR | A_VMXBERR | A_LBTO)) {
						typesig = -1;
					}
				} else {
					typesig = -1;
				}

			} else {	/* must be touching user page */
				if (typesig = fault(vaddr, p, write))
					typesig = -1;
				curpri = p->p_pri = calcppri(p);
			}
		}
		else
		{

			/*
			 *  DO FOLLOWING CHECKS ONLY IF benign_berr IS SET.
			 *
			 * 	Check that the address is valid.  Call
			 * chk_tree(), which will force the pmmu to
			 * walk all levels of the tree to verify that
			 * the fault address is valid.  If the address is
			 * is invalid, the system will panic.
			 * 	If it is valid, a check is made to see if there
			 * is an ATC entry for it with the "B" bit set.
			 * This is the "RMC lock" condition.  If this is
			 * the case, the pmmu is flushed to clear the "B" bit.
			 * 	If there is no entry in the ATC for the address
			 * then there is no need to flush the ATC.
			 * 	In either case the instruction will be retried.
			 */

			tree_psr = P_INV;  /* Don't check if benign_berr 0 */
			if(benign_berr)
				tree_psr = chk_tree(fc, vaddr);
			if ((tree_psr & P_INV) == 0) {	/* valid address */
				if (atc_psr & P_BE) {	/* RMC lock */
					mmuflush();
				}
			}
			else {
				cmn_err(CE_WARN,"      *** Kernel Bus Error ***");
				cmn_err(CE_WARN,"      %s virtual address 0x%x",
			       (write) ? "Writing" : "Reading", vaddr);
				cmn_err(CE_WARN,"      at location PC = 0x%x",*ppc);
				cmn_err(CE_PANIC,"  Kernel Bus Error System Panic");
			}
		}
	}

	if (typesig == -1) {
		*ppc = (unsigned)errret;
		if (u.u_caddrflt)
			*ppc = (unsigned)u.u_caddrflt;
	}

	u.u_ar0 = old_ar0;
	return(typesig);
}

/*
 * nonexistent system call-- signal bad system call.
 */
nosys()
{
	psignal(u.u_procp, SIGSYS);
}

stray(usp,d0,d1,d2,d3,d4,d5,d6,d7,a0,a1,a2,a3,a4,a5,a6,Status,pc)
struct ex_status Status;
{
	register int vector = (int)Status.vec_num;
	cmn_err(CE_WARN, " ***** Stray interrupt at vector number 0x%x *****\n", vector);
	cmn_err(CE_WARN, " PC = 0x%x, usp = 0x%x, psw = 0x%x\n",
		pc, usp, Status.psw);
}

parbuserr(regA, vaddr, pc, Status)
struct ex_status Status;
{
	cmn_err(CE_WARN,"Kernel Parity Error (PC=0x%x, PS=0x%x)",pc,Status.psw);
	switch (regA & (A_VMEBERR | A_VMXBERR | A_LBTO)) {
	case A_VMEBERR:
		cmn_err(CE_PANIC," VME bus error at 0x%x",vaddr);
	case A_VMXBERR:
		cmn_err(CE_PANIC," VMX bus error at 0x%x",vaddr);
	case A_LBTO:
		cmn_err(CE_PANIC," Local bus time out at 0x%x",vaddr);
	default:
		cmn_err(CE_PANIC," Unknown bus error (0x%x) at 0x%x",regA,vaddr);
	}
}
#endif
