static char ident[] = " @(#)ma_dat_ind.c	35.2        6/17/88	Copyright MOTOROLA, INC. ";

/*FH*********************** FUNCTION HEADER ***************************
**
**  FUNCTION NAME:	ma_dat_ind
**
**  DESCRIPTION:	This function will return an MA_DATA_INDICATION
**			primitive via an MAC EPB.
**
**			This is done only if the packet was received
**			without error and the destination address
**			is the node's address.
**
**
**  PARAMETERS:		
**
**	Input:		ma_epbptr	: address of MAC EPB
**
**	Output:		The MAC Statistics Table is updated with
**			packet reception information.
**
**	Returns:
**
**  COMPLEX LOCALS:
**
**  METHOD:
**
**  NOTES:
**
**********************************************************************/

/*
**	Include Files
*/

#include <sys/osi/common/types.h>	/* Abstract Type Definitions */
#include <sys/osi/common/system.h>	/* System Function Definitions */
#include <sys/osi/common/bdb.h>		/* BDB Type Definitions */
#include <sys/osi/common/epb.h>		/* EPB Type Definitions */
#include <sys/osi/common/selsizes.h>	/* MAC Address Size */
#include <sys/osi/common/ma_epb.h>	/* MAC EPB Type Definitions */
#include <sys/osi/common/macros.h>	/* Macro Definitions */
#include <sys/osi/common/com_env.h>	/* CE Definitions */

#include "ma_lance.h"			/* LANCE Controller Definitions */
#include "ma_def.h"			/* MAC Node Definitions */
#include "ma_globals.h"			/* Global MAC Node Definitions */

VOID
ma_dat_ind(ma_epbptr)

register MA_EPB *ma_epbptr; 	/* Address of MAC EPB */
{
	VOID	 o_retbdb(),			/* Return BDB */
		 o_retepb(),			/* Return EPB */
		 o_post(),			/* Post EPB */
		 o_bytecopy(),			/* Copy Bytes */
		 ma_rx_gbuf();			/* Get RX Buffer */

	register MA_HDR	 *hdrptr;		/* Packet Header */

	register MA_REG	 *regptr;		/* Registration Entry */

	register BDB	 *bdbptr;		/* BDB Pointer */

	register BOOLEAN retflag;		/* Discard Flag */

	register USHORT	 type;			/* Length/Type */

	register USHORT	 i;			/* Just A Counter */

	DEFAULT	 error;				/* Error Code */

/*
**  First, we must initialize all function variables.
*/

	retflag = TRUE;

/*
**  Next, the EPB header status is checked for errors. If any
**  errors occurred, the data packet is discarded and the function
**  is terminated.
**
**  The dual (length/type) field is checked to determine the format of
**  the packet. If the field is a valid length (length <= MA_DATAMAX)
**  the packet is IEEE 802.3 and the packet header is then parsed in
**  order to fill the MAC EPB address parameters. Otherwise it is
**  Ethernet and the (length/type) field is an Ethernet Type.
**
**  An attempt is then made to deliver the packet to a MAC
**  client (the IEEE 802.3 LLC sublayer or a registered Ethernet
**  client). Otherwise the data packet is discarded.
*/

	if (ma_epbptr->hdr.status == RET_OK)
	{
		hdrptr = (MA_HDR *)(bdbptr = ma_epbptr->hdr.bdb)->data;

		if ((type = (USHORT)GET_16(hdrptr->data_len)) <= MA_DATAMAX)
		{
			bdbptr->buff_offset += sizeof(MA_HDR);
			bdbptr->buff_length  = type;

			o_bytecopy(hdrptr->loc_addr, ma_epbptr->rem_addr, MACSZ);
			o_bytecopy(hdrptr->rem_addr, ma_epbptr->loc_addr, MACSZ);

			ma_epbptr->hdr.dst_id   = LINK_QID;
			ma_epbptr->hdr.epb_size = sizeof(MA_EPB);

			ma_epbptr->status	= RET_OK;
			ma_epbptr->svc_class	= 0;

			ma_epbptr->Response_Ptr		= (BDB *)NULL;
			ma_epbptr->Response_Status	= 0;
			ma_epbptr->Response_Svc_Class	= 0;

			retflag = FALSE;
		}
		else
		{
			for (regptr = (MA_REG *)ma_reg_tbl, i = 1; i < REG_TBL_SIZE; ++i)
				if (regptr[i].type == type)
					break;

			if (i == REG_TBL_SIZE)
				i = 0;

			if ((ma_epbptr->hdr.dst_id = regptr[i].ip) != 0)
			{
				ma_epbptr->hdr.user_def = (ULONG)type;

				retflag = FALSE;
			}
		}
	}

/*
**  Next, the return/discard flag is checked. If set, all blocks are
**  returned. If not set, the MA_DATA_INDICATION is posted.
*/

	if (retflag == TRUE)
	{
		o_retbdb(ma_epbptr->hdr.bdb);

		o_retepb(ma_epbptr);
	}
	else
		o_post(ma_epbptr);

/*
**  The third section will attemp to refill the receive ring
**  with another Buffer Descriptor Block.
*/

	ma_rx_gbuf(&error);
}
