static char ident[] = " @(#)ma_dat_req.c	35.2        6/17/88	Copyright MOTOROLA, INC. ";

/*FH*********************** FUNCTION HEADER ***************************
**
**  FUNCTION NAME:	ma_dat_req
**
**  DESCRIPTION:	This function responds to an MA_DATA_REQUEST
**			primitive by initiating the transmission of
**			an MAC Protocol Data Unit. When the PDU
**			has been sent with or without errors, an
**			MA_DATA_CONFIRM primitive will be returned.
**
**
**  PARAMETERS:		
**
**	Input:		ma_epbptr		: address of MAC EPB
**			user_def		: zero    - IEEE 802.3 Packet
**						  nonzero - Ethernet Type
**
**	Output:		The MAC Statistics Table is updated with
**			packet transmission information.
**
**	Returns:
**
**  COMPLEX LOCALS:
**
**  METHOD:
**
**  NOTES:
**
**********************************************************************/

/*
**	Include Files
*/

#include <sys/osi/common/types.h>	/* Abstract Type Definitions */
#include <sys/osi/common/system.h>	/* System Function Definitions */
#include <sys/osi/common/bdb.h>		/* BDB Type Definitions */
#include <sys/osi/common/epb.h>		/* EPB Type Definitions */
#include <sys/osi/common/selsizes.h>	/* MAC Address Size */
#include <sys/osi/common/ma_epb.h>	/* MAC EPB Type Definitions */
#include <sys/osi/common/macros.h>	/* Macro Definitions */

#include <sys/bppv2/mvme147/bpp_local.h>

#include "ma_lance.h"			/* LANCE Controller Definitions */
#include "ma_def.h"			/* MAC Node Definitions */
#include "ma_globals.h"			/* Global MAC Node Definitions */

/*
**	External Variables
*/

extern ULONG	bpp_disable_level;		/* Environment Disable Level */

VOID
ma_dat_req(ma_epbptr)

register MA_EPB *ma_epbptr;			/* Address of MAC EPB */
{
	BOOLEAN	o_bytecmp();			/* Compare Bytes */

	EPB	*o_getepb();			/* Get EPB Blocks */

	BDB	*o_getbdb();			/* Get BDB Block */

	VOID	o_bytecopy(),			/* Copy Bytes */
		o_retbdb(),			/* Return BDB */
		o_retepb(),			/* Return EPB */
		o_post(),			/* Post EPB */
		ma_rst();			/* Restart LANCE */

	USHORT	ma_get_rdp();			/* Get Data Register */

	register EPB	*newepb;		/* New MAC EPB Pointer */

	register BDB	*newbdb;		/* New BDB Pointer */
	register BDB	*oldbdb;		/* Linked BDB Pointer */

	register LONG	pri;			/* Saved Priority */

	register USHORT lentot;			/* Data Length */

	register BOOLEAN retflag;		/* Discard Flag */

	register MA_HDR	*hdrptr;		/* Packet Header Ptr. */

	register struct ringentry *rngptr;	/* Ring Address */

/*
**  First, we must initialize all function variables.
*/

	retflag  = TRUE;

/*
**  Next, a MAC PDU must be built with packet header information (i.e.
**  source address, destination address, and the dual (length/type)
**  field) and data from the passed Buffer Data Block. A MAC PDU
**  is built by placing packet header information in front of the
**  data.
**
**  The packet header's dual (length/type) field is filled according
**  to the protocol being used. This is determined by the
**  'user_def' field of the MAC EPB (i.e. if non-zero, then it
**  is an Ethernet packet). If this is an IEEE 802.3 packet, the
**  field is filled with the packet's generated total length.
**  If this is an Ethernet packet, the field is filled with the
**  specified Ethernet Type (i.e. 'user_def' value), but only if the
**  packet is long enough to meet the minimum length requirement.
**
**  If any errors occured in building the PDU an error will be
**  returned with a MA_DATA_CONFIRM.
*/

	if ((oldbdb = ma_epbptr->hdr.bdb) != (BDB *)NULL)
	{
		lentot = oldbdb->buff_length;

		if (ma_epbptr->hdr.user_def == 0)
		{
			if (oldbdb->buff_offset >= sizeof(MA_HDR))
			{
				oldbdb->buff_offset -= sizeof(MA_HDR);
				oldbdb->buff_length += sizeof(MA_HDR);

				hdrptr = (MA_HDR *)(oldbdb->data + oldbdb->buff_offset);

				o_bytecopy(ma_epbptr->rem_addr, hdrptr->rem_addr, MACSZ);
				o_bytecopy(ma_epbptr->loc_addr, hdrptr->loc_addr, MACSZ);

				hdrptr->data_len[0] = (UBYTE)(lentot >> 8);
				hdrptr->data_len[1] = (UBYTE)lentot;

				retflag = FALSE;
			}
		}
		else
			if (lentot >= MA_DATAMIN)
				retflag = FALSE;
	}

/*
**  At this point, if any errors have occured, an MA_DATA_CONFIRM
**  primitive is generated with the error status and is sent
**  back to the originator. Processing is complete and the
**  function is terminated.
*/

	if (retflag == TRUE)
	{
		if (ma_epbptr->hdr.bdb != (BDB *)NULL)
			o_retbdb(ma_epbptr->hdr.bdb);

		if (ma_epbptr->hdr.user_def == 0)
		{
			ma_epbptr->hdr.dst_id = ma_epbptr->hdr.src_id;
			ma_epbptr->hdr.src_id = MAC_QID;
			ma_epbptr->hdr.type   = MA_DATA_CONFIRM;
			ma_epbptr->hdr.status = RET_OK;
			ma_epbptr->hdr.bdb    = (BDB *)NULL;

			ma_epbptr->status     = MA_ERR_TXP;

			ma_epbptr->Response_Ptr		= (BDB *)NULL;
			ma_epbptr->Response_Status	= 0;
			ma_epbptr->Response_Svc_Class	= 0;

			o_post(ma_epbptr);
		}
		else
			o_retepb(ma_epbptr);

		return;
	}

/*
**  Next, the MAC destination address is examined to determine if the
**  packet needs to be delivered to someone on this node.
*/

	/*
	**  If the packet is an IEEE 802.3 packet and if the destination
	**  address is this node's physical address or is a valid
	**  multicast address, the packet is duplicated and sent
	**  back via an MA_DATA_INDICATION primitive. The packet
	**  still must be transmitted onto the LAN, so no reply
	**  is given yet, and processing continues on.
	*/

	if ((ma_epbptr->hdr.user_def == 0)
	 && ((o_bytecmp(ma_epbptr->rem_addr, ma_ts_addr, MACSZ) == SUCCESS)
	 ||  (o_bytecmp(ma_epbptr->rem_addr, ma_gladr, MACSZ) == SUCCESS)))
	{
		if ((newepb = o_getepb(EPB_PID)) != (EPB *)NULL)
		{
			newepb->src_id   = MAC_QID;
			newepb->dst_id   = MAC_QID;
			newepb->epb_size = sizeof(MA_EPB);
			newepb->type     = MA_DATA_INDICATION;
			newepb->status   = RET_OK;

			if ((newbdb = newepb->bdb = o_getbdb(BDB_PID)) != (BDB *)NULL)
			{
				hdrptr = (MA_HDR*)newbdb->data;

				o_bytecopy((oldbdb->data + oldbdb->buff_offset), hdrptr, oldbdb->buff_length);

				o_bytecopy(ma_epbptr->rem_addr, hdrptr->loc_addr, MACSZ);
				o_bytecopy(ma_epbptr->loc_addr, hdrptr->rem_addr, MACSZ);

				newbdb->buff_offset = 0;
				newbdb->buff_length = oldbdb->buff_length;

				LOG("MAC: Data request addressed to ourself\n");

				o_post(newepb);
			}
			else
			{
				LOG("MAC: Can't get BDB\n");

				o_retepb(newepb);
			}
		}
		else
		{
			LOG("MAC: Can't get EPB\n");
		}
	}

/*
**  In case MAC is restricted to local activities, we discard
**  the request rather than sending it to the LANCE.
*/

	if (ma_state == MAC_LOCAL)
	{
		LOG("MAC: Discarded packet to be sent, only local activities enabled\n");

		if (ma_epbptr->hdr.bdb != (BDB *)NULL)
			o_retbdb(ma_epbptr->hdr.bdb);

		o_retepb(ma_epbptr);

		return;
	}

/*
**  The LANCE CSR0 is read to insure that the LANCE is running
**  with transmitter and receiver on and interrupts enabled. If
**  not, the LANCE must be restarted.
*/

	if ((ma_get_rdp() & CSR0_GOOD) != CSR0_GOOD)
		ma_rst();

/*
**  Now, all the transmit pre-processing has been successfully
**  completed and the ring entry must be filled with the
**  BDB. If the packet is an IEEE 802.3 packet and it's total
**  length is not long enough, padding is added to the entry.
**
**  Also the passed EPB pointer is loaded into the Transmit
**  Buffer Ring for retrival.
*/

	DISABLE(pri, bpp_disable_level);

	rngptr = ma_lnc_txrgptr.tail;

	while (rngptr->md[2] != 0);

	LOAD_RING((oldbdb->data + oldbdb->buff_offset), oldbdb->buff_length, rngptr);

	ma_lnc_txrgptr.tail = INCR_RING(rngptr, ma_lnc_txrgptr);

	if ((ma_epbptr->hdr.user_def == 0) && (lentot < MA_DATAMIN))
		rngptr->md[2] += lentot - MA_DATAMIN;

	*(ma_txrgptr.tail) = (ULONG)ma_epbptr;

	ma_txrgptr.tail = INCR_RING(ma_txrgptr.tail, ma_txrgptr);

/*
**  The packet is now ready for transmission. The STP and ENP bits
**  are set in the ring entry of the packet. Then the entry is turned
**  over to the LANCE. When the ownership bit of the entry is
**  relinquished, the LANCE may begin transmission of the packet.
*/

	rngptr->md[1] |= (TMD1_STP | TMD1_ENP | TMD1_OWN);

	ENABLE(pri);
}
