/* @(#)ma_def.h	35.3        6/17/88	Copyright MOTOROLA, INC. */

/*IH*********************** INCLUDE HEADER ****************************
**
**  INCLUDE NAME:	ma_def.h
**
**  DESCRIPTION:	This file contains constant, type and
**			macro definitions involved with the
**			interface to the MAC sublayer of the
**			Data Link Layer subsystem.
**
**  NOTES:
**
**********************************************************************/

/*
**	The following two definitions are made for mapping between ci
**	driver, MAC, and new BPP buffer structures.  They must be identical
**	to definitions in .../mvme147/bp_ci.h.  A reorganization of header
**	could possibly prevent this kludge.
*/

#define BUF_FILL		32
#define BDB_HDR_SIZE		28
#define BUF_HDR_SIZE		8

/*
**	The following are constant definitions used through
**	out the MAC sublayer.
*/

	/*
	**	MAC Return/Reply Code Definitions
	*/

#define MA_ERR_MEM 	0x201	/* MAC Resource Allocation Error */
#define MA_ERR_INT 	0x202	/* MAC Initialization Error */
#define MA_ERR_TXP 	0x203	/* MAC Transmit Packet Error */
#define MA_ERR_RXP 	0x204	/* MAC Receive  Packet Error */

	/*
	**	MAC Packet Size (IEEE 802.3 )Definitions
	*/

#define MA_TOTMIN  64		/* Minimum Length Of MAC Packet */
#define MA_TOTMAX  1518		/* Maximum Length Of MAC Packet */
#define MA_FCSLEN  4		/* Length Of MAC Frame Check Sequence */

#define MA_DATAMIN (MA_TOTMIN-sizeof(struct ma_header)-MA_FCSLEN)
#define MA_DATAMAX (MA_TOTMAX-sizeof(struct ma_header)-MA_FCSLEN)

/*
**	The following is a macro definition for accessing word values
**	defined as byte arrays.
*/

#define GET_16(ptr)	((ptr[0] << 8) | ptr[1])

/*
**	The following are the MAC Task state definitions.
*/

#define MAC_INIT	1	/* MAC requires initialization */
#define MAC_LOCAL	2	/* MAC is restricted to local activities */

/*
**	The following is the MAC Ethernet type registration
**	table size definition.
*/

#define	 REG_TBL_SIZE	25	/* Ethernet type registration table size */

/*
**	The following are structure definitions for a MAC
**	IEEE 802.3 Protocol Data Unit (PDU).
*/

	/*
	**	The following is a type definition for the
	**	MAC packet addressing format.
	*/

typedef UBYTE MA_ADDR[6];


	/*
	**	The following is a type definition for the
	**	MAC packet data length format.
	*/

typedef UBYTE MA_DLEN[2];

	/*
	**	The following is a structure definition for the
	**	header of a MAC Protocol Data Unit.
	*/

typedef struct ma_header
{
	MA_ADDR		rem_addr;	/* Destination Address */
	MA_ADDR		loc_addr;	/* Source Address */
	MA_DLEN		data_len;	/* Data Length */
} MA_HDR;

/*
**	The following structure defines a MAC registration table
**	entry.
*/

typedef struct ma_reg
{
	ULONG		ip;		/* Client's Interface Point */
	USHORT		type;		/* Client's Ethernet Type */
} MA_REG;

/*
**	The following structure defines the MAC Statistics Table.
*/

typedef struct ma_stat
{
	/*
	**  Transmit related statistics.
	*/

	ULONG	tx_more;	/* Number Of MAC Packets Transmitted
				   Without Error After More Than One
				   Collision */

	ULONG	tx_one;		/* Number Of MAC Packets Transmitted
				   Without Error After Exactly One
				   Collision */

	ULONG	tx_def;		/* Number Of MAC Packet Transmissions
				   Deferred Because Channel Is Busy */

	ULONG	tx_buff;	/* Number Of MAC Packet Transmissions
				   Aborted Due To Lack Of Ownership
				   Of Next Buffer */

	ULONG	tx_uflo;	/* Number Of MAC Packet Transmissions
				   Aborted Due To Underflow Error */

	ULONG	tx_lcol;	/* Number Of MAC Packet Transmissions
				   Aborted Due To Late Collision */

	ULONG	tx_lcar;	/* Number Of MAC Packet Transmissions
				   Aborted Due To Loss Of Carrier */

	ULONG	tx_rtry;	/* Number Of MAC Packet Transmissions
				   Aborted Due To Excessive
				   Collisions */

	ULONG	tx_tdr;		/* Time Doman Reflectometry Counter */

	/*
	**  Receive related statistics.
	*/

	ULONG	rx_fram;	/* Number Of MAC Packets Received
				   With A Framing Error */

	ULONG	rx_oflo;	/* Number Of MAC Packets Received
				   With An Overflow Error */

	ULONG	rx_crc;		/* Number Of MAC Packets Received
				   With An CRC Error */

	ULONG	rx_buff;	/* Number Of MAC Packets Received
				   While Data Chaining But Lost Due
				   To Lack Of Ownership Of Buffer */

	/*
	**  Hardware related statistics.
	*/

	ULONG	hard_babl;	/* Number Of Times The Transmitter
				   Timed Out */

	ULONG	hard_cerr;	/* Number Of Times The Transmission
				   Transceiver Test Collision Failed */

	ULONG	hard_miss;	/* Number Of MAC Packets Received
				   But Lost Due To Lack Of Buffers */

	ULONG	hard_merr;	/* Number Of Memory Addressing
				   Errors */
} MA_STAT;
