# @(#)ma_isr_asm.s	35.2        6/17/88	Copyright MOTOROLA, INC.

#*MH************************ MODULE HEADER **************************
#*
#*  MODULE NAME:	ma_isr_asm.s
#*
#*  DESCRIPTION:	This module contains the assembly to C
#*			interfaces for the LANCE interrupt service
#*			routine.
#*
#*  CONTENTS:		ma_intr()	- normal ISR,
#*			ma_set_rap()	- set address register,
#*			ma_set_rdp()	- set data register,
#*			ma_get_rdp()	- get data register.
#*
#*  SYSTEM GLOBALS:
#*
#*  MODULE GLOBALS:
#*
#*  NOTES:
#*
#*********************************************************************/

	file	"ma_isr_asm.s"		# This is who we are.

	set	LAR,	0xfffe1802	# LANCE Address Register
	set	LDR,	0xfffe1800	# LANCE Data Register
#
#*FH*********************** FUNCTION HEADER ***************************
#*
#*  FUNCTION NAME:	ma_intr
#*
##  DESCRIPTION:	This routine will be called through an interrupt
##			generated by the TBC. 
#*
#*  PARAMETERS:		
#*
#*	Input:
#*
#*	Output:
#*
#*	Returns:
#*
#*  COMPLEX LOCALS:
#*
#*  METHOD:
#*
#*  NOTES:
#*
#*********************************************************************

	global	ma_intr

	text				# Start text section.

ma_intr:
	mov.l	&LDR,%a0		# -- Read
	mov.w	(%a0),%d0		# -- CSR 0
	bclr	&6,%d0			# Clear Interrupt Enable Bit
	mov.w	%d0,(%a0)		# Clear Interrupt
	mov.l	%d0,-(%sp)		# Pass CSR 0.
	jsr	ma_isr			# Service LANCE interrupt.
	addq.l	&4,%sp			# Restore Stack
	mov.w	&0x0040,LDR		# Enable Interrupts.
	rts				# Exit
#
#*FH*********************** FUNCTION HEADER ***************************
#*
#*  FUNCTION NAME:	ma_set_rap
#*
##  DESCRIPTION:	This routine will set the LANCEs address register
##			to point to the passed value.
#*
#*  PARAMETERS:		
#*
#*	Input:
#*
#*	Output:
#*
#*	Returns:
#*
#*  COMPLEX LOCALS:
#*
#*  METHOD:
#*
#*  NOTES:
#*
#*********************************************************************

	global	ma_set_rap

	text				# Start text section.

ma_set_rap:
	mov.w	6(%sp),%d0		# Get CSR Address Select
	mov.l	&LAR,%a0		# -- Set CSR
	mov.w	%d0,(%a0)		# -- Address
	rts				# Exit
#
#*FH*********************** FUNCTION HEADER ***************************
#*
#*  FUNCTION NAME:	ma_set_rdp
#*
##  DESCRIPTION:	This routine will set the LANCEs data register
##			to the passed value.
#*
#*  PARAMETERS:		
#*
#*	Input:
#*
#*	Output:
#*
#*	Returns:
#*
#*  COMPLEX LOCALS:
#*
#*  METHOD:
#*
#*  NOTES:
#*
#*********************************************************************

	global	ma_set_rdp

	text				# Start text section.

ma_set_rdp:
	mov.w	6(%sp),%d0		# Get Data Register Select
	mov.l	&LDR,%a0		# -- Set Data
	mov.w	%d0,(%a0)		# -- Register
	rts				# Exit
#
#*FH*********************** FUNCTION HEADER ***************************
#*
#*  FUNCTION NAME:	ma_get_rdp
#*
##  DESCRIPTION:	This routine will return the LANCEs data register.
#*
#*  PARAMETERS:		
#*
#*	Input:
#*
#*	Output:
#*
#*	Returns:
#*
#*  COMPLEX LOCALS:
#*
#*  METHOD:
#*
#*  NOTES:
#*
#*********************************************************************

	global	ma_get_rdp

	text				# Start text section.

ma_get_rdp:
	clr.l	%d0			# Clear Return Register
	mov.l	&LDR,%a0		# -- Set Data
	mov.w	(%a0),%d0		# -- Register
	rts				# Exit
