static char ident[] = " @(#)ma_main.c	35.2        6/17/88	Copyright MOTOROLA, INC. ";

/*MH************************ MODULE HEADER ****************************
**
**  MODULE NAME:	ma_main.c
**
**  DESCRIPTION:	This module provides central services to the
**			IEEE 802.3 MAC sublayer components. It is
**			given control when the MAC sublayer task is
**			created. It calls the necessary initialization
**			routines, then initiates steady-state MAC
**			operation in which service requests are pulled
**			off the task's queue and processed in a serial
**			fashion.
**
**
**  CONTENTS:		The following functions are included in
**			this module:
**
**			ma_main()		: main entry point to
**						  MAC sublayer
**
**  SYSTEM GLOBALS:
**
**  MODULE GLOBALS:
**
**  NOTES:
**
**********************************************************************/

/*
**	Include Files
*/

#include <sys/osi/common/types.h>	/* Abstract Type Definitions */
#include <sys/osi/common/system.h>	/* System Function Definitions */
#include <sys/osi/common/bdb.h>		/* BDB Type Definitions */
#include <sys/osi/common/epb.h>		/* EPB Type Definitions */
#include <sys/osi/common/selsizes.h>	/* MAC Address Size */
#include <sys/osi/common/ma_epb.h>	/* MAC EPB Type Definitions */
#include <sys/osi/common/macros.h>	/* Macro Definitions */
#include <sys/osi/common/com_env.h>	/* CE Definitions */

#include <sys/bppv2/bpp.h>
#include <sys/bppv2/bm.h>
#include <sys/bppv2/ccb.h>
#include <sys/bppv2/mvme147/bpp_local.h>
#include <sys/bppv2/mvme147/bp_ci.h>

#include "ma_lance.h"			/* LANCE Controller Definitions */
#include "ma_def.h"			/* MAC Node Definitions */
#include "ma_globals.h"			/* Global MAC Node Definitions */


VOID
ma_main(mpb, arg)

register BP_MPB_T *mpb;		/* pointer to the message */
register ULONG	 arg;		/* argument is not used for now */
{
	VOID	
		ma_dat_con(),	/* MAC Data Confirm */
		ma_dat_ind(),	/* MAC Data Indicate */
		ma_dat_req(),	/* MAC Data Request */
		ma_reg_req(),	/* MAC Registration Request */
		ma_drg_req(),	/* MAC Deregistration Request */
		ma_mgmt(),	/* MAC Station Management Request */
		ma_rx_gbuf(),	/* Get RX Buffer */
		o_retbdb(),	/* Return BDB Block */
		o_retepb();	/* Return EPB Block */

	register EPB *epbptr;	/* Pointer To EPB */

	DEFAULT	error;		/* Init Return Code */

	LOG("MAC: Processing event\n");

	/*
	**	First adjust mpb pointer to EPB for MAC
	*/

		epbptr = (EPB *) &(((PSEPB *)mpb)->epb);

		switch (epbptr->type)
		{
			case MA_DATA_CONFIRM:
			{
				ma_dat_con(epbptr);
				break;
			}
	
			case MA_DATA_REQUEST:
			{
				ma_dat_req(epbptr);
				break;
			}
	
			case MA_DATA_INDICATION:
			{
				ma_dat_ind(epbptr);
				break;
			}

			case MA_PARAM_REQUEST:
			{
				ma_mgmt(epbptr);
				break;
			}

			case MA_REG_REQUEST:
			{
				ma_reg_req(epbptr);
				break;
			}

			case MA_DEREG_REQUEST:
			{
				ma_drg_req(epbptr);
				break;
			}
	
			default:
			{
				LOG("MAC: Bad event 0x%x\n", epbptr->type);

				if (epbptr->bdb != (BDB *)NULL)
					o_retbdb(epbptr->bdb);

				o_retepb(epbptr);
			}
		}

	/*
	**  This section of the function will attempt to refill the 
	**  receive ring with Buffer Data Blocks.
	*/

		do
		{
			ma_rx_gbuf(&error);
		}
		while (error == RET_OK);
}
