static char ident[] = " @(#)ma_mgmt.c	35.2        6/17/88	Copyright MOTOROLA, INC. ";

/*FH*********************** FUNCTION HEADER ***************************
**
**  FUNCTION NAME:	ma_mgmt
**
**  DESCRIPTION:	This function will handle all station managment
**			requests.
**
**  PARAMETERS:		
**
**	Input:		epbptr		: EPB Pointer
**
**	Output:
**
**	Returns:
**
**  COMPLEX LOCALS:
**
**  METHOD:
**
**  NOTES:
**
**********************************************************************/

/*
**	Include Files
*/

#include <sys/osi/common/types.h>	/* Abstract Type Definitions */
#include <sys/osi/common/system.h>	/* System Function Definitions */
#include <sys/osi/common/bdb.h>		/* BDB Type Definitions */
#include <sys/osi/common/epb.h>		/* EPB Type Definitions */
#include <sys/osi/common/selsizes.h>	/* MAC Address Size */
#include <sys/osi/common/ma_epb.h>	/* MAC EPB Type Definitions */
#include <sys/osi/common/sm_epb.h>	/* SMA EPB Type Definitions */
#include <sys/osi/common/macros.h>	/* Macro Definitions */

#include <sys/bppv2/mvme147/bpp_local.h>

#include "ma_lance.h"			/* LANCE Controller Definitions */
#include "ma_def.h"			/* MAC Node Definitions */
#include "ma_globals.h"			/* Global MAC Node Definitions */

/*
**	External Variables
*/

extern ULONG	bpp_disable_level;	/* Environment Disable Level */


VOID
ma_mgmt(epbptr)

register SM_EPB	*epbptr;		/* EPB Pointer */
{
	VOID	 ma_rst(),		/* LANCE Restart Function */
		 o_post(),		/* Post EPB */
		 o_retepb();		/* Return EPB */

	register BOOLEAN restart;	/* LANCE Restart Flag */

	register LONG	pri;		/* Saved Priority */


/*
**  First, prepare the return EPB header.
*/

	epbptr->hdr.type   = MA_PARAM_CONFIRM;
	epbptr->hdr.dst_id = epbptr->hdr.src_id;
	epbptr->hdr.src_id = MAC_QID;

	epbptr->board = ma_board;

/*
**  Initialize the LANCE restart flag.
*/

	restart = FALSE;

/*
**  Switch on the parameter request selection.
*/

	switch (epbptr->mac_sel)
	{
		case MAC_RD_STATS:		/* Read MAC Statistics */
		{
			o_bytecopy(&ma_stat_tbl, epbptr->mac.stats, sizeof(struct ma_stat));

			o_bytecopy(ma_ts_addr, epbptr->station_addr, MACSZ);

			epbptr->tx_ctr = ma_tx_ctr;
			epbptr->rx_ctr = ma_rx_ctr;

			break;
		}

		case MAC_RD_RING:		/* Read LANCE Ring Sizes */
		{
			epbptr->mac.ring = (USHORT)((1 << TXRLEN) << 8) | (1 << RXRLEN);

			break;
		}

		case MAC_RD_MODE:		/* Read LANCE Mode */
		{
			epbptr->mac.mode = ma_lnc_iblkptr->mode;

			break;
		}

		case MAC_RD_NODE:		/* Read LANCE Node Address */
		{
			o_bytecopy(ma_ts_addr, epbptr->station_addr, MACSZ);

			break;
		}

		case MAC_RD_GROUP:		/* Read LANCE Group Address */
		{
			o_bytecopy(ma_ga_mask, epbptr->mac.group, 16);

			break;
		}

		case MAC_WR_MODE:		/* Write LANCE Mode */
		{
			ma_lnc_iblkptr->mode = epbptr->mac.mode;

			restart = TRUE;

			break;
		}

		case MAC_WR_NODE:		/* Write LANCE Node Address */
		{
			o_bytecopy(epbptr->station_addr, ma_ts_addr, MACSZ);

			restart = TRUE;

			break;
		}

		case MAC_WR_GROUP:		/* Write LANCE Group Address */
		{
			o_bytecopy(epbptr->mac.group, ma_ga_mask, 16);

			restart = TRUE;

			break;
		}

		default:
		{
			epbptr->hdr.status = -1;
		}
	}

/*
**  Next, if the restart flag is set, then restart the LANCE.
*/

	DISABLE(pri, bpp_disable_level);

	if (restart == TRUE)
	{
		ma_rst();

		if (ma_state == MAC_LOCAL)
			epbptr->hdr.status = -1;
	}

/*
**  Return the parameters confirm EPB.
*/

	o_post(epbptr);

	ENABLE(pri);
}
