static char ident[] = " @(#)ma_regdreg.c	35.2        6/17/88	Copyright MOTOROLA, INC. ";

/*MH************************ MODULE HEADER ****************************
**
**  MODULE NAME:	ma_redreg.c
**
**  DESCRIPTION:	This module will hold all the MAC registration
**			and deregistration routines.
**
**  CONTENTS:		ma_reg_req()
**			ma_drg_req()
**
**  SYSTEM GLOBALS:
**
**  MODULE GLOBALS:
**
**  NOTES:
**
**********************************************************************/

/*
**	Include Files
*/

#include <sys/osi/common/types.h>	/* Abstract Type Definitions */
#include <sys/osi/common/system.h>	/* System Function Definitions */
#include <sys/osi/common/bdb.h>		/* BDB Type Definitions */
#include <sys/osi/common/epb.h>		/* EPB Type Definitions */
#include <sys/osi/common/selsizes.h>	/* MAC Address Size */
#include <sys/osi/common/ma_epb.h>	/* MAC EPB Type Definitions */
#include <sys/osi/common/macros.h>	/* Macro Definitions */
#include <sys/osi/common/com_env.h>	/* CE Definitions */

#include "ma_lance.h"			/* LANCE Controller Definitions */
#include "ma_def.h"			/* MAC Node Definitions */
#include "ma_globals.h"			/* Global MAC Node Definitions */

/*
**	External Functions
*/

extern	VOID	o_clean();	/* Clear Bytes */
extern	VOID	o_retepb();	/* Return EPB */
extern	VOID	o_post();	/* Post EPB */

/*FH*********************** FUNCTION HEADER ***************************
**
**  FUNCTION NAME:	ma_reg_req
**
**  DESCRIPTION:	This function performs Ethernet client
**			registration. The passed type and interface
**			point are placed in the registration table.
**
**  PARAMETERS:		
**
**	Input:		epbptr	: EPB pointer
**
**	Output:
**
**	Returns:
**
**  COMPLEX LOCALS:
**
**  METHOD:
**
**  NOTES:
**
**********************************************************************/

VOID
ma_reg_req(epbptr)

register MA_EPB	*epbptr;		/* Pointer To MAC EPB */
{
	register MA_REG	 *regptr;	/* Registration Entry */

	register USHORT	i;		/* Just A Counter */

/*
**  First, we must prepare the MA_REG_CONFIRM EPB to be returned
**  to the originator.
*/

	epbptr->hdr.dst_id = epbptr->hdr.src_id;
	epbptr->hdr.src_id = MAC_QID;
	epbptr->hdr.type   = MA_REG_CONFIRM;

	epbptr->status = -1;

/*
**  Next, we must see if the passed client's Ethernet type has
**  already been registered. If so, an error is returned to the
**  originator.
**
**  If the Ethernet type has not been registered, then the table
**  is searched for the first available entry. The interface point
**  and type (i.e. EPB header fields 'dst_id' and 'user_def') are
**  placed in the entry.
*/

	regptr = (MA_REG *)ma_reg_tbl;

	if (epbptr->hdr.user_def == 0)
	{
		if (regptr->ip == 0)
		{
			regptr->ip = epbptr->hdr.dst_id;

			epbptr->status = 0;
		}
	}
	else
	{
		for (i = 1; i < REG_TBL_SIZE; ++i)
			if (regptr[i].type == (USHORT)epbptr->hdr.user_def)
				break;

		if (i == REG_TBL_SIZE)
			for (i = 1; i < REG_TBL_SIZE; ++i)
				if (regptr[i].type == 0)
				{
					regptr[i].ip   = epbptr->hdr.dst_id;
					regptr[i].type = epbptr->hdr.user_def;

					epbptr->status = 0;

					break;
				}
	}

/*
**  Finally, the function will return the MA_REG_CONFIRM EPB to
**  the originator.
*/

	o_post(epbptr);
}

/*FH*********************** FUNCTION HEADER ***************************
**
**  FUNCTION NAME:	ma_drg_req
**
**  DESCRIPTION:	This function performs Ethernet client
**			deregistration. The passed type and interface
**			point are removed from the registration table.
**
**  PARAMETERS:		
**
**	Input:		epbptr	: EPB pointer
**
**	Output:
**
**	Returns:
**
**  COMPLEX LOCALS:
**
**  METHOD:
**
**  NOTES:
**
**********************************************************************/

VOID
ma_drg_req(epbptr)

register MA_EPB	*epbptr;		/* Pointer To MAC EPB */
{
	register MA_REG	 *regptr;	/* Registration Entry */

	register USHORT	i;		/* Just A Counter */

/*
**  First, we must prepare the MA_DEREG_CONFIRM EPB to be returned
**  to the originator.
*/

	epbptr->hdr.dst_id = epbptr->hdr.src_id;
	epbptr->hdr.src_id = MAC_QID;
	epbptr->hdr.type   = MA_DEREG_CONFIRM;

	epbptr->status = -1;

/*
**  Next, we the registration table is searched to see if the passed
**  client's Ethernet type has been registred. If not, an error is
**  returned to the originator.
**
**  If the Ethernet type has been registered, then the table
**  entry is cleared.
*/

	regptr = (MA_REG *)ma_reg_tbl;

	if (epbptr->hdr.user_def == 0)
	{
		if (regptr->ip != 0)
		{
			o_clean(regptr, sizeof(MA_REG));

			epbptr->status = 0;
		}
	}
	else
		for (i = 1; i < REG_TBL_SIZE; ++i)
			if (regptr[i].type == (USHORT)epbptr->hdr.user_def)
			{
				o_clean((regptr + i), sizeof(MA_REG));

				epbptr->status = 0;

				break;
			}

/*
**  Finally, the function will return the MA_DEREG_CONFIRM EPB to
**  the originator.
*/

	o_post(epbptr);
}
