static char ident[] = " @(#)ma_restart.c	35.2        6/17/88	Copyright MOTOROLA, INC. ";

/*MH************************ MODULE HEADER ****************************
**
**  MODULE NAME:	ma_restart.c
**
**  DESCRIPTION:	This module will hold all the LANCE restart
**			routines.
**
**  CONTENTS:		ma_rst()
**			ma_rst_rng()
**
**  SYSTEM GLOBALS:
**
**  MODULE GLOBALS:
**
**  NOTES:
**
**********************************************************************/

/*
**	Include Files
*/

#include <sys/osi/common/types.h>	/* Abstract Type Definitions */
#include <sys/osi/common/system.h>	/* System Function Definitions */
#include <sys/osi/common/bdb.h>		/* BDB Type Definitions */
#include <sys/osi/common/epb.h>		/* EPB Type Definitions */
#include <sys/osi/common/selsizes.h>	/* MAC Address Size */
#include <sys/osi/common/ma_epb.h>	/* MAC EPB Type Definitions */
#include <sys/osi/common/macros.h>	/* Macro Definitions */

#include "ma_lance.h"			/* LANCE Controller Definitions */
#include "ma_def.h"			/* MAC Node Definitions */
#include "ma_globals.h"			/* Global MAC Node Definitions */

/*
**	External Functions
*/

extern	VOID	ma_int_lnc();	/* LANCE Self-Initialization*/
extern	VOID	ma_isr_tx();	/* Process Transmit Interrupt */
extern	VOID	ma_isr_rx();	/* Process Receive Interrupt */
extern	VOID	ma_set_rap();	/* Set Address Register */
extern	VOID	ma_set_rdp();	/* Set Data Register */
extern	USHORT	ma_get_rdp();	/* Get Data Register */

/*FH*********************** FUNCTION HEADER ***************************
**
**  FUNCTION NAME:	ma_rst
**
**  DESCRIPTION:	This function is very similar to ma_init(),
**			except that ma_rst() is intended for use
**			after initialization is complete, for the
**			purpose of reconfiguring the LANCE under a
**			different mode or simply restarting LANCE
**			operations without reinitializing all MAC
**			data structures.
**
**  PARAMETERS:		
**
**	Input:
**
**	Output:
**
**	Returns:
**
**  COMPLEX LOCALS:
**
**  METHOD:
**
**  NOTES:
**
**********************************************************************/

VOID
ma_rst()

{
	VOID	ma_rst_rng();	/* Reset A LANCE Descriptor Ring */

	register struct txbufrng *rptr;	/* Pointer TX Buffer Ring */

	register ULONG	*iptr;		/* Ring Indexing Pointer */

	register ULONG	tptr;		/* Temp Head Entry */

	register USHORT i;	 	/* Counter */

/*
**  First, the function explicitly stops the LANCE's 
**  operations by setting the STOP bit in LANCE CSR0.
**  This insures integrity in subsequent LANCE init.
*/

	ma_set_rap(RAP_CSR0);

	ma_set_rdp(CSR0_STOP);

/*
**  All completed transmits and receives must be processed before
**  reseting the rings for chip reinitialization.
*/

	ma_isr_tx();
	ma_isr_rx();

/*
**  Now the function ma_rst_rng() is called once for each ring, to reset
**  the rings for chip reinitialization without losing any pending
**  entries or losing track of the chip's place in the ring.
*/

	ma_rst_rng(&ma_lnc_txrgptr);
	ma_rst_rng(&ma_lnc_rxrgptr);

/*
**  Also the Transmit Buffer Ring must be reset for chip
**  reinitialization without losing any pending transmits.
**
**  In order to preserve all ring entries, we move the full 
**  entries (defined as those between the head and the tail)
**  as a group by one entry at a time until the head is at the
**  top of the ring.
*/

	rptr = &ma_txrgptr;

	while (rptr->head != rptr->top)
	{

	/*
	**  In each move of the entries, the head entry is saved in
	**  temporary storage to be restored after the rest of the
	**  full entries have been moved. This prevents overwriting
	**  of full ring entries in the case that all entries are
	**  full.
	*/

		tptr = *(rptr->head);

	/*
	**  This loop moves all full entries toward the ring's top
	**  by one entry.
	*/

		iptr = INCR_RING(rptr->head, *rptr);

		while (iptr != rptr->tail)
		{
			*(DECR_RING(iptr, *rptr)) = *iptr;

			iptr = INCR_RING(iptr, *rptr);
		}

	/*
	**  The ring's tail pointer is moved up one entry, and the
	**  contents of the new tail entry are cleared.
	*/

		rptr->tail = DECR_RING(rptr->tail, *rptr);

		*(rptr->tail) = 0;

	/*
	**  The ring's head pointer is moved up one entry, and the
	**  head entry's contents are restored in the new head entry.
	*/

		rptr->head = DECR_RING(rptr->head, *rptr);

		*(rptr->head) = tptr;
	}

/*
**  Now all is ready for the reinitialization of the chip, which is
**  performed via a call to the standard LANCE init function
**  ma_int_lnc(). When this function returns, the LANCE should be
**  running in normal interrupt-driven operation.
*/

	ma_int_lnc();

/*
**  Upon return, the LANCE CSR0 is read to insure that
**  the LANCE is running with transmitter and receiver
**  on and interrupts enabled. The task state is changed
**  to indicate an error if this is not the case.
*/

	if (ma_get_rdp() != (USHORT)CSR0_GOOD)
	{
		LOG("MAC: LANCE reinitialization error, ");
		LOG("node restricted to local activities\n");

		ma_state = MAC_LOCAL;
	}
	else
		ma_state = MAC_INIT;
}

/*FH*********************** FUNCTION HEADER ***************************
**
**  FUNCTION NAME:	ma_rst_rng
**
**  DESCRIPTION:	This function manipulates a LANCE Descriptor
**			Ring to allow a LANCE initialization without
**			losing any pending ring entries. When
**			initialized, the LANCE begins processing of
**			ring entries at the top of a ring, so this
**			functions moves all full ring entries, in order,
**			to the top of the ring.
**
**			This function assumes the LANCE is stopped upon
**			entry, and therefore knowingly violates the
**			ownership rules that normally apply to ring
**			entries.
**
**  PARAMETERS:		
**
**	Input:		rptr		: pointer to ring's
**					  governing structure
**
**	Output:
**
**	Returns:
**
**  COMPLEX LOCALS:
**
**  METHOD:
**
**  NOTES:
**
**********************************************************************/

VOID
ma_rst_rng(rptr)

register struct ringptr *rptr;	/* Pointer To Ring's Governing Structure */
{
	struct	 ringentry tentry;	 /* Temp Head Entry */

	register struct ringentry *iptr; /* Ring Indexing Pointer */
	register struct ringentry *tptr; /* Temp Head Entry Pointer */

	register USHORT i;		 /* Counter */

/*
**  In order to preserve all ring entries, we move the full entries
**  (defined as those between the head and the tail which are owned
**  by the chip) as a group by one entry at a time until the head is
**  at the top of the ring.
*/

	tptr = &tentry;

	while (rptr->head != rptr->top)
	{

	/*
	**  In each move of the entries, the head entry is saved in
	**  temporary storage to be restored after the rest of the
	**  full entries have been moved. This prevents overwriting
	**  of full ring entries in the case that all entries are
	**  full.
	*/

		tptr->md[0] = rptr->head->md[0];
		tptr->md[1] = rptr->head->md[1];
		tptr->md[2] = rptr->head->md[2];
		tptr->md[3] = rptr->head->md[3];

	/*
	**  This loop moves all full entries toward the ring's top
	**  by one entry.
	*/

		iptr = INCR_RING(rptr->head, *rptr);

		while (iptr != rptr->tail)
		{
			for (i = 0; i < 4; ++i)
				(DECR_RING(iptr, *rptr))->md[i] = iptr->md[i];

			iptr = INCR_RING(iptr, *rptr);
		}

	/*
	**  The ring's tail pointer is moved up one entry, and the
	**  contents of the new tail entry are cleared.
	*/

		rptr->tail = DECR_RING(rptr->tail, *rptr);

		rptr->tail->md[0] = 0;
		rptr->tail->md[1] = 0;
		rptr->tail->md[2] = 0;
		rptr->tail->md[3] = 0;

	/*
	**  The ring's head pointer is moved up one entry, and the
	**  head entry's contents are restored in the new head entry.
	*/

		rptr->head = DECR_RING(rptr->head, *rptr);

		rptr->head->md[0] = tptr->md[0];
		rptr->head->md[1] = tptr->md[1];
		rptr->head->md[2] = tptr->md[2];
		rptr->head->md[3] = tptr->md[3];
	}
}
