static char ident[] = " @(#)sm_init.c	35.2        6/17/88	Copyright MOTOROLA, INC. ";

/*MH************************ MODULE HEADER ****************************
**
**  MODULE NAME:        sm_init.c
**
**  DESCRIPTION:        Station Manager initialization
**
**  CONTENTS:           Check whether the station address is set. If not,
**			set it to the manufacturer's assigned address.
**
**  SYSTEM GLOBALS:	none
**
**  MODULE GLOBALS:	none
**
**  NOTES:		This module is only valid for the MVME147
**			environment.
**
**********************************************************************/

#include <sys/osi/common/types.h>
#include <sys/osi/common/macros.h>
#include <sys/osi/board/mvme147.h>
#include <sys/nvram.h>

#define	BUG147_TS_ADDR_INDEX	(0x778 - 0x5f8)
#define	BUG147_TS_ADDR_SIZE	3

#define	TS_BLOCK_ADDR_SIZE	3

STATIC	UBYTE	ts_block_addr[TS_BLOCK_ADDR_SIZE] = { 0x08, 0x00, 0x3e };

/*FH************************ FUNCTION HEADER ****************************
**
**  FUNCTION NAME:	sm_init()
**
**  DESCRIPTION:        Read the network station address. If it is all 0's
**			read the maunfacturer's assigned sub address and
**			combine it with the fixed address part.
**
**  PARAMETERS:		
**
**	Input:		None
**
**	Output:		None
**
**	Returns:	Void
**
*************************************************************************/

VOID
sm_init()

{
	UBYTE	*p;
	LONG	err;
	UBYTE	buf[MVME147_TS_ADDR_SIZE];

	/*
	**  Read station address and check for all 0's. If it's non-
	**  zero there is nothing to do.
	*/

	if ((err = nvram(READ, NET, buf, MVME147_TS_ADDR_SIZE, MVME147_TS_ADDR_INDEX)) < OK)
	{
		LOG("SMA: nvram() read NET station address error %d\n", err);
		return;
	}

	for (p = buf; p < &buf[MVME147_TS_ADDR_SIZE]; p++)
		if (*p)
			return;		/* Non-zero station address */

	/*
	**  Set the address to the manufactorer assigned address built
	**  from a fixed address block and a serial number stored in
	**  the mvme147 nvram area.
	*/

	o_bytecopy(ts_block_addr, buf, TS_BLOCK_ADDR_SIZE);

	if ((err = nvram(READ, BUG, &buf[TS_BLOCK_ADDR_SIZE], BUG147_TS_ADDR_SIZE, BUG147_TS_ADDR_INDEX)) < OK)
	{
		LOG("SMA: nvram() read BUG station address error %d\n", err);
		return;
	}

	if ((err = nvram(WRITE, NET, buf, MVME147_TS_ADDR_SIZE, MVME147_TS_ADDR_INDEX)) < OK)
	{
		LOG("SMA: nvram() write NET station address error %d\n", err);
	}
}
