static char ident[] = " @(#)sm_nvram.c	35.2        6/17/88	Copyright MOTOROLA, INC. ";

/*MH************************ MODULE HEADER ****************************
**
**  MODULE NAME:	sm_nvram.c
**
**  DESCRIPTION:        This module process incoming nvram events.
**
**  CONTENTS:           sm_nvram_get()	convert nibble to byte aligned
**			sm_nvram_put()	convert byte to nibble aligned
**			sm_nvram()	processes events
**
**  SYSTEM GLOBALS:	none
**
**  MODULE GLOBALS:	none
**
***********************************************************************/

#include <sys/osi/common/types.h>
#include <sys/osi/common/system.h>
#include <sys/osi/common/selsizes.h>
#include <sys/osi/common/bdb.h>
#include <sys/osi/common/epb.h>
#include <sys/osi/common/sm_epb.h>
#include <sys/osi/common/macros.h>
#include <sys/osi/board/mvme374.h>
#include <sys/osi/board/mvme147.h>
#include <sys/osi/board/nvram.h>
#include <sys/nvram.h>

extern	 VOID	sm_swap();
extern	 VOID	o_retepb();
extern	 VOID	o_clean();

/*FH*********************** FUNCTION HEADER ***************************
**
**  FUNCTION NAME:	sm_nvram_get()
**
**  DESCRIPTION:	This function will read nibbles from memory structured
**			like non-volatile RAM, collect them into bytes, and
**			store them at the user specified location.
**
**  PARAMETERS:		
**
**	Input:		pointer to byte array, nibble offset into the array,
**			and the number of bytes to read.
**
**	Output:		Source contents (in nibble format) is put into
**			destination in byte aligned format.
**
**	Returns:	Void
**
**********************************************************************/

VOID
sm_nvram_get(dst, src, offset, count)

register UBYTE	*dst;		/* destination to byte storage	*/
register UBYTE	*src;		/* source of nibble storage	*/
register ULONG	offset;		/* nibble offset		*/
register ULONG	count;		/* byte count			*/
{
	register UBYTE	*end;		/* end of byte storage		*/

	src	+= offset / 2; 
	end	= &dst[count];

	if (offset & 1)
	{
		while (dst < end)
		{
			*dst 	= *src++ << 4;
			*dst++	|= *src >> 4;
		}
	}
	else
	{
		while (dst < end)
			*dst++ = *src++;
	}
}

/*FH*********************** FUNCTION HEADER ***************************
**
**  FUNCTION NAME:	sm_nvram_put()
**
**  DESCRIPTION:	This function will write data into a memory area
**			structured like non-volital RAM of the 37x.
**
**  PARAMETERS:		
**
**	Input:		Pointer to source, pointer to destination, nibble
**			offset into area, and number of bytes to store.
**
**	Output:		Source contents is put at destination in nibble
**			format (including a 1 nibble checksum at the end)
**
**	Returns:	Void
**
**********************************************************************/

VOID
sm_nvram_put(src, dst, offset, count)

register UBYTE	*src;		/* source of bytes to write	*/
register UBYTE	*dst;		/* destination of result	*/
register ULONG	offset;		/* nibble offset		*/
register ULONG	count;		/* byte count			*/
{
	register UBYTE	*end;		/* end of byte storage		*/
	register UBYTE	chk;		/* checksum variable		*/
	register UBYTE	tmp;

	chk  = 0xff;
	dst += offset / 2; 
	end  = &src[count];

	if (offset & 1)
	{
		while (src < end)
		{
			tmp	= *src >> 4;
			chk	^= tmp;
			*dst++	= (*dst & 0xf0) | tmp;
			tmp	= *src++ & 0x0f;
			chk	^= tmp;
			*dst	= (*dst & 0x0f) | tmp << 4;
		}
		*dst	= (*dst & 0xf0) | (chk & 0x0f);
	}
	else
	{
		while (src < end)
		{
			chk	^= *src >> 4;
			chk	^= *src & 0x0f;
			*dst++ = *src++;
		}
		*dst	= (*dst & 0x0f) | (chk << 4);
	}
}

/*FH******************** FUNCTION HEADER *********************************
**
**  FUNCTION NAME:      sm_nvram()
**
**  DESCRIPTION:        This module process user events by calling
**			routines in system to perform NVRAM read and
**			NVRAM write.
**
**  PARAMETERS:		
**
**	Input:		Pointer to station manager nvram request epb
**
**	Output:		Posts nvram read confirm if input was nvram read
**			request.
**
**	Returns:	Void
**
**  NOTES:		This routine simulates the behaviour of a mvme374
**			station manager agent. The only nvram parameters
**			supported for now are station address and group
**			address mask. All other nvram fields are cleared
**			to generate a checksum error in case they will be
**			displayed by the host resident station manager.
**
**********************************************************************/

VOID
sm_nvram(sm_epb)

register SM_EPB	*sm_epb;
{
	register LONG	err;
		 UBYTE	buf[64];

        switch (sm_epb->hdr.type)
        {
                case SM_NVRAM_RD_REQUEST:	/* Read NVRAM */
		{
			sm_epb->hdr.status = 0;

			o_clean(sm_epb->nvram, NVRAM_SIZE/2);

			if ((err = nvram(READ, NET, buf, MVME147_TS_ADDR_SIZE, MVME147_TS_ADDR_INDEX)) < OK)
			{
				LOG("SMA: nvram() read NET station address error %d\n", err);
				sm_epb->hdr.status = -1;
			}
			else
			{
				sm_nvram_put(buf, sm_epb->nvram, TS_ADDR_INDEX, TS_ADDR_SIZE);
			}

			if ((err = nvram(READ, NET, buf, MVME147_GA_MASK_SIZE, MVME147_GA_MASK_INDEX)) < OK)
			{
				LOG("SMA: nvram() read NET group address mask error %d\n", err);
				sm_epb->hdr.status = -1;
			}
			else
				sm_nvram_put(buf, sm_epb->nvram, GA_MASK_INDEX, GA_MASK_SIZE);

			sm_swap(SM_NVRAM_RD_CONFIRM, sm_epb);

			LOG("SMA: posted nvram() read confirm to host\n");

			break;
		}

                case SM_NVRAM_WR_REQUEST:	/* Write NVRAM */
		{
			sm_nvram_get(buf, sm_epb->nvram, TS_ADDR_INDEX, TS_ADDR_SIZE);

			if ((err = nvram(WRITE, NET, buf, MVME147_TS_ADDR_SIZE, MVME147_TS_ADDR_INDEX)) < OK )
			{
				LOG("SMA: nvram() write NET station address error %d\n", err);
			}

			sm_nvram_get(buf, sm_epb->nvram, GA_MASK_INDEX, GA_MASK_SIZE);

			if ((err = nvram(WRITE, NET, buf, MVME147_GA_MASK_SIZE, MVME147_GA_MASK_INDEX)) < OK )
			{
				LOG("SMA: nvram() write NET group address mask error %d\n", err);
			}

			o_retepb(sm_epb);

			break;
		}
	}
}
