static char ident[] = " @(#)o_buffmgmt.c	35.2        6/17/88	Copyright MOTOROLA, INC. ";

/*MH************************ MODULE HEADER ****************************
**
**  MODULE NAME:	o_buffmgmt.c
**
**  DESCRIPTION:	This module contains the functions to do
**			buffer management for BDB's and EPB's using the 
**			ci_driver get and release buffer functions.
**
**  CONTENTS:		BDB   *o_getbdb(pid);
**			EPB   *o_getepb(pid);
**			VOID  o_retbdb(bdb_pointer);
**			VOID  o_retepb(epb_pointer);
**
**
**  SYSTEM GLOBALS:
**
**  MODULE GLOBALS:
**
**  NOTES:
**
**********************************************************************/

#include <sys/c37x/ci_driver.h>
#include <sys/c37x/ci_const.h>
#include <sys/c37x/ci_struct.h>
#include <sys/c37x/ci_global.h>

#include <sys/bppv2/bpptypes.h>
#include <sys/bppv2/bpp.h>
#include <sys/bppv2/bm.h>
#include <sys/bppv2/ccb.h>
#include <sys/bppv2/mvme147/bpp_local.h>
#include <sys/bppv2/mvme147/bp_ci.h>

#include <sys/osi/board/mvme147.h>
#include <sys/osi/common/bdb.h>
#include <sys/osi/common/system.h>
#include <sys/osi/common/macros.h>

/*
**	External Functions
*/

extern	EPB	*ci_get_epb();
extern	BDB	*ci_get_buf();
extern	VOID	ci_rel_epb();
extern	VOID	ci_rel_buf();

/*
**	External Variables
*/

extern	CPU_ENT	bp_cpu_ent;
extern	CPU_ENT	ci_cpu[];

/*FH*********************** FUNCTION HEADER ***************************
**
**  FUNCTION NAME:	o_getbdb (pid)
**
**  DESCRIPTION:	This function returns a pointer to a buffer
**			descriptor block of the specified partition.
**
**  PARAMETERS:
**
**	Input:		Partition id.
**
**	Output:
**
**	Returns:	Pointer to BDB, or NULL if none available.
**
**  METHOD:
**
**  NOTES:		The parameter pid could be omitted for this
**			implementation, but may be useful in the
**			future when BDBs of different sizes might be
**			implemented.
**
**********************************************************************/

BDB *
o_getbdb(pid)

DEFAULT	pid;			/* Partition id */
{
	register BDB	*bdb;		/* Pointer to BDB */

	if ((bdb = ci_get_buf(&ci_cpu[VME147_CPU_NO - CPU_OFFSET])) != (BDB *)NULL)
	{
		/*
		**	Adjust BUF pointer back to provide BDB header
		*/

		bdb = (BDB *)((BYTE *)bdb - (BUF_FILL + BDB_HDR_SIZE));

		bdb->pid	 = pid;
		bdb->data	 = bdb->dummy + BUF_FILL + BUF_HDR_SIZE;
		bdb->next_bdb	 = (BDB *)NULL;
		bdb->buff_size	 = PDU_SIZE;
		bdb->buff_offset = PDU_SIZE;
		bdb->buff_length = 0;
	}

	return (bdb);
}

/*FH*********************** FUNCTION HEADER ***************************
**
**  FUNCTION NAME:	o_getepb (pid)
**
**  DESCRIPTION:	This function returns a pointer to an event
**			parameter block of the specified partition.
**
**  PARAMETERS:
**
**	Input:		Partition id
**
**	Output:
**
**	Returns:	Pointer to EPB, or NULL if none available.
**
**  METHOD:
**
**  NOTES:		The parameter pid could be omitted for this
**			implementation, but may be useful in the
**			future when EPBs of different sizes might be
**			implemented.
**
**********************************************************************/

EPB *
o_getepb(pid)

DEFAULT pid;
{
	register EPB	*epb;		/* Pointer to EPB */

	if ((epb = (EPB *)ci_get_epb(&bp_cpu_ent)) != (EPB *)NULL)
	{
		epb->next   = (EPB *)NULL;
		epb->buf    = (BUF *)NULL;
	}

	return (epb);
}

/*FH*********************** FUNCTION HEADER ***************************
**
**  FUNCTION NAME:	o_retbdb (bdb_pointer)
**
**  DESCRIPTION:	This function returns a buffer descriptor
**			block to the partition it came from.
**
**  PARAMETERS:
**
**	Input:		Buffer descriptor block pointer.
**
**	Output:
**
**	Returns:
**
**  METHOD:
**
**  NOTES:
**
**********************************************************************/

VOID
o_retbdb(bdb)

register BDB	*bdb;
{
	/*
	**	Adjust BDB pointer to get the BUF pointer.
	*/

	bdb = (BDB *)((BYTE *)bdb + (BUF_FILL + BDB_HDR_SIZE));

	ci_rel_buf(&ci_cpu[VME147_CPU_NO - CPU_OFFSET], bdb);
}

/*FH*********************** FUNCTION HEADER ***************************
**
**  FUNCTION NAME:	o_retepb (epb_pointer)
**
**  DESCRIPTION:	This function will return an event parameter
**			block to the partition it came from.
**
**  PARAMETERS:
**
**	Input:		Event parameter block pointer.
**
**	Output:
**
**	Returns:
**
**  METHOD:
**
**  NOTES:
**
**********************************************************************/

VOID
o_retepb(epb)

register EPB	*epb;
{
	ci_rel_epb(&bp_cpu_ent, epb);
}
