#  @(#)o_bytecopy.s	35.2        6/17/88

###FH######################## FUNCTION HEADER ################################
##
##  FUNCTION NAME:	o_bytecopy.s
##  
##  DESCRIPTION:	This assembly language subroutine moves a 
##			specified number of bytes of memory  from a
##			source address to a destination address.
##  
## 			The parameters the subroutine uses are: 
##			      	Source address
##			     	Destination address
##				Count of the number of bytes to move	
##
##			These parameters are expected to be passed to the  
##			subroutines on the stack.
##
##			All registers are preserved except the following-
##				A0, A1, D0, D1, SR
##
##			The following registers will contain these values
##			during execution of the subroutine:
##				A0 - Source Address
##				A1 - Destination Address
##				D0 - Initially total count, then block
##				     move count
##				D1 - Word and byte move count
##
##
##  NOTES:
##
##############################################################################


	file	"o_bytecopy.s"

	set	SRC,	0x4	# Offset from top of stack to SRC parm
 	set	DST,	0x8	# Offset from top of stack to DST parm
	set	CNT,	0xc	# Offset from top of stack to CNT parm

	set	REGS,	10	# Number of registers available for MOVEM
	set	LWRD,	0x4	# Number of bytes in a long word
	
	set	MAXMV,	REGS*LWRD	# Max bytes moved

	set	PUSH_REGS,	0x3f3c	# Mask to move D2-D7 and A2-A5 to stack
	set	PULL_REGS,	0x3cfc	# Mask to move A5-A2 and D7-D2 from stk

	set	SRC_REGS,	0x3cfc	# Move data to these registers
	set	DST_REGS,	0x3f3c	# Move data from these regs to mem	


        global  o_bytecopy
	text

o_bytecopy:
	mov.l	SRC(%sp),%a0	# Retrieve the parameters from the stack
	mov.l	DST(%sp),%a1
	mov.l	CNT(%sp),%d0
	beq.b	out		# Leave if the count is zero
	movm.l	&PUSH_REGS,-(%sp)	# Save registers
	mov.l	%a0,%d1		# Copy the source address 
	and.w	&03,%d1 	# Isolate the two least significant bits so we
	beq.b	div_count	# can align the source address on a long word
	neg.w	%d1		# boundary.
	add.w	&3,%d1

align:
	mov.b	(%a0)+,(%a1)+	# Move a byte at a time until the source
	subq.l	&1,%d0		# address is on a long word boundary 
	beq.b	done
	dbra.w	%d1,align

div_count:
	divu.w	&MAXMV,%d0	# Divide the total count by the number of bytes
	mov.l	%d0,%d1		# that can be moved in a block.  The quotient
	swap	%d1		# will be in D0 and used in the block move and
				# the remainder will be in D1 and used in the 
				# word and byte move.
	

	sub.w	&1,%d0		# Adjust the count for the block move
	bmi.b	word_sub	# and if the count is not zero then adjust the
	add.l	&MAXMV,%a1	# pointer to the destination

mv_block:
	movm.l	(%a0)+,&SRC_REGS	# Move a block of memory 
	movm.l	&DST_REGS,-(%a1)
	add.l	&2*MAXMV,%a1	# -- Point the dst address register to
	dbra.w	%d0,mv_block	# -- the end of this block so we can
        sub.l   &MAXMV,%a1	# -- move memory using the predecrement
	bra.b	word_sub	# -- move multiple.

mv_word:
	mov.w	(%a0)+,(%a1)+	# Move memory a long word at a time

word_sub:
	sub.w	&2,%d1
	bge.b	mv_word
	add.w	&1,%d1		# Move memory a byte at a time.
	bmi.b	done
	mov.b	(%a0)+,(%a1)+

done:	
	movm.l	(%sp)+,&PULL_REGS

out:
	rts
