static char ident[] = " @(#)o_util.c	35.2        6/17/88	Copyright MOTOROLA, INC. ";

/*MH************************ MODULE HEADER ****************************
**
**  MODULE NAME:        o_util.c    
**
**  DESCRIPTION:        This module contains miscellaneuos system routines.
**
**  CONTENTS:
**			o_post() 	- swap EPB Ids and post it.
**  			o_bytecmp()	- compare two unsigned byte strings
**					  for equality.
**			o_clean()	- clear a consecutive string of bytes.
**			o_strlength()	- returns the length of a
**					  null-terminated character string.
**			o_strcopy()	- Copy null-terminated character string.
**
**  NOTES:
**
**********************************************************************/

#include <sys/osi/common/types.h>
#include <sys/osi/common/system.h>
#include <sys/osi/common/selsizes.h>
#include <sys/osi/common/bdb.h>
#include <sys/osi/common/epb.h>
#include <sys/osi/common/macros.h>

#include <sys/bppv2/bpp.h>

/*FH*********************** FUNCTION HEADER ****************************
**
**  FUNCTION NAME:      o_post()
**
**  DESCRIPTION:	Put passed type into passed EPB, swap EPB Ids
**			and post EPB.
**
**  PARAMETERS:		
**
**	Input:		Pointer to EPB.
**
**	Output:		
**
**	Returns:	SUCCESS if EPB is 'posted',
**			FAILURE if EPB is not 'posted'.
**
**  COMPLEX LOCALS:
**
**  METHOD:
**
**  NOTES:
**
***********************************************************************/

BOOLEAN
o_post(epb)

register EPB	*epb;
{
	if (bp_send(epb->dst_id, epb->src_id, ((BYTE *)epb - sizeof(BP_MPB_T)), epb->epb_size, epb) != BPP_SUCCESSFUL)
	{
		/*
		**	Unable to deliver message.
		*/

		LOG("o_post: Unable to route message\n");

		if (epb->bdb)
			o_retbdb(epb->bdb);

		o_retepb(epb);

		return (FAILURE);
	}

	return (SUCCESS);
}

/*FH*********************** FUNCTION HEADER ***************************
**
**  FUNCTION NAME:	o_bytecmp()
**
**  DESCRIPTION:	This function will compare two unsigned byte
**			strings for equality.  The strings are assumed
**			to be of equal length.  No attempt is made to
**			determine which string is "greater than" the
**			other.
**
**  PARAMETERS:		
**
**	Input:		Two UBYTE pointers and a count.
**
**	Output:		Neither string is altered.
**
**	Returns:	SUCCESS if the two strings match
**			FAILURE if the two strings do not match.
**
**  COMPLEX LOCALS:
**
**  METHOD:
**
**  NOTES:
**
**********************************************************************/

BOOLEAN
o_bytecmp(strg1, strg2, count)

register UBYTE	*strg1;
register UBYTE	*strg2;
register DEFAULT count;
{
	while(count--)
		if(*strg1++ != *strg2++)
			return(FAILURE);
	
	return(SUCCESS);
}

/*FH*********************** FUNCTION HEADER ***************************
**
**  FUNCTION NAME:	o_clean()
**
**  DESCRIPTION:	This function will clear (place binary zeros into)
**			a consecutive string of bytes.
**
**  PARAMETERS:		
**
**	Input:		UBYTE pointer to the start of the string,
**			DEFAULT byte count.
**
**	Output:		Memory zero'd out.
**
**	Returns:	Nothing
**
**  COMPLEX LOCALS:
**
**  METHOD:
**
**  NOTES:
**
**********************************************************************/

VOID
o_clean(ptr, count)

register UBYTE	*ptr;
register DEFAULT count;
{
	while (count--)
		*ptr++ = 0;
}

/*FH*********************** FUNCTION HEADER ***************************
**
**  FUNCTION NAME:	o_strlength()
**
**  DESCRIPTION:	Returns the length of a null-terminated character
**			string.
**
**  PARAMETERS:
**
**	Input:		s - pointer to null-terminated string
**
**	Output:
**
**	Returns:	length (in bytes) of the string
**
**  COMPLEX LOCALS:
**
**  METHOD:
**
**  NOTES:
**
**********************************************************************/

DEFAULT 
o_strlength(s)

register UBYTE	*s;
{
	register DEFAULT n;

	n = 0;

	while (*s++ != 0)
		n++;

	return (n);
}

/*FH*********************** FUNCTION HEADER ***************************
**
**  FUNCTION NAME:	o_strcopy()
**
**  DESCRIPTION:	Copy all bytes up to and including the first zero
**			byte from the src pointer to the destination ptr.
**
**  PARAMETERS:
**
**	Input:		src	- source byte string pointer,
**			dst	- destination byte string pointer.
**
**	Output:
**
**	Returns:
**
**  COMPLEX LOCALS:
**
**  METHOD:
**
**  NOTES:
**
**********************************************************************/

VOID
o_strcopy(src, dst)

register UBYTE	*src;
register UBYTE	*dst;
{
	while ((*dst++ = *src++) != 0);
}
