#ident "@(#)socket.c	35.4"

/*
** Stub routines for Socket system calls.  For socket(), force error
** return and 'imply' absence of package as reason.  For remainder,
** assume you can't really have a socket descriptor, if you can't do
** socket().  Should probably do ENOPKG for these, but lean toward
** stock BSD flavor...
**
** If read/write/ioctl/close are called, someone
** set the ISOCKET flag in the inode.  It's panic time....
*/

#include <sys/types.h>
#include <sys/param.h>
#include <sys/fs/s5dir.h>
#include <sys/signal.h>
#include <sys/user.h>
#include <sys/errno.h>
#include <sys/cmn_err.h>

socket()
{
	u.u_error = EPROTONOSUPPORT;
	return (0);
}

sck_bind()
{
	u.u_error = ENOTSOCK;
	return (0);
}

sck_connect()
{
	u.u_error = ENOTSOCK;
	return (0);
}

sck_listen()
{
	u.u_error = ENOTSOCK;
	return (0);
}

sck_accept()
{
	u.u_error = ENOTSOCK;
	return (0);
}

sck_getsockname()
{
	u.u_error = ENOTSOCK;
	return (0);
}

sck_getpeername()
{
	u.u_error = ENOTSOCK;
	return (0);
}

sck_send()
{
	u.u_error = ENOTSOCK;
	return (0);
}

sck_sendto()
{
	u.u_error = ENOTSOCK;
	return (0);
}

sck_sendmsg()
{
	u.u_error = ENOTSOCK;
	return (0);
}

sck_recv()
{
	u.u_error = ENOTSOCK;
	return (0);
}

sck_recvfrom()
{
	u.u_error = ENOTSOCK;
	return (0);
}

sck_recvmsg()
{
	u.u_error = ENOTSOCK;
	return (0);
}

sck_shutdown()
{
	u.u_error = ENOTSOCK;
	return (0);
}

setsockopt()
{
	u.u_error = ENOTSOCK;
	return (0);
}

getsockopt()
{
	u.u_error = ENOTSOCK;
	return (0);
}

/*
 *	Catch impossible read/write/ioctl/close operations.
 * 	noreach() should really be in trap.c
 */

static
noreach(routine)
char *routine;
{
    cmn_err (CE_PANIC,
	     "Call to internal routine of uninstalled package (%s)", routine);
}

sck_close()
{
	noreach("sck_close");
}

sck_read()
{
	noreach("sck_read");
}

sck_write()
{
	noreach("sck_write");
}

sck_ioctl()
{
	noreach("sck_ioctl");
}

sck_select()
{
	noreach("sck_select");
}
