/*****************************************************************************
 *									     *
 *	Layout of MC68681 DUART registers for the ME252			     *
 *									     *
 *	Date: 24/4/88	Authors: M. Woods	P. Callow		     *	
 *									     *
 ****************************************************************************/


struct port {
	char d_mr12,	/* mode register 1 and 2 */
	     d_srcs,	/* status register | clock select register */
             d___cm,	/*     --------    | command register */
	     d_data;	/*			data register */
};

struct mc68681 {
	struct port A;
	char   d_ipac,	/* input port change | auxilary control register */
	       d_isim,	/* interrupt status  | interrupt mask register */
	       d_ctu,	/* counter/timer upper byte */
	       d_ctl;	/* counter/timer lower byte */
	struct port B;
	char   d_iv,		/* interrupt vector register */
	       d_ipopc,	/* input port		| output port configuration */
	       d_bcsop,	/* begin counter    | set output port bits */
	       d_ecrop;	/* stop  counter    | reset output port bits */
};

#define NPORTS	2		/* ports per duart */

/* port mode register 1 coding (port.d_mr12) */

#define RXRTSC		0x80
#define NOPARITY	0x10
#define PEVEN		0x00
#define PODD		0x04
#define BPC5		0x00
#define BPC6		0x01
#define BPC7		0x02
#define BPC8		0x03

#define MR1		(NOPARITY|BPC8)

/* port mode register 2 coding (port.d_mr12) */

#define TXRTSC	0x20
#define TXCTSE	0x10
#define STOP1	0x07
#define STOP2	0x0f
#define LOCLOOP	0x80

#define MR2		(STOP1)

/* port clock select register coding (port.d_srcs) */

#define S75		0x00
#define S110	0x11
#define S134	0x22
#define S150	0x33
#define S300	0x44
#define S600	0x55
#define S1200	0x66
#define S2000	0x77
#define S2400	0x88
#define S4800	0x99
#define S9600	0xbb
#define S19200	0xcc

#define CS		S9600

/* port commands (port.d___cm) */

#define RXEN	0x01
#define RXDIS	0x02
#define TXEN	0x04
#define TXDIS	0x08

#define R_MRP  	0x10	/* reset mode register pointer */
#define R_RCV	0x20	/* reset receiver and disable (like HW reset) */
#define R_TXM	0x30	/* reset transmitter and dis. (like HW reset) */
#define R_ERR	0x40
#define R_CBI	0x50	/* reset channel break change interrupt */
#define SBRK	0x60	/* start break */
#define EBRK	0x70	/* stop break */

/* port status bits (port.d_srcs) */

#define BRK		0x80
#define FRAME	0x40
#define PARITY	0x20
#define OVERUN	0x10
#define TXEMT	0x08
#define CANSEND	0x04
#define GOTCHAR	0x01

#define ERRBITS	(BRK|FRAME|PARITY|OVERRUN)

/* duart interrupt status and mask bits (mc68681.d_isim) */

#define IRTXA	0x01	/* transmitter ready port A */
#define IRRXA	0x02	/* receiver ready A */
#define IRTXB	0x10	/* transmitter ready port B */
#define IRRXB	0x20	/* receiver ready B */

/* duart auxilary control register (mc68681.d_ipac) */

#define ACR		0x80	/* selects baudrate set 2 */
