/*		Copyright (c) 1986, 1987 Motorola
 *		All Rights Reserved		
 *
 *		bpp.h - buffered pipe protocol	defines and structs
 *
 *		@(#)bpp.h	35.2
 */


/*
 *	Packet Structure
 */

struct b_packet {
	unsigned long	p_eye;			/* eyecatcher		*/
	unsigned char	p_cmdp;			/* command pipe number	*/
	unsigned char	p_statp;		/* status pipe number	*/
#ifdef	m88k
	unsigned short	p_numhi;		/* packet number	*/
	unsigned short	p_numlo;		/* packet number	*/
#else
	unsigned long	p_num;			/* packet number	*/
#endif
	unsigned short	p_cmd;			/* command		*/
	unsigned char	p_dtype;		/* device type		*/
	unsigned char	p_fill1;		/* reserved		*/
	unsigned char	p_am;			/* address modifier	*/
	unsigned char	p_busw;			/* data bus width	*/
	unsigned char	p_unit;			/* primary unit		*/
	unsigned char 	p_sunit;		/* secondary unit	*/
#ifdef	m88k
	unsigned short  pa_pp_blknohi;		/* Needs a better def. */
	unsigned short  pa_pp_blknolo;
#else
	union {
		unsigned long	pp_blkno;	/* for read/write	*/
		struct {			/* for formats		*/
			ushort		pp_cyl;	/* starting cylinder	*/
			unsigned char	pp_hd;	/* starting head	*/
			unsigned char 	pp_sec; /* sector number 	*/
		} fmt;
	} pa;					/* primary address	*/
#endif
#ifdef	m88k
	unsigned short	p_sahi;			/* secondary address	*/
	unsigned short	p_salo;			/* secondary address	*/
#else
	unsigned long	p_sa;			/* secondary address	*/
#endif
#ifdef	m88k
	unsigned short	p_counthi;		/* transfer count	*/
	unsigned short	p_countlo;		/* transfer count	*/
#else
	unsigned long	p_count;		/* transfer count	*/
#endif
#ifdef	m88k
	ushort	op_pp_cp1;
	ushort	op_pp_cp2;
	ushort	op_pp_cp3;
#else
		struct {
			ushort	pp_cp1;
			ushort	pp_cp2;
			ushort	pp_cp3;
		} op;
#endif
	unsigned long	p_cstat;		/* completion status	*/
	unsigned short	p_estat;		/* extended status	*/
	unsigned char	p_sberror;		/* sys bus error status	*/
	unsigned char	p_lberror;		/* local bus status	*/
	unsigned short	p_retry;		/* retry count		*/
#ifdef	m88k
	unsigned short	pp_psahi;
	unsigned short	pp_psalo;
#else
	union {
		unsigned long	pp_psa;
		struct {			/* ending values	*/
			ushort		pp_cyl;	/* logical cylinder	*/
			unsigned char	pp_hd;	/* logical head		*/
			unsigned char	pp_sec;	/* logical sector	*/
		} log;
	} psa;					/* primary status addr	*/
#endif
#ifdef	m88k
	short		p_sstathi;		/* secondary stat addr	*/
	short		p_sstatlo;		/* secondary stat addr	*/
	unsigned short	p_termcnthi;		/* termination xfer cnt	*/
	unsigned short	p_termcntlo;		/* termination xfer cnt	*/
#else
	long		p_sstat;		/* secondary stat addr	*/
	unsigned long	p_termcnt;		/* termination xfer cnt	*/
#endif
#ifdef	m88k
	unsigned short ops_opstat[3];
#else
	union {
		unsigned short opstat[3];
		struct {			/* ending values	*/
			ushort 		pp_cyl;	/* physical cylinder	*/
			unsigned char	pp_hd;	/* physical head	*/
			unsigned char	pp_s7261;	/* 7261 status	*/
			unsigned char 	pp_seccnt;	/* sector count	*/
			unsigned char	pp_sksts;	/* seek status	*/
		} phys;
	} ops;					/* op code status	*/
#endif
};

#define	p_blkno		pa.pp_blkno
#define	p_cyl		pa.fmt.pp_cyl
#define	p_hd		pa.fmt.pp_hd
#define p_sec		pa.fmt.pp_sec
#define	p_cp1		op.pp_cp1
#define	p_cp2		op.pp_cp2
#define	p_cp3		op.pp_cp3
#define	p_badcnt	op.pp_cp1
#define	p_badtrk	op.pp_cp2
#define	p_psa		psa.pp_psa;
#define	p_lcyl		psa.log.pp_cyl
#define	p_lhd		psa.log.pp_hd
#define	p_lsec		psa.log.pp_sec
#define p_opstat	ops.opstat
#define	p_pcyl		ops.phys.pp_cyl
#define	p_phd		ops.phys.pp_hd
#define	p_s7261		ops.phys.pp_s7261
#define	p_seccnt	ops.phys.pp_seccnt
#define	p_sksts		ops.phys.pp_sksts

/*
 *	Envelope Structure
 */

struct b_envelope {
	struct b_envelope	*e_link;	/* next envelope  */
	struct b_packet		*e_pkt;		/* packet pointer */
	unsigned char		e_valid;	/* valid flag	  */
	char			e_res1;		/* reserved	  */
	short			e_res2;		/* reserved	  */
};

#define	INVALID	0
#define	VALID	1

/*
 *	Pipe Structure
 */

struct b_pipe {
	struct b_envelope	*p_head;	/* pipe head pointer */
	struct b_envelope	*p_tail;	/* pipe tail pointer */
};

/*
 *	Channel Structure
 */

struct b_channel {
	struct b_pipe	c_cmd;		/* cmd pipe to dequeue packets form */
	struct b_pipe	c_stat;		/* stat pipe to enqueue packets to  */
	unsigned char	c_level;	/* interrupt level		    */
	unsigned char	c_vector;	/* interrupt vector number	    */
	unsigned char	c_prio;		/* priority associated with channel */
	unsigned char	c_am;		/* function codes for memory map    */
	unsigned char	c_chan;		/* channel number		    */
	unsigned char	c_valid;	/* vaild channel flag		    */
};

/*
 *	IPC Descriptor Structure
 */

struct b_ipc {
	char		i_model[8];	/* model name			     */
	char		i_data[8];	/* release date 		     */
	ushort		i_release;	/* release number		     */
	ushort		i_level;	/* release level		     */
	ushort		i_age_enable;	/* aging enabled		     */
	ushort		i_v_sz;		/* VMEbus DMA block size	     */
	ushort		i_fill1;	/* reserved			     */
	ushort		i_max_buf_sz;	/* maximum buffer size		     */
	ushort		i_halfful;	/* high water mark for ready queues  */
	ushort		i_fastslc;	/* time before swap when > 1/2 full  */
	ushort		i_slowslc;	/* time before swap when < 1/2 full  */
	ushort		i_timelmt;	/* ready queues aging factor	     */
	ushort		i_heartbeat;	/* length of timer tick		     */
	unsigned long	i_pkt_cnt;	/* cumulative packet count	     */
	unsigned long	i_mtroff_dly;	/* delay till floppy drvs turned off */
};

struct b_device {
#ifdef	m88k
	unsigned short	m_addrhi;	/* IPC Address Register		*/
	unsigned short	m_addrlo;	/* IPC Address Register		*/
#else
	unsigned	m_addr;		/* IPC Address Register		*/
#endif
	unsigned char	m_am,	m_f0;	/* IPC Address Modifier Register*/
	unsigned char	m_cr,	m_f1;	/* IPC Control Register		*/
	unsigned char	m_sr,	m_f2;	/* IPC Status Register		*/
	unsigned char	m_md,	m_f3;	/* IPC Model Data Register	*/
	unsigned char	m_av,	m_f4;	/* IPC Abort Vector Register	*/
	unsigned short	m_tas;		/* IPC TAS Register		*/
};

/*
 *	struct device defines
 *
 *	m_cr
 */

#define B_BSY		0x80	/* busy bit */
#define	B_RESET		0x40	/* IPC reset signal */
#define	B_ATTN		0x20	/* attention bit */
#define	B_SFINH		0x10	/* sysfail inhibit */

/*
 *	m_tas
 */

#define	TASBIT	0x8000		/* TAS reservation bit */
#define	VCMD	0x4000		/* valid command flag */
#define	VSTAT	0x2000		/* valid status flag */
#define	CCOMP	0x1000		/* command complete */
#define	TASCMD	0x0fff		/* TAS command mask */
#define	TASMASK	0xf000		/* TAS status mask */
#define	CREATE	0x0001		/* channel create command */
#define	DELETE	0x0002		/* channel delete command */

/*
 *	struct packet defines
 */

/*
 *	p_dtype
 */

#define	D_HARD		0x0		/* hard disk */
#define	D_FLOP		0x1		/* floppy */
#define D_TAPE		0x2		/* streamer tape */
#define D_PRT		0x3		/* printer/parallel port */
#define D_SERIAL	0x4		/* serial port */
#define D_IPC		0xf		/* controller */
#define D_HTOHARD	0x10		/* hard disk to hard disk */
#define D_HTOTAPE	0x11		/* hard disk to tape */
#define D_TTOHARD	0x12		/* tape to hard disk */

/*
 *	p_cmd
 */

#define C_INIT	0x1000			/* init */
#define	C_DINIT	0x1001			/* device init */
#define	C_SINIT	0x1011			/* strategy function init */
#define	C_CINIT	0x1012			/* cache buffers init */

#define	C_READ	0x2000			/* read */
#define	C_READL	0x2001			/* read logical */
#define	C_READP	0x2002			/* read physical */
#define C_RIPC	0x2003			/* read ipc descriptor */
#define C_READA	0x2004			/* read address */
#define C_READT	0x2005			/* read track */
#define	C_STAT	0x2006			/* read status */
#define	C_READS	0x2008			/* read strategy */
#define	C_REOF	0x2031			/* read to eof */

#define	C_WRT	0x3000			/* write */
#define	C_WRTL	0x3001			/* write logical */
#define	C_WRTP	0x3002			/* write physical */
#define	C_WIPC	0x3003			/* write ipc descriptor */ 
#define	C_WRTS	0x3008			/* write strategy */
#define C_VRFY	0x3011			/* verify data */
#define	C_WEOF	0x3031			/* write eof */

#define C_IOCTL	0x4000			/* ioctl */
#define C_SEEK	0x4001			/* seek */
#define C_RESTR	0x4002			/* restore */
#define C_FMT	0x4011			/* format */
#define C_FMTV	0x4012			/* format verify */
#define C_VFMT	0x4013			/* verify format */
#define C_SCAN	0x4014			/* scan */
#define C_VCRC	0x4015			/* verify CRC/ECC */
#define C_FIXBS 0x4016			/* fix bad spot */
#define C_FMTT	0x4021			/* format track */
#define C_FMTD	0x4022			/* format disk */
#define	C_REW	0x4031			/* rewind */
#define	C_RET	0x4032			/* retension */
#define	C_ERASE	0x4033			/* erase */
#define	C_SKEOD	0x4034			/* seek to eod */
#define	C_TREAD	0x4035			/* terminate read */
#define	C_SELCT	0x4051			/* select mode */

#define C_XFER	0x7000			/* transfer */
#define C_XFERL	0x7001			/* transfer logical */
#define C_XFERP	0x7002			/* transfer physical */

#define C_RSTRT	0xe000			/* restart */
#define	C_RSTRU	0xe001			/* restart unit */

#define	C_ABORT	0xf000			/* abort */
#define	C_ABRTU	0xf001			/* abort unit */

/*
 *	p_cstat - device independent
 */

#define	S_CTOD		(1<<20)		/* Controller-To-Device. */
#define	S_ITOC		(1<<21)		/* IPC-To-Controller. */
#define	S_STOI		(1<<22)		/* System-To-IPC. */
#define	S_TIMEOUT	(1<<23)		/* Timeout. */
#define	S_BADUNIT	(1<<24)		/* Bad Unit. */
#define	S_BADDEV	(1<<25)		/* Bad Device. */
#define	S_BADCMD	(1<<26)		/* Bad Command. */
#define	S_ABORT		(1<<27)		/* IPC Abort. */
#define	S_ETERM		(1<<28)		/* Early Termination. */
#define	S_CPUABORT	(1<<29)		/* CPU Abort. */
#define	S_FATAL		(1<<30)		/* Fatal Error. */
#define S_ERROR		(1<<31)		/* Error */

/* Strategy routines defines */
#define	FIFO	0x0			/* FIFO strategy */
#define	SCAN	0x1			/* two way scan strategy */
#define	CSCAN	0x2			/* one way scan strategy */
#define	ELEV	0x3			/* UNIX "sawtooth" strategy */
