/*@(#)bpp.h	35.1*/
/* Well known interface point definitions */
#define	BP_IR_IP	0			/* local BPPIR interface point */
#define	BP_IPM_IP	1			/* interface point management */
#define	BP_DLD_IP	1			/* download/upload management */
#define	BP_BFM_IP	2			/* buffer management */
#define	BP_SNS_IP	3			/* system name server */
#define	BP_DIA_IP	4			/* diagnostic services */
#define	BP_FTS_IP	5			/* file transfer server */
#define	BP_ERR_IP	6			/* system error logger */
#define	BP_RSC_IP	7			/* resource management */
#define	BP_RLD_IP	8			/* relocating loader */

typedef	UBYTE	CPU_ID;			/* CPU identification */
typedef	UBYTE	POOL_ID;		/* buffer pool identification */

typedef struct bp_pool_t
{
	CPU_ID	cpu;		/* CPU number */
	POOL_ID	pool_id;	/* pool identification number */
} BM_POOL_T;

/* Default buffer pool id;  Since a static buffer is not really par
	of a pool it has a statically-defined id.
*/
#define	MPB_STATIC_POOL_ID	0x101

typedef struct bp_ip_t
{
	USHORT		cpu;		/* cpu number of id */
	USHORT		ip;		/* interface point of id */
} BP_IP_T;


typedef struct bp_syshdr_t
{
	BM_POOL_T	creator;	/* buffer creator identification */
	BM_POOL_T	manager;	/* manager of this buffer pool	 */
	CPU_ID		partnercpu;	/* cpu partner for subpool mgt.	 */
	UBYTE		buf_status;	/* misc. status/type information */
#define			BUF_IN_POOL	0x01	/* buffer is in buffer pool  */
#define			BUF_FREE	0x02	/* buffer is free (not in pool) */
#define			STATIC_BUF	0x80	/* Buffer is static area	 */
#define			SHARED_BUF	0x40	/* buffer shared by 2 CPU's	 */
	USHORT		window_id;	/* window that owns this buffer  */
	USHORT		mpb_size;		/* size of buffer (incl. header) */
	USHORT		reserved;	/* reserved */
	struct bp_syshdr_t	*bmh_link;	/* link to next buffer in pool	 */
} BP_SYSHDR_T;

#define MPB_HDR_SIZE	40		/* length of following strucure */
typedef struct bp_mpb_t
{
	BP_SYSHDR_T	sys_hdr;	/* BPP internal use area 	*/
	struct bp_mpb_t	*next;	/* pointer to next message 		*/
	BP_IP_T		dst_id,		/* destination id 				*/
				src_id;		/* origin id 					*/
	USHORT		msg_status;	/* message status 				*/
	USHORT		data_size;	/* length of data field in msg 	*/
	UBYTE		*data;		/* pointer to data 				*/
	ULONG		reserved;
} BP_MPB_T;


typedef struct bpe_t
{
	struct bpe_t *next;	/* forward link */
	union 
	{
		ULONG command;		/* command (bit 0 set) */
		BP_MPB_T *message;	/* message (bit 0 clear) */
	} m;
} BPE_T;

/* BPPIR registration table entry definition */

typedef struct reg_ent_t
{
   USHORT	ip;				/* interface point */
   USHORT	status;			/* state of this registration entry */
#define	IP_REGISTERED	0x0001	/* table entry in use */
   PTF		route_in;		/* input message routing function */
   ULONG	arg_in;			/* argument used in call to route_in() */
   PTF		route_out;		/* output message service function */
   ULONG	arg_out;		/* argument used in call to route_out() */
   BP_MPB_T	*que_head;		/* head of queued messages */
   BP_MPB_T	*que_tail;		/* tail of message queue */
   USHORT	control;		/* misc. operation and control information */
} REG_ENT_T;

#define	MSG	m.message		/* shorthand for BPE message field */
#define CMD	m.command		/* shorthand for BPE command field */

/* Download/Upload request format */
typedef struct bp_load_t
{
	BP_SYSHDR_T	sys_reserved;/* internal use area */
	struct bp_mpb_t	*next;	/* pointer to next message */
	BP_IP_T		dst_id,		/* destination id */
				src_id;		/* origin id */
	USHORT		status;		/* message status */
	USHORT		data_size;	/* length of data field in msg */
	UBYTE		*data;		/* pointer to data */
	ULONG		reserved;
	USHORT		type;		/* request type */
	UBYTE		*source_addr;	/* source address -- pointer to some 
					   			physical memory which may be the
					   			data section of this message.	*/
	UBYTE		*dest_addr;		/* destination address -- pointer to 
					   			physical memory which may be the
					   			data section of this message.	*/
	ULONG		argument;	/* Argument to execute function call */
	USHORT		length;		/* length of data to transfer in bytes */
	BP_IP_T		dest_id;	/* Used for upload requests to route
					   			data to an IP other than requestor's. */
	USHORT		load_data[1];	/* Variable length array containing
					   			load data. */
} BP_LOAD_T;


/* Upload/Download Request Definitions

#define	BP_UPLOAD_REQ	1		/* Move from memory to buffer and
					   send to dest_id. */
#define	BP_UPLOAD_RSP	2		/* Response to upload request */
#define	BP_DOWNLOAD_REQ	3		/* Move data from buffer to memory */
#define	BP_DOWNLOAD_RSP	4		/* response to download request */
#define	BP_EXECUTE_REQ	5		/* execute request */
#define BP_EXECUTE_RSP	6		/* execute response */

/*
**  Buffer Management - Initialize Pool Message Format
*/

typedef struct bm_init_pool_t
{
	BP_SYSHDR_T	sys_reserved;/* internal use area */
	struct bp_mpb_t	*next;	/* pointer to next message */
	BP_IP_T		dst_id,		/* destination id */
				src_id;		/* origin id */
	USHORT		status;		/* message status */
	USHORT		data_size;	/* length of data field in msg */
	UBYTE		*data;		/* pointer to data */
	ULONG		reserved;
	USHORT		type;			/* type of request or response  */
	BM_POOL_T	creator;		/* creator CPU and pool ID		*/
	BM_POOL_T	manager;		/* manager CPU and pool ID		*/
	ULONG		resp_code;		/* response from remote CPU		*/
	ULONG		buffer_size;	/* size of buffers in this pool	*/
	USHORT		buffer_count;	/* number of buffers in pool	*/
	BP_SYSHDR_T	*get_buf;		/* address of first buffer in 
									local manager's subpool		*/
	BP_SYSHDR_T *ret_buf;		/* address of last buffer in 
									remote creator's subpool	*/
	BYTE		*start;			/* address of start of memory 
									for local manager's subpool	*/
	BYTE		*end;			/* address of end of memory area
									for local manager's subpool */
} BM_INIT_POOL_T;

/********************************************************************
*	This section defines the function return codes, the MPB status
*	codes, and specific MPB-request type return codes for the base
*	BPP/BPPIR services.  The defines are grouped by function call
*	or MPB field to which they apply, with some codes serving multiple
*	purposes.
********************************************************************/

/* BPP function return codes */

#define	BPP_SUCCESSFUL	0			/* successful completion of request */
#define	BPP_BAD_CPU_ID	0xf001		/* invalid cpu number parameter */
#define	BPP_REQ_TIMEOUT	0xf002		/* response timeout during service req */
#define INSUFF_MEMORY	0xf003		/* no memory available for request */
#define BPP_INIT_ERR	0xf004		/* generic initialization error */

/* bp_open_channel() return codes */
/* BPP_SUCCESSFUL, BPP_REQ_TIMEOUT, BPP_BAD_CPU_ID */
#define	INVALID_CIT_PARM	0xf004	/* bad channel initialization table */
#define	CPU_INIT_FAILED		0xf005	/* non-zero return remote_cpu_init() */
#define	REMOTE_CPU_DEAD		0xf006	/* non-zero return remote_cpu_probe() */
#define	BPP_READY_FAILURE	0xf007	/* channel initialization stage failure */
#define	BPP_PIPE_FAILURE	0xf008	/* channel initialization stage failure */
#define	BPP_RUN_FAILURE		0xf009	/* channel initialization stage failure */
#define	BPP_DATA_FAILURE	0xf00a	/* channel initialization stage failure */

/* bp_close_channel() return codes */
/* BPP_SUCCESSFUL, BPP_REQ_TIMEOUT, BPP_BAD_CPU_ID */

/* bp_pwrite() and bp_pread() return codes */

/* BPP_SUCCESSFUL, BPP_BAD_CPU_ID */
#define	BPP_NO_PIPE		0xf010		/* no pipe exsits to specified cpu */
#define	BPP_PIPE_EMPTY	0xf011		/* no messages on pipe to read */

/* bp_get_pipe_status() return codes and definitions */
#define	BP_CPU_INDICATOR	0x80000000	/* remote cpu request flag */

/* pipe status return values */
#define	ALIVE		0x00000001		/* pipe is active */
#define	DEAD		0x00000002		/* remote cpu not servicing pipe */
#define	RUNNING		0x00000004		/* remote cpu is running */
#define	CLOSE_PENDING	0x00000008	/* close channel command issued */
#define	CLOSED		0x00000000		/* channel is closed */
#define	INTERRUPT	0x00000100		/* cpu request interrupt on next message */
#define	PENDING_BUF	0x00000200		/* cpu requests interrupt on next release */

/* bp_get_memory() and bp_free_memory() return codes */
/* INSUFF_MEMORY */
#define	INVALID_MEM_PARM	0xf020	/* bad request parameters */

/* bp_get_bpe() and bp_rel_bpe() return codes */

#define	BPP_NO_BPE		0xf030		/* no BPE's available */

/* bp_open_ip() return codes */
#define	BPP_IP_IN_USE		0xf040		/* IP already registered */
#define BPP_INVALID_IP		0xf041		/* out of range IP specified */

/* bp_send(), bp_receive() return codes */
#define	BPP_NO_MESSAGE		0xf050		/* no message for requested IP */
#define	BPP_IP_NOT_OPEN		0xf052		/* requested IP is not open */

/***********************************************************************/
/* BP_SYSHDR_T.status codes */
#define	BP_DST_NOT_REGISTERED	0xff00
