/*@(#)ccb.h	35.1*/
/* ccb.h
**  
** 	This module defines the Channel Control Block (CCB) structure
**	which is used to initiate and manage buffered pipes between
**	two processors in a VME system.  The CCB is originally created
**	by the "pipe initiator" in an area of shared memory which is
**	addressable by both processors.  The VMEbus address of the CCB
**	is passed to the "pipe responder" during channel initialization.
*/

typedef struct	pipe_ctrl_tbl	/* Pipe Control Table	*/
{
	UBYTE   bpp_type;  	/* BPP driver type (I/R/X)     	*/
#		define	INITIATOR 'I'	/* Driver only initiates pipes	*/
#		define	RESPONDER 'R'	/* Driver only responds		*/
#		define 	INIT_RESP 'X'	/* Driver can both initiate and	*/
					/*  respond 			*/
	UBYTE	bpp_mode;	/* BPP operating mode (I/R)	*/
	UBYTE   bpp_version[8];	/* BPP driver version number 	*/
	USHORT	cpu_id;		/* CPU number of this processor */
	UBYTE 	cpu_type;	/* Type of this processor	*/
	UBYTE	irq_mode;     	/* Type of interrupt mechanism  */
				/*  to use to interrupt this 	*/
				/*  processor			*/
#		define	POLLED	0	/* No interrupt needed - polled	*/
#		define  VECTOR	1	/* Assert VME_IRQ and status/id	*/
#		define	ADDRESS	2	/* Write to system bus address	*/
	UBYTE	irq_level;		/* Interrupt Request Level 	*/
	UBYTE	sts_id_length;	/* 1, 2, or 4 byte status/id	*/
	ULONG	sts_id_value;	/* 1 - 4 byte status/id value	*/
	ULONG	irq_address;	/* System bus address to write  */
							/*  to cause an interrupt	*/
	BPE_T 	*get_bpe;	/* BPE pool head pointer 	*/
	BPE_T 	*ret_bpe;	/* BPE pool tail pointer 	*/
	BPE_T 	*bpp_read;	/* Buffered Pipe head pointer 	*/
	BPE_T 	*bpp_write;	/* Buffered Pipe tail pointer 	*/
	BPE_T 	*first_bpe;	/* 1st BPE originally created  	*/
				/*  for this pipe		*/
	BPE_T 	*last_bpe;	/* Last BPE created for pipe    */
	USHORT	num_bpes;	/* Number of BPE_T's originally	*/
				/*  created for this pipe	*/
	ULONG	pipe_status;	/* Supplemental BPP status info	*/
	PTF		rmt_intr;	/* pointer to a board-specific interrupt
							function.  The function is stored here
							by the open channel function to provide
							efficient means to interrupt the remote
							cpu.  The caller of open channel can
							provide such a function in the CIT or
							the default function will be installed. */
	ULONG	rmt_arg;	/* Argument to be used in calling rmt_intr */
} PCT_T;

typedef	struct	ccb	/* Channel Control Block Structure */
{
	PCT_T	prt;	/* Pipe Responder Table */
	PCT_T	pit;	/* Pipe Initiator Table */
} CCB_T;

