/*@(#)bpp_local.h	35.1*/
/**********************************************************************
**
**  INCLUDE NAME:	bpp_local.h
**
**  DESCRIPTION:	This file defines board/implementation-specific
**					parameters for BPP compiled for the MVME147.
**
**  NOTES:		
**			
**  UPDATE LOG:
**  Date		Name		V/R	Comments
**  --------	-----------	---	-----------------------------
**	12/22/87	jeh			1.0	Initial coding.
**********************************************************************/

/* Define the maximum number of BPP channels for this board.  The number
	of pipe control structure pointers allocated for local and remote
	will be determined by this value.
*/
#define	BP_MAX_CHAN		8

/* Define the maximum number of MPB pools that may be managed by this
	cpu.  This number should always be greater than one.  At least one
	is required for the local environment and at least one for each
	channel defined.
*/
#define BP_MAX_POOLS	8

/* Define the number of windows.  This value determines the number of 
	window control structures.  The number need depends upon application
	usage.
*/
#define	BP_MAX_WINDOWS	32

/* Define the maximum interface point that can be registered */
#define	BP_MAX_REGISTERED	32

/* Define the number of request pend structures. */
#define BP_MAX_PEND	32

/* Define parameters of the local default MPB pool */
#define	BP_DEF_POOL_ID		1
#define	BP_DEF_MPB_SIZE		1600
#define	BP_DEF_LOCAL_MPBS	64

/* Macros are used for services that depend upon the local hardware
	environment in order to make the source code board independent.

/* Interrupt control macros */

#define DISABLE(pri, lev)	pri = splx(lev)
#define	ENABLE(pri)	splx(pri)

/* Address mapping macros
	Some boards impose a different address mapping for local and remote
	access of shared memory.  Macros are used since some boards have the
	same address view, and thus require no mapping; in these cases, null
	macros may be used.  All addresses sent to other processors must be
	VMEbus address view.  The VME_ADDRESS macro translates from local to
	VMEbus view.  Addresses received from other processors, on the other
	hand, are in VMEbus view and must be translated for local use.  The
	macro LOCAL_ADDRESS performs this translation.  Note that the need
	for mapping is potentially specific to a pool;  if pools were created
	in different memory or by different processors they may have different
	requirements. The macros must then make use of the mapping function
	fields provided in the PCB to perform the mapping.
*/
#define	VME_ADDRESS(addr)	addr		/* no mapping required */
#define	LOCAL_ADDRESS(addr)	addr		/* no mapping required */

/* Logging macro */
#ifndef LOG			/* log may be included with ci driver stuff */
extern ULONG	log_mask;

#define LOW	0x1
#define MED	0x2
#define	HIGH	0x3

#define	LOG_LOW		if(((ULONG)(LOW << SHIFT) & log_mask) == (ULONG)(LOW << SHIFT)) printf
#define LOG_MED		if(((ULONG)(MED << SHIFT) & log_mask) == (ULONG)(MED << SHIFT)) printf
#define LOG_HIGH	if(((ULONG)(HIGH << SHIFT) & log_mask) == (ULONG)(HIGH << SHIFT)) printf

#define	LOG	LOG_HIGH	/* Interim macro while layer source code   */
				/* upgrades to specific Macros above	   */
#endif

/* Definitions for c37x channel initialization kludge */
#define VME147_CPU_NO	8	/* cpu number of the 147 under new BPP */
#define VME147			7	/* type of cpu */

/* ci driver cpu number definition -- */
#define CPU_NO		1

#define	BP_LOC_EPB_ID		1
#define	BP_RMT_EPB_ID		2
#define BP_LOC_BUF_ID		3
#define BP_RMT_BUF_ID		4

/* Local interface point definitions */
#define MAC_IP		1		/* MAC sublayer interface point */
#define LLC_IP		2		/* LLC layer interface point */
#define SMA_IP		14		/* SMA layer interface point */
