/*@(#)ci_const.h	35.2*/
/*IH*********************** INCLUDE HEADER ****************************
**
**  INCLUDE NAME:	ci_const.h
**
**  DESCRIPTION:	This file contains defines of various constants
**			and macros used in the Common Environment
**			driver.
**
**  NOTES:
**
**  UPDATE LOG:
**  Date	Name		V/R	Comments
**  --------	-----------   	---	-----------------------------
**  04/01/86	Dana Holgate	1.1	Initial Coding.
**  08/28/86	Dana Holgate	1.2	Re-organized includes
**  12/16/86	Martin Dorr	1.3	Changed buffer pool sizes
**  05/20/87	E. Skinner	1.4	Added "BPP_HWRESET" code.
**  06/09/87	R. Robinson	1.5	Changed names for generic 37x.
**  11/04/87	F. Christiansen		(see SCCS comments)
**
**********************************************************************/

/*
**	General defines for data types.
*/

#define	BOOLEAN	unsigned char	/* Two valued                */

#define	DEFAULT	int		/* Default size		     */

#define	BYTE	char		/* Signed byte (8 bits)	     */
#define	UBYTE	unsigned char	/* Unsigned byte	     */

#define	SHORT	short		/* Signed word (16 bits)     */
#define	USHORT	unsigned short	/* Unsigned word	     */

#define	LONG	long		/* Signed long (32 bits)     */
#define	ULONG	unsigned long	/* Unsigned long	     */

#define	VOID	void		/* Void function return	     */

#define REG	register	/* Use a register	     */
/*
**	Channel table defines.
*/

#define	CI_VER		"I01.01.00"	/* Version stamp */
#define	CI_VER_SZE	sizeof(CI_VER)	/* Version stamp size */

#define CPU_NO		1		/* Host CPU number */
#define CPU_OFFSET	2		/* CPU offset number */

#define NUM_REG_ENTRIES	50		/* Registration table size */
#define NUM_CPU_ENTRIES	8		/* CPU table size */
#define CH_TBL_SZ	sizeof(CH_TBL)	/* Size of channel table */
#define N_BPES		30		/* Number of BPEs per side */
#define N_EPBS		30		/* Number of EPBs per side */
#define MIN_R_BUFS	12		/* Minimum number of remote BUFs */
#define MAX_R_BUFS	24		/* Maximum number of remote BUFs */

#define N_R_BUFS	((MIN_R_BUFS + MAX_R_BUFS) / 2)	/* Number of remote BUFs */
#define N_L_BUFS	(MAX_R_BUFS + 2 * N_EPBS)	/* Number of local BUFs */

/*
**	Memory pool size defines.
*/

#define BPE_POOL_SZ	((2 * N_BPES) * sizeof(BPE))
#define EPB_POOL_SZ	((2 * N_EPBS) * MAX_CEEPB_SIZE)
#define BUF_POOL_SZ	((N_L_BUFS + N_R_BUFS) * MAX_BUF_SIZE)

#define MEM_POOL_SZ	(CH_TBL_SZ + BPE_POOL_SZ + EPB_POOL_SZ + BUF_POOL_SZ)

/*
**	Macro definitions.
*/

	/*
	**	Get and set the VME130's control register 5 to
	**	16 bit data bus.
	*/

        /* 
        **      Register offset definitions for boards
        */

#define MVME372_OFFSET  0x1FBFC
#define MVME374_OFFSET  0xfff7c
#define MVME147_OFFSET  147                          /* not used by bppv2 */

	/*
	**	Write to BPP's data register.
	*/

#define DROFFSET        0
#define BPPDR(i,d)	(*(ULONG *)(ci_cpu[i].boardadr + ci_cpu[i].regbase + DROFFSET) = (d))

	/*
	**	Write to BPP's command register.
	*/

#define CROFFSET        5
#define BPPCR(i,c)	(*(UBYTE *)(ci_cpu[i].boardadr + ci_cpu[i].regbase + CROFFSET) = (c))

	/*
	**	Read BPP's status register.
	*/

#define SROFFSET        7
#define BPPSR(i)	(*(UBYTE *)(ci_cpu[i].boardadr + ci_cpu[i].regbase + SROFFSET))

	/*
	**	Get owner's PID.
	*/
	
#define OWNER(pid)	((pid) >> 16)

	/*
	**	Get remote's PID.
	*/

#define REMOTE(pid)	((pid) & 0xFFFF)

	/*
	**	Set owner's PID.
	*/

#define PUT_PID(owner,remote)	(((owner) << 16) + (remote))

	/*
	**	Change an interrupt level into splx() form.
	*/

#define SPLEV(level)	( 0x2000 | level << 8 )

/*
**	BPP command defines.
*/

#define BPP_INIT	0
#define BPP_PIPE	1
#define BPP_RUN		2
#define BPP_RESET	3
#define BPP_BPE		4
#define BPP_HWRESET	5

/*
**	BPP status defines.
*/

#define NO_ERROR	0	/* Good */
#define NO_PIPE		-1	/* Pipe doesn't exist */
#define NO_BPE		-2	/* No shared BPE available */

/*
**	Off CPU board memory starting address.
*/

/*
**	Function return values.
*/

#define	FAILURE		0xFF		/* Function failure return   */
#define	SUCCESS		0x00		/* Function success return   */

#define	TRUE		(1)		/* Function TRUE value	     */
#define	FALSE		(0)		/* Function FALSE value	     */

#define	NULL		0		/* Null value		     */

/*
**	Miscellaneous definitions.
*/

#define BPP_WAIT_TIME	0x30000 	/* Response wait time */

#define TERMINATOR	1		/* EPB pool terminator */
#define MEMBER		0		/* EPB pool member */

#define EMPTY		0xFFFF		/* Empty registration table entry */
