/*@(#)ci_driver.h	35.2*/
/*IH*********************** INCLUDE HEADER ****************************
**
**  INCLUDE NAME:	ci_driver.h
**
**  DESCRIPTION:	Contains definitions of all general defines,
**			data structures, and constant defines needed
**			by applications using the Common Environment
**			driver.
**
**  NOTES:		The following structure defintions are
**			contained in this file;
**
**			BUF
**			EPB
**			CE_EPB
**
**  UPDATE LOG:
**  Date	Name		V/R	Comments
**  --------	-----------   	---	-----------------------------
**  08/27/86	Dana Holgate	1.1	Initial Coding.
**  04/20/87	R. Robinson	1.2	Changed names for generic 37x.
**  06/01/87	J. Richey	1.2	Added signal support.
**  01/19/88	J. Holbrook	1.5	Added cpu entry for VME147
**
**********************************************************************/

/*
**	Common Environment constant defines.
*/

	/*
	**	EPB and BUF size defines.
	*/

#define MAX_CEEPB_SIZE	386	/* Maximum size of EPB including user data */
#define BUF_HDR_SIZE	8	/* Size of header fields in BUF */
#define MAX_CEDATA_SIZE 1518	/* Maximum size of data in BUF */

	/*
	**	Maximum size of BUF including header
	*/

#define MAX_BUF_SIZE	(MAX_CEDATA_SIZE + BUF_HDR_SIZE)

	/*
	**	IOCTL command argument defines.
	*/

#define RESET		1	/* Recreate the Buffered Pipe */
#define WAIT		2	/* Do wait reads */
#define NOWAIT		3	/* Do nowait reads */
#define DEBUG_ON	4	/* Turn debug printing on */
#define DEBUG_OFF	5	/* Turn debug printing off */
#define ADDR		6	/* Request shared memory address */
#define ARM_SIGNAL	7	/* Arm sleep on read */
#define DIS_ARM_SIGNAL	8	/* Disarm sleep on read */
#define CONFIG		9	/* Request CPU description table */

	/*
	**	"Sleep on read" signal support.
	*/

#define BLOCK		0x01	/* Block mode flag indicator */
#define ST_SIGNAL	0x02	/* Signal mode flag indicator */
#define ST_DATA		0x04	/* Maybe data before signal */

	/*
	**	Common Environment EPB type defines.
	*/

		/*
		**	Registration Types
		*/

#define CE_REGISTRATION_REQUEST		0xFF00
#define CE_REGISTRATION_RESPONSE	0xFF01
#define CE_DEREGISTRATION_REQUEST	0xFF02

		/*
		**	Bootloader Types
		*/

#define CE_LOAD_REQUEST			0xFF03
#define CE_LOAD_DATA			0xFF04
#define CE_LOAD_RESPONSE		0xFF05
#define CE_EXECUTE_REQUEST		0xFF06
#define CE_PROTECT_REQUEST		0xFF07
#define CE_PROTECT_RESPONSE		0xFF08
#define CE_UNPROTECT_REQUEST		0xFF09
#define CE_UNPROTECT_RESPONSE		0xFF0A

		/*
		**	Buffer Transfer Types
		*/

#define CE_GETEPB_REQUEST		0xFF0B	/* not used by driver */
#define CE_GETEPB_RESPONSE		0xFF0C	/* not used by driver */
#define CE_RETEPB_REQUEST		0xFF0D	/* not used by driver */
#define CE_GETBUF_REQUEST		0xFF0E
#define CE_GETBUF_RESPONSE		0xFF0F
#define CE_RETBUF_REQUEST		0xFF10

		/*
		**	Message Channel Types
		*/

#define CE_PIPE_INIT_REQUEST		0xFF11
#define CE_BPE_INDICATION		0xFF12

	/*
	**	Common Environment EPB status defines.
	*/

#ifndef NO_ERROR
#define NO_ERROR			0
#endif
#define CE_STATUS			0xFFF0
#define DST_NOT_REGISTERED		0xFFF0
#define UNDELIVERABLE			0xFFF1
#define REG_TABLE_FULL			0xFFF2
#define INVALID_IP			0xFFF3
#define IP_IN_USE			0xFFF4
#define NO_PIPE_TO_DST			0xFFF5
#define INVALID_CE_REQUEST		0xFFF6
#define BL_BUSY				0xFFF7
#define BAD_MEM_ACCESS			0xFFF8
#define MEM_PROTECTED			0xFFF9

	/*
	**	Miscellaneous Common Environment defines.
	*/

#define CE_IP		0	/* Common Interfaces reserved IP */
#define LOCAL_CPU	0	/* Local CPU identifier */

	/*
	**	Common Environment macro defines.
	*/

#define PUT_ID(cpu,ip)	(((cpu) << 16) + (ip))
#define GET_CPU(id)	((id) >> 16)
#define GET_IP(id)	((id) & 0xFFFF)

/*
**	Common Environment structure and type definitions.
*/

	/*
	**	Common Environment CONFIG ioctl(2) type definition.
	*/

typedef struct cpu_descr
{
	unsigned short	cpu_id;		/* user level CPU ID */
	unsigned short	cpu_type;	/* literally 372 or 374 */
	unsigned char	cpu_present;	/* bprobe() found it */
	unsigned char	cpu_running;	/* host & board are communicating */
} CPU_TAB;	
	
	/*
	**	Data buffer structure.
	*/

typedef struct buf
{
	struct	 buf	*next;			/* Forward link to next BUF */
	unsigned long	length;			/* BUF data length */
	unsigned char	data[MAX_CEDATA_SIZE];	/* BUF data */
} BUF;

	/*
	**	Event parameter block (EPB) structure.
	*/

typedef struct epb
{
	struct	 epb	*next;		/* Forward link to next EPB */
	unsigned long	mem_id;		/* EPB/CPU identifier */
	unsigned long	src_id;		/* Source interface point */
	unsigned long	dst_id;		/* Destination interface point */
	unsigned short	epb_size;	/* EPB size */
	unsigned short	type;		/* EPB type */
	unsigned short	status;		/* EPB status */
		 BUF	*buf;		/* Data buffer pointer */
	unsigned long	user_def;	/* User definable field */
} EPB;

	/*
	**	Common Environment EPB structure.
	*/

typedef struct ce_epb
{
		 EPB	hdr;		/* EPB Header */

		 /*
		 **	The following field is used for registration.
		 */

	unsigned short	qid;		/* Registering task queue ID */

		 /*
		 **	The following fields are used for message
		 **	channel creation.
		 */

	unsigned short	sh_epb_size;	/* Size of shared EPBs */
	unsigned short	sh_buf_size;	/* Size of shared BUFs */
	unsigned short	minimum_bufs;	/* Mimimum number of BUFs */
	unsigned short	maximum_bufs;	/* Maximum number of BUFs */
	unsigned long	epbbuf_count;	/* Number of transfered BUFs */
		 EPB	*epb_pool_head;	/* Shared EPB pool head */
		 EPB	*epb_pool_tail;	/* Shared EPB pool tail */

		 /*
		 **	The following fields are used by the
		 **	bootloader.
		 */

	unsigned long	addr;		/* Execution/starting data address */
	unsigned long	length;		/* Length of data in all BUFs */
		 char	file[32];	/* Boot file name */
	unsigned char	eot;		/* End of transfer flag */
	unsigned long	exec_arg;	/* Bootloader exec. function Argument */
} CE_EPB;
