/*@(#)ci_struct.h	35.2*/
/*IH*********************** INCLUDE HEADER ****************************
**
**  INCLUDE NAME:	ci_struct.h
**
**  DESCRIPTION:	Contains definitions of all data structures
**			used only internally by the ci_driver.c module.
**
**  NOTES:		The following structure defintions are
**			contained in this file;
**
**			MAX_EPB
**			BPE
**			SH_RES
**			CH_TBL
**			REG_ENT
**			CPU_ENT
**
**  UPDATE LOG:
**  Date	Name		V/R	Comments
**  --------	-----------   	---	-----------------------------
**  03/20/86	Dana Holgate	1.1	Initial Coding.
**  04/20/87	R. Robinson	1.2	Changed names for generic 37x.
**
**********************************************************************/

/*
**	The following structure definition generates the maximum
**	EPB size.
*/

typedef struct max_epb
{
		EPB	hdr;
		BYTE	user_data[MAX_CEEPB_SIZE - sizeof(EPB)];
} MAX_EPB;

/*
**	The following structure definitions is the Buffer Pipe
**	Envelope (BPE).
**
**	Note:	The destination and source interface points consist
**		of the CPU number (high word) and the IP (low word).
**		Bit 0 of the flags indicates whether the BPE is the
**		terminating BPE (set) or a member of a list (clear).
**		
*/

typedef struct bp_env
{
	struct	bp_env	*next;		/* Forward link */

	union 
	{
		ULONG	command;	/* Command (bit 0 set) */
		EPB	*message;	/* Message (bit 0 clear) */
	} mc;

		ULONG	dst_id;		/* Destination Interface Point */
		ULONG	src_id;		/* Source Interface Point */

		USHORT	flags;		/* BPE flags */
} BPE;

#define msg mc.message			/* BPE message member */
#define cmd mc.command			/* BPE command member */

/*
**	The following structure definition is the resource table.
*/

typedef struct sh_res
{
		EPB	*epb_head;	/* Head of EPB pool */
		EPB	*epb_tail;	/* Tail of EPB pool */
		BUF	*buf_head;	/* Head of BUF pool */
		BUF	*buf_tail;	/* Tail of BUF pool */
} SH_RES;

/*
**	The following structure definition is the channel table.
*/

typedef struct ch_tbl
{
		BYTE	version[CI_VER_SZE];	/* Version stamp */
		USHORT	local_cpu;		/* Local CPU's number */
		USHORT	remote_cpu;		/* Remote CPU's number */
		UBYTE	intr_level;		/* Interrupt level */
		UBYTE	intr_vector;		/* Interrupt vector */
		BPE	*get_bpe;		/* BPE pool head pointer */
		BPE	*ret_bpe;		/* BPE pool tail pointer */
		BPE	*bpp_read;		/* Buffered Pipe head pointer */
		BPE	*bpp_write;		/* Buffered Pipe tail pointer */
} CH_TBL;

/*
**	The following structure definition is a registration table
**	entry.
*/

typedef struct reg_ent
{
		USHORT	process_id;	/* Process ID of user process */
		UBYTE	block;		/* Block/Signal read indicator flags */
		BOOLEAN	reg_notified;	/* Initial notification indicator */
	struct  proc	*proc;		/* Pointer to process slot */
		EPB	*rq_head;	/* Read queue head */
		EPB	*rq_tail;	/* Read queue tail */
		VOID	(*rx_int)();	/* Receive handler for this reg'n */
} REG_ENT;

/*
**	The following structure definition is a CPU table entry.
*/

typedef struct cpu_ent
{
	REG_ENT	rt[NUM_REG_ENTRIES];	/* Registration table */
	CE_EPB	*res_epb;		/* Pointer to shared resource EPB */
	SH_RES	rr;			/* Remote shared resource table */
	SH_RES	lr;			/* Local  shared resource table */
	CH_TBL	*rc;			/* Pointer to remote channel table */
	CH_TBL	lc;			/* Local channel table */
	BOOLEAN	no_com;			/* Indicates no communication, at all */
	BOOLEAN	no_ipc;			/* Indicates no communication to IPC */
	ULONG   regbase;                /* offset from boardadr to registers */
	ULONG	boardadr;		/* Board base address */
	ULONG	memadr;			/* Physical memory address */
} CPU_ENT;
