/*	@(#)dk.h	35.3	*/
/*
 *	Copyright (c) 1985, 1986, 1987	Motorola, Inc.  
 *		All Rights Reserved.
 */
/*
 *	dk.h: Descriptions of Motorola Disk Format Standards
 *
 *
 *	Standard Disk Layout: optional fields can be zero length.
 *		In that case, the following field will immediately follow
 *		the previous field.
 *
 *	Size 	Contents
 *	----	--------
 *	  1	Volume ID Block
 *	  1	Disk Configuration Block
 *	 BH	Bad Track Hash Table (optional)
 *	 BT	Bad Track Table (optional)
 *	 DD	Diagnostic Test Area Directory (optional)
 *	 BO	Bootloader (up to 16 Kbytes)
 *	 FI	Cylinder Fill
 *	 DT	Diagnostic Tracks (optional)
 *	  N	Usable Disk Area (including alternate tracks)
 *	 DT	Diagnostic Tracks (optional)
 *
 *	Where:
 *	------
 *	BH	= if included then 1 else 0.
 *	BT	= if included then round(((tpc*maxbad+1)*4)/bps) else 0.
 *	DD	= if included then 1 else 0.
 *	BO	= 16K/bps.
 *	FI	= round((3+DD+BT+BO)/spc)*spc-(3+DD+BT+BO).
 *	DT	= if included then spc else 0.
 *	 N	= (spc*cyls)-(2+DD+BT+BO+FI+2*DT).
 *	 A	= maxbad*tpc*spt
 *
 *	and:
 *	----
 *	bps	= bytes per sector (>= 256)
 *	spc	= sectors per cylinder (== tpc*spt)
 *	spt	= sectors per track
 *	tpc	= tracks per cylinder
 *	cyls	= cylinder per disk
 *	maxbad	= maximum number of (software correctable) bad tracks
 *			allowed per surface (recording head).
 */

/*
 *	Volume ID structure: on first sector of disk
 */

struct	volumeid {
	unsigned	vid_vol;	/* Volume ID */
	unsigned char	vid_f0[16];	/* Zero fill */
	unsigned	vid_oss;	/* Bootloader starting sector */
	unsigned short	vid_osl;	/* Bootloader sector length */
#ifndef m88k
	unsigned	vid_ose;	/* Bootloader execution address */
	unsigned	vid_osa;	/* Bootloader load address */
#else
	unsigned short	vid_ose_u;	/* Bootloader execution address */
	unsigned short	vid_ose_l;	/* Bootloader execution address */
	unsigned short	vid_osa_u;	/* Bootloader load address */
	unsigned short	vid_osa_l;	/* Bootloader load address */
#endif
	unsigned char	vid_f1[4];	/* Zero fill */
	unsigned char	vid_vd[20];	/* ASCII Volume description */
	unsigned char	vid_f2[6];	/* Zero fill */
	unsigned char	vid_dtp[64];	/* Diagnostic test pattern */
	unsigned	vid_dta;	/* Diagnostic test area sector */
	unsigned char	vid_f3[12];	/* Zero fill */
	unsigned	vid_cas;	/* Configuration area starting sector */
	unsigned char	vid_cal;	/* Configuration area sector length */
	unsigned char	vid_f4[1];	/* Zero fill */
	unsigned short	vid_btl;	/* Number of bad tracks */
	unsigned short	vid_typ;	/* Disk type */
	unsigned char	vid_strat;	/* s/w = -1,  h/w = 1  */
	unsigned char	vid_f5[1];	/* Zero fill */
	unsigned	vid_rfs;	/* root file sys. offset in K bytes */
	unsigned char	vid_f6[88];	/* Zero fill */
	unsigned char	vid_mac[8];	/* ASCII string "MOTOROLA" */
};

/*
 *	config block: second sector on disk
 */

struct dkconfig {
	unsigned char	ios_f0[4];	/* Zero fill */
	unsigned short	ios_atm;	/* Attributes Mask */
	unsigned char	ios_f1[2];	/* Zero fill */
	unsigned short	ios_atw;	/* Attributes Word */
	unsigned short	ios_rec;	/* Logical sector size */
	unsigned	ios_rsz;	/* Number of logical sectors */
	unsigned char	ios_f2[8];	/* Zero fill */
	unsigned char	ios_spt;	/* Sectors per track */
	unsigned char	ios_hds;	/* Heads */
	unsigned short	ios_trk;	/* Number of cylinders */
	unsigned char	ios_ilv;	/* Interleave factor of media */
	unsigned char   ios_sof;	/* Spiral offset of media */
	unsigned short	ios_psm;	/* Physical sector size */
	unsigned short	ios_shn;	/* Starting head number of drive */
	unsigned char	ios_f3[2];	/* Zero fill */
	unsigned short	ios_pcom;	/* Precompensation cylinder */
	unsigned char	ios_f4[1];	/* Zero fill */
	unsigned char	ios_sr;		/* Stepping rate */
	unsigned char	ios_f5[2];	/* Zero fill */
	unsigned short  ios_ecc;	/* ECC data burst length */
	unsigned short  ios_eatm;	/* Extended attributes mask */
	unsigned short  ios_eprm;	/* Extended parameters mask */
	unsigned short  ios_eatw;	/* Extended attributes word */
	unsigned char	ios_gpb1;	/* Gap byte 1 */
	unsigned char	ios_gpb2;	/* Gap byte 2 */
	unsigned char	ios_gpb3;	/* Gap byte 3 */
	unsigned char	ios_gpb4;	/* Gap byte 4 */
	unsigned char	ios_ssc;	/* Sector slip count */
	unsigned char	ios_f6[1];	/* Zero fill */
	unsigned short	ios_alts;	/* Number of alternate tracks */
	unsigned char	ios_f7[70];	/* Zero fill */
	unsigned long	ios_slcblk;	/* Slice table block number */
	unsigned short	ios_slccnt;	/* Number of entries in slice table */
#ifdef m88k
	unsigned short	ios_catr_u;	/* Controller specific attributes */
	unsigned short	ios_catr_l;	/* Controller specific attributes */
#else
	unsigned long	ios_catr;	/* Controller specific attributes */
#endif
	unsigned short	ios_uss;	/* Unformatted sector size in bytes */
	unsigned char	ios_f8[116];	/* Zero fill */
};

/* Bit definitions for ios_atw */

#define ATWTK0	0x2000	/* Track 0 desity (1=same destity, 0=single) */
#define ATWASA	0x1000	/* Assign alternate capabilty */
#define ATWDEF	0x0800	/* Format with defect list capability */
#define	ATWAS	0x0400	/* Alternate Sectors? 1=yes, 0=no */
#define	ATWFS	0x0200	/* Floppy size (0=5 1/4, 1=8) */
#define	ATWPC	0x0100	/* 0=pre-write precomp, 1=post-read precomp */
#define	ATWSK	0x0080	/* Seek after head change? 0=no, 1=yes */
#define	ATWDD	0X0040	/* Track Density of Drive (0=48tpi, 1=96tpi) */
#define	ATWEN	0X0020	/* Drive encoding capability (0=FM, 1=MFM) */
#define	ATWDT	0X0010	/* 0=floppy, 1=hard */
#define	ATWSN	0X0008	/* Sector Numbering (0=Motorola, 1=IBM) */
#define	ATWDS	0x0004	/* No. of sides (1=two, 0=one) */
#define	ATWTD	0x0002	/* Track Density of diskette (0=single, 1=double) */
#define	ATWMF	0x0001	/* Data Density of medium (0=48tpi, 1=96tpi) */

/*
 *	struct vid overlays (for use by ddefs(1M) and dinit(1M))
 */

#define	vid_maxbad	vid_btl		/* maxbad */
#define	vid_rsize	vid_cas		/* root fs size */
#define	vid_usize	vid_vol		/* /usr fs size */
#ifndef m88k
#define	vid_uslice	vid_osa		/* /usr fs slice */
#else
#define	vid_u_uslice	vid_osa_u	/* /usr fs slice */
#define	vid_l_uslice	vid_osa_l	/* /usr fs slice */
#endif
#define	vid_ssize	vid_oss		/* swap size */
#define	vid_sslice	vid_osl		/* swap slice */
#ifndef m88k
#define	vid_reserved	vid_ose		/* end-of-disk reserved blocks */
#else
#define	vid_u_reserved	vid_ose_u	/* end-of-disk reserved blocks */
#define	vid_l_reserved	vid_ose_l	/* end-of-disk reserved blocks */
#endif

/*
 *	combined volumeid and dkconfig block
 */

struct dkblk0 {
	struct volumeid dk_vid;
	struct dkconfig dk_ios;
};

/*
 *	diagnostic track support
 */

struct	diagent {
	unsigned	d_track1;	/* Sector number of first track */
	unsigned short	d_len1;		/* Length of first track */
	unsigned	d_track2;	/* Sector number of second track */
	unsigned short	d_len2;		/* Length of second track */
};

/*
 *	bad track support definitions
 */

#define	HASHSIZE	256	/* size of hash table */

/*
 *	struct badent used for bad track binary search tree
 */

struct badent {
	unsigned short b_bad;		/* bad track */
	unsigned short b_alt;		/* bad track alternate */
};

struct	bentry {
	unsigned short	b_stat;			/* status */
	unsigned short	b_tsize;		/* tree size */
	unsigned char	*b_hash;		/* hash table pointer */
	struct	badent	*b_tree;		/* tree pointer */
};

struct	pent {	/* must be kept at a size that is a power of 2 */
	struct pent	*nxtblk;
	unsigned 	blksize;
};

/*
 *	Slice Table Structure
 */

struct slice {
	unsigned long	nblocks;		/* slice size (in blocks) */
	unsigned long	blkoff;			/* block offset of slice */
};

/*
 *	Common ioctl(2) commands
 */

#define	DKIOCTL		('D'<<8)

/*
 *	ioctl types
 */

#define	DKGET		(DKIOCTL|0x00)	/* get information */
#define	DKSET		(DKIOCTL|0x01)	/* set information */
#define	DKCMD		(DKIOCTL|0x80)	/* general command */

/*
 *	information types
 */

#define	DKSLC		(DKIOCTL|0x00)	/* slice table information */
#define	DKVID		(DKIOCTL|0x02)	/* volume id information */
#define	DKCFG		(DKIOCTL|0x04)	/* config block information */

/*
 *	get/set commands
 */

#define	DKGETSLC	(DKGET|DKSLC)	/* get slice table */
#define	DKSETSLC	(DKSET|DKSLC)	/* set slice table */

#define	DKGETVID	(DKGET|DKVID)	/* get volumeid information */
#define	DKSETVID	(DKSET|DKVID)	/* set volumeid information */

#define	DKGETCFG	(DKGET|DKCFG)	/* get dkconfig information */
#define	DKSETCFG	(DKSET|DKCFG)	/* set dkconfig information */

#define	DKGETINFO	(DKGET|DKVID|DKCFG)	/* get vol-id/dkconfig info. */
#define	DKSETINFO	(DKSET|DKVID|DKCFG)	/* set vol-id/dkconfig info. */

/*
 *	general commands
 */

#define	DKFORMAT	(DKIOCTL|DKCMD|0x01)	/* format tracks */
#define DKADDBADSPOT	(DKIOCTL|DKCMD|0x02)	/* add to bad spot list */
#define DKFIXBADSPOT	(DKIOCTL|DKCMD|0x04)	/* fix new bad spot */

/*
 *	bad spot and alternates list
 *
 *	Hardware bad spot replacement uses cylinder, head, sector
 *	or cylinder, head, byte offset.
 */

struct dkbadlst {		/* keep #defines close by..   */
	unsigned long	dk_cyl;			/* cylinder number OR track # */
	unsigned long	dk_head;		/* head number */
	unsigned long	dk_sector;		/* sector number OR offset */
	unsigned long	dk_alt;			/* assigned alternate */
};

#define	dk_track	dk_cyl		/* keep #defines close by..   */
#define	dk_offset	dk_sector	/* keep #defines close by..   */

/*
 *	bad spot strategy
 */

#define	DKSOFTWARE	(-1)	/* Software Bad TRACK Replacement */
#define	DKPERFECT	0	/* Perfect Media */
#define	DKHARDWARE	1	/* Hardware: track replacement only */
#define	DKHARDSEC	2	/* Hardware: cylinder, head, SECtor */
#define	DKHARDSPOT	3	/* Hardware: cylinder, head, SPOT offset  */

/*
 *	format ioctl(2) structure
 */

struct dkfmt {
	unsigned long	dk_trkcnt;	/* track count */
	unsigned long	dk_trkno;	/* track number */
	unsigned long	dk_badcnt;	/* bad spot count */
	struct dkbadlst *dk_badlst;	/* bad spot list */
	unsigned long	dk_altcnt;	/* alternate tracks count */
	unsigned long	*dk_altlst;	/* alternate tracks list */
};
