

/*****************************************************************************

	Rev = 1.1		Date of 'get'   = 09:15:43 1/6/89
				Date of 'delta' = 09:15:01 1/6/89

	File = f250cmds.h

*****************************************************************************/



/**********************************************************************
 *								      *
 *		Copyright (c) Ferranti plc 1988			      *
 *								      *
 *	This is unpublished proprietory source code of Ferranti plc.  *
 *		The copyright of which vests in Ferranti plc.	      *
 **********************************************************************/


/**********************************************************************
 *								      *
 *	This file contains command structure and definitions          *
 *    							              *	
 **********************************************************************/




struct me250cmd {
	short              cmd;		  /* Command number */
	short              cntrlid;       /* Controller and l.u.n */
	short              intsreg;	  /* Instruction register */
	unsigned long      addr;   	  /* Logical block address */
	unsigned long      count;  	  /* Logical block count */
	char              *buff_addr;	  /* Address of any data */
	unsigned short     error;   	  /* Error code */
	struct me250cmd   *next_cmd;      /* pointer to next cmd */
	struct buf        *bufp;          /* Address of any buf structure  */
};


/*	Command numbers 
 *	Some commands do have other uses for fields
 *		addr
 *		count
 *	  and	buff_addr.
 */


#define READ_CMD	0x01	/* Read data from drive */
#define WRITE_CMD	0x02	/* Write data to drive */
#define FORMAT_CMD	0x03	/* Format floppy drive */
#define WRITEH_CMD	0x04	/* Write tape header to tape drive */	
#define READH_CMD	0x05	/* Read tape header from tape drive */
#define REWIND_CMD	0x06	/* Rewind tape drive */
#define WRFMKS_CMD	0x07	/* Write file marks to tape */
#define RETEN_CMD	0x08	/* Retension tape */
#define TEST_CMD	0x09	/* Test Unit Ready */
#define INIT_CMD	0x0a	/* Initialise ME250 controller */
#define SPFMKS_CMD	0x0b	/* Skip file marks on tape */
#define STPARAM_CMD	0x0c	/* Set drive parameters */
#define GTPARAM_CMD	0x0d	/* Get drive parameters */
#define COPY_CMD	0x0e	/* COPY specified disk to tape */
#define RESTORE_CMD	0x0f	/* Restore specified disk from tape */
#define SIZE_CMD	0x10	/* Find the size of a partition device */
#define REASSIGN_CMD	0x11	/* Reassign Block */

/*	Field macros for WRFMKS_CMD */
#define no_flmks 	count		/* No. of file marks to write        */
#define no_badflmks	buff_addr       /* No. of file marks not written     */	

/* 	Field macros for INIT_CMD   */
#define cpu_mon		addr		/* Address of CPU's location monitor */
#define cpu_vmeaddr	count		/* VME address of CPU's Memory       */
#define conf_data	buff_addr	/* Address of configuration data     */

/* 	Field macros for SPFMKS_CMD */
#define no_space	count           /* No. of file marks to skip         */
#define no_badsp	buff_addr	/* No of file marks not skipped      */


/* 	Macros for placing controller and L.U.N. easliy in command */

#define slun(cmd,x)	((cmd.cntrlid) |= (x))	/* For structures */
#define scntl(cmd,x)	((cmd.cntrlid) |= ((x) << 4))
#define plun(cmd,x)	((cmd->cntrlid) |= (x)) /* For pointers to structures */
#define pcntl(cmd,x)	((cmd->cntrlid) |= ((x) << 4))


/* Macros for instruction register */

#define INTERR		1	/* Interrupt after execution, bit 0 */
#define DRIV_CMD	128	/* Driver issued command, bit 7     */


/*  structure used by copy and restore utilities */
/*  It is the argument passed to the ioctl call. */

struct cp_struct {
	int fd; 	/*  file descriptor for tape device */
	long offset;    /*  block offset 		    */
	long nblocks;   /*  number of blocks		    */
};	

