/*
 *	Copyright (c) 1987 Motorola Inc
 *	All Rights Reserved
 *	@(#)hawk.h	35.1
 */

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF Motorola	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ident	"@(#)hawk.h	35.1"

/*
 *
 * The following defines the organization of the shortio space on the
 * Interphase V/Ethernet 3207 (HAWK) ethernet interface board
 *
 */
struct hwkdevice {
        ushort board_csr;      /* hawk board control/status register, (w/r) */ 
        ushort board_ivec;     /* hawk interrupt vector register (w)        */
        ushort lance_dport;    /* lance data port (r/w)                     */ 
        ushort lance_aport;    /* lance address port (r/w)                  */
        ushort novram_acc;     /* hawk novram access register (r/w)         */
};

/*
 * This section contains the register definitions for the HAWK board
 */

/*
 * Board Control Register: Address = 0, write only     
 *
 * |D15|D14|D13|D12|D11|D10|D09|D08|D07|D06|D05|D04|D03|D02|D01|D00| 
 *   X   X   X   X   X   X   X   X   |   |   |   |   |   |   |   |
 *                                   |   |   |   |   |   |   |   IPL0(L)
 *                                   |   |   |   |   |   |   IPL1(L)
 *                                   |   |   |   |   |   IPL2(L)
 *                                   |   |   |   |   NOV RAM CE(L)
 *                                   |   |   |   INT CLR(L)
 *                                   |   |   PERR CLR(L)
 *                                   |   HW RESET(L)
 *                                   SYS FAIL(H)
 * X    =   Don't Care
 * SYS  =   To set the VME bus Sysfail line, this bit needs to be set to a 1.
 * FAIL     clear the Sysfail lin, clear this bit to a 0. This turns the front
 *          panel LED green. This bit is set upon power up or a hardware or
 *          software reset. The suggested use is upon power up, to run a test
 *          of the HAWK board. If the test passes, then clear this bit. If
 *          failure then leave this bit alone thus leaving the LED red and the
 *          Sysfail line set.
 * HW   =   Clear this bit to a 0 to reset the board.  This is the same as a
 * RESET    SYSRESET on the bus.  The board will be busy for 15us after setting 
 *          this bit. There is no need to set this bit after clearing.
 * PERR =   Clear this bit to a 0 to reset a parity error. This bit should be
 * CLR      cleared during initiaization to clear the parity bits in the 
 *          status register. There is no need to set this bit after clearing.
 * INT  =   Clear this bit to a 0 to reset an onboard interrupt. This bit should
 * CLR      be cleared at initialization time. There is no need to set this bit
 *          after clearing.
 * NOV  =   This bit must be cleared to a zero when writing or reading the non
 * RAM      volatile ram. It should be cleared during the whole cycle (see ZYCOR
 * CE       2444 spec for further information). When done with the cycle this 
 *          bit should be set to a 1.
 * IPL0-2 = These three bits set the interrupt level 1 through 7. These bits are
 *          the inversion of the actual level, ie, if level 3 is required then
 *          these bits should be set to IPL2=1,IPL1=0,IPL0=0. Level 0 specifies
 *          no interrupts. Level 0 is set upon a hardware or software reset.
 */

#define HBC_SYSFAIL             0x0080
#define HBC_LED                 0x0080
#define HBC_HW_RESET            0x0040
#define HBC_PERR_CLR            0x0020
#define HBC_INT_CLR             0x0010
#define HBC_NOV_RAM             0x0008
#define HBC_IPL2                0x0004
#define HBC_IPL1                0x0002
#define HBC_IPL0                0x0001

#define HBC_BITS  \
"\020\10SYSFAIL\7RESET\6PERRCLR\5INTCLR\4NOVRAM\3IPL2\2IPL1\1IPL0"

/*
 * Board Status Register: Address = 0, read only
 *
 * |D15|D14|D13|D12|D11|D10|D09|D08|D07|D06|D05|D04|D03|D02|D01|D00| 
 *   X   X   X   X   X   X   X   X   X   X   X   X   |   |   |   |
 *                                                   |   |   |   PERR BANK 0
 *                                                   |   |   PERR BANK 1
 *                                                   |   PERR BANK 2
 *                                                   PERR BANK 3
 * X      =    Don't care
 * PERR   =    Parity error in specified bank.
 *
 */

#define HBS_PERR_BANK3          0x0008
#define HBS_PERR_BANK2          0x0004
#define HBS_PERR_BANK1          0x0002
#define HBS_PERR_BANK0          0x0001

#define HBS_BITS  \
"\020\4PERR3\3PERR2\2PERR1\1PERR0"

/*
 * Interrupt Vector Register: Address = 2, write only
 *
 * |D15|D14|D13|D12|D11|D10|D09|D08|D07|D06|D05|D04|D03|D02|D01|D00| 
 *   X   X   X   X   X   X   X   X   |   |   |   |   |   |   |   |
 *                                   |   |   |   |   |   |   |   IVEC0
 *                                   |   |   |   |   |   |   IVEC1
 *                                   |   |   |   |   |   IVEC2
 *                                   |   |   |   |   IVEC3
 *                                   |   |   |   IVEC4
 *                                   |   |   IVEC5
 *                                   |   IVEC6
 *                                   IVEC7
 *
 * X     =    Don't Care
 * IVEC0-7 =  Interrupt vector, 8 bit code which is released to the VME
 *            bus in response to an interrupt acknowledge cycle. When
 *            writing to this register, the value written should be inverted, 
 *            i.e., if the vector 0x55 is required then an 0xAA should be
 *            written.
 */

#define HIV_IVEC7               0x0080
#define HIV_IVEC6               0x0040
#define HIV_IVEC5               0x0020
#define HIV_IVEC4               0x0010
#define HIV_IVEC3               0x0008
#define HIV_IVEC2               0x0004
#define HIV_IVEC1               0x0002
#define HIV_IVEC0               0x0001

/*
 * Lance Data Port: Address = 4, read/write
 *
 * |D15|D14|D13|D12|D11|D10|D09|D08|D07|D06|D05|D04|D03|D02|D01|D00| 
 *
 */

#define HLDP_D15                0x8000
#define HLDP_D14                0x4000
#define HLDP_D13                0x2000
#define HLDP_D12                0x1000
#define HLDP_D11                0x0800
#define HLDP_D10                0x0400
#define HLDP_D09                0x0200
#define HLDP_D08                0x0100
#define HLDP_D07                0x0080
#define HLDP_D06                0x0040
#define HLDP_D05                0x0020
#define HLDP_D04                0x0010
#define HLDP_D03                0x0008
#define HLDP_D02                0x0004
#define HLDP_D01                0x0002
#define HLDP_D00                0x0001

/*
 * Lance Register Address Port: Address = 6, read/write
 *
 * |D15|D14|D13|D12|D11|D10|D09|D08|D07|D06|D05|D04|D03|D02|D01|D00| 
 *
 */

#define HLAP_D15                0x8000
#define HLAP_D14                0x4000
#define HLAP_D13                0x2000
#define HLAP_D12                0x1000
#define HLAP_D11                0x0800
#define HLAP_D10                0x0400
#define HLAP_D09                0x0200
#define HLAP_D08                0x0100
#define HLAP_D07                0x0080
#define HLAP_D06                0x0040
#define HLAP_D05                0x0020
#define HLAP_D04                0x0010
#define HLAP_D03                0x0008
#define HLAP_D02                0x0004
#define HLAP_D01                0x0002
#define HLAP_D00                0x0001

/*
 * Hawk Board Novram Access Register: Address = 8, read/write
 *
 * |D15|D14|D13|D12|D11|D10|D09|D08|D07|D06|D05|D04|D03|D02|D01|D00| 
 *   X   X   X   X   X   X   X   X   X   X   X   X   X   X   X   d
 *
 * X     =     Don't Care
 * d     =     Data bit used to write and read the non-volatile ram. The
 *             device used for non-volatile storage of the Ethernet address
 *             is a 1 bit byte EEPROM. See specification of Xycor X244M
 *             Device for detailed programming information. The general
 *             procedure for storing or retreiving information from this device
 *             is detailed below.
 *
 *      1) Clear bit 3 in Control register (address 0)
 *      2) One bit at a time write the command code to the part, ie, read cmd
 *      3) If the command was a read the next bit after the last bit of the 
 *         read command will be coming from the device, and thus must be a read
 *         operation from this port.
 *         Else write data one bit at a time to the part.
 *      4) Continue the operation until done.
 *      5) Deassert the Nov Ram Chip Enable in the command register.
 *
 *          Special Notes:
 *               It's not quite as easy as above. There are a few hardware
 *               requirements that must be fulfilled by software.
 *               They are:
 *                    1) When writing the command code to the part, the code
 *                       called out in the data sheet must be inverted. There
 *                       is, however, no need to invert the actual stored data
 *                       being written or read from the part.
 *                    2) When reading data from the part, the first bit
 *                       output after the last bit of the read command is
 *                       actually bit 1 instead of bit 0. Following the read 
 *                       of bit 15, then bit 0 is read next. In other words,
 *                       the data is skewed by one bit location. 
 */

#define HNAR_D00                0x0001
/*
 * Novram "Opcode" masks (inverted due to 74AS646's)
 */
#define WRD_NOVRAM              0xE0            /* Reset write enable latch   */
#define STO_NOVRAM              0x60            /* Store ram to eeprom        */
#define SLP_NOVRAM              0xA0            /* Novram into low power mode */
#define WRT_NOVRAM              0x20            /* Writes word from location x*/
#define WRE_NOVRAM              0xC0            /* Set write enable latch     */
#define RCL_NOVRAM              0x40            /* Recall eeprom data into ram*/
#define RDE_NOVRAM              0x00            /* Reads word from location x */
/*
 * Novram "structure" pointers based on the layout above
 */
#define PHYS_0                  0x00            /* Physical word 0            */
#define PHYS_1                  0x01            /* Physical word 1            */
#define PHYS_2                  0x02            /* Physical word 2            */
#define LOG_0                   0x03            /* Logical word 0             */
#define LOG_1                   0x04            /* Logical word 1             */
#define LOG_2                   0x05            /* Logical word 2             */
#define LOG_3                   0x06            /* Logical word 3             */
#define SIG                     0x07            /* Ascii Signature            */

/*
 *      Structure of the Novram on HAWK.  It occupies 256 bytes.
 *
 *
 *      ________________________
 *      |      Physical 0      |  Bits 0 - 15    |
 *      ------------------------                 |   Physical Ethernet Address
 *      |      Physical 1      |  Bits 16 - 31   |  48 bits, with bit 0 forced 0
 *      ------------------------                 |
 *      |      Physical 2      |  Bits 32 - 47   |
 *      ------------------------ 
 *      |      Logical  0      |  Bits 0 -15     |
 *      ------------------------                 |
 *      |      Logical  1      |  Bits 16 - 31   |   Logical Ethernet Address
 *      ------------------------                 |  64 bits, with bit 0 forced 1
 *      |      Logical  2      |  Bits 32 - 47   |
 *      ------------------------                 |
 *      |      Logical  3      |  Bits 48 - 63   |
 *      ------------------------
 *                :               Unused
 *                :                  |
 *      ________________________     |
 *      |       Word 128       |    \ /
 *      ------------------------  ---"---   
 */


/*
 * AMD 7990 LANCE Ethernet controller registers.
 *
 * The LANCE chip saves address pins by accessing
 * several registers with one address pin by first writing the
 * register address to an internal address register, then reading
 * or writing a data register.  To use this safely, care must be
 * taken that the driver isn't reentered between the writing
 * of the address register and the access to the data register,
 * lest the reentered code try to touch the registers and
 * screw up the sequence.
 *
 * There are 4 registers accessible with this scheme, CSR0, CSR1,
 * CSR2, and CSR3.  In normal operation, only CSR0 can be or
 * needs to be accessed, so the driver normally leaves a 0 in the
 * Register Address Port, allowing CSR0 to be accessed simply by
 * accessing the Register Data Port.  During the initialization
 * sequence, when the other CSRs need to be accessed, the appropriate
 * CSR address is written into the address port, and afterwards
 * the 0 is put back in the address port.
 */


/*
 * Buffer size is chosen to give room for:
 */
#define LANCE_DADDR       6      /*  -   The destination address  */
#define LANCEE_SADDR      6      /*  -   The source address       */
#define LANCEE_TYPE       2      /*  -   The type field           */
#define LANCE_DATA     1500      /*  -   Max data bytes in the LANCE */
#define LANCEE_CRC        4      /*  -   Four byte CRC            */
#define LANCE_OVRN       48      /*  -   and a fifo overrun       */
/*
 * Total buffer size, is therefore 1566. We allow 2000.
 */
#define RX_BUFSZ       2000     /* maximum rx packet with slop     */
#define TX_BUFSZ       2000     /* maximum tx packet with slop     */

#define LANCE_MINPACK    60     /* minimum output packet length */
#define LANCE_MAXPACK  1566     /* maximum output packet length */
#define LANCE_MIN_TU    100     /* minimum length of first buffer of
                                   multi-buffer output packet */


#define CSR0              0
#define CSR1              1
#define CSR2              2
#define CSR3              3

/*
 * Control and status bits for CSR0.
 * These behave somewhat strangely, but the net effect is that
 * bit masks may be written to the register which affect only
 * those functions for which there is a one bit in the mask.
 * The exception is the interrupt enable, which must be explicitly
 * set to the correct value in each mask that is used.
 *
 * RO - Read Only, writing has no effect
 * RC - Read, Clear.  Writing 1 clears, writing 0 has no effect
 * RW - Read, Write.
 * W1 - Write with 1 only.  Writing 1 sets, writing 0 has no effect.
 *        Reading gives unpredictable data but doesn't hurt anything.
 * RW1 - Read, Write with 1 only.  Writing 1 sets, writing 0 has no effect.
 */

#define CSR0_ERR          0x8000          /* RO BABL | CERR | MISS | MERR */
#define CSR0_BABL         0x4000          /* RC transmitted too many bits */
#define CSR0_CERR         0x2000          /* RC No Heartbeat */
#define CSR0_MISS         0x1000          /* RC Missed an incoming packet */
#define CSR0_MERR         0x0800          /* RC Memory Error; no acknowledge */
#define CSR0_RINT         0x0400          /* RC Received packet Interrupt */
#define CSR0_TINT         0x0200          /* RC Transmitted packet Interrupt */
#define CSR0_IDON         0x0100          /* RC Initialization Done */
#define CSR0_INTR         0x0080          /* RO BABL|MISS|MERR|RINT|TINT|IDON */
#define CSR0_INEA         0x0040          /* RW Interrupt Enable */
#define CSR0_RXON         0x0020          /* RO Receiver On */
#define CSR0_TXON         0x0010          /* RO Transmitter On */
#define CSR0_TDMD         0x0008          /* W1 Transmit Demand (send it now) */
#define CSR0_STOP         0x0004          /* RW1 Stop */
#define CSR0_STRT         0x0002          /* RW1 Start */
#define CSR0_INIT         0x0001          /* RW1 Initialize */

/*
 * CSR1  is   the low 16 bits of the address of the initialization block
 * CSR2  is   the high 8 bits of the address of the initialization block
 *            the high 8 bits of the register must be 0
 * CSR3 mode bits:
 *
 */
#define CSR0_BSWP         0x4     /* Byte Swap (on for 68000 byte order) */
#define CSR0_ACON         0x2     /* ALE Control (on for active low ALE) */
#define CSR0_BCON         0x1     /* Byte Control (see the manual) */

/* The address contained in this structure must be longword aligned */
struct le_drp {                 /* Descriptor Ring Pointer */ 
        ushort  drp_laddr;     /* Low 16 bits of ring address */
        ushort  drp_haddr_len; /* High 8 bits of ring address - exponent */
};


/*
 * Initialization Block.  This structure is constructed in memory,
 * and it's address is written into the chip during initialization.
 * The chip then fetches it's initialization info from the structure.
 */
struct le_iblock {
        /* In the normal mode, these 16 bits are all 0 */
        ushort ib_mode;           /* Chip's operating parameters */
        /*
         * The bytes must be swapped within the word, so that, for example,
         * the address 8:0:20:1:25:5a is written in the order
         *             0 8 1 20 5a 25
         */
        uchar  ib_padr[6];

        uchar  ib_ladrf[8];

        struct  le_drp ib_rdrp;  /* Receive Descriptor Ring Pointer */
        struct  le_drp ib_tdrp;  /* Transmit Descriptor Ring Pointer */ 
};
/*
 * Initialization block mode bit positions
 */
#define MODE_PROM         0x8000  /* Promiscuous mode */
#define MODE_INTL         0x0040  /* Internal Loopback */
#define MODE_DRTY         0x0020  /* Disable Retry */
#define MODE_COLL         0x0010  /* Force Collision */
#define MODE_DTCR         0x0008  /* Disable Transmit CRC */
#define MODE_LOOP         0x0004  /* Loopback */
#define MODE_DTX          0x0002  /* Disable the transmitter */
#define MODE_DRX          0x0001  /* Disable the receiver */

struct  le_md {                 /* Message Descriptor */
        ushort md_ladr;        /* Low Order 16 Address Bits */
#define md_md0  md_ladr         /* --or message descriptor 0 */
        uchar  md_flags;       /* Flags                     */
        uchar  md_hadr;        /* High Order 8 Address Bits */
        ushort md_bcnt;        /* Buffer Byte Count (maximum length) */
#define md_md2  md_bcnt         /* --or message descriptor 2 */
        ushort md_mcnt;        /* Message Byte Count (actual length) */
#define md_md3  md_mcnt         /* --or message descriptor 3 */
};

/*
 * Receive message descriptor bits
 */
/* RMD1, md_flags bits */
#define RMD1_OWN         0x80    /* Chip owns the descriptor */
#define RMD1_ERR         0x40    /* Error occurred           */
#define RMD1_FRAM        0x20    /* Framming Error           */
#define RMD1_OFLO        0x10    /* Overflow Error           */
#define RMD1_CRC         0x08    /* CRC Error                */
#define RMD1_BUFF        0x04    /* Buffer Error             */
#define RMD1_STP         0x02    /* Start of Packet          */
#define RMD1_ENP         0x01    /* End of Packet            */
#define RMD1_BITS  \
"\010\10OWN\7ERR\6FRAM\5OFLOW\4CRC\3BUFF\2STP\1ENP"

/*
 * Transmit message descriptor bits
 */
/* TMD1, md_flags bits */
#define TMD1_OWN         0x80    /* Chip owns the descriptor */
#define TMD1_ERR         0x40    /* Error occurred           */
#define TMD1_RES         0x20    /* Reserved, chip writes 0  */
#define TMD1_MORE        0x10    /* More than one retry      */
#define TMD1_ONE         0x08    /* One retry needed         */
#define TMD1_DEF         0x04    /* Defered                  */
#define TMD1_STP         0x02    /* Start of Packet          */
#define TMD1_ENP         0x01    /* End of Packet            */
#define TMD1_BITS  \
"\010\10OWN\7ERR\6RES\5MORE\4ONE\3DEF\2STP\1ENP"

/* TMD3, md_md3 bits */
#define TMD3_BUFF 0x8000         /* Buffer Error (imples underflow too) */
#define TMD3_UFLO 0x4000         /* Underflow Error                     */
#define TMD3_RES  0x2000         /* Reserved bit                        */
#define TMD3_LCOL 0x1000         /* Late Collision                      */
#define TMD3_LCAR 0x0800         /* Loss of Carrier                     */
#define TMD3_RTRY 0x0400         /* More than 16 Retry's                */
#define TMD3_TDR  0x003f         /* Time Domain Reflectometry counter mask */
#define TMD3_BITS \
"\6\20BUFF\17UFLO\16RES\15LCOL\14LCAR\13RTRY"

/*
 *
 */
#define RX_NRINGS       16      /* Number of rx ring entries */
#define TX_NRINGS       16      /* Number of tx ring entries */
#define LANCE_ERRMASK   0xF800  /* Fix mask */
#define LANCE_INTMASK   0x0780  /* Fix mask */

/*
 * HAWK memory buffer structure
 */
struct hwk_buf {
	struct datab	datab;			/* STREAMS datab	*/
	struct hwk_buf *active;			/* active pointer	*/
	IFARG		ifa;			/* IP interface arg	*/
	char 		data[LANCE_MAXPACK - sizeof (e_frame)];
};

/*
 * Structure of HAWK's vme bus memory
 *
 *	+---------------------------------+
 *	|                                 |
 *	| Transmit Descriptors (TX_NRINGS)|
 *	|                                 |
 *	+---------------------------------+
 *	|                                 |
 *	| Receive Descriptors (RX_NRINGS) |
 *	|                                 |
 *	+---------------------------------+
 *	|                                 |
 *	|   Lance Initialization Block    |
 *	|                                 |
 *	+---------------------------------+
 *	|				  |
 *	|  hwk_buf buffers		  |
 *	+---------------------------------+
 *	|				  |
 *	|  slop				  |
 *	+---------------------------------+
 */
struct vmemem {
        /*
         * the first descriptor must be alligned on a quad word boundary  
         * so we set him at lance base 0 always and don't worry about it      
         */
        struct le_md	 vm_tmd[TX_NRINGS];	/* tx msg descriptors       */
        struct le_md	 vm_rmd[RX_NRINGS];	/* rx msg descriptors       */
        struct le_iblock vm_ib;			/* lance init block         */
	struct hwk_buf	 vm_buf[1];		/* buffer memory	*/
};
/*
 * Offsets for use with LANCE-referenced HAWK memory (zero-based)
 */
#define lance_base	((struct vmemem *) 0)
#define TXMD_OFF(x)	(uint)(&lance_base->vm_tmd[0])
#define RXMD_OFF(x)	(uint)(&lance_base->vm_rmd[0])
#define IBLOCK_OFF	(uint)(&lance_base->vm_ib)
#define BUF_PKTOFF	(int)&((struct hwk_buf *) 0)->data[- sizeof(e_frame)]
#define	vme_off(x)	((caddr_t) (x) - (caddr_t) hwk_vme)
#define buf_to_pkt(bp)	&((caddr_t) (bp))[BUF_PKTOFF]
#define pkt_to_buf(p)	(struct hwk_buf *) &((caddr_t) (p))[-BUF_PKTOFF]

/*
 * HAWK/LANCE status record 
 */
struct status {
        uint   st_nxt_rxring;                  /* Next receive ring pointer  */
        uint   st_nxt_txring;                  /* Next transmit ring pointer */
        uint   st_interrupts;                  /* number of interrupts srved */
        ushort st_bcsr_image;                  /* HAWK board csr write image */
        ushort st_bcsr_value;                  /* HAWK board csr read value  */
        ushort st_perr;                        /* HAWK board parity errors   */
        ushort st_lance_csr0;                  /* LANCE csr0 value           */
        ushort st_lance_csr1;                  /* LANCE csr1 value           */
        ushort st_lance_csr2;                  /* LANCE csr2 value           */
        ushort st_lance_csr3;                  /* LANCE csr3 value           */
};

/*
 * HAWK statistics log 
 */
struct stats {
	struct ifi_stats st_ifi;		/* generalised if stats	     */
	uint	st_nomem;			/* dropped due to no mem     */
	uint	st_copy;			/* rx pkts copied	     */
        uint   st_csr0_babl;                   /* LANCE babble error         */
        uint   st_csr0_cerr;                   /* LANCE collision error      */
        uint   st_csr0_miss;                   /* LANCE missed packet error  */
        uint   st_csr0_merr;                   /* LANCE memory error         */
        uint   st_rmd1_fram;                   /* RX framing error           */
        uint   st_rmd1_oflo;                   /* RX overflow error          */
        uint   st_rmd1_crc;                    /* RX CRC error               */
        uint   st_rmd1_buff;                   /* RX buff error              */
        uint   st_tmd1_more;                   /* TX more log                */
        uint   st_tmd1_one;                    /* TX one log                 */
        uint   st_tmd1_def;                    /* TX deferred log            */
        uint   st_tmd3_buff;                   /* TX buff error              */
        uint   st_tmd3_uflo;                   /* TX underflow error         */
        uint   st_tmd3_lcol;                   /* TX late collision error    */
        uint   st_tmd3_lcar;                   /* TX loss of carrier error   */
        uint   st_tmd3_rtry;                   /* TX retry error             */
        struct status st_status;                /* Board status              */
};
#define st_ipackets	st_ifi.ipkts
#define	st_opackets	st_ifi.opkts
#define st_ierrors	st_ifi.ierrs
#define st_oerrors	st_ifi.oerrs

/*
 * HAWK specific I/O controls
 */
#define HWKIOC(x)	((('h' << 8 | 'w') << 8 | 'k') << 8 | (x))
#define HWKIOC_RESET	HWKIOC(1)
#define HWKIOC_GETSTATS	HWKIOC(2)
