/*
 *	Copyright (c) 1984 AT&T
 *	Copyright (c) 1987 Motorola Inc
 *	All Rights Reserved
 *	@(#)kern-port:sys/immu.h	35.5
 */

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF AT&T	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ident	"@(#)kern-port:sys/immu.h	35.5"

/*
 * Page Descriptor Entry Definitions
 */

#ifdef CMMU
typedef union cpde {	/* CMMU pde */
/*         short page descriptor entry
 *
 *  +---------------------+---+-----+--+--+--+--+--+---+---+--+--+--+
 *  |      Page Address   |lck|ndref|wt|sp| g|ci| r|mod|ref|wp| r| v|
 *  +---------------------+---+-----+--+--+--+--+--+---+---+--+--+--+
 *             20           1   1     1  1  1  1  1  1   1   1  1  1
 */
	struct {
		uint	pg_pfn  : 20,	/* Physical page frame (hardware) */
			pg_lock	:  1,	/* Lock in core      (software)	*/
			pg_ndref:  1,	/* Need a reference  (software)	*/
			pg_wt   :  1,	/* Write through cac (hardware) */
			pg_sp   :  1,	/* Supervisor Only   (hardware) */
			pg_g	:  1,	/* Global page       (hardware)	*/
			pg_ci	:  1,	/* Cache inhibit     (hardware)	*/
			pg_r2	:  1,	/* Reserved			*/
			pg_mod	:  1,	/* Page modified     (hardware)	*/
			pg_ref	:  1,	/* Page referenced   (hardware)	*/
			pg_wp	:  1,	/* Write protect     (hardware) */
			pg_r1	:  1,	/* Reserved			*/
			pg_val	:  1;	/* Valid Page desc.  (hardware) */
	} pgm;

	struct {
		uint	pg_spde; /* CMMU page descriptor (table) entry */
	} pgi;
} cpde_t;	/* CMMU Page Descriptor */
#else
typedef union spde {
/*         short page descriptor entry
 *
 *  +---------------------+---+-----+--+--+--+---+---+--+--+
 *  |      Page Address   |lck|ndref| g|ci| l|mod|ref|wp|dt|
 *  +---------------------+---+-----+--+--+--+---+---+--+--+
 *             22           1   1     1  1  1  1   1   1  2
 */
	struct {
		uint	pg_pfn  : 22,	/* Physical page frame (hardware) */
			pg_lock	:  1,	/* Lock in core      (software)	*/
			pg_ndref:  1,	/* Need a reference  (software)	*/
			pg_g	:  1,	/* Gate page         (hardware)	*/
			pg_ci	:  1,	/* Cache inhibit     (hardware)	*/
			pg_l	:  1,	/* Cache desc. lock  (hardware)	*/
			pg_mod	:  1,	/* Page modified     (hardware)	*/
			pg_ref	:  1,	/* Page referenced   (hardware)	*/
			pg_wp	:  1,	/* Write protect     (hardware) */
			pg_dt	:  2;	/* Descriptor type   (hardware)	*/
	} pgm;

	struct {
		uint	pg_spde; /* short page descriptor (table) entry */
	} pgi;
} spde_t;

typedef union lpde {
/*                    long page descriptor entry	(* not used *)
 *
 * +---+-----+---+---+--+-+-+--+-+---+---+--+--+     +---------+-+-----+----+--+
 * |l/u|limit|ral|wal|sg|s|g|ci|l|mod|ref|wp|dt|     |Page Addr|-|ndref|lock|--|
 * +---+-----+---+---+--+-+-+--+-+---+---+--+--+     +---------+-+-----+----+--+
 *   1    15   3   3  1  1 1 1  1  1   1  1   2          24     1    1    1   5
 */
	struct {
		uint	pg_lu	:  1,	/* Lower / upper     (hardware)	*/
			pg_limit: 15,	/* Table limit       (hardware)	*/
			pg_ral	:  3,	/* Read access level (hardware)	*/
			pg_wal	:  3,	/* Write access level(hardware)	*/
			pg_sg	:  1,	/* Shared globally   (hardware)	*/
			pg_s	:  1,	/* Supervisor page   (hardware)	*/
			pg_g	:  1,	/* Gate page         (hardware)	*/
			pg_ci	:  1,	/* Cache inhibit     (hardware)	*/
			pg_l	:  1,	/* Cache desc. lock  (hardware)	*/
			pg_mod	:  1,	/* Page modified     (hardware)	*/
			pg_ref	:  1,	/* Page referenced   (hardware)	*/
			pg_wp	:  1,	/* Write protect     (hardware) */
			pg_dt	:  2;	/* Descriptor type   (hardware)	*/

		uint	pg_pfn  : 22,	/* Physical page frame (hardware) */
				:  3,	/* Unused/available  (software)	*/
			pg_ndref:  1,	/* Needs reference   (software)	*/
			pg_lock	:  1,	/* Lock in core      (software)	*/
				:  5;	/* Unused/available  (software)	*/
	} pgm;

	struct {
		uint	pg_hlpde; /* hardware long page descriptor entry */
		uint	pg_slpde; /* software long page descriptor entry */
	} pgi;
} lpde_t;
#endif	/* def CMMU */

#ifdef CMMU
#define pde_t	cpde_t			/* all pde's are CMMU type */
#define	NPDEPP	1			/* number of PDE's per page */
#define NPGPT		1024		/* Nbr of pages per page table (seg). */
#else
#define pde_t	spde_t			/* all pde's are short for MMB */
#define	NPDEPP	4			/* number of PDE's per page */
#define NPGPT		64		/* Nbr of pages per page table (seg). */
#endif

/*
 *	Page Table
 */

typedef union ptbl {
	int page[NPGPT*NPDEPP];
} ptbl_t;

/* Page descriptor (table) entry dependent constants */

#define	NBPP		4096		/* Number of bytes per page */
#ifdef CMMU
#define	NBPPT		4096		/* Number of bytes per page table */
#define	BPTSHFT		12 		/* LOG2(NBPPT) if exact */
#else
#define	NBPPT		1024		/* Number of bytes per page table */
#define	BPTSHFT		10 		/* LOG2(NBPPT) if exact */
#endif
#define NDPP		8		/* Number of disk blocks per page */
#define DPPSHFT		3		/* Shift for disk blocks per page. */

#define	FILESHFT	0		/* Shift for file disk blocks per page*/
					/* 1KFS specific - Not true for 512 FS*/
					/* Dependent on page and file size    */

#define PNUMSHFT	12		/* Shift for page number from addr. */
#define POFFMASK        0xFFF		/* Mask for offset into page. */
#ifdef CMMU
#define PNUMMASK	0x3FF		/* Mask for page number in segment. */
#define	NPDEPPT		1024		/* Nbr of descriptors per page table */
#else
#define PNUMMASK	0x3F		/* Mask for page number in segment. */
#define	NPDEPPT		256		/* Nbr of descriptors per page table */
#endif

#define	PG_VALID	1		/* bit indep. valid flag */

#define PG_INV		0		/* Invalid descriptor */
#define PG_VPD		1		/* Valid page descriptor */

/* byte addr to virtual page */

#define pnum(X)   ((uint)(X) >> PNUMSHFT) 

/* page offset */

#define poff(X)   ((uint)(X) & POFFMASK)

/* byte address to page within segment */

#define psnum(X)   (((uint)(X) >> PNUMSHFT) & PNUMMASK)

/*	Disk blocks (sectors) and pages.
*/

#define	ptod(PP)	((PP) << DPPSHFT)
#define	dtop(DD)	(((DD) + NDPP - 1) >> DPPSHFT)
#define dtopt(DD)	((DD) >> DPPSHFT)

/*	Disk blocks (sectors) and bytes.
*/

#define	dtob(DD)	((DD) << SCTRSHFT)
#define	btod(BB)	(((BB) + NBPSCTR - 1) >> SCTRSHFT)
#define	btodt(BB)	((BB) >> SCTRSHFT)

/*	Page tables (256 entries == 1024 bytes) to pages.
 *	May be considered a "standard" table size since a few of 
 *	these macros are used for segment, domain and rlist table management.
 */

#ifdef CMMU
/*	The CMMU page and segment tables are 4K bytes == 1K entries.
 *	r_list size is still 1K ( provides 256 page table == 4Meg * 256 = 1Gig )
 *	so only r_lists use small page alloc for the CMMU.
 */ 
#define	NPTPP		1		/* Nbr of page tables per page.	*/
#define	NPTPPSHFT	0		/* Shift for NPTPP. */
#else
#define	NPTPP		4		/* Nbr of page tables per page.	*/
#define	NPTPPSHFT	2		/* Shift for NPTPP. */
#endif

#define	PTBLSZ		1		/* Nbr of std tbl units per page tbl */
#define	PT_PAIR		2		/* Std tbl units per pde/dbd tbl pair */

#define	pgtopt(X)	((uint)(X) << NPTPPSHFT)
#define	pttopgs(X)	(((uint)(X) + NPTPP - 1) >> NPTPPSHFT)
#define	pttob(X)	((uint)(X) << BPTSHFT)
#define	btopt(X)	(((uint)(X) + NBPPT - 1) >> BPTSHFT)

#ifdef CMMU	/* rlists are smaller than page tables */
#define BRLSHFT	10	/* rlists are 1K */
#define NBPRL	1024
#define	rltob(X)	((uint)(X) << BRLSHFT)
#define	btorl(X)	(((uint)(X) + NBPRL - 1) >> BRLSHFT)
#else		/* rlists are the same size as page tables */
#define btorl(X)	btopt(X)
#define rltob(X)	pttob(X)
#endif

/*	Tables to small-subpage-chunks. (see smallpage.c)
*/

#define NSMPGCPT	1	/* Nbr small-subpage-chunks per table */ 
#define pttosmpg(X)	((uint)(X)*NSMPGCPT)

/*	NOTE: Page, segment and r_list tables are all the same size at present
 *	so pttosmpg, pttob and btopt can be used when managing all of
 *	these tables.  -psr
 */


/* Minimum address to be mapped for ROM and IO */
#define IO_32_MIN 0xe0000000			/* must be on 16M boundary */

#define NULLADDR	0

/*
 * The following macros help access the disk block descriptors.
 * They are always located immediately following the page table.
 */

#ifdef CMMU
#define	DBDOFF	1				/* offset between dbd's */
#else
#define	DBDOFF	4				/* offset between dbd's */
#endif
#define	db_getnxt(dbd)	((dbd) + DBDOFF)	/* assumes dbd_t *dbd */

/*
 * A collection of 4 (1K) page descriptors that define a 4K page
 */

#ifndef CMMU /* dont use or need this for CMMU */
typedef	struct	Pde {
			pde_t a;
			pde_t b;
			pde_t c;
			pde_t d;
} Pde_t;
#else
#define	Pde_t	pde_t
#endif

#define BITSET		1
#define BITCLR		0
#define	PG_REF		0x00000008
#ifdef CMMU
#define	PG_NDREF	0x00000400
#else
#define	PG_NDREF	0x00000100
#endif

#ifndef CMMU
/*
 * Page macros for 68k
 */
#define pg_setvalid(P) { \
	(P)->pgm.pg_dt = PG_VPD; \
	(P+1)->pgm.pg_dt = PG_VPD; \
	(P+2)->pgm.pg_dt = PG_VPD; \
	(P+3)->pgm.pg_dt = PG_VPD; }

#define pg_clrvalid(P) { \
	(P)->pgm.pg_dt = PG_INV; \
	(P+1)->pgm.pg_dt = PG_INV; \
	(P+2)->pgm.pg_dt = PG_INV; \
	(P+3)->pgm.pg_dt = PG_INV; }

#define pg_tstvalid(P) ((P)->pgm.pg_dt == PG_VPD)

#define pg_setlock(P) ((P)->pgm.pg_lock = BITSET)
#define pg_clrlock(P) ((P)->pgm.pg_lock = BITCLR)
#define pg_tstlock(P) ((P)->pgm.pg_lock)

#define pg_setmod(P) { \
	(P)->pgm.pg_mod = BITSET; \
	(P+1)->pgm.pg_mod = BITSET; \
	(P+2)->pgm.pg_mod = BITSET; \
	(P+3)->pgm.pg_mod = BITSET; }

#define pg_clrmod(P) { \
	(P)->pgm.pg_mod = BITCLR; \
	(P+1)->pgm.pg_mod = BITCLR; \
	(P+2)->pgm.pg_mod = BITCLR; \
	(P+3)->pgm.pg_mod = BITCLR; }

#define pg_tstmod(P) \
       ((P)->pgm.pg_mod || \
	(P+1)->pgm.pg_mod || \
	(P+2)->pgm.pg_mod || \
	(P+3)->pgm.pg_mod)

#define pg_setcw(P) { \
	(P)->pgm.pg_wp = BITSET; \
	(P+1)->pgm.pg_wp = BITSET; \
	(P+2)->pgm.pg_wp = BITSET; \
	(P+3)->pgm.pg_wp = BITSET; }

#define pg_clrcw(P) { \
	(P)->pgm.pg_wp = BITCLR; \
	(P+1)->pgm.pg_wp = BITCLR; \
	(P+2)->pgm.pg_wp = BITCLR; \
	(P+3)->pgm.pg_wp = BITCLR; }

#define pg_tstcw(P) ((P)->pgm.pg_wp)

#define pg_setref(P) { \
	(P)->pgm.pg_ref = BITSET; \
	(P+1)->pgm.pg_ref = BITSET; \
	(P+2)->pgm.pg_ref = BITSET; \
	(P+3)->pgm.pg_ref = BITSET; }

#define pg_clrref(P) { \
	(P)->pgm.pg_ref = BITCLR; \
	(P+1)->pgm.pg_ref = BITCLR; \
	(P+2)->pgm.pg_ref = BITCLR; \
	(P+3)->pgm.pg_ref = BITCLR; }

#define pg_tstref(P) \
       ((P)->pgm.pg_ref || \
	(P+1)->pgm.pg_ref || \
	(P+2)->pgm.pg_ref || \
	(P+3)->pgm.pg_ref)

#define pg_setndref(P) ((P)->pgm.pg_ndref = BITSET)
#define pg_clrndref(P) ((P)->pgm.pg_ndref = BITCLR)
#define pg_tstndref(P) ((P)->pgm.pg_ndref)

#define pg_setci(P) { \
	(P)->pgm.pg_ci = BITSET; \
	(P+1)->pgm.pg_ci = BITSET; \
	(P+2)->pgm.pg_ci = BITSET; \
	(P+3)->pgm.pg_ci = BITSET; }

#define pg_clrci(P) { \
	(P)->pgm.pg_ci = BITCLR; \
	(P+1)->pgm.pg_ci = BITCLR; \
	(P+2)->pgm.pg_ci = BITCLR; \
	(P+3)->pgm.pg_ci = BITCLR; }

#define pg_tstci(P) ((P)->pgm.pg_ci)

#define pg_setpfn(P,PFN) { \
	(P)->pgm.pg_pfn = (PFN * 4); \
	(P+1)->pgm.pg_pfn = (PFN * 4) + 1; \
	(P+2)->pgm.pg_pfn = (PFN * 4) + 2; \
	(P+3)->pgm.pg_pfn = (PFN * 4) + 3; }

#define pg_getpfn(P) (((P)->pgm.pg_pfn) / 4)

#define pg_setpde(P,PDE) { \
	(P)->pgi.pg_spde = PDE; \
	(P+1)->pgi.pg_spde = PDE; \
	(P+2)->pgi.pg_spde = PDE; \
	(P+3)->pgi.pg_spde = PDE; }

#define pg_getpde(P) ((P)->pgi.pg_spde)
#define pg_getaddr(P) (ctob(pg_getpfn(P)))
#define pg_gettfn(P) ((P)->pgm.pg_pfn)

#define pg_chkref(P,MASK) \
	(((P)->pgi.pg_spde & MASK) || \
	 ((P+1)->pgi.pg_spde & MASK) || \
	 ((P+2)->pgi.pg_spde & MASK) || \
	 ((P+3)->pgi.pg_spde & MASK))

#define pg_pdecp(P1,P2) { \
	*P2 = *P1; \
	*(P2 + 1) = *(P1 + 1); \
	*(P2 + 2) = *(P1 + 2); \
	*(P2 + 3) = *(P1 + 3); }

#else /* CMMU */

#define pg_setvalid(P) ((P)->pgm.pg_val = PG_VPD)
#define pg_clrvalid(P) ((P)->pgm.pg_val = PG_INV)
#define pg_tstvalid(P) ((P)->pgm.pg_val == PG_VPD)

#define pg_setlock(P) ((P)->pgm.pg_lock = BITSET)
#define pg_clrlock(P) ((P)->pgm.pg_lock = BITCLR)
#define pg_tstlock(P) ((P)->pgm.pg_lock)

#define pg_setmod(P) ((P)->pgm.pg_mod = BITSET)
#define pg_clrmod(P) ((P)->pgm.pg_mod = BITCLR)
#define pg_tstmod(P) ((P)->pgm.pg_mod)

#define pg_setcw(P) ((P)->pgm.pg_wp = BITSET)
#define pg_clrcw(P) ((P)->pgm.pg_wp = BITCLR)
#define pg_tstcw(P) ((P)->pgm.pg_wp)

#define pg_setref(P) ((P)->pgm.pg_ref = BITSET)
#define pg_clrref(P) ((P)->pgm.pg_ref = BITCLR)
#define pg_tstref(P) ((P)->pgm.pg_ref)

#define pg_setndref(P) ((P)->pgm.pg_ndref = BITSET)
#define pg_clrndref(P) ((P)->pgm.pg_ndref = BITCLR)
#define pg_tstndref(P) ((P)->pgm.pg_ndref)

#define pg_setci(P) ((P)->pgm.pg_ci = BITSET)
#define pg_clrci(P) ((P)->pgm.pg_ci = BITCLR)
#define pg_tstci(P) ((P)->pgm.pg_ci)

#define pg_setpfn(P,PFN) ((P)->pgm.pg_pfn = PFN)
#define pg_getpfn(P) ((P)->pgm.pg_pfn)

#define pg_setpde(P,PDE) ((P)->pgi.pg_spde = PDE)
#define pg_getpde(P) ((P)->pgi.pg_spde)

#define pg_getaddr(P) (ctob(pg_getpfn(P))
#define pg_gettfn(P) ((P)->pgm.pg_pfn)

#define pg_chkref(P,MASK) ((P)->pgi.pg_spde & MASK)

#endif

/*
 * Segment Descriptor Entry Definitions
 */

#ifdef CMMU
typedef union csde {	/* CMMU sde */
/*         CMMU segment descriptor entry
 *
 *  +---------------------+----+--+--+--+--+-----+--+--+--+
 *  |   Table   Address   | res|wt|sp| g|ci| res |wp| r| v|
 *  +---------------------+----+--+--+--+--+-----+--+--+--+
 *             20            2   1  1  1  1   3    1  1  1
 */
	struct {
		uint	pt_tpfn  : 20,	/* Physical Table pg (hardware) */
			pt_r3	:  2,	/* Reserved			*/
			pt_wt   :  1,	/* Write through cac (hardware) */
			pt_sp   :  1,	/* Supervisor Only   (hardware) */
			pt_g	:  1,	/* Global page       (hardware)	*/
			pt_ci	:  1,	/* Cache inhibit     (hardware)	*/
			pt_r2	:  3,	/* Reserved			*/
			pt_wp	:  1,	/* Write protect     (hardware) */
			pt_r1	:  1,	/* Reserved			*/
			pt_val	:  1;	/* Valid Table desc. (hardware) */
	} ptm;

	struct {
		uint	pt_ssde; /* CMMU segment descrpt. (table) entry */
	} pti;
} csde_t;	/* CMMU Segment Descriptor */

#define sde_t	csde_t			/* all sde's are CMMU type */

#else /* ndef CMMU */
typedef union ssde {
/*         short segment descriptor entry	(* not used *)
 *
 *      +---------------------+---+--+--+
 *      |    Table  Address   |ref|wp|dt|
 *      +---------------------+---+--+--+
 *                 22+6           1   1  2
 */

	struct {
		uint	pt_tpfn : 22,	/* Physical table pfn (hardware) */
				:  6,	/* Zeros for table address (hardware) */
			pt_ref	:  1,	/* Page referenced   (hardware)	*/
			pt_wp	:  1,	/* Write protect     (hardware) */
			pt_dt	:  2;	/* Descriptor type   (hardware)	*/
	} ptm;

	struct {
		uint	pt_ssde; /* short page descriptor (table) entry */
	} pti;
} ssde_t;


typedef union lsde {
/*                    long segment descriptor entry
 *
 * +---+-----+---+---+--+-+-+-+-+-+---+--+--+ +-------------+-+-+-+-+
 * |l/u|limit|ral|wal|sg|s|-|-|-|-|ref|wp|dt| |Table Address|-|-|-|-|
 * +---+-----+---+---+--+-+-+-+-+-+---+--+--+ +-------------+-+-+-+-+
 *   1    15   3   3  1  1 1 1 1 1  1   1  2         22+6      1 1 1 1
 */

	struct {
		uint	pt_lu	:  1,	/* Lower / upper     (hardware)	*/
			pt_limit: 15,	/* Table limit       (hardware)	*/
			pt_ral	:  3,	/* Read access level (hardware)	*/
			pt_wal	:  3,	/* Write access level(hardware)	*/
			pt_sg	:  1,	/* Shared globally   (hardware)	*/
			pt_s	:  1,	/* Supervisor page   (hardware)	*/
			pt_zero	:  4,	/* Unused zero bits  (hardware) */
			pt_ref	:  1,	/* Page referenced   (hardware)	*/
			pt_wp	:  1,	/* Write protect     (hardware) */
			pt_dt	:  2;	/* Descriptor type   (hardware)	*/

		uint	pt_tpfn : 22,	/* Physical table pfn (hardware) */
				:  6,	/* Zeros for table address (hardware) */
			        :  4;	/* Unused/available  (software) */
	} ptm;

	struct {
		uint	pt_hlsde; /* hardware long segment descriptor entry */
		uint	pt_slsde; /* software long segment descriptor entry */
	} pti;
} lsde_t;

#define sde_t	lsde_t			/* all sde's are long for MMB */

#define PT_VSD		2		/* Valid pointer to short descr. */
#define PT_VLD		3		/* Valid pointer to long descr. */

#endif /* CMMU */

#define PT_INV		0		/* Invalid pointer */

#ifdef CMMU
#define PT_VLD		1		/* Valid pointer to CMMU descr. */
#endif

/*	The following macros are used to set, clear, and test
 *	the value of the bits in a segment descriptor (table) entry 
 */

#ifdef CMMU
#define pt_setvalid(P,V) ((P)->ptm.pt_val = (V))	/* Set valid bit. */
#define pt_clrvalid(P)	 ((P)->ptm.pt_val = PT_INV)	/* Clear valid bit. */
#define pt_tstvalid(P,V) ((P)->ptm.pt_val == (V))	/* Test valid bit. */
#else
#define pt_setvalid(P,V) ((P)->ptm.pt_dt = (V))		/* Set valid bit. */
#define pt_clrvalid(P)	 ((P)->ptm.pt_dt = PT_INV)	/* Clear valid bit. */
#define pt_tstvalid(P,V) ((P)->ptm.pt_dt == (V))	/* Test valid bit. */

#define pt_setlim(P,L)	((P)->ptm.pt_limit = L)		/* Set index limit. */
#endif

#define pt_setref(P)	((P)->ptm.pt_ref = BITSET) 	/* Set ref bit.	*/
#define pt_clrref(P)	((P)->ptm.pt_ref = BITCLR)	/* Clear ref bit. */
#define pt_tstref(P)	((P)->ptm.pt_ref)		/* Test ref bit. */

#define pt_setprot(P)	((P)->ptm.pt_wp = BITSET)	/* Set wrt prot bit. */
#define pt_clrprot(P)	((P)->ptm.pt_wp = BITCLR)	/* Clear wrt prot bit */
#define pt_tstprot(P)	((P)->ptm.pt_wp)		/* Test wrt prot bit. */

#define pt_settfn(P,TFN) ((P)->ptm.pt_tpfn = TFN) /* Set Table frame no. */
#define	pt_gettfn(P)	((P)->ptm.pt_tpfn)	/* Get Table frame no. */
#define	pt_getaddr(P)	((caddr_t)pttob(pt_gettfn(P)))	/* Get table address. */

#ifndef CMMU
/*	The following macro sets up several pointer descriptor 
 *	sets up fields with default values.  Individual fields 
 *	may be changed after setup but most are fixed for the port.
 *
 *	Setup modifies only the upper 28 bits of the long descriptor.
 *	Software bits, valid and write protect and address fields are
 *	not touched.
 *
 *	Default setups for the MMB:
 *		L/U = L = 0		(* always *)
 *		seven bits = 0		(* always *)
 *		limit = 255		(*  255 is for page table *)
 *		ral = 7			(* always *)
 *		wal = 7			(* always *)
 *		next six bits = 0	(* always *)
 */
#define SDEFGP0  0x00fffc00	/* setup value for MMB pointer table entries */
#define pt_setup(P) ((P)->pti.pt_hlsde = SDEFGP0)


/*	Segment descriptor (table) dependent constants.	*/

#define NBPS		0x40000 /* Number of bytes per segment */
#define SNUMSHFT	18	/* Shift for segment number from address. */
#define SNUMMASK	0x7F	/* Mask for segment number after shift. */
#define SOFFMASK	0x3FFFF	/* Mask for offset into segment. */

#define	SEG_RW		1		/* user permissions read/write */
#define	SEG_RO		2		/* user permissions read-only */

#else /* def CMMU */

/*	Segment descriptor (table) dependent constants.	*/

#define NBPS		0x400000	/* Number of bytes per segment */
#define SNUMSHFT	22	/* Shift for segment number from address. */
#define SNUMMASK	0x3FF	/* Mask for segment number after shift. */
#define SOFFMASK	0x3FFFFF	/* Mask for offset into segment. */

#define	SEG_RW		0		/* user permissions read/write */
#define	SEG_RO		1		/* user permissions read-only */

#endif /* CMMU */

#define snum(X)   (((uint)(X) >> SNUMSHFT) & SNUMMASK)
#define soff(X)   ((uint)(X) & SOFFMASK)

#define	SEGSIZE		btoc(NBPPT)	/* seg tbl size in clicks (rounded up)*/
#define	STBLSZ		1		/* Nbr of std tbl units per seg tbl */
#define	SEG_RW		1		/* user permissions read/write */
#define	SEG_RO		2		/* user permissions read-only */

#ifndef CMMU
/*
 * Domain Descriptor Entry Definitions
 *
 * The domain descriptors are the same format as the segment descriptors
 * in the current implementation.
 */
#define dde_t sde_t

/*	Domain descriptor (table) dependent constants.	*/

#define DNUMSHFT	25		/* Shift for domain number from addr. */
#define DNUMMASK	0x7F		/* Mask for domain number after shft. */
#define DOFFMASK	0x1FFFFFF	/* Mask for offset into domain. */

#define dnum(X)   (((uint)(X) >> DNUMSHFT) & DNUMMASK)
#define doff(X)   ((uint)(X) & DOFFMASK)

/*
 * Memory Management Unit Definitions
 */

typedef struct _VAR {	/*  virtual address     */
	uint v_ssl	: 14,	/*  segment number      */
	     v_psl	:  6,	/*  page number         */
	     v_byte	: 12;	/*  offset within page  */
} VAR;

/*  masks to extract portions of a VAR  */

#define MSK_IDXSEG  0x3ffff  /*  lower 18 bits == PSL || BYTE  */


/* 
 * MMB control interface definitions
 *
 * Two 32 bit registers provide the MMB programmer interface.
 * They appear in the CPU address space at address 0x10000.
 *
 * The root pointer is set to the base of the first translation table
 * used in a table search.
 * The translation control register is defined below.
 *
 *		+---------------------+
 *	0x10000	|     root pointer    |	 R/W
 *		+---------------------+
 *	0x10004	| translation control |  R/W
 *		+---------------------+
 */

typedef union tcr {

/*         translation control register
 *
 *  +-+-----+-----+-----+----+----+-----+-----+-----+-----+
 *  |e| zero| sre | fcl | ps | is | tia | tib | tic | tid |
 *  +-+-----+-----+-----+----+----+-----+-----+-----+-----+
 *   1   5     1     1    4     4    4     4     4     4
 */
	struct {
		uint	tc_e	:  1,	/* Enable address translation */
				:  5,	/* Unused, taken to be zero */
			tc_sre	:  1,	/* Supervisor root pointer enable */
			tc_fcl	:  1,	/* Function code lookup enable */
			tc_ps	:  4,	/* Page size = (2**tc_ps) */
			tc_is	:  4,	/* Initial shift */
			tc_tia	:  4,	/* Table index A */
			tc_tib	:  4,	/* Table index B */
			tc_tic	:  4,	/* Table index C */
			tc_tid	:  4;	/* Table index D */
	} tcm;

	struct {
		uint	tc_tcr;	/* translation control register */
	} tci;
} tcr_t;


/*  MMB-specific routines  */

extern	void mmbstore();	/* Routine to set MMB register */

/*  68030 MMU-specific routines  */

extern	void tt0store();	/* Routine to set tt0 register */
extern	void tt1store();	/* Routine to set tt1 register */

/*  68030 MMU-specific constants  */

#define TT0_ENB 1
#define TT1_ENB 2

/*  MMB-specific constants  */

#define MMB_RP		0	/* Root pointer */
#define MMB_TCR		4	/* Translation Control Register */

/*  68030 MMU-TTx specific constants  */

#define MSK_TTRLAB	0xff000000	/* Mask for TTR Logical Address Base */

/*	The following constants are the values the MMB expects/allows
 *	for the associated tcr field assuming a 32 bit logical address.
 */

#define MMB_SRE		0	/* Supervisor root pointer not enabled */
#define MMB_FCL		1	/* Function code lookup required */
#define MMB_PS		0xa	/* Page size is 1K */
#define MMB_IS32	0	/* Logical address must be 32 bits */
#define MMB_TIA		7	/* Table index A */
#define MMB_TIB		7	/* Table index B */
#define MMB_TIC		8	/* Table index C */
#define MMB_TID		0	/* Table index D */

#define MMB_ENBL	0x80000000	/* enable MMB */

/* The fixed values for 32 bit operation are summed up in the macro below. */
#define MMB_FIXED	0x1a07780	/* Fixed values for 32 bit addresses */

#define mmb_enable()	mmbstore(MMB_TCR,(MMB_ENBL|MMB_FIXED))
#define mmb_settcr(T)	mmbstore(MMB_TCR,T)
#define mmb_setrp(RP)	mmbstore(MMB_RP,RP)

#define mmu_enable()	mmb_enable()
#define mmu_setup()	mmb_setrp(fcltbl)

typedef union ttr {
/*         transparent translation registers on 68030
 *
 *      31                    24 23                    16
 *       +----------------------+----------------------+
 *       | Logical Address Base | Logical Address Mask |
 *       +----------------------+----------------------+
 *                  8                       8
 *      15                     8 7                     0
 *       +----------------------+----------------------+
 *       |e|0|0|0|0| Ci |R/W|RWM|0| FC Base | FC Mask  |
 *       +----------------------+----------------------+
 *                  8                       8
 */
	struct {
		uint	tt_lab  : 8,	/* Logical Address Base */
			tt_lam	: 8,	/* Logical Address Mask */
			tt_e    : 1,	/* Enable */
			tt_z0   : 4,	/* Zero */
			tt_ci	: 1,	/* Cache Inhibit */
			tt_rw	: 1,	/* Read/Write */
			tt_rwm	: 1,	/* Read/Write Mask */
			tt_z1   : 1,	/* Zero */
			tt_fcb	: 3,	/* Function Code Base */
			tt_z2   : 1,	/* Zero */
			tt_fcm	: 3	/* Function Code Mask */
	} ttm;

	struct {
		uint	tt_ttr;	/* transparent translation register */
	} tti;
} ttr_t;

#define tt0_enable(AB,AM,CI,RW) tt0store(\
((AB)&MSK_TTRLAB)|(((AM)&MSK_TTRLAB)>>8)|((0x81|((CI)<<2)|((RW)<<1))<<8)|0x43)

#define tt1_enable(AB,AM,CI,RW) tt1store(\
((AB)&MSK_TTRLAB)|(((AM)&MSK_TTRLAB)>>8)|((0x81|((CI)<<2)|((RW)<<1))<<8)|0x43)

#else /* def CMMU */

/* #define P_CMMU_FIXED	0x82C0AA00	/* ena, no funcd, sre, 4K, is = 0 */

extern cmmu_store();	/* code in ML to store CMMU regs	*/
extern cmmuflush();	/* code in ML to flush ATCI		*/

#define CMMU_TCR 0	/* used for pmmu simulation of CMMU	*/
#define CMMU_URP 4	/* store CPU root pointer (USER)	*/
#define CMMU_SRP 8	/* store SUPV root pointer		*/

#endif /* CMMU */

/*	The following variables describe most memory managed by
**	the kernel.  Only certain I/O areas are not included.
*/

extern pde_t	*kptbl;		/* Kernel page table.  Used to	*/
				/* map a contiguous block of	*/
				/* kernel virtual space based	*/
				/* at SYSSEGS.			*/
extern int	kpbase;		/* The address of the start of	*/
				/* the first physical page of	*/
				/* memory above the kernel.	*/
				/* Physical memory from here to	*/
				/* the end of physical memory	*/
				/* is represented in the pfdat.	*/
extern int	maxmem;		/* Maximum available free	*/
				/* memory.			*/
extern int	freemem;	/* Current free memory.		*/
extern int	availrmem;	/* Available resident (not	*/
				/* swapable) memory in pages.	*/
extern int	availsmem;	/* Available swapable memory in	*/
				/* pages.			*/


/*
**	Implementation specific constants and variables.
*/

#define ALLFFFS 0xffffffff	/* Last 32 bit address */

#ifndef CMMU
#define SYSSEGS 0x04000000	/* Kernel virtual address of syssegs area */
#define FCLTSZ	8		/* size of function code lookup table */
#define UDTLIM	0		/* User domain table index limit */
#define SDTLIM	127		/* System domain table index limit */
#define USTLIM	127		/* User segment table index limit */
#define SSTLIM	127		/* System segment table index limit */
#define PGTLIM	255		/* Page table limit (default in pt_setup(P) ) */

/*	The following variables are used in tree walks of the
**	MMU translation tree.
*/

extern	sde_t *udtbl;		/* user domain table 		*/
extern	sde_t *sdtbl;		/* system domain table 		*/

#else /* def CMMU */

#define SYSSEGS 0x01000000	/* Kernel virtual address of syssegs area */

extern	sde_t *ustbl;		/* user segment table 		*/
extern	sde_t *sstbl;		/* system segment table 	*/

#endif

/*	Conversion macros
*/

/*	Get page number from system virtual address.  */

#define	svtop(X)	((uint)(X) >> PNUMSHFT) 

/*	Get system virtual address from page number.  */

#define	ptosv(X)	((uint)(X) << PNUMSHFT)


/*	These macros are used to map between kernel virtual
**	and physical address.  Note that all physical memory
**	within the same contiguous block as the kernel
**	is mapped into kernel virtual address space
**	at the actual physical address.
*/

#define kvtophys(kva) (((caddr_t)pg_getaddr(kvtopde(kva)))+poff(kva))
#define phystokv(paddr) (paddr)


/*	Between kernel virtual address and physical page frame number.
*/

#define pfntokv(pfn)   ((pfn) << PNUMSHFT)

#ifndef CMMU

/*	Between kernel virtual addresses and the kernel
**	domain table entry.
*/

#define	kvtodde(kva)  ((sde_t *)&(sdtbl[dnum(kva)]))


/*	Between kernel virtual addresses and the kernel
**	segment table entry.
*/

#define	kvtosde(kva) ((sde_t *)&((sde_t *)pt_getaddr(kvtodde(kva)))[snum(kva)])

#else

/*	Between kernel virtual addresses and the kernel
**	segment table entry.
*/

#define	kvtosde(kva) ((sde_t *)&(sstbl[snum(kva)]))

#endif

/*	Between kernel virtual addresses and the kernel
**	page table entry.
*/

#define	kvtopde(kva) ((pde_t *)&((pde_t *)pt_getaddr(kvtosde(kva)))[psnum(kva)*NPDEPP])


/*	Between kernel virtual addresses and the kernel syssegs page table. 
**	Only valid for addresses mapped by kptbl.
*/

#define	kvtokptbl(kva)	(&kptbl[btopt(((uint)(kva) - SYSSEGS))])


/*	Between kernel virtual address and its physical page frame.
**	Only valid for addresses mapped by kptbl.
*/

#define	kvtotfn(kva)	(pg_gettfn((pde_t *)kvtokptbl(kva)))


#ifndef CMMU
/*	Between current user virtual addresses and the current user
**	domain table entry.
*/

#define	uvtodde(uva)  (udtbl)	/* There is only one */


/*	Between current user virtual addresses and the current user
**	segment table entry.
*/

#define	uvtosde(uva) ((sde_t *)&((sde_t *)pt_getaddr(uvtodde(uva)))[snum(uva)])

#else

/*	Between current user virtual addresses and the current user
**	segment table entry.
*/

#define	uvtosde(uva) ((sde_t *)&(ustbl[snum(uva)]))

#endif

/*	Between current user virtual addresses and the current user page
**	table.
*/

#define	uvtopde(uva) ((pde_t *)&((pde_t *)pt_getaddr(uvtosde(uva)))[psnum(uva)*NPDEPP])


/*	The following routines are involved with the pfdat
**	table described in pfdat.h
*/

#define	pdetopfdat(pde)	(&pfdat[pg_getpfn(pde)])
#define	pfntopfdat(pfn)	(&pfdat[pfn])
#define	pfdattopfn(pfd)	(pfd - pfdat)
#define	pdetpdbd(pde)	(dpe + (NPGPT * NPDEPP))


/* flags used in ptmemall() call
*/

#define PHYSCONTIG 02
#define NOSLEEP    01

/* CPU function codes
*/
#define USERDATA	1
#define USERPROG	2
#define SUPVDATA	5
#define SUPVPROG	6
#define CPUSPACE	7

/*
 *	PMMU Status Register (PSR)
 */

#define	P_LVL	0x0007		/* number of levels */
#define	P_GSH	0x0080		/* globally sharable */
#define	P_GATE	0x0100		/* gate */
#define	P_MOD	0x0200		/* modified */
#define	P_INV	0x0400		/* invalid */
#define	P_WP	0x0800		/* write protected */
#define	P_ALV	0x1000		/* access level violation */
#define	P_SUP	0x2000		/* supervisor only */
#define	P_LIM	0x4000		/* limit violation */
#define	P_BE	0x8000		/* bus error */
