/*
 *	Copyright (c) 1987 Motorola Inc
 *	All Rights Reserved
 *		@(#)m68681.h	35.1
 */
/*	Copyright (c) 1984 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF AT&T	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ident	"@(#)m68681.h	35.1"

/*********************** COMPILER SWITCHES ***************************/

/* #define HOWFAR		printing debug messages  */
/* #define STATISTICS		do some statistics */
/* #define M136DETECT		when 136 board is different from 335 */
/* M136DETECT was an attempt to find out which board we were on by */
/* reading a register that is not defined on the 335. Now that we are */
/* using two drivers instead of one, we don't need it. */



/*  XREGS is defined for the 136 cpu card and NOT defined for the 335 card. */

#ifdef	XREGS
#define	m681ckdcd	x681ckdcd
#define	m681_sv_reg	x681_sv_reg
#define	m681_openduarts	x681_openduarts
#define	m681_speeds	x681_speeds
#define	m681_scanflg	x681_scanflg
#define	m681_work	x681_work
#define	m681init	x681init
#define	m681_cnt	x681_cnt
#define	m681_addr	x681_addr
#define	m681_ivec	x681_ivec
#define	m681open	x681open
#define	m681scan	x681scan
#define	m681_tty	x681_tty
#define	m681proc	x681proc
#define	m681param	x681param
#define	m681modem	x681modem
#define	m681close	x681close
#define	m681read	x681read
#define	m681write	x681write
#define	m681ioctl	x681ioctl
#define	m681intr	x681intr
#define	m681sendchar	x681sendchar
#define	m681limit	x681limit
#define m681wclose	x681wclose
#define m681break	x681break
#endif

/*********************** CONFIGURATION *******************************/
#define MAXBOARDS	6		/* max count of BOARDS */
#define MAXDUARTS	(MAXBOARDS*2)	/* max count of DUARTS */
#define MAXPORTS	(MAXDUARTS*2)	/* max count of PORTS  */

/*********************** DEFINITIONS *********************************/
#define	MISSING	0	/* flag, missing a configured DUART */
#define ON	1	/* flag for modem() */
#define OFF	0	/* flag for modem() */
#define A_PORT  0	/* flag for port a */
#define B_PORT  1	/* flag for port b */


/*********************** TYP DEFINTIONS *****************************/
typedef enum {
	FALSE,
	TRUE
} 
BOOL;


/*********************** MACROS ***********************************/
#define SPLCLOCK   spl6		/* set clock priority level */
#define SPLM681    spl5		/* set m681 driver priority level */

/* the access to the same command register must be delayed,
   because the chip has some hardware problems in this case */
#define DELAY_CR   { register i; for ( i=0; i<250; ++i ); }


/*********************** MC68681 DEFINITIONS ************************/
#define PORTOFFSET	0x10	/* offset for port b address space */

/* mode register 1: MR1x operations */
#define	RXRTS	0x80	/* enable receiver RTS */
#define PAREN	0x00	/* with parity */
#define PARDIS	0x10	/* no parity */
#define EVENPAR	0x00	/* even parity */
#define ODDPAR	0x04	/* odd parity */
#define CL5	0x00	/* 5 bits per char */
#define CL6	0x01	/* 6 bits per char */
#define CL7	0x02	/* 7 bits per char */
#define CL8	0x03	/* 8 bits per char */
#define PARMODEMASK 0x18  /* parity mode mask */
#define PARTYPEMASK 0x04  /* parity type mask */
#define CLMASK	    0x03  /* character length mask */

/* mode register 2: MR2x operations */
#define TXRTS	0x20	/* enable transmitter RTS */
#define TXCTS	0x10	/* enable transmitter CTS */
#define SB2	0x0f	/* 2 stop bits */
#define SB1	0x07	/* 1 stop bit */
#define SB1L5	0x00	/* 1 stop bit at 5 bits per character */

#define SBMASK	0x0f	/* stop bit mask */

/* clock-select register: CSRx operations */
#define NOBAUD	-1	/* 50 and 200 baud are not possible */
/* they are not in Baud register set 2 */
#define BD75	0x00	/* 75 baud */
#define BD110	0x11	/* 110 baud */
#define BD134	0x22	/* 134.5 baud */
#define BD150	0x33	/* 150 baud */
#define BD300	0x44	/* 300 baud */
#define BD600	0x55	/* 600 baud */
#define BD1200	0x66	/* 1200 baud */
#define BD1800	0xaa	/* 1800 baud */
#define BD2400	0x88	/* 2400 baud */
#define BD4800	0x99	/* 4800 baud */
#define BD9600	0xbb	/* 9600 baud */
#define BD19200	0xcc	/* 19200 baud */

#define DEFBAUD BD9600	/* default value if baudrate is not possible */


/* channel command register: CRx operations */
#define MRRESET 0x10	/* reset mr pointer to mr1 */
#define RXRESET 0x20	/* reset receiver */
#define TXRESET 0x30	/* reset transmitter */
#define ERRRESET 0x40	/* reset error status */
#define BRKINTRESET 0x50 /* reset channel's break interrupt */
#define BRKSTART 0x60	/* start break */
#define BRKSTOP 0x70	/* stop break */
#define	TXDIS	0x08	/* disable transmitter */
#define	TXEN	0x04	/* enable transmitter */
#define	RXDIS	0x02	/* disable receiver */
#define	RXEN	0x01	/* enable receiver */

/* status register: SRx status */
#define RBRK	0x80	/* received break */
#define FRERR	0x40	/* frame error */
#define PERR	0x20	/* parity error */
#define ROVRN	0x10	/* receiver overrun error */
#define TXEMT	0x08	/* transmitter empty */
#define TXRDY	0x04	/* transmitter ready */
#define FFULL	0x02	/* receiver FIFO full */
#define RXRDY	0x01	/* receiver ready */

/* output port configuration  register: OPCR operations */
#define	OPSET	0x00	/* set all op lines to op function */
#define	OPSETTO	0x04	/* use OP3 for timer output */

/* output port register: OP operations */
#define OPDTRB	0x20	/* DTR line output b  use OP5 */
#define OPDTRA	0x04	/* DTR line output a */
#define OPRTSB	0x02	/* RTS line output b */
#define OPRTSA	0x01	/* RTS line output a */

/* auxiliary control register: ACR operations */
#define BDSET1	0x00	/* baudrate generator set 1 */
#define BDSET2	0x80	/* baudrate generator set 2 */
#define CCLK1	0x60	/* timer clock: external rate */
#define CCLK16	0x30	/* counter clock: x1 clk divided by 16 */
#define	SLCTIM	0x7800	/* timer count to get 60 Hz time slice (16.6ms ticks) */

/* input port register: IP operations */
#define IPDCDIB 0x08	/* IP3 change == DCD input on port B of mvme136 */
#define IPDCDIA 0x04	/* IP2 change == DCD input on port A of mvme136 */

/* Defines for mvme335 */
#define IPDCDB	0x20 	/* DCD line input b */
#define IPDCDA	0x10  	/* DCD line input a */


#define IPDSRB  0x08	/* DSR line input b */
#define IPDSRA  0x04	/* DSR line input a */
#define IPCTSB  0x02	/* CTS line input b */
#define IPCTSA  0x01	/* CTS line input a */

/* interrupt status and mask register: ISR status and IMR mask */
#define IIPCHG	0x80	/* input port change */
#define IBRKB	0x40	/* delta break b */
#define IRXRDYB	0x20	/* receiver ready b */
#define ITXRDYB	0x10	/* transmitter ready b */
#define IBRKA	0x04	/* delta break a */
#define IRXRDYA	0x02	/* receiver ready a */
#define ITXRDYA	0x01	/* transmitter ready a */


/* interrupts from port a or b */
#define AINTPORT	( IRXRDYA | ITXRDYA )
#define BINTPORT	( IRXRDYB | ITXRDYB )


/* HW write register index for m681_wr_regs[] */
#define MR1A	0	/* mode register 1 a */
#define CSRA    1	/* clock-select register a*/
#define CRA	2	/* command register a */
#define TBA	3	/* transmitter buffer a */
#define ACR	4	/* auxialiary control register*/
#define IMR	5	/* interrupt mask register */
#define CTUR	6	/* counter/timer upper reg */
#define	CTLR	7 	/* counter/timer lower reg */
#define MR1B	8	/* mode register 1 b */
#define CSRB	9	/* clock-select register b*/
#define CRB     10	/* command register b */
#define TBB	11	/* transmitter buffer b */
#define IVR	12	/* interrupt vector register */
#define OPCR	13	/* output port config reg */
#define OPRSET  14	/* output port: bit set cmd */
#define OPRRESET 15	/* output port: bit reset cmd */
#define MR2A	16	/* mode register 2 a */
#define MR2B	17	/* mode register 2 b */
#define MAXREG	18	/* max count of registers */

#ifdef	XREGS

/*
 *	Structure of MC68681 hardware read registers.
 */
struct m681_rd_reg
{
	unsigned char	rd_mra;		/* mode register a */
	unsigned char	rd_sra;		/* status register a*/
	unsigned char	dummy5;		/* do not access */
	unsigned char	rd_rba;		/* receiver buffer a */
	unsigned char	rd_ipcr;	/* input port change register */
	unsigned char	rd_isr;		/* interrupt status register */
	unsigned char	rd_cur;		/* current MSB of counter */
	unsigned char	rd_clr;		/* current LSB of counter */
	unsigned char	rd_mrb;		/* mode register b */
	unsigned char	rd_srb;		/* status register b*/
	unsigned char	dummyxb;	/* do not access */
	unsigned char	rd_rbb;		/* receiver buffer b */
	unsigned char	rd_ivr;		/* interrupt vector register */
	unsigned char	rd_ip;		/* input port (unlatched) */
	unsigned char	rd_ctstart;	/* start-counter command */
	unsigned char	rd_ctstop;	/* stop-counter command */
};

/*
 *	Structure of MC68681 hardware write registers.
 */
struct m681_wr_reg
{
	unsigned char	wr_mra;		/* mode register a */
	unsigned char	wr_csra;	/* clock-select register a*/
	unsigned char	wr_cra;		/* command register a */
	unsigned char	wr_tba;		/* transmitter buffer a */
	unsigned char	wr_acr;		/* auxialiary control register*/
	unsigned char	wr_imr;		/* interrupt mask register */
	unsigned char	wr_ctur;	/* counter/timer upper reg */
	unsigned char	wr_ctlr;	/* counter/timer lower reg */
	unsigned char	wr_mrb;		/* mode register b */
	unsigned char	wr_csrb;	/* clock-select register b*/
	unsigned char	wr_crb;		/* command register b */
	unsigned char	wr_tbb;		/* transmitter buffer b */
	unsigned char	wr_ivr;		/* interrupt vector register */
	unsigned char	wr_opcr;	/* output port config reg */
	unsigned char	wr_oprset;	/* output port: bit set cmd */
	unsigned char	wr_oprreset;	/* output port: bit reset cmd */
};

#else

/*
 *	Structure of MC68681 hardware read registers.
 */
struct m681_rd_reg
{
	unsigned char	dummy0;		/* start at odd address */
	unsigned char	rd_mra;		/* mode register a */
	unsigned char	dummy2;
	unsigned char	rd_sra;		/* status register a*/
	unsigned char	dummy4;
	unsigned char	dummy5;		/* do not access */
	unsigned char	dummy6;
	unsigned char	rd_rba;		/* receiver buffer a */
	unsigned char	dummy8;
	unsigned char	rd_ipcr;	/* input port change register */
	unsigned char	dummy10;
	unsigned char	rd_isr;		/* interrupt status register */
	unsigned char	dummy12;
	unsigned char	rd_cur;		/* current MSB of counter */
	unsigned char	dummy14;
	unsigned char	rd_clr;		/* current LSB of counter */
	unsigned char	dummy16;
	unsigned char	rd_mrb;		/* mode register b */
	unsigned char	dummy18;
	unsigned char	rd_srb;		/* status register b*/
	unsigned char	dummy20;
	unsigned char	dummyxb;	/* do not access */
	unsigned char	dummy22;
	unsigned char	rd_rbb;		/* receiver buffer b */
	unsigned char	dummy24;
	unsigned char	rd_ivr;		/* interrupt vector register */
	unsigned char	dummy26;
	unsigned char	rd_ip;		/* input port (unlatched) */
	unsigned char	dummy28;
	unsigned char	rd_ctstart;	/* start-counter command */
	unsigned char	dummy30;
	unsigned char	rd_ctstop;	/* stop-counter command */
};

/*
 *	Structure of MC68681 hardware write registers.
 */
struct m681_wr_reg
{
	unsigned char	dummy0;		/* start at odd address */
	unsigned char	wr_mra;		/* mode register a */
	unsigned char	dummy2;
	unsigned char	wr_csra;	/* clock-select register a*/
	unsigned char	dummy4;
	unsigned char	wr_cra;		/* command register a */
	unsigned char	dummy6;
	unsigned char	wr_tba;		/* transmitter buffer a */
	unsigned char	dummy8;
	unsigned char	wr_acr;		/* auxialiary control register*/
	unsigned char	dummy10;
	unsigned char	wr_imr;		/* interrupt mask register */
	unsigned char	dummy12;
	unsigned char	wr_ctur;	/* counter/timer upper reg */
	unsigned char	dummy14;
	unsigned char	wr_ctlr;	/* counter/timer lower reg */
	unsigned char	dummy16;
	unsigned char	wr_mrb;		/* mode register b */
	unsigned char	dummy18;
	unsigned char	wr_csrb;	/* clock-select register b*/
	unsigned char	dummy20;
	unsigned char	wr_crb;		/* command register b */
	unsigned char	dummy22;
	unsigned char	wr_tbb;		/* transmitter buffer b */
	unsigned char	dummy24;
	unsigned char	wr_ivr;		/* interrupt vector register */
	unsigned char	dummy26;
	unsigned char	wr_opcr;	/* output port config reg */
	unsigned char	dummy28;
	unsigned char	wr_oprset;	/* output port: bit set cmd */
	unsigned char	dummy30;
	unsigned char	wr_oprreset;	/* output port: bit reset cmd */
};

#endif

union m681_io {
	struct m681_rd_reg	read;		/* read registers */
	struct m681_wr_reg	write;		/* write registers */
};

#ifdef	XREGS


/*
 *	Structure of MC68681 hardware port dependent read registers.
 */
struct m681_prd_reg
{
	unsigned char	rd_mr;		/* mode register */
	unsigned char	rd_sr;		/* status register */
	unsigned char	dummy5;
	unsigned char	rd_rb;		/* receiver buffer */
	unsigned char	dummy9;
	unsigned char	dummy11;
	unsigned char	dummy13;
	unsigned char	dummy15;
};

/*
 *	Structure of MC68681 hardware port dependent write registers.
 */
struct m681_pwr_reg
{
	unsigned char	wr_mr;		/* mode register */
	unsigned char	wr_csr;		/* clock-select register */
	unsigned char	wr_cr;		/* command register */
	unsigned char	wr_tb;		/* transmitter buffer */
	unsigned char	dummy9;
	unsigned char	dummy11;
	unsigned char	dummy13;
	unsigned char	dummy15;
};

#else

/*
 *	Structure of MC68681 hardware port dependent read registers.
 */
struct m681_prd_reg
{
	unsigned char	dummy0;		/* start at odd address */
	unsigned char	rd_mr;		/* mode register */
	unsigned char	dummy2;
	unsigned char	rd_sr;		/* status register */
	unsigned char	dummy4;
	unsigned char	dummy5;
	unsigned char	dummy6;
	unsigned char	rd_rb;		/* receiver buffer */
	unsigned char	dummy8;
	unsigned char	dummy9;
	unsigned char	dummy10;
	unsigned char	dummy11;
	unsigned char	dummy12;
	unsigned char	dummy13;
	unsigned char	dummy14;
	unsigned char	dummy15;
};

/*
 *	Structure of MC68681 hardware port dependent write registers.
 */
struct m681_pwr_reg
{
	unsigned char	dummy0;		/* start at odd address */
	unsigned char	wr_mr;		/* mode register */
	unsigned char	dummy2;
	unsigned char	wr_csr;		/* clock-select register */
	unsigned char	dummy4;
	unsigned char	wr_cr;		/* command register */
	unsigned char	dummy6;
	unsigned char	wr_tb;		/* transmitter buffer */
	unsigned char	dummy8;
	unsigned char	dummy9;
	unsigned char	dummy10;
	unsigned char	dummy11;
	unsigned char	dummy12;
	unsigned char	dummy13;
	unsigned char	dummy14;
	unsigned char	dummy15;
};

#endif

union m681_pt_io {
	struct m681_prd_reg	read;		/* read registers */
	struct m681_pwr_reg	write;		/* write registers */
};

/*
 *	Structure of saved registers
 */
struct m681_sv_reg
{
	unsigned char	sv_mr1[2];	/* mode register 1 a */
	unsigned char	sv_mr2[2];	/* mode register 2 a */
	unsigned char	sv_csr[2];	/* clock-select register a*/
	unsigned char	sv_cr[2];		/* command register a */
	unsigned char	sv_acr;		/* auxialiary control register*/
	unsigned char	sv_imr;		/* interrupt mask register */
	unsigned char	sv_ivr;		/* interrupt vector register */
};

/* defines for receiver and DCD scanner */
#define M681CSCAN 2	/* clock ticks for character scan */
#define M681_CHAR 01	/* character arrived flag (m681_work) */
#define M681_ACT  02	/* character in buffer flag (m681_work) */
#define M681_DCD  04	/* DCD wait flag (m681_work) */



/********************* SPECIAL IOCTL COMMAND DEFINITIONS ******************/
/* HWHAND ioctls same as in MVME331 driver */
#define MSETHWHAND	(('m'<<8)+20)	/* set hardware handshake */
#define MCLEARHWHAND	(('m'<<8)+21)	/* clear hardware handshake */
#define MGETHWHAND	(('m'<<8)+22)	/* get hardware handshake mode */

#define M681STATUS	(('m'<<8)+40 )  /* get status of duarts */
#define M681GETSTAT	(('m'<<8)+42 )  /* get statistics */

struct debugioctl {
	char *buf;	/* debug buffer start */
	int  size;	/* size of debug buffer */
	char *pt;	/* current pointer in debug buffer */
};

#ifdef STATISTICS
struct m681_stat {
	int open_cnt;	/* count of open calls */
	int intr_cnt;	/* count of interrupts */
	int tx_cnt;	/* count of transmitted characters */
	int rx_cnt;	/* count of received characters */
	int brk_cnt;	/* count of break events */
	int rovrn_cnt;	/* count of overruns */
	int frerr_cnt;	/* count of frame errors */
	int perr_cnt;	/* count of parity errors */
	int rxoff_cnt;	/* count of received XOFF chars */
	int rxon_cnt;	/* count of received XON  chars */
	int txoff_cnt;	/* count of transmitted XOFF chars */
	int txon_cnt;	/* count of transmitted XON  chars */
};
#endif


/****************** DEFINITIONS FOR DEBUG PRINTING *****************/
#ifdef HOWFAR

/* DEBUG levels */
#define LOG_START	10	/* start a subroutine */
#define LOG_RETURN	10	/* return form a subroutine */
#define	LOG_WARN	 5	/* print a warning message */
#define LOG_INTERN	20	/* debug messages in a function */
#define LOG_MAX		20	/* highest debug level */

#define LOG( level, message ) 			\
    if ( (m681_debug&0xff) >= level ) {		\
	    printf message ;	 		\
    }
#else
#define LOG( level, message ) {} /* do nothing */
#endif

/*********************** EXTERNALS ***********************************/
/* Declarations in config.c */
extern int m681_cnt;		/* count of serial ports */
extern int m681_ivec[];		/* interrupt vectors */
extern union m681_io *m681_addr[MAXDUARTS];	/* address table */
extern struct tty m681_tty[];	/* tty structures */

/********************************************************************/
