/*	@(#)macs323.h	35.1	*/
/*
 *	Copyright (c) 1988	Motorola Inc.  
 *		All Rights Reserved.
 */
/*
 *	macs323.h: include file for Motorola MVME323 disk controller
 * 	with MACSI (Multiple Active Command Software Interface) firmware.
 */

/*
 * 	Controller Initialization Block:
 */

#define NQENT	8
#define NQCMDS 63 
#define RNDROBCT 18	/* round robin count */
#define BBWTCNT 18	/* back to back write count */

struct  m323cib {
/*  0 */ ushort		cib_ncqe;	/* Number of Command Queue Entries */
/*  2 */ ushort		cib_crb;	/* Offset of CRB (not checked) */
/*  4 */ ushort		cib_flags;	/* MACSI flags */
/*  6 */ ushort		cib_nwq;	/* Number of commands to queue */
/*  8 */ ushort		cib_rsvr1;	/* reserved, must be zero. */
/*  A */ ushort		cib_rsvr2;	/* reserved, must be zero. */
/*  C */ ushort 	cib_bbwc;	/* back to back write count */
/*  E */ ushort		cib_rrc;	/* round robin count */
/*  0x10 */ ushort	cib_rsvr3;	/* reserved, must be zero. */
};

/*
 *	Command Queue Entry Format:
 */

struct	m323cqent {
/*  0 */ ushort		qe_creg; 	/* Queue Entry Control Register */
/*  2 */ ushort		qe_pbaddr; 	/* Queue Entry IOPB Address */
/*  4 */ char		*qe_cmdtag;	/* Queue Entry Command Tag */
/*  8 */ unsigned char	qe_pblen;	/* Queue Entry IOPB Length */
/*  9 */ unsigned char	qe_wqnum;	/* Queue Entry Work Queue Number */
/*  A */ ushort 	qe_res0;	/* Reserved */
};

/*
 *	Command Response Block Format:
 */

struct	m323crb {
/*  0 */ ushort		csr; 		/* Command response status register */
/*  2 */ ushort		r0; 		/* Reserved */
/*  4 */ char		*cmdtag;	/* Command Tag */
/*  8 */ unsigned char	pblen;		/* Queue Entry IOPB Length */
/*  9 */ unsigned char	wqnum;		/* Queue Entry Work Queue Number */
/*  A */ ushort		r1;		/* Reserved */
/*  C */ struct m323iopb  rtn_iopb;	/* Return IOPB */
};
/*
 * Short I/O Memory Map:
 */

struct	m323map {
/*  0 */ ushort		mm_msr;		/* Master Status Register */
/*  2 */ ushort		mm_mcr;		/* Master Control Register */
/*  4 */ unsigned char	mm_iliqar;	/* Interupt level for IAQR */
/*  5 */ unsigned char	mm_iviqar;	/* Interupt vector for IAQR */
/*  6 */ ushort		mm_qhp;		/* Queue Head Pointer */
/*  8 */ ushort		mm_res0;	/* Reserved */
/*  A */ ushort		mm_res1;	/* Reserved */
/*  C */ ushort		mm_res2;	/* Reserved */
/*  E */ ushort		mm_res3;	/* Reserved */
/* 10 */ struct m323cqent mm_cqe[8];	/* Eight Command Queue Entries */
/* 70 */ struct m323iopb  mm_iopbs[8];	/* Eight IOPBs */
/*170 */ ushort		mm_fill[27];	/* 54 bytes of filler */
/*1A6 */ struct m323crb	mm_crb;		/* Command Response Block */
};

/*
 *	Controller Initialization Block, FLAGS bit assignments:
 */

#define CIBF_OS		0x02	/* Enable Overlapped Seeks */
#define CIBF_RES	0x04	/* Enable auto-restore of drive faults */

/*
 *	Master Control Status Register, bit assignments:
 *	(bits 3 - 15 reserved)
 */

#define MCSR_CNA	0x01	/* Controller not available */
#define MCSR_BOK	0x02	/* Board Okay, passed diagnostics */
#define MCSR_QFC	0x04	/* Queue Flush Complete */

/* 
 * Interupt on Queue Available Register, bit assignments:
 */

#define IQEA	0x80

/*
 *	Master Control Register, bit assignments:
 *	(bits 1, 3 - 10, 12, 14,15 reserved)
 */

#define MCR_SQM		0x01	/* Start Queue Mode */
#define MCR_FLQR	0x04	/* Flush Queue and Report */
#define MCR_FLQ		0x800	/* Flush Queue, no report */
#define MCR_SFEN	0x2000  /* Sysfail Enable */

/*
 *	Queue Entry Control Register, bit assignments:
 *	(bits 3 - 7, 12 -15 reserved)
 *	(bits 8 - 11 will be used to set iopb type)
 */

#define QECR_GOBSY	0x01	/* Set to initiate action on queue entry */
#define QECR_AA		0x02	/* Abort Acknowledge */
#define QECR_HPC	0x04	/* High Priority Command */
#define QECR_SORT	0x80	/* Sort IOPBs */
#define QECR_MYST	0x100	/* Mystery bit */

/*
 *	Command Response Status Word, bit assignments:
 *	(bits 7 -13 reserved)
 */

#define CRS_CRBV	0x01	/* Command Response Block Valid/Cear Intr */
#define CRS_CC		0x02	/* Command Complete */
#define CRS_ER		0x04	/* Command completed with error */
#define CRS_EX		0x08	/* Command completed with exception */
#define CRS_ABORT	0x10	/* Aborted all work queues after error */
#define CRS_QMS		0x20	/* Queue mode started */
#define CRS_CQEA	0x40	/* Command Queue Entry Available */

