/*
 *	Copyright (c) 1987 Motorola Inc
 *	All Rights Reserved
 *	@(#)mc88200.h	35.1
 */

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF Motorola	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ident	"@(#)mc88200.h	35.1"

/*  Field definitions for the MC88200 CMMU                       */

/*  Naming Convention:
	Identifiers without prefixes are 32 bit addresses
		eg. BASE

	Identifiers prefixed by R_ are register offsets from the base.
		eg. R_CMMU_ID

	Identifiers prefixed by B_ are bit numbers (0 = LSB)
		eg. B_SSR_BE

	Identifiers prefixed by F_ are field masks, ie an AND will select the
	field.
		eg. F_SSR_PROBE

	Identifiers prefixed by S_ are shift values used to align fields.
		eg. S_BATC_PBA
*/

#define BASE		0xfff00000	/* base address of all CMMU's */
#define AFCBASE		BASE+0x6f000	/* base of code CMMU (ANGELSFIRE) */
#define AFDBASE		BASE+0x7f000	/* base of data CMMU (ANGELSFIRE) */

/*			Register Offsets		*/

#define R_CMMU_ID	0x000		/* ID Register */
#define R_CMMU_SADR	0x004		/* System Address Register */
#define R_CMMU_SCR	0x008		/* System Control Register */
#define R_CMMU_SSR	0x00c		/* System Status Register */
#define R_CMMU_LADR	0x100		/* Local Address Register */
#define R_CMMU_LCR	0x104		/* Local Control Register */
#define R_CMMU_LSR	0x108		/* Local Status Register */
#define R_CMMU_SAPR	0x200		/* Supervisor Area Pointer Reg */
#define R_CMMU_UAPR	0x204		/* User Area Pointer Reg */
#define R_CMMU_BWP0	0x400		/* Block ATC Write Port 0 */
#define R_CMMU_BWP1	0x404		/* Block ATC Write Port 1 */
#define R_CMMU_BWP2	0x408		/* Block ATC Write Port 2 */
#define R_CMMU_BWP3	0x40c		/* Block ATC Write Port 3 */
#define R_CMMU_BWP4	0x410		/* Block ATC Write Port 4 */
#define R_CMMU_BWP5	0x414		/* Block ATC Write Port 5 */
#define R_CMMU_BWP6	0x418		/* Block ATC Write Port 6 */
#define R_CMMU_BWP7	0x41c		/* Block ATC Write Port 7 */
#define R_CMMU_CDR0	0x800		/* Cache data register 0 */
#define R_CMMU_CDR1	0x804		/* Cache data register 1 */
#define R_CMMU_CDR2	0x808		/* Cache data register 2 */
#define R_CMMU_CDR3	0x80c		/* Cache data register 3 */
#define R_CMMU_CTR0	0x840		/* Cache tag register 0 */
#define R_CMMU_CTR1	0x844		/* Cache tag register 1 */
#define R_CMMU_CTR2	0x848		/* Cache tag register 2 */
#define R_CMMU_CTR3	0x84c		/* Cache tag register 3 */
#define R_CMMU_CLKVR	0x880		/* Cache Set Status Register */

/*			Field Definitions		*/

/*	Local Control Register		*/

#define B_LCR_PE	2		/* Parity Enable */
#define B_LCR_SE	1		/* Snoop Enable */
#define B_LCR_PR	0		/* Priority Arbitration */

/*	Local Status Register		*/

#define B_LSR_SF	20		/* Segment Fault Exception */
#define B_LSR_PF	19		/* Page Fault Exception */
#define B_LSR_SV	18		/* Supervisor Privilege Exception */
#define B_LSR_WV	17		/* Write Protection Exception */
#define B_LSR_BE	16		/* Bus Error Exception */

/*	ID Register			*/

#define F_ID_ID		0xff000000	/* ID Field */
#define F_ID_TYPE	0x00e00000	/* Type of component (= 0x5) */
#define F_ID_VERS	0x001f0000	/* Silicon Version Field */

/*	System Status Register		*/

#define B_SSR_BE	15		/* M-Bus error/data cache flush error */
#define F_SSR_PROBE	0x000003ff	/* Probe status mask */
#define B_SSR_WT	9		/* Write-though update caching */
#define B_SSR_SP	8		/* Supervisor privilege required */
#define B_SSR_G		7		/* Global */
#define B_SSR_CI	6		/* Cache inhibit */
#define B_SSR_MOD	4		/* Data modified */
#define B_SSR_USED	3		/* Data has been used */
#define B_SSR_WP	2		/* Write protected */
#define B_SSR_BH	1		/* Hit in BATC */
#define B_SSR_V		0		/* Valid */

/*	BATC Write Ports			*/

#define F_BATC_LBA	0xfff80000	/* Logical block address */
#define F_BATC_PBA	0x0007ffc0	/* Physical block address */
#define S_BATC_PBA	19		/* Right Shift value for PBA */
#define B_BATC_S	5		/* Supervisor address */
$define B_BATC_WT	4		/* Write though on updates */
#define B_BATC_G	3		/* Global */
#define B_BATC_CI	2		/* Cache inhibit */
#define B_BATC_WP	1		/* Write protected */
#define B_BATC_V	0		/* Valid */

/*	Supervisor/ User Area Pointer Registers		*/

/*  Definitions are as for the Area Descriptor  (see below)   */

/*				DESCRIPTORS			*/

/*		Area Descriptor			*/

#define F_AD_ADDR	0xfffff000	/* Segment Table Base Address */
#define B_AD_WT		9		/* Write through on update */
#define B_AD_G		7		/* Global */
#define B_AD_CI		6		/* Cache inhibit */
#define B_AD_TE		0		/* Translation Enable */

/*		Segment Descriptor		*/

#define F_SD_ADDR	0xfffff000	/* Page Table Base Address */
#define B_SD_WT		9		/* Write through on update */
#define B_SD_SP		8		/* Supervisor protected */
#define B_SD_G		7		/* Global */
#define B_SD_CI		6		/* Cache inhibit */
#define B_SD_WP		2		/* Write protected */
#define B_SD_TE		0		/* Translation Enable */

/*		Page Descriptor			*/

#define F_PD_ADDR	0xfffff000	/* Page Frame Address */
#define B_PD_WT		9		/* Write through on update */
#define B_PD_SP		8		/* Supervisor protected */
#define B_PD_G		7		/* Global */
#define B_PD_CI		6		/* Cache inhibit */
#define B_PD_M		4		/* Modified  */
#define B_PD_U		3		/* Used */
#define B_PD_WP		2		/* Write protected */
#define B_PD_TE		0		/* Translation Enable */

/*	Commands for writing to System Control Register		*/

#define C_SCR_IUPD	0x00000030	/* Invalidate user PATC descriptors */
#define C_SCR_ISPD	0x00000034	/* Invalidate supv PATC descriptors */
#define C_SCR_DCI	0x00000014	/* Invalidate data cache */
#define C_SCR_DCW	0x00000018	/* Data cache write to memory */
#define C_SCR_DCWI	0x0000001c	/* Data cache write and invalidate */

/*	Granularity Modifiers to above		*/

#define GG_LINE		0		/* Line */
#define GG_PAGE		1		/* Page */
#define GG_SEGM		2		/* Segment */
#define GG_ALL		3		/* All */
