/*		Copyright (c) 1985 AT&T		*/
/*		All Rights Reserved		*/
 /*		@(#)mfp.h	35.1			*/
 /*
  *
  *	Definitions/declarations for MC68901 Multi-Function Peripheral
  *
  */

#define CTSOFF          0x02            /* module control register         */

/* MC68901 interrupt vector register definitions */
#define SOFTEOI		0x08		/* end-of-interrupt mode bit	 */
#define UVECTOR		0xf0		/* user-supplied vector 	 */

/* MC68901 interrupt control definitions (A registers) */
#define ITIMERB         0x01            /* timer B                       */
#define ITXERR          0x02            /* transmit error                */
#define ITXRDY          0x04            /* transmitter buffer empty      */
#define IRXERR          0x08            /* receive error                 */
#define IRXRDY          0x10            /* receiver buffer full          */
#define ITIMERA         0x20            /* timer A                       */
#define IGPIP6		0x40		/* gpip6			 */
#define IGPIP7		0x80		/* gpip7			 */

/* MC68901 interrupt control definitions (B registers) */
#define	IGPIP0		0x01		/* gpip0			 */
#define IGPIP1		0x02		/* gpip1			 */
#define IGPIP2		0x04		/* gpip2			 */
#define IGPIP3		0x08		/* gpip3			 */
#define ITIMERD		0x10		/* timer D			 */
#define ITIMERC		0x20		/* timer C			 */
#define IGPIP4		0x40		/* gpip4			 */
#define IGPIP5		0x80		/* gpip5			 */

/* MC68901 definitions for GPIP control and data registers */
#define GPIP0		0x01		/* gpip0			 */
#define GPIP1		0x02		/* gpip0			 */
#define GPIP2		0x04		/* gpip0			 */
#define GPIP3		0x08		/* gpip0			 */
#define GPIP4		0x10		/* gpip0			 */
#define GPIP5		0x20		/* gpip0			 */
#define GPIP6		0x40		/* gpip0			 */
#define GPIP7		0x80		/* gpip0			 */

/* MC68901 timers A and B mode definitions */
#define ABDIV4             0x01    /* delay mode, divide by 04  prescaler   */ 
#define ABDIV10            0x02    /* delay mode, divide by 10  prescaler   */ 
#define ABDIV16            0x03    /* delay mode, divide by 16  prescaler   */ 
#define ABDIV50            0x04    /* delay mode, divide by 50  prescaler   */ 
#define ABDIV64            0x05    /* delay mode, divide by 64  prescaler   */ 
#define ABDIV100           0x06    /* delay mode, divide by 100 prescaler   */ 
#define ABDIV200           0x07    /* delay mode, divide by 200 prescaler   */ 
#define EVNTCNT		   0x08	   /* event count mode			    */
#define PWDIV4		   0x09	   /* pulse width mode, divide by 04 ps     */
#define PWDIV10		   0x0a	   /* pulse width mode, divide by 10 ps     */
#define PWDIV16		   0x0b	   /* pulse width mode, divide by 16 ps     */
#define PWDIV50		   0x0c	   /* pulse width mode, divide by 50 ps     */
#define PWDIV64		   0x0d	   /* pulse width mode, divide by 64 ps     */
#define PWDIV100	   0x0e	   /* pulse width mode, divide by 100 ps    */
#define PWDIV200	   0x0f	   /* pulse width mode, divide by 200 ps    */
#define RESETABO	   0x10	   /* reset A or B output lines		    */
#define TMABMODE	   0x0f    /* timers A and B mode bits              */

/* MC68901 timer C mode definitions */
#define CDIV4              0x10    /* delay mode, divide by 04  prescaler   */ 
#define CDIV10             0x20    /* delay mode, divide by 10  prescaler   */ 
#define CDIV16             0x30    /* delay mode, divide by 16  prescaler   */ 
#define CDIV50             0x40    /* delay mode, divide by 50  prescaler   */ 
#define CDIV64             0x50    /* delay mode, divide by 64  prescaler   */ 
#define CDIV100            0x60    /* delay mode, divide by 100 prescaler   */ 
#define CDIV200            0x70    /* delay mode, divide by 200 prescaler   */ 
#define TMCMODE            0x70    /* timer C mode bits                     */

/* MC68901 timer D mode definitions */
#define DDIV4              0x01    /* delay mode, divide by 04  prescaler   */ 
#define DDIV10             0x02    /* delay mode, divide by 10  prescaler   */ 
#define DDIV16             0x03    /* delay mode, divide by 16  prescaler   */ 
#define DDIV50             0x04    /* delay mode, divide by 50  prescaler   */ 
#define DDIV64             0x05    /* delay mode, divide by 64  prescaler   */ 
#define DDIV100            0x06    /* delay mode, divide by 100 prescaler   */ 
#define DDIV200            0x07    /* delay mode, divide by 200 prescaler   */ 
#define TMDMODE            0x07    /* timer D mode bits                     */

/* MC68901 USART control register (UCR) definitions */
#define NULLREG         0x01	/* not used 				    */
#define PAREVEN         0x02	/* even parity 				    */
#define PARENAB         0x04	/* parity enable 			    */
#define SYNCH           0x00    /* synchronous mode (bits 3 and 4 = 0) 	    */
#define ASYNST1         0x08    /* asynchronous - 1   stop bit  	    */
#define ASYNST15        0x10    /* asynchronous - 1.5 stop bits 	    */
#define ASYNST2         0x18    /* asynchronous - 2   stop bits 	    */
#define WORDLEN8        0x00    /* 8 bits per word 			    */
#define WORDLEN7        0x20    /* 7 bits per word 			    */
#define WORDLEN6        0x40    /* 6 bits per word 			    */
#define WORDLEN5        0x60    /* 5 bits per word 			    */
#define CLOCK16         0x80    /* clock divided by 16 			    */

/* MC68901 receiver status register (RSR) definitions */
#define RXENAB          0x01    /* receiver enable 			    */
#define SYNCSTRP        0x02    /* synchronous strip enable 		    */
#define WORDASM         0x04    /* match/character in progress 		    */
#define RXBREAK         0x08    /* break detected 			    */
#define FRAMERR         0x10    /* frame error 				    */
#define PARERR          0x20    /* parity error 			    */
#define OVERERR         0x40    /* overrun error 			    */
#define RXRDY           0x80    /* receive buffer full 			    */

/* MC68901 transmitter status register (TSR) definitions */
#define TXENAB          0x01    /* transmitter enable 			    */
#define TXOUTHZ         0x00    /* disabled tx output - high impedance 	    */
#define TXOUTLO         0x02    /* disabled tx output - low 		    */
#define TXOUTHI         0x04    /* disabled tx output - high 		    */
#define TXOUTLP         0x06    /* disabled tx output - loopback 	    */
#define TXBREAK         0x08    /* transmit break 			    */
#define TXEOT           0x10    /* end of transmission 			    */
#define TXAUTO          0x20    /* auto turnaround 			    */
#define UNDERR          0x40    /* underrun error 			    */
#define TXRDY           0x80    /* transmit buffer empty 		    */ 

/*
 *      MC68901 Register Allocation    
 *      Assumes an odd starting address.
 */

struct mfp_io
{
        unsigned char   gpip;   /* general-purpose i/o port   */
        unsigned char   fill0;
	unsigned char 	aer;	/* active edge register       */
        unsigned char   fill1;
	unsigned char	ddr;	/* data direction register    */
        unsigned char   fill2;
        unsigned char   iera;   /* interrupt enable reg A     */
        unsigned char   fill3;
        unsigned char   ierb;   /* interrupt enable reg B     */
        unsigned char   fill4;
        unsigned char   ipra;   /* interrupt pending reg A    */
        unsigned char   fill5;
        unsigned char   iprb;   /* interrupt pending reg B    */
        unsigned char   fill6;
        unsigned char   isra;   /* interrupt in-service reg A */
        unsigned char   fill7;
        unsigned char   isrb;   /* interrupt in-service reg B */
        unsigned char   fill8;
	unsigned char 	imra;	/* interrupt mask register A  */
        unsigned char   fill9;
	unsigned char 	imrb;	/* interrupt mask register B  */
        unsigned char   filla;
	unsigned char	vr;	/* interrupt vector register  */
        unsigned char   fillb;
	unsigned char 	tacr;	/* timer A control register   */
        unsigned char   fillc;
	unsigned char 	tbcr;	/* timer B control register   */
        unsigned char   filld;
        unsigned char   tcdcr;  /* timers C and D control reg */
        unsigned char   fille;
       	unsigned char 	tadr;	/* timer A data register      */ 
        unsigned char   fillf;
       	unsigned char 	tbdr;	/* timer B data register      */ 
        unsigned char   fill10;
       	unsigned char 	tcdr;	/* timer C data register      */ 
        unsigned char   fill11;
       	unsigned char 	tddr;	/* timer D data register      */ 
        unsigned char   fill12;
	unsigned char 	scr;	/* synchronous character reg  */
        unsigned char   fill13;
        unsigned char   ucr;    /* USART control register     */
        unsigned char   fill14;
        unsigned char   rsr;    /* receiver status register   */
        unsigned char   fill15;
        unsigned char   tsr;    /* transmitter status reg     */
        unsigned char   fill16;
        unsigned char   udr;    /* USART data register        */
};

