/*
 *	Copyright (c) 1987 Motorola Inc
 *	All Rights Reserved
 *	@(#)mvme136.h	35.1
 */

/*
 *	Status and Control registers for the MVME136 CPU
 */

/*
 *	Definition of status regsiter 1 (STAT1):
 *	Access to this register reads Port A of the Z8036 timer chip.
 *	This register reflects the settings of the front panel switch S4 and
 *	is defined by the 136bug.  See the 136bug manual for details.
 */

#define STAT1   ((char*)0xfffb0031)     /* Status register 1            */

#define	STAT1_S43	0x80		/* ENV0, 0=ON=136bug at base 0	*/
#define	STAT1_S44	0x40		/* ENV1, 0=ON=136bug at base 0	*/
#define	STAT1_S45	0x20		/* D32,  0=ON=32 bit data VMEbus*/
#define	STAT1_S46	0x10		/* A32,  0=ON=32 bit addr VMEbus*/
#define	STAT1_S47	0x08		/* VSBD, 0=ON=Bounce mode select*/
#define	STAT1_S48	0x04		/* VSBEN,0=ON=VSB enabled	*/
#define	STAT1_S49	0x02		/* MPSUP,0=ON=don't poll MP bit	*/
#define	STAT1_S410	0x01		/* BOOT, 0=ON=Select manual boot*/

/*
 *	Definition of status register 2 (STAT2)
 */

#define	STAT2	((char*)0xfffb0032)	/* Status Register 2		*/

#define	STAT2_RAMERR	0x80		/* RAM error			*/
#define	STAT2_RMCERR	0x40		/* RMC lock condition		*/
#define	STAT2_ABORT	0x20		/* Abort button switch		*/
#define	STAT2_SYSFAIL	0x10		/* VME bus SYSFAIL signal	*/
#define	STAT2_VMEBERR	0x08		/* VME bus BERR signal		*/
#define	STAT2_VMXBERR	0x04		/* VMX32bus BERR signal		*/
#define	STAT2_MMUBERR	0x02		/* MMU bus error signal		*/
#define	STAT2_LBTO	0x01		/* local bus timeout BERR signal*/


/*
 *	Control Registers CNT1, CNT2, CNT3, CNT4, CNT5
 */

#define	CNT1	((char*)0xfffb000e)	/* Control register 1		*/

#define	CNT1_MPIRQ	0x80		/* (IN) Multiprocessor Interrupt*/
					/*         Request		*/
#define	CNT1_WWP	0x40		/* (OUT) 0=write wrong parity	*/
#define	CNT1_PE		0x20		/* (OUT) 0=Parity enabled on	*/
					/*         local DRAM		*/
#define	CNT1_VSBE	0x10		/* (OUT) 1=VSB Interrupt enabled*/

#define	CNT2	((char*)0xfffb0038)	/* Control register 2		*/

#define	CNT2_VBIMSK	0xff		/* VME bus interrupt masks	*/

#define	CNT3	((char*)0xfffb0039)	/* Control register 3		*/

#define	CNT3_BSY	0x80		/* 1=This 136 busy		*/
#define	CNT3_IE		0x40		/* 0=All interrupts are enabled	*/
#define	CNT3_SI		0x20		/* 0=SYSFAIL interrupt enabled	*/
#define	CNT3_SH		0x10		/* 0=Signal hi pri int enabled	*/
#define	CNT3_SL		0x08		/* 0=Signal low pri int enabled	*/
#define	CNT3_LM		0x04		/* 0=Location Monitor int enable*/
#define	CNT3_BI		0x02		/* 0=Broadcast Interrupt Output	*/
#define	CNT3_VI		0x01		/* 0=VSB Interrupt Output	*/

#define	CNT4	((char*)0xfffb003a)	/* Control register 4		*/

#define	CNT4_GR		0x80		/* 0=200mSEC System Reset,global*/

#define	CNT5	((char*)0xfffb003b)	/* Control register 5		*/

#define CNT5_W0		0x80		/* Watchdog control bit 0	*/
#define CNT5_W1		0x40		/* Watchdog control bit 1	*/
#define CNT5_LT		0x20		/* 1=Local timeout generator ena*/
#define CNT5_VT		0x10		/* 1=VME timeout generator enabl*/
#define CNT5_DM		0x08		/* 1=Local DRAM mapped at 0x0000*/
#define	CNT5_32_24	0x04		/* VME bus address size 32/24	*/
#define	CNT5_32_16	0x02		/* VME bus address size 32/16	*/
#define	CNT5_BDFAIL	0x01		/* Board failure/SYSFAIL signal	*/
