/*
 *	Copyright (c) 1988 Motorola Inc
 *	All Rights Reserved
 *	@(#)mvme141.h	35.1
 */

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF MOTOROLA	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ident	"@(#)mvme141.h	35.1"

/*
 *	Board Specific Defines for the MVME141 CPU
 */

/*
 *	Memory Map
 */

/* addresses */
#define V141_LCSR	((struct vme_lcsr *)0xfffb0000)	/* VMEchip LCSR */
#define V141_VSBCSR	((char *)0xfffa0000)	/* VSBchip CSR (d32) */
#define V141_TTY	((char *)0xfff70000)	/* DUART (d8) */
#define V141_CSRA	((char *)0xfff60008)	/* CSR PORT A (d8) */
#define V141_CSRB	((char *)0xfff60009)	/* CSR PORT B (d8) */
#define V141_CSRC	((char *)0xfff6000c)	/* CSR PORT C (d8) */
#define V141_CSRD	((char *)0xfff6000d)	/* CSR PSR (d8) */
#define V141_TODC	((char *)0xfff507f8)	/* Time-of-day Clock (d8) */
#define V141_NVRAM	((char *)0xfff50000)	/* NVRAM (2040 bytes d8) */
#define V141_RAM	((char *)0xfff40000)	/* Local RAM (32k d8) */
#define V141_ROM	((char *)0xfff00000)	/* Local ROM (128k d16) */

/* VME chip interrupt mask register 
   (must be unsigned to avoid sign extension compare problem) */
#define	V141_VME_IMSK	((unsigned char *)0xfffb000f)
/* VME chip bus-error status register */
#define	V141_VME_BERR	((unsigned char *)0xfffb0019)
/* VSB chip CSR */
#define	V141_VSB_CSR	((unsigned short *)0xfffa0002)

/*
 *	Control Registers
 */

/*
 * Note: The bit masks defined below select control bits in one of the
 * four CSR bytes. Some of the bits are be asserted when the corresponding
 * bit is a one and others are asserted when the bit is zero. 
 * Symbols for positive asserted bits use an underbar (_) and symbols
 * for negative asserted bits use a lower case n (n).
 * (I wish I could use a tilde but the preprocessor doesn't like it ).
 */

/* CSR A */
#define V141_MCTL	0xe0	/* Monitor Control Mask */
#define V141_CCLR	0x10	/* Cache Clear = 1     */
#define V141nCWEN	0x08	/* Cache Write Enable = 0 */
#define V141nCREN	0x04	/* Cache Read Enable = 0 */
#define V141nS02	0x02	/* GP switch ON = 0 (GPSW2)     */
#define V141nS01	0x01	/* GP switch ON = 0 (GPSW1)     */

/* CSR B */
#define V141_GIM	0x80	/* Global Interrupt Mask =1    */ 
#define V141_VSBIM	0x40	/* VSB Interrupt Mask =1    */ 
#define V141_TMRIM	0x20	/* Timer Interrupt Mask =1    */ 
#define V141_BUSEL	0x10	/* VSB Bus Select =1 */
#define V141nVA24	0x08	/* VMEbus 24bit Address Mode = 0 */
#define V141nWPEN	0x04	/* Write-Post Enable = 0 */
#define V141nS04	0x02	/* GP switch ON = 0 (GPSW4)     */
#define V141nS03	0x01	/* GP switch ON = 0 (GPSW3)     */

/* CSR C */
#define V141nTIACK	0x80	/* Timer Interrupt Acknowledge = 0 */
#define V141nPIACK	0x40	/* Port Interrupt Acknowledge = 0 */
#define V141nPIRQ	0x20	/* Port Interrupt Request = 0 */
#define V141_TOUT	0x08	/* Timer Output */
#define V141_TIN	0x04	/* Timer Input */

/* CSR D (PSR) */
#define V141nPBERR	0x80	/* Processor Bus Error =  0 */
#define V141nABORT	0x40	/* SW Abort Switch = 0 */
#define V141_ACFL	0x20	/* AC Fail = 1 */
#define V141nLWPER	0x10	/* Local Write Posting Error = 0 */
#define V141_H4S	0x08	/* Latched Processor Bus Error = 1 */
#define V141_H3S	0x04	/* Latched SW Abort = 1 */
#define V141_H2S	0x02	/* Latched AC Fail = 1 */
#define V141_H1S	0x01	/* Latched Local Write Post Error = 1 */

/* CSR VME BUS-ERR status register */
#define	V141_RMC	0x08	/* VME chip : VME RMC lock error */
#define	V141_VBERR	0x04	/* VME chip : VME bus-error */
#define	V141_ACTO	0x02	/* VME chip : VME access time-out */
#define	V141_LBTO	0x01	/* VME chip : Local access time-out */

/* CSR VSB BUS-ERR status register */
#define	V141nVBE	0x8000	/* VSB chip : VSB bus-error */

/*
 *	Aliases
 */

#define V141_BERR	V141_H4S

/*         PI/T port A (address 0xfff60008)
 *
 *          07      06      05      04      03      02      01      00
 *       +-----------------------+---------------------------------------+
 *       | MCTL2 | MCTL1 | MCTL0 | CCLR  | CWEN* | CREN* | GPSW2 | GPSW1 |
 *       +-----------------------+---------------------------------------+
 *
 *         PI/T port B (address 0xfff60009)
 *
 *          07      06      05      04      03      02      01      00
 *       +-----------------------+---------------------------------------+
 *       |  GIM  | VSBIM | TMRIM | BUSEL*| VA24* | LWPEN*| GPSW4 | GPSW3 |
 *       +-----------------------+---------------------------------------+
 *
 */
typedef union v141_csr_abu {
	struct {
		struct {
			unsigned short
				mctl2   : 1,  /* Monitor Control Line 2 */
				mctl1   : 1,  /* Monitor Control Line 1 */
				mctl0   : 1,  /* Monitor Control Line 0 */
				cclr    : 1,  /* Cache Clear */
				cwen_   : 1,  /* Cache Write Enable */
				cren_   : 1,  /* Cache Read Enable */
				gpsw2   : 1,  /* General Purpose Switch 2 */
				gpsw1   : 1,  /* General Purpose Switch 1 */
				gim     : 1,  /* Global Interrupt Mask */
				vsbim   : 1,  /* VSB Interrupt Mask */
				tmrim   : 1,  /* Timer Interrupt Mask */
				busel_  : 1,  /* Bus Select Bit */
				va24_   : 1,  /* VME 24 Bit Address Mode */
				lwpen_  : 1,  /* Write-Posting Enable */
				gpsw4   : 1,  /* General Purpose Switch 4 */
				gpsw3   : 1   /* General Purpose Switch 3 */
		} csr_ab;
		unsigned short	z0;
	} csr_abm;
	unsigned int csr_abi;
} csr_ab_t;
/* CSR PORTS A and B -- use when possible race condition */
#define V141_CSR_AB	((csr_ab_t *)0xfff60008)
