/*	@(#)mvme323.h	35.1 	*/
/*
 *	Copyright (c) 1985	Motorola Inc.  
 *		All Rights Reserved.
 */
/*
 *	mvme323.h: include file for Motorola MVME323 disk controller
 */

/*
 *	global defines
 */

#ifdef	MVME323
#define	BADBLK	1			/* at location 512 bytes into disk */
#endif

#define M323NLCF	0xff		/* null configuration magic number */

#define M323MAXSLC	16		/* maximum slice count */
#define MAXDEV		4		/* maximum devices per controller */

/*
 *	ioctl functions
 */

#define	M323FMTT	0	/* format track */
#define	M323GET		1	/* get configuration */
#define	M323SET		2	/* set configuration */
#define	M323RST		3	/* restore drive */
#define	M323CLRF	4	/* clear fault */
#define	M323VRFY	5	/* verify track */
#define	M323COFF	6	/* cache off */
#define	M323CON		7	/* cache on */
#define M323MPT		10	/* map alternate track */
#define M323MPS		11	/* map track with sector slip */
#define	M323RFMT	12	/* reformat track saving alternates */
/*
 *	On the ioctl function,
 *
 *	M323FMTT: requires that arg be a pointer to a struct m323ctl.
 *	M323GET: requires that arg be a pointer to a struct config.
 *	M323SET: requires that arg be a pointer to a struct config.
 *	M323MTP: requires that arg be a pointer to a struct m323ctl.
 */

typedef struct dkbadlst m323fmt;	/* cyl, head, offset & alt */



/*
 *	Configuration Control: Only needs certain fields from dkconfig
 */

struct m323config {
/*  0 */ unsigned char	cf_vsh;		/* volume starting head */
/*  1 */ unsigned char	cf_vnh;		/* volume number of heads */
/*  2 */ unsigned char	cf_spt;		/* sectors per track */
/*  3 */ unsigned char	cf_skew;	/* spiral skewing factor */
/*  4 */ ushort		cf_bps;		/* bytes per sector */
/*  6 */ unsigned char	cf_gap1;	/* words in gap 1 */
/*  7 */ unsigned char	cf_gap2;	/* words in gap 2 */
/*  8 */ unsigned char	cf_intlv;	/* sector interleave factor */
/*  9 */ unsigned char	cf_retry;	/* number of retries on data error */
/* 0a */ ushort		cf_cyls;	/* number of cylinders */
/* 0c */ ushort		cf_attrib;	/* attributes */
/* 0e */ ushort		cf_ubps;	/* unformatted bytes per sector */
};

/*
 *	configuration attribute flags & masks
 */

/* driver selectable */
#define	M323RSK		0x01		/* restore & reseek after retry */
#define	M323MBD		0x02		/* move bad data into memory */
#define	M323INH		0x04		/* increment by head */
#define	M323DLP		0x08		/* dual port protocol enable */
#define	M323STC		0x10		/* interrupt on status change */
/* user selectable */
#define	M323CEN		0x20		/* sector caching enable */
#define	M323SSE		0x40		/* spare sector enable */

#define M323DATR (M323RSK|M323INH)	/* driver selected attributes */
#define M323UATR	0x60		/* user selectable attribute mask */


/*
 *	MVME323 Unit Initialization Block
 */

typedef struct {
	unsigned char	hd_strt;	/* starting head */
	unsigned char	hd_cnt;		/* number of heads */
} headrec;

struct	m323uib {
/*  0 */ headrec	ib_hd[2];	/* per volume head record */
/*  4 */ unsigned char	ib_spt;		/* sectors per track */
/*  5 */ unsigned char	ib_skew;	/* spiral skewing factor */
/*  6 */ ushort		ib_bps;		/* bytes per sector */
/*  8 */ unsigned char	ib_gap1;	/* words in gap 1 */
/*  9 */ unsigned char	ib_gap2;	/* words in gap 2 */
/* 0a */ unsigned char	ib_intlv;	/* sector interleave factor */
/* 0b */ unsigned char	ib_retry;	/* number of retries on data error */
/* 0c */ ushort		ib_cyls;	/* number of cylinders */
/* 0e */ unsigned char	ib_attrib;	/* attribute flags */
/* 0f */ unsigned char	ib_r0;		/* reserved */
/* 10 */ unsigned char	ib_silev;	/* status change interrupt level */
/* 11 */ unsigned char	ib_sivec;	/* status change interrupt vector */
/* 		Remanining fields, not looked at by controller 		*/
/* 12 */ ushort		ib_ubps;	/* unformatted bytes per sector */
};

#define ib_sh(x) ib_hd[x].hd_strt
#define ib_nh(x) ib_hd[x].hd_cnt


/*
 *	MVME323 I/O Parameter Block
 */

struct	m323iopb {
/*  0 */ unsigned char	pb_cmd;		/* command */
/*  1 */ unsigned char	pb_opt;		/* command option */
/*  2 */ unsigned char	pb_stat;	/* status code */
/*  3 */ unsigned char	pb_err;		/* error code */
/*  4 */ union {
		unsigned long	iop_lsect;	/* logical sector */
		struct {
			ushort	iop_cyl;		/* cylinder */
			unsigned char	iop_trk;	/* track */
			unsigned char	iop_psect;	/* sector */
		} pb_typ1;
	} pb_un0;
/*  8 */ ushort		pb_scnt;	/* sector count */
/*  a */ union {
		char *iop_badr;		/* buffer address */
		struct {
			unsigned char	iop_alttrk;	/* alternate track */
			unsigned char	iop_f0;		/* fill */
			ushort	iop_pat;		/* data field pattern */
		} pb_typ2;
	 } pb_un1;
/*  e */ unsigned char	pb_bmtyp;	/* buffer memory type */
/*  f */ unsigned char	pb_bamod;	/* buffer address modifier */
/* 10 */ unsigned char	pb_ilev;	/* interrupt level */
/* 11 */ unsigned char	pb_nivec;	/* normal interrupt vector */
/* 12 */ unsigned char	pb_dmac;	/* DMA count */
/* 13 */ unsigned char	pb_eivec;	/* error interrupt vector */
/* 14 */ struct m323iopb *pb_iopbp;	/* iopb pointer */
/* 18 */ unsigned char	pb_imtyp;	/* iopb memory type */
/* 19 */ unsigned char	pb_iamod;	/* iopb address modifier */
/* 1a */ unsigned char	pb_skew;	/* spiral skew */
/* 1b */ unsigned char	pb_entries;	/* entry count */
};

#ifdef	MVME323
#define pb_lsect	pb_un0.iop_lsect
#define pb_cyl		pb_un0.pb_typ1.iop_cyl
#define pb_altcyl	pb_scnt
#define pb_trk		pb_un0.pb_typ1.iop_trk
#define pb_psect	pb_un0.pb_typ1.iop_psect
#define	pb_badr		pb_un1.iop_badr
#define	pb_pat		pb_un1.pb_typ2.iop_pat
#define	pb_alttrk	pb_un1.pb_typ2.iop_alttrk
#endif


struct  m323list {
	 ushort	bcount;		/* byte count */
	 char	*addr;		/* address */
	 unsigned char memtype;	/* link bit, reserve bits, and mem type */
	 unsigned char   addrmod; /* address modifier */
};
#define M323SECSIZ 512		/* sector size */
#define M323MXCOAL M323SECSIZ/sizeof(struct m323list) /* max scatter/gather */


struct	m323ctlr {
/*  0 */ unsigned int	cc_flags;	/* controller flags */
/*  4 */ struct iobuf	*cc_actp;	/* active iobuf pointer */
	 unsigned int	rev;    	/* controller firmware revision */
};

struct m323eregs {
/*  0 */ unsigned int	er_blkoff;	/* block offset of error */
/*  4 */ unsigned char	er_ds[4];	/* drive status for four drives */
/*  6 */ ushort		er_csr;		/* control status register */
/*  8 */ unsigned char	er_cmd;		/* command */
/*  9 */ unsigned char	er_opt;		/* command option */
/* 10 */ unsigned char	er_stat;	/* status code */
/* 11 */ unsigned char	er_err;		/* error code */
};

#define d0erstat	er_ds[1]
#define d1erstat	er_ds[0]
#define d2erstat	er_ds[3]
#define d3erstat	er_ds[2]
#define M323EREG	(sizeof(struct m323eregs)/sizeof(short))

#define RDMFRLIST  3001		/* ioctl command for reading 
				  manufacturers defect list from disk */

/* ioctl call arguments to read manufacturers defect list */
struct m323mlargs {
	char *buffer;
	uint    listsize;
	uint	maxcyl;
	uint	heads;
};

