/*		Copyright (c) 1986, 1987, 1988 Motorola		*/
/*		All Rights Reserved				*/
/*				 				*/
/*		@(#)mvme332xt.h	35.3 	*/

/* MDC sccs 7.3 - 87/11/24 (internal use only) */

#include "bpp.h"

#define CB_SELECT	/* BSD4.3 select syscall hooks are in */

#define	NPKTS		60	/* number of packets per controller 	*/

#ifdef	EMULATE
#define NTTY		1	/* total number of serial ports		*/
#define NPRT		0	/* total number of printer port 	*/	
#else
#define NTTY		8	/* total number of serial ports		*/
#define NPRT		1	/* total number of printer port 	*/	
#define PRTPORT		8	/* printer port number is 8		*/
#endif


/*
 **********************************************************************
 *
 *	MVME332xt Communication Controller Driver Minor Number Assignment 
 *
 *		 <-     4    -><-     4     ->
 *		 -----------------------------
 *		|  controller  |     port     |
 *		 -----------------------------
 *
 **********************************************************************
 */

/*
 *	Minor Number Manipulating Macros
 */
#define	CTL(d)		((d>>4)&0x0f)	/* dev -> controller	*/
#define	UNIT(d)		(d & 0xf)	/* dev -> unit 		*/
#define	DEV(ctl, unit)	(((ctl)<<4) + (unit)) /* ctl, unit -> dev */

/*
 * Maximum number of minor devices allocated per controller.
 */
#define	MAXDEV		(1<<4)

/*
 * Number of devices per controller.
 */
#define NDEV_PER_CTL	(NTTY+NPRT)


/*	Command Access Address Modifier */
#define	DATA24	0x3d			/* 24-bit data */
#define	DATA32	0x0d			/* 32-bit data */

#define DEF_LINE	0	/* default line discipline number is 0	*/	


/*	
 *	mvme332xt specific csr command
 */
#define CPUTC	255		/* putc command */

/*
 *	packet->p_cmd : command
 */

/* definitions of commands 						*/

#define INIT		0x0	/* generic INITcmd type			*/
#define R_WAKEUP	0x1	/* read wakeup command			*/
#define W_WAKEUP	0x2	/* write logical  block	command		*/
#define OPEN		0x3	/* open device command			*/
#define IOCTL		0x4	/* generic i/o control command type	*/
#define CLOSE		0x5	/* close device command			*/
#define EVENT		0x6	/* evnent command 			*/

/*
 *	packet->p_iocmd : ioctl command
 */
/* non unix standard IOCTL command  */
#define	TCSETHW	(TIOC|64)	/* set hardware flow control option 	*/
				/* pkt->p_ioarg = 0 : disable		*/
				/* pkt->p_ioarg = 1 : enable		*/
#define	TCGETHW	(TIOC|65)	/* get hardware flow control setting	*/
				/* *pkt->p_ioarg = 0 : disable		*/
				/* *pkt->p_ioarg = 1 : enable		*/
#define	TCGETDL	(TIOC|66)	/* get download area address and size	*/
#define	TCDLOAD	(TIOC|67)	/* download an object file		*/
#define	TCLINE	(TIOC|68)	/* load linesw table from downloaded obj */
#define	TCEXEC	(TIOC|69)	/* execute user function in download obj */
#define	TCGETVR	(TIOC|70)	/* get firmware version and revision number */
#define	TCGETDF	(TIOC|71)	/* get open's default termio structure. */
				/* This ioctl command is identical to	*/
				/* TCGETA command excepts that it returns the */
				/* open's default termio which is used to con-*/
				/* figure the port for every first open call. */
				/* The open's default termio is also setup in */
				/* the driver's initialization routine. */
#define	TCSETDF	(TIOC|72)	/* set open's default termio structure.	*/
				/* This ioctl command is identical with	*/
				/* TCSETA command excepts that it changes */ 
				/* the open's default termio and port termio. */
#define	TCGETSYM (TIOC|73)	/* get symbol table */
#define	TCWHAT	(TIOC|74)	/* get Firmware file SCCS id */
#define	TCSETAS	(TIOC|75)	/* same as TCSETA but arg from system space */

/*
 *	packet->p_cstat : event flags
 */

/* Event bits definition */
#define	E_INTR		0x0001	/* send Interrupt Signal to host */
#define	E_QUIT		0x0002	/* send Quit Signal to host */
#define	E_HUP		0x0004	/* send Hang-Up Signal to host */
#define	E_DCD		0x0008	/* notify host that DCD is asserting */
#define	E_DSR		0x0010	/* notify host that DSR is asserting */
#define	E_CTS		0x0020	/* notify host that CTS is asserting */
#define E_NO_DCD	0x0040	/* lost carrier */				
#define E_NO_DSR	0x0080	/* lost DSR */
#define E_NO_CTS	0x0100	/* lost CTS */

#define	E_PR_FAULT	0x0200	/* notify host that printer has fault */
#define	E_PR_POUT	0x0400	/* notify host that printer is out of paper */
#define	E_PR_SELECT	0x0800	/* notify host that printer has been selected */
#define	E_SWTCH		0x4000	/* received Switch character */
#define	E_BREAK		0x8000	/* received Break */

/*
 *	packet->p_state : packet state.
 */
#define	P_DONE		0	/* packet is completely returned */
#define	P_BUSY		1	/* packet is still on ICC side */
#define	P_STALE		2	/* packet should be returned to free pool */


/*
 *	packet->p_error : packet return error.
 */
#define	ERR_LOW		ERR_BUSY	/* lowest error number */
#define	ERR_BUSY	0xfb		/* device is busy */
#define	ERR_PARM	0xfc		/* invalid parameter */
#define	ERR_UNIT	0xfd		/* invalid logical unit number */
#define ERR_CMD		0xfe		/* invalid command */
#define	ERR_CHAN_NO	0xff		/* invalid channel number */
#define	IS332ERR(x)	((x) >= ERR_LOW && (x) <= 0xff)

/* definition of ring sizes( they have to be power of 2) */
#define	RD_SIZE0	2048	/* port-0 read ring size */
#define	WR_SIZE0	4096	/* port-0 write ring size */
#define	RD_SIZE1	2048	/* port-1 read ring size */
#define	WR_SIZE1	4096	/* port-1 write ring size */
#define	RD_SIZE2	2048	/* port-2 read ring size */
#define	WR_SIZE2	4096	/* port-2 write ring size */
#define	RD_SIZE3	2048	/* port-3 read ring size */
#define	WR_SIZE3	4096	/* port-3 write ring size */
#define	RD_SIZE4	2048	/* port-4 read ring size */
#define	WR_SIZE4	4096	/* port-4 write ring size */
#define	RD_SIZE5	2048	/* port-5 read ring size */
#define	WR_SIZE5	4096	/* port-5 write ring size */
#define	RD_SIZE6	2048	/* port-6 read ring size */
#define	WR_SIZE6	4096	/* port-6 write ring size */
#define	RD_SIZE7	2048	/* port-7 read ring size */
#define	WR_SIZE7	4096	/* port-7 write ring size */
#define	RD_SIZE8	2	/* port-8 read ring size */
#define	WR_SIZE8	4096	/* port-8 write ring size */

/* share ram buffer constants */
#define SHARE_SIZE	0x10000	/* 64k shared ram */
#define DUMP_SIZE	0xd0 /* 0x100 - ipc csr reg space - ctdesc space */
#define VEC_SIZE	0x10

/*
 * Generic ring structure
 */
struct ring {
	unsigned short	stat;
	unsigned short	put;
	unsigned short	get;
	unsigned char	buf[2];
};

/*
 *	r_ring - read ring buffer definition
 */
struct r_ring0 {		/* port-0 read ring structure */
	struct ring 	rring0;
	unsigned char	buf[RD_SIZE0-2];
};

struct r_ring1 {		/* port-1 read ring structure */
	struct ring 	rring1;
	unsigned char	buf[RD_SIZE1-2];
};

struct r_ring2 {		/* port-2 read ring structure */
	struct ring 	rring2;
	unsigned char	buf[RD_SIZE2-2];
};

struct r_ring3 {		/* port-3 read ring structure */
	struct ring 	rring3;
	unsigned char	buf[RD_SIZE3-2];
};

struct r_ring4 {		/* port-4 read ring structure */
	struct ring 	rring4;
	unsigned char	buf[RD_SIZE4-2];
};

struct r_ring5 {		/* port-5 read ring structure */
	struct ring 	rring5;
	unsigned char	buf[RD_SIZE5-2];
};

struct r_ring6 {		/* port-6 read ring structure */
	struct ring 	rring6;
	unsigned char	buf[RD_SIZE6-2];
};

struct r_ring7 {		/* port-7 read ring structure */
	struct ring 	rring7;
	unsigned char	buf[RD_SIZE7-2];
};

struct r_ring8 {		/* port-8 read ring structure */
	struct ring 	rring8;
};

/*
 *	w_ring - write ring buffer definition
 */
struct w_ring0 {		/* port-0 write ring structure */
	struct ring 	wring0;
	unsigned char	buf[WR_SIZE0-2];
};

struct w_ring1 {		/* port-1 write ring structure */
	struct ring 	wring1;
	unsigned char	buf[WR_SIZE1-2];
};

struct w_ring2 {		/* port-2 write ring structure */
	struct ring 	wring2;
	unsigned char	buf[WR_SIZE2-2];
};

struct w_ring3{			/* port-3 write ring structure */
	struct ring 	wring3;
	unsigned char	buf[WR_SIZE3-2];
};

struct w_ring4 {		/* port-4 write ring structure */
	struct ring 	wring4;
	unsigned char	buf[WR_SIZE4-2];
};

struct w_ring5 {		/* port-5 write ring structure */
	struct ring 	wring5;
	unsigned char	buf[WR_SIZE5-2];
};

struct w_ring6 {		/* port-6 write ring structure */
	struct ring 	wring6;
	unsigned char	buf[WR_SIZE6-2];
};

struct w_ring7 {		/* port-7 write ring structure */
	struct ring 	wring7;
	unsigned char	buf[WR_SIZE7-2];
};

struct w_ring8 {		/* port-8 write ring structure */
	struct ring 	wring8;
	unsigned char	buf[WR_SIZE8-2];
};


/*
 * Initialization information
 */
struct dev_info{
	struct ring	*w_ring;	/* write ring pointer */
	struct ring	*r_ring;	/* read ring pointer */
	unsigned short	w_size;		/* write ring size */
	unsigned short	r_size;		/* read ring size */
	struct termio	def_tio;	/* open's default termio */
	unsigned char	dual;		/* ISP dual language mode */
	unsigned char	init_b[15];	/* reseved for future expansion */
	unsigned short	init_w[8];	/* reseved for future expansion */
	unsigned long	init_l[8];	/* reseved for future expansion */
};

/*
 * Structure used to get and set download information.
 */
struct	dl_info {
	unsigned long	host_addr;	/* host address */
	unsigned long	ipc_addr;	/* IPC address	*/
	unsigned long	count;		/* number of bytes to be transferred */
	unsigned long	ucount;		/* work area 0 */
	unsigned short	id;		/* work area 1 */
};

#define SYMEND		-1	/* end of symbol table */

/* 
 * 	ring buffer del flag(del) definition
 */
#define	RD_DEL	0x01	/* delimiter occurred	*/

/*
 * 	cb - mvme332xt tty control block
 */
struct cb {
	short		stat;			/* status		*/
	short 		rsv0;			/* reserved space	*/
	struct b_envelope *write_env;		/* write_wakeup envelope */
	struct b_envelope *read_env;		/* read_wakeup envelope */
	struct ring 	*w_ring;		/* ptr to write ring	*/
	struct ring	*r_ring;		/* ptr to read ring	*/
	int		w_size;			/* write ring size	*/
	int		r_size;			/* read ring size	*/
	short		sxtcon;			/* SXT controlling channel */
	short		pgrp[MAXPCHAN];		/* process group per channel */
	char		wsleep[MAXPCHAN];	/* write sleep address */
	char		rsleep[MAXPCHAN];	/* read sleep address */
#ifdef CB_SELECT
	struct	proc *t_rsel;	/* process who selected this device for read */
	struct	proc *t_wsel;	/* process who selected this device for write */
#endif
};

/*
 *	m332cb status bit definition		
 */
#define		DEV_READY	0x01		/* port status is OK    */
#define 	W_PENDING	0x02		/* W_WAKEUP PENDING	*/
#define 	R_PENDING	0x04		/* R_WAKEUP PENDING	*/
#define		IS_OPEN		0x08		/* port is openned	*/
#ifdef CB_SELECT
#define		CB_RCOLL	0x10		/* collision in read select */
#define		CB_WCOLL	0x20		/* collision in write select */
#endif

#define DL_BUF_SIZE	512	/* download buffer size */

/* Confidence Test Descriptor */
struct ctdesc{
	unsigned short	csw,		/* composite status word */
	resv;
	unsigned int	magic,		/* magic number */
	lcnt,		/* loop counter */
	fatal,		/* fatal error counter */
	error,		/* non-critical error counter */
	faddr,		/* fault address */
	expdata,	/* expect data */
	readata;	/* read data */
};

/*
 *	m332_dev - mvme332xt shared ram structure definition 
 */
struct m332_dev {
	/* fixed structure, required by the MVME332xt firmware */
	struct b_device ipc;		/* ipc csr register */
	struct ctdesc ctdesc;		/* rsvd for confidence test on board */
	char dump[DUMP_SIZE];		/* error dump area */

	/* the following are flexible, up to the driver */
	struct b_channel channel;	/* channel structure */

	struct b_envelope *freepkt;	/* pointer to free pkt pool */
	struct b_envelope env[NPKTS];	/* envelope space */
	struct b_packet	pkt[NPKTS];	/* packet space */
	struct cb cb[NDEV_PER_CTL];	/* control block space */
	struct r_ring0 r_ring0;		/* port-0 read ring */
	struct r_ring1 r_ring1;		/* port-1 read ring */
	struct r_ring2 r_ring2;		/* port-2 read ring */
	struct r_ring3 r_ring3;		/* port-3 read ring */
	struct r_ring4 r_ring4;		/* port-4 read ring */
	struct r_ring5 r_ring5;		/* port-5 read ring */
	struct r_ring6 r_ring6;		/* port-6 read ring */
	struct r_ring7 r_ring7;		/* port-7 read ring */
	struct r_ring8 r_ring8;		/* port-8 read ring */
	struct w_ring0 w_ring0;		/* port-0 write ring */
	struct w_ring1 w_ring1;		/* port-1 write ring */
	struct w_ring2 w_ring2;		/* port-2 write ring */
	struct w_ring3 w_ring3;		/* port-3 write ring */
	struct w_ring4 w_ring4;		/* port-4 write ring */
	struct w_ring5 w_ring5;		/* port-5 write ring */
	struct w_ring6 w_ring6;		/* port-6 write ring */
	struct w_ring7 w_ring7;		/* port-7 write ring */
	struct w_ring8 w_ring8;		/* port-8 write ring */
	char		buf_busy;	/* download buffer busy flag */
	union {
		/* initialization information */
		struct dev_info init_info[NDEV_PER_CTL];

		/* download buffer */
		char		dl_buf[DL_BUF_SIZE];
	} misc;
};

/* ipc csr bit definition */
#define IPC_BUSY	0x80	/* ipc control register busy bit */
#define IPC_TIMEOUT	0x10000	/* wait for ipc ready timeout value */ 

/* packet element alias name */
#define	p_error		p_cstat
#define	p_retcode	p_estat
#define	p_ilev		p_busw	/* interrupt level field in init packet */
#define	p_ivec		p_count	/* interrupt vector field in init packet */
#define	p_extadr	p_sa	/* address extension field in init packet */
#define p_state		p_sunit /* packet state */
#define p_cb		p_num	/* pointer to control block */
#define p_rtn_env	p_num	/* pointer to envelope when pkt is returned */
#define p_init_info	pa.pp_blkno /* pointer to init information block */
#define p_iocmd		pa.pp_blkno /* ioctl command  */
#define	p_iomode	p_count	/* ioctl mode */
#define p_ioarg		p_sa 	/* ioctl argument */
#define p_termio	psa 	/* termio space */
#define p_termcb	psa 	/* termcb space */
#define p_sgttyb	psa 	/* sgttyb space */
#define p_param		psa.pp_psa 	/* parameter space */
#define	p_csopt		psa.pp_psa	/* ISP character set option */

extern struct b_envelope *m332bppget();

/* macro definition */
#define minimum(x,y)	((x)>(y) ? (y) : (x))

