
/*		Copyright (c) 1987 Motorola			*/
/*		All Rights Reserved				*/
/*		@(#)mvme336s.h	35.1		*/
/*
 */
/*
	mvme336s.h - sio include file 

*/
/* 
        This module describes the structures necessary to
        communicate with the sio cluster.
*/

typedef unsigned char UCHAR;
typedef unsigned short USHORT;
typedef unsigned long ULONG;
typedef unsigned int UINT;
typedef int (*PFI)();


#define R6CSCAN 2	/* clock ticks for character scan */
#define R6_CHAR 01	/* character arrived flag (m5_work) */
#define R6_ACT	02	/* character in buffer flag (m5_work) */

#define	HI_MARK	18	/* clist high-water-mark */

#define MAXOUTPKT	3	/* maximum number of outstanding packets */


#define ON (1)
#define OFF (0)

#define ODD(x) (x & 0x01)

#define HELP (1)

#define SIO(x) (x << 4)

/*
	sio tty structure commands
*/

/*
	tty structure states
*/
#define WCLOSE		0x00010000
/*
	sio logical commands
*/
#define WAIT (1)
#define NOWAIT (0)

/*
        This structure describes the information necessary to
        initialize an SCC-2698 asynchronous receiver/transmitter
*/
struct port_init
{
        UCHAR mode_reg1;       /* mode register 1  */
        UCHAR mode_reg2;       /* mode register 2  */
        UCHAR clock_sel;       /* clock select register  */
        UCHAR cmd_reg;       /* command register  */
        UCHAR rts;            /* DTR enable port  */
        UCHAR dtr;            /* DTR enable port  */
        UCHAR xoff_any_enable; /* xoff enable port  */
	/*
		bit 0 - set - stop transmitting on rx of xoff
		bit 1 - set - start transmitting on rx of any character
		bit 2 - set - ignore xoff setup
		bit 3 - set - send xoff when receiver full
		bit 4 - set - RTS is used for input flow control
	*/
        UCHAR xoff;       /* xoff character register  */
        UCHAR xon;         /* xon character register  */
	UCHAR xoff_high;	/* send xoff when receive full - at high water*/
	UCHAR xoff_low;	/* start when receive empty - at low water*/
};

#define XON	(0x11)
#define XOFF	(0x13)

#define EN_XON	(0x01)
#define EN_XANY	(0x02)
#define EN_NONE	(0x04)
#define EN_XOFF	(0x08)
#define EN_RTS  (0x10)
/*

        transmitter packet command information
        to be sent to the cluster controller
*/

#define XMIT_DATA       0x00    /* transmit data command */
#define INIT_PORT	0x01    /* initialize a port pair */
#define RS232_STAT      0x02    /* get RS-232 line status */
#define FLUSH_XMIT      0x03    /* flush transmit data */
#define RESET_UART      0x04    /* reset the usart */
#define GET_TST		0x05	/* get the cluster test bits */
#define CMD_REG         0x06    /* command register specific-RTS, 1 = assert */
#define DTR_CHG         0x07    /* change condition of DTR, 1 = assert */
#define ACK_RXCNT	0x08	/* send number of rx chars back */
#define ROMREV		0x09	/* rom rev returned as BCD */
#define LINK_RETRY	0x0A	/* count of recovery mode detected in cluster */


/*
        receive data packet status information
*/

/* type of packet */
#define RX_DATA         0x00    /* receive data packet */       
#define CHAR_STAT       0x01    /* status of character, PARITY, etc */
/* #define RS232_STAT   0x02    /* status of Rs-232 line on port */
#define IPCR            0x03    /* input port change of state data */
#define TX_ACK          0x04    /* transmitter data packet acknowledge */
#define CMD_ACK         0x05    /* command packet acknowledge */
#define SELF_TST	0x06	/* get cluster test results */
#define DATA_LOST       0x07    /* data lost on this cluster */
#define RXSTOPPED	0x08	/* receive scan loop stopped - no more room */
/* #define ROMREV	0x09	/* rom rev returned as BCD */
/* #define LINK_RETRY	0x0A	/* count of recovery mode detected in cluster */


/*
	information necessary to control the flow of
	data to and from the cluster controller
*/
struct sio_ctrl
{
        UCHAR dcdflg;
        UCHAR ctsflg;
        UCHAR dsrflg;
        UCHAR brkflg;
	UCHAR hwflow;
	UCHAR ipcr;
	UCHAR rs232;
	UCHAR char_stat;
        UCHAR device;
	UCHAR sio_work;
	UCHAR romrev;
        struct tty *tp;
	UINT t_state;
	struct port_init port_init;
	UINT condition;
	UINT outstanding;
	UINT max_outstanding;
	UCHAR tx_fifo[MAXOUTPKT+2];
	UCHAR *ack;
	UCHAR *sent;
	UCHAR pkt_id;
	UCHAR rx_cnt;
	UCHAR high_rxcnt;
	UINT wclose;
	UINT cmdack;
	UINT wflush;
	UINT rflush;
};




/*
	MC-2698 baud rate values
*/
#define BD75 	0
#define BD110	1
#define BD38400	2
#define BD150	3
#define BD300	4
#define BD600	5
#define BD1200	6
#define BD2000	7
#define BD2400	8
#define BD4800	9
#define BD1800	0x0a
#define BD9600	0x0b
#define BD19200	0x0c

/*********************************
	mode register 1
**********************************
*/
/*
	Parity mode
*/
#define PRTY_MASK	0x18
#define PRTY_ON		0x00
#define PRTY_OFF	0x10

/*
	Parity type
*/
#define PRTY_OEM	0x04	/* parity odd/even mask */
#define EVEN_PRTY	0x00
#define ODD_PRTY	0x04

/*
	bits per character
*/
#define BPC_MASK	0x03
#define BPC5	0x00
#define BPC6	0x01
#define BPC7	0x02
#define BPC8	0x03

/*********************************
	mode register 2
**********************************

/*
	number of stop bits per character
*/
#define STOP_MASK	0x0f	/* mask these bits */
#define STOPB1		0x07	/* 1 stop bit per character */
#define STOPB2		0x0F	/* 2 stop bits per character */

#define AUTO_CTS	0x10	/* CTS asserted enables the transmitter */

/*********************************
	command register values
**********************************
*/
#define RXMSK		0x03
#define RX_ENABLE	0x01
#define RX_DISABLE	0x02
#define TX_ENABLE	0x04
#define TX_DISABLE	0x08

#define NO_CMD		0x00
#define RESET_MR	0x10
#define RX_RESET	0x20
#define TX_RESET	0x30
#define ERR_RESET	0x40
#define BRK_RESET	0x50
#define BRK_START	0x60
#define BRK_STOP	0x70
#define RTS_ON		0x80
#define RTS_OFF		0x90

/* */
#define DTR_ON 0x01
#define DTR_OFF 0x00
/*
	character status bits
*/
#define OVRRUN	0x10
#define PARERR	0x20
#define FRMERR	0x40
#define RCVBRK	0x80


/* don't use the RTS and DTR modifier in port init */
#define NOT_USED 0x02;

/*
	RS-232 control line bit definitions
*/
#define DSR	0x01
#define DCD	0x02
#define CTS	0x04

/*
	RS-232 control line changed bit definition
*/
#define DDSR	0x10
#define DDCD	0x20
#define DCTS	0x40

/*
	sio trace information
*/
#define SIOID	(0x53494f00)

#define SIOOPEN		(SIOID|1)
#define SIOCLOSE	(SIOID|2)
#define SIOREAD		(SIOID|3)
#define SIOWRITE	(SIOID|4)
#define SIOIOCTL	(SIOID|5)
#define SIOPARAM	(SIOID|6)
#define SIOPROC		(SIOID|7)
#define SIORXINTR	(SIOID|8)
#define SIOTXINTR	(SIOID|9)
#define SIOSCAN		(SIOID|0x0a)
#define SIOMODEM	(SIOID|0x0b)
#define SIOSENDDATA	(SIOID|0x0c)
#define SIOSENDCMD	(SIOID|0x0d)
#define SIORX		(SIOID|0x0e)
#define SIOCHKSTATUS	(SIOID|0x0f)
#define SIORESET	(SIOID|0x10)
#define SIOINIT		(SIOID|0x11)

/*
	hardware flow control defines
*/
#define MSETHWHAND 	(('m'<<8)+20)
#define MRESETHWHAND	(('m'<<8)+21)
#define MGETHWHAND	(('m'<<8)+22)
#define MGETROMREV	(('m'<<8)+23)
