/*		Copyright (c) 1987 Motorola			*/
/*		All Rights Reserved				*/
/*		@(#)mvme336x.h	35.1		*/
/*
 */
/*
	mvme336x.h - xpc include file

*/




/*
	Definition for X.25 Protocol Controller

*/
typedef unsigned char UCHAR;
typedef unsigned short USHORT;
typedef unsigned long ULONG;
typedef unsigned int UINT;
typedef int (*PFI)();


/*
	trace information
*/
#define XPCID		(0x58504300)

#define REGISTER	(XPCID|0x01)
#define DEREGISTER	(XPCID|0x02)
#define INIT		(XPCID|0x03)
#define MEMINIT		(XPCID|0x04)
#define GETPKT		(XPCID|0x05)
#define SENDPKT		(XPCID|0x06)
#define XPCTX		(XPCID|0x07)
#define RETTXPKT	(XPCID|0x08)
#define XPCTXINT	(XPCID|0x09)
#define XPCRXINT	(XPCID|0x0a)
#define XPCINT		(XPCID|0x0b)
#define CATCH		(XPCID|0x0c)
#define SENDCMD		(XPCID|0x0d)
#define WAITSEMA	(XPCID|0x0e)
#define SENDDATA	(XPCID|0x0f)
#define SENDIV		(XPCID|0x10)
#define RESTART		(XPCID|0x11)
#define XPCREAD		(XPCID|0x12)
#define XPCWRITE	(XPCID|0x13)
#define XPCRD		(XPCID|0x14)
#define XPCWRT		(XPCID|0x15)
#define XPCERR		(XPCID|0x16)


#define ODD(x) (x & 0x01)

/* address of the local device */
#define LOCAL 0xd8

/* address of the remote device */
#define REMOTE 0xd9

/* the xpc is ready */
#define READY 0xff

/* end of table */
#define EOT 0x80
/* link to the start of the receive table */
#define LINK 0x80

#define TX_BUSY (0x0001)

/*
	xpc 0	port 00 - 15	0x00 - 0x0f
	xpc 1	port 16 - 31	0x10 - 0x1f
	xpc 2	port 32 - 47	0x20 - 0x2f
	xpc 3	port 48 - 63	0x30 - 0x3f
	xpc 4	port 64 - 79	0x40 - 0x4f
	xpc 5	port 80 - 95	0x50 - 0x5f
*/
/* the hardware address of the xpc registers and memory */
#define XPC_ADDR 0x00d40000

/* extract the xpc number out of the device address */
#define XPC(device)	(device >> 4)

/*  creat the device number from the xpc number */
#define XPC_DEV(device)	(device << 4)

/* the siz of the xpc memory */
#define XPC_MEM	 0x20000
/* the total size of the xpc area */
#define MAX_XPC_MEM     0x3ffff

/* the maximum number of xpc's on a board */
#define MAX_XPC	6


#define MAX_TX  128 	/* maximum number of transmit packets per xpc */
#define MAX_RX  8	/* maximum number of receive packets per xpc */

#define REGIS_ERROR	(0)
#define REGIS_OK	(1)



#ifdef XPCMACRO

/* xpc macros to access the xpc registers */
#define SEND_CMD(XP,CMD) while(XP->cmd_sem != READY); XP->cmd_sem = CMD;

#define SEND_DATA(DATA, XP, CMD) while(XP->cmd_sem != READY); \
					XP->data_hi = ((UINT)DATA >> 16); \
					XP->data_lo = ((UINT)DATA); \
					XP->cmd_sem = CMD;

#define WAIT_SEMA(XP) while(XP->cmd_sem != READY);

#define SEND_IV(IV,XP,CMD) while(XP->cmd_sem != READY); \
				XP->ivect = IV; \
				XP->cmd_sem = CMD;
#else

#define SEND_CMD(XP,CMD) 	x336send_cmd(XP,CMD)
#define WAIT_SEMA(XP) 		x336wait_sema(XP)
#define SEND_DATA(DATA,XP,CMD)	x336send_data(DATA,XP,CMD)
#define SEND_IV(IV,XP,CMD) 	x336send_iv(IV, XP, CMD)

#endif



struct xpc_tst
{
	int state;
	struct rx_packet *rd_data;
	int rd_cnt;
};


struct xpc_ctrl
{
	UCHAR alive;	/* xpc availability */
	UCHAR local_adr;	/* local station address */
	UCHAR rmt_adr;		/* remote stattion address */
	UCHAR iv;	/* interrupt vector */
	USHORT mode_desc;
	USHORT dummy;
	UINT device;
	UINT state;
	UINT rx_err;
	UINT tx_err;
	UINT rx_reset;
	UINT tx_reset;
	UINT tx_connect;
	PFI rx_func;	/* receive service function */
	PFI tx_func;	/* transmit service function */
	PFI error_func;	/* error service function */
	struct station *station;	/* station table address */
	struct xpc_cmd *xpc_cmd;	/* pointer to xpc hardware */
	struct xmit_pkt *head_txq;	/* head of the transmit data q */
	struct xmit_pkt *tail_txq;	/* tail of the transmit data q */
	struct xmit_pkt *cur_tx_ptr;	/* current xpc transmit pkt pointer */
	struct xmit_pkt *cur_tx_tail;	/* current xpc transmit tail pointer */
	struct rx_frame *cur_rx_ptr;	/* current xpc receive frame pointer */
	struct xmit_frame *tx_frame;	/* start of the transmit frame ptr */
	struct rx_frame *rx_frame;	/* start of the receive frame ptrs */
	struct xmit_pkt *tx_pkt;	/* start of the transmit packets */
	struct rx_packet *rx_pkt;	/* start of the receive packets */
};



typedef struct xpc_cmd
{
	UCHAR fill;
	UCHAR cmd_sem;
	UCHAR fill1;
	UCHAR ivect;
	USHORT data_hi;
	USHORT data_lo;
	UCHAR filler[8];
} XPC_CMD;

/*
	values for the interrupt control
	register
*/
#define INT_ENABLE	0x0080
#define ILEV_MASK	0x03

typedef struct xpc_adr
{
	USHORT intr_ctrl;
	USHORT nu;
	USHORT nu0;
	USHORT nu1;
	USHORT nu2;
	USHORT nu3;
	USHORT nu4;
	USHORT nu5;
	XPC_CMD xpc_cmd[MAX_XPC];
} ALL_XPC;



/*
	command register values
*/
#define LD_FC   0x80    /* load function code */
#define LD_STPT 0x81    /* load station table pointer */
#define LD_OPB  0x82    /* load options bits */
#define LD_PV   0x83    /* load preset values */
#define LD_ADR  0x84    /* load addresses */
#define LD_TXTP 0x85    /* load transmit table pointer */
#define LD_RXTP 0x86    /* load receive table pointer */
#define LD_STPR 0x87    /* load station table parameters */
#define UP_STAT 0x88    /* update status */
#define CL_TLST 0x89    /* clear transmit/link status */
#define CL_RHST 0x8A    /* clear receive/host status */
#define CL_STAT 0x8B    /* clear status */
#define DMP_PAR 0x8C    /* dump parameters */
#define STRT_LK 0x8D    /* start link */
#define STOP_LK 0x8E    /* stop link */
#define DMP_REG 0x8F    /* dump registers */
#define DMA_XFR 0x90    /* DMA transfer */
#define SER_LP  0x91    /* serial loopback */
#define MON_TST 0x92    /* monitor test */
#define END_MON 0x93    /* end monitor test */
#define RESET   0xFF    /* reset */
#define SET_STA 0xA0    /* set station configuration */
#define SET_HDW 0xC0    /* set hardware configuration */


/*
	station table configration
*/
struct station
{
	USHORT  option;
	USHORT  time_out;
	USHORT  time_scale:8;
	USHORT  pad_time:8;
	USHORT  out_frame:8;
	USHORT  retries:8;
	USHORT  rx_mask;
	USHORT  tx_mask;
	USHORT  rx_stat_clr;
	USHORT  tx_stat_clr;
	USHORT  local_adr;
	USHORT  remote_adr;
	USHORT  rtfc;
	struct 	rx_frame *rx_tbl;
	USHORT  ttfc;
	struct xmit_frame *tx_tbl;
	USHORT  dafc;
	ULONG   *dmp_tbl;

	USHORT  rx_status;
	USHORT  tx_status;
	USHORT  mode_desc:8;
	USHORT  fr_desc:8;
	USHORT  vs:8;
	USHORT  vr:8;
	USHORT  fufc;
	ULONG   *first_unack;   
	USHORT  ntfc;
	struct xmit_frame *next_tx;       
	USHORT  nrfc;
	struct rx_frame *next_rx;       
	USHORT  refc;
	ULONG   *rx_error;
	USHORT  tefc;
	ULONG   *tx_error;
	USHORT  rx_frmr_1;
	USHORT  rx_frmr_2;
	USHORT  rx_frmr_3;
};


struct xmit_frame
{
	USHORT tx_tbl_stat:8;
	USHORT tx_buf_fc:8;
	UCHAR  *tx_buf_adr;
	USHORT tx_buf_len;
};




struct rx_frame
{
	USHORT rx_tbl_stat:8;
	USHORT rx_buf_fc:8;
	struct rx_packet  *rx_buf_adr;
	USHORT rx_buf_len;
	USHORT rx_final_cnt;
};


/* station table timeout values */
#define MAX_TIMEOUT	2262

/* station table time scale values */
#define TS512	0x00
#define TS1024	0x01
#define TS2048 0x02
#define TS4096	0x04
#define TS8192 0x08

/*  station tabel pad time select */
#define PAD_TIME	40	/* set for 40 @ 8 us =/char = 320 us */


/* station table outstanding frames limit register */
#define MAX_OSFL	1


/* station table retries */
#define MAX_RETRY	255


/*
	station table receive/host mask
	This mask is used for selectivly masking specific
	receiver/host interrupts
*/
#define BTS	0x8000	/* buffer too short */
#define NBA	0x4000	/* no receive buffers available */
#define ETST	0x0400	/* end of test mode */
#define UDEF	0x0200	/* undefined host command */
#define ILL	0x0100	/* illegal command */
#define RX_AERR	0x0080	/* address error */
#define RX_BERR	0x0040	/* buss error */
#define ABORT	0x0020	/* abort sequence */
#define OVRN	0x0010	/* fifo over run error */
#define RXIDLE	0x0008	/* receiver is idle */
#define RXI	0x0004	/* receive information frame */
#define RXFRMR	0x0002	/* received frame reject */
#define RTE	0x0001	/* receive frame specification tabel ended */


/* station table transmitter /link status mask */
#define NEWMD	0x0200	/* new mode */
#define RTRYL	0x0100	/* retry limit has been reached */
#define TX_AERR	0x0080	/* address error in transmitter */
#define TX_BERR	0x0040	/* buss error in transmitter */
#define CTSL	0x0020	/* clear to send has been lost */
#define URUN	0x0010	/* tx fifo underrun */
#define IFAK	0x0001	/* all transmitted I frames acknowledged */
 

/* station table option bits */
#define CRC_REJECT	0x00
#define CRC_ACCEPT	0x01



/* mode descriptor register */
#define TBEN	0x40	/* transmit buffer enable */
#define ECNT	0x20	/* extended control */
#define CONN	0x10	/* connected */
#define WACK	0x08	/* waiting for acknowledge */
#define FRMR	0x04	/* frame reject */
#define RCVR	0x02	/* recovery mode */
#define RBSY	0x01	/* remote station busy */


/* hardware configuration register  */
#define NRZI	0x10	/* non return to zero inverted */
#define NRZ	0x00	/* non return to zero */
#define BSRC	0x08	/* burst set to 8 bytes */
#define DORGM	0x02	/* data organized as INTEL */
#define NOT_DORGM	0x00	/* data organized as MOTOROLA */
#define BUSW	0x01	/* bus width is 16 bits */


/* station configuration register  */
#define SCR	0xa0	/* default value of station config register */
#define ECRC	0x08	/* 32 bit CRC */
#define NCRC	0x00	/* 16 bit CRC */
#define ST_ECNT	0x02	/* extended control field enabled */
#define ST_NECNT	0x00	/* normal control field size */


/* 
	trace table definitions
*/

