/*@(#)mvme372.h	35.1*/

/*IH*********************** INCLUDE HEADER ****************************
**
**  INCLUDE NAME:	mvme372.h
**
**  DESCRIPTION:	This file contains all mvme372 specific defines.
**
**  NOTES: (1) There is a nybble at the end of each parameter value that
**	   is a checksum for the value.  There is also a nybble at the end
**	   of NVRAM that is a checksum for all of NVRAM.
**
**	   (2) Beginning at nybble 246 of NVRAM, we have 1 byte for the
**	   factory test value and 1 nybble for its checksum.
**
**  UPDATE LOG:
**  Date	Name		V/R	Comments
**  --------	-----------	---	-----------------------------
**  06/05/86	Roy K. Larsen	1.0	Original coding.
**  04/13/87	Peggy Hassett	1.1	Added TBC parms.
**  06/17/87    Mark Huth       1.2     Added MDM channel select fields
**
**********************************************************************/

/*
**  NVRAM index values for parameter storage.  The index relates to a nibble
**  index, not a byte index.  There is always a single nibble checksum value
**  that follows any parameter. Sizes are given in bytes (not nybbles).
*/

#define	BASE_ADDR_INDEX		0	/* 1 byte vme base address for board */
#define	BASE_ADDR_SIZE		1	/* 1 byte vme base address for board */
#define	TS_ADDR_INDEX		3	/* 6 bytes this station address	     */
#define	TS_ADDR_SIZE		6	/* 6 bytes this station address	     */
#define	GA_MASK_INDEX		16	/* 6 bytes group address mask        */
#define	GA_MASK_SIZE		6	/* 6 bytes group address mask        */
#define	IA_MASK_INDEX		29	/* 6 bytes individual address mask   */
#define	IA_MASK_SIZE		6	/* 6 bytes individual address mask   */
#define	SLOT_TIME_INDEX		42	/* 2 bytes slot time                 */
#define	SLOT_TIME_SIZE		2	/* 2 bytes slot time                 */
#define	SLOT_TIME_DFLT		0x100	/* 2 bytes slot time                 */
#define	MODEM_INDEX		47	/* 1 bytes modem selection           */
#define	MODEM_SIZE		1	/* 1 bytes modem selection           */
#define	MODEM_DFLT		2	/* 1 bytes modem value = Fairchild   */
#define	TX_PRTY_INDEX		50	/* 1 byte MAC transmit priority	     */
#define TX_PRTY_SIZE		1	/* 1 byte MAC transmit priority      */
#define TX_PRTY_DFLT		7	/* 1 byte MAC transmit priority      */
#define	N_NSTT_INDEX		53	/* 1 byte No Stat Tracking           */
#define	N_NSTT_SIZE		1	/* 1 byte No Stat Tracking           */
#define	N_NSTT_DFLT		1	/* 1 byte No Stat Tracking           */
#define	N_INRD_INDEX		56	/* 1 byte In Ring Desired            */
#define	N_INRD_SIZE		1	/* 1 byte In Ring Desired            */
#define	N_INRD_DFLT		1	/* 1 byte In Ring Desired            */
#define	N_PS3_INDEX		59	/* 1 byte Prescaler Mode             */
#define	N_PS3_SIZE		1	/* 1 byte Prescaler Mode             */
#define	N_PS3_DFLT		1	/* 1 byte Prescaler Mode             */
#define	N_HI_PRI_THT_INDEX	62	/* 2 bytes High Priority Token Hold Time*/
#define	N_HI_PRI_THT_SIZE	2	/* 2 bytes High Priority Token Hold Time*/
#define	N_HI_PRI_THT_DFLT	0x03ff	/* 2 bytes High Priority Token Hold Time*/
#define	N_TAR_ROT_4_INDEX	67	/* 2 bytes Target Rotation Time (4)  */
#define	N_TAR_ROT_4_SIZE	2	/* 2 bytes Target Rotation Time (4)  */
#define	N_TAR_ROT_4_DFLT	0x7fff	/* 2 bytes Target Rotation Time (4)  */
#define	N_TAR_ROT_2_INDEX	72	/* 2 bytes Target Rotation Time (2)  */
#define	N_TAR_ROT_2_SIZE	2	/* 2 bytes Target Rotation Time (2)  */
#define	N_TAR_ROT_2_DFLT	0x7fff	/* 2 bytes Target Rotation Time (4)  */
#define	N_TAR_ROT_0_INDEX	77	/* 2 bytes Target Rotation Time (0)  */
#define	N_TAR_ROT_0_SIZE	2	/* 2 bytes Target Rotation Time (0)  */
#define	N_TAR_ROT_0_DFLT	0x7fff	/* 2 bytes Target Rotation Time (4)  */
#define	N_TAR_ROT_RM_INDEX	82	/* 2 bytes Target Rotation Time Ring Maintenance*/
#define	N_TAR_ROT_RM_SIZE	2	/* 2 bytes Target Rotation Time (0)  */
#define	N_TAR_ROT_RM_DFLT	0x7fff	/* 2 bytes Target Rotation Time (0)  */
#define	N_RM_INT_INDEX		87	/* 2 bytes Ring Maintenance Initial Value*/
#define	N_RM_INT_SIZE		2	/* 2 bytes Target Rotation Time (0)  */
#define	N_RM_INT_DFLT		0x7fff	/* 2 bytes Target Rotation Time (0)  */
#define	N_MAX_INT_SOL_INDEX	92	/* 1 bytes Max Inter Solicit Count   */
#define	N_MAX_INT_SOL_SIZE	1	/* 1 bytes Max Inter Solicit Count   */
#define	N_MAX_INT_SOL_DFLT	0x10	/* 1 bytes Max Inter Solicit Count   */
#define	N_PTP_INDEX		95	/* 1 bytes Max Inter Solicit Count   */
#define	N_PTP_SIZE		2	/* 1 bytes Max Inter Solicit Count   */
#define	N_PTP_DFLT		0x9a55	/* 1 bytes Max Inter Solicit Count   */

#define MDM_CHANNEL_INDEX	100	/* 1 byte broadband channel select   */
#define MDM_CHANNEL_SIZE	1	/* 1 byte broadband channel select   */

#define MDM_LEVEL_INDEX		103	/* 1 byte broadband transmit level   */
#define MDM_LEVEL_SIZE		1	/* size is one byte		     */

#define MDM_USER_INDEX		106	/* area for user programming of      */
#define MDM_USER_SIZE		6	/*  modems - currently 6 bytes       */

/* next free index value is     119					     */

/* field indicator values */
#define  BASE_ADDR		0
#define  TS_ADDR		1
#define  GA_MASK		2
#define  IA_MASK		3
#define  SLOT_TIME		4
#define  MODEM			5
#define  TX_PRTY		6
#define  N_NSTT			7
#define  N_INRD			8
#define  N_PS3			9
#define  N_HI_PRI_THT		10
#define  N_TAR_ROT_4		11
#define  N_TAR_ROT_2		12
#define  N_TAR_ROT_0		13
#define  N_TAR_ROT_RM		14
#define  N_RM_INT		15
#define  N_MAX_INT_SOL		16
#define  N_PTP			17
