/*@(#)com_env.h	35.1*/
/*IH*********************** INCLUDE HEADER ****************************
**
**  INCLUDE NAME:	com_env.h
**
**  DESCRIPTION:	This file contains all of the Common Environment
**			structures and defines used in the uMap code.
**
**  NOTES:		epb.h must be included before this file.
**
**********************************************************************/

#define	MAX_CEEPB_SIZE		386	/* maximum CE EPB size*/
#define MAX_CEDATA_SIZE 	1518	/* maximum size of data in BUF */
#define BUF_HDR_SIZE		8	/* size of header fields in BUF */

/*
** Maximum size of buffer with hdr
*/

#define MAX_BUF_SIZE		(MAX_CEDATA_SIZE + BUF_HDR_SIZE)

/*
**	The following structure definition generates the maximum
**	EPB size.
*/

typedef struct max_epb
{
		EPB	hdr;
		BYTE	user_data[MAX_CEEPB_SIZE - sizeof(EPB)];
} MAX_EPB;

/*
**  Common Environment EPB types
*/

	/* registration */
#define CE_REGISTRATION_REQUEST		0xff00
#define CE_REGISTRATION_RESPONSE	0xff01
#define CE_DEREGISTRATION_REQUEST	0xff02

	/* boot loader */
#define CE_LOAD_REQUEST			0xff03
#define CE_LOAD_DATA			0xff04
#define CE_LOAD_RESPONSE		0xff05
#define CE_EXECUTE_REQUEST		0xff06
#define CE_PROTECT_REQUEST		0xff07
#define CE_PROTECT_RESPONSE		0xff08
#define CE_UNPROTECT_REQUEST		0xff09
#define CE_UNPROTECT_RESPONSE		0xff0a

	/* buffer transfer */
#define CE_GETEPB_REQUEST		0xff0b
#define CE_GETEPB_RESPONSE		0xff0c
#define CE_RETEPB_REQUEST		0xff0d
#define CE_GETBUF_REQUEST		0xff0e
#define CE_GETBUF_RESPONSE		0xff0f
#define CE_RETBUF_REQUEST		0xff10

	/* message channel */
#define CE_PIPE_INIT_REQUEST		0xff11
#define CE_BPE_INDICATION		0xff12

/*
**  EPB status values reserved for CE
*/

#define	NO_ERROR		0
#define DST_NOT_REGISTERED	0xfff0
#define UNDELIVERABLE		0xfff1
#define REG_TABLE_FULL		0xfff2
#define INVALID_IP		0xfff3
#define IP_IN_USE		0xfff4
#define NO_PIPE_TO_DST		0xfff5
#define INVALID_CE_REQUEST	0xfff6
#define BL_BUSY			0xfff7
#define BAD_MEM_ACCESS		0xfff8
#define MEM_PROTECTED		0xfff9

/*
**  Misc Common Environment defines
*/

#define	CE_VECTOR		0x3fc
#define	LOCAL_CPU		0
#define	CE_IP			0

typedef struct buf	/* Data Buffer structure */
{
	struct buf	*next;	/* forward link to next buf */
	unsigned long	length;	/* length of the data in the buf */
	unsigned char	data[MAX_CEDATA_SIZE]; /* actual data */
} BUF;

/*
**  Useful Macros
*/

#define PUT_ID(cpu,ip)		(((cpu) << 16) + (ip))

/*
**  Common Environment structures
*/

typedef struct ce_epb
{
	EPB	hdr;
	USHORT	qid;		/* VRTX queue id of task registering */
/* message channel creation */
	USHORT	sh_epb_size;	/* Size of shared EPBs used by Driver */
	USHORT	sh_buf_size;	/* Size of shared BUFs used by Driver */
	USHORT	minimum_bufs;	/* mimimum number of buffers to keep */
	USHORT	maximum_bufs;	/* maximum number of buffers to keep */
	ULONG	epbbuf_count;	/* number of buffers being transfered */
	EPB	*epb_pool_head;	/* shared epb pool head */
	EPB	*epb_pool_tail;	/* shared epb pool tail */
/* boot loader */
	ULONG	addr;		/* starting address of data or for execution */
	ULONG	length;		/* length of data only in all BUFs */
	BYTE	file[32];	/* boot file name */
	BOOLEAN	eot;		/* end of transmission - last of series */
} CE_EPB;

/*
**  ROM structure definition
*/

typedef struct ce_rom
{
	BYTE		*stack;		/* address of initial stack	*/
	BYTE		version[10];	/* version number		*/
	USHORT		cpu_number;	/* hard coded cpu number	*/
	BYTE		*vbr;		/* vector base register value	*/
	ULONG		vct_vector;	/* VCT offset in EVT		*/
	struct vct 	*vct;		/* address of vrtx config table	*/
	BYTE		*vrtx;		/* address of VRTX code		*/
	DEFAULT		(*ce_start)();	/* address of CE startup fuction*/
	ULONG		ram_vector;	/* RAM vector offset in EVT	*/
	struct ce_ram	*ram;		/* address of private ram	*/
	ULONG		ram_size;	/* address of private ram	*/
	VOID		(*ce_task)();	/* common environment task address */
	VOID 		(*user_start)();/* user start function vector	*/
} CE_ROM;

/*
**  Common Environment channel table structure
*/

typedef struct ce_chtbl
{
	BYTE	version[10];	/* version number */
	USHORT	remote_cpu;	/* remote's CPU number */
	USHORT	local_cpu;	/* local's CPU number */
} CE_CHTBL;

/*
**  private global RAM memory variables
*/

typedef struct ce_ram
{
	CE_ROM	*ce_rom;	/* address of CE ROM variables		      */
	BYTE	*wrk_ram_start;	/* starting address VRTX work space ram       */
	BYTE	*wrk_ram_end;	/* ending address VRTX work space ram         */
	VOID	(*user_setup)();/* user setup function address                */
	EPB	*(*proc_in)();	/* user supplied proc_in func addr            */
	EPB	*(*proc_out)();	/* user supplied proc_out func addr           */
	VOID	(*buf_release)(); /* user supplied buf_release func addr      */
	VOID	(*symbol)();	/* user supplied boot load symbol lookup func */
	USHORT	sh_epb_size;	/* shared epb size 			      */
	USHORT	sh_buf_size;	/* shared buf size			      */
	USHORT	rt_entries;	/* number of registration table entries	      */
	USHORT	*rt;		/* registration table address		      */
	USHORT	user_init;	/* if 0xAAAA then doing user's VRTX init      */
	CE_CHTBL ch;		/* channel information */
} CE_RAM;
