/*@(#)epb.h	35.2*/
/*IH*********************** INCLUDE HEADER ****************************
**
**  INCLUDE NAME:	epb.h
**
**  DESCRIPTION:	This file will hold all declarations for
**			commonly used structures, defines, etc.
**			relating to event parameter blocks (EPB).
**
**
**********************************************************************/


/*
**  Layer IDs (src_id / dst_id) which are usually queue IDs
**
**  NOTE:  Any new queue defines must have their queue depth defined
**         in system.h.  The total queues define in system.h must reflect
**	   the total number of queues to be created by VRTX.
*/

#define	CE_QID		0	/* Common Environment (BPP) queue ID.*/
#define	MAC_QID		1	/* MAC layer queue ID.		     */
#define LINK_QID	2	/* Link layer queue ID.		     */
#define	NET_QID		3	/* Network layer queue ID.	     */
#define	XPORT_QID	4	/* Transport layer queue ID	     */
#define	SESSION_QID	5	/* Session layer queue ID 	     */
#define PRES_QID	6	/* Presentation layer queue ID 	     */
#define ACSE_QID	7       /* ACSE queue ID                     */
#define FTAM_QID	8       /* FTAM queue ID		     */
#define NET_MGR_QID	9	/* Network management QID	     */
#define DS_QID		10	/* Directory services queue ID	     */
#define	ROSE_QID	11	/* ROSE queue ID.		     */
#define	TIMER_QID	12	/* System timer queue ID.	     */
#define OD_QID          13      /* Object Dictionary queue ID        */
#define SM_QID		14	/* Station Management queue ID.      */

#define MAX_QID		14	/* highest task QID on board	     */


/*
**  This is a generic EPB which may be used in switch statements to identify
**  a given EPB.  All EPB's must contain these fields.
**
**  The EPB header structure as defined here is only used by the IPC board
**  microMAP software.  The host utilities should use the definition of
**  the EPB header structure as defined in the include <sys/ci_driver.h>.
**
**  The only difference between these two structures is the pointer to an
**  attached BDB (board) or BUF (host).
*/

#ifndef COMMON_INTERFACE
typedef	struct	epb
{
	struct	epb    *next;		/* Pointer to next EPB.	     */
	ULONG		mem_id;		/* memory pool ID	     */
	ULONG		src_id;		/* Id of message poster.     */
	ULONG		dst_id;		/* Id of message postee.     */
	USHORT		epb_size;	/* EPB size in bytes.	     */
	USHORT		type;		/* Type of EPB.		     */
	USHORT		status;		/* return status	     */
	BDB		*bdb;		/* Pointer to attached BDB   */
	ULONG		user_def;	/* user defined		     */
} EPB;
#endif
