/*	@(#)l_epb.h	35.1	*/
/*IH*********************** INCLUDE HEADER ****************************
**
**  INCLUDE NAME:	l_epb.h
**
**  DESCRIPTION:	LLC event interface definitions
**
**  NOTES:		
**			
**  UPDATE LOG:
**  Date	Name		V/R	Comments
**  --------	-----------	---	-----------------------------
**  10/12/87			3.1	Added Type 3 for LLC Class III
**					implementation for MiniMap.
**
**********************************************************************/

typedef struct	l_epb_status
{
	USHORT		llc_layer;	     /* status for type3 LLC   */
	USHORT		mac_layer;	     /* MAC status             */
	UBYTE		cmd_rsp;	     /* status returned in     */
					     /* response PDU           */
	UBYTE		reserved;	     /* reserved for later use */
} L_STATUS;

typedef	struct	l_epb
{
	EPB		hdr;		     /* EPB header structure.  */
	ULONG		Transaction_ID;	     /* Transaction Tag        */
	UBYTE		rem_addr;	     /* remote LSAP            */
	UBYTE		rem_mac_addr[MACSZ]; /* remote MAC address     */
	UBYTE		loc_addr;	     /* local LSAP 	       */
	UBYTE		loc_mac_addr[MACSZ]; /* local MAC address      */
	UBYTE		svc_class;	     /* service class 	       */
	UBYTE		reserved;	     /* reserved for later use */
	L_STATUS	status;		     /* type 3 status struct   */
} L_EPB;

/*
**  LLC sublayer types have a base value of 0x280
*/

#define	L_BASE			0x280
#define	L_DATA_REQUEST		0x281
#define L_DATA_INDICATION	0x282
#define	L_DATA_CONFIRM		0x283
#define	L_CREDIT_RESET		0x284
#define	L_REJECT_RECEIVED	0x285

#define	L_DATA_ACK_INDICATION	0x286
#define	L_DATA_ACK_REQUEST	0x287
#define	L_DATA_ACK_STS_IND	0x288
#define	L_REPLY_UPDATE_REQUEST	0x289
#define	L_REPLY_UPDATE_STS_IND	0x28A
#define	L_REPLY_REQUEST		0x28B
#define	L_REPLY_INDICATION	0x28C
#define	L_REPLY_STS_IND		0x28D

#define LM_SET_VALUE_INVOKE	0x290
#define LM_SET_VALUE_REPLY	0x291
#define LM_GET_VALUE_INVOKE	0x292
#define LM_GET_VALUE_REPLY	0x293
#define LM_COMPARE_AND_SET_VALUE_INVOKE	0x294
#define LM_COMPARE_AND_SET_VALUE_REPLY	0x295
#define LM_ACTION_INVOKE	0x296
#define LM_ACTION_REPLY	0x297

#define	L_REGISTER_REQUEST	0x2A0
#define	L_REGISTER_CONFIRM	0x2A1
#define	L_DEREGISTER_REQUEST	0x2A2

	/*                         **
	** status.llc_layer values **
	**                         */

#define L_STATUS_OK         0x0000	/* Status indicates no errors   */

#define L_PHASE_MASK        0x8000	/* Mask when the error occured
					   clear -> while FSM is being created
					   set -> while FSM is running */
#define L_FSM_CREATE        0x0000	/* Error occured during         */
					/* creation of the FSM          */
#define L_FSM_RUNNING       0x8000	/* Error occured during regular */
					/* operation of the FSM         */
#define L_ERROR             0x2000
#define L_INVALID_PRIORITY  0x1000	/* invalid priority was passed  */
					/* to LLC via the svc_class in  */
					/* the l_epb                    */
#define L_RESET             0x0800	/* LLC has been reset           */
#define L_FSM_TIME_OUT      0x0400	/* Type 3 FSM hasn't received   */
					/* an event in the alloted time */
					/* and has been destroyed       */
#define L_MAC_TIME_OUT      0x0200	/* a response to a Type 3       */
					/* command has not been received*/
					/* from MAC in the alloted time */
#define L_BAD_INDICATION    0x0100	/* LLC has received a response  */
					/* during resynchronization that*/
					/* has a command/response status*/
					/* word indicating an error     */
#define L_BAD_CONFIRMATION  0x0080	/* LLC has received a response  */
					/* from MAC with no data to     */
					/* return due to a MAC layer    */
					/* failure                      */
#define L_INACTIVE_LSAP     0x0040	/* The local LSAP given is not  */
					/* activated for the desired    */
					/* LLC Type command             */
#define L_INVALID_LSAP      0x0020	/* The LSAP given is invalid    */
					/* i.e. over 8 bits, or group   */
					/* LSAP for source of command   */
#define L_PCI_SPACE         0x0010	/* there is not enough room in  */
					/* the BDB to prepend the       */
					/* Protocol Control Information */
#define L_BDB_SPACE         0x0008	/* cannot get an BDB to complete*/
					/* the command                  */
#define L_EPB_SPACE         0x0004	/* cannot get an EPB to complete*/
					/* the command                  */
#define L_FDB_SPACE         0x0002	/* MAC cannot get an FDB to     */
					/* complete the command         */
#define L_FSM_SPACE         0x0001	/* cannot get an FSM to complete*/
					/* the command                  */

	/*                       **
	** status.cmd_rsp values **
	**                       */

#define	STATUS_CCCC	0x0f	/* Status of the Command PDU */

#define	STATUS_C_OK	0x00	/* Command accepted          */
#define	STATUS_C_RS	0x01	/* Inactivated LSAP          */
#define STATUS_C_UE	0x05	/*------- LLC user interface */
				/*-------------------- error */
#define STATUS_C_PE	0x06	/*----------- Protocol error */
#define STATUS_C_IP	0x07	/*----- Perm. implementation */
				/*---------- dependent error */
#define	STATUS_C_UN	0x09	/* Resources temporiarly     */
				/*    unavailable            */
#define	STATUS_C_IT	0x0f	/* Temp. implementation      */
				/*    dependent error        */

#define STATUS_RRRR	0xf0	/* Status of the Response PDU */

#define	STATUS_R_OK	0x00	/* Response LSDU present      */
#define	STATUS_R_RS	0x10	/* Inactivated LSAP           */
#define	STATUS_R_NE	0x30	/* Response not ready         */
#define	STATUS_R_NR	0x40	/* Response LSDU not          */
				/*    requested               */
#define	STATUS_R_UE	0x50	/* LLC user interface error   */
#define	STATUS_R_IP	0x70	/* Perm. implementation       */
				/*    dependent error         */
#define	STATUS_R_UN	0x90	/* Resources temporiarly      */
				/*    unavailable             */
#define	STATUS_R_IT	0xf0	/* Temp. implementation       */
				/*    dependent error         */
typedef struct
        {
        UBYTE   Services;       /* conformance classes supported by this
                                   implemention */
        UBYTE   Manu_ID[3];     /* IEEE administered manufacturer code */
        UBYTE   Product_Code;   /* Manufacturer's product code for this
                                   implementation */
        UBYTE   Revision;       /* Revision level of this implementation */
        }       LAYER_PROFILE;

typedef struct /* llc management parameters */
{
        LAYER_PROFILE   Profile;        /* LLC profile structure        */
        ULONG           Inact_LSAP;     /* PDUs discarded for inactive LSAP */
        ULONG           Buffer_Problems;/* Buffer Problems counter      */
        ULONG           Disc_PDUs;      /* Discarded invalid PDUs */
        ULONG           Active_LSAPs[4];/* Set of active LSAPs for this node */
        ULONG           Buffer_Size;    /* Total LLC buffer space in bytes */
} LM_GEN_PARAMS;

typedef struct
{
        ULONG           Private_Params;  /* Reserved for Motorola use */
        ULONG           Max_Retrans;    /* Retry parameter              */
        ULONG           Max_PDU;        /* Maximum Type 3 command PDU size */
        ULONG           Ack_Timer;       /* Acknowledgement timer        */
        ULONG           Rx_Var_Timer;    /* Receive variable lifetime    */
        ULONG           Tx_Var_Timer;   /* Transmit variable lifetime   */
        ULONG           Retrans_Ctr;    /* Frames retransmitted         */
} LM_GEN_TYPE3_PARAMS;

typedef struct
{
        UBYTE           Lsap;           /* Lsap to which parameters apply */
        UBYTE           Supported_Types;/* 1=t1,2=t2,4=t3; or of types    */
        UBYTE           Active_Types;   /* see above - active types       */
        UBYTE           Type3_Resources;/* boolean to enable rcv buffer   */
        ULONG           Private_Params; /* reserved for motorola          */
        ULONG           User_Name;      /* Name of user (QID) for LSAP  */
        ULONG           Unrec_PDU_Ctr;  /* Unrecognized ctrl field counter */
        ULONG           Buffer_Size;    /* Buffer space for this lsap    */
        ULONG           Max_SDU_Len;    /* Maximum SDU size for lsap     */
        ULONG           XID_Cmds_Rcvd_Ctr; /* count of xids received     */
        ULONG           XID_Cmds_Txd_Ctr;
        ULONG           XID_Resp_Rcvd_Ctr;
        ULONG           XID_Resp_Txd_Ctr;
        ULONG           Test_Cmds_Rcvd_Ctr;/* Test command PDUs received   */
        ULONG           Test_Cmds_Txd_Ctr;
        ULONG           Test_Resp_Rcvd_Ctr;
        ULONG           Test_Resp_Txd_Ctr;/* Test responses transmitted   */
        ULONG           Abrv_Test_Resp_Rcvd_Ctr;
        ULONG           Abrv_Test_Resp_Txd_Ctr;
        ULONG           UI_Rcvd_Ctr;
        ULONG           UI_Txd_Ctr;
        ULONG           PDUs_Disc_Resources_Ctr;
        ULONG           No_Resp_Ctr;    /* No response received counter */
        ULONG           Protocol_Violation_Ctr; /* protocol viol counter */
} LM_LSAP_PARAMS;

#define	L_TYPE_1	0x01	/* LLC type 1 */
#define	L_TYPE_2	0x02	/* LLC type 2 */
#define	L_TYPE_3	0x04	/* LLC type 3 */

typedef struct
{
	ULONG	null;		/* this is a place holder for later definition*/
} LM_LSAP2_PAIR_PARAMS;

typedef struct
{
	ULONG	null;		/* this is a place holder for later definition*/
} LM_LSAP3_PAIR_PARAMS;

typedef struct
{
        UBYTE          Action;          /* FF -> private action
                                        /* 0 -> initialize              */
        UBYTE          Reserved;
        ULONG          Private_Param;
} LM_GEN_ACTION_PARAMS;

typedef struct
{
        UBYTE           Action;          /* FF -> private action
                                            0 -> activate
                                            1 -> deactivate
                                            2 -> send Test command
                                            3 -> send XID command
                                         */
        UBYTE           Dest_Lsap;
        UBYTE           Dest_Addr[MACSZ];
        ULONG           Lsap[8];         /* bit mask for lsap
                                              if all 8 are null, then
                                              llc will use next avail */
        ULONG           QID[16];         /* qid for activation
                                            16 lowest weighted bits in mask
                                            are taken as Lsap to affect */
        USHORT          Lsap_Status[16]; /* status of the Action taken */
        UBYTE           Type;            /* activation type mask
        				    1=t1,2=t2,4=t3; or of types    */
        UBYTE           Reserved;
} LM_LSAP_ACTION_PARAMS;

#define	L_ACT_PRIVATE		0xFF	/* private action - reserved    */
#define L_ACT_ACTIVATE		0x00	/* activate one or more LSAPs   */
#define L_ACT_DEACTIVATE	0x01	/* deactivate one or more LSAPs */
#define L_ACT_SEND_TEST		0x02	/* send TEST command from LSAPs */
#define L_ACT_SEND_XID		0x03	/* send XID command from LSAPs  */

#define	LM_OK			0x00	/* action completed with no errors    */
#define	LM_NO_LSAPS		0x01	/* cannot select a LSAP for the user  */
					/* since there is no more individual  */
					/* LSAPs left to assign.              */
#define	LM_NO_STATS		0x02	/* no more statistical blocks left    */
					/* to assign the the LSAP             */
#define	LM_QID_MISMATCH		0x03	/* the LSAP already has a QID and it  */
					/* does not match the one passed in   */
					/* the lm_epb.                        */
#define	LM_SET_STATUS_INVALID_LSAP	0x04	/* the EPAC (TBC) does not    */
						/* recognize the LSAP to be   */
						/* valid.  EPAC problems      */
#define	LM_SET_STATUS_CMD_DIED	0x05	/* the EPAC command timed out and the */
					/* transaction failed.  EPAC problems */
#define	LM_SET_RSP_NO_FDB	0x06	/* the EPAC had no more FDBs left to  */
					/* complete the transaction.          */
#define	LM_INACTIVE_LSAP	0x07	/* the LSAP specified is currently    */
					/* inactive (not valid for            */
					/* activation).                       */
#define	LM_INVALID_LSAP		0x08	/* the LSAP specified is not valid    */
					/* for the desired transaciton, i.e   */
					/* LSAP = 0.                          */

typedef struct
{
	ULONG	null;		/* this is a place holder for later definition*/
} LM_LSAP_PAIR_ACTION_PARAMS;

typedef struct
{
	ULONG	null;		/* this is a place holder for later definition*/
} LM_GEN_EVENT_PARAMS;

typedef struct
{
	ULONG	null;		/* this is a place holder for later definition*/
} LM_LSAP_EVENT_PARAMS;

typedef struct
{
	ULONG	null;		/* this is a place holder for later definition*/
} LM_LSAP_PAIR_EVENT_PARAMS;

typedef struct /* llc management epb */
{
        EPB             hdr;            /* EPB header structure         */
	ULONG		Transaction_ID;	/* Transaction Tag */
	USHORT		Status;		/* Reply Status */
        UBYTE           Parameter_Set;  /* parameter set selector for command
                                           0 -> LM_GEN_PARAMS
                                           1 -> LM_TYPE3_GEN_PARAMS
                                           2 -> LM_LSAP_PARAMS
                                           3 -> LM_LSAP2_PAIR_PARAMS
                                           4 -> LM_LSAP3_PAIR_PARAMS
                                           10 -> LM_GEN_ACTION_PARAMS
                                           11 -> LM_LSAP_ACTION_PARAMS
                                           12 -> LM_LSAP_PAIR_ACTION_PARAMS
                                           20 -> LM_GEN_EVENT_PARAMS
                                           21 -> LM_LSAP_EVENT_PARAMS
                                           22 -> LM_LSAP_PAIR_EVENT_PARAMS */
        UBYTE           Reserved;
        ULONG           Parameter_Mask; /* Bit mask selecting parameters to
                                              be operated upon - bit 0
				              is first in structure list    */
        union { LM_GEN_PARAMS Gen_Params;
                LM_GEN_TYPE3_PARAMS Type3_Gen_Params;
                LM_LSAP_PARAMS Lsap_Params;
                LM_LSAP2_PAIR_PARAMS Lsap2_Pair_Params;
                LM_LSAP3_PAIR_PARAMS Lsap3_Pair_Params;
                LM_GEN_ACTION_PARAMS Gen_Action_Params;
                LM_LSAP_ACTION_PARAMS Lsap_Action_Params;
                LM_LSAP_PAIR_ACTION_PARAMS Lsap_Pair_Action_Params;
                LM_GEN_EVENT_PARAMS Gen_Event_Params;
                LM_LSAP_EVENT_PARAMS Lsap_Event_Params;
                LM_LSAP_PAIR_EVENT_PARAMS Lsap_Pair_Event_Params;
              } Mgmt_Params;      /* LLC management parameters */
} LM_EPB;

#define	LM_ERROR_DETECTED	0x01	/* an error was detected during     */
					/* processing, but was not          */
					/* catastrophic (i.e. several LSAPs */
					/* were registered and one          */
					/* encountered an error).           */
#define	LM_NOT_SUPPORTED	0x02	/* the transaction is not supported */
					/* at this time.                    */
#define	LM_ERROR		0x03	/* an LLC error has occured.        */

#define LM_GEN_PARAM_SET                  0
#define LM_TYPE3_GEN_PARAM_SET            1
#define LM_LSAP_PARAM_SET                 2
#define LM_LSAP2_PAIR_PARAM_SET           3
#define LM_LSAP3_PAIR_PARAM_SET           4
#define LM_GEN_ACTION_PARAM_SET          10
#define LM_LSAP_ACTION_PARAM_SET         11
#define LM_LSAP_PAIR_ACTION_PARAM_SET    12
#define LM_GEN_EVENT_PARAM_SET           20
#define LM_LSAP_EVENT_PARAM_SET          21
#define LM_LSAP_PAIR_EVENT_PARAM_SET     22

