/*	@(#)l_lm_epb.h	35.1	*/
/*IH*********************** INCLUDE HEADER ****************************
**
**  INCLUDE NAME:	l_lm_epb.h
**
**  DESCRIPTION:	Layer Management EPB definitions for LLC
**
**  NOTES:		
**			
**  UPDATE LOG:
**  Date	Name		V/R	Comments
**  --------	-----------	---	-----------------------------
**  02/02/88	wlv		3.1	Initial creation.
**  02/17/88	mwn		3.2	Added attribute mask definitions
**  03/31/88	mwn		3.3	Changed definition of mib
**********************************************************************/

/*
** 	Attribute Selector Mask Bit Definitions
*/
#define	L_PROFILE		0	/* llc profile */
#define	L_UNRECOGNIZED_PDU	1	/* unrecognized pdu */
#define	L_BUFFER_PROBLEMS	2	/* buffer problems */
#define	L_MAX_RETRANS_N4	3	/* maximum retransmissions */
#define	L_MAX_PDU_N3		4	/* maximum pdu n3 */
#define	L_ACK_TIME_T1		5	/* acknowledgement time t1 */
#define	L_RECD_VAR_LIFE_T2	6	/* receive variable lifetime t1 */
#define	L_XMIT_VAR_LIFE_T3	7	/* transmit variable lifetime t3 */
#define	L_ATTR_MAX		L_XMIT_VAR_LIFE_T3

					/* type 1 per lsap */
#define	L_TST_CMD_RECD		8	/* test command received */
#define	L_TST_RESP_SENT		9	/* test response sent */

					/* type 3 per lsap */
#define	L_NO_RESPONSE		10	/* no response */
#define	L_PROT_VIOLATION	11	/* protocol violation */


/*
**	LLC INTEGER16 Attributes
*/
#define	L_INT16_ATTR		((1 << L_MAX_RETRANS_N4)	| \
				 (1 << L_MAX_PDU_N3)		| \
				 (1 << L_ACK_TIME_T1)		| \
				 (1 << L_RECD_VAR_LIFE_T2)	| \
				 (1 << L_XMIT_VAR_LIFE_T3))

/*
**	LLC INTEGER32 Attributes
*/
#define	L_INT32_ATTR		NULL

/*
**	LLC COUNTER16 Attributes (read-only access)
*/
#define	L_CTR16_ATTR		NULL

/*
**	LLC COUNTER32 Attributes (read-only access)
*/
#define	L_CTR32_ATTR		((1 << L_UNRECOGNIZED_PDU)	| \
				 (1 << L_BUFFER_PROBLEMS)	| \
				 (1 << L_TST_CMD_RECD)		| \
				 (1 << L_TST_RESP_SENT)		| \
				 (1 << L_NO_RESPONSE)		| \
				 (1 << L_PROT_VIOLATION))

/*
**	Mask for All LLC Attributes (required for MAP 3.0 Product)
*/
#define	L_ALL_ATTR		((1 << L_PROFILE)	| \
				  L_INT16_ATTR 		| \
				  L_INT32_ATTR 		| \
				  L_CTR16_ATTR 		| \
				  L_CTR32_ATTR)

/*
**	LLC Attributes Required for ENE
*/
#define	L_ENE_ATTR		NULL

/*
**	LLC Read-Only Attributes
*/
#ifdef	M_TEST			/* Allow write access during test */

#define	L_READ_ONLY_ATTR	(1 << L_PROFILE)

#else
#define	L_READ_ONLY_ATTR	((1 << L_PROFILE)	| \
				  L_CTR16_ATTR 		| \
				  L_CTR32_ATTR)
#endif

/*
** Link Profile Options
*/
#define	L_O_TYPE1		0x4000	/* type 1 */
#define	L_O_TYPE2		0x2000	/* type 2 */
#define	L_O_TYPE3		0x1000	/* type 3 */
#define	L_O_SIZE		4	/* # bits in bit string */


typedef	struct	l_mib
{
						/* attribute list */
	UBYTE		count;			/* # valid pids in array */
	M_PROFILE	profile;		/* llc profile */
	M_PARM		parameter[M_ATTR_MAX];
} L_MIB;

typedef	struct	l_lm_epb
{
	M_ECB		ecb;		/* event control block */
	UBYTE		lsap;		/* data link per-lsap id */
	L_MIB		mib;		/* management information base */
} L_LM_EPB;
