/*	@(#)m_defs.h	35.1	*/
/*IH*********************** INCLUDE HEADER ****************************
**
**  INCLUDE NAME:	m_defs.h
**
**  DESCRIPTION:	Network Management Agent - common definitions
**
**  NOTES:		
**			
**  UPDATE LOG:
**  Date	Name		V/R	Comments
**  --------	-----------	---	-----------------------------
**  02/02/88	wlv		3.1	Initial creation.
**  02/03/88	wlv		3.2	Moved NM agent specific defines
**					and structures to new files
**					"m_pdu.h" and "m_machine.h".
**  02/04/88	wlv		3.2	Added THRESHOLD typedef.
**  02/09/88	wlv		3.2	Added M_SET_THRESHOLD macro
**					and M_NO_ERROR define.
**  02/18/88	mwn		3.2	Added M_ENE definition.
**  03/25/88	wlv		3.4	Changed M_ECB invoke_id to LONG
**					and added local/remote initiate,
**					terminate, & abort definitions
**					for use by NM Upper Tester / SMUI.
**  03/31/88	mwn		3.5	Added M_PARM, M_SYNC*, and
**					M_ACCESS* definitions.
**********************************************************************/
#define	M_TEST	TRUE
#define	M_SAFE	TRUE
#define	M_ENE	TRUE


#define	M_EVENT_CHECK(up_counter, threshold) \
      	(((up_counter++ >= 0) && (threshold.value == 0)) ? 0 : \
	((--threshold.count == 0) ? (threshold.count = threshold.value) : 0))

/*
** This macro is used by LMEs to increment counters that have thresholds.
** If an event should be generated, this macro returns a non-zero
** value.  Otherwise, it returns a zero.
**
** M_EVENT_CHECK always increments the up_counter.  If the threshold.value
** is non-zero, the threshold.count is decremented.
** When the threshold.count becomes zero, it is reset to the value of the 
** threshold, and the macro returns a non-zero value.  
** The macro assumes that up_counter and the threshold are unsigned 
** (always positive) and that the threshold has been declared as a
** THRESHOLD type structure as defined below.
*/

#define M_SET_THRESHOLD(threshold, new_value) \
	threshold.count = threshold.value = (INTEGER16) new_value;

/*
** This macro is used by LMEs to set the initial or new value of a 
** structure that has been declared to be of type THRESHOLD (which
** is defined below).  Use of the macro and THRESHOLD type ensure
** that all LMEs properly handle the setting of thresholds that will
** used in the M_EVENT_CHECK macro shown above.  The THRESHOLD type
** conforms to the current definitions of all thresholds in the MAP
** 3.0 NM specification as 16-bit integers.  If 32-bit thresholds
** are later added to the spec, then this macro may require change.
*/


#define	M_SYSTEM_QID	MAX_QID+1		/* fictitious system QID */
/*
** Common Management Array Size Definitions
*/
#define	M_ACSZ		16		/* max # bytes in access control */
#define	M_MANUSZ	3		/* size of manufacturer's id */
#define	M_MGRTBLSZ	1		/* max # of manager table entries */
#define	M_RCSZ		OBJIDSZ		/* max # bytes in resource class str */
#define M_OPTSZ		OBJIDSZ		/* max # bytes in profile.optional */
#define M_ATTR_MAX	32		/* max # of attributes in parm list */

/*
** Indexes into Resource Class table
*/
#define	M_RC_INDEX_O		9	/* System Resource Class */
#define	M_RC_INDEX_A		8	/* ACSE Resource Class */
#define	M_RC_INDEX_P		7	/* Presentation Resource Class */
#define	M_RC_INDEX_S		6	/* Session Resource Class */
#define	M_RC_INDEX_T		5	/* Transport Resource Class */
#define	M_RC_INDEX_N		4	/* Network Resource Class */
#define	M_RC_INDEX_L		3	/* Data Link Resource Class */
#define	M_RC_INDEX_MA		2	/* MAC 802.4 Resource Class */
#define	M_RC_INDEX_PH		1	/* Physical 802.4 Resource Class */


/* 
** 	These typedefs should be used in Layer Management Entities (LMEs)
**	to define their local management information base (MIB) elements.
*/
typedef	USHORT	INTEGER16;	/* range: 0..2**16-1, wraps to 0, Read-only */
typedef	USHORT	COUNTER16;	/* same as INTEGER16, but is setable */
typedef	ULONG	INTEGER32;	/* range: 0..2**32-1, wraps to 0, Read-only */
typedef	ULONG	COUNTER32;	/* same as INTEGER32, but is setable */

typedef struct	threshold	/* threshold structure - two 16 bit values */
{
	INTEGER16  value;	/* read/write value accessible to Manager SMAP
				**  used to set/reset "count" field
				*/
	INTEGER16  count;	/* down counter - not accessible to Mgr. SMAP
				**  when decremented to zero, an event report
				**  should be sent to the Manager SMAP
				*/
} THRESHOLD;

/* 
** Constant values for System Profile definitions:
**	Revision = 0;		==>	map3.0 revision
**	Options = 0;		==>	nmclass0
**	Mfr_Id[] = 0x000000	==>	Motorola
**
**	NOTE:  The profile.optional[] is an OCTET STRING
**		type structure:  the first byte is the length in bytes of
**		the information in the rest of the field.  If the first byte
**		is zero, the field is interpretted as containing no valid
**		information.
*/
#define	MAP_REV	0			/* value for MAP 3.0 revision	*/
#define NMCLASS	0			/* NM options field value	*/
#define MOTOROLA_MFR_ID	{'M','O','T'}	/* Motorola Manufacturer ID (?)	*/

typedef	struct	profile
{
	UBYTE	revision;		/* MAP standard revision */
	USHORT	options;		/* MAP supported options */
	UBYTE	mfr_id[M_MANUSZ];	/* IEEE manufacturer's id */
	UBYTE	optional[M_OPTSZ];	/* optional info */
	SHORT	index;			/* index into m_rc_table */
	UBYTE	instance;		/* resource instance */
} M_PROFILE;


typedef	struct	paddr
{
	UBYTE		psel[PSAPSZ];	/* presentation selector */
	UBYTE		ssel[SSAPSZ];	/* session selector */
	UBYTE		tsel[TSAPSZ];	/* transport selector */
	UBYTE		nsap[NSAPSZ];	/* network service access point */
} M_PADDR;


typedef	struct	saddr
{
	UBYTE		ssel[SSAPSZ];	/* session selector */
	UBYTE		tsel[TSAPSZ];	/* transport selector */
	UBYTE		nsap[NSAPSZ];	/* network service access point */
} M_SADDR;


typedef	struct	taddr
{
	UBYTE		tsel[TSAPSZ];	/* transport selector */
	UBYTE		nsap[NSAPSZ];	/* network service access point */
} M_TADDR;	


/*
** Definitions for the m_parm "es" (error status) field.
*/
#define	M_ES_ACCESS_DENIED	1	/* access denied */
#define	M_ES_NO_SUCH_OPERATION	2	/* no such operation */
#define	M_ES_NO_SUCH_ATTR	3	/* no such attribute */
#define	M_ES_INVALID_ATTR_VALUE	4	/* invalid attribute value */

typedef	struct	m_parm
{
	UBYTE		pid;		/* parameter id */
	UBYTE		es;		/* error status (if any) */
	ULONG		value;		/* parameter value */
} M_PARM;


/*
** CMIS Operations Definitions (see section 11.1.5) - 
**	specifies the operation-value found in the ROSE PDU.
*/
#define	M_U_EVENT		2	/* unconfirmed event report */
#define	M_C_GET			3	/* confirmed read */
#define	M_C_SET			4	/* confirmed modify */
#define	M_C_ACTION		5	/* confirmed action */
#define	M_U_SET			8	/* unconfirmed modify */
#define	M_U_ACTION		9	/* unconfirmed action */

/*
** CMIS Pseudo-Operations (normally mapped directly onto ACSE services)
** 	Defined here to allow local initiation & termination of the
**	Manager-Agent association for testing & station mgr. purposes.
*/
#define M_INIT			0x20	/* m-initialize */
#define M_TERM    		0x21	/* m-terminate */
#define M_ABORT   		0x22	/* m-abort */

#define M_LOCAL			0x00	/* local activation */
#define M_REMOTE		0x10	/* remote activation */


/* 
** CMIS/LME Event Types -
**	These values represent valid epb header "type" field values 
**	for Network Management operations.  All managed resources 
**	should respond to these values to perform NM operations.
*/
#define	M_BASE			0x900	/* Event Type Base Number */
#define M_REQ			0x80	/* request type */
#define	M_CON			0x40	/* confirm type */

#define	M_U_ACTION_REQ		M_BASE+M_U_ACTION+M_REQ
#define	M_U_ACTION_CON		M_BASE+M_U_ACTION+M_CON
#define	M_U_EVENT_REQ		M_BASE+M_U_EVENT+M_REQ
#define	M_U_SET_REQ		M_BASE+M_U_SET+M_REQ
#define	M_U_SET_CON		M_BASE+M_U_SET+M_CON

#define	M_C_GET_REQ		M_BASE+M_C_GET+M_REQ
#define	M_C_GET_CON		M_BASE+M_C_GET+M_CON
#define	M_C_SET_REQ		M_BASE+M_C_SET+M_REQ
#define	M_C_SET_CON		M_BASE+M_C_SET+M_CON
#define	M_C_ACTION_REQ		M_BASE+M_C_ACTION+M_REQ
#define	M_C_ACTION_CON		M_BASE+M_C_ACTION+M_CON

/*
**  NM Upper Tester Event Types - these service events are all mapped
**	onto the A_EPB structure.  For local requests, the EPB header
**	and the user_id is the only required information in the A_EPB.
**	Remote requests must conform to the ACSE A_EPB requirements.
*/
#define M_INIT_LOC_REQ		M_BASE+M_INIT+M_LOCAL+M_REQ
#define M_INIT_LOC_CON		M_BASE+M_INIT+M_LOCAL+M_CON
#define M_INIT_REM_REQ		M_BASE+M_INIT+M_REMOTE+M_REQ
#define M_INIT_REM_CON		M_BASE+M_INIT+M_REMOTE+M_CON

#define M_TERM_LOC_REQ		M_BASE+M_TERM+M_LOCAL+M_REQ
#define M_TERM_LOC_CON		M_BASE+M_TERM+M_LOCAL+M_CON
#define M_TERM_REM_REQ		M_BASE+M_TERM+M_REMOTE+M_REQ
#define M_TERM_REM_CON		M_BASE+M_TERM+M_REMOTE+M_CON

#define M_ABORT_LOC_REQ		M_BASE+M_ABORT+M_LOCAL+M_REQ
#define M_ABORT_REM_REQ		M_BASE+M_ABORT+M_REMOTE+M_REQ


/* 
** Definitions for the ecb "result" field, and when they may be used. 
** 	CMIS Error Definitions (see section 11.1.5 of MAP 3.0 spec.)
**
** noSuchResourceClass: event, get, set, action
**	the ResourceClass specified is not recognized by the responder SMAE.
**
** accessDenied: get, set, action
**	the attributes were not read because the value of AccessControl
**	was not acceptable or access was denied for other reasons.
**
** noSuchAttribute: event, get, set
**	the EventInfo(event) or AttributeId(get, set) specified is not
**	supported.
**
** invalidAttributeValue: event, set
**	the attribute value specified is out of range, or otherwise
**	inappropriate.
**
** getListError: get
**	One or more attributes were not read for one of the above errors;
**	the values read are returned.
**
** setListError: set
**	One or more attributes were not set for one of the above errors;
**	the attributes that could be set are indicated.
**
** noSuchAction: action
**	the ActionType specified is not supported.
**
** noSuchActionArg: action
**	one or more of the specified action arguments are not supported.
**
** noSuchResourceInstance: event, get, set, action
**	the Resource Instance specified is not recognized by the
**	responder SMAE, or no ResourceInstance was specified and
**	multiple instances exist.
**
** eventUnknown: event
**	the event was not logged as the event was not known to the receiver.
**
** processingFailure: event, action
**	a general failure in processing the event or action was encountered.
**
** syncNotSupported: get, set
**	the type of synchronization specified by the Synchronization
**	parameter is not supported.
*/
#define	M_NO_SUCH_CLASS		0x02	/* no such resource class */
#define	M_ACCESS_DENIED		0x03	/* access denied */
#define	M_NO_SUCH_ATTR		0x05	/* no such attribute */
#define	M_INVALID_ATTR_VALUE	0x06	/* invalid attribute value */
#define	M_SYNC_NOT_SUPPORTED	0x07	/* sync not supported */
#define	M_GET_LIST_ERROR	0x08	/* get list error */
#define	M_SET_LIST_ERROR	0x09	/* set list error */
#define	M_NO_SUCH_ACTION	0x0a	/* no such action */
#define	M_NO_SUCH_ACTION_ARG	0x0b	/* no such action arg */
#define M_NO_SUCH_INSTANCE	0x0f	/* no such resource instance */
#define	M_EVENT_UNKNOWN		0x10	/* event unknown */
#define	M_PROCESSING_FAILURE	0x11	/* processing failure */

#define M_NO_ERROR		0
#define	M_NOT_ACTIVE		M_BASE+1
#define	M_NO_MEMORY		M_BASE+2
#define	M_UNEXPECTED		M_BASE+3	/* unexpected event */


/*
** Definitions for the ecb "sync" field.
*/
#define	M_SYNC_BEST_EFFORT	0	/* All operations that can be done do
					** get done, the ones that can't, don't.
					** The ordering is not important.
					*/
#define	M_SYNC_ORDERED		1	/* All operations that can get done do
					** get done, the ones that can't, don't.
					** Operations are attempted in the
					** sequence they are presented.
					*/
#define	M_SYNC_STOP_ON_ERROR	2	/* The operations are processed
					** sequentially in the order they
					** appear in the epb, and when an
					** operation fails, the remaining are
					** not attempted.
					*/
#define	M_SYNC_ATOMIC		3	/* All operations are checked to see
					** if they can be performed.  Either
					** all are performed successfully, or
					** none are performed.
					*/

/*
** Definitions for the ecb "access_granted" bit field.  If an attribute 
** may be read, and the only way to change its value is by performing an 
** action (eg. the System managerNames PARAMETERID), then...
** if the Network Manager Application tries to M_SET that parameter id,
** the layer will set the m_parm.es (error status) field for that parameter id 
** to M_ES_ACCESS_DENIED, and the ecb access_granted bit field to 0x50
** (ie. M_ACCESS_GET + M_ACCESS_ACTION).
*/
#define	M_ACCESS_NONE		0x80000000	/* cannot be accessed */
#define	M_ACCESS_GET		0x40000000	/* value may be read */
#define	M_ACCESS_SET		0x20000000	/* value may be modified */
#define	M_ACCESS_ACTION		0x10000000	/* may perform actions on this pid */
#define	M_ACCESS_LOG		0x08000000
#define	M_ACCESS_LENGTH		5			/* length in bits */


/* 
** Event Control Block
*/
typedef	struct	m_ecb
{
	EPB		hdr;		/* EPB header structure */
	struct	m_ecb	*wait_queue;	/* reserved */
	LONG		invoke_id;	/* reserved - copy into resp lm_epb */
	UBYTE		*user_id;	/* reserved - copy into resp lm_epb */
	UBYTE		sync;		/* synchronization for operation */

	USHORT		result;		/* returned result of operation */
	ULONG		access_granted;	/* access granted to single pid */
}	M_ECB;


