/*@(#)ma_epb.h	35.2*/

/*IH*********************** INCLUDE HEADER ****************************
**
**  INCLUDE NAME:	ma_epb.h
**
**  DESCRIPTION:	This file will hold all declarations for event
**			parameter blocks (EPB) used by MAC.
**
**  NOTES:		All EPB's will be declared as typedefs.
**			All EPB's will be included in the common EPB
**			structure.
**			
**  UPDATE LOG:
**  Date	Name		V/R	Comments
**  --------	-----------	---	-----------------------------
**  11/23/87	-----------	3.0	Handle the new EPB structure of
**					includes and LLC type 3 changes.
**
**********************************************************************/

typedef	struct	ma_epb
{
	EPB		hdr;		     /* EPB header structure */
	UBYTE		rem_addr[MACSZ];     /* destination address  */
	UBYTE		loc_addr[MACSZ];     /* source address       */
	USHORT		status;		     /* Tx/Rx status         */
	UBYTE		svc_class;	     /* service class 	     */
	BDB		*Response_Ptr;	     /* Pointer to response bdb */
	USHORT		Response_Status;     /* Receive status if response */
	UBYTE		Response_Svc_Class;  /* Freame control for response */
} MA_EPB;
/*
**  Data Link layer types have a base value of 0x200
*/

	/*
	**  MAC sublayer types have a base value of 0x200
	*/

#define	MA_BASE			0x200
#define	MA_NULL_EPB		0x200	/* usedf to kick mac with no action */
#define MA_DATA_REQUEST		0x201
#define MA_DATA_INDICATION	0x202
#define MA_DATA_CONFIRM		0x203
#define MA_DATA_CONF_IND	0x204   /* Immed. Rsp. Conf and Indication */
#define MA_READ_REQUEST		0x208   /* MAC parameters read request	*/
#define MA_READ_CONFIRM		0x209   /* MAC parameters read confirm	*/
#define MA_PARAM_REQUEST	0x20A
#define MA_PARAM_CONFIRM	0x20B
#define MA_REG_REQUEST		0x20C
#define MA_REG_CONFIRM		0x20D
#define MA_DEREG_REQUEST	0x20E
#define MA_DEREG_CONFIRM	0x20F

/* The following defines are constants for the MAC frame type
   OR these together with MAC action and priority constants to
   produce the quality of service parameter */

#define		LLC_DATA	0x02
#define		STA_MAN_DATA	0x01
#define		SPECIAL_DATA	0x03
#define		FAME_TYPE_MASK	0x07

/* The following defines are constants specifying the MAC action */

#define		RWNR		0x00	/* request with no response */
#define		RWR		0x10	/* request with response    */
#define		RSP		0x08	/* response                 */
#define		MAC_ACTION_MASK	0x1C

/* The following defines are constants specifying the MAC service
   class requested */

#define		PRI_7		0xE0
#define		PRI_6		0x60
#define		PRI_5		0xA0
#define		PRI_4		0x20
#define		PRI_3		0xC0
#define		PRI_2		0x40
#define		PRI_1		0x80
#define		PRI_0		0x00
#define		PRIORITY_MASK	0xE0

/* The following define MAC status returned in the epb */

#define		MA_CONFIRMED		0x8000
#define		MA_NEGATIVE		0x4000
#define		MA_EMPTY		0x2000
#define		MA_FRAME_TOO_LONG	0x1000
#define		MA_UNDERRUN		0x0400
#define		MA_NO_RESPONSE		0x0200
#define		MA_UNEXPECTED_FRAME	0x0100
#define		MA_RETRY_COUNT		0x00FF
#define		MA_ABORT		0x0001
#define		MA_RESET		0x7FFF
#define		MA_REJ_NULL_BDB		0x7FFE

