/*@(#)sm_epb.h	35.1*/

/*IH*********************** INCLUDE HEADER ****************************
**
**  INCLUDE NAME:	sm_epb.h
**
**  DESCRIPTION:	Station Manager event interface definitions
**
**  NOTES:		
**			
**  UPDATE LOG:
**  Date	Name		V/R	Comments
**  --------	-----------	---	-----------------------------
**
**********************************************************************/

/*
**  This is the EPB for the event that collects station management
**  statistics from the MAC sublayer.
*/

typedef	struct	sm_epb
{
	EPB		hdr;		/* EPB header structure		*/
	UBYTE		station_addr[MACSZ]; /* station address		*/
	ULONG		tx_ctr;		/* transmitted packets		*/
	ULONG		rx_ctr;		/* received packets		*/
	UBYTE		mac_sel;	/* MAC operation selection	*/
	USHORT		board;		/* Board Number			*/
	ULONG		log_mask;	/* log mask			*/
	union
	{
		ULONG	stats[20];	/* MAC Statistics		*/

		USHORT	ring;		/* LANCE Ring Sizes		*/
		USHORT	mode;		/* LANCE Mode			*/
		USHORT	group[4];	/* LANCE Group Address        	*/

		UBYTE	priority;	/* TBC Transmit Priority	*/
	} mac;
	union
	{
		ULONG	clock;          /* starting clock value         */
		ULONG	bdb_count;      /* BDB count                    */
		ULONG	o_user_limit;   /* User limit                   */
		ULONG	o_mgmt_limit;   /* Management limit             */
		ULONG	o_netw_limit;   /* Network limit                */
		UBYTE	t_min_credit;   /* Minimum credit               */
		UBYTE	t_max_credit;   /* Maximum credit               */
		USHORT	t_max_connects; /* Xport max # connectrions     */
		USHORT	t_max_retries;  /* Xport max # of retries       */
		USHORT	t_retry_ticks;  /* Xport # of retry ticks       */
		USHORT	t_exp_retry_ticks;/* # of expedited retry ticks */
		USHORT	t_window_ticks; /* Xport # of window ticks      */
		USHORT	t_ref_wait_ticks;/* # of reference wait ticks   */
	} sys;
	USHORT		nvram_write;	/* See below			*/
	UBYTE		nvram[128];	/* NVRAM data			*/
} SM_EPB;

/*
**	Defines to be used in sm_epb->nvram_write to specify where the
**	data is to be written.
*/

#define	NVRAM_ONLY	0		/* Write bytes to NVRAM only	*/
#define	NVRAM_EEPROM	1		/* Write bytes to NVRAM and	*/
					/* EEPROM			*/


/*
**	Defines to be used by Station Manager on board and console
**	to monitor traffic.
*/

#define	MON_STRT	1		/* Start monitoring traffic	*/
#define	MON_STOP	2		/* Stop monitoring traffic	*/
#define	MON_READ	3		/* Read traffic stats		*/
#define MON_SMPL	4		/* Sample traffic		*/

/*
**	Global defines to be used in sm_epb->mac_sel to specify where the
**	MAC data is to be read/written.
*/

#define MAC_RD_STATS	1		/* Read MAC Statistics		*/

/*
**	MVME374 defines to be used in sm_epb->mac_sel to specify where the
**	MAC data is to be read/written.
*/

#define MAC_RD_RING	2		/* Read LANCE Ring Sizes	*/
#define MAC_RD_MODE	3		/* Read LANCE Mode 		*/
#define MAC_RD_NODE	4		/* Read LANCE Node Address	*/
#define MAC_RD_GROUP	5		/* Read LANCE Group Address	*/
#define MAC_WR_MODE	6		/* Write LANCE Mode 		*/
#define MAC_WR_NODE	7		/* Write LANCE Node Address	*/
#define MAC_WR_GROUP	8		/* Write LANCE Group Address	*/

/*
**	MVME372 defines to be used in sm_epb->mac_sel to specify where the
**	MAC data is to be read/written.
*/

#define MAC_RD_PRIOR	2		/* Read TBC TX Priority  	*/
#define MAC_WR_PRIOR	3		/* Write TBC TX Priority  	*/

/*
**  Station Manager event types
*/

#define	SM_BASE				0xC00	/* Station Manager base	      */
#define	SM_NVRAM_RD_REQUEST		0xC01	/* Read values from NVRAM     */
#define	SM_NVRAM_RD_CONFIRM		0xC02	/* Values from NVRAM	      */
#define	SM_NVRAM_WR_REQUEST		0xC03	/* Write values into NVRAM    */
#define	SM_LOG_MASK_RD_REQUEST		0xC04	/* Read log_mask	      */
#define	SM_LOG_MASK_RD_CONFIRM		0xC05	/* Log_mask returned 	      */
#define	SM_LOG_MASK_SET_REQUEST		0xC06	/* Set log_mask		      */
#define	SM_TRMON_REQUEST		0xC07	/* Traffic monitor request    */
#define	SM_TRMON_RESPONSE		0xC08	/* Traffic monitor response   */
#define	SM_BDB_COUNT_RD_REQUEST 	0xC09	/* Read BDB count	      */
#define	SM_BDB_COUNT_RD_CONFIRM 	0xC0A	/* Read BDB count confirm     */
#define	SM_USER_LIM_RD_REQUEST  	0xC0B	/* Read user limit	      */
#define	SM_USER_LIM_RD_CONFIRM  	0xC0C	/* Read user limit confirm    */
#define	SM_USER_LIM_SET_REQUEST 	0xC0D	/* Change user limit	      */
#define	SM_MGMT_LIM_RD_REQUEST  	0xC0E	/* Read management limit      */
#define	SM_MGMT_LIM_RD_CONFIRM  	0xC0F	/* Read management limit confir*/
#define	SM_MGMT_LIM_SET_REQUEST 	0xC10	/* Change management limit    */
#define	SM_NETW_LIM_RD_REQUEST  	0xC11	/* Read network limit	      */
#define	SM_NETW_LIM_RD_CONFIRM  	0xC12	/* Read network limit confirm */
#define	SM_NETW_LIM_SET_REQUEST 	0xC13	/* Change network limit	      */
#define	SM_MAX_CONNECTS_RD_REQUEST  	0xC14	/* Read max # of connections  */
#define	SM_MAX_CONNECTS_RD_CONFIRM  	0xC15	/* Read max # connects confirm*/
#define	SM_MAX_CONNECTS_SET_REQUEST  	0xC16	/* Change max # connects      */
#define	SM_MIN_CRED_RD_REQUEST  	0xC17	/* Read minimum credit	      */
#define	SM_MIN_CRED_RD_CONFIRM  	0xC18	/* Read minimum credit confirm*/
#define	SM_MIN_CRED_SET_REQUEST 	0xC19	/* Change minimum credit      */
#define	SM_MAX_CRED_RD_REQUEST  	0xC1A	/* Read maximum credit	      */
#define	SM_MAX_CRED_RD_CONFIRM  	0xC1B	/* Read maximum credit confirm*/
#define	SM_MAX_CRED_SET_REQUEST 	0xC1C	/* Change maximum credit      */
#define	SM_MAX_RETRIES_RD_REQUEST  	0xC1D	/* Read maximum retries	      */
#define	SM_MAX_RETRIES_RD_CONFIRM  	0xC1E	/* Read maximum retries confirm*/
#define	SM_MAX_RETRIES_SET_REQUEST 	0xC1F	/* Change maximum retries     */
#define	SM_RETRY_TICKS_RD_REQUEST  	0xC20	/* Read retry ticks	      */
#define	SM_RETRY_TICKS_RD_CONFIRM  	0xC21	/* Read retry ticks confirm   */
#define	SM_RETRY_TICKS_SET_REQUEST 	0xC22	/* Change retry ticks         */
#define	SM_EXP_RETRY_TICKS_RD_REQUEST  	0xC23	/* Read expedited retry ticks */
#define	SM_EXP_RETRY_TICKS_RD_CONFIRM  	0xC24	/* Read exp retry ticks confirm*/
#define	SM_EXP_RETRY_TICKS_SET_REQUEST 	0xC25	/* Change exp retry ticks     */
#define	SM_WINDOW_TICKS_RD_REQUEST  	0xC26	/* Read window ticks          */
#define	SM_WINDOW_TICKS_RD_CONFIRM  	0xC27	/* Read window ticks confirm  */
#define	SM_WINDOW_TICKS_SET_REQUEST 	0xC28	/* Change window ticks        */
#define	SM_REF_WAIT_TICKS_RD_REQUEST  	0xC29	/* Read reference wait ticks  */
#define	SM_REF_WAIT_TICKS_RD_CONFIRM  	0xC2A	/* Read ref wait ticks confirm*/
#define	SM_REF_WAIT_TICKS_SET_REQUEST 	0xC2B	/* Change reference wait ticks*/
