/*@(#)stats.h	35.1*/

/*IH*********************** INCLUDE HEADER ****************************
**
**  INCLUDE NAME:	stats.h
**
**  DESCRIPTION:	This file contains declarations for all
**			statistics tracked by all layers running
**			under VRTX.
**
**  NOTES:		This file contains declarations only.  The
**			actual definition (storage allocation) will
**			be made in another file which is at this
**			time unknown.  (And unwritten.)
**
**  UPDATE LOG:
**  Date	Name		V/R	Comments
**  --------	-----------   	---	-----------------------------
**  07/10/85	Roy Larsen	1.0	Original submital to SCCS
**  04/20/87	Martin Dorr	1.0	Added traffic flow structure
**
**********************************************************************/

/*
**  Declare module traffic flow statistics.
*/

struct tr_flow
{
	USHORT	epbs;
	USHORT	bdbs;
	ULONG	numb;
};

#define	TR_SZ	11

struct traffic
{
	struct	tr_flow	flow[TR_SZ][TR_SZ];
	ULONG		stime;
	ULONG		etime;
	ULONG		ints;
	ULONG		host_int;
	ULONG		txf_cnt;
	ULONG		txe_cnt;
	ULONG		rxf_cnt;
	ULONG		rxe_cnt;

	ULONG		err_gblock;
	ULONG		cnt_qpost;
	ULONG		cnt_qpend;

	USHORT		max_l_epbs;
	USHORT		max_l_bdbs;
	USHORT		max_u_epbs;
	USHORT		max_u_bdbs;
	USHORT		cur_epbs;
	USHORT		cur_bdbs;
	USHORT		cnt_in_epbs;
	USHORT		cnt_in_bdbs;
	USHORT		cnt_out_epbs;
	USHORT		cnt_out_bdbs;
	USHORT		cnt_ib_epbs;
	USHORT		cnt_ib_bdbs;
	USHORT		cnt_ob_epbs;
	USHORT		cnt_ob_bdbs;
	USHORT		max_ir_epbs;
	USHORT		max_ir_bdbs;
	USHORT		max_or_epbs;
	USHORT		max_or_bdbs;

	USHORT		suc_r_bdbs;
	USHORT		err_r_bdbs;
	USHORT		suc_r_epbs;
	USHORT		err_r_epbs;
};

/*
**  Declare link layer statistics.
*/

struct link
{
	UBYTE	name[8];

	ULONG	dm_sent;
	ULONG	sabm_sent;
	ULONG	disc_sent;
	ULONG	ua_sent;
	ULONG	frmr_sent;
	ULONG	rej_sent;
	ULONG	rr_sent;
	ULONG	rnr_sent;
	ULONG	i_sent;

	ULONG	dm_rcvd;
	ULONG	sabm_rcvd;
	ULONG	disc_rcvd;
	ULONG	ua_rcvd;
	ULONG	frmr_rcvd;
	ULONG	rej_rcvd;
	ULONG	rr_rcvd;
	ULONG	rnr_rcvd;
	ULONG	i_rcvd;

	ULONG	tx_underflow;
	ULONG	rx_char_overrun;
	ULONG	rx_buffer_overrun;
	ULONG	rx_crc_error;
	ULONG	rx_sdlc_abort;
	ULONG	illegal_address;
	ULONG	bad_command;

	ULONG	timer_interrupts;
	ULONG	enter_timer_recovery;
	ULONG	exit_timer_recovery;
	ULONG	timer_request_count;
	ULONG	timer_transmit_count;
	ULONG	enter_dm_timeout;

	ULONG	ciq_full;
	ULONG	valid_frames_rcvd;
	ULONG	went_disconnected;

	ULONG	bad_nr;
	ULONG	poll_rcvd;
	ULONG	unknown_s_frame;
	ULONG	discard_i_frame;
	ULONG	short_i_frame;
	ULONG	credit_frames;

	ULONG	link_queue_full;
	ULONG	net_queue_full;
	ULONG	timer_queue_full;
};

/*
**  Declare network layer statistics.
*/

struct network
{
	UBYTE	name[8];
};

/*
**  Declare transport layer statistics.
*/

struct xport
{
	UBYTE	name[8];
	ULONG	bad_t_event;	/* incremented on unknown EPB types   */
};

/*
**  Declare uci interface statistics.
*/

struct uci
{
	UBYTE	name[8];
};

/*
**  Declare network management statistics.
*/

struct manage
{
	UBYTE	name[8];
};

/*
**  Declare operating system statistics.
*/

struct os
{
	UBYTE	name[8];

	ULONG	get_bdb_error;
	ULONG	get_epb_error;
	ULONG	return_bdb_error;
	ULONG	return_epb_error;
};

/*
**  Combine all statistics into common structure.
*/

struct stats
{
	struct	link	link;
	struct	network	network;
	struct	xport	xport;
	struct	uci	uci;
	struct	manage	manage;
	struct	os	os;
};

/*
**  Declare entire statistics structure.
*/

struct	stats	stats;

/*
**  Define abbreviations for convenience.
*/

#define	L_STATS	stats.link
#define	N_STATS	stats.network
#define	T_STATS	stats.xport
#define	U_STATS	stats.uci
#define	M_STATS	stats.manage
#define	O_STATS	stats.os
