/*@(#)system.h	35.1*/
/*IH*********************** INCLUDE HEADER ****************************
**
**  INCLUDE NAME:	system.h
**
**  DESCRIPTION:	This file contains function declarations and
**			error mnemonics for the VRTX C language inter-
**			face.
**
**  NOTES:
**
**********************************************************************/

/*
**  Define error mnemonics.
*/

/*	Mnemon	Value	   Meaning		Affected commands    */
/*	------	-----	   --------------------	-------------------- */

#define	RET_OK	0	/* Sucessful return.	[Valid commands.]    */
#define	ER_TID	1	/* Task ID error.	TCREATE, TDELETE,    */
			/*			TSUSPEND, TRESUME,   */
			/*			TPRIORITY, TINQUIRY  */
#define	ER_TCB	2	/* No TCBs available.	TCREATE		     */
#define	ER_MEM	3	/* No memory available.	GBLOCK, PCREATE,     */
			/*			PEXTEND, QCREATE     */
#define	ER_NMB	4	/* Not a memory block.	RBLOCK		     */
#define	ER_MIU	5	/* Mailbox in use.	POST		     */
#define	ER_ZMW	6	/* Zero message.	POST, QPOST	     */
#define	ER_BUF	7	/* Buffer full.		RXCHR		     */
#define	ER_WTC	8	/* WAITC in progress.	WAITC		     */
#define	ER_ISC	9	/* Invalid system call.	[Invalid commands.]  */
#define	ER_TMO	10	/* Time-out.		PEND, QPEND	     */
#define	ER_NMP	11	/* No message present.	ACCEPT, QACCEPT	     */
#define	ER_QID	12	/* Queue Id error.	QPOST, QPEND,	     */
			/*			QACCEPT, QCREATE     */
#define	ER_QFL	13	/* Queue full.		QPOST		     */
#define	ER_PID	14	/* Partition ID error.	GBLOCK, RBLOCK,	     */
			/*			PCREATE, PEXTEND     */
#define	ER_INI	15	/* Workspace too small.	VRTX_INIT	     */
#define	ER_ICP	17	/* Invalid VCT param.	VRTX_INIT	     */

/*
**  Miscellaneous mnemonics.
*/

#define	NO_TIMEOUT	0	/* Used to indicate no timeout value */
				/* on calls to PEND and QPEND.	     */
#ifdef SIM
#define MSPT		1000	/* Milliseconds per "VRTX" tick*/
#else
#define	MSPT		16	/* Milliseconds per "VRTX" tick.     */
#endif

#define USER_MODE	0	/* User mode for tasks	 	     */
#define SUPER_MODE	1	/* Supervisor mode for tasks	     */

/*
**  Total number of queues are defined here.  The actual queue number
**  assignments are made in epb.h
*/

#define	TOTAL_QUEUES	14	/* Total number of queues.	     */

/*
**  Queue depth.
**  NOTE:  Any new queue depth defines must be added to the total
**         queue depth define.
*/

#define TIMER_Q_DEPTH	300	/* # of elements in timer queue.    */
#define	MAC_Q_DEPTH	300	/* # of elements in MAC queue.	    */
#define LINK_Q_DEPTH	300	/* # of elements in link queue.     */
#define NET_Q_DEPTH	300	/* # of elements in network queue.  */
#define	XPORT_Q_DEPTH	300	/* # of elements in transport queue.*/
#define	SESSION_Q_DEPTH	250	/* # of elements in session queue.  */
#define	PRES_Q_DEPTH	250	/* # of elements in presentation queue	    */
#define	ACSE_Q_DEPTH	250	/* # of elements in ACSE queue	    */
#define	ROSE_Q_DEPTH	250	/* # of elements in ROSE queue	    */
#define	FTAM_Q_DEPTH	250	/* # of elements in FTAM queue	    */
#define	DIRS_Q_DEPTH	50	/* # of elements in Directory Ser.  */
#define OD_Q_DEPTH      50      /* # of elements in OD queue        */
#define NET_MGR_Q_DEPTH	100	/* # of elements in network_mgr Q   */
#define SM_Q_DEPTH	100	/* # of elements in station_mgr Q   */
#define	CE_Q_DEPTH	300	/* # of elements in CE queue	    */
#define	EG_Q_DEPTH	10	/* # of elements in EG queue	    */

/*
**  Total number of VRTX controlled queue elements.  (This is needed
**  information for the configuration table.)
*/

#define TOTAL_Q_DEPTH ( TIMER_Q_DEPTH	+ \
			MAC_Q_DEPTH	+ \
			LINK_Q_DEPTH	+ \
			NET_Q_DEPTH	+ \
			XPORT_Q_DEPTH	+ \
			SESSION_Q_DEPTH	+ \
			PRES_Q_DEPTH	+ \
			ACSE_Q_DEPTH	+ \
			ROSE_Q_DEPTH	+ \
			FTAM_Q_DEPTH	+ \
			NET_MGR_Q_DEPTH + \
			SM_Q_DEPTH 	+ \
			DIRS_Q_DEPTH	+ \
                        OD_Q_DEPTH      + \
			EG_Q_DEPTH	+ \
			CE_Q_DEPTH	)

/*
**  Partition ID mnemonics.
**  NOTE:  When adding or deleting a memory partition define, the 
**         total partitions define must be updated.
*/

#define	EPB_PID		1	/* Event Parameter Block pool ID.    */
#define BDB_PID		2	/* Buffer Descriptor Block pool ID.  */
#define FDB_PID		3	/* Frame Descriptor Block pool ID.   */

#define TOTAL_PARTITIONS 3	/* Total number of memory partitions.*/

#define TOTAL_BLOCKS	700	/* Total number of memory blocks.    */

/*
**  Task ID mnemonics.
**  NOTE:  As tasks are added or deleted from this list, the total
**         tasks define must reflect the total number of VRTX tasks.
*/

#define TIMER_TID	1	/* System timer task ID.	    */
#define	MAC_TID		2	/* MAC task ID.			    */
#define	LINK_TID	3	/* Link inbound task ID.	    */
#define	NET_TID		4	/* Network task ID.		    */
#define XPORT_TID	5	/* Transport task ID.		    */
#define	SESSION_TID	6	/* Session task ID.		    */
#define	PRES_TID	7	/* PRES task ID.		    */
#define	ACSE_TID	8	/* ACSE task ID.		    */
#define	ROSE_TID	9	/* ROSE task ID.		    */
#define	FTAM_TID	10	/* FTAM task ID.		    */
#define NET_MGR_TID	11	/* Network management task ID	    */
#define SM_TID		12	/* Station management task ID	    */
#define	DIRS_TID	13	/* Directory Services task ID	    */
#define OD_TID          14      /* Object Dictionary task ID        */
#define	CE_TID		15	/* Common Environment task ID	    */
#define	EG_TID		16	/* Exception Generator task ID	    */

#define	TOTAL_TASKS	16	/* Total numer of task under VRTX.  */

/*
**  Task priorities.
*/

#define	EG_PRI		10	/* Exception Generator task priority */
#define TIMER_PRI	10	/* System timer task priority.	    */
#define XPORT_PRI	10	/* Transport task priority.	    */
#define	MAC_PRI		10	/* MAC task priority.		    */
#define	LINK_PRI	10	/* Link inbound task priority.	    */
#define	NET_PRI		10	/* Network task priority.	    */
#define	CE_PRI		10	/* Common Environment task priority */
#define	SESSION_PRI	10	/* Session task priority.	    */
#define	PRES_PRI	10	/* PRES task priority.		    */
#define	ACSE_PRI	10	/* ACSE task priority.		    */
#define	ROSE_PRI	10	/* ROSE task priority.		    */
#define	FTAM_PRI	10	/* FTAM task priority.		    */
#define NET_MGR_PRI	10	/* Network management task priority */
#define SM_PRI		10	/* Station management task priority */
#define	DIRS_PRI	10	/* Directory Services task priority */
#define OD_PRI          10      /* Object Dictonary task priority   */

/*
**  Task timeslice values.
*/

#define	EG_SLICE	1	/* Exception Generator task timeslice */
#define TIMER_SLICE	1	/* System timer task timeslice.	    */
#define	MAC_SLICE	3	/* MAC task timeslice.		    */
#define	LINK_SLICE	1	/* Link inbound task timeslice.	    */
#define	NET_SLICE	1	/* Network task timeslice.	    */
#define XPORT_SLICE	4	/* Transport task timeslice.	    */
#define	CE_SLICE	3	/* Common Environment task timeslice */
#define	SESSION_SLICE	2	/* Session task timeslice.	    */
#define	PRES_SLICE	2	/* PRES task timeslice.		    */
#define	ACSE_SLICE	2	/* ACSE task timeslice.		    */
#define	ROSE_SLICE	2	/* ROSE task timeslice.		    */
#define	FTAM_SLICE	2	/* FTAM task timeslice.		    */
#define NET_MGR_SLICE	1	/* Network management task timeslice */
#define SM_SLICE	1	/* Station management task timeslice */
#define	DIRS_SLICE	1	/* Directory Services task timeslice */
#define OD_SLICE        1       /* Object Dictionary task timeslice  */

/*
**  Task stack sizes.
*/

#define	TIMER_STK_SIZE		1024
#define	MAC_STK_SIZE		1024
#define	LINK_STK_SIZE		1024
#define	NET_STK_SIZE		1024
#define	XPORT_STK_SIZE		1024
#define	SESSION_STK_SIZE	1024
#define	PRES_STK_SIZE		1024
#define	ACSE_STK_SIZE		1024
#define	ROSE_STK_SIZE		1024
#define	FTAM_STK_SIZE		1024
#define	NET_MGR_STK_SIZE	1024
#define	SM_STK_SIZE		1024
#define	DIRS_STK_SIZE		1024
#define OD_STK_SIZE             1024
#define	CE_STK_SIZE		1024
#define	EG_STK_SIZE		1024

/*
**  VRTX system defines.
*/

#define	VBR		0xfff00000	/* VBR base address	    */

#define	VRTX_VCT_VECTOR	(VBR + 0x100)	/* VCT pointer location     */
#define	VRTX_ENTRY_ADDR	0xffe8d000	/* Location of VRTX in RAM  */

#define	AUTO_VECTOR_1	(VBR + 0x64)	/* autovector 1 address	    */
#define	AUTO_VECTOR_2	(VBR + 0x68)	/* autovector 2 address	    */
#define	AUTO_VECTOR_3	(VBR + 0x6C)	/* autovector 3 address	    */
#define	AUTO_VECTOR_4	(VBR + 0x70)	/* autovector 4 address	    */
#define	AUTO_VECTOR_5	(VBR + 0x74)	/* autovector 5 address	    */
#define	AUTO_VECTOR_6	(VBR + 0x78)	/* autovector 6 address	    */
#define	AUTO_VECTOR_7	(VBR + 0x7C)	/* autovector 7 address	    */

#define	TRAP_VECTOR_0	(VBR + 0x80)	/* trap 0 address	    */
#define	TRAP_VECTOR_1	(VBR + 0x84)	/* trap 1 address	    */
#define	TRAP_VECTOR_2	(VBR + 0x88)	/* trap 2 address	    */
#define	TRAP_VECTOR_3	(VBR + 0x8C)	/* trap 3 address	    */
#define	TRAP_VECTOR_4	(VBR + 0x90)	/* trap 4 address	    */
#define	TRAP_VECTOR_5	(VBR + 0x94)	/* trap 5 address	    */
#define	TRAP_VECTOR_6	(VBR + 0x98)	/* trap 6 address	    */
#define	TRAP_VECTOR_7	(VBR + 0x9C)	/* trap 7 address	    */
#define	TRAP_VECTOR_8	(VBR + 0xA0)	/* trap 8 address	    */
#define	TRAP_VECTOR_9	(VBR + 0xA4)	/* trap 9 address	    */
#define	TRAP_VECTOR_10	(VBR + 0xA8)	/* trap 10 address	    */
#define	TRAP_VECTOR_11	(VBR + 0xAC)	/* trap 11 address	    */
#define	TRAP_VECTOR_12	(VBR + 0xB0)	/* trap 12 address	    */
#define	TRAP_VECTOR_13	(VBR + 0xB4)	/* trap 13 address	    */
#define	TRAP_VECTOR_14	(VBR + 0xB8)	/* trap 14 address	    */
#define	TRAP_VECTOR_15	(VBR + 0xBC)	/* trap 15 address	    */

/*
**  The formula for the following define is given on page 4-9 of the
**  VRTX User's Guide.
*/

#define VRTX_STACK_SIZE	0
#define TMP_STACK_SIZE	28	/* Size(bytes) of VRTX /task stack   */
#define ISR_STACK_SIZE	0	/* VRTX Interrupt stack size	    */
#define COMP_DISABLE	7	/* Interrupt disable level (0=>7)   */
#define USER_STACK_SIZE	0	/* User task size reported in VCT.  */

#define VRTX_RAM_SIZE ( 				    \
			316				  + \
			((80 + VRTX_STACK_SIZE 		  + \
			  USER_STACK_SIZE) * TOTAL_TASKS) + \
			(48 * TOTAL_PARTITIONS)		  + \
			( 2 * TOTAL_BLOCKS)		  + \
			(14 * TOTAL_QUEUES)		  + \
			( 4 * TOTAL_Q_DEPTH)		  + \
			(ISR_STACK_SIZE))

/*
**  VRTX configuration table (vct) structure. 
**	See page 4-2 of the VRTX User's Manual.
*/

struct	vct
{
	BYTE	*sys_ram;	/* pointer to vrtx system ram	    */
	LONG	 sys_ram_size;	/* vrtx system ram size		    */
	SHORT	 sys_stk_size;	/* vrtx system stack size	    */
	SHORT	 ISR_stk_size;	/* ISR stack space		    */
	LONG	 reserved1;	/* Must be zero			    */
	LONG	 reserved2;	/* Must be zero			    */
	SHORT	 comp_disable;	/*				    */
	SHORT	 user_stk_size;	/* user stack size		    */
	LONG	 reserved3;	/* Must be zero			    */
	SHORT	 user_task_cnt;	/* total number of tasks	    */
	SHORT	 reserved4;	/* Must be zero			    */
	VOID	(*txrdy_addr)();/* TXRDY driver address		    */
	VOID	(*sys_tcreate)();/* pointer to tcreate hook function */
	VOID	(*sys_tdelete)();/* pointer to tdelete hook function */
	VOID	(*sys_tswch)();	/* pointer to tswitch hook function */
	BYTE	*cvt_addr;	/* Component Vector Table address   */
};

/*
** VRTX TCB structure.  (See page C-4 of the VRTX User's Manual.)
*/

struct	tcb
{
struct	tcb	*tbnext;
struct	tcb	*tblink;
	BYTE	*tbext;
	BYTE	 tbpri;
	BYTE	 tbid;
	SHORT	 tbstat;
	DEFAULT	 tbd0;
	DEFAULT	 tbd1;
	DEFAULT	 tbd2;
	DEFAULT	 tbd3;
	DEFAULT	 tbd4;
	DEFAULT	 tbd5;
	DEFAULT	 tba0;
	DEFAULT	 tba1;
	DEFAULT	 tba2;
	DEFAULT	 tba3;
	BYTE	*tbsp;
	BYTE	*tbusp;
	BYTE	*tbtstack;
	SHORT	 tbflags;
	DEFAULT	 tbdelay;
	SHORT	 tbqid;
	DEFAULT	 reserved;
};

/*
**	Declarations for user arrays which receive information from VRTX calls.
*/

typedef	struct t_info_rec
{	DEFAULT	task_id;
	DEFAULT	task_priority;
	DEFAULT	task_status;
}T_INFO_REC;


typedef	struct q_info_rec
{	DEFAULT	msg_addr;
	DEFAULT	count;
}Q_INFO_REC;
