/*
 *	Copyright (c) 1987 Motorola Inc
 *	All Rights Reserved
 *	@(#)vrtx.h	35.1
 */

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF Motorola	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ident	"@(#)vrtx.h	35.1"

/*IH*********************** INCLUDE HEADER ****************************
**
**  INCLUDE NAME:	vrtx.h
**
**  DESCRIPTION:	This file contains function declarations and
**			error mnemonics for the VRTX C language inter-
**			face.
**
**  NOTES:
**
**********************************************************************/

/*
**  Define error mnemonics.
*/

/*	Mnemon	Value	   Meaning		Affected commands    */
/*	------	-----	   --------------------	-------------------- */

#define	RET_OK	0	/* Sucessful return.	[Valid commands.]    */

#define	ER_TID	1	/* Task ID error.	TCREATE, TDELETE,    */
			/*			TSUSPEND, TRESUME,   */
			/*			TPRIORITY, TINQUIRY  */

#define	ER_TCB	2	/* No TCBs available.	TCREATE		     */

#define	ER_MEM	3	/* No memory available.	GBLOCK, PCREATE,     */
			/*			PEXTEND, QCREATE     */

#define	ER_NMB	4	/* Not a memory block.	RBLOCK		     */

#define	ER_MIU	5	/* Mailbox in use.	POST		     */

#define	ER_ZMW	6	/* Zero message.	POST, QPOST	     */

#define	ER_BUF	7	/* Buffer full.		RXCHR		     */

#define	ER_WTC	8	/* WAITC in progress.	WAITC		     */

#define	ER_ISC	9	/* Invalid system call.	[Invalid commands.]  */

#define	ER_TMO	10	/* Time-out.		PEND, QPEND	     */

#define	ER_NMP	11	/* No message present.	ACCEPT, QACCEPT	     */

#define	ER_QID	12	/* Queue Id error.	QPOST, QPEND,	     */
			/*			QACCEPT, QCREATE     */

#define	ER_QFL	13	/* Queue full.		QPOST		     */

#define	ER_PID	14	/* Partition ID error.	GBLOCK, RBLOCK,	     */
			/*			PCREATE, PEXTEND     */

#define	ER_INI	15	/* Workspace too small.	VRTX_INIT	     */

#define	ER_ICP	17	/* Invalid VCT param.	VRTX_INIT	     */

/*
**  Miscellaneous mnemonics.
*/

#define	NO_TIMEOUT	0	/* Used to indicate no timeout value */
				/* on calls to PEND and QPEND.	     */
#define	MSPT		50	/* Milliseconds per "VRTX" tick.     */

#define USER_MODE	0	/* User mode for tasks	 	     */
#define SUPER_MODE	1	/* Supervisor mode for tasks	     */

/*
**  Queue ID mnemonics.
**  NOTE:  Any new queue defines must have their queue depth defined
**         below.  The total queues define must reflect the total
**         number of queues to be created by VRTX.
*/

#define	CI_QID		1	/* Common Interface queue ID.	     */
#define	TIMER_QID	2	/* System timer queue ID.	     */
#define	MAC_QID		3	/* MAC layer queue ID.		     */

#define	TOTAL_QUEUES	3	/* Total number of queues.	     */


/*
**  Queue depth.
**  NOTE:  Any new queue depth defines must be added to the total
**         queue depth define.
*/

#define TIMER_Q_DEPTH	5	/* # of elements in Timer queue.    */
#define CI_Q_DEPTH	200	/* # of elements in CI queue.       */
#define	MAC_Q_DEPTH	50	/* # of elements in MAC queue.	    */

/*
**  Total number of VRTX controlled queue elements.  (This is needed
**  information for the configuration table.)
*/

#define TOTAL_Q_DEPTH ( TIMER_Q_DEPTH	+ \
			CI_Q_DEPTH	+ \
			MAC_Q_DEPTH	  )

/*
**  Partition ID mnemonics.
**  NOTE:  When adding or deleting a memory partition define, the 
**         total partitions define must be updated.
*/

#define	RT_PID		0	/* Registration Table Partition ID.  */
#define	EPB_PID		1	/* Event Parameter Block pool ID.    */
#define BUF_PID		2	/* Data Buffer Block Partition ID.   */

#define TOTAL_PARTITIONS 3	/* Total number of memory partitions.*/

#define EPB_BLOCKS	50	/* Number of EPB Blocks */
#define BUF_BLOCKS	25	/* Number of BUF Blocks */

#define TOTAL_BLOCKS	( 1 + \
			 EPB_BLOCKS + \
			 BUF_BLOCKS )

/*
**  Task ID mnemonics.
**  NOTE:  As tasks are added or deleted from this list, the total
**         tasks define must reflect the total number of VRTX tasks.
*/

#define	CI_TID		1	/* CI task ID.			    */
#define TIMER_TID	2	/* System timer task ID.	    */
#define	MAC_TID		3	/* MAC task ID.			    */

#define	TOTAL_TASKS	3	/* Total number of task under VRTX.  */

/*
**  Task priorities.
*/

#define TIMER_PRI	1	/* System timer task priority.	    */
#define	CI_PRI		2	/* CI task priority.		    */
#define	MAC_PRI		3	/* MAC task priority.		    */

/*
**  68000 Trap/Auto Vector Locations
*/

#define	VBR		0x00000000	/* VBR base address	     */

#define	AUTO_VECTOR_1	(VBR + 0x64)	/* autovector 1 address	    */
#define	AUTO_VECTOR_2	(VBR + 0x68)	/* autovector 2 address	    */
#define	AUTO_VECTOR_3	(VBR + 0x6C)	/* autovector 3 address	    */
#define	AUTO_VECTOR_4	(VBR + 0x70)	/* autovector 4 address	    */
#define	AUTO_VECTOR_5	(VBR + 0x74)	/* autovector 5 address	    */
#define	AUTO_VECTOR_6	(VBR + 0x78)	/* autovector 6 address	    */
#define	AUTO_VECTOR_7	(VBR + 0x7C)	/* autovector 7 address	    */

#define	TRAP_VECTOR_0	(VBR + 0x80)	/* trap 0 address	    */
#define	TRAP_VECTOR_1	(VBR + 0x84)	/* trap 1 address	    */
#define	TRAP_VECTOR_2	(VBR + 0x88)	/* trap 2 address	    */
#define	TRAP_VECTOR_3	(VBR + 0x8C)	/* trap 3 address	    */
#define	TRAP_VECTOR_4	(VBR + 0x90)	/* trap 4 address	    */
#define	TRAP_VECTOR_5	(VBR + 0x94)	/* trap 5 address	    */
#define	TRAP_VECTOR_6	(VBR + 0x98)	/* trap 6 address	    */
#define	TRAP_VECTOR_7	(VBR + 0x9C)	/* trap 7 address	    */
#define	TRAP_VECTOR_8	(VBR + 0xA0)	/* trap 8 address	    */
#define	TRAP_VECTOR_9	(VBR + 0xA4)	/* trap 9 address	    */
#define	TRAP_VECTOR_10	(VBR + 0xA8)	/* trap 10 address	    */
#define	TRAP_VECTOR_11	(VBR + 0xAC)	/* trap 11 address	    */
#define	TRAP_VECTOR_12	(VBR + 0xB0)	/* trap 12 address	    */
#define	TRAP_VECTOR_13	(VBR + 0xB4)	/* trap 13 address	    */
#define	TRAP_VECTOR_14	(VBR + 0xB8)	/* trap 14 address	    */
#define	TRAP_VECTOR_15	(VBR + 0xBC)	/* trap 15 address	    */

/*
**  VRTX system defines.
*/

#define	VRTX_VCT_VECTOR	(VBR + 0x100)	/* VCT pointer location      */
#define	VRTX_ENTRY_ADDR	0x00f85a00	/* Location of VRTX in EPROM */

	/*  VRTX Configuration Control Flags  */

#define VRTX_COMPONENTS		   1	/* Number of VRTX components */
#define DISABLE_LEVEL		   7	/* VRTX inter. disable level */
#define ISR_STACK_SWITCH	   0	/* ISR Stack Switching Flag  */
					/*   0 ==> No stack switch   */
					/*   1 ==> Switch	     */

#define USER_TASKS	(TOTAL_TASKS)

	/*  VRTX System Variables and Control Block Sizes  */

#define SYS_VARIABLES   	 308	/* Size of VRTX Working Storage	*/
#define TCB_SIZE	  	  80	/* VRTX Task Control Block size	*/ 
#define PCB_SIZE		  20	/* Partition Control Block size	*/
#define QCB_SIZE		  14	/* VRTX Queue Control Block size*/
#define BLK_PTR_SIZE		   8	/* Space per VRTX memory block	*/
#define Q_ENTRY_SIZE		   4	/* Space per VRTX queue element	*/

	/*  VRTX and User Stack Size Defintions  */

#define ISR_STACK_SIZE		 100
#define USER_STACK_SIZE		 512

#if USER_STACK_SIZE == 0
#define TEMP_STK_SIZE             64   	    
#else
#define TEMP_STK_SIZE    	   0
#endif

#if USER_TASKS != 0
#define UMODE_XFR_STK		  32
#else
#define UMODE_XFR_STK		   0 
#endif

#if ISR_STACK_SWITCH == 0
#define SYS_ISR_STACK_SIZE	(ISR_STACK_SIZE)
#else
#define SYS_ISR_STACK_SIZE	   0
#endif

#define VRTX_STACK_SIZE	(( VRTX_COMPONENTS * TEMP_STK_SIZE) + \
			 UMODE_XFR_STK + \
			 SYS_ISR_STACK_SIZE )

/*
**  The formula for the following define is given on pages 4-5 thru 4-7
**  in version 3.0 of the VRTX User's Guide.
*/

#define VRTX_RAM_SIZE 	( ( SYS_VARIABLES ) 			\
			+ ( TCB_SIZE * TOTAL_TASKS )		\
			+ ( VRTX_STACK_SIZE ) 			\
			+ ( PCB_SIZE * TOTAL_PARTITIONS ) 	\
			+ ( BLK_PTR_SIZE * TOTAL_BLOCKS )	\
			+ ( QCB_SIZE * TOTAL_QUEUES ) 		\
			+ ( Q_ENTRY_SIZE * TOTAL_Q_DEPTH ) 	\
			+ ( USER_STACK_SIZE * USER_TASKS ) 	\
			+ ( ISR_STACK_SWITCH * ISR_STACK_SIZE ) )

/*
**  VRTX configuration table (vct) structure. 
**	See page 4-2 of the VRTX User's Manual.
*/

struct	vct
{
	BYTE	*sys_ram;	/* pointer to vrtx system ram	    */
	LONG	 sys_ram_size;	/* vrtx system ram size		    */
	SHORT	 sys_stk_size;	/* vrtx system stack size	    */
	SHORT	 isr_stk_size;	/* ISR stack space		    */
	LONG	 reserved_1;	/* Must be zero			    */
	LONG	 reserved_2;	/* Must be zero			    */
	SHORT	 disable_level;	/*				    */
	SHORT	 user_stk_size;	/* user stack size		    */
	LONG	 reserved_3;	/* Must be zero			    */
	SHORT	 user_task_cnt;	/* total number of tasks	    */
	SHORT	 reserved_4;	/* Must be zero			    */
	VOID	(*txrdy_addr)();/* TXRDY driver address		    */
	VOID	(*sys_tcreate)();/* pointer to tcreate hook function */
	VOID	(*sys_tdelete)();/* pointer to tdelete hook function */
	VOID	(*sys_tswch)();	/* pointer to tswitch hook function */
	BYTE	*cvt_addr;	/* Component Vector Table address   */
};

/*
** VRTX TCB structure.  (See page C-4 of the VRTX User's Manual.)
*/

struct	tcb
{
struct	tcb	*tbnext;
struct	tcb	*tblink;
	BYTE	*tbext;
	BYTE	 tbpri;
	BYTE	 tbid;
	SHORT	 tbstat;
	DEFAULT	 tbd0;
	DEFAULT	 tbd1;
	DEFAULT	 tbd2;
	DEFAULT	 tbd3;
	DEFAULT	 tbd4;
	DEFAULT	 tbd5;
	DEFAULT	 tba0;
	DEFAULT	 tba1;
	DEFAULT	 tba2;
	DEFAULT	 tba3;
	BYTE	*tbsp;
	BYTE	*tbusp;
	BYTE	*tbtstack;
	SHORT	 tbflags;
	DEFAULT	 tbdelay;
	SHORT	 tbqid;
	DEFAULT	 reserved;
};

/*
**	Declarations for user arrays which receive information from VRTX calls.
*/

typedef	struct t_info_rec
{	DEFAULT	task_id;
	DEFAULT	task_priority;
	DEFAULT	task_status;
}T_INFO_REC;


typedef	struct q_info_rec
{	DEFAULT	msg_addr;
	DEFAULT	count;
}Q_INFO_REC;
