/* @(#)x409.h	35.1 */
/*IH*********************** INCLUDE HEADER ****************************
**
**  FILE NAME:		x409.h                    
**
**  DESCRIPTION:	This file contains definitions related to
**			the parsing and building of X.409 data 
**			streams.
**
**********************************************************************/

/*
**	Universal Primitive Types:
*/
#define	UP_EOC			0x00
#define	UP_BOOLEAN		0x01
#define	UP_INTEGER		0x02
#define	UP_BIT_STRING		0x03
#define	UP_OCTET_STRING		0x04
#define	UP_NULL			0x05
#define UP_OBJECT_ID		0x06
#define UP_OBJECT_DESC		0x07
#define	UP_PRT_STRING		0x13
#define	UP_T61_STRING		0x14
#define UP_IA5_STRING		0x16
#define	UP_UTC_TIME		0x17
#define	UP_GENERALIZED_TIME	0x18

/*
**	Universal Constructor Types:
*/
#define	UC_SEQUENCE		0x30
#define	UC_SET			0x31
#define UC_EXTERNAL		0x28

/*
**	ASN.1 Identifier encoding
*/
#define	AP_CLASS		0x40		/* Application class */
#define	CS_CLASS		0x80		/* Context specific class */
#define	PR_CLASS		0xc0		/* Private class */

#define CONSTRUCTED		0x20

/*
** 	Context Specific - Encoding Choice codes
*/
#define SINGLE_ASN1_TYPE	0x80
#define OCTET_ALIGNED		0x81
#define ARBITRARY		0x82


/*
**	X.409 Information Structure Definition:
*/
typedef	struct
{
	ULONG		type;		/* contains class, form, & id	*/
	ULONG		length;		/* length (in bytes) of item	*/
	UBYTE		*contents;	/* pointer to item contents	*/
	UBYTE		*limit;		/* contents cannot go past here	*/
	BOOLEAN		 indef;		/* this item has indefinite len */
} X409_INFO;

/*
** Object Ids
*/
#define OBJIDSZ			6	/* MAX No of Components in OBJECT ID  */


/*
** network management 
*/
#define  NOOFNMIDS  		10	/* No. of valid NM Resource IDs	*/


typedef struct Obj_Id {		/* ASN.1 OBJECT IDENTIFIER TYPE C-struct */
	LONG cnt;
	LONG element[OBJIDSZ];
} OBJ_ID;

/*
** OBJECT ID descriptive information
*/
#define BAD_OBJID	-1		/* objid not found in table */

#define PACKING_MULT	40		/* The multiplier used to pack the */
					/* first two object subids into the*/
					/* first octet.			   */

/*
** Abstract Syntax Names
*/
#define NOOFAS			12	/* No of abstract syntax names      */

#define NULL_ASOBJ		0	/* NULL as name value		      */
#define ACSE_ASOBJ		1	/* ACSE abstract syntax name index    */
#define FTAM_PCI_ASOBJ		2	/* FTAM PCI abstract syntax index     */
#define FTAM_NBS1_ASOBJ		3	/* FTAM NBS1 abstract syntax index    */
#define MMS_ASOBJ		4	/* MMS abstract syntax index          */
#define FTAM_NBS2_ASOBJ		5	/* FTAM NBS2 abstract syntax index    */
#define UNSTRUCT_BIN_ASOBJ	6	/* UNSTRUCT BINARY abstract syntax index*/
#define PRIVATE_ASOBJ		7	/* PRIVATE abstract syntax index      */
#define FTAM_FADU		8
#define UNSTRUCT_TEXT_ASOBJ 	9
#define DUA_ASOBJ		10
#define NM_AGENT_ASOBJ		11	/* Net Mgt Agent abstract syntax index*/



/*
** Transfer Syntax Names
*/
#define NOOFTS			4	/* No of TS supported in MAP3.0 */

#define NULL_OBJ		0	/* NULL ts name value		      */
#define NBS_TS1_OBJ		1	/* NBS-TS1 transfer syntax name index */
#define ISO_8825_OBJ		2	/* ISO ASN.1 transfer syntax index */
#define SIMPLE_BIN_OBJ		3	/* SIMPLE BINARY transfer syntax index*/


/*
** Application Contexts
*/
#define NOOFAC			8	/* No of ACs supported in MAP 3.0 */

/*				0		  ?????????	*/
#define	MMS_ACOBJ		1	/* MMS Application Context */
#define FTAM_ACOBJ		2	/* FTAM Application Context */
#define ROSE_M1A1O1_ACOBJ	3	/* ROSE Mapping to ACSE/Pres 1, 
					    Association Class 1,
					    Operation Class 1   */
#define ROSE_M1A1O2_ACOBJ	4	/* ROSE Mapping to ACSE/Pres 1, 
					    Association Class 1,
					    Operation Class 2   */
#define PRIVATE_ACOBJ		5	/* PRIVATE Application Context 	*/
#define DUA_ACOBJ		6	/* DUA Application Context	*/
#define NM_AGENT_ACOBJ		7	/* NM Agent Appl Context index	*/


/*
** These constants are the values used in different branches of the
** Registration Authority tree.  They are used to create the Object
** Identifier Tables.
*/
		/* subid 1 */
#define ISO			1
#define JOINT			2
#define CCITT			3
/* 
** Network Management -- the resource class object ID tree for MAP NM 
** 			 is temporarily rooted at the root-level.  It
**			 will later be moved to an appropriate sublevel
*/
#define	RC_NETM			1		


		/* subid 2 */
#define IDENT_ORG		3	/* Identified Organization 	*/
#define STANDARD		0	/* International Standard  	*/
#define	DS5			5	/* DUA subid		   	*/
#define	ASN1			1	/* Abstract Syntax Notation 1 	*/
#define ASSOC_CONTROL 		2	/* Association Control		*/
			/* Network Management arcs */
#define	RC_LAYER		1	/* Layer */
#define	RC_SYSM			2	/* Systems Management */


		/* subid 3 */
#define NBS			4
#define ICD			9999	/* */
#define	MMS_SPEC		9506
#define FTAM_SPEC		8571
#define BASIC_ENCODING		1
#define DS3			3
#define DS9			9
#define ABSTRACT_SYNTAX_3	1
			/* net management */
#define	RC_PHYS			1	/* Physical */
#define	RC_LINK			2	/* Data Link */
#define	RC_NETW			3	/* Network */
#define	RC_TRAN			4	/* Transport */
#define	RC_SESS			5	/* Session */
#define	RC_PRES			6	/* Presentation */
#define	RC_APPL			7	/* Application */

		/* subid 4 */
#define ULSIG			0	/* NBS Upper Layer SIG */
#define ORG_CODE		1	/* Organization code */
#define ABSTRACT_SYNTAX_4	2
#define APPL_CONTEXT		1
#define APDUS			0	/* ACSE pdus */
#define ENE_BASE		41	/* Base of ENE-specific subtree */
			/* net management */
#define	RC_802_4		4	/* Physical 802.4 */
#define	RC_802_2		1	/* Data Link 802.2 */
#define	RC_MAC			2	/* Data Link MAC */
#define	RC_8473			1	/* Network CLNS 8473 */
#define	RC_8073			1	/* Transport 8073 */
#define	RC_8327			1	/* Session 8327 */
#define	RC_8823			1	/* Presentation 8823 */
#define	RC_ACSE			1	/* Application 8649/2 ACSE */


		/* subid 5 */
#define ACSE_V1			1	/* ACSE Abstract Syntax */
#define FTAM_PCI		1	/* FTAM Abstract Syntax */
#define MMS_PCI			1	/* MMS Abstract Syntax */
#define PRIVATE_AS		3	/* Private Abstract Syntax */
#define ABSTRACT_SYNTAX_5	2
#define UNSTRUCT_BIN_AS		4	/* Unstructured Binary	*/
#define FTAM_FADU_AS		2
#define UNSTRUCT_TEXT_AS 	3

#define NBS_TS1_TS		8825	/* NBS Transfer Syntax 1 */
#define ISO_8825_TS		0	/* ISO 8825 Transfer Syntax */
#define BIN_TS			2	/* FTAM Simple Binary Transfer Syntax */

#define ISO_MMS			1	/* MMS Application Context */
#define ISO_FTAM		1	/* FTAM Application Context */
#define ROSE_M1A1O1_AC		3	/* ROSE Application Context A1M1O1 */
#define ROSE_M1A1O2_AC  	4	/* ROSE Application Context A1M1O2 */
#define PRIVATE_AC		5	/* Private Application Context */
#define ENE_DS			0	/* Directory Svc base for ENE */
#define ENE_NM			1	/* NM AC/AS base for ENE */
#define ENE_AP_TTL		3	/* AP Title base for ENE */

			/* net management */
#define	RC_CLASS4		4	/* Transport 8073 Class 4 */

		/* subid 6 */
#define NBS_AS1			1
#define NBS_AS2			2
#define NM_AC			3
#define NM_AS			4
#define AE_TTL_ATTR		0	/* DS AE Title Attribute for ENE */
