/*
 *	Copyright (c) 1987 Motorola Inc
 *	All Rights Reserved
 *	@(#)ma_def_330.h	35.1
 */

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF Motorola	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ident	"@(#)ma_def_330.h	35.1"

/*IH*********************** INCLUDE HEADER ****************************
**
**  INCLUDE NAME:	ma_def_330.h
**
**  DESCRIPTION:	This file contains constant, type and macro
**			definitions for the hardwareassociated with
**			the MVME330 Ethernet board.
**
**  NOTES:		Refer to Microsystems MVME330 Ethernet
**			Controller User's Manual for a more complete
**			description.
**
**			"vrtx.h" must be included before this file.
**
**  UPDATE LOG:
**  Date	Name		V/R	Comments
**  --------	-----------	---	-----------------------------
**  01/30/86    R. Robinson     1.0     Original coding.
**
**********************************************************************/

/*
**	The following is the MVME330 memory map definitions.
*/

#define MM_MPUIO	0x00FE0000	/* MVME330 I/O Registers */
#define MM_ROM		0x00F80000	/* MVME330 ROM */
#define MM_WINRAM	0x00F1FE00	/* MVME330 Window Inter. RAM */
#define MM_MPURAM	0x00F00000	/* MVME330 RAM */
#define MM_VMEIO	0x00EF0000	/* VME Short I/O Space */
#define MM_VMERAM	0x00001000	/* VME Memory Space */
#define MM_MPUEVT	0x00000000	/* MVME330 EVT RAM */

/*
**	The following are the MVME330 hardware address definitions.
*/

	/*
	**	The following are the MVME330 Control/Status 
	**	register address definitions. Each address
	**	will access a byte.
	*/

#define VECTR	0x00FE0081		/* Host Interrupt Vector Reg. */
#define CSR	0x00FE00A1		/* Control Status Register */
#define IER	0x00FE00C1		/* Interrupt Enable Register */
#define TIR	0x00FE00C3		/* Transmit Interrupt Reg. */
#define RIR	0x00FE00C5		/* Receive Interrupt Reg. */
#define UIR	0x00FE00C7		/* Utility Interrupt Reg. */
#define MAPR	0x00FE00CF		/* Map Control Register */
#define EXR	0x00FE00E1		/* Exception Register */
#define UUR1	0x00FE00C9		/* Unused Hardware Register 1 */
#define UUR2	0x00FE00CB		/* Unused Hardware Register 2 */
#define UUR3	0x00FE00CD		/* Unused Hardware Register 3 */

	/*
	**	The following are the MVME330 LANCE register
	**	address definitions. Each address will access
	**	a word.
	*/

#define LDR	0x00FE0200		/* LANCE Data Register */
#define LAR	0x00FE0202		/* LANCE Address Register */

	/*
	**	The following are the MVME330 Network Parameters
	**	PROM address definitions. Each address will access
	**	a word with bits 0-3 of each word having any
	**	significance.
	**
	**	The unique node address is stored in this PROM
	**	as follows;
	**
	**		02CF1Fx0yyyy
	**
	**	where 'x' indentifies the board type (i.e. 0 -
	**	MVM33 (ENP20), 1 - MVME 330 (ENP10)), and 'yyyy'
	**	is the board's serial number.
	*/

#define NPP_STR	0x00FE0400		/* Starting PROM Address */
#define NPP_END	0x00FE0800		/* Ending PROM Address */
#define NPP_SZE	(NPP_END - NPP_STR)	/* PROM Size */

/*
**	The following are the autovector addresses used by
**	the MVME330 hardware functions. Note that the TIMER,
**	ACLO, ABO, PER, and RTO all cause the same interrupt
**	(i.e. autovector 7). The interrupt service routine
**	that services autovector 7 must read the EXR (exception
**	register) to determine the source of the autovector
**	7 interrupt.
*/

#define VEC_TIMER	0x0000007C	/* 7 - RTC/Refresh Timeout */
#define VEC_ACLO	0x0000007C	/* 7 - Power Failing */
#define VEC_ABO		0x0000007C	/* 7 - Software Abort Button */
#define VEC_PER		0x0000007C	/* 7 - RAM Parity Error */
#define VEC_RTO		0x0000007C	/* 7 - Resource Time Out */
#define VEC_LANCE	0x00000078	/* 6 - LANCE Interrupt */
#define VEC_RINT	0x00000074 	/* 5 - Receive Interrupt */
#define VEC_TINT	0x00000070	/* 4 - Transmit Interrupt */
#define VEC_BINT	0x0000006C	/* 3 - Host To MVME330 Int. */
#define VEC_UINT	0x00000068	/* 2 - Utility Interrupt */

/*
**	The following definitions are constants used in
**	bit-manipulation of the MVME330 hardware registers.
*/

	/*
	**	Control/Status Register
	*/

#define CSR_SUSPEND	0x08		/* Read/Write - LANCE DMA */
#define CSR_TIMER	0x10		/* Read Only  - Timer Expired */
#define CSR_RESET	0x20		/* Read/Write - System Reset */
#define CSR_FAIL	0x40		/* Read/Write - MVME330 Fail */
#define CSR_VIRQ	0x80		/* Read Only  - Vectored Int. */

	/*
	**	Exception Register
	*/

#define EXR_RTO		0x01		/* Resource Timeout */
#define EXR_PER		0x02		/* Parity Error */
#define EXR_ABO		0x04		/* Abort Button */
#define EXR_ACLO	0x08		/* AC Line Voltage Low */
