/*
 *	Copyright (c) 1987 Motorola Inc
 *	All Rights Reserved
 *		@(#)pcc.h	35.2
 */
#ident	"@(#)pcc.h	35.2"

/*
 *	Defines for the Peripheral Channel Controller interface.
 */
/* Definitions common to several PCC registers */
/* If a particular bit definition for a PCC register is missing,
   look in these defines */

#define PCC_IL0	0x00	/* Interrupt levels */
#define PCC_IL1	0x01
#define PCC_IL2	0x02
#define PCC_IL3	0x03
#define PCC_IL4	0x04
#define PCC_IL5	0x05
#define PCC_IL6	0x06
#define PCC_IL7	0x07

#define PCC_INTSTAT	0x80	/* Interrupt Status */
#define PCC_IEN		0x08	/* Interrupt Enable */
#define PCC_ICLR	0x80	/* Interrupt Clear */
#define PCC_ENAB	0x01	/* Enable Function */

/* Tick Timer Control Register */

#define PCC_CTRENA	0x02	/* Counter Enable */
#define PCC_CLROVF	0x04	/* Clear Overflow */

/* General Purpose Control Register */

#define PCC_MINTEN	0x10	/* Master Interrupt Enable */
#define PCC_LBTOEN	0x08	/* Enable PCC Local Bus Timer */
#define PCC_TSTPAR	0x04	/* Test Parity - write bad parity if high */
#define PCC_PMODE2 	0x03	/* Parity Enable - BERR on next cycle */
#define PCC_PMODE3	0x01	/* Parity Enable - BERR on current cycle */
#define PCC_PARMSK	(PCC_PARDIS | PCC_PMODE2 | PCC_PMODE3)
#define PCC_PARDIS	0x00	/* Parity Disable */

/* General Purpose Status Register */

#define PCC_PARERR	0x01	/* Parity Error */

/* Printer Interrupt Control Register */

#define PCC_FALTINT 	0x40	/* Fault Interrupt Status */
#define PCC_ACKINT	0x20	/* ACK Interrupt Status */
#define PCC_ACKPOL	0x10	/* ACK Edge Sensitivity bit */
#define PCC_PIENAB	0x08	/* Printer Interrupt Enable */

/* Printer Control Register */

#define PCC_INPRIM	0x08	/* Input Prime */
#define PCC_STROBE	0x04	/* Manual Printer Strobe */
#define PCC_STBTIM	0x02	/* Printer Strobe Timing */
#define PCC_MODE	0x01	/* Strobe Mode (0=auto, 1=manual) */

/* Printer Status Register */

#define PCC_ACK		0x80	/* Printer Acknowledge */
#define PCC_FAULT	0x40	/* Fault */
#define PCC_SELECT	0x20	/* Printer Select */
#define PCC_PE		0x10	/* Paper Empty */
#define PCC_BSY		0x08	/* Printer Busy */ 
#define PCC_STATMSK	(PCC_ACK | PCC_FAULT | PCC_SELECT | PCC_PE | PCC_BSY)

/* Serial Port Interrupt Control Register */

#define PCC_INTEXT	0x10	/* Vector Source - 0 = serial chip, 1 = pcc */
#define PCC_SIENAB	0x08	/* Serial Port Interrupt Enable */

/* Slave Base Address Register */

#define PCC_WAITRMC	0x20	/* Wait for VMEbus on Read-Modify-Write */

struct pcc
{
	unsigned int	tar;   	/* table address register */
	unsigned int	dmar; 	/* dma address register */
	unsigned int	bcr;	/* byte count register */
	unsigned int 	dhr;	/* data holding register */
	unsigned short	tpr1;	/* tick timer 1 preload register */
	unsigned short 	tctr1;	/* tick timer 1 counter register */
	unsigned short	tpr2;	/* tick timer 2 preload register */
	unsigned short 	tctr2;	/* tick timer 2 counter register */
	unsigned char	ticr1;	/* tick timer 1 interrupt register */
	unsigned char	tcr1;	/* tick timer 1 control register */
	unsigned char	ticr2;	/* tick timer 2 interrupt register */
	unsigned char	tcr2;	/* tick timer 2 control register */
	unsigned char 	acfcr;	/* ac fail control register */
	unsigned char	wdtcr;	/* watch dog timer control register */
	unsigned char	picr;	/* printer interrupt control register */
	unsigned char	pcr;	/* printer control register */
	unsigned char	dmaicr;	/* dma interrupt control register */
	unsigned char 	dmacsr;	/* dma control and status register */
	unsigned char 	beicr;	/* bus error interrupt control register */
	unsigned char	dmasr;	/* dma status register */
	unsigned char	aicr;	/* abort interrupt control register */	
	unsigned char	tafcr;	/* table address function code register */
	unsigned char	spicr;	/* serial port interrupt control register */
	unsigned char	gpcr;	/* general purpose control register */
	unsigned char	licr;	/* lan interrupt control register */
	unsigned char	gpsr;	/* general purpose status register */
	unsigned char	scsicr;	/* scsi port interrupt control register */
	unsigned char	sbar;	/* slave base address register */
	unsigned char	swicr1;	/* software interrupt 1 control register */
	unsigned char 	ibvr;	/* interrupt base vector register */
	unsigned char	swicr2;	/* software interrupt 2 control register */
	unsigned char	rlr;	/* revision level register */
};



