/*
 *	Copyright (c) 1987 Motorola Inc
 *	All Rights Reserved
 *		@(#)kern-port:sys/proc.h	35.3
 */
/*	Copyright (c) 1984 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF AT&T	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ident	"@(#)kern-port:sys/proc.h	35.3"

/*	One structure allocated per active process. It contains all
**	data needed about the process while the process may be swapped
**	out.  Other per process data (user.h) may swapped with the
**	process but in fact it is not.  The pregion structure is an
**	extension of the proc structure.  It is allocated dynamically
**	during system initialization so that the number of pregions
**	per process is not fixed by this structure.  Increasing the
**	number of pregions per process does not require recompiling the
**	system.
*/

typedef struct	proc	{
	char	p_stat;			/* Status of process.		*/
	char	p_pri;			/* priority, negative is high	*/
	char	p_cpu;			/* cpu usage for scheduling	*/
	char	p_nice;			/* nice for cpu usage		*/
	uint	p_flag;			/* Flags defined below.		*/
	ushort	p_uid;			/* real user id			*/
	ushort	p_suid;			/* saved (effective) user id	*/
					/* from exec.			*/
	short	p_pgrp;			/* name of process group	*/
					/* leader			*/
	short	p_pid;			/* unique process id		*/
	short	p_ppid;			/* process id of parent		*/
	ushort	p_sgid;			/* saved (effective) group	*/
					/* id from exec.		*/
	pde_t	p_ubpgtbl[USIZE*NPDEPP];/* Page table for the U-block	*/
#ifdef CMMU
#define ONPDEPP	4
	pde_t	p_ubfill[(USIZE*ONPDEPP)-USIZE];
#endif
	sde_t	*p_pttbl;		/* pointer table array		*/
	int	p_sig;			/* signals pending to this	*/
					/* process			*/
	struct	proc	*p_flink;	/* linked list of processes	*/
	struct	proc	*p_blink;	/* linked list of processes	*/
	caddr_t	p_wchan;		/* Wait addr for sleeping  	*/
					/* processes.			*/
	struct	proc	*p_parent;	/* ptr to parent process	*/
	struct	proc	*p_child;	/* ptr to first child process	*/
	struct	proc	*p_sibling;	/* ptr to next sibling		*/
					/* process on chain		*/
	int     p_clktim;               /* time to alarm clock signal   */
	uint	p_size;			/* size of swappable image	*/
					/* in pages.			*/
					/* sigset turns on this bit	*/
					/* signal does not turn on this bit */
	time_t	p_utime;		/* user time, this proc		*/
	time_t	p_stime;		/* system time, this proc	*/
	struct  proc *p_mlink;		/* link list of processes	*/
					/* sleeping on memwant or	*/
					/* swapwant.			*/
	preg_t	*p_region;		/* process regions		*/
	short	p_xstat;		/* exit status for wait		*/
	ushort	p_mpgneed;		/* number of memory pages	*/
					/* needed in memwant.		*/
	short	p_epid;			/* effective pid		*/
					/* normally - same as p_pid	*/
					/* if server - p_pid that sent msg */
	sysid_t p_sysid;		/* normally - same as sysid	*/
					/* if server - system that sent msg */
	struct	rcvd  *p_minwd;		/* server msg arrived on this queue */
	struct	proc  *p_rlink;		/* linked list for server	*/
	int	p_trlock;
	struct inode *p_trace;		/* pointer to /proc inode	*/
	long	p_sigmask;		/* tracing signal mask for /proc */
	int	p_hold;			/* hold signal bit mask		*/
	int	p_chold;		/* defer signal bit mask	*/
	char	p_time;			/* resident time for scheduling	*/
	char    p_fill[31];

} proc_t;

#define	p_link	p_flink

extern struct proc proc[];	/* the proc table itself	*/

/* stat codes */

#define	SSLEEP	1		/* Awaiting an event.		*/
#define	SRUN	2		/* Running.			*/
#define	SZOMB	3		/* Process terminated but not	*/
				/* waited for.			*/
#define	SSTOP	4		/* Process stopped by signal	*/
				/* since it is being traced by	*/
				/* its parent.			*/
#define	SIDL	5		/* Intermediate state in	*/
				/* process creation.		*/
#define SXBRK	6		/* Process being xswapped       */

/* flag codes */

#define	SSYS	0x0001		/* System (resident) process.	*/
#define	STRC	0x0002		/* Process is being traced.	*/
#define	SWTED	0x0004		/* Stopped process has been	*/
				/* given to parent by wait	*/
				/* system call.  Don't return	*/
				/* this process to parent again	*/
				/* until it runs first.		*/
#define SNWAKE	0x0008		/* Process cannot wakeup by	*/
				/* a signal.			*/
#define SLOAD	0x0010		/* in core                      */
#define SLOCK   0x0020		/* Process cannot be swapped.	*/
#define	SRSIG	0x0040		/* Set when signal goes remote	*/
#define SPOLL	0x0080		/* Process in stream poll	*/
#define SPRSTOP	0x0100		/* process is being stopped via /proc */
#define SPROCTR	0x0200		/* signal tracing via /proc	*/
#define SPROCIO	0x0400		/* doing I/O via /proc, so don't swap */
#define SSEXEC	0x0800		/* stop on exec			*/
#define SPROPEN	0x1000		/* process is open via /proc	*/
#define SSEL	0x2000		/* process select()ing		*/

#define PTRACED(p)	((p)->p_flag&(STRC|SPROCTR|SSEXEC|SPROPEN))
