/*		Copyright (c) 1988 Motorola		*/
/*		All Rights Reserved				*/
/*		@(#)scsi.h	35.2 */
/*
	Definitions for the 147 SCSI Firm-ware packets
*/

#define MAXWRK (400)

/*
	macros to make using these unions easier
*/
#define DISK1	disk1
#define EDISK1	cmd1.addisk1
#define DISK2	disk2
#define EDISK2	cmd2.addisk2
#define TAPE1	tape1
#define TAPE2	tape2
#define ETAPE1	cmd1.adtape1
#define ETAPE2	cmd2.adtape2
#define RDWR	cmd1.rdwr
#define FMT	cmd1.fmt
#define REWIND	cmd1.rewind
#define LDUNL	cmd1.load_unload
#define SPACE	cmd1.space
#define WRTFM	cmd1.wrtfm


/*
	SCSI work space
	allocated on a per device basis
*/
struct work {
	unsigned char work_area[MAXWRK];
};

/*
	the work_area may also 
	be viewed as having this structure
*/
struct cmd_tbl {
	unsigned char status;
	unsigned char retry;
	unsigned char *link_ptr;
	unsigned short cmd_lgth;
	unsigned short cdb0[6];
	unsigned char scsi_status;
	unsigned char dummy0;
	unsigned long data_lgth;
	unsigned char *data;
	unsigned short msg_in_lgth;
	unsigned char *msg_in;
	unsigned short msg_out_lgth;
	unsigned char *msg_out;
	unsigned short reserved;
	unsigned short dma_size;
	unsigned char resv[0x32];
	unsigned long sector_error;
	unsigned char scsi_ctrl_err;
	unsigned char resv1;
	unsigned char *transfer_address;
	unsigned char resv2[8];
	unsigned char pkt;
	unsigned char offset;
};

	

/*
	read/write packet specific information
*/
struct rdwr {
	unsigned char *address;
	unsigned long block;
	unsigned short block_count;
	unsigned short dummy1;
	unsigned short scga_count;	/* scatter/gather count */
	unsigned short dummy2;
};

/* scatter/gather list entry */
struct scga {
	unsigned char *address;
	unsigned long link:1;
	unsigned long dummy:4;
	unsigned long func_code:3;
	unsigned long count:24;
};

/* valid bits in disk attribute word */
#define SCS_ATW 0x0fff


/* CACHE enable mask */
#define SCS_CACHE	0x80

/* format list mask */
#define SCS_FMTLST 0x0f

/* format list types */
#define SCS_BLOCK 0
#define SCS_SECTOR 1
#define SCS_BFI 2


/*
	attach/detach packet information for disk
*/
struct addisk1 {
	unsigned char interleave; /* resv used for interleave storage ios_ilv */
	unsigned char step_rate; /* ios_sr */
	unsigned char cache; /* resv used for cache enable storage */
	unsigned char number_heads; /* ios_hds  */
	unsigned short number_cyls; /* ios_trk */
	unsigned short precomp_cyl; /* ios_pcom */
	unsigned short sect_per_track; /* ios_spt */
	unsigned short fmt_list; /* rsv used for format list types */
	unsigned short disk_attr; /* ios_atw */
	unsigned char controller_type; /* vid_typ high byte */
	unsigned char device_type; /* vid_typ low byte */
};

struct addisk2 {
	unsigned char dummy1;
	unsigned char retry;
	unsigned short bytes_per_block; /* ios_psm */
	unsigned char sector_slip_count; /* ios_ssc */
	unsigned char alt_resv; /* ios_alts */
	unsigned char *work_area;
};

struct scs_disk {
	struct addisk1 disk1;
	struct addisk2 disk2;
};

struct adtape1 {
	unsigned char dummy1;
	unsigned char number_tracks;
	unsigned char dummy2;
	unsigned char min_rdxfer;
	unsigned char dummy3;
	unsigned char min_wrtxfer;
	unsigned char dummy4;
	unsigned char stream_count;
	unsigned short dummy5;
	unsigned short dummy6;
	unsigned short tape_attr;
	unsigned char controller_type;
	unsigned char device_type;
};

struct adtape2 {
	unsigned char dummy1;
	unsigned char retry;
	unsigned short bytes_per_block;
	unsigned short dummy2;
	unsigned long *work_area;
};

struct scs_tape {
	struct adtape1 tape1;
	struct adtape2 tape2;
};


/*
	format control bit positions
*/
#define SCS_DEFECT	0x80	/* use user supplied defect list */
#define SCS_PRIMARY	0x40	/* use manufacturers defect list */
#define SCS_CERTIFY	0x20	/* use certification method of formatting */
#define SCS_STOP	0x10	/* stop format if inaccessable defect list */

/* only used for re-assigning a bad block */
struct scs_dfctblk {
	unsigned short list_code;
	unsigned long reserved;
	unsigned long lsn;
};

/* defect list header */
struct scs_dfcthdr {
	unsigned short list_code;
	unsigned long count;
};

struct fmt {
	unsigned short number_defects;
	struct scs_dfctblk *defect_list;
	unsigned short dummy1;
	unsigned short dummy2;
	unsigned char interleave;
	unsigned char dummy3;
	unsigned short dummy4;
	unsigned short dummy5;
};

struct scs_defect {
	unsigned long cylinder;
	unsigned short head;
	unsigned long offset; /* may be sector number or bytes from index */
};

/* block defect */
struct scs_blockdfct {
	unsigned long offset; /* logical sector number/block number of defect */
};


struct assign {
	/* not used , SASI only */
	unsigned short dummy[8];
};

struct  busreset {
	unsigned short dummy[8];
};

struct erase {
	unsigned short dummy[8];
};

struct rewind {
	unsigned short dummy1[5];
	unsigned char immediate;
	unsigned char dummy2;
	unsigned short dummy3[2];
};

struct rdblock {
	unsigned long max_block_lgth;
	unsigned short min_block_lgth;
	unsigned short dummy[5];
};

struct space {
	unsigned long number; /* number of data blocks or file marks */
	unsigned short dummy1[3];
	unsigned char mode;
	unsigned char dummy2;
	unsigned char dummy3[2];
};

struct wrtfm {
	unsigned short number_of_fm;
	unsigned short dummy1[7];
};

struct verify {
	unsigned long number_of_blocks;
	unsigned short dummy1[6];
};

struct tpmode1 {
	/* not necessary, done with attach */
	unsigned short dummy1[8];
};

struct setparm1 {
	/* not used */
	unsigned short dummy1[8];
};

struct inquiry {
	/* not used */
	unsigned short dummy1[8];
};

struct load_unload {
	unsigned short dummy1[5];
	unsigned char mode;
	unsigned char dummy2;
	unsigned short dummy3[2];
};

struct recover {
	/* not used */
	unsigned short dummy1[8];
};

struct req_sense {
	/* not used */
	unsigned short dummy1[8];
};

struct req_status {
	unsigned short dummy1[8];
};


struct tpmode2 {
	unsigned short dummy1[2];
};

struct setparm2 {
	unsigned short dummy1[1];
};

struct scs_pkt
{
	unsigned char scs_ctrlr;	/* controller number */
	unsigned char scs_dev;		/* device (lun) number */
	unsigned char scs_stat0;	/* status byte 0 */
	unsigned char scs_stat1;	/* status byte 1 */
	union cmd1 {
		struct rdwr rdwr; 	/* read/write */ 
		struct addisk1 addisk1;	/* attach/detach disk */
		struct adtape1 adtape1;	/* attach/dettach tape */
		struct fmt fmt;		/* format/assign alternate sector */
		struct assign assign;	/* assign alternate track */
		struct  busreset busreset; /* scsi bus reset */
		struct erase erase;	/* erase tape */
		struct rewind rewind;	/*  rewind tape */
		struct rdblock rdblock;	/* */
		struct space space;	/* */
		struct wrtfm wrtfm;	/* */
		struct verify verify;	/* */
		struct tpmode1 tpmode1;	/* */
		struct setparm1 setparm1;	/* */
		struct inquiry inquiry;	/* */
		struct load_unload load_unload;	/* */
		struct recover recover;	/* */
		struct req_sense req_sense;	/* */
	} cmd1;
	unsigned char cmd_ctrl;		/* rd/wrt cache control 0x80 = enable */
	unsigned char scs_funcc;	/* function codes */
	unsigned char scs_level;	/* interrupt level 0 or 2 */
	unsigned char scs_vnum;	/* vector number */
	unsigned char scs_stat2;	/* status byte 2 */
	unsigned char scs_stat3;	/* status byte 3 */
	union cmd2 {
		struct addisk2 addisk2;	/* */
		struct adtape2 adtape2;	/* */
		struct tpmode2 tpmode2;	/* */
		struct setparm2 setparm2;	/* */
	} cmd2;
	struct iobuf *utab;	/* saved utab for easier interrupt processing */
	/*
	   be-carefull about changing the offset of the interrupt handler
	   in this structure. The offset must also be changed in scsi.m4
	*/
	int (*inthand)();	/* interrupt handler */
	struct scs_pkt *next;	/* queue next pointer */
};


/*
	SCSI function codes
*/

#define SCS_READ	0x00
#define SCS_WRITE	0x04
#define SCS_DKATTACH	0x08
#define SCS_DETACH	0x0c
#define SCS_FORMAT	0x10
#define SCS_AAS		0x14
#define SCS_AAT		0x18
#define SCS_CUST	0x1c
#define SCS_BURESET	0x20
#define SCS_CTRLRESET	0x24
#define SCS_TPATTACH	0x28
#define SCS_ERASE	0x2c
#define SCS_REW		0x30
#define SCS_RDBLKLIM	0x34
#define SCS_SPACE	0x38
#define SCS_WEOF	0x3c
#define SCS_VERCRC	0x40
#define SCS_TPMODESEL	0x44
#define SCS_TPMODESEN	0x48
#define SCS_SETPARM	0x4c
#define SCS_INQUIRY	0x50
#define SCS_RET		0x54
#define SCS_RECBUF	0x58
#define SCS_REQSENSE	0x5c
#define SCS_STAT	0x60

/*
	SCSI return control flags
*/

#define COME_AGAIN	0x08
#define CMD_PEND	0x10
#define RTE_FLAG	0x20
#define ADD_STAT	0x40
#define FINAL_STAT	0x80
#define FINAL		0x00

/*
	intermediate status codes in status packet 1
*/
#define WAIT_TIME	0x00	/* register A1 used for delay time */
#define WFI_CLOSE	0x01
#define WFI_OPEN	0x02
#define LINK_FLAG	0x03
/*
	custom scsi sequences not used
*/

/*
	final return status/error codes
*/
#define GOOD		0x00
#define UNDEFINED	0x01
#define REACTIVE	0x02
#define NOINT		0x03
#define NORESEL		0x04
#define BADSEQ		0x05
/* 
	SCSI return codes
*/
#define S_EOM		0x04		/* end of media */
#define S_FM		0x08		/* file mark */
#define S_TOUT		0x12		/* controller timeout */
#define S_OFFLINE	0x22		/* drive not ready */
#define S_ILLEGAL	0x25		/* illegal request */
#define S_WP		0x27		/* write protected */
#define S_EOD		0x28		/* end of media */

/*
	Hard coded values for dsdd5 and pcat
*/
#define M147DSDD5 0x0207
#define M147PCAT  0x028f

/*
	SCSI disk attributes
*/
#define FM		0x0000
#define MFM		0x0001
#define SINGLETRK	0x0000
#define DOUBLETRK	0x0002
#define SINGLESIDE	0x0000
#define DOUBLESIDE	0x0004
#define FIVE14		0x0000
#define EIGHT		0x0008
#define FLOPP		0x0000
#define RIGID		0x0010
#define NORESERVE	0x0000
#define RESERVE		0x0020
#define NOBUFF		0x0000
#define BUFFERED	0x0040
#define STDIBM		0x0000
#define NONSTD		0x0080
#define SOFTSECT	0x0000
#define HARDSECT	0x0100
#define FIXED		0x0000
#define REMOVE		0x0200
#define ALT_CYL		0x0000
#define ALT_SECT	0x0400
#define ZONE_CYL	0x0000
#define ZONE_TRK	0x0800

/* 
	SCSI tape retension attributes
	the load/unload command is used only to retension
*/
#define RETENSION 0x02


/*
	SCSI tape space attributes
*/
#define SKEOD 0x03		/* seek to end of data */
#define SKNEXT 0x01		/* seek to end of file */

/*
	SCSI device types - as used by attach
*/
#define SCS_DISK 0x00
#define SCS_TAPE 0x08

#define RECOV_ERR	0x21	/* recovered from disk error */
