/*		Copyright (c) 1986,1987,1988 Motorola, Inc.
 *		All Rights Reserved		
 *
 *		sobpp.h - son of the buffered pipe protocol "bpp"
 *
 *		@(#)sobpp.h	35.3
 */


/*
 *	Packet Structure
 */

struct sb_packet {
	unsigned char	p_cmd;			/* command		*/
	unsigned char	p_cmdctrl;		/* command control	*/
	unsigned char	p_pdtype;		/* primary device type	*/
	unsigned char	p_punit;		/* primary unit		*/
	unsigned char	p_sdtype;		/* secondary device type*/
	unsigned char 	p_sunit;		/* secondary unit	*/
	unsigned char	p_am;			/* address modifier	*/
	unsigned char	p_busw;			/* data bus width	*/
	unsigned long	p_paddr;		/* primary address	*/
	unsigned long	p_saddr;		/* secondary address	*/
	unsigned long	p_tcount;		/* transfer count	*/
	unsigned short	p_sgcount;		/* scatter/gather count	*/
	unsigned short	p_cparm1;		/* command parameter 1	*/
	unsigned short	p_cparm2;		/* command parameter 2	*/
	unsigned short	p_cparm3;		/* command parameter 3	*/
	unsigned char	p_faterr;		/* fatal error code	*/
	unsigned char	p_recover;		/* recover error status	*/
	unsigned short	p_deverr;		/* device error status	*/
	unsigned char	p_retry;		/* retry count		*/
	unsigned char	p_f0;			/* 	reserved	*/
	unsigned long	p_erraddr;		/* error status addr	*/
	unsigned long	p_termcnt;		/* termination xfer cnt	*/
	unsigned short	p_sparm1;		/* status parameter 1	*/
	unsigned short	p_sparm2;		/* status parameter 2	*/
	unsigned short	p_sparm3;		/* status parameter 3	*/
	unsigned long	p_utab;			/* pointer struct iobuf	*/
};


/*
 *	Envelope Structure
 */

struct sb_envelope {
	struct sb_envelope	*e_link;	/* next envelope  */
	struct sb_packet	*e_pkt;		/* packet pointer */
	unsigned char		e_valid;	/* valid flag	  */
	char			e_res1;		/* reserved	  */
	short			e_res2;		/* reserved	  */
};

#define	INVALID	0
#define	VALID	1

/*
 *	Channel Structure
 */

struct sb_channel {
	struct sb_envelope	*c_cmdhead;	/* command pipe head pointer */
	struct sb_envelope	*c_cmdtail;	/* command pipe tail pointer */
	struct sb_envelope	*c_stathead;	/* status pipe head pointer  */
	struct sb_envelope	*c_stattail;	/* status pipe tail pointer  */
	unsigned char		c_level;	/* interrupt level	     */	
	unsigned char		c_vector;	/* interrupt vector number   */
	unsigned char		c_prio;		/* priority of channel	     */
	unsigned char		c_am;		/* address modifier          */
	unsigned char		c_chan;		/* channel number	     */
	unsigned char		c_valid;	/* vaild channel flag	     */
	unsigned char		c_busw;		/* data bus width	     */
	unsigned char		c_f0;		/* 	reserved	     */
};

struct sb_device {
	unsigned	m_addr;		/* IPC Address Register		*/
	unsigned char	m_am;		/* IPC Address Modifier Register*/
	unsigned char	m_busw;		/* IPC Data Bus Width		*/
	unsigned char	m_cr;		/* IPC Control Register		*/
	unsigned char	m_f0;		/* 	Reserved		*/
	unsigned char	m_sr;		/* IPC Status Register		*/
	unsigned char	m_diag;		/* IPC Diagnostic  Register	*/
	unsigned	m_f1;		/* 	Not Used		*/
	unsigned short	m_tas;		/* IPC TAS Register		*/
};

/*
 *	Physical Sector bad spot format (type 1)
 */

struct sb_badsec {
	unsigned	b_cyl;		/* cylinder number	*/
	unsigned short  b_head;		/* head number		*/
	unsigned	b_sec;		/* defect sector	*/
};

/*
 *	Bytes from Index bad spot format (type 2)
 */

struct sb_badspot {
	unsigned	b_cyl;		/* cylinder number	*/
	unsigned short 	b_head;		/* head number		*/
	unsigned	b_bfi;		/* bytes from index	*/
};


/*
 *	Scatter/Gather list format
 */

struct sb_scatlist {
	unsigned char *address;
	unsigned count;
};

/*
 *	struct device defines
 *
 *	m_cr
 */

#define SB_BSY		0x80	/* busy bit		*/
#define	SB_RESET	0x40	/* IPC reset signal	*/
#define	SB_ATTN		0x20	/* attention bit	*/
#define	SB_SFINH	0x10	/* sysfail inhibit	*/

/*
 *	m_sr
 */

#define SB_OKAY		0x00	/* correct command completion	*/
#define SB_INVALID	0x01	/* invalid command		*/
#define SB_CHREAD	0x02	/* couldn't read channel header	*/
#define SB_CHWRITE	0x03	/* couldn't write channel header*/
#define SB_DMAREAD	0x04	/* VME read failed		*/
#define SB_DMAWRITE	0x05	/* VME write failed		*/
#define SB_NOFREE	0x06	/* No free channels		*/
#define SB_EXIST	0x07	/* channel does not exist	*/
#define SB_NORAM	0x08	/* not enough ram left 		*/
#define SB_ADDRERR	0xaa	/* local address error		*/
#define SB_ACFAIL	0xac	/* AC fail condition		*/
#define SB_CONTST	0xcc	/* confidence test error	*/
#define SB_UNEXP	0xee	/* unexpected exception error	*/

/*
 *	m_tas
 */

#define	TASBIT	0x8000		/* TAS reservation bit		*/
#define	VCMD	0x4000		/* valid command flag		*/
#define	VSTAT	0x2000		/* valid status flag		*/
#define	CCOMP	0x1000		/* command complete		*/
#define	TASCMD	0x0fff		/* TAS command mask		*/
#define	TASMASK	0xf000		/* TAS status mask		*/
#define	CREATE	0x0001		/* channel create command	*/
#define	DELETE	0x0002		/* channel delete command	*/
#define GETMEM	0x0003		/* get memory command		*/
#define FREEMEM	0x0004		/* free memory command		*/

/*
 *	struct packet defines
 */

/*
 *	p_pdtype
 */

#define	SD_HARD		0x00		/* hard disk		*/
#define	SD_FLOP		0x01		/* floppy		*/
#define SD_TAPE		0x02		/* streamer tape	*/
#define SD_PRT		0x03		/* printer/parallel port*/
#define SD_SERIAL	0x04		/* serial port 		*/
#define SD_SCSI1	0x05		/* SCSI bus 1		*/
#define SD_SCSI2	0x06		/* SCSI bus 2		*/
#define SD_GSP		0x07		/* 393 GSP		*/

#define SD_IPC		0x0f		/* controller */

/*
 *	p_cmd
 */

#define SC_BPP		0x00			/* BPP test		*/
#define	SC_READ		0x01			/* read			*/
#define	SC_WRITE	0x02			/* write		*/
#define SC_RIPC		0x03			/* read ipc descriptor	*/
#define	SC_WIPC		0x04			/* write ipc descriptor */ 
#define SC_FMT		0x05			/* format disk		*/
#define SC_FIXBS	0x06			/* fix bad spot		*/
#define SC_SEEK		0x08			/* seek			*/
#define SC_RECAL	0x08			/* recalibrate		*/
#define	SC_CINIT	0x09			/* cache buffers init	*/

#define	SC_STAT		0x10			/* read status		*/
#define	SC_RET		0x11			/* retension		*/
#define	SC_WEOF		0x12			/* write eof		*/
#define	SC_REW		0x13			/* rewind		*/
#define	SC_ERASE	0x14			/* erase		*/
#define SC_SPACE	0x15			/* space		*/

#define SC_EABLE	0x20			/* enable target	*/
#define SC_DABLE	0x21			/* disable target	*/
#define SC_RESRV	0x22			/* reserve unit 	*/
#define SC_RELSE	0x23			/* release unit	 	*/
#define SC_ALMSG 	0x24			/* allocate message buff*/
#define SC_RSCSI	0x25			/* reset SCSI		*/
#define SC_CSCSI	0x26			/* custom SCSI command	*/
#define SC_STEST	0x27			/* self test		*/
#define SC_STOP		0x28			/* stop			*/
#define SC_RSTRT	0x29			/* restart		*/
#define SC_DOWNL	0x2a			/* download		*/
#define SC_SSCSI	0x2b			/* set SCSI level	*/
#define SC_DSCSI	0x2c			/* define SCSI level	*/
#define SC_OPEN		0x2d			/* Open SCSI device	*/


/*
 *	p_cmdctrl
 */

#define T_NOCACHE	0x8		/* bypass cache		*/


/*
 *	p_busw
 */

#define BIT32	2			/* 32 bit data transfers */
#define BIT16	1			/* 16 bit data transfers */


/*
 *	Disk Descriptor
 */

struct d_disk {
	unsigned char	d_ctype;	/* controller type	*/
	unsigned char	d_ptype;	/* peripheral type	*/
	unsigned char	d_heads;	/* number of heads	*/
	unsigned char	d_fixed;	/* fixed or removeable	*/
	unsigned 	d_cyls;		/* number of cylinders	*/
	unsigned short	d_bps;		/* bytes per sector	*/
	unsigned short	d_bsize;	/* logical block size	*/
	unsigned char	d_spt;		/* sectors per track	*/
	unsigned char	d_rspt;		/* reserved sects/track */
	unsigned char	d_hrd;		/* hard/soft sector flag*/
	unsigned char	d_ileave;	/* interleave factor	*/
	unsigned char	d_fchar;	/* format init char	*/
	unsigned char	d_retry;	/* retry count		*/
	unsigned char	d_step;		/* step rate		*/
	unsigned char	d_flop;		/* floppy format	*/
	unsigned short	d_precomp;	/* pre-compensation cyl	*/
	unsigned short	d_redwrite;	/* reduced write cyl	*/
	unsigned char	d_alttype;	/* type of alternates	*/
	unsigned char	d_altnum;	/* number of alternates */
	unsigned char	d_ecc;		/* ECC flag		*/
	unsigned char	d_spiral;	/* spiral offset	*/
	unsigned char	d_gap1;		/* gap1 size		*/
	unsigned char	d_gap2;		/* gap2 size		*/
	unsigned char	d_gap3;		/* gap3 size		*/
	unsigned char	d_gap4;		/* gap4 size		*/
	unsigned char	d_strat;	/* disk strategy select	*/
	unsigned char	d_block;	/* disk blocking factor	*/
	unsigned char	d_skew;		/* cylinder skew	*/
	unsigned char	d_cache;	/* cache entry size	*/
};

/*
 *	d_fixed - media type
 */

#define SD_FIXED	0x0		/* fixed media		*/
#define SD_REMOVE	0x1		/* removable media	*/

/*
 *	d_flop	 floppy format
 */

#define MFMENC	(1<<0)			/* MFM encoding,FM=0,MFM=1	*/
#define DRATE	(1<<1)			/* data rate 0=5.25,1=8		*/
#define D96TPI	(1<<2)			/* drive 96 track/inch,0=48	*/
#define M96TPI	(1<<3)			/* media 96 track/inch,0=48	*/
#define T0DENS	(1<<4)			/* track 0 density		*/
#define IBMFMT	(1<<5)			/* format 0=mot,1=IBM		*/

/*
 *	tape descriptor
 */

struct t_tape {
	unsigned char	t_ctype;	/* controller type	*/
	unsigned char	t_dtype;	/* drive type		*/
	unsigned char	t_phystracks;	/* number of tracks	*/
	unsigned char	t_logtracks;	/* number of tracks	*/
	unsigned char	t_extwrt;	/* extend on write flag */
	unsigned char	t_byteswap;	/* byte swap mode flag */
	unsigned char	t_buffmode;	/* buffered mode flag */
	unsigned char 	t_resv0;	/* reserved */
	unsigned short	t_physbpb;	/* physical bytes per block */
	unsigned short	t_logbpb;	/* logical bytes per block */
	unsigned short	t_qic;		/* QIC format		*/
	unsigned short	t_scount;	/* streaming count	*/
	unsigned char 	t_resv1;	/* reserved */
	unsigned char 	t_retrycnt;	/* retry count */
	unsigned short	t_minrd;	/* minimum read xfer	*/
	unsigned short	t_minwrt;	/* minimum write xfer	*/
	unsigned short	t_resv3[7];	/* 	reserved	*/
};


/*
 *	Error Codes
 */

/* 
 *	p_faterr
 */

/* 	BPP errors	*/

#define E_BADD		0x01		/* bad descriptor	*/
#define	E_BADC		0x02		/* bad command		*/
#define	E_UNCMD		0x03		/* unimplemented comm	*/		
#define E_BADDRV	0x04		/* bad drive		*/
#define E_BADADDR	0x05		/* bad logical address	*/
#define E_BADSCAT	0x06		/* bad scatter/gather	*/
#define E_UNDEV		0x07		/* unimplemented device */
#define E_NOINIT	0x08		/* unit not initialized	*/

/*	Media errors	*/

#define E_NOID		0x10		/* no id found on track	*/
#define E_SEEK		0x11		/* seek error		*/
#define E_RELCAT	0x12		/* relocate track error	*/
#define E_RNFID		0x13		/* record not found,ID	*/
#define E_DSF		0x14		/* data sync fault	*/
#define E_ECC		0x15		/* ECC error		*/
#define E_RNF		0x16		/* record not found	*/

/*	Drive errors	*/

#define E_DRVF		0x20		/* drive fault		*/
#define E_WRTP		0x21		/* write protected	*/
#define E_NOMOT		0x22		/* motor not on		*/
#define E_DOOR		0x23		/* door open		*/
#define E_NRDY		0x24		/* disk not ready	*/
#define E_BUSY		0x25		/* drive busy		*/

/*	VME DMA errors	*/

#define E_BUS		0x30		/* VME bus error	*/
#define E_BADAL		0x31		/* bad address alignment*/
#define E_BTIME		0x32		/* bus timeout		*/
#define E_INVAL		0x33		/* invalid DMA xfer count*/

/*	disk format errors	*/

#define E_ALT		0x40		/* not enough alternats	*/
#define E_FMT		0x41		/* format failed	*/
#define E_VER		0x42		/* verify error		*/
#define E_BPARAM	0x43		/* bad format params	*/
#define E_NOFIX		0x44		/* can't fix bad spot	*/

/*
 *		Custom SCSI Command 
 */


struct sb_scsipkt {

	struct scsipkt	*link;
	unsigned short	ctrlword;
	unsigned char	cmdlength;
	unsigned char	res1;
	unsigned char	cdb0;
	unsigned char	cdb1;
	unsigned char	cdb2;
	unsigned char	cdb3;
	unsigned char	cdb4;
	unsigned char	cdb5;
	unsigned char	cdb6;
	unsigned char	cdb7;
	unsigned char	cdb8;
	unsigned char	cdb9;
	unsigned char	cdba;
	unsigned char	cdbb;
	unsigned long	datalength;
	unsigned char	*dataptr;
	unsigned char	scsistat;
	unsigned char	initid;
	unsigned char	mesginflg;
	unsigned char	mesgoutflg;
	unsigned short	mesginlength;
	unsigned char	*mesginptr;
	unsigned char	mesgin1;
	unsigned char	mesgin2;
	unsigned char	mesgin3;
	unsigned char	mesgin4;
	unsigned char	mesgin5;
	unsigned char	mesgin6;
	unsigned short	mesgoutlength;
	unsigned char	*mesgoutptr;
	unsigned char	mesgout1;
	unsigned char	mesgout2;
	unsigned char	mesgout3;
	unsigned char	mesgout4;
	unsigned char	mesgout5;
	unsigned char	mesgout6;
	unsigned char	script1;
	unsigned char	script2;
	unsigned char	script3;
	unsigned char	script4;
	unsigned char	script5;
	unsigned char	script6;
	unsigned char	script7;
	unsigned char	script8;
};
