/*	Copyright (c) 1988 MOTOROLA */
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF MOTOROLA	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ident	"@(#)tpimux.h	35.1"

/*
 *
 * Definitions pertaining to the TPImux
 *
 * Copyright (c) 1987, Motorola Microcomputer Division
 */
#include "netinet/types.h"

/*
 * protocol specific stuff
 */
#define NPRIM T_ORDREL_REQ+1
#define BADEVENT TE_NOEVENTS
#define NONEVENT 0

typedef struct proto {
	char *name;			/* proto name	  		     */
	dev_t maj;			/* major device			     */
	int (*start)();			/* start up the proto		     */
	int (*finis)();			/* terminate the proto		     */
	int (*prreq)();			/* lower level proto request	     */
	int (*open)();			/* proto specific open() processing  */
	int (*close)();			/* proto specific close() processing */
	int (*ioctl)();			/* proto specific ioctl() processing */
	int (*input)();			/* input to protocol		     */
	int (*data)();			/* data to protocol		     */
	int (*listen)();		/* arrange to listen for connex	     */
	int (*accept)();		/* proto spec t_accept action	     */
	int (*connect)();		/* proto specific t_connect action   */
	int (*shutdown)();		/* shutdown activity for TCB	     */
	int (*abort)();			/* proto spec. abort		     */
	int (*bind)();			/* bind a protocol address	     */
	int (*options)();		/* options setting		     */
	int (**timers)();		/* ^ to array of timers		     */
	struct T_info_ack info;		/* info structure in convient form   */
	uchar prim_to_event[NPRIM];	/* map TLI primitives to proto events*/
	queue_t *below;			/* ^ to queue below		     */
	queue_t *ctrl;			/* ^ to ctrl queue		     */
	struct socket *socket;		/* ^ to sockets			     */
	struct socket *socketNSOCKET;	/* ^ to end of sockets		     */
	struct socket *lastrun;		/* scheduling info		     */
	struct tm_tcb **free_tcb;	/* free TCB's for proto		     */
	uchar max_conind;		/* Max connection indications	     */
	uchar optsize;			/* size of options		     */
	uchar creqsize;			/* connection request size	     */
	uchar cressize;			/* connection response size	     */
	uchar disconsize;		/* disconnect request size	     */
	uchar hdrsize;			/* lower level proto header size     */
	uchar addrsize;			/* size of address		     */
	ushort tidu;			/* transport interface data unit     */
	caddr_t optdef;			/* ^ to default options		     */
} proto_t;

/*
 * TPImux's idea of a protocol address.
 * We include the internet address form for convenience of internet protocol
 * code.
 */
union tpim_addr {
	struct {		/* internet address form	*/
		ushort family;	/* Berzerkely address format	*/
		ushort port;	/* port				*/
		HOST host;	/* host addr			*/
	} inet;
	struct {		/* AF_UNIX address form		*/
		ushort family;	/* AF_UNIX			*/
		uchar  name[6];	/* name				*/
	} un;
	uchar addr[8];		/* 64 bits of address		*/
};
/*
 * the TPI mux has an abstract idea of what a TCB looks like.
 * Its view of TCB's is that they are just the linked list of pending
 * connexion requests.
 */
struct tm_tcb {
	struct tm_tcb *link;	/* ^ to next TCB in list		*/
	queue_t *q;		/* ^ to upstream queue			*/
	union tpim_addr src;	/* source address			*/
	union tpim_addr dst;	/* destination address			*/
	ushort seqno;		/* TLI sequence #			*/
	uchar  extra;		/* extra space needed in header		*/
	uchar  tcbstate;	/* flags: following TCB state		*/
	ushort tflags;		/* timer flags: see tpimux:ursrv()	*/
};

/*
 * tm_tcb.tcbstate flags definitions
 */
#define TCBS_DISSOCIATED	0x01	/* TCB has been dissociated	*/
#define TCBS_DISCONNECT		0x02	/* TCB has seen a disconnect	*/
#define TCBS_FREE		0x04	/* TCB is on free list		*/
#define tpim_is_free(t)		((t)->tcbstate &  TCBS_FREE)
#define tpim_is_dissoc(t)	((t)->tcbstate &  TCBS_DISSOCIATED)
#define tpim_is_discon(t)	((t)->tcbstate &  TCBS_DISCONNECT)
#define tpim_mark_free(t)	((t)->tcbstate |= TCBS_FREE)
#define tpim_mark_dissoc(t)	((t)->tcbstate |= TCBS_DISSOCIATED)
#define tpim_mark_discon(t)	((t)->tcbstate |= TCBS_DISCONNECT)

/*
 * a TLImux communication endpoint.
 */
typedef struct socket {
	uchar ti_state;		/* TLI state			*/
	uchar so_state;		/* socket state 		*/
	uchar flags;		/* socket flags			*/
	uchar maxcon;		/* max conn ind on this sock	*/
	uchar numcon;		/* #of conn ind on this sock	*/
	uchar numsyn;		/* #of partially synch conns	*/
	ushort uid;		/* user id from open		*/
	union tpim_addr addr;	/* bound address		*/
	proto_t *proto;		/* ^ to protocol ctrl struct	*/
	struct tm_tcb *tcb;	/* ^ to xmission ctl blk	*/
	struct tm_tcb *conreq;	/* ^ to TCBs for conn requests	*/
	struct tm_tcb *syn;	/* ^ to TCBs for partial syns	*/
	queue_t *q;		/* streams q ^			*/
	mblk_t *options;	/* options from t_optmgmt()	*/
} socket_t;

/*
 * so_state definitions
 */
#define SS_CLOSED	0	/* socket closed			*/
#define SS_OPEN		1	/* socket has been opened		*/
#define SS_BOUND	2	/* socket has been bound to local port	*/
#define SS_LISTEN	4	/* listening for connections		*/
#define SS_FATAL	5	/* fatal error has occurred		*/
#define SS_PENDING	6	/* connection pending			*/
#define SS_UP		7	/* link up				*/

/*
 * so_flags definitions
 */
#define SF_NEED_ENABLE	0x01	/* upper write queue needs to be polled	*/
#define SF_NEED_TCB	0x02	/* socket needs a TCB; none available	*/

/*
 * Some common return codes used
 */
#define PROCESSED	0	/* processed a mblk_t based request	*/
#define FLOW_CTRLED	-1	/* met with flow control restrictions	*/
#define BUFCALLED	-2	/* needed space, bufcall() will enable	*/
#define NEED_ENABLE	-3	/* needed space, need timer based enable*/

/*
 * misc.
 */
#define TPIM_NOFLOW	1	/* request for no flow control on input	*/
